package com.nordicpeak.flowengine.evaluators.addflowinstancemanagersevaluator;

import java.io.File;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.emailutils.framework.FileAttachment;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.SingleTagSource;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowFamily;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.UserValuesQuery;
import com.nordicpeak.flowengine.interfaces.UserValuesQueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.utils.FlowFamilyUtils;

public class AddFlowInstanceManagersEvaluationProviderModule extends BaseEvaluationProviderModule<AddFlowInstanceManagersEvaluator> {

	protected static final AnnotatedBeanTagSourceFactory<Flow> FLOW_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<Flow>(Flow.class, "$flow.");
	private static final AnnotatedBeanTagSourceFactory<FlowInstance> FLOWINSTANCE_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<FlowInstance>(FlowInstance.class, "$flowInstance.");
	protected static final AnnotatedBeanTagSourceFactory<User> POSTER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$poster.");
	protected static final AnnotatedBeanTagSourceFactory<User> MANAGER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$manager.");
	protected static final AnnotatedBeanTagSourceFactory<User> SUBMITTER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$submitter.");

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "PDF filename (without file extension)", description = "Filename of the attached PDF (without file extension). Available tags: $flow.name, $flowInstance.flowInstanceID, $poster.*", required = true)
	protected String pdfFilename = "$flow.name - $flowInstance.flowInstanceID";

	protected AnnotatedDAO<AddFlowInstanceManagersEvaluator> evaluatorDAO;

	protected QueryParameterFactory<AddFlowInstanceManagersEvaluator, Integer> evaluatorIDParamFactory;

	protected AddFlowInstanceManagersEvaluatorCRUD evaluatorCRUD;

	@InstanceManagerDependency
	protected PDFProvider pdfProvider;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	protected FlowInstanceAdminModule flowInstanceAdminModule;

	@InstanceManagerDependency(required = true)
	protected StandardFlowNotificationHandler standardFlowNotificationHandler;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AddFlowInstanceManagersEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(AddFlowInstanceManagersEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new AddFlowInstanceManagersEvaluatorCRUD(AddFlowInstanceManagersEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, AddFlowInstanceManagersEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (!(queryInstance instanceof UserValuesQueryInstance)) {

			log.warn("Query instance " + queryInstance + " does not implement interface " + UserValuesQueryInstance.class.getName());

			return null;
		}

		if (queryInstance.getQueryInstanceDescriptor().isPopulated()) {

			UserValuesQueryInstance UserValuesQueryInstance = (UserValuesQueryInstance) queryInstance;
			Collection<User> newManagers = UserValuesQueryInstance.getUserValues(queryHandler);

			if (newManagers == null) {

				log.warn("Unable to find new managers for flow instance " + queryInstance + " from query instance " + UserValuesQueryInstance);
				throw new RuntimeException("New managers not found");
			}
		}

		return null;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 50)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {

		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();

		List<User> managingUsers = getAllowedFlowManagers(flowInstanceManager.getFlowInstance().getFlow().getFlowFamily());

		if (CollectionUtils.isEmpty(managingUsers)) {
			return;
		}

		List<UserValuesQueryInstance> queryInstances = flowInstanceManager.getQueries(UserValuesQueryInstance.class);

		if (queryInstances != null) {

			Timestamp now = TimeUtils.getCurrentTimestamp();

			for (UserValuesQueryInstance queryInstance : queryInstances) {

				if (queryInstance == null || !queryInstance.getQueryInstanceDescriptor().isPopulated() || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {

					continue;
				}

				User poster = flowInstanceManager.getFlowInstance().getPoster();
				User submitter = event.getEvent().getPoster();

				for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {

					if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {

						AddFlowInstanceManagersEvaluator evaluator = getEvaluator(evaluatorDescriptor);

						Collection<User> newManagers = queryInstance.getUserValues(queryHandler);

						if (newManagers != null) {

							//Workaround for immutable collections
							newManagers = new ArrayList<User>(newManagers);
							
							for (Iterator<User> it = newManagers.iterator(); it.hasNext();) {

								User newManager = it.next();

								if (!managingUsers.contains(newManager)) {

									it.remove();
									log.warn("Attempt to add user " + newManager + " as manager for flow instance " + flowInstanceManager.getFlowInstance() + ", but the user is not a manager for that family");
								}
							}

							if (!CollectionUtils.isEmpty(newManagers)) {

								StringBuilder stringBuilder = new StringBuilder();

								for (User manager : newManagers) {

									if (stringBuilder.length() > 0) {

										stringBuilder.append(", ");
									}

									stringBuilder.append(manager.getFirstname());
									stringBuilder.append(" ");
									stringBuilder.append(manager.getLastname());
								}

								String newManagersString = stringBuilder.toString();

								log.info("Adding " + newManagersString + " as managers for flow instance " + flowInstanceManager.getFlowInstance());

								FlowInstance flowInstance = flowInstanceAdminModule.getFlowInstance(flowInstanceManager.getFlowInstanceID());

								if (flowInstance.getManagers() == null) {
									flowInstance.setManagers(new ArrayList<User>(newManagers.size()));
								}

								flowInstance.getManagers().addAll(newManagers);
								
								flowInstance.setManagers(CollectionUtils.removeDuplicates(flowInstance.getManagers()));

								HighLevelQuery<FlowInstance> updateQuery = new HighLevelQuery<FlowInstance>(FlowInstance.MANAGERS_RELATION);
								flowInstanceAdminModule.getDAOFactory().getFlowInstanceDAO().update(flowInstance, updateQuery);

								if (evaluator.isShowInLog()) {

									flowInstanceAdminModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(flowInstanceManager.getFlowInstance(), EventType.MANAGERS_UPDATED, newManagersString, submitter, now);
								}

								systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(FlowInstance.class, CRUDAction.UPDATE, flowInstance), EventTarget.ALL);

								if (evaluator.sendEmail()) {

									for (User manager : newManagers) {
										sendEmail(evaluator, flowInstance, event.getEvent().getEventID(), poster, manager, submitter);
									}
								}
							}
						}
					}
				}
			}
		}
	}

	public void sendEmail(AddFlowInstanceManagersEvaluator evaluator, FlowInstance flowInstance, Integer submitEventID, User poster, User manager, User submitter) {

		SimpleEmail email = null;

		try {

			log.info("Sending email about new manager for flow instance " + flowInstance + " to " + manager + " trigged by evaluator " + evaluatorDescriptor);

			TagReplacer tagReplacer = new TagReplacer(FLOW_TAG_SOURCE_FACTORY.getTagSource(flowInstance.getFlow()));
			tagReplacer.addTagSource(FLOWINSTANCE_TAG_SOURCE_FACTORY.getTagSource(flowInstance));
			tagReplacer.addTagSource(new SingleTagSource("$flowInstance.url", standardFlowNotificationHandler.getFlowInstanceAdminModuleAlias(flowInstance) + "/overview/" + flowInstance.getFlowInstanceID()));
			tagReplacer.addTagSource(MANAGER_TAG_SOURCE_FACTORY.getTagSource(manager));

			if (poster != null) {
				tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(poster));
			}

			if (submitter != null) {
				tagReplacer.addTagSource(SUBMITTER_TAG_SOURCE_FACTORY.getTagSource(submitter));
			}

			String message = tagReplacer.replace(evaluator.getEmailMessage());
			String subject = tagReplacer.replace(evaluator.getEmailSubject());
			String filename = tagReplacer.replace(pdfFilename) + ".pdf";

			File file = null;

			if (evaluator.attachPDF() && submitEventID != null) {

				if (pdfProvider != null) {

					file = pdfProvider.getPDF(flowInstance.getFlowInstanceID(), submitEventID);

				} else {

					log.warn("No pdf provider found, pdf cant be attached to generated email triggered by evaluator " + evaluatorDescriptor + " for flow instance " + flowInstance + " submitted by user " + submitter);
				}
			}

			email = new SimpleEmail(systemInterface.getEncoding());

			email.setSenderName(standardFlowNotificationHandler.getEmailSenderName(flowInstance));
			email.setSenderAddress(standardFlowNotificationHandler.getEmailSenderAddress(flowInstance));
			email.setSubject(subject);
			email.setMessage(message);
			email.setMessageContentType(SimpleEmail.HTML);
			email.addRecipient(manager.getEmail());

			if (file != null) {

				email.add(new FileAttachment(file, filename));
			}

			systemInterface.getEmailHandler().send(email);

		} catch (Exception e) {

			log.error("Unable to send email " + email + " about submitted flow instance " + flowInstance + " submitted by user " + submitter, e);
		}
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		AddFlowInstanceManagersEvaluator evaluator = new AddFlowInstanceManagersEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	protected AddFlowInstanceManagersEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {

		return evaluatorDAO.get(new HighLevelQuery<AddFlowInstanceManagersEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		AddFlowInstanceManagersEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<AddFlowInstanceManagersEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<AddFlowInstanceManagersEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return UserValuesQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		AddFlowInstanceManagersEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<AddFlowInstanceManagersEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

	}

	@Override
	public AddFlowInstanceManagersEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		AddFlowInstanceManagersEvaluator evaluator = new AddFlowInstanceManagersEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	public QueryHandler getQueryHandler() {

		return queryHandler;
	}

	protected List<User> getAllowedFlowManagers(ImmutableFlowFamily flowFamily) {

		return FlowFamilyUtils.getAllowedManagerUsers(flowFamily, systemInterface.getUserHandler());
	}

}
