<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateAgeQueryStateEvaluator" />
		</div>

	</xsl:template>

	<xsl:template name="createCustomFormFields">
		
		<xsl:param name="queryStateEvaluator" />
		
		<xsl:variable name="fieldsCount" select="count(TextFields/TextField)" />

			<label class="floatleft full">
				<xsl:value-of select="$i18n.fields.title" />
			</label>
	
			<xsl:choose>
				<xsl:when test="$fieldsCount > 1">
					
					<div class="clearboth floatleft marginbottom">
						<xsl:value-of select="$i18n.fields.description" />
					</div>
				
					<xsl:apply-templates select="TextFields/TextField">
						<xsl:with-param name="queryStateEvaluator" select="$queryStateEvaluator" />
					</xsl:apply-templates>
				</xsl:when>
				<xsl:when test="$fieldsCount = 1">
					
					<div class="clearboth floatleft marginbottom">
						<xsl:value-of select="$i18n.fields.single.description" />
					</div>
					
					<xsl:call-template name="createHiddenField">
						<xsl:with-param name="id" select="'textFieldID'" />
						<xsl:with-param name="name" select="'textFieldID'" />
						<xsl:with-param name="value" select="TextFields/TextField/fieldID" />
					</xsl:call-template>
					
					<p class="floatleft full bold nomargin"><xsl:value-of select="TextFields/TextField/name" /></p>
					
				</xsl:when>
				<xsl:when test="TextFields">
					<p>
						<xsl:value-of select="$i18n.QueryHasNoTextFields" />
					</p>
				</xsl:when>
				<xsl:otherwise>
				</xsl:otherwise>
			</xsl:choose>
		
			<label class="floatleft full">
				<xsl:value-of select="$i18n.age.title" />
			</label>
			
			<div class="clearboth floatleft marginbottom">
				<xsl:value-of select="$i18n.age.description" />
			</div>
			
			<div class="clearboth floatleft marginbottom twenty">
				<xsl:value-of select="$i18n.from" />
				<span class="required">*</span>
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'fromValue'" />
					<xsl:with-param name="name" select="'fromValue'" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
				</xsl:call-template>
			</div>
		
			<div class="clearboth floatleft marginbottom twenty">
				<xsl:value-of select="$i18n.to" />
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'toValue'" />
					<xsl:with-param name="name" select="'toValue'" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
				</xsl:call-template>
			</div>
			
			<label class="floatleft full">
				<xsl:value-of select="$i18n.calculateMode.title" />
			</label>
			
			<div class="clearboth floatleft marginbottom">
				<xsl:value-of select="$i18n.calculateMode.description" />
			</div>
			
			<div class="floatleft full marginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'by_year'" />
						<xsl:with-param name="name" select="'calculateMode'" />
						<xsl:with-param name="value" select="'BY_YEAR'" />
						<xsl:with-param name="element" select="$queryStateEvaluator" />
					</xsl:call-template>

					<label for="by_year">
						<xsl:value-of select="$i18n.CalculateMode.BY_YEAR" />
					</label>
				</div>
			</div>

			<div class="floatleft full marginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'by_date'" />
						<xsl:with-param name="name" select="'calculateMode'" />
						<xsl:with-param name="value" select="'BY_DATE'" />
						<xsl:with-param name="element" select="$queryStateEvaluator" />
					</xsl:call-template>

					<label for="by_date">
						<xsl:value-of select="$i18n.CalculateMode.BY_DATE" />
					</label>
				</div>
			</div>
		
	</xsl:template>

	<xsl:template match="UpdateAgeQueryStateEvaluator">

		<xsl:call-template name="createQueryStateEvaluatorForm">
			<xsl:with-param name="queryStateEvaluator" select="AgeQueryStateEvaluator" />
		</xsl:call-template>

	</xsl:template>

	<xsl:template match="TextField">

		<xsl:param name="queryStateEvaluator" />

		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createRadio">
					<xsl:with-param name="id">
						<xsl:value-of select="'textfield'"/>
						<xsl:value-of select="fieldID"/>
					</xsl:with-param>				
					<xsl:with-param name="name" select="'textFieldID'" />
					<xsl:with-param name="value" select="fieldID" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
				</xsl:call-template>

				<label for="textfield{fieldID}">
					<xsl:value-of select="name" />
				</label>
			</div>
			
		</div>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'FromValueBiggerThanToValue']">

		<p class="error">
			<xsl:value-of select="$i18n.FromValueBiggerThanToValue" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TextFieldNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.TextFieldNotFound" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'queryState'">
				<xsl:value-of select="$i18n.queryState" />
			</xsl:when>
			<xsl:when test="$fieldName = 'toValue'">
				<xsl:value-of select="$i18n.to" />
			</xsl:when>
			<xsl:when test="$fieldName = 'fromValue'">
				<xsl:value-of select="$i18n.from" />
			</xsl:when>
			<xsl:when test="$fieldName = 'calculateMode'">
				<xsl:value-of select="$i18n.calculateMode.title" />
			</xsl:when>
			<xsl:when test="$fieldName = 'textFieldID'">
				<xsl:value-of select="$i18n.fields.title" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>