package com.nordicpeak.flowengine.queries.alternativelistcheckboxquery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativeQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

@Table(name = "alternative_list_checkbox_query_instances")
@XMLElement
public class AlternativeListCheckboxQueryInstance extends BaseQueryInstance implements FixedAlternativesQueryInstance, StringValueQueryInstance {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field ALTERNATIVES_RELATION = ReflectionUtils.getField(AlternativeListCheckboxQueryInstance.class, "alternatives");
	public static final Field QUERY_RELATION = ReflectionUtils.getField(AlternativeListCheckboxQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged
	@XMLElement
	private Integer minChecked;

	@DAOManaged
	@XMLElement
	private Integer maxChecked;

	@DAOManaged
	@XMLElement
	private String freeTextAlternativeValue;

	@DAOManaged(columnName="queryID")
	@ManyToOne
	@XMLElement
	private AlternativeListCheckboxQuery query;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase=true)
	private List<AlternativeListCheckboxAlternative> alternatives;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	public Integer getMinChecked() {

		return minChecked;
	}

	public void setMinChecked(Integer minChecked) {

		this.minChecked = minChecked;
	}

	public Integer getMaxChecked() {

		return maxChecked;
	}

	public void setMaxChecked(Integer maxChecked) {

		this.maxChecked = maxChecked;
	}

	@Override
	public String getFreeTextAlternativeValue() {
		return freeTextAlternativeValue;
	}

	public void setFreeTextAlternativeValue(String freeTextAlternativeValue) {
		this.freeTextAlternativeValue = freeTextAlternativeValue;
	}

	@Override
	public AlternativeListCheckboxQuery getQuery() {

		return query;
	}

	public void setQuery(AlternativeListCheckboxQuery query) {

		this.query = query;
	}

	@Override
	public List<AlternativeListCheckboxAlternative> getAlternatives() {

		return alternatives;
	}

	public void setAlternatives(List<AlternativeListCheckboxAlternative> alternatives) {

		this.alternatives = alternatives;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.alternatives = null;
		
		if(query.isSetAsAttribute()) {
			
			resetAttribute(attributeHandler);
		}
		
		super.reset(attributeHandler);
	}

	public void copyQueryValues() {

		this.minChecked = query.getMinChecked();
		this.maxChecked = query.getMaxChecked();
	}

	@Override
	public String toString() {

		return "AlternativeListCheckboxQueryInstance (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		FixedAlternativeQueryUtils.appendExportXMLAlternatives(doc, element, this);

		return element;
	}
	
	@Override
	public String getStringValue() {
		
		if (alternatives == null && freeTextAlternativeValue == null) {
			
			return null;
		}
		
		StringBuilder stringBuilder = new StringBuilder();
		
		if (alternatives != null) {
			
			for (AlternativeListCheckboxAlternative alternative : alternatives) {
				
				if (stringBuilder.length() != 0) {
					
					stringBuilder.append(", ");
				}
				
				stringBuilder.append(alternative.getName());
			}
		}
		
		if (freeTextAlternativeValue != null) {
			
			if (stringBuilder.length() != 0) {
				
				stringBuilder.append(", ");
			}
			
			stringBuilder.append(freeTextAlternativeValue);
		}
		
		return stringBuilder.toString();
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler){
		
		attributeHandler.removeAttribute(query.getAttributeName());
	}
	
	public void setAttribute(MutableAttributeHandler attributeHandler){
		
		List<String> attributeValues = null;
		
		if (alternatives != null) {
			
			attributeValues = new ArrayList<String>(alternatives.size());
			
			for (AlternativeListCheckboxAlternative bean : alternatives) {
				
				if (!StringUtils.isEmpty(bean.getAttributeValue())) {
					
					attributeValues.add(bean.getAttributeValue());
					
				} else {
					
					attributeValues.add(bean.getName());
				}
			}
		}
		
		if (!StringUtils.isEmpty(freeTextAlternativeValue)) {
			
			attributeValues = CollectionUtils.addAndInstantiateIfNeeded(attributeValues, freeTextAlternativeValue);
		}
		
		if (attributeValues != null) {
			
			attributeHandler.setAttribute(query.getAttributeName(), StringUtils.toCommaSeparatedString(attributeValues));
		}
	}

}
