package com.nordicpeak.flowengine.queries.alternativelistdropdownquery;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.annotations.TextTagReplace;
import com.nordicpeak.flowengine.interfaces.ExportAlternativeID;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesBaseQuery;

@Table(name = "alternative_list_drop_down_query")
@XMLElement
public class AlternativeListDropDownQuery extends FixedAlternativesBaseQuery implements AlternativeListQuery, ExportAlternativeID {
	
	private static final long serialVersionUID = -842191226937409416L;
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;
	
	@TextTagReplace
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String shortDescription;
	
	@DAOManaged
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String listName;
	
	@TextTagReplace
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String emptyListDescription;
	
	@WebPopulate(maxLength = 255)
	@DAOManaged
	@XMLElement
	private String freeTextAlternative;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setIDAttribute;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setIDAttribute")
	@XMLElement
	private String idAttributeName;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean exportAlternativeID;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean lockForManagerUpdate;
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<AlternativeListDropDownQueryInstance> instances;
	
	private AlternativeListInstance alternativeList;
	
	@Override
	public Integer getQueryID() {
		
		return queryID;
	}
	
	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {
		
		if (alternativeList != null) {
			
			return alternativeList.getAlternatives();
		}
		
		return null;
	}
	
	public String getListName() {
		return listName;
	}
	
	public void setListName(String listIdentifier) {
		this.listName = listIdentifier;
	}
	
	public AlternativeListInstance getAlternativeList() {
		return alternativeList;
	}
	
	public void setAlternativeList(AlternativeListInstance alternativeList) {
		this.alternativeList = alternativeList;
	}
	
	@Override
	public String getFreeTextAlternative() {
		return freeTextAlternative;
	}
	
	public void setFreeTextAlternative(String freeTextAlternative) {
		this.freeTextAlternative = freeTextAlternative;
	}
	
	public List<AlternativeListDropDownQueryInstance> getInstances() {
		
		return instances;
	}
	
	public void setInstances(List<AlternativeListDropDownQueryInstance> instances) {
		
		this.instances = instances;
	}
	
	public void setQueryID(int queryID) {
		
		this.queryID = queryID;
	}
	
	@Override
	public String toString() {
		
		if (this.queryDescriptor != null) {
			
			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}
		
		return "AlternativeListDropDownQuery (queryID: " + queryID + ")";
	}
	
	public String getShortDescription() {
		
		return shortDescription;
	}
	
	public void setShortDescription(String shortDescription) {
		
		this.shortDescription = shortDescription;
	}
	
	public boolean isSetAsAttribute() {
		return setAsAttribute;
	}
	
	public void setSetAsAttribute(boolean setAsAttribute) {
		this.setAsAttribute = setAsAttribute;
	}
	
	public String getAttributeName() {
		return attributeName;
	}
	
	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}

	public boolean isSetIDAttribute() {
		
		return setIDAttribute;
	}
	
	public void setSetIDAttribute(boolean setIDAttribute) {
	
		this.setIDAttribute = setIDAttribute;
	}
	
	public String getIdAttributeName() {
	
		return idAttributeName;
	}

	public void setIdAttributeName(String idAttributeName) {
	
		this.idAttributeName = idAttributeName;
	}
	
	@Override
	public String getXSDTypeName() {
		
		return "AlternativeListDropDownQuery" + queryID;
	}
	
	@Override
	public void toXSD(Document doc) {
		
		toXSD(doc, 1);
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		shortDescription = XMLValidationUtils.validateParameter("shortDescription", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		emptyListDescription = XMLValidationUtils.validateParameter("emptyListDescription", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		listName = XMLValidationUtils.validateParameter("listName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		freeTextAlternative = XMLValidationUtils.validateParameter("freeTextAlternative", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		lockForManagerUpdate = xmlParser.getPrimitiveBoolean("lockForManagerUpdate");
		
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		if (attributeName != null) {
			
			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}
		
		exportAlternativeID  = xmlParser.getPrimitiveBoolean("exportAlternativeID");
		
		idAttributeName = XMLValidationUtils.validateParameter("idAttributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (idAttributeName != null) {
			
			setIDAttribute = xmlParser.getPrimitiveBoolean("setIDAttribute");
		}
		
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}
	
	@Override
	public Element toXML(Document doc) {
		
		Element element = super.toXML(doc);
		
		if (alternativeList != null) {
			
			//TODO don't append alternatives on export
			XMLUtils.append(doc, element, "Alternatives", alternativeList.getAlternatives());
		}
		
		return element;
	}
	
	@Override
	public String getAlternativeListName() {
		return listName;
	}
	
	@Override
	public boolean isExportAlternativeID() {
		
		return exportAlternativeID;
	}

	
	public void setExportAlternativeID(boolean exportAlternativeID) {
	
		this.exportAlternativeID = exportAlternativeID;
	}

	public boolean isLockForManagerUpdate() {

		return lockForManagerUpdate;
	}

	public void setLockForManagerUpdate(boolean lockForManagerUpdate) {

		this.lockForManagerUpdate = lockForManagerUpdate;
	}

}
