<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>
	
	<xsl:template match="Document">

		<div class="contentitem alternativelistprovider errands-wrapper">

			<xsl:apply-templates select="ListAlternativeLists" />
			<xsl:apply-templates select="ShowAlternativeList" />
			<xsl:apply-templates select="AddAlternativeList" />
			<xsl:apply-templates select="UpdateAlternativeList" />
			<xsl:apply-templates select="ShowAlternativeListEvents" />
			<xsl:apply-templates select="ShowAlternativeListFlowUsage" />

		</div>

	</xsl:template>
	
	<xsl:template match="ListAlternativeLists">

		<h1><xsl:value-of select="$i18n.ListAlternativeLists" /></h1>

		<xsl:apply-templates select="validationException/validationError"/>
		<xsl:apply-templates select="validationError"/>
		
		<p>
			<xsl:value-of select="$i18n.ListAlternativeLists.Description" />
		</p>
		
		<xsl:choose>
			<xsl:when test="not(AlternativeLists/AlternativeList)">
			
				<span>
					<strong>
						<xsl:value-of select="$i18n.ListAlternativeLists.NoListsFound" />
					</strong>
				</span>
				
			</xsl:when>
			<xsl:otherwise>
			
				<table class="full coloredtable sortabletable oep-table">
					<thead class="sortable">
						<tr>
							<xsl:call-template name="ListTableColumns"/>
						</tr>
					</thead>
					<tbody>
						
						<xsl:apply-templates select="AlternativeLists/AlternativeList" mode="list"/>
						
					</tbody>
				</table>
				
			</xsl:otherwise>
		</xsl:choose>

		<div class="floatright marginright bigmargintop">
			<a class="btn btn-green btn-inline" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/add" title="{$i18n.Add}">
				<xsl:value-of select="$i18n.Add" />
			</a>
		</div>

	</xsl:template>
	
	<xsl:template name="ListTableColumns">
	
		<th><xsl:value-of select="$i18n.Name" /></th>
		<th><xsl:value-of select="$i18n.Size" /></th>
		<th class="nosort" style="width: 61px;"></th>
	
	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="list">
	
		<tr>
			<td data-title="{$i18n.Name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/show/{listID}">
					<xsl:value-of select="name"/>
				</a>
			</td>
			
			<td data-title="{$i18n.Size}">
				<xsl:value-of select="count(Alternatives/Alternative)"/>
			</td>
			
			<td>
				<xsl:apply-templates select="." mode="adminLinks"/>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="ShowAlternativeList">

		<div class="floatright">

			<xsl:apply-templates select="AlternativeList" mode="adminLinks"/>
			
		</div>

		<h1>
			<xsl:value-of select="$i18n.Show" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="AlternativeList/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>
		<xsl:apply-templates select="validationError"/>

		<fieldset>
			<legend><xsl:value-of select="$i18n.Show.Alternatives"/></legend>
		
			<table class="coloredtable sortabletable oep-table">
				<thead class="sortable">
					<tr>
						<th><xsl:value-of select="$i18n.Show.Alternative.Name" /></th>
						<th><xsl:value-of select="$i18n.Show.Alternative.Value" /></th>
						<xsl:call-template name="ShowAlternativeListHeaderExtra"/>
					</tr>
				</thead>
				<tbody>
					
					<xsl:apply-templates select="AlternativeList/Alternatives/Alternative" mode="list"/>
					<xsl:apply-templates select="Alternatives/Alternative" mode="list"/>
					
				</tbody>
			</table>
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Events.Title"/></legend>
			
			<p class="nomargin"><xsl:value-of select="$i18n.Events.Description"/></p>
	
				<xsl:choose>
					<xsl:when test="Events/AlternativeListEvent">
					
						<table class="full coloredtable sortabletable oep-table" cellspacing="0">
							<thead>	
								<tr>
									<th><xsl:value-of select="$i18n.Events.message" /></th>
									<th><xsl:value-of select="$i18n.Events.poster" /></th>
									<th><xsl:value-of select="$i18n.Events.added" /></th>
								</tr>
							</thead>
							<tbody>
							
								<xsl:apply-templates select="Events/AlternativeListEvent" mode="list"/>
								
							</tbody>
						</table>
						
					</xsl:when>
					<xsl:otherwise>
						<p><xsl:value-of select="$i18n.Events.NoEvents"/></p>
					</xsl:otherwise>
				</xsl:choose>
				
				<div class="floatright marginright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/events/{AlternativeList/listID}" title="{$i18n.Events.ShowAll}">
						<xsl:value-of select="$i18n.Events.ShowAll"/>
					</a>
				</div>
				
		</fieldset>
		
		<a class="btn btn-blue floatright margintop" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}"><xsl:value-of select="$i18n.Back"/></a>
		
		<xsl:call-template name="ShowAlternativeListButtonsExtra"/>
		
	</xsl:template>
	
	<xsl:template name="ShowAlternativeListHeaderExtra"/>
	<xsl:template name="ShowAlternativeListButtonsExtra"/>
	
	<xsl:template match="Alternative" mode="list">
		
		<tr>
			<td class="paddingright" data-title="{$i18n.Show.Alternative.Name}"><xsl:value-of select="name"/></td>
			<td class="paddingright" data-title="{$i18n.Show.Alternative.Value}"><xsl:value-of select="alternativeID"/></td>
		</tr>
			
	</xsl:template>
	
	<xsl:template match="AddAlternativeList">

		<h1>
			<xsl:value-of select="$i18n.Add" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>

		<form id="addListForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="editList"/>

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.Add}" />
			</div>

		</form>

	</xsl:template>
	
	<xsl:template match="UpdateAlternativeList">

		<h1>
			<xsl:value-of select="$i18n.Update" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="AlternativeList/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>

		<form id="updateListForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="editList"/>

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template name="editList">

		<div class="floatleft full bigmarginbottom">
			<label for="name" class="floatleft required"><xsl:value-of select="$i18n.Name" /></label>
			
			<p class="floatleft full error tiny"><xsl:value-of select="$i18n.Update.Description" /></p>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'name'"/>
					<xsl:with-param name="name" select="'name'"/>
					<xsl:with-param name="title" select="$i18n.Name"/>
					<xsl:with-param name="element" select="AlternativeList" />
				</xsl:call-template>
			</div>
		</div>
		
		<xsl:call-template name="editListDetails"/>
		
	</xsl:template>
	
	<xsl:template match="ShowAlternativeListFlowUsage">

		<h1>
			<xsl:value-of select="$i18n.FlowUsage.Title" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="AlternativeList/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>
		<xsl:apply-templates select="validationError"/>

		<xsl:choose>
			<xsl:when test="not(Queries/Flow)">
			
				<span><xsl:value-of select="$i18n.Show.UsedInFlows.NotUsed" /></span>
				
			</xsl:when>
			<xsl:otherwise>
			
				<table class="coloredtable sortabletable oep-table">
					<thead class="sortable">
						<tr>
							<th><xsl:value-of select="$i18n.Show.UsedInFlows.FlowName" /></th>
							<th><xsl:value-of select="$i18n.Show.UsedInFlows.FlowVersion" /></th>
							<th><xsl:value-of select="$i18n.Show.UsedInFlows.StepAndQueries" /></th>
						</tr>
					</thead>
					<tbody>
						
						<xsl:apply-templates select="Queries/Flow"/>
						
					</tbody>
				</table>
			
			</xsl:otherwise>
		</xsl:choose>
		
		<a class="btn btn-blue floatright margintop" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}"><xsl:value-of select="$i18n.Back"/></a>

	</xsl:template>
	
	<xsl:template match="Flow">
	
		<tr>
			<td class="paddingright" data-title="{$i18n.Show.UsedInFlows.FlowName}"><xsl:value-of select="name"/></td>
			<td class="paddingright" data-title="{$i18n.Show.UsedInFlows.FlowVersion}"><xsl:value-of select="version"/></td>
			<td data-title="{$i18n.Show.UsedInFlows.StepAndQueries}">
				
				<xsl:apply-templates select="Step/Query"/>
				
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="Query">
	
		<div class="full">
			<xsl:value-of select="../name"/>
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="QueryDescriptor/name"/>
			
			<xsl:if test="ExtraUsageInfo">
				<xsl:text> - </xsl:text>
				<xsl:value-of select="ExtraUsageInfo" />
			</xsl:if>
		</div>
	
	</xsl:template>
	
	<xsl:template match="ShowAlternativeListEvents">
	
		<h1>
			<xsl:value-of select="$i18n.Events.Full.Title"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="AlternativeList/name"/>
		</h1>
	
		<p class="nomargin">
			<xsl:value-of select="$i18n.Events.Full.Description"/>
		</p>

		<xsl:choose>
			<xsl:when test="Events/AlternativeListEvent">
			
				<table class="full coloredtable sortabletable oep-table flowfamilyeventstable" cellspacing="0">
					<thead>
						<tr>
							<th><xsl:value-of select="$i18n.Events.message" /></th>
							<th><xsl:value-of select="$i18n.Events.poster" /></th>
							<th><xsl:value-of select="$i18n.Events.added" /></th>
						</tr>
					</thead>
					<tbody>
					
						<xsl:apply-templates select="Events/AlternativeListEvent" mode="list"/>
						
					</tbody>
				</table>

			</xsl:when>
			<xsl:otherwise>
			
				<p><xsl:value-of select="$i18n.Events.NoEvents"/></p>
				
			</xsl:otherwise>
		</xsl:choose>
		
		<a class="btn btn-blue floatright margintop" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}"><xsl:value-of select="$i18n.Back"/></a>
			
	</xsl:template>
	
	<xsl:template match="AlternativeListEvent" mode="list">
		
		<tr>
			<td>
				<xsl:value-of select="message" />
			</td>
			<td>
				<xsl:call-template name="PrintAdminPostedBy">
					<xsl:with-param name="poster" select="poster"/>
				</xsl:call-template>
			</td>
			<td>
				<xsl:value-of select="added" />
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template name="PrintAdminPostedBy">
		<xsl:param name="poster"/>
		
		<xsl:choose>
		
			<xsl:when test="$poster">
				<xsl:call-template name="printUser">
					<xsl:with-param name="user" select="$poster" />
				</xsl:call-template>
			</xsl:when>
			
			<xsl:otherwise>
				<xsl:value-of select="$i18n.RemovedUser" />
			</xsl:otherwise>
			
		</xsl:choose>
	
	</xsl:template>
	
	<xsl:template name="printUser">
		<xsl:param name="user" />
		
		<xsl:value-of select="$user/firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="$user/lastname" />
		
	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="adminLinks">
		
		<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/usage/{listID}" title="{$i18n.FlowUsage.Button}">
			<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/magnifying_glass.png" alt="" />
		</a>		
		
		<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/update/{listID}" title="{$i18n.UpdateList}">
			<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/pen.png" alt="" />
		</a>
		
		<xsl:choose>
			<xsl:when test="../Queries/Flow">
			
				<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteListBlockedInUse}'); return false;" title="{$i18n.DeleteListBlockedInUse}">
					<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/delete_gray.png" alt="" />
				</a>
			
			</xsl:when>
			<xsl:otherwise>
			
				<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/delete/{listID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.DeleteList} {name}">
					<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/delete.png" alt="" />
				</a>
				
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'DeleteFailedAlternativeListNotFound' or messageKey = 'UpdateFailedAlternativeListNotFound']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.ListNotFound" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'UnableToDeleteListInUse']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.UnableToDeleteListInUse" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ListNameAlreadyInUse']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.ListNameAlreadyInUse" />
			<xsl:text> </xsl:text>
			<xsl:value-of select="AlternativeListProvider/name" />
			<xsl:text>!</xsl:text>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'TooShort']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.TooShort"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:apply-templates select="fieldName" mode="common" />!
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'Other']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.Other"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:apply-templates select="fieldName" mode="common" />!
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName" mode="common">
	
		<xsl:variable name="fieldName" select="." />
		
		<xsl:choose>
			<xsl:when test="$fieldName = 'name'">
				
				<xsl:value-of select="$i18n.Name" />
				
			</xsl:when>
			<xsl:otherwise>
			
				<xsl:apply-imports/>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
</xsl:stylesheet>