package com.nordicpeak.flowengine.queries.alternativelistradiobuttonquery;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativeQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

@Table(name = "alternative_list_radio_button_query_instances")
@XMLElement
public class AlternativeListRadioButtonQueryInstance extends BaseQueryInstance implements FixedAlternativesQueryInstance, StringValueQueryInstance {
	
	
	private static final long serialVersionUID = -7761759005604863873L;
	
	public static final Field QUERY_RELATION = ReflectionUtils.getField(AlternativeListRadioButtonQueryInstance.class, "query");
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;
	
	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private AlternativeListRadioButtonQuery query;
	
	@DAOManaged
	@XMLElement
	private Integer alternativeID;
	
	@DAOManaged
	@XMLElement
	private String alternativeName;
	
	@DAOManaged
	@XMLElement
	private String alternativeXMLValue;
	
	@DAOManaged
	@XMLElement
	private String alternativeAttributeValue;
	
	@DAOManaged
	@XMLElement
	private String freeTextAlternativeValue;
	
	public Integer getQueryInstanceID() {
		
		return queryInstanceID;
	}
	
	public void setQueryInstanceID(Integer queryInstanceID) {
		
		this.queryInstanceID = queryInstanceID;
	}
	
	@Override
	public AlternativeListRadioButtonQuery getQuery() {
		
		return query;
	}
	
	public void setQuery(AlternativeListRadioButtonQuery query) {
		
		this.query = query;
	}
	
	@Override
	public void reset(MutableAttributeHandler attributeHandler) {
		
		this.alternativeID = null;
		this.alternativeName = null;
		freeTextAlternativeValue = null;
		
		if (query.isSetAsAttribute()) {
			
			resetAttribute(attributeHandler);
		}
		
		super.reset(attributeHandler);
	}
	
	public void setAttribute(MutableAttributeHandler attributeHandler) {
		
		if (!StringUtils.isEmpty(freeTextAlternativeValue)) {
			
			attributeHandler.setAttribute(query.getAttributeName(), freeTextAlternativeValue);
			
		} else if (alternativeID != null) {
			
			if (!StringUtils.isEmpty(alternativeAttributeValue)) {
				
				attributeHandler.setAttribute(query.getAttributeName(), alternativeAttributeValue);
				
			} else {
				
				attributeHandler.setAttribute(query.getAttributeName(), alternativeName);
			}
		}
	}
	
	public void resetAttribute(MutableAttributeHandler attributeHandler) {
		
		attributeHandler.removeAttribute(query.getAttributeName());
	}
	
	public void copyQueryValues() {}
	
	@Override
	public String toString() {
		
		return "AlternativeListRadioButtonQueryInstance (queryInstanceID=" + queryInstanceID + ")";
	}
	
	public AlternativeListRadioButtonAlternative getAlternative() {
		
		if (alternativeID != null) {
			
			return new AlternativeListRadioButtonAlternative(alternativeID, alternativeName, alternativeXMLValue, alternativeAttributeValue);
		}
		
		return null;
	}
	
	public void setAlternative(ImmutableAlternative alternative) {
		
		if (alternative == null) {
			
			alternativeID = null;
			alternativeName = null;
			alternativeXMLValue = null;
			alternativeAttributeValue = null;
			
		} else {
			
			alternativeID = alternative.getAlternativeID();
			alternativeName = alternative.getName();
			alternativeXMLValue = alternative.getExportXMLValue();
			alternativeAttributeValue = alternative.getAttributeValue();
		}
	}
	
	@Override
	public String getFreeTextAlternativeValue() {
		
		return freeTextAlternativeValue;
	}
	
	public void setFreeTextAlternativeValue(String freeTextAlternativeValue) {
		
		this.freeTextAlternativeValue = freeTextAlternativeValue;
	}
	
	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {
		
		AlternativeListRadioButtonAlternative alternative = getAlternative();
		
		if (alternative == null) {
			
			return null;
		}
		
		return Collections.singletonList(alternative);
	}
	
	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {
		
		Element element = getBaseExportXML(doc);
		
		FixedAlternativeQueryUtils.appendExportXMLAlternatives(doc, element, this);
		
		return element;
	}
	
	@Override
	public String getStringValue() {
		
		if (alternativeID != null) {
			
			return alternativeName;
			
		} else if (freeTextAlternativeValue != null) {
			
			return freeTextAlternativeValue;
		}
		
		return null;
	}
}
