package com.nordicpeak.flowengine.queries.alternativelistradiobuttonquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.alternativelist.AlternativeListHandlerModule;
import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListProvider;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQueryProvider;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.populators.FreeTextAlternativePopulator;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class AlternativeListRadioButtonQueryProviderModule extends BaseQueryProviderModule<AlternativeListRadioButtonQueryInstance> implements BaseQueryCRUDCallback, AlternativeListQueryProvider {
	
	
	@XSLVariable(prefix = "java.")
	private String missingList = "Missing";
	
	@XSLVariable(prefix = "java.")
	private String countText = "Count";
	
	@XSLVariable(prefix = "java.")
	private String alternativesText = "Alternative";
	
	private AnnotatedDAO<AlternativeListRadioButtonQuery> queryDAO;
	private AnnotatedDAO<AlternativeListRadioButtonQueryInstance> queryInstanceDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;
	
	private AlternativeListRadioButtonQueryCRUD queryCRUD;
	
	private QueryParameterFactory<AlternativeListRadioButtonQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<AlternativeListRadioButtonQuery, String> listNameParamFactory;
	private QueryParameterFactory<AlternativeListRadioButtonQueryInstance, Integer> queryInstanceIDParamFactory;
	
	@InstanceManagerDependency(required = true)
	private AlternativeListHandlerModule alternativeListHandlerModule;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AlternativeListRadioButtonQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);
		
		queryDAO = daoFactory.getDAO(AlternativeListRadioButtonQuery.class);
		queryInstanceDAO = daoFactory.getDAO(AlternativeListRadioButtonQueryInstance.class);
		
		queryCRUD = new AlternativeListRadioButtonQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<AlternativeListRadioButtonQuery>(AlternativeListRadioButtonQuery.class), "AlternativeListRadioButtonQuery", "query", null, this);
		
		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		listNameParamFactory = queryDAO.getParamFactory("listName", String.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		
		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());
		
		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}
	
	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {
		
		AlternativeListRadioButtonQuery query = new AlternativeListRadioButtonQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, null);
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {
		
		AlternativeListRadioButtonQuery query = new AlternativeListRadioButtonQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);
		
		this.queryDAO.add(query, transactionHandler, null);
		
		return query;
	}
	
	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {
		
		AlternativeListRadioButtonQuery query = this.getQuery(descriptor.getQueryID(), extraData);
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		AlternativeListRadioButtonQuery query = this.getQuery(descriptor.getQueryID(), true, transactionHandler);
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {
		
		AlternativeListRadioButtonQueryInstance queryInstance;
		
		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance = new AlternativeListRadioButtonQueryInstance();
			
		} else {
			
			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());
			
			if (queryInstance == null) {
				
				return null;
			}
		}
		
		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID(), true));
		
		if (queryInstance.getQuery() == null) {
			
			return null;
		}
		
		if (req != null) {
			
			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);
			
			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());
		
		queryInstance.set(descriptor);
		
		//If this is a new query instance copy the default values
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance.copyQueryValues();
			queryInstance.getQuery().setAlternativeList(getAlternativeListInstance(queryInstance.getQuery(), null, poster));
		}
		
		return queryInstance;
	}
	
	@Override
	public ImmutableQueryInstance getImmutableQueryInstance(MutableQueryInstanceDescriptor descriptor, HttpServletRequest req, InstanceMetadata instanceMetadata) throws Throwable {
		
		if (descriptor.getQueryInstanceID() == null) {
			return null;
		}
		
		AlternativeListRadioButtonQueryInstance queryInstance = getQueryInstance(descriptor.getQueryInstanceID());
		
		if (queryInstance == null) {
			return null;
		}
		
		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID(), false));
		
		if (queryInstance.getQuery() == null) {
			return null;
		}
		
		if (req != null) {
			
			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);
			
			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());
		
		queryInstance.set(descriptor);
		
		return queryInstance;
	}
	
	@Override
	public QueryResponse getFormHTML(AlternativeListRadioButtonQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {
		
		if (queryInstance.getQuery().getAlternativeList() == null || queryInstance.getQuery().getAlternativeList().requiresReload(attributeHandler)) {
			
			queryInstance.getQuery().setAlternativeList(getAlternativeListInstance(queryInstance.getQuery(), attributeHandler, poster));
		}
		
		//Check if the chosen alternetive is still available in the list, otherwise reset the query
		if(queryInstance.getAlternative() != null) {
		
			Integer chosenAlternativeID = queryInstance.getAlternative().getAlternativeID();
			
			boolean chosenAlternativeStillInList = false;
			
			if (queryInstance.getQuery().getAlternativeList() != null) {
				
				List<? extends ImmutableAlternative> availableAlternatives = queryInstance.getQuery().getAlternativeList().getAlternatives();
				
				if (!CollectionUtils.isEmpty(availableAlternatives) && chosenAlternativeID != null) {
					
					for (ImmutableAlternative alternative : availableAlternatives) {
						
						if (alternative.getAlternativeID().equals(chosenAlternativeID)) {
							
							chosenAlternativeStillInList = true;
							break;
						}
					}
				}
			}
			
			if(!chosenAlternativeStillInList) {

				if(attributeHandler instanceof MutableAttributeHandler){
				
					queryInstance.reset((MutableAttributeHandler)attributeHandler);
				}
			}
		}
		
		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}
	
	private AlternativeListRadioButtonQuery getQuery(Integer queryID, boolean withAlternativeList) throws SQLException {
		
		HighLevelQuery<AlternativeListRadioButtonQuery> getQuery = new HighLevelQuery<AlternativeListRadioButtonQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		AlternativeListRadioButtonQuery query = queryDAO.get(getQuery);
		
		if (query != null && withAlternativeList) {
			query.setAlternativeList(getAlternativeListInstance(query, null, null));
		}
		
		return query;
	}
	
	private AlternativeListRadioButtonQuery getQuery(Integer queryID, boolean withAlternativeList, TransactionHandler transactionHandler) throws SQLException {
		
		HighLevelQuery<AlternativeListRadioButtonQuery> getQuery = new HighLevelQuery<AlternativeListRadioButtonQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		AlternativeListRadioButtonQuery query = queryDAO.get(getQuery, transactionHandler);
		
		if (query != null && withAlternativeList) {
			query.setAlternativeList(getAlternativeListInstance(query, null, null));
		}
		
		return query;
	}
	
	private AlternativeListRadioButtonQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {
		
		HighLevelQuery<AlternativeListRadioButtonQueryInstance> query = new HighLevelQuery<AlternativeListRadioButtonQueryInstance>(AlternativeListRadioButtonQueryInstance.QUERY_RELATION);
		
		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));
		
		return queryInstanceDAO.get(query);
	}
	
	@Override
	public void save(AlternativeListRadioButtonQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {
		
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {
			
			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());
			
			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);
			
		} else {
			
			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}
	
	@Override
	public void populate(AlternativeListRadioButtonQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {
		
		List<? extends ImmutableAlternative> availableAlternatives = null;
		
		if (queryInstance.getQuery().getAlternativeList() != null) {
			
			availableAlternatives = queryInstance.getQuery().getAlternativeList().getAlternatives();
		}
		
		if (CollectionUtils.isEmpty(availableAlternatives)) { // If the parent query doesn't have any alternatives then there is no population to do
			
			if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && !allowPartialPopulation) {
				
				throw new ValidationException(new ValidationError("RequiredQuery"));
				
			} else {
			
				queryInstance.reset(attributeHandler);
				return;
			}
		}
		
		Integer alternativeID = NumberUtils.toInt(req.getParameter("q" + queryInstance.getQuery().getQueryID() + "_alternative"));
		
		boolean alternativeSelected = false;
		
		ImmutableAlternative selectedAlternative = null;
		
		if (alternativeID != null) {
			
			for (ImmutableAlternative alternative : availableAlternatives) {
				
				if (alternative.getAlternativeID().equals(alternativeID)) {
					
					selectedAlternative = alternative;
					alternativeSelected = true;
					break;
				}
			}
		}
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		String freeTextAlternativeValue = null;
		
		if (queryInstance.getQuery().getFreeTextAlternative() != null && !alternativeSelected) {
			
			freeTextAlternativeValue = FreeTextAlternativePopulator.populate(queryInstance.getQuery().getQueryID(), "_alternative", req, validationErrors);
			
			if (freeTextAlternativeValue != null) {
				
				if (!validationErrors.isEmpty() && !ValidationUtils.containsValidationErrorWithMessageKey("FreeTextAlternativeValueRequired", validationErrors)) {
					
					allowPartialPopulation = false;
				}
				
				alternativeSelected = true;
			}
		}
		
		//If partial population is allowed, skip validation
		if (allowPartialPopulation) {
			
			queryInstance.setAlternative(selectedAlternative);
			queryInstance.setFreeTextAlternativeValue(freeTextAlternativeValue);
			queryInstance.getQueryInstanceDescriptor().setPopulated(alternativeSelected);
			
			if (queryInstance.getQuery().isSetAsAttribute()) {
				
				queryInstance.resetAttribute(attributeHandler);
				queryInstance.setAttribute(attributeHandler);
			}
			
			return;
		}
		
		//Check if this query is required and if the user has selected any alternative
		if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && !alternativeSelected) {
			
			validationErrors.add(new ValidationError("RequiredQuery"));
		}
		
		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
		
		queryInstance.setFreeTextAlternativeValue(freeTextAlternativeValue);
		queryInstance.setAlternative(selectedAlternative);
		queryInstance.getQueryInstanceDescriptor().setPopulated(alternativeSelected);
		
		if (queryInstance.getQuery().isSetAsAttribute()) {
			
			queryInstance.resetAttribute(attributeHandler);
			queryInstance.setAttribute(attributeHandler);
		}
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.queryCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {
		
		AlternativeListRadioButtonQuery query = getQuery(descriptor.getQueryID(), false);
		
		if (query == null) {
			
			return false;
		}
		
		this.queryDAO.delete(query, transactionHandler);
		
		return true;
	}
	
	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		AlternativeListRadioButtonQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());
		
		if (queryInstance == null) {
			
			return false;
		}
		
		this.queryInstanceDAO.delete(queryInstance, transactionHandler);
		
		return true;
	}
	
	@Override
	public String getTitlePrefix() {
		
		return this.moduleDescriptor.getName();
	}
	
	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {
		
		AlternativeListRadioButtonQuery query = getQuery(sourceQueryDescriptor.getQueryID(), false, transactionHandler);
		
		query.setQueryID(copyQueryDescriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, new RelationQuery());
	}
	
	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, AlternativeListRadioButtonQueryInstance queryInstance, AttributeHandler attributeHandler) {
		
		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);
		
		if (queryInstance.getQuery().getDescription() != null) {
			
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}
	
	@Override
	protected Class<AlternativeListRadioButtonQueryInstance> getQueryInstanceClass() {
		
		return AlternativeListRadioButtonQueryInstance.class;
	}
	
	public String getMissingAlternativeList() {
		
		return missingList;
	}
	
	protected Collection<AlternativeListDescriptor> getAlternativeLists() {
		
		return alternativeListHandlerModule.getAlternativeLists();
	}
	
	protected AlternativeListProvider getAlternativeListProvider(String providerID) {
		
		return alternativeListHandlerModule.getProvider(providerID);
	}
	
	protected AlternativeListDescriptor getAlternativeListDescriptor(AlternativeListQuery query) {
		
		if (!StringUtils.isEmpty(query.getAlternativeListName())) {
			
			return alternativeListHandlerModule.getAlternativeList(query.getAlternativeListName());
		}
		
		return null;
	}
	
	protected AlternativeListInstance getAlternativeListInstance(AlternativeListQuery query, AttributeHandler attributeHandler, User poster) {
		
		AlternativeListDescriptor descriptor = getAlternativeListDescriptor(query);
		
		if (descriptor != null) {
			
			return alternativeListHandlerModule.getListInstance(descriptor, attributeHandler, poster);
		}
		
		return null;
	}
	
	@Override
	public Collection<? extends AlternativeListQuery> getQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) {
		
		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<Integer>(queryDAO.getDataSource(), "SELECT queryID FROM " + queryDAO.getTableName() + " WHERE listName = ?", IntegerPopulator.getPopulator());
			idQuery.setString(1, descriptor.getName());
			
			List<Integer> queryIDs = idQuery.executeQuery();
			
			if (!CollectionUtils.isEmpty(queryIDs)) {
				
				List<AlternativeListRadioButtonQuery> queries = new ArrayList<AlternativeListRadioButtonQuery>(queryIDs.size());
				
				for (Integer queryID : queryIDs) {
					
					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(queryID);
					
					if (queryDescriptor != null) {
						
						queries.add((AlternativeListRadioButtonQuery) getQuery(queryDescriptor, false));
					}
				}
				
				return queries;
			}
			
		} catch (SQLException e) {
			
			log.error("Unable to get queries using alternative list " + descriptor, e);
		}
		
		return null;
	}
	
	
	@Override
	public boolean hasQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) throws SQLException {

		HighLevelQuery<AlternativeListRadioButtonQuery> query = new HighLevelQuery<>();
		query.addParameter(listNameParamFactory.getParameter(descriptor.getName()));
		query.setRowLimiter(MySQLRowLimiter.SINGLE_ROW);
		return queryDAO.getBoolean(query);
	}	
}
