<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/common/js/queryadmin.js
		/js/tablequeryadmin.js?v=4
	</xsl:variable>

	<xsl:template match="Column" mode="list">
				
		<div class="floatleft hover border full marginbottom border-radius lightbackground">
			
			<div class="padding floatleft thirty">
				<div><xsl:value-of select="label" /></div>
			</div>
			<div class="padding floatleft" style="width: 100px;">
				<xsl:variable name="columnType" select="columnType"/>
				<div><xsl:value-of select="../../../ColumnType[value=$columnType]/name" /></div>
			</div>
			<div class="padding floatleft text-align-center" style="width: 80px;">
				<xsl:if test="required='true'">
					<div><xsl:value-of select="$i18n.Required" /></div>
				</xsl:if>
			</div>
			<div class="padding floatleft text-align-center" style="width: 80px;">
				<xsl:if test="summarise='true'">
					<div><xsl:value-of select="$i18n.Summarised" /></div>
				</xsl:if>
			</div>
			
			<div class="padding floatright">
				<div class="floatright marginleft marginright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletecolumn/{columnID}" onclick="return confirm('{$i18n.DeleteColumn.Confirm}: {label}?');" title="{$i18n.DeleteColumn}: {label}">
						<img src="{$imagePath}/delete.png"/>
					</a>
				</div>
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatecolumn/{columnID}" title="{$i18n.UpdateColumn}: {label}">
						<img src="{$imagePath}/pen.png"/>
					</a>
				</div>
			</div>
			
		</div>
		
	</xsl:template>
	
	<xsl:template name="AddTableColumn">
	
		<h1><xsl:value-of select="$i18n.AddColumn" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
	
		<form id="addColumnForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
			
			<xsl:call-template name="showColumnForm" />
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.Add}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template name="UpdateTableColumn">
	
		<h1><xsl:value-of select="$i18n.UpdateColumn" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="Column/label" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="udpateColumnForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="showColumnForm">
				<xsl:with-param name="element" select="Column" />
			</xsl:call-template>
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template name="showColumnForm">
		
		<xsl:param name="element" select="null" />
		
		<script type="text/javascript">
			i18n = {
				"DropdownEmptyAlternativeDefault": "<xsl:value-of select="$i18n.DropdownEmptyAlternativeDefault"/>",
			}
		</script>
		
		<div class="floatleft full bigmarginbottom">
			<label for="label" class="floatleft clearboth"><xsl:value-of select="$i18n.Label" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'label'"/>
					<xsl:with-param name="name" select="'label'"/>
					<xsl:with-param name="title" select="$i18n.Label"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'required'" />
					<xsl:with-param name="name" select="'required'" />
					<xsl:with-param name="element" select="$element" /> 
					<xsl:with-param name="class" select="'vertical-align-middle'" />
				</xsl:call-template>
				
				<label for="required">
					<xsl:value-of select="$i18n.RequiredIfRowRequired" />
				</label>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'summarise'" />
					<xsl:with-param name="name" select="'summarise'" />
					<xsl:with-param name="element" select="$element" /> 
					<xsl:with-param name="class" select="'vertical-align-middle'" />
				</xsl:call-template>
					
				<label for="summarise">
					<xsl:value-of select="$i18n.Summarise" />
				</label>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="width" class="floatleft clearboth"><xsl:value-of select="$i18n.Width" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'width'"/>
					<xsl:with-param name="name" select="'width'"/>
					<xsl:with-param name="title" select="$i18n.Width"/>
					<xsl:with-param name="size" select="'30'" />
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
				<label for="columnType" class="floatleft clearboth"><xsl:value-of select="$i18n.ColumnType" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'columnType'"/>
						<xsl:with-param name="name" select="'columnType'"/>
						<xsl:with-param name="valueElementName" select="'value'" />
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="element" select="ColumnType" />
						<xsl:with-param name="selectedValue" select="$element/columnType" />
						<xsl:with-param name="width" select="'320px'" />
					</xsl:call-template>
		    </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="minContentLength" class="floatleft clearboth"><xsl:value-of select="$i18n.Field.MinLength" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'minContentLength'"/>
					<xsl:with-param name="name" select="'minContentLength'"/>
					<xsl:with-param name="title" select="$i18n.Field.MinLength"/>
					<xsl:with-param name="size" select="'30'"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="maxContentLength" class="floatleft clearboth"><xsl:value-of select="$i18n.MaxLength" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'maxContentLength'"/>
					<xsl:with-param name="name" select="'maxContentLength'"/>
					<xsl:with-param name="title" select="$i18n.MaxLength"/>
					<xsl:with-param name="size" select="'30'"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>

		<div class="floatleft full bigmarginbottom">
			<label for="textAreaRows" class="floatleft clearboth"><xsl:value-of select="$i18n.TextAreaRows" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'textAreaRows'"/>
					<xsl:with-param name="name" select="'textAreaRows'"/>
					<xsl:with-param name="title" select="$i18n.TextAreaRows"/>
					<xsl:with-param name="size" select="'30'"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="formatValidator" class="floatleft clearboth"><xsl:value-of select="$i18n.FormatValidator" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="id" select="'formatValidator'" />
					<xsl:with-param name="name" select="'formatValidator'" />
					<xsl:with-param name="valueElementName" select="'className'" />
					<xsl:with-param name="labelElementName" select="'name'" />
					<xsl:with-param name="element" select="FormatValidator" />
					<xsl:with-param name="addEmptyOption" select="'&#160;'" />
					<xsl:with-param name="selectedValue" select="$element/formatValidator" />
				</xsl:call-template>
				<xsl:apply-templates select="FormatValidator" />
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="invalidFormatMessage" class="floatleft clearboth"><xsl:value-of select="$i18n.InvalidFormatMessage" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'invalidFormatMessage'"/>
					<xsl:with-param name="name" select="'invalidFormatMessage'"/>
					<xsl:with-param name="title" select="$i18n.InvalidFormatMessage"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div class="floatleft full bigmarginbottom" id="dropdownEmptyAlternative">
			<label for="dropdownChooseText" class="floatleft clearboth"><xsl:value-of select="$i18n.DropdownEmptyAlternative" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'dropdownChooseText'"/>
					<xsl:with-param name="name" select="'dropdownChooseText'"/>
					<xsl:with-param name="title" select="$i18n.DropdownEmptyAlternative"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
		<div id="dropdownAlternatives">
			<xsl:call-template name="createAlternativesForm">
				<xsl:with-param name="alternatives" select="$element/DropDownAlternatives/DropDownAlternative" />
				<xsl:with-param name="allowFreeTextAlternative" select="'false'" />
			</xsl:call-template>
		</div>
		
		<div id="alternativeListAlternatives" class="floatleft full bigmarginbottom">

			<label for="listName">
				<xsl:value-of select="$i18n.listName" />
			</label>

			<div class="floatleft full">
			
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="id" select="'listName'"/>
					<xsl:with-param name="name" select="'listName'"/>
					<xsl:with-param name="element" select="AlternativeLists/AlternativeList"/>
					<xsl:with-param name="labelElementName" select="'Name'"/>
					<xsl:with-param name="valueElementName" select="'ID'"/>
					<xsl:with-param name="selectedValue" select="CurrentAlternativeList"/>
					<xsl:with-param name="class" select="'full'"/>
				</xsl:call-template>
			
			</div>
		</div>

		<div id="rowSumAddColumns">
			<label class="floatleft clearboth"><xsl:value-of select="$i18n.RowSum.AddColumns" /></label>
			<p class="floatleft clearboth add">
				<xsl:value-of select="$i18n.RowSum.AddColumns.description" />
			</p>
			<p class="floatleft clearboth multiply">
				<xsl:value-of select="$i18n.RowSum.AddMultiplyColumns.description" />
			</p>
			
			<ol class="clearboth">
				<xsl:apply-templates select="Columns/Column" mode="RowSumAdd">
					<xsl:with-param name="element" select="$element" />
				</xsl:apply-templates>
			</ol>
		</div>
		
		<div id="rowSumSubtractColumns">
			<label class="floatleft clearboth"><xsl:value-of select="$i18n.RowSum.SubtractColumns" /></label>
			<p class="floatleft clearboth add">
				<xsl:value-of select="$i18n.RowSum.SubtractColumns.description" />
			</p>
			<p class="floatleft clearboth multiply">
				<xsl:value-of select="$i18n.RowSum.SubtractMultiplyColumns.description" />
			</p>
			
			<ol class="clearboth">
				<xsl:apply-templates select="Columns/Column" mode="RowSumSubtract">
					<xsl:with-param name="element" select="$element" />
				</xsl:apply-templates>
			</ol>
		</div>
		
		<div id="listSearchSettings">
		
			<div class="floatleft full bigmarginbottom">
				<label for="placeholderText" class="floatleft clearboth"><xsl:value-of select="$i18n.PlaceholderText" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'placeholderText'"/>
						<xsl:with-param name="name" select="'placeholderText'"/>
						<xsl:with-param name="title" select="$i18n.PlaceholderText"/>
						<xsl:with-param name="element" select="$element" />
					</xsl:call-template>
				</div>
			</div>	
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'showAlternativesOnEmptySearch'" />
						<xsl:with-param name="name" select="'showAlternativesOnEmptySearch'" />
						<xsl:with-param name="element" select="$element" /> 
						<xsl:with-param name="class" select="'vertical-align-middle'" />
					</xsl:call-template>
						
					<label for="showAlternativesOnEmptySearch">
						<xsl:value-of select="$i18n.ShowAlternativesOnEmptySearch" />
					</label>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'showExactMatchesFirst'" />
						<xsl:with-param name="name" select="'showExactMatchesFirst'" />
						<xsl:with-param name="element" select="$element" /> 
						<xsl:with-param name="class" select="'vertical-align-middle'" />
					</xsl:call-template>
						
					<label for="showExactMatchesFirst">
						<xsl:value-of select="$i18n.ShowExactMatchesFirst" />
					</label>
				</div>
			</div>
					
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="helpText" class="floatleft clearboth"><xsl:value-of select="$i18n.HelpText" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'helpText'"/>
					<xsl:with-param name="name" select="'helpText'"/>
					<xsl:with-param name="title" select="$i18n.HelpText"/>
					<xsl:with-param name="element" select="$element" />
				</xsl:call-template>
		  </div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="DropDownAlternative">
		
		<xsl:call-template name="createAlternative">
			<xsl:with-param name="alternativeID" select="alternativeID" />
			<xsl:with-param name="sortOrder" select="sortIndex" />
			<xsl:with-param name="value" select="name" />
		</xsl:call-template>
			
	</xsl:template>
	
	<xsl:template match="Column" mode="RowSumAdd">
		<xsl:param name="element" />
	
		<xsl:variable name="disabled" select="preceding-sibling::*[columnID = $element/columnID] or columnID = $element/columnID" />

		<li>
			<xsl:call-template name="createCheckbox">
				<xsl:with-param name="id">
					<xsl:value-of select="'rowsumadd'"/>
					<xsl:value-of select="columnID"/>
				</xsl:with-param>
				<xsl:with-param name="name" select="'rowsumadd'" />
				<xsl:with-param name="value" select="columnID" />
				<xsl:with-param name="disabled" select="$disabled" />
				<xsl:with-param name="checked" select="$element/RowSumAddedColumns/value = columnID" />
				<xsl:with-param name="requestparameters" select="../../requestparameters" />
			</xsl:call-template>

			<label for="rowsumadd{columnID}">
				<xsl:if test="$disabled">
					<xsl:attribute name="title"><xsl:value-of select="$i18n.RowSum.PreOnly"/></xsl:attribute>
				</xsl:if>
				
				<xsl:value-of select="label" />
			</label>
		</li>
	
	</xsl:template>
	
	<xsl:template match="Column" mode="RowSumSubtract">
		<xsl:param name="element" />
		
		<xsl:variable name="disabled" select="preceding-sibling::*[columnID = $element/columnID] or columnID = $element/columnID" />
		
		<li>
			<xsl:call-template name="createCheckbox">
				<xsl:with-param name="id">
					<xsl:value-of select="'rowsumsubtract'"/>
					<xsl:value-of select="columnID"/>
				</xsl:with-param>
				<xsl:with-param name="name" select="'rowsumsubtract'" />
				<xsl:with-param name="value" select="columnID" />
				<xsl:with-param name="disabled" select="$disabled" />
				<xsl:with-param name="checked" select="$element/RowSumSubtractedColumns/value = columnID" />
				<xsl:with-param name="requestparameters" select="../../requestparameters" />
			</xsl:call-template>

			<label for="rowsumsubtract{columnID}">
				<xsl:if test="$disabled">
					<xsl:attribute name="title"><xsl:value-of select="$i18n.RowSum.PreOnly"/></xsl:attribute>
				</xsl:if>
				
				<xsl:value-of select="label" />
			</label>
		</li>
	
	</xsl:template>
	
	<xsl:template match="FormatValidator">
		
		<xsl:call-template name="createHiddenField">
			<xsl:with-param name="id" select="concat('validatorMessage-', formatValidatorID)" />
			<xsl:with-param name="name" select="concat('validatorMessage-', formatValidatorID)" />
			<xsl:with-param name="value" select="validationMessage" />
		</xsl:call-template>
		
	</xsl:template>
	
	<xsl:template match="SortColumns">
		
		<h1><xsl:value-of select="$i18n.SortColumns" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="*[contains(name(), 'TableQuery')]/QueryDescriptor/name" /></h1>
		
		<form id="sortColumnsForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<div class="floatleft full sortable">
			
				<xsl:apply-templates select="*[contains(name(), 'TableQuery')]/Columns/Column" mode="sort" />
			
			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>
		
	</xsl:template>
	
	<xsl:template match="Column" mode="sort">
	
		<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
	
		<div id="column_{columnID}" class="floatleft hover border full marginbottom lightbackground cursor-move border-radius">
			<div class="padding">
				<img class="vertical-align-middle marginright" src="{$commonImagePath}/move.png" title="{$i18n.MoveAlternative}"/>
				<xsl:value-of select="label" />
				<xsl:call-template name="createHiddenField">
					<xsl:with-param name="id" select="concat('sortorder_', columnID)" />
					<xsl:with-param name="name" select="concat('sortorder_', columnID)" />
					<xsl:with-param name="value" select="sortIndex" />
					<xsl:with-param name="class" select="'sortorder'" />
					<xsl:with-param name="requestparameters" select="//requestparameters" />
				</xsl:call-template>
			</div>
		</div>
	
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />
	
		<xsl:choose>
			<xsl:when test="$fieldName = 'minContentLength'">
				<xsl:value-of select="$i18n.Field.minLength" />
			</xsl:when>
			<xsl:when test="$fieldName = 'maxContentLength'">
				<xsl:value-of select="$i18n.maxLength" />
			</xsl:when>
			<xsl:when test="$fieldName = 'label'">
				<xsl:value-of select="$i18n.label" />
			</xsl:when>
			<xsl:when test="$fieldName = 'width'">
				<xsl:value-of select="$i18n.width" />
			</xsl:when>
			<xsl:when test="$fieldName = 'invalidFormatMessage'">
				<xsl:value-of select="$i18n.invalidFormatMessage" />
			</xsl:when>
			<xsl:when test="$fieldName = 'columnType'">
				<xsl:value-of select="$i18n.columnType" />
			</xsl:when>
			<xsl:when test="$fieldName = 'listName'">
				<xsl:value-of select="$i18n.listName" />
			</xsl:when>
			<xsl:when test="$fieldName = 'placeholderText'">
				<xsl:value-of select="$i18n.placeholderText" />
			</xsl:when>
			<xsl:when test="$fieldName = 'helpText'">
				<xsl:value-of select="$i18n.helpText" />
			</xsl:when>
			<xsl:when test="$fieldName = 'RowSumSelection'">
				<xsl:value-of select="$i18n.RowSum.Selection" />
			</xsl:when>
			<xsl:when test="$fieldName = 'rowsumadd'">
				<xsl:value-of select="$i18n.RowSum.AddColumns" />
			</xsl:when>
			<xsl:when test="$fieldName = 'rowsumsubtract'">
				<xsl:value-of select="$i18n.RowSum.SubtractColumns" />
			</xsl:when>
			<xsl:when test="$fieldName = 'decimalsToShow'">
				<xsl:value-of select="$i18n.Query.decimalsToShow.update" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="fieldName">
					<xsl:with-param name="fieldName" select="$fieldName"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

</xsl:stylesheet>