package com.nordicpeak.flowengine.queries.basetablequery;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public abstract class TableQueryCRUD<DropAlternative extends BaseDropDownAlternative, CellValue extends TableCellValue<DropAlternative, Query, Row, Column>, Row extends TableRow<DropAlternative, Column, Query>, QueryInstance extends BaseTableQueryInstance<DropAlternative, Row, Column, Query, CellValue>, Column extends BaseTableColumn<DropAlternative, Column, Query>, Query extends BaseTableQuery<DropAlternative, Query, Column>, QueryProvider extends TableQueryProviderModule<DropAlternative, Column, Query, Row, CellValue, QueryInstance, QueryProvider>> extends BaseQueryCRUD<Query, QueryProvider> {

	protected AnnotatedDAOWrapper<Query, Integer> queryDAO;

	public TableQueryCRUD(Class<Query> queryClass, AnnotatedDAOWrapper<Query, Integer> queryDAO, BeanRequestPopulator<Query> populator, String typeElementName, String typeLogName, String listMethodAlias, QueryProvider callback) {

		super(queryClass, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.queryDAO = queryDAO;
	}

	public ForegroundModuleResponse sortColumns(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Query query = this.getRequestedBean(req, res, user, uriParser, UPDATE);

		if (query != null) {

			this.checkUpdateAccess(query, user, req, uriParser);

			if (req.getMethod().equalsIgnoreCase("POST")) {

				List<Column> tableColumns = query.getColumns();

				if (!CollectionUtils.isEmpty(tableColumns)) {

					for (Column tableColumn : tableColumns) {

						String sortIndex = req.getParameter("sortorder_" + tableColumn.getColumnID());

						if (NumberUtils.isInt(sortIndex)) {

							tableColumn.setSortIndex(NumberUtils.toInt(sortIndex));
						}
					}

					queryDAO.update(query);

					callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) query.getQueryDescriptor()), EventTarget.ALL);

					callback.redirectToQueryConfig(query, req, res);

					return null;
				}
			}

			Document doc = this.callback.createDocument(req, uriParser, user);
			Element updateTypeElement = doc.createElement("SortColumns");
			doc.getFirstChild().appendChild(updateTypeElement);

			updateTypeElement.appendChild(query.toXML(doc));

			return new SimpleForegroundModuleResponse(doc);
		}

		throw new URINotFoundException(uriParser);
	}

	@Override
	protected Query populateFromUpdateRequest(Query bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		Query query = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<>();

		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		if (query.getDecimalsToShow() != null && query.getDecimalsToShow() > 5) {
			validationErrors.add(new ValidationError("decimalsToShow", ValidationErrorType.TooLong));
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}

	@Override
	protected void appendShowFormData(Query bean, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {

		super.appendShowFormData(bean, doc, showTypeElement, user, req, res, uriParser);

		XMLUtils.appendNewElement(doc, showTypeElement, "showFlowURL", callback.getFlowAdminModule().getFlowQueryRedirectURL(req, bean.getQueryDescriptor().getStep().getFlow().getFlowID()));

		appendColumnTypes(doc, showTypeElement);
	}

	@Override
	protected void appendUpdateFormData(Query bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);

		appendColumnTypes(doc, updateTypeElement);

		updateTypeElement.appendChild(bean.getQueryDescriptor().getStep().getFlow().getFlowType().toXML(doc));
	}

	@Override
	protected ForegroundModuleResponse beanUpdated(Query bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQueryDescriptor()), EventTarget.ALL);

		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQueryDescriptor().getName() + "\"", bean.getQueryDescriptor().getStep().getFlow(), user);

		callback.redirectToQueryConfig(bean, req, res);

		return null;

	}

	protected void appendColumnTypes(Document doc, Element element) {

		for (ColumnType layout : ColumnType.values()) {

			Element fieldLayoutElement = doc.createElement("ColumnType");
			element.appendChild(fieldLayoutElement);

			XMLUtils.appendNewElement(doc, fieldLayoutElement, "name", callback.getColumnTypeName(layout));
			XMLUtils.appendNewElement(doc, fieldLayoutElement, "value", layout);
		}
	}

	@Override
	protected void checkShowAccess(Query query, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		super.checkShowAccess(query, user, req, uriParser);

		callback.checkUpdateQueryAccess(user, query);
	}

}
