(function () {
	if (typeof tableQueryQueuedAjaxRequests === "undefined") {
		tableQueryQueuedAjaxRequests = {};
	}
}) ();

function initTableQuery(queryID, queryRequestURL) {
	
	var query = $("#query_" + queryID);
	var table = query.find("table.tablequery");
	
	tableQueryQueuedAjaxRequests[queryID] = [];
	
	initTableQueryValidationErrors(query);
	
	table.find("td.rowsum").each(function () {
		
		var td = $(this);
		var tr = td.parent();
		var span = td.find("span");
		var columnID = span.data("columnid");
		var rowSumInput = span.siblings("input");
		
		updateTableQueryRowSum(span, tr, columnID);
		
		tr.children("td").each(function(loopIndex, cell){
			var inputs = $(cell).find("input").not(rowSumInput);
			
			addTableQueryRowListener(inputs, span, tr, columnID);
		});
	});
	
	table.find("td[data-summarise='true']").each(function () {
		
		var td = $(this);
		var columnIndex = td.data("columnindex");
		
		updateTableQuerySummary(td, columnIndex);
		
		td.closest("table").children("tbody").children("tr").each(function(ignore, row){
			$(row).children("td").each(function(loopIndex, cell){
			
				if (loopIndex == columnIndex) {
					
					var inputs = $(cell).find("input");
					
					addTableQueryColumnListener(inputs, td, columnIndex);
				}
			})
		});
	});
	
	if (!Modernizr.inputtypes.date) {
		
		table.find("input[type='date']").datepicker({
			changeMonth: true,
			changeYear: true,
			showWeek: true,
			dayNamesShort: true,
			onSelect: function() {
				$(this).trigger("blur").trigger("change");
			}
		});
	}
	
	table.find("textarea").each(function() {
		this.style.height = (this.scrollHeight) + "px";
		
	}).on("input", function () {
		this.style.height = "auto";
		this.style.height = (this.scrollHeight) + "px";
	});
	
	if (query.hasClass("enableAjaxPosting")) {
		
		bindTableQueryCellChangeEvent(query.find("table.tablequery tbody tr"), query, queryID);
	}
	
	addListSearchListeners(query, queryRequestURL);
}

function processNextTableQueryAjaxRequest(queryID) {
	var queryAjaxQueue = tableQueryQueuedAjaxRequests[queryID];
	
	if (queryAjaxQueue.length > 0) {
		var nextAjax = queryAjaxQueue[0];
		
		nextAjax().always(function(){
			queryAjaxQueue.shift();
			processNextTableQueryAjaxRequest(queryID);
		});
	}
}

function tableQueryCellChangeEventListener() {
	
	var input = $(this);
	
	var query = input.closest(".query");
	var queryID = query.prop("id").substring("query_".length);
	
	runTableQueryEvaluators(query, queryID);
}
	
function bindTableQueryCellChangeEvent(rows, query, queryID) {
	
	rows.find("input, select, textarea").on("change", tableQueryCellChangeEventListener);
}

function unBindTableQueryCellChangeEvent(rows) {
	
	rows.find("input, select, textarea").off("change", tableQueryCellChangeEventListener);
}

function runTableQueryEvaluators(query, queryID) {
	
	var runImmediately = tableQueryQueuedAjaxRequests[queryID].length === 0;
	
	tableQueryQueuedAjaxRequests[queryID].push(function(){
		
		// Can still be run before column sums triggered by rowsum changes if user is very quick or has a slow connection. 
		
		var parameters = [];
		
		query.find("table.tablequery tbody").find("input, select, textarea").each(function () {
			
			var input = $(this);
			
			if (input.parent().hasClass("input-error")) {
				
				input.find("i[title]").tooltip("destroy");
				input.parent().removeClass("invalid input-error").find("i[title]").remove();
			}
			
			if (input.prop("type") == "checkbox") {
				
				if (input.prop('checked')) {
					
					parameters.push({name: input.attr("name"), value: input.val()});
				}
				
			} else {
				
				parameters.push({name: input.attr("name"), value: input.val()});
			}
		});
		
		parameters.push({name: "queryID", value: queryID});
	
		runQueryEvaluators(queryID, parameters);
		
		return currentAjaxRequest;
	});
	
	if (runImmediately) {
		processNextTableQueryAjaxRequest(queryID);
	}
}

function addTableQueryRowListener(inputs, span, tr, columnID) {
	
	// Avoids double update from keyup event and then change event when defocusing input.
	// Both events needed for responsive UI and when editing via ex right click menu. 
	var lastInput = null;
	var lastInputValue = null;
	
	inputs.on("keyup change", function(event){
		var input = this;
		var inputValue = input.type == "checkbox" ? input.checked : input.value;
		
		if (input !== lastInput || inputValue != lastInputValue) {
			lastInput = input;
			lastInputValue = inputValue;
			
			updateTableQueryRowSum(span, tr, columnID);
		}
	});
}

var tableQueryColumnLastInput = null;
var tableQueryColumnLastInputValue = null;

function addTableQueryColumnListener(inputs, td, columnIndex) {
	
	var lastInput = null;
	var lastInputValue = null;
	
	inputs.on("keyup change", function(event){
		var input = this;
		var inputValue = input.type == "checkbox" ? input.checked : input.value;
		
		if (input !== lastInput || inputValue != lastInputValue) {
			lastInput = input;
			lastInputValue = inputValue;
			
			updateTableQuerySummary(td, columnIndex);
		}
	});
	
	inputs.on("sumchange", function(event){
		updateTableQuerySummary(td, columnIndex);
	});
}

function updateTableQuerySummary(td, columnIndex) {
	
	var table = td.closest("table");
	var query = table.closest(".query");
	var queryID = query.attr("id").substring(6);
	
	var runImmediately = tableQueryQueuedAjaxRequests[queryID].length === 0;
	
	tableQueryQueuedAjaxRequests[queryID].push(function() {
		var parameters = [];
		parameters.push({name: "queryID", value: queryID});
		
		table.children("tbody").children("tr").each(function(ignore, row) {
			$(row).children("td").each(function(loopIndex, cell) {
			
				if (loopIndex == columnIndex) {
					
					var input = $(cell).find("input, select");
					
					if (input.is("input[type='checkbox']")) {
						
						parameters.push({name: "checkbox", value: input.prop('checked')});
						
					} else {
						
						parameters.push({name: "value", value: input.val()});
					}
				}
			})
		});
		
		return $.ajax({
			type: "POST",
			cache: false,
			url: query.find("input[name='summaryURL']").val(),
			data: parameters,
			dataType: "json",
			contentType: "application/x-www-form-urlencoded;charset=UTF-8",
			
		}).done(function(result) {
			
			td.find("span").text(result.Sum);
			
		}).fail(function(jqXHR) {
			
			td.find("span").text("?");
		});
	});
	
	if (runImmediately) {
		processNextTableQueryAjaxRequest(queryID);
	}
}

function updateTableQueryRowSum(span, tr, columnID) {
	
	var table = tr.closest("table");
	var query = table.closest(".query");
	var queryID = query.attr("id").substring(6);
	var rowSumInput = span.siblings("input");
	
	var oldValue = rowSumInput.val();
	
	var runImmediately = tableQueryQueuedAjaxRequests[queryID].length === 0;
	
	tableQueryQueuedAjaxRequests[queryID].push(function() {
		var parameters = [];
		parameters.push({name: "queryID", value: queryID});
		parameters.push({name: "columnID", value: span.data("columnid")});
		parameters.push({name: "rowID", value: span.data("rowid")});
		
		var inputs = tr.children("td").find("input").not(rowSumInput);
		
		inputs.each(function(ignore, input){
			input = $(input);
			
			if (input.is("input[type='checkbox']")) {
				
				parameters.push({name: input.prop("name"), value: input.prop('checked')});
				
			} else {
				
				parameters.push({name: input.prop("name"), value: input.val()});
			}
		});
		
		return $.ajax({
			type: "POST",
			cache: false,
			url: query.find("input[name='rowsumURL']").val(),
			data: parameters,
			dataType: "json",
			contentType: "application/x-www-form-urlencoded;charset=UTF-8",
			
		}).done(function(result) {
			
			span.text(result.Sum);
			rowSumInput.val(result.Sum);
			
			if (oldValue != rowSumInput.val()) {
				rowSumInput.trigger("sumchange");
			}
			
		}).fail(function(jqXHR) {
			
			span.text("?");
			rowSumInput.val("?");
			
			if (oldValue != rowSumInput.val()) {
				rowSumInput.trigger("sumchange");
			}
		});
	});
	
	if (runImmediately) {
		processNextTableQueryAjaxRequest(queryID);
	}
}

function initTableQueryValidationErrors($query) {

// TODO re-enable when placement is correct
//	$query.find(".input-error input").tooltip({
//		position : {
//			my : "right top-38",
//			at : "right+3 top",
//			collision : "none"
//		},
//		track : false,
//		content : function() {
//			return $(this).next().prop("title") + "<span class=\"marker\"></span>";
//		},
//		show : {
//			effect : 'none'
//		},
//		hide : {
//			effect : 'none'
//		}
//	}).off("mouseover mouseout");
	
	$query.find("i[title]").tooltip({
		position: {
			my: "right+29 bottom-22",
			collision: "flipfit"
		},
		track: false,
		content: function () {
          return $(this).prop('title') + "<span class=\"marker\"></span>";
      	},
      	show: {
      		effect: 'none'
      	},
      	hide: {
      		effect: 'none'
      	}
	});
}

function makeTableQueryRequired(queryID) {
	
	var query = $("#query_" + queryID); 
	
	query.find(".heading-wrapper h2").addClass("required");
	query.find("table").addClass("required");
}

function setTableFieldValidationError(queryID, validationErrors) {
	
	var query = $("#query_" + queryID);
	var table = query.find("table.tablequery");
	
	$.each(validationErrors, function(i, error) {
		
		var input = query.find("input[name='q" + queryID + error.field + "'], select[name='q" + queryID + error.field + "'], textarea[name='q" + queryID + error.field + "']");
		
		if (input.length > 0) {
		
			input.parent().addClass("invalid input-error");
			
			var message = getValidationErrorMessage(error);
			
			input.parent().append($("<i data-icon-after='!'' title='" + message + "' />"));
		}
	});
	
	initTableQueryValidationErrors(query);
}

function addListSearchListeners($parent, queryRequestURL) {
	
	var $searchInputs = $parent.find("input.list-search");
	
	$searchInputs.each(function(){
		
		var $searchInput = $(this);
		var $hiddenInput = $searchInput.siblings(".hidden-list-search");
		var $deleteButton = $searchInput.siblings(".list-search-delete");
		var allowEmptySearch = $searchInput.hasClass("allowemptysearch");
		
		$searchInput.autocomplete({
			source: function(request, response) {
				return searchList(request, response, queryRequestURL, $searchInput);
			},
			
			select: function( event, ui ) {
				
				if (ui.item.value != "null") {
					
					$searchInput.val(ui.item.Name);
					$hiddenInput.val(ui.item.value);
					
					$searchInput.trigger("change").trigger("blur");
				}
				
				return false;
			},
			
			focus: function(event, ui) {
				event.preventDefault();
			},
			
			minLength: allowEmptySearch ? 0 : 1
		});
		
		if (allowEmptySearch) {
			
			$searchInput.on("focus click", function() {
				
				if ($searchInput.val() == "") {
					
					$searchInput.autocomplete("search", "");
				}
			});
		}
		
		$searchInput.on("change", function(){
			
			 var valueIsSelected = $searchInput.val() != "" && $hiddenInput.val() != "";
			
			$searchInput.toggleClass("value-selected", valueIsSelected);
			$deleteButton.toggleClass("visibility-hidden", !valueIsSelected);
		})
		
		$deleteButton.on("click", function(){
			
			if ($searchInput.prop("disabled")) {
				
				return;
			}
			
			$searchInput.val("");
			$hiddenInput.val("");

			$searchInput.trigger("change");
		});
		
		if ($searchInput.val() != "" && $hiddenInput.val() == "") {
			
			$searchInput.val("");
		}

		$searchInput.trigger("change");
	});
	
}

function searchList(request, response, url, $searchInput) {
	
	$searchInput.addClass("ui-autocomplete-loading");
	
	var $cell = $searchInput.closest("td");
	var columnID = $cell.data("column-id");
	
	$.ajax({
		url : url,
		dataType : "json",
		contentType: "application/x-www-form-urlencoded;charset=UTF-8",
		data : {
			q : encodeURIComponent(request.term),
			columnID : encodeURIComponent(columnID),
		},
		success : function(data) {
			
			if(data.hits != undefined && data.hits.length > 0) {
				
				response($.map(data.hits, function(item) {
					
					return {
						label : item.Name,
						value : item.ID,
						Name  : item.Name,
						ID : item.ID,
					}
				}));
				
			} else {
				
				response(null);
			}
			
			$searchInput.removeClass("ui-autocomplete-loading");
			
		},
		error : function() {
			
			$searchInput.removeClass("ui-autocomplete-loading");
		}
	});
}