package com.nordicpeak.flowengine.booking;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.function.Predicate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.poi.openxml4j.exceptions.OpenXML4JRuntimeException;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.WorkbookUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.UserMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.crud.FragmentLinkScriptFilter;
import se.unlogic.hierarchy.core.utils.crud.TransactionRequestFilter;
import se.unlogic.hierarchy.foregroundmodules.rest.RESTMethod;
import se.unlogic.hierarchy.foregroundmodules.rest.URIParam;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.QueryOperators;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.SwedishSocialSecurityPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.threads.ExeptionHandlingRunnable;
import se.unlogic.standardutils.threads.ThreadUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.http.enums.ContentDisposition;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.booking.beans.Booking;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingEvent;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.beans.ExternalMessage;
import com.nordicpeak.flowengine.booking.beans.InternalMessage;
import com.nordicpeak.flowengine.booking.beans.InternalMessageAttachment;
import com.nordicpeak.flowengine.booking.beans.ManualContact;
import com.nordicpeak.flowengine.booking.beans.PrimaryContact;
import com.nordicpeak.flowengine.booking.beans.Repeating;
import com.nordicpeak.flowengine.booking.comparators.BookingStartComparator;
import com.nordicpeak.flowengine.booking.cruds.BookingCRUD;
import com.nordicpeak.flowengine.booking.cruds.BookingSlotCRUD;
import com.nordicpeak.flowengine.booking.cruds.InternalMessageCRUD;
import com.nordicpeak.flowengine.booking.cruds.RepeatingCRUD;
import com.nordicpeak.flowengine.booking.enums.BookingEventType;
import com.nordicpeak.flowengine.booking.enums.ExternalMessageType;
import com.nordicpeak.flowengine.booking.enums.InternalMessageType;
import com.nordicpeak.flowengine.booking.enums.UserType;
import com.nordicpeak.flowengine.booking.events.BookingArchiveEvent;
import com.nordicpeak.flowengine.booking.events.BookingManuallyAddedEvent;
import com.nordicpeak.flowengine.booking.interfaces.BookingExtensionProvider;
import com.nordicpeak.flowengine.booking.interfaces.BookingSlotFormExtensionProvider;
import com.nordicpeak.flowengine.booking.notificationhandler.BookingNotificationHandler;
import com.nordicpeak.flowengine.booking.validationerrors.ErrorSendingToRecipientViaEmailValidationError;
import com.nordicpeak.flowengine.booking.validationerrors.InvalidRecipientEmailValidationError;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.personprovider.Person;
import com.nordicpeak.personprovider.PersonProvider;

public class BookingSlotAdminModule extends BaseBookingModule implements CRUDCallback<User>, AccessInterface {

	//@formatter:off
	private static final Field[] BOOKING_RELATIONS = { 
		Booking.CONTACT_RELATION, 
		Booking.MANUAL_CONTACT_RELATION, 
		Booking.BOOKING_SLOT_RELATION, 
		BookingSlot.MANAGERS_RELATION, 
		BookingSlot.CALENDAR_RELATION, 
		BookingCalendar.MANAGER_USERS_RELATION, 
		BookingCalendar.MANAGER_GROUPS_RELATION, 
		BookingCalendar.ADMIN_GROUPS_RELATION
	};
	
	private static final Field[] BOOKING_EXPORT_RELATIONS = { 
		Booking.MANUAL_CONTACT_RELATION, 
		Booking.BOOKING_SLOT_RELATION, 
		Booking.EXTERNAL_MESSAGES_RELATION, 
		Booking.EVENTS_RELATION,
		BookingSlot.CALENDAR_RELATION,
	};

	private static final Field[] ARCHIVE_ALL_BOOKINGS_RELATIONS = { 
		Booking.CONTACT_RELATION, 
		Booking.MANUAL_CONTACT_RELATION, 
		Booking.EXTERNAL_MESSAGES_RELATION, 
		Booking.EVENTS_RELATION, 
		Booking.BOOKING_SLOT_RELATION, 
		BookingSlot.MANAGERS_RELATION, 
		BookingSlot.CALENDAR_RELATION, 
		BookingCalendar.MANAGER_USERS_RELATION, 
		BookingCalendar.MANAGER_GROUPS_RELATION, 
		BookingCalendar.ADMIN_GROUPS_RELATION
	};
	
	private static final Field[] EXPORT_BOOKING_SLOT_RELATIONS = { 
		Booking.CONTACT_RELATION, 
		Booking.MANUAL_CONTACT_RELATION, 
		Booking.EXTERNAL_MESSAGES_RELATION, 
		Booking.EVENTS_RELATION,
		BookingSlot.BOOKINGS_RELATION, 
		BookingSlot.CALENDAR_RELATION, 
		BookingCalendar.MANAGER_USERS_RELATION, 
		BookingCalendar.MANAGER_GROUPS_RELATION, 
		BookingCalendar.ADMIN_GROUPS_RELATION
	};
	//@formatter:on

	public static final SwedishSocialSecurityPopulator CITIZEN_ID_POPULATOR = new SwedishSocialSecurityPopulator(true, true, false, true);

	private static final ValidationError NO_RECIPIENT_TYPES_VALIDATION_ERROR = new ValidationError("NoRecipientTypesSelected");
	private static final ValidationError NO_RECIPIENT_STATE_VALIDATION_ERROR = new ValidationError("NoRecipientStateSelected");
	private static final ValidationError EMAIL_SENDER_NOT_SET_VALIDATION_ERROR = new ValidationError("EmailSenderNotSet");

	private static final BookingStartComparator BOOKING_START_COMPARATOR = new BookingStartComparator();

	private static final BeanRequestPopulator<ManualContact> MANUAL_CONTACT_POPULATOR = new AnnotatedRequestPopulator<>(ManualContact.class);

	@XSLVariable(name = "i18n.BookingSlot.Date")
	private String columnBookingSlotDate;

	@XSLVariable(name = "i18n.BookingSlot.Time")
	private String columnBookingSlotTime;

	@XSLVariable(name = "i18n.BookingSlot.Size")
	private String columnBookingSlotSize;

	@XSLVariable(name = "i18n.BookingSlot.Location")
	private String columnBookingSlotLocation;

	@XSLVariable(name = "i18n.BookingSlot.Description")
	private String columnBookingSlotDescription;

	@XSLVariable(name = "i18n.Booking.FlowInstanceID")
	private String columnBookingFlowInstanceID;

	@XSLVariable(name = "i18n.Booking.Added")
	private String columnBookingAdded;

	@XSLVariable(name = "i18n.Booking.Rebooked")
	private String columnBookingRebooked;

	@XSLVariable(name = "i18n.Booking.Archived")
	private String columnBookingArchived;

	@XSLVariable(name = "i18n.Booking.Comment")
	private String columnBookingComment;

	@XSLVariable(name = "i18n.Booking.Digital")
	private String columnBookingDigital;

	@XSLVariable(name = "i18n.Booking.DigitalNotContactable")
	private String columnBookingDigitalNotContactable;

	@XSLVariable(name = "i18n.Booking.FlowInstanceURL")
	private String columnBookingFlowInstanceURL;

	@XSLVariable(name = "i18n.Booking.URL")
	private String columnBookingURL;

	@XSLVariable(name = "i18n.Yes")
	private String columnBookingYes;

	@XSLVariable(name = "i18n.No")
	private String columnBookingNo;

	@XSLVariable(name = "i18n.Booking.Preliminary")
	private String preliminaryBookingText;

	@XSLVariable(name = "i18n.Booking.NoFlowInstance")
	private String bookingNoFlowInstanceText;

	@XSLVariable(prefix = "java.")
	private String bookingSlotNotBookableTextOpens = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	private String bookingSlotNotBookableTextClosed = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	private String bookingSlotNotBookableTextBetween = "This variable should be set by your stylesheet";

	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Admin groups", description = "Groups allowed to administrate global parts of this module such as deleting booking slots")
	private List<Integer> adminGroupIDs;

	@ModuleSetting(allowsNull = true)
	@UserMultiListSettingDescriptor(name = "Admin users", description = "Users allowed to administrate global parts of this module such as deleting booking slots")
	private List<Integer> adminUserIDs;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Default archive text", description = "The default prefilled text when archiving bookings")
	private String defaultArchiveText;

	private AnnotatedDAO<BookingCalendar> calendarDAO;
	private AnnotatedDAO<ManualContact> manualContactDAO;

	private AnnotatedDAOWrapper<BookingCalendar, Integer> calendarDAOWrapper;
	private AdvancedAnnotatedDAOWrapper<Booking, Integer> bookingDAOWrapper;
	private AdvancedAnnotatedDAOWrapper<BookingSlot, Integer> bookingSlotDAOWrapper;

	private QueryParameterFactory<BookingSlot, BookingCalendar> bookingSlotCalendarParamFactory;
	private QueryParameterFactory<BookingSlot, Repeating> bookingSlotRepeatingParamFactory;
	private QueryParameterFactory<BookingSlot, Timestamp> bookingSlotStartParamFactory;
	private QueryParameterFactory<BookingCalendar, Boolean> calendarEnabledParamFactory;
	private QueryParameterFactory<Booking, BookingSlot> bookingBookingSlotParamFactory;

	private BookingCRUD bookingCRUD;
	private BookingSlotCRUD bookingSlotCRUD;
	private RepeatingCRUD repeatingCRUD;

	private InternalMessageCRUD internalMessageCRUD;

	private ManagerRequestProcessor managerRequestProcessor;

	private CopyOnWriteArrayList<BookingSlotFormExtensionProvider> bookingSlotExtensionProviders = new CopyOnWriteArrayList<>();
	private CopyOnWriteArrayList<BookingExtensionProvider> bookingExtensionProviders = new CopyOnWriteArrayList<>();

	@InstanceManagerDependency(required = true)
	private FlowAdminModule flowAdminModule;

	@InstanceManagerDependency(required = true)
	private FlowInstanceAdminModule flowInstanceAdminModule;

	@InstanceManagerDependency
	private PersonProvider personProvider;

	@InstanceManagerDependency(required = true)
	private PDFProvider pdfProvider;

	@InstanceManagerDependency(required = true)
	private CalendarAdminModule calendarAdminModule;

	@InstanceManagerDependency(required = true)
	private BookingNotificationHandler notificationHandler;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		managerRequestProcessor = new ManagerRequestProcessor(this, systemInterface.getEncoding());

		if (!systemInterface.getInstanceHandler().addInstance(BookingSlotAdminModule.class, this)) {

			throw new RuntimeException("Unable to register module in global instance handler using key " + BookingSlotAdminModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(BookingSlotAdminModule.class, this);

		super.unload();
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, true, false);

		calendarDAO = daoFactory.getDAO(BookingCalendar.class);
		manualContactDAO = daoFactory.getDAO(ManualContact.class);

		bookingSlotCalendarParamFactory = bookingSlotDAO.getParamFactory("calendar", BookingCalendar.class);
		bookingSlotRepeatingParamFactory = bookingSlotDAO.getParamFactory("repeating", Repeating.class);
		bookingSlotStartParamFactory = bookingSlotDAO.getParamFactory("start", Timestamp.class);
		calendarEnabledParamFactory = calendarDAO.getParamFactory("enabled", boolean.class);
		bookingBookingSlotParamFactory = bookingDAO.getParamFactory("bookingSlot", BookingSlot.class);

		AdvancedAnnotatedDAOWrapper<Repeating, Integer> repeatingDAOWrapper = daoFactory.getDAO(Repeating.class).getAdvancedWrapper(Integer.class);
		repeatingDAOWrapper.getGetQuery().addRelations(Repeating.CALENDAR_RELATION, BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION, BookingCalendar.ADMIN_GROUPS_RELATION);

		calendarDAOWrapper = calendarDAO.getWrapper(Integer.class);
		calendarDAOWrapper.addRelations(BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION, BookingCalendar.ADMIN_GROUPS_RELATION);
		calendarDAOWrapper.setUseRelationsOnGet(true);

		bookingDAOWrapper = bookingDAO.getAdvancedWrapper(Integer.class);
		bookingDAOWrapper.getGetQuery().addRelations(Booking.EVENTS_RELATION, Booking.CONTACT_RELATION, Booking.MANUAL_CONTACT_RELATION, Booking.BOOKING_SLOT_RELATION, BookingSlot.CALENDAR_RELATION, BookingSlot.MANAGERS_RELATION, BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION, BookingCalendar.ADMIN_GROUPS_RELATION, Booking.INTERNAL_MESSAGES_RELATION, Booking.EXTERNAL_MESSAGES_RELATION, InternalMessage.ATTACHMENTS_RELATION, ExternalMessage.ATTACHMENTS_RELATION, BookingEvent.ATTRIBUTES_RELATION);
		bookingDAOWrapper.getAddQuery().addRelations(Booking.EVENTS_RELATION, Booking.CONTACT_RELATION, Booking.MANUAL_CONTACT_RELATION);
		bookingDAOWrapper.getUpdateQuery().addRelations(Booking.EVENTS_RELATION, Booking.CONTACT_RELATION, Booking.MANUAL_CONTACT_RELATION);

		bookingSlotDAOWrapper = bookingSlotDAO.getAdvancedWrapper(Integer.class);
		bookingSlotDAOWrapper.getGetQuery().addRelations(BookingSlot.BOOKINGS_RELATION, Booking.CONTACT_RELATION, Booking.MANUAL_CONTACT_RELATION, BookingSlot.REPEATING_RELATION, BookingSlot.CALENDAR_RELATION, BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION, BookingCalendar.ADMIN_GROUPS_RELATION, BookingSlot.MANAGERS_RELATION);
		bookingSlotDAOWrapper.getAddQuery().addRelations(BookingSlot.REPEATING_RELATION, BookingSlot.MANAGERS_RELATION, BookingSlot.BOOKINGS_RELATION, Booking.CONTACT_RELATION, Booking.EVENTS_RELATION, Booking.MANUAL_CONTACT_RELATION);
		bookingSlotDAOWrapper.getUpdateQuery().addRelations(BookingSlot.MANAGERS_RELATION);

		internalMessageCRUD = new InternalMessageCRUD(daoFactory.getDAO(InternalMessage.class), daoFactory.getDAO(InternalMessageAttachment.class), this, true);

		bookingSlotCRUD = new BookingSlotCRUD(bookingSlotDAOWrapper, this);
		bookingSlotCRUD.addRequestFilter(new TransactionRequestFilter(dataSource));
		bookingSlotCRUD.addModuleResponseFilter(new FragmentLinkScriptFilter<>());

		bookingCRUD = new BookingCRUD(bookingDAOWrapper, internalMessageCRUD, externalMessageCRUD, this);
		bookingCRUD.addRequestFilter(new TransactionRequestFilter(dataSource));
		bookingCRUD.addModuleResponseFilter(new FragmentLinkScriptFilter<>());

		repeatingCRUD = new RepeatingCRUD(repeatingDAOWrapper, bookingSlotDAOWrapper, this);
		repeatingCRUD.addRequestFilter(new TransactionRequestFilter(dataSource));
		repeatingCRUD.addModuleResponseFilter(new FragmentLinkScriptFilter<>());
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return listCalendars(req, user, uriParser, null);
	}

	public ForegroundModuleResponse listCalendars(HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		Document doc = createDocument(req, uriParser, user);

		Element listCalendarsElement = XMLUtils.appendNewElement(doc, (Element) doc.getFirstChild(), "ListCalendars");

		List<BookingCalendar> calendars;

		if (hasAdminAccess(user)) {

			calendars = getAllCalendars();

		} else {

			calendars = getCalendars(user);
		}

		BookingUtils.setCalendarsTransientData(calendars, calendarDAO);

		XMLUtils.append(doc, listCalendarsElement, "Calendars", calendars);

		if (validationErrors != null) {

			XMLUtils.append(doc, listCalendarsElement, validationErrors);
		}

		return new SimpleForegroundModuleResponse(doc);
	}

	@WebPublic(alias = "list")
	public ForegroundModuleResponse listBookingSlots(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer calendarID;
		BookingCalendar calendar;

		if (uriParser.size() >= 3 && (calendarID = uriParser.getInt(2)) != null && (calendar = calendarDAOWrapper.get(calendarID)) != null) {

			BookingSlotCRUD.setRequestCalendar(req, calendar);

			return bookingSlotCRUD.list(req, res, user, uriParser, null);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "addslot")
	public ForegroundModuleResponse addBookingSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer calendarID;
		BookingCalendar calendar;

		if (uriParser.size() >= 3 && (calendarID = uriParser.getInt(2)) != null && (calendar = calendarDAOWrapper.get(calendarID)) != null) {

			if (!isCalendarManager(calendar, user)) {

				throw new AccessDeniedException("User " + user + " is not a manager of calendar " + calendar);
			}

			BookingSlotCRUD.setRequestCalendar(req, calendar);

			return bookingSlotCRUD.add(req, res, user, uriParser);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "showslot")
	public ForegroundModuleResponse showBookingSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return bookingSlotCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(alias = "updateslot")
	public ForegroundModuleResponse updateBookingSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return bookingSlotCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "deleteslot")
	public ForegroundModuleResponse deleteBookingSlot(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return bookingSlotCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "updaterepeating")
	public ForegroundModuleResponse updateRepeating(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return repeatingCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "deleterepeating")
	public ForegroundModuleResponse deleteRepeating(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return repeatingCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "add")
	public ForegroundModuleResponse addBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer bookingSlotID;
		BookingSlot bookingSlot;

		if (uriParser.size() >= 3 && (bookingSlotID = uriParser.getInt(2)) != null && (bookingSlot = bookingSlotDAOWrapper.get(bookingSlotID)) != null) {

			if (!isCalendarManager(bookingSlot.getCalendar(), user)) {

				throw new AccessDeniedException("User " + user + " is not a manager of calendar " + bookingSlot.getCalendar());
			}

			BookingUtils.setBookingSlotTransientData(bookingSlot, bookingSlotDAO);
			BookingCRUD.setRequestBookingSlot(req, bookingSlot);

			return bookingCRUD.add(req, res, user, uriParser);
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "show")
	public ForegroundModuleResponse showBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return bookingCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(alias = "messages")
	public ForegroundModuleResponse showBookingMessages(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws IOException, URINotFoundException {

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2))) {

			redirectToMethod(req, res, "/show/" + uriParser.get(2) + "#messages");

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "update")
	public ForegroundModuleResponse updateBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return bookingCRUD.update(req, res, user, uriParser);
	}

	@RESTMethod(alias = "archive/{bookingID}", method = { "post" }, requireLogin = true)
	public ForegroundModuleResponse archiveBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingID") Integer bookingID) throws SQLException, URINotFoundException, IOException {

		Booking booking = getBooking(bookingID, BOOKING_RELATIONS);

		if (booking == null) {

			throw new URINotFoundException(uriParser);
		}

		markBookingAsArchived(req, user, booking);

		if (req.getParameter("show") != null) {

			redirectToMethod(req, res, "/show/" + booking.getBookingID());

		} else {

			redirectToMethod(req, res, "/list/" + booking.getBookingSlot().getCalendar().getCalendarID());
		}

		return null;
	}

	private void markBookingAsArchived(HttpServletRequest req, User user, Booking booking) throws SQLException {

		if (booking.isArchived()) {

			return;
		}

		String comment = req.getParameter("comment");

		log.info("User " + user + " archiving booking " + booking + " with reason: " + StringUtils.toLogFormat(comment, 50));

		addBookingEvent(booking, BookingEventType.ARCHIVED, eventArchiveMessage + (StringUtils.isEmpty(comment) ? "" : getEventSeeMessageText(booking)), user, null, null);

		booking.setArchived(true);

		booking.setBookingPassedEventSent(true);

		bookingDAO.update(booking);

		addBookingComment(booking, comment, user, InternalMessageType.ARCHIVE_BY_MANAGER, ExternalMessageType.ARCHIVE_BY_MANAGER);

		getEventHandler().sendEvent(Booking.class, new BookingArchiveEvent(booking, user), EventTarget.ALL);
	}

	@RESTMethod(alias = "unarchive/{bookingID}", method = "post", requireLogin = true)
	public ForegroundModuleResponse unarchiveBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingID") Integer bookingID) throws Exception {

		Booking booking = bookingDAOWrapper.get(bookingID);

		if (booking == null) {

			throw new URINotFoundException(uriParser);
		}

		BookingSlot bookingSlot = booking.getBookingSlot();

		if (booking.isArchived()) {

			BookingUtils.setBookingSlotTransientData(bookingSlot, bookingSlotDAO);

			BookingCalendar calendar = booking.getBookingSlot().getCalendar();
			PrimaryContact primaryContact = booking.getPrimaryContact();

			try {

				BookingCRUD.validateBookingSlot(bookingSlot, booking.getSize(), true);

			} catch (ValidationException e) {

				return bookingCRUD.show(req, res, user, uriParser, e.getErrors());
			}

			if (primaryContact != null) {

				String citizenIdentifier = primaryContact.getCitizenIdentifier();

				try {

					BookingCRUD.validateCitizenIdentifier(citizenIdentifier, calendar, bookingSlot, false, dataSource);

				} catch (ValidationException e) {

					return bookingCRUD.show(req, res, user, uriParser, e.getErrors());
				}
			}

			String comment = req.getParameter("comment");

			log.info("User " + user + " restoring booking " + booking + " with reason: " + StringUtils.toLogFormat(comment, 50));

			try (TransactionHandler transactionHandler = bookingSlotDAO.createTransaction()) {

				booking.setArchived(false);
				booking.setBookingPassedEventSent(false);

				addBookingEvent(booking, BookingEventType.SUBMITTED, eventAdminRestoredMessage + (StringUtils.isEmpty(comment) ? "" : getEventSeeMessageText(booking)), user, null, null);

				bookingDAO.update(booking, transactionHandler, null);
				transactionHandler.commit();

				addBookingComment(booking, comment, user, InternalMessageType.RESTORE, ExternalMessageType.RESTORE);

				getEventHandler().sendEvent(Booking.class, new BookingManuallyAddedEvent(booking, user), EventTarget.ALL);
			}
		}

		if (req.getParameter("show") != null) {

			redirectToMethod(req, res, "/show/" + booking.getBookingID());

		} else {

			redirectToMethod(req, res, "/list/" + bookingSlot.getCalendar().getCalendarID());
		}

		return null;
	}

	private String getEventSeeMessageText(Booking booking) {

		if (BookingUtils.getUserType(booking, getUserHandler(), true, false) == UserType.ANALOG) {

			return eventSeeInternalMessage;
		}

		return eventSeeExternalMessage;
	}

	private void addBookingComment(Booking booking, String comment, User user, InternalMessageType internalMessageType, ExternalMessageType externalMessageType) throws SQLException {

		if (!StringUtils.isEmpty(comment)) {

			if (BookingUtils.getUserType(booking, getUserHandler(), true, false) == UserType.ANALOG) {

				addBookingInternalMessage(booking, comment, user, internalMessageType, null);

			} else {

				addBookingExternalMessage(booking, comment, true, user, externalMessageType, null);
			}
		}
	}

	@WebPublic(alias = "delete")
	public ForegroundModuleResponse deleteBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return bookingCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "download")
	public ForegroundModuleResponse downloadBookingPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer bookingID;
		Booking booking;

		if (uriParser.size() >= 3 && (bookingID = uriParser.getInt(2)) != null && (booking = bookingDAOWrapper.get(bookingID)) != null) {

			if (!hasBookingAccess(booking, user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			sendFlowInstancePDF(booking, booking.getFlowInstanceID(), req, res, user, uriParser);

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "preview")
	public ForegroundModuleResponse showFlowInstancePreview(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer bookingID;
		Booking booking;

		if (uriParser.size() >= 3 && (bookingID = uriParser.getInt(2)) != null && (booking = bookingDAOWrapper.get(bookingID)) != null) {

			if (!hasBookingAccess(booking, user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			res.sendRedirect(req.getContextPath() + flowInstanceAdminModule.getFlowInstancePreviewURL(booking.getFlowInstanceID()));

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public List<BookingCalendar> getAllCalendars() throws SQLException {

		HighLevelQuery<BookingCalendar> query = new HighLevelQuery<>();

		query.addParameter(calendarEnabledParamFactory.getParameter(true));

		return calendarDAO.getAll(query);
	}

	public List<BookingCalendar> getCalendars(User user) throws SQLException {

		return getCalendars(user, (Field) null);
	}

	public List<BookingCalendar> getCalendars(User user, Field... relations) throws SQLException {

		return calendarAdminModule.getCalendars(user, relations);
	}

	public BookingCalendar getCalendar(Integer calendarID, Field... relations) throws SQLException {

		HighLevelQuery<BookingCalendar> query = new HighLevelQuery<>(BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION, BookingCalendar.ADMIN_GROUPS_RELATION);

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(calendarDAOWrapper.getParameterFactory().getParameter(calendarID));

		return calendarDAO.get(query);
	}

	public BookingSlot getBookingSlot(Integer bookingSlotID, Field... relations) throws SQLException {

		HighLevelQuery<BookingSlot> query = new HighLevelQuery<>();

		query.addParameter(bookingSlotDAOWrapper.getParameterFactory().getParameter(bookingSlotID));

		if (relations != null) {

			query.addRelations(relations);
		}

		BookingSlot bookingSlot = bookingSlotDAO.get(query);

		BookingUtils.setBookingSlotTransientData(bookingSlot, bookingSlotDAO);

		return bookingSlot;
	}

	public List<BookingSlot> getBookingSlots(BookingCalendar calendar, Field... relations) throws SQLException {

		return getBookingSlots(calendar, null, relations);
	}

	public List<BookingSlot> getBookingSlots(BookingCalendar calendar, Date newerThan, Field... relations) throws SQLException {

		HighLevelQuery<BookingSlot> query = new HighLevelQuery<>();

		query.addCachedRelations(BookingSlot.MANAGERS_RELATION, BookingSlot.CALENDAR_RELATION);

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(bookingSlotCalendarParamFactory.getParameter(calendar));

		if (newerThan != null) {

			query.addParameter(bookingSlotStartParamFactory.getParameter(new Timestamp(newerThan.getTime()), QueryOperators.BIGGER_THAN_OR_EQUALS));
		}

		List<BookingSlot> bookingSlots = bookingSlotDAO.getAll(query);

		BookingUtils.setBookingSlotsTransientData(bookingSlots, bookingSlotDAO);

		return bookingSlots;
	}

	public List<BookingSlot> getBookingSlots(Repeating repeating) throws SQLException {

		return getBookingSlots(repeating, (Field) null);
	}

	public List<BookingSlot> getBookingSlots(Repeating repeating, Field... relations) throws SQLException {

		HighLevelQuery<BookingSlot> query = new HighLevelQuery<>();

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(bookingSlotRepeatingParamFactory.getParameter(repeating));

		List<BookingSlot> bookingSlots = bookingSlotDAO.getAll(query);

		BookingUtils.setBookingSlotsTransientData(bookingSlots, bookingSlotDAO);

		return bookingSlots;
	}

	public Booking getBooking(Integer bookingID, Field... relations) throws SQLException {

		HighLevelQuery<Booking> query = new HighLevelQuery<>(Booking.CONTACT_RELATION);

		query.addParameter(bookingDAOWrapper.getParameterFactory().getParameter(bookingID));

		if (relations != null) {

			query.addRelations(relations);
		}

		return bookingDAO.get(query);
	}

	public List<Booking> getBookingsByBookingSlot(BookingSlot bookingSlot, Field... relations) throws SQLException {

		HighLevelQuery<Booking> query = new HighLevelQuery<>();

		query.addParameter(bookingBookingSlotParamFactory.getParameter(bookingSlot));

		if (relations != null) {

			query.addRelations(relations);
		}

		return bookingDAO.getAll(query);
	}

	public List<Booking> getBookings(BookingCalendar calendar, Timestamp fromDate, Timestamp toDate, Field... relations) throws SQLException {

		StringBuilder sql = new StringBuilder();

		sql.append("SELECT b.bookingID FROM (SELECT bookingSlotID FROM booking_slots WHERE calendarID = ? AND start >= ?");

		if (toDate != null) {

			sql.append(" AND start < ?");
		}

		sql.append(") s INNER JOIN bookings b ON s.bookingSlotID = b.bookingSlotID WHERE b.archived = 0");

		ArrayListQuery<Integer> idQuery = new ArrayListQuery<>(dataSource, sql.toString(), IntegerPopulator.getPopulator());

		idQuery.setInt(1, calendar.getCalendarID());

		if (fromDate == null) {

			idQuery.setDate(2, new java.sql.Date(System.currentTimeMillis()));

		} else {

			idQuery.setTimestamp(2, fromDate);
		}

		if (toDate != null) {

			idQuery.setTimestamp(3, toDate);
		}

		List<Integer> IDs = idQuery.executeQuery();

		if (CollectionUtils.isEmpty(IDs)) {

			return null;
		}

		HighLevelQuery<Booking> query = new HighLevelQuery<>(Booking.CONTACT_RELATION);

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(bookingDAOWrapper.getParameterFactory().getWhereInParameter(IDs));

		return bookingDAO.getAll(query);
	}

	public List<Booking> getArchivedBookings(BookingCalendar calendar, Timestamp fromDate, Timestamp toDate, Field... relations) throws SQLException {

		StringBuilder sql = new StringBuilder();

		sql.append("SELECT b.bookingID FROM (SELECT start, bookingSlotID FROM booking_slots WHERE calendarID = ?) s INNER JOIN bookings b ON s.bookingSlotID = b.bookingSlotID WHERE (s.start < ? OR b.archived = 1)");

		if (fromDate != null) {

			sql.append(" AND s.start >= ?");
		}

		if (toDate != null) {

			sql.append(" AND s.start < ?");
		}

		ArrayListQuery<Integer> idQuery = new ArrayListQuery<>(dataSource, sql.toString(), IntegerPopulator.getPopulator());
		int i = 1;

		idQuery.setInt(i++, calendar.getCalendarID());
		idQuery.setDate(i++, new java.sql.Date(System.currentTimeMillis()));

		if (fromDate != null) {

			idQuery.setTimestamp(i++, fromDate);
		}

		if (toDate != null) {

			idQuery.setTimestamp(i++, toDate);
		}

		List<Integer> IDs = idQuery.executeQuery();

		if (CollectionUtils.isEmpty(IDs)) {

			return null;
		}

		HighLevelQuery<Booking> query = new HighLevelQuery<>(Booking.CONTACT_RELATION);

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(bookingDAOWrapper.getParameterFactory().getWhereInParameter(IDs));

		return bookingDAO.getAll(query);
	}

	public List<Booking> getAllBookings(BookingCalendar calendar, Field... relations) throws SQLException {

		String sql = "SELECT b.bookingID FROM (SELECT bookingSlotID FROM booking_slots WHERE calendarID = ?) s INNER JOIN bookings b ON s.bookingSlotID = b.bookingSlotID";

		ArrayListQuery<Integer> idQuery = new ArrayListQuery<>(dataSource, sql, IntegerPopulator.getPopulator());

		idQuery.setInt(1, calendar.getCalendarID());

		List<Integer> IDs = idQuery.executeQuery();

		if (CollectionUtils.isEmpty(IDs)) {

			return null;
		}

		HighLevelQuery<Booking> query = new HighLevelQuery<>(Booking.CONTACT_RELATION);

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(bookingDAOWrapper.getParameterFactory().getWhereInParameter(IDs));

		return bookingDAO.getAll(query);
	}

	public FlowInstance getFlowInstance(Integer flowInstanceID) throws SQLException {

		if (flowInstanceID == null) {

			return null;
		}

		return flowAdminModule.getFlowInstance(flowInstanceID);
	}

	// Copied from com.nordicpeak.flowengine.infomodule.FlowInstanceInfoModule
	public void sendFlowInstancePDF(Booking booking, Integer flowInstanceID, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		FlowInstance flowInstance = getFlowInstance(flowInstanceID);

		if (flowInstance == null) {

			throw new URINotFoundException(uriParser);
		}

		FlowInstanceAdminModule.GENERAL_ACCESS_CONTROLLER.checkFlowInstanceAccess(flowInstance, user);

		log.info("User " + user + " requesting PDF for flow instance " + flowInstance);

		FlowInstanceEvent event = getLatestPDFEvent(flowInstance);

		if (event == null) {

			log.info("Unable to find any PDF events for flow instance " + flowInstance + " requested by user " + user);

			throw new URINotFoundException(uriParser);
		}

		File pdfFile = pdfProvider.getPDF(flowInstanceID, event.getEventID());

		if (pdfFile == null) {

			log.warn("Unable to find PDF file for event " + event + " for flow instance " + flowInstance + " requested by user " + user);

			throw new URINotFoundException(uriParser);
		}

		HTTPUtils.sendFile(pdfFile, booking.getBookingSlot().getCalendar().getName() + " " + DateUtils.DATE_TIME_FORMATTER.format(booking.getBookingSlot().getStart()) + " " + columnBookingFlowInstanceID + " " + flowInstanceID + ".pdf", req, res, ContentDisposition.ATTACHMENT);
	}

	// Copied from com.nordicpeak.flowengine.infomodule.FlowInstanceInfoModule
	private FlowInstanceEvent getLatestPDFEvent(FlowInstance flowInstance) {

		if (CollectionUtils.isEmpty(flowInstance.getEvents())) {

			return null;
		}

		for (int i = flowInstance.getEvents().size() - 1; i > -1; i--) {

			FlowInstanceEvent event = flowInstance.getEvents().get(i);

			if (Boolean.TRUE.equals(event.getAttributeHandler().getBoolean("pdf"))) {

				return event;
			}
		}

		return null;
	}

	public boolean hasPersonProvider() {

		return personProvider != null;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateContact(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Booking booking;

		if (uriParser.size() != 3 || !NumberUtils.isInt(uriParser.get(2)) || (booking = bookingDAOWrapper.get(Integer.valueOf(uriParser.get(2)))) == null) {

			return defaultMethod(req, res, user, uriParser);
		}

		bookingCRUD.checkUpdateAccess(booking, user, req, uriParser);

		User poster = null;

		if (!StringUtils.isEmpty(booking.getPrimaryContactCitizenIdentifier())) {

			poster = getUserHandler().getUserByAttribute("citizenIdentifier", booking.getPrimaryContactCitizenIdentifier(), false, true);
		}

		if (poster != null) {

			redirectToMethod(req, res, "/show/" + booking.getBookingID());
			return null;
		}

		ManualContact contact = booking.getManualContact();

		log.info("User " + user + " requesting update contact form for booking slot " + booking);

		Document doc = this.createDocument(req, uriParser, user);

		Element updateContactElement = doc.createElement("UpdateContact");

		doc.getDocumentElement().appendChild(updateContactElement);

		List<ValidationError> validationErrors = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			validationErrors = new ArrayList<>();

			try {

				contact = MANUAL_CONTACT_POPULATOR.populate(contact, req);

			} catch (ValidationException e) {

				validationErrors.addAll(e.getErrors());
			}

			if (CollectionUtils.isEmpty(validationErrors)) {

				log.info("User " + user + " updating contact for booking slot " + booking);

				if (contact.isEmpty()) {

					if (contact.getBookingID() != null) {

						manualContactDAO.delete(contact);
					}

				} else if (contact.getBookingID() == null) {

					contact.setBookingID(booking.getBookingID());

					manualContactDAO.add(contact);

				} else {

					manualContactDAO.update(contact);
				}

				redirectToMethod(req, res, "/show/" + booking.getBookingID());
				return null;
			}
		}

		updateContactElement.appendChild(booking.toXML(doc));

		if (validationErrors != null) {

			validationErrors.forEach(error -> updateContactElement.appendChild(error.toXML(doc)));

			updateContactElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, booking.getBookingSlot().getCalendar().getName(), getDefaultBreadcrumb());

	}

	@WebPublic(alias = "internalattachment")
	public ForegroundModuleResponse getInternalMessageAttachment(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, URINotFoundException {

		return internalMessageCRUD.getRequestedMessageAttachment(req, res, user, uriParser);
	}

	@Override
	public Collection<Integer> getAllowedGroupIDs() {

		return adminGroupIDs;
	}

	@Override
	public Collection<Integer> getAllowedUserIDs() {

		return adminUserIDs;
	}

	@Override
	public boolean allowsAdminAccess() {

		return false;
	}

	@Override
	public boolean allowsUserAccess() {

		return false;
	}

	@Override
	public boolean allowsAnonymousAccess() {

		return false;
	}

	@Override
	public boolean hasCalendarAccess(BookingCalendar calendar, User user) {

		return calendar.isEnabled() && (AccessUtils.checkAccess(user, calendar) || hasAdminAccess(user));
	}

	public boolean hasBookingAccess(Booking booking, User user) {

		BookingCalendar calendar = booking.getBookingSlot().getCalendar();

		return calendar.isEnabled() && !booking.isPreliminary() && (AccessUtils.checkAccess(user, calendar) || hasAdminAccess(user));
	}

	public boolean hasAdminAccess(User user) {

		return AccessUtils.checkAccess(user, this);
	}

	private ForegroundModuleResponse showExportForm(BookingCalendar calendar, HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Throwable {

		log.info("User " + user + " viewing export form for calendar" + calendar);

		Document doc = createDocument(req, uriParser, user);

		Element exportElement = doc.createElement("ExportCalendar");
		doc.getDocumentElement().appendChild(exportElement);

		exportElement.appendChild(calendar.toXML(doc));

		if (validationErrors != null) {

			XMLUtils.append(doc, exportElement, validationErrors);
			exportElement.appendChild(RequestUtils.getRequestParameters(req, doc, "flowID"));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	@RESTMethod(alias = "export/{calendarID}", method = { "get", "post" }, requireLogin = true)
	public ForegroundModuleResponse exportCalendarToExcel(HttpServletRequest req, final HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "calendarID") Integer calendarID) throws Throwable {

		final BookingCalendar calendar = getCalendar(calendarID, BookingCalendar.BOOKING_SLOTS_RELATION);

		if (calendar == null) {

			throw new URINotFoundException(uriParser);
		}

		if (!hasCalendarAccess(calendar, user)) {

			throw new AccessDeniedException(getSectionDescriptor());
		}

		if (!req.getMethod().equalsIgnoreCase("POST")) {

			return showExportForm(calendar, req, user, uriParser, null);
		}

		log.info("User " + user + " exporting calendar " + calendar + " to excel");

		List<ValidationError> validationErrors = new ArrayList<>();

		java.sql.Date sqlFromDate = ValidationUtils.validateParameter("fromDate", req, false, DatePopulator.getPopulator(), validationErrors);
		Timestamp fromDate = sqlFromDate != null ? Timestamp.valueOf(sqlFromDate.toLocalDate().atStartOfDay()) : null;

		java.sql.Date sqlToDate = ValidationUtils.validateParameter("toDate", req, false, DatePopulator.getPopulator(), validationErrors);
		Timestamp toDate = sqlToDate != null ? Timestamp.valueOf(sqlToDate.toLocalDate().atTime(LocalTime.MAX)) : null;

		if (!validationErrors.isEmpty()) {

			return showExportForm(calendar, req, user, uriParser, validationErrors);
		}

		BookingUtils.setBookingSlotsTransientData(calendar.getBookingSlots(), bookingSlotDAO);

		if (calendar.getBookingSlots() != null && (fromDate != null || toDate != null)) {

			Predicate<BookingSlot> startBeforeFromDate = slot -> fromDate != null && slot.getStart().before(fromDate);
			Predicate<BookingSlot> startAfterToDate = slot -> toDate != null && slot.getStart().after(toDate);

			calendar.getBookingSlots().removeIf(startBeforeFromDate.or(startAfterToDate));
		}

		final List<Booking> bookings = getBookings(calendar, fromDate, toDate, BOOKING_EXPORT_RELATIONS);
		final List<Booking> archivedBookings = getArchivedBookings(calendar, fromDate, toDate, BOOKING_EXPORT_RELATIONS);

		if (!CollectionUtils.isEmpty(bookings)) {

			Collections.sort(bookings, BOOKING_START_COMPARATOR);

			for (Booking booking : bookings) {

				BookingUtils.setBookingSlotTransientData(booking.getBookingSlot(), bookingSlotDAO);

				booking.setFlowInstance(getFlowInstance(booking.getFlowInstanceID()));
			}
		}

		if (!CollectionUtils.isEmpty(archivedBookings)) {

			Collections.sort(archivedBookings, BOOKING_START_COMPARATOR);

			for (Booking booking : archivedBookings) {

				BookingUtils.setBookingSlotTransientData(booking.getBookingSlot(), bookingSlotDAO);
			}
		}

		final String nowString = DateUtils.DATE_FORMATTER.format(new Date());

		String filename = calendar.getName() + " " + nowString + ".xlsx";

		res.setHeader("Content-Disposition", "attachment;" + FileUtils.toContentDispositionFilename(filename));
		res.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		//Workaround for thread local usage in XMLBeans used by XSSF
		ExeptionHandlingRunnable runnable = new ExeptionHandlingRunnable() {

			private Exception exception;

			@Override
			public void run() {

				try (Workbook workbook = new XSSFWorkbook()) {

					Sheet sheet1 = workbook.createSheet("Kommande bokningar");

					int rowIndex = 0;
					int columnIndex = 0;

					Row titleRow = sheet1.createRow(rowIndex++);

					columnIndex = addBookingSlotTitleCells(titleRow, columnIndex);
					columnIndex = addBookingTitleCells(titleRow, columnIndex);

					columnIndex = SpreadSheetUtils.addContactTitleCells(titleRow, columnIndex);

					columnIndex = SpreadSheetUtils.addCells(titleRow, columnIndex, calendar.getExportAttributes());

					if (bookings != null) {

						for (Booking booking : bookings) {

							Row row = sheet1.createRow(rowIndex++);
							columnIndex = 0;

							BookingSlot bookingSlot = booking.getBookingSlot();
							booking.setFlowInstance(getFlowInstance(booking.getFlowInstanceID()));

							columnIndex = SpreadSheetUtils.addBookingSlotCells(calendar, bookingSlot, row, columnIndex);
							columnIndex = addBookingCells(calendar, booking, row, columnIndex, workbook.getCreationHelper());
						}
					}

					autoSizeSheet(sheet1, columnIndex);

					Sheet sheet2 = workbook.createSheet("Arkiverade bokningar");

					rowIndex = 0;
					columnIndex = 0;

					titleRow = sheet2.createRow(rowIndex++);

					columnIndex = addBookingSlotTitleCells(titleRow, columnIndex);
					columnIndex = addBookingTitleCells(titleRow, columnIndex);

					columnIndex = SpreadSheetUtils.addContactTitleCells(titleRow, columnIndex);

					columnIndex = SpreadSheetUtils.addCells(titleRow, columnIndex, calendar.getExportAttributes());

					if (archivedBookings != null) {

						for (Booking booking : archivedBookings) {

							Row row = sheet2.createRow(rowIndex++);
							columnIndex = 0;

							BookingSlot bookingSlot = booking.getBookingSlot();
							booking.setFlowInstance(getFlowInstance(booking.getFlowInstanceID()));

							columnIndex = SpreadSheetUtils.addBookingSlotCells(calendar, bookingSlot, row, columnIndex);
							columnIndex = addBookingCells(calendar, booking, row, columnIndex, workbook.getCreationHelper());
						}
					}

					autoSizeSheet(sheet2, columnIndex);

					Sheet sheet3 = workbook.createSheet("Lediga Tider");

					rowIndex = 0;
					columnIndex = 0;

					titleRow = sheet3.createRow(rowIndex++);

					columnIndex = addBookingSlotTitleCells(titleRow, columnIndex);

					if (calendar.getBookingSlots() != null) {

						for (BookingSlot bookingSlot : calendar.getBookingSlots()) {

							if (bookingSlot.getFreeSlots() > 0) {

								Row row = sheet3.createRow(rowIndex++);
								columnIndex = 0;

								columnIndex = SpreadSheetUtils.addBookingSlotCells(calendar, bookingSlot, row, columnIndex);
							}
						}
					}

					autoSizeSheet(sheet3, columnIndex);

					Sheet sheet4 = workbook.createSheet("Alla Tider");

					rowIndex = 0;
					columnIndex = 0;

					titleRow = sheet4.createRow(rowIndex++);

					columnIndex = addBookingSlotTitleCells(titleRow, columnIndex);

					if (calendar.getBookingSlots() != null) {

						for (BookingSlot bookingSlot : calendar.getBookingSlots()) {

							Row row = sheet4.createRow(rowIndex++);
							columnIndex = 0;

							columnIndex = SpreadSheetUtils.addBookingSlotCells(calendar, bookingSlot, row, columnIndex);
						}
					}

					autoSizeSheet(sheet4, columnIndex);

					workbook.write(res.getOutputStream());

				} catch (Exception e) {

					exception = e;
				}
			}

			@Override
			public Exception getException() {

				return exception;
			}
		};

		ThreadUtils.runAndWait(runnable, "Excel export of calendar " + calendar + " started by user " + user, true);

		if (runnable.getException() != null && !(runnable.getException() instanceof OpenXML4JRuntimeException)) {

			throw runnable.getException();
		}

		return null;
	}

	private void autoSizeSheet(Sheet sheet, int columns) {

		for (int i = 0; i <= columns; i++) {

			sheet.autoSizeColumn(i);
		}
	}

	private int addBookingSlotTitleCells(Row row, int columnIndex) {

		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingSlotDate);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingSlotTime);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingSlotSize);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingSlotLocation);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingSlotDescription);

		return columnIndex;
	}

	private int addBookingTitleCells(Row row, int columnIndex) {

		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingFlowInstanceID);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingAdded);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingRebooked);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingArchived);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingComment);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingDigital);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingFlowInstanceURL);
		SpreadSheetUtils.addCell(row, columnIndex++, columnBookingURL);

		return columnIndex;
	}

	private int addBookingCells(BookingCalendar calendar, Booking booking, Row row, int columnIndex, CreationHelper creationHelper) throws SQLException {

		SpreadSheetUtils.addCell(row, columnIndex++, booking.getFlowInstanceID());

		SpreadSheetUtils.addCell(row, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(booking.getAdded()));

		Date rebookedDate = getBookingRebookedDate(booking);

		if (rebookedDate != null) {

			SpreadSheetUtils.addCell(row, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(rebookedDate));

		} else {

			columnIndex++;
		}

		Date archivedDate = getBookingArchivedDate(booking);

		if (archivedDate != null) {

			SpreadSheetUtils.addCell(row, columnIndex++, DateUtils.DATE_TIME_SECONDS_FORMATTER.format(archivedDate));

		} else {

			columnIndex++;
		}

		ExternalMessage commentExternalMessage = null;

		if (!booking.isPreliminary() && !CollectionUtils.isEmpty(booking.getExternalMessages())) {

			commentExternalMessage = CollectionUtils.find(booking.getExternalMessages(), message -> message.getType() == ExternalMessageType.ENTRANCE || message.getType() == ExternalMessageType.MANUAL_ENTRANCE);
		}

		if (commentExternalMessage != null) {

			SpreadSheetUtils.addCell(row, columnIndex++, commentExternalMessage.getMessage());

		} else {

			columnIndex++;
		}

		SpreadSheetUtils.addCell(row, columnIndex++, getUserTypeIsDigitalText(booking));

		String flowInstanceAdminURL = getFlowInstanceAdminURL(booking.getFlowInstance());

		if (flowInstanceAdminURL != null) {

			SpreadSheetUtils.addHyperlinkCell(row, columnIndex++, creationHelper, flowInstanceAdminURL);

		} else {

			SpreadSheetUtils.addCell(row, columnIndex++, bookingNoFlowInstanceText);
		}

		SpreadSheetUtils.addHyperlinkCell(row, columnIndex++, creationHelper, getBookingAdminURL(booking));

		if (booking.isPreliminary()) {

			Contact contact = new Contact();
			contact.setFirstname(preliminaryBookingText);
			columnIndex = SpreadSheetUtils.addContactCells(contact, row, columnIndex);

		} else {

			Contact contact = BookingUtils.getContact(booking, getUserHandler());
			columnIndex = SpreadSheetUtils.addContactCells(contact, row, columnIndex);
		}

		Map<String, String> attributesFromFlowInstance = BookingUtils.getBookingFlowInstanceAttributes(booking, calendar, flowAdminModule);

		if (!CollectionUtils.isEmpty(calendar.getExportAttributes())) {

			for (String attribute : calendar.getExportAttributes()) {

				if (!CollectionUtils.isEmpty(attributesFromFlowInstance) && attributesFromFlowInstance.get(attribute) != null) {

					SpreadSheetUtils.addCell(row, columnIndex++, attributesFromFlowInstance.get(attribute));

				} else {

					SpreadSheetUtils.addCell(row, columnIndex++, "");
				}
			}
		}

		return columnIndex;
	}

	private Date getBookingRebookedDate(Booking booking) {

		return getMostRecentEvent(booking, BookingEventType.CHANGED_SLOT);
	}

	private Date getBookingArchivedDate(Booking booking) {

		if (booking.isArchived()) {

			return getMostRecentEvent(booking, BookingEventType.ARCHIVED);
		}

		return null;
	}

	private Date getMostRecentEvent(Booking booking, BookingEventType eventType) {

		List<BookingEvent> events = booking.getEvents();

		if (!CollectionUtils.isEmpty(events)) {

			for (int i = events.size() - 1; i >= 0; i--) {

				BookingEvent event = events.get(i);

				if (event.getEventType() == eventType) {

					return event.getAdded();
				}
			}
		}

		return null;
	}

	private String getFlowInstanceAdminURL(FlowInstance flowInstance) {

		if (flowInstance == null) {

			return null;
		}

		return notificationHandler.getFlowInstanceAdminModuleAlias(flowInstance) + "/overview/" + flowInstance.getFlowInstanceID();
	}

	private String getBookingAdminURL(Booking booking) {

		if (booking == null) {

			return null;
		}

		return notificationHandler.getBookingSlotAdminModuleAlias(booking) + "/show/" + booking.getBookingID();
	}

	@RESTMethod(alias = "archiveAllBookings/{bookingSlotID}", method = "post", requireLogin = true)
	public ForegroundModuleResponse archiveAllBookings(HttpServletRequest req, final HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingSlotID") Integer bookingSlotID) throws Throwable {

		final BookingSlot bookingSlot = getBookingSlot(bookingSlotID);

		if (bookingSlot == null) {

			throw new URINotFoundException(uriParser);
		}

		List<Booking> bookings = getBookingsByBookingSlot(bookingSlot, ARCHIVE_ALL_BOOKINGS_RELATIONS);

		if (CollectionUtils.isEmpty(bookings)) {

			throw new URINotFoundException(uriParser);
		}

		for (Booking booking : bookings) {

			markBookingAsArchived(req, user, booking);
		}

		redirectToMethod(req, res, "/showslot/" + bookingSlotID);

		return null;
	}

	@RESTMethod(alias = "exportslot/{bookingSlotID}", method = "get", requireLogin = true)
	public ForegroundModuleResponse exportBookingSlotToExcel(HttpServletRequest req, final HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingSlotID") Integer bookingSlotID) throws Throwable {

		final BookingSlot bookingSlot = getBookingSlot(bookingSlotID, EXPORT_BOOKING_SLOT_RELATIONS);

		if (bookingSlot == null) {

			throw new URINotFoundException(uriParser);
		}

		final BookingCalendar calendar = bookingSlot.getCalendar();

		if (!hasCalendarAccess(calendar, user)) {

			throw new AccessDeniedException(getSectionDescriptor());
		}

		log.info("User " + user + " exporting booking slot " + bookingSlot + " to excel");

		final String title = bookingSlot.getCalendar().getName() + " " + DateUtils.DATE_TIME_FORMATTER.format(bookingSlot.getStart());

		final String filename = title + ".xlsx";

		//Follows RFC6266 filename encoding
		res.setHeader("Content-Disposition", "attachment;" + FileUtils.toContentDispositionFilename(filename));
		res.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		//Workaround for thread local usage in XMLBeans used by XSSF
		ExeptionHandlingRunnable runnable = new ExeptionHandlingRunnable() {

			private Exception exception;

			@Override
			public void run() {

				try (Workbook workbook = new XSSFWorkbook()) {

					int allowedNameLength = 31;

					String sheetName = StringUtils.substring(filename, allowedNameLength, "...");
					sheetName = WorkbookUtil.createSafeSheetName(sheetName);

					Sheet sheet = workbook.createSheet(sheetName);

					Font titleFont = workbook.createFont();
					titleFont.setBold(true);
					titleFont.setFontHeightInPoints((short) 16);

					CellStyle titleStyle = workbook.createCellStyle();
					titleStyle.setFont(titleFont);

					Font subTitleFont = workbook.createFont();
					subTitleFont.setBold(true);

					CellStyle subTitleStyle = workbook.createCellStyle();
					subTitleStyle.setFont(subTitleFont);

					int rowIndex = 0;

					Row titleRow = sheet.createRow(rowIndex++);
					SpreadSheetUtils.addCell(titleRow, 0, title, titleStyle);

					Row locationRow = sheet.createRow(rowIndex++);
					SpreadSheetUtils.addCell(locationRow, 0, columnBookingSlotLocation + ": " + (bookingSlot.getLocation() == null ? "" : bookingSlot.getLocation()), subTitleStyle);

					Row descriptionRow = sheet.createRow(rowIndex++);
					SpreadSheetUtils.addCell(descriptionRow, 0, columnBookingSlotDescription + ": " + (bookingSlot.getDescription() == null ? "" : bookingSlot.getDescription()), subTitleStyle);

					Row columnsRow = sheet.createRow(rowIndex++);
					int columnIndex = 0;

					columnIndex = addBookingTitleCells(columnsRow, columnIndex);

					columnIndex = SpreadSheetUtils.addContactTitleCells(columnsRow, columnIndex);

					columnIndex = SpreadSheetUtils.addCells(columnsRow, columnIndex, calendar.getExportAttributes());

					if (bookingSlot.getBookings() != null) {

						for (Booking booking : bookingSlot.getBookings()) {

							booking.setBookingSlot(bookingSlot);
							booking.setFlowInstance(getFlowInstance(booking.getFlowInstanceID()));

							Row row = sheet.createRow(rowIndex++);
							columnIndex = 0;

							columnIndex = addBookingCells(calendar, booking, row, columnIndex, workbook.getCreationHelper());
						}
					}

					autoSizeSheet(sheet, columnIndex);

					workbook.write(res.getOutputStream());

				} catch (Exception e) {

					exception = e;
				}
			}

			@Override
			public Exception getException() {

				return exception;
			}
		};

		ThreadUtils.runAndWait(runnable, "Excel export of calendar " + calendar + " started by user " + user, true);

		if (runnable.getException() != null) {

			throw runnable.getException();
		}

		return null;
	}

	private String getUserTypeIsDigitalText(Booking booking) {

		UserType userType = BookingUtils.getUserType(booking, getUserHandler(), true, false);

		if (userType == UserType.DIGITAL) {

			return columnBookingYes;
		}

		if (userType == UserType.DIGITAL_NOT_CONTACTABLE) {

			return columnBookingYes + " (" + columnBookingDigitalNotContactable + ")";
		}

		return columnBookingNo;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse sendMessages(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if (uriParser.size() == 4) {

			String target = uriParser.get(2);
			String id = uriParser.get(3);

			if (target.equalsIgnoreCase("bookingslot")) {

				BookingSlot bookingSlot;

				if (NumberUtils.isInt(id) && (bookingSlot = bookingSlotDAOWrapper.get(Integer.valueOf(id))) != null) {

					BookingCalendar calendar = bookingSlot.getCalendar();
					List<Booking> bookings = bookingSlot.getBookings();

					if (bookings != null) {

						bookings.forEach(booking -> booking.setBookingSlot(bookingSlot));
					}

					Document doc = createDocument(req, uriParser, user);
					Element sendMessageElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "SendMessageForm");

					XMLUtils.appendNewElement(doc, sendMessageElement, "Target", target);

					bookingSlot.setBookings(null);
					XMLUtils.append(doc, sendMessageElement, bookingSlot);

					return sendMessageToBookings(req, user, doc, sendMessageElement, calendar, bookings);
				}

				throw new URINotFoundException(uriParser);
			}

			if (target.equalsIgnoreCase("calendar")) {

				BookingCalendar calendar;

				if (NumberUtils.isInt(id) && (calendar = calendarDAOWrapper.get(Integer.valueOf(id))) != null) {

					List<Booking> bookings = getAllBookings(calendar, Booking.MANUAL_CONTACT_RELATION, Booking.BOOKING_SLOT_RELATION);

					if (bookings != null) {

						bookings.forEach(booking -> booking.getBookingSlot().setCalendar(calendar));
					}

					Document doc = createDocument(req, uriParser, user);
					Element sendMessageElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "SendMessageForm");

					XMLUtils.appendNewElement(doc, sendMessageElement, "Target", target);
					XMLUtils.append(doc, sendMessageElement, calendar);

					return sendMessageToBookings(req, user, doc, sendMessageElement, calendar, bookings);
				}
			}
		}

		throw new URINotFoundException(uriParser);
	}

	private ForegroundModuleResponse sendMessageToBookings(HttpServletRequest req, User user, Document doc, Element sendMessageElement, BookingCalendar calendar, List<Booking> bookings) throws AccessDeniedException {

		if (!hasCalendarAccess(calendar, user)) {

			throw new AccessDeniedException(getSectionDescriptor());
		}

		List<ValidationError> validationErrors = new ArrayList<>();

		notificationHandler.setCalendarTemplatedFields(calendar);

		if (StringUtils.isEmpty(calendar.getEmailSenderName()) || StringUtils.isEmpty(calendar.getEmailSenderAddress())) {

			validationErrors.add(EMAIL_SENDER_NOT_SET_VALIDATION_ERROR);
		}

		if (req.getMethod().equals("POST")) {

			int sentMessages = sendMessageToBookings(req, user, bookings, validationErrors);

			if (sentMessages > 0) {

				XMLUtils.appendNewElement(doc, sendMessageElement, "SentEmails", sentMessages);
			}
		}

		if (!validationErrors.isEmpty()) {

			XMLUtils.append(doc, sendMessageElement, "ValidationErrors", validationErrors);
		}

		appendBookingSlotCounts(doc, sendMessageElement, bookings);

		sendMessageElement.appendChild(RequestUtils.getRequestParameters(req, doc));

		log.info("User " + user + " requesting send message form");

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
	}

	private int sendMessageToBookings(HttpServletRequest req, User user, List<Booking> bookings, List<ValidationError> validationErrors) {

		int sentEmails = 0;

		String emailSubject = ValidationUtils.validateParameter("emailSubject", req, true, 1, 255, validationErrors);
		String emailMessage = ValidationUtils.validateParameter("emailMessage", req, true, 1, 65535, validationErrors);

		boolean toDigitalUsers = !StringUtils.isEmpty(req.getParameter("toDigital"));
		boolean toAnalogWithEmail = !StringUtils.isEmpty(req.getParameter("toAnalogWithEmail"));

		if (!toDigitalUsers && !toAnalogWithEmail) {

			validationErrors.add(NO_RECIPIENT_TYPES_VALIDATION_ERROR);
		}

		boolean toPendingBookings = !StringUtils.isEmpty(req.getParameter("pending"));
		boolean toArchivedBookings = !StringUtils.isEmpty(req.getParameter("archived"));

		if (!toPendingBookings && !toArchivedBookings) {

			validationErrors.add(NO_RECIPIENT_STATE_VALIDATION_ERROR);
		}

		if (!validationErrors.isEmpty()) {

			return 0;
		}

		if (CollectionUtils.isEmpty(bookings)) {

			return 0;
		}

		for (Booking booking : bookings) {

			Contact contact = BookingUtils.getContact(booking, getUserHandler());

			if (shouldSendMessageToBooking(booking, contact, toDigitalUsers, toAnalogWithEmail, toPendingBookings, toArchivedBookings, validationErrors)) {

				log.info("User " + user + " sending message '" + emailSubject + "' to booking slot " + booking);

				if (notificationHandler.sendContactEmail(booking, contact, null, emailSubject, emailMessage, null)) {

					sentEmails++;

				} else {

					validationErrors.add(new ErrorSendingToRecipientViaEmailValidationError(booking, contact.getEmail()));
				}
			}
		}

		return sentEmails;
	}

	private boolean shouldSendMessageToBooking(Booking booking, Contact contact, boolean toDigitalUsers, boolean toAnalogWithEmail, boolean toPendingBookings, boolean toArchivedBookings, List<ValidationError> validationErrors) {

		boolean digital = BookingUtils.getUserType(booking, getUserHandler(), false, false) == UserType.DIGITAL;

		if ((booking.isArchived() || booking.getBookingSlot().hasPassed()) && !toArchivedBookings) {

			return false;
		}

		if (!booking.isArchived() && !booking.getBookingSlot().hasPassed() && !toPendingBookings) {

			return false;
		}

		if (contact == null || StringUtils.isEmpty(contact.getEmail())) {

			return false;
		}

		if (digital ? !toDigitalUsers : !toAnalogWithEmail) {

			return false;
		}

		if (!EmailUtils.isValidEmailAddress(contact.getEmail())) {

			validationErrors.add(new InvalidRecipientEmailValidationError(booking, contact.getEmail()));
			return false;
		}

		return true;
	}

	private void appendBookingSlotCounts(Document doc, Element sendMessageElement, List<Booking> bookings) {

		Element bookingSlotCounts = XMLUtils.appendNewElement(doc, sendMessageElement, "BookingSlotCounts");

		int digitalCount = 0;
		int analogWithEmailCount = 0;
		int pendingCount = 0;
		int archivedCount = 0;

		if (!CollectionUtils.isEmpty(bookings)) {

			for (Booking booking : bookings) {

				Contact contact = BookingUtils.getContact(booking, getUserHandler());

				if (BookingUtils.getUserType(booking, getUserHandler(), false, false) == UserType.DIGITAL) {

					digitalCount++;

				} else if (contact != null && !StringUtils.isEmpty(contact.getEmail())) {

					analogWithEmailCount++;

				} else {

					continue;
				}

				if (booking.isArchived() || booking.getBookingSlot().hasPassed()) {

					archivedCount++;

				} else {

					pendingCount++;
				}
			}
		}

		XMLUtils.appendNewElement(doc, bookingSlotCounts, "Digital", digitalCount);
		XMLUtils.appendNewElement(doc, bookingSlotCounts, "AnalogWithEmail", analogWithEmailCount);
		XMLUtils.appendNewElement(doc, bookingSlotCounts, "Pending", pendingCount);
		XMLUtils.appendNewElement(doc, bookingSlotCounts, "Archived", archivedCount);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse searchManager(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer calendarID;
		BookingCalendar calendar;

		if (uriParser.size() == 3 && (calendarID = uriParser.getInt(2)) != null && (calendar = calendarDAOWrapper.get(calendarID)) != null) {

			if (!hasCalendarAccess(calendar, user)) {

				throw new AccessDeniedException(getSectionDescriptor());
			}

			managerRequestProcessor.processRequest(calendar, req, res, user);
			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse getCalendarEvents(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer calendarID;
		BookingCalendar calendar;

		if (uriParser.size() != 3 || (calendarID = uriParser.getInt(2)) == null || (calendar = getCalendar(calendarID, BookingCalendar.BOOKING_SLOTS_RELATION, BookingSlot.MANAGERS_RELATION, BookingSlot.BOOKINGS_RELATION, Booking.CONTACT_RELATION)) == null) {

			throw new URINotFoundException(uriParser);
		}

		if (!hasCalendarAccess(calendar, user)) {

			throw new AccessDeniedException("User " + user + " not allowed to get events for calendar " + calendar);
		}

		Integer managerID = NumberUtils.toInt(req.getParameter("m"));

		List<BookingSlot> bookingSlots = calendar.getBookingSlots();

		if (!CollectionUtils.isEmpty(bookingSlots) && managerID != null) {

			bookingSlots = new ArrayList<>(bookingSlots);

			bookingSlots.removeIf(bookingSlot -> CollectionUtils.find(bookingSlot.getManagers(), manager -> manager.getUserID().equals(managerID)) == null);
		}

		BookingUtils.setBookingSlotsTransientData(bookingSlots, bookingSlotDAO);
		BookingUtils.setBookingSlotsBookingUsers(calendar, bookingSlots, getUserHandler());

		JsonObject jsonObject = new JsonObject(2);
		jsonObject.putField("titleEvents", BookingUtils.getBookingSlotsTitleJSON(bookingSlots, true, calendar));
		jsonObject.putField("detailedEvents", BookingUtils.getBookingSlotsJSON(calendar, true, preliminaryBookingText, bookingSlots));

		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
		return null;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse testSSN(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		String citizenID = uriParser.get(2);

		JsonObject jsonObject = new JsonObject();

		if (!StringUtils.isEmpty(citizenID)) {

			citizenID = citizenID.trim().replace("-", "");
		}

		if (StringUtils.isEmpty(citizenID) || !CITIZEN_ID_POPULATOR.validateFormat(citizenID)) {

			HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

			return null;
		}

		if (citizenID.length() == 10) {

			citizenID = SwedishSocialSecurityPopulator.addCentury(citizenID);
		}

		User citizenUser = systemInterface.getUserHandler().getUserByAttribute("citizenIdentifier", citizenID, false, true);

		if (citizenUser != null) {

			String phone = citizenUser.getAttributeHandler().getString("mobilePhone");

			if (phone != null) {

				jsonObject.putField("phone", phone);
			}

			addFormattedFullNameToJSON(jsonObject, citizenUser.getFirstname(), citizenUser.getLastname());

			log.info("User " + user + " validated citizen ID " + citizenID + " belonging to user " + citizenUser);

		} else {

			log.info("User " + user + " searching for citizen ID " + citizenID + " in person provider");

			PrimaryContact contact = getPrimaryContact(citizenID, null);

			if (contact != null) {

				addFormattedFullNameToJSON(jsonObject, contact.getFirstname(), contact.getLastname());

				log.info("User " + user + " validated citizen ID " + citizenID + " belonging to person " + contact);

			} else {

				log.info("User " + user + " validated citizen ID " + citizenID + ", but couldn't find any user or person with that ID");
			}
		}

		jsonObject.putField("isValid", true);

		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

		return null;
	}

	private void addFormattedFullNameToJSON(JsonObject jsonObject, String firstName, String lastName) {

		if (!StringUtils.isEmpty(firstName) && !StringUtils.isEmpty(lastName)) {

			firstName = StringUtils.toFirstLetterUppercase(firstName.toLowerCase());
			lastName = StringUtils.toFirstLetterUppercase(lastName.toLowerCase());

			jsonObject.putField("fullName", firstName + " " + lastName);
		}
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse resendBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer bookingID = uriParser.getInt(2);
		Booking booking = null;

		if (bookingID == null || (booking = getBooking(bookingID, BOOKING_RELATIONS)) == null) {

			throw new URINotFoundException(uriParser);
		}

		notificationHandler.sendManualReminderNotification(user, booking);

		redirectToMethod(req, res, "/showslot/" + booking.getBookingSlot().getBookingSlotID());

		return null;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse searchBookings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		String[] calendarIDParameters = req.getParameterValues("c");
		String query = RequestUtils.parseParameter(req, systemInterface.getEncoding(), req.getParameter("q"));

		if (calendarIDParameters != null && !StringUtils.isEmpty(query)) {

			Set<Integer> calendarIDs = new HashSet<>();

			for (String calendarIDParameter : calendarIDParameters) {

				Integer calendarID = NumberUtils.toInt(calendarIDParameter);
				BookingCalendar calendar;

				if (calendarID != null && (calendar = getCalendar(calendarID)) != null) {

					if (!hasCalendarAccess(calendar, user)) {

						throw new AccessDeniedException("User " + user + " not allowed to search bookings for calendar " + calendar);
					}

					calendarIDs.add(calendarID);
				}
			}

			if (!CollectionUtils.isEmpty(calendarIDs)) {

				LowLevelQuery<Booking> bookingQuery = new LowLevelQuery<>();
				bookingQuery.addRelations(Booking.CONTACT_RELATION, Booking.BOOKING_SLOT_RELATION);

				StringBuilder bookingSQL = new StringBuilder("SELECT * FROM ");
				bookingSQL.append(bookingDAO.getTableName());
				bookingSQL.append(" b INNER JOIN ");
				bookingSQL.append(bookingSlotDAO.getTableName());
				bookingSQL.append(" bs ON (b.bookingSlotID = bs.bookingSlotID)");
				bookingSQL.append(" INNER JOIN bookings_contacts bc ON (b.bookingID = bc.bookingID)");
				bookingSQL.append(" WHERE bs.calendarID in (");

				for (Iterator<Integer> it = calendarIDs.iterator(); it.hasNext();) {

					Integer calendarID = it.next();

					bookingQuery.addParameter(calendarID);

					bookingSQL.append(it.hasNext() ? "?," : "?");
				}

				bookingSQL.append(") AND (");

				String[] terms = query.split("[ ]+");

				for (int i = 0; i < terms.length; i++) {

					appendBookingSearchParams(bookingQuery, bookingSQL, terms[i]);

					if (terms.length - i > 1) {

						bookingSQL.append(" AND ");
					}
				}

				bookingSQL.append(") AND b.preliminary = 0 ORDER BY b.added DESC");

				bookingQuery.setSql(bookingSQL.toString());

				List<Booking> bookings = bookingDAO.getAll(bookingQuery);

				JsonArray resultArray = new JsonArray(CollectionUtils.getSize(bookings));

				CollectionUtils.forEach(bookings, booking -> resultArray.addNode(booking.toJson()));

				JsonObject jsonObject = new JsonObject(1);
				jsonObject.putField("data", resultArray);

				HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
				return null;
			}
		}

		HTTPUtils.sendReponse(new JsonObject().toJson(), JsonUtils.getContentType(), res);
		return null;
	}

	protected void appendBookingSearchParams(LowLevelQuery<Booking> query, StringBuilder paramBuilder, String searchTerm) {

		paramBuilder.append("(firstname LIKE ? OR lastname LIKE ? OR citizenIdentifier LIKE ?)");

		query.addParameter("%" + searchTerm + "%");
		query.addParameter("%" + searchTerm + "%");
		query.addParameter("%" + searchTerm + "%");
	}

	public boolean hasSMSSender() {

		return notificationHandler.hasSMSSender();
	}

	public PrimaryContact getPrimaryContact(String citizenIdentifier, String manuallyEnteredPhone) {

		User personUser = systemInterface.getUserHandler().getUserByAttribute("citizenIdentifier", citizenIdentifier, false, true);

		if (personUser != null) {

			PrimaryContact contact = new PrimaryContact(personUser);

			if (contact.getMobilePhone() == null) {

				contact.setMobilePhone(manuallyEnteredPhone);
			}

			return contact;
		}

		if (personProvider != null) {

			Person person = personProvider.getPerson(citizenIdentifier);

			if (person != null) {

				PrimaryContact contact = new PrimaryContact(person, null);

				contact.setMobilePhone(manuallyEnteredPhone);

				return contact;
			}
		}

		return null;
	}

	public String getPreliminaryBookingText() {

		return preliminaryBookingText;
	}

	public boolean addBookingSlotExtensionProvider(BookingSlotFormExtensionProvider provider) {

		return bookingSlotExtensionProviders.add(provider);
	}

	public boolean removeBookingSlotExtensionProvider(BookingSlotFormExtensionProvider provider) {

		return bookingSlotExtensionProviders.remove(provider);
	}

	public List<BookingSlotFormExtensionProvider> getBookingSlotExtensionProviders() {

		return bookingSlotExtensionProviders;
	}

	public boolean addBookingExtensionProvider(BookingExtensionProvider provider) {

		return bookingExtensionProviders.add(provider);
	}

	public boolean removeBookingExtensionProvider(BookingExtensionProvider provider) {

		return bookingExtensionProviders.remove(provider);
	}

	public List<BookingExtensionProvider> getBookingExtensionProviders() {

		return bookingExtensionProviders;
	}

	public String getDefaultArchiveText() {

		return defaultArchiveText;
	}

	public boolean isOverrideContactBySMS() {

		return calendarAdminModule.isOverrideContactBySMS();
	}

	public String getBookingSlotNotBookableTextOpens() {

		return bookingSlotNotBookableTextOpens;
	}

	public String getBookingSlotNotBookableTextClosed() {

		return bookingSlotNotBookableTextClosed;
	}

	public String getBookingSlotNotBookableTextBetween() {

		return bookingSlotNotBookableTextBetween;
	}

}
