package com.nordicpeak.flowengine.booking;

import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.function.Predicate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.UserUtils;
import se.unlogic.hierarchy.core.utils.crud.FragmentLinkScriptFilter;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLink;
import se.unlogic.hierarchy.foregroundmodules.rest.RESTMethod;
import se.unlogic.hierarchy.foregroundmodules.rest.URIParam;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.io.BinarySizeFormater;
import se.unlogic.standardutils.io.BinarySizes;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.BaseFlowModule;
import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.UserFlowInstanceMenuModule;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryInstanceDescriptor;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.booking.beans.Booking;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingEvent;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.beans.DummyInstanceRequestMetadata;
import com.nordicpeak.flowengine.booking.beans.ExternalMessage;
import com.nordicpeak.flowengine.booking.comparators.BookingSlotCalendarNameComparator;
import com.nordicpeak.flowengine.booking.enums.BookingEventType;
import com.nordicpeak.flowengine.booking.enums.ExternalMessageType;
import com.nordicpeak.flowengine.booking.events.BookingArchiveEvent;
import com.nordicpeak.flowengine.booking.events.ExternalMessageAddedEvent;
import com.nordicpeak.flowengine.booking.interfaces.BookingExtensionProvider;
import com.nordicpeak.flowengine.booking.notificationhandler.BookingNotificationHandler;
import com.nordicpeak.flowengine.booking.queries.BookingQuery;
import com.nordicpeak.flowengine.booking.queries.BookingQueryInstance;
import com.nordicpeak.flowengine.booking.queries.BookingQueryProviderModule;
import com.nordicpeak.flowengine.booking.queries.BookingQueryRequestProcessor;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.enums.SenderType;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.UserMenuProvider;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;

public class UserBookingModule extends BaseBookingModule implements UserMenuProvider {

	@XSLVariable(prefix = "java.")
	private String userMenuTabTitle = "My bookings";

	@XSLVariable(prefix = "java.")
	private String changeBookingSlotQueryName = "Choose new time";

	@XSLVariable(prefix = "java.")
	private String changeBookingSlotQueryConfirm = "Confirm";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "User menu item slot", description = "User menu item slot")
	private String userMenuExtensionLinkSlot = "50";

	@InstanceManagerDependency(required = true)
	private CalendarAdminModule calendarAdminModule;

	@InstanceManagerDependency(required = true)
	private BookingNotificationHandler notificationHandler;

	private UserFlowInstanceMenuModule userFlowInstanceMenuModule;

	private static final Comparator<Booking> QUEUE_SLOT_QUEUE_NAME_COMPARATOR = new BookingSlotCalendarNameComparator();

	private AnnotatedDAO<BookingCalendar> calendarDAO;
	private AnnotatedDAOWrapper<Booking, Integer> bookingDAOWrapper;
	private AnnotatedDAOWrapper<BookingCalendar, Integer> calendarDAOWrapper;

	private ExtensionLink userMenuLink;

	@InstanceManagerDependency(required = true)
	private QueryHandler queryHandler;

	private CopyOnWriteArrayList<BookingExtensionProvider> bookingExtensionProviders = new CopyOnWriteArrayList<>();

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(UserBookingModule.class, this)) {

			throw new RuntimeException("Unable to register module " + this.moduleDescriptor + " in global instance handler using key " + UserBookingModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}

	}
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		calendarDAO = daoFactory.getDAO(BookingCalendar.class);

		bookingDAOWrapper = bookingDAO.getWrapper(Integer.class);
		calendarDAOWrapper = calendarDAO.getWrapper(Integer.class);
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(UserBookingModule.class, this);

		if (userFlowInstanceMenuModule != null) {

			setUserFlowInstanceMenuModule(null);
		}

		super.unload();
	}

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		this.userMenuLink = new ExtensionLink(userMenuTabTitle, getFullAlias(), ">", userMenuExtensionLinkSlot);

		if (userFlowInstanceMenuModule != null) {

			userFlowInstanceMenuModule.sortProviders();
		}

	}

	@InstanceManagerDependency
	public void setUserFlowInstanceMenuModule(UserFlowInstanceMenuModule userFlowInstanceMenuModule) {

		if (userFlowInstanceMenuModule == null && this.userFlowInstanceMenuModule != null) {

			this.userFlowInstanceMenuModule.removeUserMenuProvider(this);
		}

		this.userFlowInstanceMenuModule = userFlowInstanceMenuModule;

		if (this.userFlowInstanceMenuModule != null) {

			this.userFlowInstanceMenuModule.addUserMenuProvider(this);
		}
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException, AccessDeniedException {

		if (user == null) {

			throw new AccessDeniedException("User needs to be logged in");
		}

		log.info("User " + user + " listing bookings");

		Document doc = createDocument(req, uriParser, user);

		Element userBookingSlots = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "ListUserBookings");

		XMLUtils.append(doc, userBookingSlots, getUserBookings(user));

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
	}

	@RESTMethod(alias = "overview/{bookingID}", method = { "get", "post" }, requireLogin = true)
	public ForegroundModuleResponse showBookingOverview(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingID") Integer bookingID) throws SQLException, URINotFoundException, IOException, TransformerException {

		Booking booking = getBooking(user, bookingID, Booking.EXTERNAL_MESSAGES_RELATION, ExternalMessage.ATTACHMENTS_RELATION);

		if (booking == null) {

			throw new URINotFoundException(uriParser);
		}

		log.info("User " + user + " requested booking " + booking);

		Document doc = createDocument(req, uriParser, user);

		Element showBookingSlotOverview = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "ShowBookingOverview");

		showBookingSlotOverview.appendChild(booking.toXML(doc));

		XMLUtils.appendNewElement(doc, showBookingSlotOverview, "FormatedMaxFileSize", BinarySizeFormater.getFormatedSize(maxFileSize * BinarySizes.MegaByte));

		XMLUtils.appendNewElement(doc, showBookingSlotOverview, "IsNewExternalMessagesAllowed", BookingUtils.isNewExternalMessagesAllowed(booking));

		if (req.getParameter("requestedleave") != null) {

			XMLUtils.appendNewElement(doc, showBookingSlotOverview, "RequestedLeave");
		}

		XMLUtils.appendNewElement(doc, showBookingSlotOverview, "HideSubmittedEvent", calendarAdminModule.isHideSubmittedEvent());

		appendExtensionViewFragments(booking, doc, showBookingSlotOverview, user, req, res, uriParser);

		if (req.getMethod().equalsIgnoreCase("POST") && BookingUtils.isNewExternalMessagesAllowed(booking)) {

			//TODO append message or request parameters
			ExternalMessage externalMessage = externalMessageCRUD.add(req, user, doc, showBookingSlotOverview, booking, false);

			if (externalMessage != null) {

				booking = getBooking(user, bookingID, Booking.EXTERNAL_MESSAGES_RELATION, ExternalMessage.ATTACHMENTS_RELATION, BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION);

				BookingEvent bookingEvent = addBookingEvent(booking, BookingEventType.CUSTOMER_MESSAGE_SENT, null, user, null, null);

				getEventHandler().sendEvent(Booking.class, new ExternalMessageAddedEvent(booking, bookingEvent, externalMessage, SenderType.USER), EventTarget.ALL);
				getEventHandler().sendEvent(ExternalMessage.class, new CRUDEvent<>(CRUDAction.ADD, externalMessage), EventTarget.ALL);

				res.sendRedirect(req.getContextPath() + uriParser.getFormattedURI() + "#messages");

				return null;
			}
		}

		SimpleForegroundModuleResponse moduleResponse = new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());

		moduleResponse = new FragmentLinkScriptFilter<>().filterShowBeanModuleResponse(moduleResponse, booking, doc, req, user, uriParser);

		return moduleResponse;
	}

	protected void appendExtensionViewFragments(Booking booking, Document doc, Element element, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, TransformerException {

		if (bookingExtensionProviders.isEmpty()) {

			return;
		}

		Element pluginFragments = XMLUtils.appendNewElement(doc, element, "ExtensionFragments");

		for (BookingExtensionProvider extensionProvider : bookingExtensionProviders) {

			ViewFragment extensionSettings = extensionProvider.getUserBookingViewFragment(booking, req, res, user, uriParser);

			if (extensionSettings != null) {

				pluginFragments.appendChild(extensionSettings.toXML(doc));
				FragmentLinkScriptFilter.addViewFragment(extensionSettings, req);
			}
		}
	}

	@WebPublic(alias = "messages")
	public ForegroundModuleResponse showBookingMessages(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws IOException, URINotFoundException {

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2))) {

			redirectToMethod(req, res, "/overview/" + uriParser.get(2) + "#messages");

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	@RESTMethod(alias = "leavebooking/{bookingSlotID}", method = "post", requireLogin = true)
	public ForegroundModuleResponse leaveBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingSlotID") Integer bookingSlotID) throws SQLException, URINotFoundException, IOException {

		Booking booking = getBooking(user, bookingSlotID, BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION);

		if (booking == null) {

			throw new URINotFoundException(uriParser);
		}

		if (!booking.isMutable() || booking.getBookingSlot().getCalendar().isDisableLeaving()) {

			res.sendRedirect(req.getContextPath() + getFullAlias() + "/overview/" + booking.getBookingID());
			return null;
		}

		String comment = req.getParameter("comment");

		log.info("User " + user + " archiving booking " + booking + " with reason: " + StringUtils.toLogFormat(comment, 50));

		addBookingEvent(booking, BookingEventType.ARCHIVED, eventLeaveMessage + (StringUtils.isEmpty(comment) ? "" : eventSeeExternalMessage), user, null, null);

		booking.setArchived(true);
		bookingDAO.update(booking);

		if (!StringUtils.isEmpty(comment)) {

			addBookingExternalMessage(booking, comment, false, user, ExternalMessageType.ARCHIVE_BY_USER, null);
		}

		getEventHandler().sendEvent(Booking.class, new BookingArchiveEvent(booking, user), EventTarget.ALL);

		redirectToDefaultMethod(req, res);
		return null;
	}

	private Booking getBooking(User user, Integer bookingID, Field... relations) throws SQLException {

		String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

		if (citizenIdentifier == null) {

			return null;
		}

		HighLevelQuery<Booking> query = new HighLevelQuery<>();

		query.addRelations(Booking.CONTACT_RELATION, Booking.EVENTS_RELATION, BookingEvent.ATTRIBUTES_RELATION, Booking.BOOKING_SLOT_RELATION, BookingSlot.CALENDAR_RELATION, BookingSlot.MANAGERS_RELATION);

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(bookingDAOWrapper.getParameterFactory().getParameter(bookingID));

		Booking booking = bookingDAO.get(query);

		if (booking != null && !booking.isPreliminary() && booking.getPrimaryContactCitizenIdentifier() != null && citizenIdentifier.equals(booking.getPrimaryContactCitizenIdentifier())) {

			return booking;
		}

		return null;
	}

	private List<Booking> getUserBookings(User user) throws SQLException {

		boolean internalUser = false;

		String citizenIdentifier = UserUtils.getAttribute(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE, user);

		if (StringUtils.isEmpty(citizenIdentifier)) {

			citizenIdentifier = UserUtils.getAttribute(Constants.MANAGER_CITIZEN_IDENTIFIER_ATTRIBUTE, user);

			if (!StringUtils.isEmpty(citizenIdentifier)) {

				internalUser = true;
			}
		}

		if (citizenIdentifier == null) {

			return null;
		}

		//@formatter:off
		LowLevelQuery<Booking> query = new LowLevelQuery<>(
				"SELECT b.* FROM bookings b " +
				"INNER JOIN bookings_contacts p ON b.bookingID = p.bookingID " +
				"INNER JOIN booking_slots s ON s.bookingSlotID = b.bookingSlotID " +
				"INNER JOIN booking_calendars c ON s.calendarID = c.calendarID " +
				"WHERE c.enabled = true AND b.preliminary = 0 AND p.citizenIdentifier = ?");
		//@formatter:on

		query.addRelations(Booking.CONTACT_RELATION, Booking.EVENTS_RELATION, Booking.BOOKING_SLOT_RELATION, BookingSlot.CALENDAR_RELATION);

		query.addParameter(citizenIdentifier);

		List<Booking> bookings = bookingDAO.getAll(query);

		if (bookings != null) {

			Collections.sort(bookings, QUEUE_SLOT_QUEUE_NAME_COMPARATOR);

			if (!internalUser) {

				bookings.removeIf(booking -> booking.getBookingSlot().getCalendar().isUseCalendarForInternalUsers());
			}
		}

		return bookings;
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = super.createDocument(req, uriParser, user);

		XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "ModuleURL", uriParser.getContextPath() + getFullAlias());

		return doc;
	}

	@Override
	public AccessInterface getAccessInterface() {

		return moduleDescriptor;
	}

	@Override
	public String getUserMenuAlias() {

		return getFullAlias();
	}

	@Override
	public ExtensionLink getUserMenuExtensionLink(User user) {

		return userMenuLink;
	}

	@Override
	public String getUserMenuPriority() {

		return userMenuExtensionLinkSlot;
	}

	public BookingCalendar getCalendar(Integer calendarID, Field... relations) throws SQLException {

		HighLevelQuery<BookingCalendar> query = new HighLevelQuery<>();

		if (relations != null) {

			query.addRelations(relations);
		}

		query.addParameter(calendarDAOWrapper.getParameterFactory().getParameter(calendarID));

		return calendarDAO.get(query);
	}

	private BookingQueryInstance getDummyBookingQueryInstance(BookingCalendar calendar, Booking booking) {

		QueryDescriptor queryDescriptor = new QueryDescriptor();
		queryDescriptor.setQueryID(0);
		queryDescriptor.setDefaultQueryState(QueryState.VISIBLE_REQUIRED);
		queryDescriptor.setName(changeBookingSlotQueryName);

		QueryInstanceDescriptor queryInstanceDescriptor = new QueryInstanceDescriptor(queryDescriptor);

		BookingQuery bookingQuery = new BookingQuery();
		bookingQuery.setQueryID(0);
		bookingQuery.setCalendar(calendar);
		bookingQuery.setDescription(calendar.getDescription());
		bookingQuery.setAcceptText(changeBookingSlotQueryConfirm);
		bookingQuery.setMinVisibleDays(calendar.getMinVisibleDaysRebooking());

		BookingQueryInstance queryInstance = new BookingQueryInstance();
		queryInstance.setQuery(bookingQuery);
		queryInstance.set(queryInstanceDescriptor);
		queryInstance.setSize(booking.getSize());
		queryInstance.setRebooking(true);

		return queryInstance;
	}

	@RESTMethod(alias = "updatebooking/{bookingID}", method = { "get", "post" }, requireLogin = true)
	public ForegroundModuleResponse updateBooking(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "bookingID") Integer bookingID) throws Throwable {

		Booking booking = getBooking(user, bookingID, Booking.BOOKING_SLOT_RELATION);

		if (booking == null) {

			throw new URINotFoundException(uriParser);
		}

		booking.getPoster(systemInterface.getUserHandler());

		BookingCalendar calendar = getCalendar(booking.getBookingSlot().getCalendar().getCalendarID(), BookingCalendar.MANAGER_USERS_RELATION, BookingCalendar.MANAGER_GROUPS_RELATION);
		calendar.setPrice(null);

		if (!booking.isMutable(calendar) || calendar.isDisableRebooking()) {

			res.sendRedirect(req.getContextPath() + getFullAlias() + "/overview/" + booking.getBookingID());
			return null;
		}

		BookingQueryInstance queryInstance = getDummyBookingQueryInstance(calendar, booking);
		BookingQueryProviderModule bookingQueryProviderModule = queryHandler.getAssignableQueryProviders(BookingQueryProviderModule.class).get(0);
		bookingQueryProviderModule.setBookingSlotsFromDB(queryInstance);

		List<User> currentBookingManagers = booking.getBookingSlot().getManagers();
		if (calendar.isRestrictRebookingToPreviousManager() && currentBookingManagers != null && queryInstance.getQuery().getCalendar().getBookingSlots() != null) {

			Predicate<BookingSlot> noIntersectionBetweenManagers = slot -> CollectionUtils.find(slot.getManagers(), currentBookingManagers::contains) == null;

			queryInstance.getQuery().getCalendar().getBookingSlots().removeIf(noIntersectionBetweenManagers);
		}

		if (req.getParameter("m") != null) {

			new BookingQueryRequestProcessor(queryInstance, systemInterface).processRequest(req, res, user, uriParser);
			return null;
		}

		InstanceRequestMetadata requestMetadata = getRequestMetadata(req, uriParser, booking);

		if (requestMetadata == null) {

			redirectToMethod(req, res, "/overview/" + booking.getBookingID());
			return null;
		}

		List<ValidationError> validationErrors = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			validationErrors = new ArrayList<>();

			try {

				bookingQueryProviderModule.populate(queryInstance, req, user, booking.getPoster(), false, null, requestMetadata);

			} catch (ValidationException e) {

				validationErrors.addAll(e.getErrors());
			}

			if (validationErrors.isEmpty()) {

				Timestamp now = new Timestamp(System.currentTimeMillis());

				log.info("User " + user + " updating booking " + booking + " to booking slot " + queryInstance.getBookingSlot() + " size: " + queryInstance.getSize() + " from " + booking.getBookingSlot() + " size: " + booking.getSize());

				BookingSlot oldSlot = booking.getBookingSlot();

				booking.setBookingSlot(queryInstance.getBookingSlot());
				booking.setUpdated(now);

				if (!calendar.isRestrictRebookingToCurrentBookingSize()) {

					booking.setSize(queryInstance.getSize());
				}

				HighLevelQuery<Booking> updateQuery = new HighLevelQuery<>();
				bookingDAO.update(booking, updateQuery);

				systemInterface.getEventHandler().sendEvent(Booking.class, new CRUDEvent<>(Booking.class, CRUDAction.UPDATE, booking), EventTarget.ALL);

				StringBuilder details = new StringBuilder();
				details.append(getEventChangedBookingSlotFrom());
				details.append(" ");
				details.append(DateUtils.DATE_FORMATTER.format(oldSlot.getStart()) + " " + oldSlot.getTimeFormatted(calendar));
				details.append(" ");
				details.append(getEventChangedBookingSlotTo());
				details.append(" ");
				details.append(DateUtils.DATE_FORMATTER.format(booking.getBookingSlot().getStart()) + " " + booking.getBookingSlot().getTimeFormatted(calendar));

				addBookingEvent(booking, BookingEventType.CHANGED_SLOT, details.toString(), user, now, null);

				booking.getBookingSlot().setCalendar(calendar);

				notificationHandler.sendUserRebookedNotifications(booking, calendar, oldSlot);

				if (booking.getFlowInstanceID() != null) {

					FlowInstance flowInstance = calendarAdminModule.getFlowAdminModule().getFlowInstance(booking.getFlowInstanceID());

					if (flowInstance != null) {

						calendarAdminModule.getFlowAdminModule().getFlowInstanceEventGenerator().addFlowInstanceEvent(flowInstance, EventType.OTHER_EVENT, calendar.getName() + ": " + details.toString(), user);
					}
				}

				redirectToMethod(req, res, "/overview/" + booking.getBookingID());
				return null;
			}
		}

		log.info("User " + user + " requested update booking form for " + booking);

		Document doc = createDocument(req, uriParser, user);

		Element updateBooking = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "UpdateBooking");

		updateBooking.appendChild(booking.toXML(doc));

		QueryResponse queryResponse = bookingQueryProviderModule.getFormHTML(queryInstance, req, user, booking.getPoster(), validationErrors, false, uriParser.getRequestURL(), requestMetadata, null);
		XMLUtils.append(doc, updateBooking, queryResponse);

		if (validationErrors != null) {

			XMLUtils.append(doc, updateBooking, validationErrors);
		}

		SimpleForegroundModuleResponse moduleResponse = new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());

		if (queryResponse.getLinks() != null) {

			moduleResponse.addLinks(queryResponse.getLinks());
		}

		if (queryResponse.getScripts() != null) {

			moduleResponse.addScripts(queryResponse.getScripts());
		}

		return moduleResponse;
	}

	private InstanceRequestMetadata getRequestMetadata(HttpServletRequest req, URIParser uriParser, Booking booking) throws Exception {

		if (booking.getFlowInstanceID() == null) {

			//Manager created booking
			return new DummyInstanceRequestMetadata(BaseFlowModule.OWNER_REQUEST_METADATA);
		}

		FlowInstanceManager flowInstanceManager = getFlowInstanceManager(booking, req, uriParser);

		if (flowInstanceManager == null) {

			return null;
		}

		return new InstanceRequestMetadata(BaseFlowModule.OWNER_REQUEST_METADATA, flowInstanceManager);
	}

	private FlowInstanceManager getFlowInstanceManager(Booking booking, HttpServletRequest req, URIParser uriParser) throws Exception {

		FlowInstance flowInstance = calendarAdminModule.getFlowAdminModule().getFlowInstance(booking.getFlowInstanceID());

		if (flowInstance == null) {

			log.warn("Could not get flow instance manager for booking " + booking + ", flowInstance is null");
			return null;
		}

		SiteProfile siteProfile = getFlowAdminModule().getSiteProfile(flowInstance);
		String absoluteFileURL = getFlowAdminModule().getAbsoluteFileURL(uriParser, flowInstance.getFlow());

		return new ImmutableFlowInstanceManager(flowInstance, queryHandler, req, new DefaultInstanceMetadata(siteProfile), absoluteFileURL);
	}

	private FlowAdminModule getFlowAdminModule() {

		return calendarAdminModule.getFlowAdminModule();
	}

	public boolean addUserBookingExtensionProvider(BookingExtensionProvider provider) {

		return bookingExtensionProviders.add(provider);
	}

	public boolean removeUserBookingExtensionProvider(BookingExtensionProvider provider) {

		return bookingExtensionProviders.remove(provider);
	}

	public List<BookingExtensionProvider> getUserBookingExtensionProviders() {

		return bookingExtensionProviders;
	}

}
