package com.nordicpeak.flowengine.booking.beans;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.queries.contactdetailquery.ContactDetailQueryInstance;
import com.nordicpeak.personprovider.Person;

@Table(name = "bookings_contacts")
@XMLElement
public class PrimaryContact extends BaseContact {

	private static final long serialVersionUID = 7634780772195411121L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer bookingID;

	@DAOManaged
	@OneToOne
	@XMLElement
	private Booking booking;

	@DAOManaged
	@XMLElement
	protected String citizenIdentifier;

	public PrimaryContact() {}

	public PrimaryContact(User user) {

		super(user);

		AttributeHandler attributeHandler = user.getAttributeHandler();

		if (attributeHandler != null) {

			citizenIdentifier = attributeHandler.getString("citizenIdentifier");
		}
	}

	public PrimaryContact(ContactDetailQueryInstance contactDetailQueryInstance) {

		super(contactDetailQueryInstance);

		citizenIdentifier = contactDetailQueryInstance.getCitizenID();
	}

	public PrimaryContact(Person person, User user) {

		super(person, user);

		citizenIdentifier = person.getCitizenIdentifier();
	}

	public Booking getBookingSlot() {

		return booking;
	}

	public void setBookingSlot(Booking booking) {

		this.booking = booking;
	}

	public void setCitizenIdentifier(String citizenIdentifier) {

		this.citizenIdentifier = citizenIdentifier;
	}

	public String getCitizenIdentifier() {

		return citizenIdentifier;
	}

}
