package com.nordicpeak.flowengine.booking.populators;

import java.awt.image.BufferedImage;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.math.BigDecimal;
import java.sql.Blob;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import javax.sql.rowset.serial.SerialBlob;

import org.apache.log4j.Logger;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.standardutils.base64.Base64;
import se.unlogic.standardutils.image.ImageUtils;
import se.unlogic.standardutils.populators.BaseStringPopulator;
import se.unlogic.standardutils.populators.BeanStringPopulator;
import se.unlogic.standardutils.populators.BigDecimalPopulator;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.DummyPopulator;
import se.unlogic.standardutils.populators.HSAIDPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.LongPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.PercentageStringIntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.populators.SwedishSocialSecurity10DigitsPopulator;
import se.unlogic.standardutils.populators.SwedishSocialSecurity12DigitsPopulator;
import se.unlogic.standardutils.populators.TimePopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLPopulationUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;
import se.unlogic.webutils.populators.StringHTTPURLPopulator;

import com.nordicpeak.flowengine.booking.populators.annotations.PopulateByXMLSettings;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

//TODO rewrite as dynamic caching populator based on the same structure as AnnotatedRequestPopulator
public class PopulateByXMLParser {

	private static final int DEFAULT_MINIMUM_STRING_LENGTH = 1;
	private static final int DEFAULT_MAXIMUM_STRING_LENGTH = 255;
	protected static Logger log = Logger.getLogger(PopulateByXMLParser.class);
	private static final DummyPopulator DUMMY_POPULATOR = new DummyPopulator();

	private PopulateByXMLParser() {}

	public static void populateByParser(Field[] fields, XMLParser xmlParser, Object instance, List<ValidationError> errors) {

		try {

			for (Field field : fields) {

				PopulateByXMLSettings populateXMLSettings = field.getAnnotation(PopulateByXMLSettings.class);

				if (populateXMLSettings == null) {

					continue;
				}

				Class<? extends XMLParserPopulateable> clazz = populateXMLSettings.clazz();

				if (field.getType().equals(Boolean.class) || field.getType().equals(boolean.class)) {

					ReflectionUtils.setFieldValue(field, xmlParser.getPrimitiveBoolean(field.getName()), instance);

				} else if (field.getType().equals(String.class)) {

					populateStringField(xmlParser, instance, errors, field, populateXMLSettings);

				} else if (field.getType().equals(Integer.class)) {

					populateIntegerField(xmlParser, instance, errors, field, populateXMLSettings);

				} else if (field.getType().equals(Long.class)) {

					populateLongField(xmlParser, instance, errors, field, populateXMLSettings);

				} else if (field.getType().equals(Time.class)) {

					populateTimeField(xmlParser, instance, errors, field, populateXMLSettings);

				} else if (field.getType().equals(Date.class)) {

					populateDateField(xmlParser, instance, errors, field, populateXMLSettings);

				} else if (field.getType().equals(Timestamp.class)) {

					ReflectionUtils.setFieldValue(field, TimeUtils.getCurrentTimestamp(), instance);

				} else if (field.getType().equals(List.class)) {

					populateListField(xmlParser, instance, errors, field, populateXMLSettings, clazz);

				} else if (field.getType().isEnum()) {

					populateEnumField(xmlParser, instance, errors, field);

				} else if (field.getType().equals(BigDecimal.class)) {

					populateBigDecimalField(xmlParser, instance, errors, field, populateXMLSettings);

				} else if (field.getType().equals(Blob.class)) {

					populateBlobField(xmlParser, instance, errors, field);

				} else if (!clazz.equals(BaseEvaluator.class)) {

					ReflectionUtils.setFieldValue(field, XMLPopulationUtils.populateBean(xmlParser, field.getName(), clazz, errors), instance);

				} else {

					errors.add(new ValidationError("UnableToParseRequest"));
					log.warn("Could not populate field, unimplemented type " + field.getType());
				}
			}

		} catch (Exception ex) {

			errors.add(new ValidationError(ex.getMessage()));
		}
	}

	private static void populateStringField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings) throws IllegalAccessException {

		int minLength = getMinLength(populateXMLSettings);
		int maxLength = getMaxLength(populateXMLSettings);

		BaseStringPopulator<?> populator = getPopulator(populateXMLSettings.populator(), String.class);

		ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(field.getName(), xmlParser, populateXMLSettings.required(), minLength, maxLength, populator, errors), instance);
	}

	private static void populateIntegerField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings) throws IllegalAccessException {

		BaseStringPopulator<?> populator = getPopulator(populateXMLSettings.populator(), Integer.class);

		ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(field.getName(), xmlParser, populateXMLSettings.required(), populator, errors), instance);

	}

	private static void populateLongField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings) throws IllegalAccessException {

		BaseStringPopulator<?> populator = getPopulator(populateXMLSettings.populator(), Long.class);

		ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(field.getName(), xmlParser, populateXMLSettings.required(), populator, errors), instance);

	}

	private static void populateTimeField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings) throws IllegalAccessException {

		ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(field.getName(), xmlParser, populateXMLSettings.required(), TimePopulator.getPopulator(), errors), instance);
	}

	private static void populateBigDecimalField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings) throws IllegalAccessException {

		BaseStringPopulator<?> populator = getPopulator(populateXMLSettings.populator(), BigDecimal.class);

		ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(field.getName(), xmlParser, populateXMLSettings.required(), populator, errors), instance);

	}

	private static void populateListField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings, Class<? extends XMLParserPopulateable> clazz) throws IllegalAccessException {

		String nodeName = populateXMLSettings.nodeName();

		if (StringUtils.isEmpty(nodeName)) {

			errors.add(new ValidationError("UnableToParseRequest"));
			log.warn("Faulty configuration, field is a list and attribute nodename must be specified.");
			return;
		}

		int minLength = getMinLength(populateXMLSettings);
		int maxLength = getMaxLength(populateXMLSettings);

		ParameterizedType listType = (ParameterizedType) field.getGenericType();
		Class<?> listClass = (Class<?>) listType.getActualTypeArguments()[0];

		BeanStringPopulator<?> populator = getPopulator(populateXMLSettings.populator(), listType.getClass());

		if (listClass.equals(String.class)) {

			ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameters(nodeName, xmlParser, populateXMLSettings.required(), minLength, maxLength, populator, errors), instance);

		} else if (listClass.equals(Integer.class) || listClass.equals(Long.class) || listClass.equals(Date.class)) {

			ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(nodeName, xmlParser, populateXMLSettings.required(), populator, errors), instance);

		} else if (listClass.equals(Time.class)) {

			ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(nodeName, xmlParser, populateXMLSettings.required(), TimePopulator.getPopulator(), errors), instance);

		} else if (listClass.isEnum()) {

			populateEnumListField(xmlParser, instance, errors, field, nodeName, listClass);

		} else if (listClass.equals(BigDecimal.class)) {

			ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(nodeName, xmlParser, populateXMLSettings.required(), populator, errors), instance);

		} else if (listClass.equals(clazz)) {

			ReflectionUtils.setFieldValue(field, XMLPopulationUtils.populateBeans(xmlParser, nodeName, clazz, errors), instance);

		} else {

			errors.add(new ValidationError("UnableToParseRequest"));
			log.warn("Could not populate field, unimplemented type " + listClass);
		}
	}

	private static void populateDateField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, PopulateByXMLSettings populateXMLSettings) throws IllegalAccessException {

		BaseStringPopulator<?> populator = getPopulator(populateXMLSettings.populator(), Date.class);

		ReflectionUtils.setFieldValue(field, XMLValidationUtils.validateParameter(field.getName(), xmlParser, populateXMLSettings.required(), populator, errors), instance);
	}

	private static void populateEnumListField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field, String nodeName, Class<?> listClass) throws IllegalAccessException {

		List<?> tempList = xmlParser.getStrings(nodeName);

		if (tempList != null) {

			List<Object> list = new ArrayList<>();

			Arrays.asList(listClass.getEnumConstants()).stream().forEach(e -> {

				if (tempList.stream().anyMatch(e1 -> e1.equals(String.valueOf(e)))) {

					list.add(e);
				}
			});

			if (tempList.size() != list.size()) {

				for (Object object : tempList) {

					if (Arrays.asList(listClass.getEnumConstants()).stream().noneMatch(e1 -> e1.toString().equals(String.valueOf(object)))) {

						errors.add(new ValidationError("UnableToParseRequest"));
						log.warn("Could not parse EnumValue " + object + " .Possible enumValues: " + Arrays.asList(listClass.getEnumConstants()));
					}
				}
			}

			if (!list.isEmpty()) {

				ReflectionUtils.setFieldValue(field, list, instance);
			}
		}
	}

	private static void populateEnumField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field) throws IllegalAccessException {

		Object value = xmlParser.getString(field.getName());
		Optional<?> enumValue = Arrays.asList(field.getType().getEnumConstants()).stream().filter(e -> String.valueOf(e).equals(value)).findFirst();

		if (!enumValue.isPresent()) {

			errors.add(new ValidationError("UnableToParseRequest"));
			log.warn("Could not parse EnumValue " + value + " .Possible enumValues: " + Arrays.asList(field.getType().getEnumConstants()));

		} else {

			ReflectionUtils.setFieldValue(field, enumValue.get(), instance);
		}

	}

	private static void populateBlobField(XMLParser xmlParser, Object instance, List<ValidationError> errors, Field field) {

		String imageValue = xmlParser.getString(field.getName());

		if (!StringUtils.isEmpty(imageValue)) {

			try {

				byte[] imageData = Base64.decode(imageValue);

				BufferedImage bufferedImage = ImageUtils.getImage(imageData);

				if (bufferedImage != null) {

					ReflectionUtils.setFieldValue(field, new SerialBlob(imageData), instance);

				} else {

					errors.add(new ValidationError("UnableToParseIcon"));
					log.warn("Could not parse icon-string for icon: " + imageValue);
				}

			} catch (Exception e) {

				errors.add(new ValidationError("UnableToParseIcon"));
				log.warn("Could not parse icon-string for icon: " + imageValue);
			}
		}
	}

	private static BaseStringPopulator<?> getPopulator(Class<? extends BeanStringPopulator<?>> populator, Class<?> classType) {

		if (populator.isInstance(DUMMY_POPULATOR)) {

			return getPopulatorByClassType(classType);
		}

		if (populator.isInstance(StringPopulator.getPopulator())) {

			return StringPopulator.getPopulator();
		}

		if (populator.isInstance(IntegerPopulator.getPopulator())) {

			return IntegerPopulator.getPopulator();
		}

		if (populator.isInstance(PositiveStringIntegerPopulator.getPopulator())) {

			return PositiveStringIntegerPopulator.getPopulator();
		}

		if (populator.isInstance(NonNegativeStringIntegerPopulator.getPopulator())) {

			return NonNegativeStringIntegerPopulator.getPopulator();
		}

		if (populator.isInstance(TimePopulator.getPopulator())) {

			return TimePopulator.getPopulator();
		}

		if (populator.isInstance(DatePopulator.getPopulator())) {

			return DatePopulator.getPopulator();
		}

		if (populator.isInstance(EmailPopulator.getPopulator())) {

			return EmailPopulator.getPopulator();
		}

		if (populator.isInstance(PercentageStringIntegerPopulator.getPopulator())) {

			return PercentageStringIntegerPopulator.getPopulator();
		}

		if (populator.isInstance(HSAIDPopulator.getPopulator())) {

			return HSAIDPopulator.getPopulator();
		}

		if (populator.isInstance(StringSwedishPhoneNumberPopulator.getPopulator())) {

			return StringSwedishPhoneNumberPopulator.getPopulator();
		}

		if (populator.isInstance(SwedishSocialSecurity10DigitsPopulator.getPopulator())) {

			return SwedishSocialSecurity10DigitsPopulator.getPopulator();
		}

		if (populator.isInstance(SwedishSocialSecurity12DigitsPopulator.getPopulator())) {

			return SwedishSocialSecurity12DigitsPopulator.getPopulator();
		}

		if (populator.isInstance(StringHTTPURLPopulator.getPopulator())) {

			return StringHTTPURLPopulator.getPopulator();
		}

		if (populator.isInstance(BigDecimalPopulator.getPopulator())) {

			return BigDecimalPopulator.getPopulator();
		}

		return getPopulatorByClassType(classType);
	}

	private static BaseStringPopulator<?> getPopulatorByClassType(Class<?> classType) {

		if (classType.equals(String.class)) {

			return StringPopulator.getPopulator();
		}

		if (classType.equals(Integer.class)) {

			return IntegerPopulator.getPopulator();
		}

		if (classType.equals(Long.class)) {

			return LongPopulator.getPopulator();
		}

		if (classType.equals(Date.class)) {

			return DatePopulator.getPopulator();
		}

		if (classType.equals(Time.class)) {

			return TimePopulator.getPopulator();
		}

		if (classType.equals(BigDecimal.class)) {

			return BigDecimalPopulator.getPopulator();
		}

		return StringPopulator.getPopulator();
	}

	private static int getMaxLength(PopulateByXMLSettings populateXMLSettings) {

		return (int) populateXMLSettings.maxLength() == 0 ? DEFAULT_MAXIMUM_STRING_LENGTH : (int) populateXMLSettings.maxLength();
	}

	private static int getMinLength(PopulateByXMLSettings populateXMLSettings) {

		return (int) populateXMLSettings.minLength() == 0 ? DEFAULT_MINIMUM_STRING_LENGTH : (int) populateXMLSettings.minLength();
	}
}
