package com.nordicpeak.flowengine.booking.queries;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.Templated;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.beans.BasePaymentProduct;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.PaymentProduct;
import com.nordicpeak.flowengine.interfaces.PaymentQuery;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesBaseQuery;

@Table(name = "booking_queries")
@XMLElement
public class BookingQuery extends FixedAlternativesBaseQuery implements PaymentQuery {

	public static final Field CALENDAR_RELATION = ReflectionUtils.getField(BookingQuery.class, "calendar");

	private static final Map<Integer, Integer> ALTERNATIVE_CONVERSION_MAP = Collections.singletonMap(1, 1);

	private static final long serialVersionUID = -842191226937409416L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@XMLElement
	private String acceptText;

	@DAOManaged
	@Templated
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String customPriceText;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String invoiceLineText;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String commentName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showCommentField;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showLocationInList;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showLocationFilter;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer minVisibleDays;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer archiveStatusID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer bookingSlotPassedStatusID;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	@RequiredIfSet(paramNames = "bookingSlotPassedStatusID")
	private Integer bookingSlotPassedStatusTime;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<BookingQueryInstance> instances;

	@DAOManaged(columnName = "calendarID")
	@ManyToOne
	@XMLElement
	private BookingCalendar calendar;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean addBookingSlotManagersToFlowInstance;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showFirstFreeSlot;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String noFreeSlotsText;

	private ImmutableAlternative bookingSlotSelectedAlternative;

	private String invoiceLineTextFallback;

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<BookingQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<BookingQueryInstance> instances) {

		this.instances = instances;
	}

	public BookingCalendar getCalendar() {

		return calendar;
	}

	public void setCalendar(BookingCalendar calendar) {

		this.calendar = calendar;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public String getInvoiceLineText() {

		return invoiceLineText;
	}

	public void setInvoiceLineText(String invoiceLineText) {

		this.invoiceLineText = invoiceLineText;
	}

	public String getInvoiceLineTextFallback() {

		return invoiceLineTextFallback;
	}

	public void setInvoiceLineTextFallback(String invoiceLineTextFallback) {

		this.invoiceLineTextFallback = invoiceLineTextFallback;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return "BookingQuery (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return "BookingQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		Element commentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		commentElement.setAttribute("name", "Comment");
		commentElement.setAttribute("type", "xs:string");
		commentElement.setAttribute("minOccurs", "0");
		commentElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(commentElement);

		Element acceptedElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		acceptedElement.setAttribute("name", "Accepted");
		acceptedElement.setAttribute("type", "xs:string");
		acceptedElement.setAttribute("minOccurs", "0");
		acceptedElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(acceptedElement);

		Element calendarElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		calendarElement.setAttribute("name", "Calendar");
		calendarElement.setAttribute("type", "xs:string");
		calendarElement.setAttribute("minOccurs", "1");
		calendarElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(calendarElement);

		Element dateElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		dateElement.setAttribute("name", "Date");
		dateElement.setAttribute("type", "xs:string");
		dateElement.setAttribute("minOccurs", "1");
		dateElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(dateElement);

		Element timeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		timeElement.setAttribute("name", "Time");
		timeElement.setAttribute("type", "xs:string");
		timeElement.setAttribute("minOccurs", "1");
		timeElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(timeElement);

		Element locationElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		locationElement.setAttribute("name", "Location");
		locationElement.setAttribute("type", "xs:string");
		locationElement.setAttribute("minOccurs", "0");
		locationElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(locationElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		return CollectionUtils.getList(bookingSlotSelectedAlternative);
	}

	@Override
	public String getFreeTextAlternative() {

		return null;
	}

	@Override
	public Map<Integer, Integer> getAlternativeConversionMap() {

		return ALTERNATIVE_CONVERSION_MAP;
	}

	public String getAcceptText() {

		return acceptText;
	}

	public void setAcceptText(String acceptText) {

		this.acceptText = acceptText;
	}

	//TODO dont print all calendar info when exporting query

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		acceptText = XMLValidationUtils.validateParameter("acceptText", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		invoiceLineText = XMLValidationUtils.validateParameter("invoiceLineText", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		commentName = XMLValidationUtils.validateParameter("commentName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		minVisibleDays = XMLValidationUtils.validateParameter("minVisibleDays", xmlParser, false, 1, 255, PositiveStringIntegerPopulator.getPopulator(), errors);
		archiveStatusID = XMLValidationUtils.validateParameter("archiveStatusID", xmlParser, false, 1, 255, NonNegativeStringIntegerPopulator.getPopulator(), errors);
		bookingSlotPassedStatusID = XMLValidationUtils.validateParameter("bookingSlotPassedStatusID", xmlParser, false, 1, 255, NonNegativeStringIntegerPopulator.getPopulator(), errors);
		bookingSlotPassedStatusTime = XMLValidationUtils.validateParameter("bookingSlotPassedStatusTime", xmlParser, false, 1, 255, NonNegativeStringIntegerPopulator.getPopulator(), errors);

		showCommentField = xmlParser.getPrimitiveBoolean("showCommentField");
		showLocationInList = xmlParser.getPrimitiveBoolean("showLocationInList");

		customPriceText = XMLValidationUtils.validateParameter("customPriceText", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}

	}

	public Integer getMinVisibleDays() {

		return minVisibleDays;
	}

	public void setMinVisibleDays(Integer minVisibleDays) {

		this.minVisibleDays = minVisibleDays;
	}

	public boolean showCommentField() {

		return showCommentField;
	}

	public void setShowCommentField(boolean showCommentField) {

		this.showCommentField = showCommentField;
	}

	public boolean showLocationInList() {

		return showLocationInList;
	}

	public void setShowLocationInList(boolean showLocationInList) {

		this.showLocationInList = showLocationInList;
	}

	public boolean isShowLocationFilter() {

		return showLocationFilter;
	}

	public void setShowLocationFilter(boolean showLocationFilter) {

		this.showLocationFilter = showLocationFilter;
	}

	public String getCustomPriceText() {

		return customPriceText;
	}

	public void setCustomPriceText(String customPriceText) {

		this.customPriceText = customPriceText;
	}

	public ImmutableAlternative getBookingSlotSelectedAlternative() {

		return bookingSlotSelectedAlternative;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public void setBookingSlotSelectedAlternative(ImmutableAlternative bookingSlotSelectedAlternative) {

		this.bookingSlotSelectedAlternative = bookingSlotSelectedAlternative;
	}

	public Integer getArchiveStatusID() {

		return archiveStatusID;
	}

	public void setArchiveStatusID(Integer archiveStatusID) {

		this.archiveStatusID = archiveStatusID;
	}

	public Integer getBookingSlotPassedStatusID() {

		return bookingSlotPassedStatusID;
	}

	public void setBookingSlotPassedStatusID(Integer bookingSlotPassedStatusID) {

		this.bookingSlotPassedStatusID = bookingSlotPassedStatusID;
	}

	public Integer getBookingSlotPassedStatusTime() {

		return bookingSlotPassedStatusTime;
	}

	public void setBookingSlotPassedStatusTime(Integer bookingSlotPassedStatusTime) {

		this.bookingSlotPassedStatusTime = bookingSlotPassedStatusTime;
	}

	public boolean shouldAddBookingSlotManagersToFlowInstance() {

		return addBookingSlotManagersToFlowInstance;
	}

	public void setAddBookingSlotManagersToFlowInstance(boolean addBookingSlotManagersToFlowInstance) {

		this.addBookingSlotManagersToFlowInstance = addBookingSlotManagersToFlowInstance;
	}

	public boolean isShowFirstFreeSlot() {

		return showFirstFreeSlot;
	}

	public void setShowFirstFreeSlot(boolean showFirstFreeSlot) {

		this.showFirstFreeSlot = showFirstFreeSlot;
	}

	public String getNoFreeSlotsText() {

		return noFreeSlotsText;
	}

	public void setNoFreeSlotsText(String noFreeSlotsText) {

		this.noFreeSlotsText = noFreeSlotsText;
	}

	@Override
	public List<? extends PaymentProduct> getPaymentProducts() {

		if (calendar.getPrice() != null && calendar.getPrice() > 0) {

			return Collections.singletonList(new BasePaymentProduct(calendar.getPrice(), invoiceLineText));
		}

		return Collections.emptyList();
	}

}
