
var BookingQueryi18n = {
	OneSlot: "not set",
	MultipleSlots: "not set",
	Free: "not set",
	Booked: "not set",
	NotBookable: "not set",
	LocationTitle: "not set",
	DescriptionTitle: "not set",
	ManagersTitle: "not set",
	SlotPassed: "not set",
	Week: "not set",
	Slots: "not set",
	More: "not set",
	Choose: "not set"
};

var DATE_FORMAT = "DD MMMM YYYY";

$(function() {
	
	setQueryRequiredFunctions["BookingQueryInstance"] = makeBookingQueryRequired;
});

function toggleView($query, listView) {
	
	var $selecting = $query.find(".selecting");
	var $selected = $query.find(".selected");
	var $listView = $query.find(".list-view");
	var $calendarView = $query.find(".calendar-view");
	var $selectingHeader = $query.find(".selecting-header");
	var $selectedHeader = $query.find(".selected-header");
	
	$calendarView.toggle(!listView);
	$listView.toggle(listView);

	$listView.find("select[name$='bookingSlotID']").prop('disabled', !listView);
	$calendarView.find("input[name$='bookingSlotID']").prop('disabled', listView);
	
	var valueIsSelected = !!$query.find('*[name$="bookingSlotID"]:enabled').val();
	
	$selecting.toggle(!(listView || valueIsSelected));
	$selected.toggle(listView || valueIsSelected);
	$selected.toggleClass("list-view", listView);
	$selected.toggleClass("list-view-selecting-value", listView && !valueIsSelected);
	
	$selectingHeader.toggle(listView && !valueIsSelected);
	$selectedHeader.toggle(!(listView && !valueIsSelected));
	
	$selectedHeader.find(".selected-header-divider, .revert-choice").toggle(!(listView && valueIsSelected));

	if ($query.hasClass("enableAjaxPosting")) {
			
		runBookingEvaluators($query);
	}
}

function initBookingQuery(queryID, showFullyBookedDaysInCalendar, fullyBookedCalendarDayText, showFirstFreeSlot, noFreeSlots, calendarWeekView, availableViewMode) {
	
	var query = $("#query_" + queryID);
			
	if(query.hasClass("enableAjaxPosting")) {
		
		bindBookingChangeEvent(query);
	}
	
	if (query.find(".calendar").length > 0) {
		
		var titleEvents = $.parseJSON(query.find(".events-title-json").text());
		var detailedEvents = $.parseJSON(query.find(".events-json").text());
		initBookingQueryCalendar(query, titleEvents, detailedEvents, showFullyBookedDaysInCalendar, fullyBookedCalendarDayText, showFirstFreeSlot, noFreeSlots, calendarWeekView);
		
	} 
	
	if (query.find('.list-view').length > 0) {
	
		var dropdown = query.find(".selected select");
		
		dropdown.on("change", function(){
			
			var descriptionField = query.find(".selected .selected-description");
			
			descriptionField.empty();
			descriptionField.append(getSlotDescription(query, dropdown.val()));
			
			toggleView(query, true);
		});
	}
	
	toggleView(query, availableViewMode == "LIST");
	
	initBookingSizeSelector(query);
}

function getSlotDescription($query, id) {
	
	var $element = $query.find(".slot-description div[data-id='" + id  + "']");
	
	$element.find(":disabled").prop("disabled", false);
	
	return $element.html();
}

function bindBookingChangeEvent($query) {
	
	var $inputs = $query.find("input[name$='accept']");
	
	$inputs.on("change", function() {
		
		runBookingEvaluators($query);
	});
}

function runBookingEvaluators($query) {
	
	var queryID = $query.data("id");
	var parameters = {};
	
	var $inputWrapper = $query.find(".selected-body");
	
	$inputWrapper.find("*[name$='bookingSlotID']:enabled").each(function () {
		
		parameters[this.name] = this.value;
		
	});
	
	$inputWrapper.find("input[name$='accept']").each(function () {
		
		parameters[this.name] = this.checked;
	});
	
	runQueryEvaluators(queryID, parameters);
}

function makeBookingQueryRequired(queryID) {
	
	$("#query_" + queryID).find(".heading-wrapper h2").addClass("required");
}

// Calendar code below

function bookingQueryUpdateFilter(event, dropdown){

	var $dropdown = $(dropdown);
	var query = $dropdown.closest(".query");
	
	var managerID = query.find(".managerfilter").val();
	var location = query.find(".locationfilter").val();
	
	var url = $dropdown.data("url");
	
	url += "?m=" + managerID;
	
	url += "&l=" + encodeURIComponent(location);
	
	$.ajax({
		url: url,
		cache: false,
		
	}).done(function(data, textStatus, jqXHR){
		
		if (data.titleEvents != undefined && data.detailedEvents != undefined) {
		
			reloadBookingQueryCalendar(query, managerID, location, data.titleEvents, data.detailedEvents)
			
		} else {
			
			if (console != undefined){
				console.error("no calendar events: " + textStatus + ", " + data);
			}
			
			showErrorDialog();
		}
		
	}).fail(function(jqXHR, textStatus, errorThrown){
		
		if (console != undefined){
			console.error("failed to get calendar events: " + textStatus + ", " + errorThrown);
		}
		
		showErrorDialog();
	});
}

var bookingQueryCalendars = [];
var bookingQueryDetailedEvents = [];
var bookingQueryCalendarManager = [];
var bookingQueryCalendarLocation = [];

function reloadBookingQueryCalendar(query, managerID, location, titleEvents, detailedEvents){
	
	var id = query.attr('id');
	
	bookingQueryDetailedEvents[id] = detailedEvents;
	
	var calendar = bookingQueryCalendars[id];
	calendar.setEvents(titleEvents);
	
	var calendarElement = query.find('.calendar');
	calendarElement.find(".managerfilter").val(managerID);
	bookingQueryCalendarManager[id] = managerID;
	
	calendarElement.find(".locationfilter").val(location);
	bookingQueryCalendarLocation[id] = location;
}

function initBookingQueryCalendar(query, titleEvents, detailedEvents, showFullyBookedDaysInCalendar, fullyBookedCalendarDayText, showFirstFreeSlot, noFreeSlots, calendarWeekView){
	
	var id = query.attr('id');
	
	bookingQueryDetailedEvents[id] = detailedEvents;
	
	var calendarElement = query.find('.calendar');
	var calendarTemplate = Handlebars.compile(query.find('.calendar-template').html());
	var dayTemplate = Handlebars.compile(query.find('.calendar-day-template').html());
	
	var openDay = function (events, date) {
    	
        var data = {};

        data.events = events;
        data.day = date.format('D MMMM YYYY');
        data.date = date.format(DATE_FORMAT);
        data.showDuration = calendarElement.data("showduration");
        data.showManagers = calendarElement.data("showmanagers");

        var day = $(dayTemplate(data));
    	var config = {

    			afterContent: function(){
    				var featherlight = this;
    				var $content = featherlight.$content;
    				$content.removeClass("no-sections");
    				$content.closest(".featherlight-content").addClass("calendar-day-modal");
    				
    				$content.find(".detail-slot-active a").first().trigger("focus");
    				
    				$content.find("li.detail-slot-active a").on("click", function(e){
    					
    					selectSlot(this);
    					
    					featherlight.close();
    				});
    			},
    	};
    	
    	$.featherlight(day, config);
    }

	var adjacentMonthDay = undefined;
	
    var cal = calendarElement.clndr({
        startWithMonth: moment(),
        daysOfTheWeek: ['M<span class="hide-mobile">åndag</span>', 'T<span class="hide-mobile">isdag</span>', 'O<span class="hide-mobile">nsdag</span>', 'T<span class="hide-mobile">orsdag</span>', 'F<span class="hide-mobile">redag</span>', 'L<span class="hide-mobile">ördag</span>', 'S<span class="hide-mobile">öndag</span>'],
        dateParameter: "start",
        events: titleEvents,
        render: function(data) {
            return calendarTemplate(data);
        },
        clickEvents: {
            click: function(target) {
            	
            	if (this.options.extras.weekView) return;
            	
                var dayElement = $(target.element);
                var events = bookingQueryDetailedEvents[id][target.date.format('YYYY-MM-DD')];

                if (dayElement.hasClass('disabled') || !dayElement.hasClass('activities'))
                    return;
                
                openDay(events, target.date);
            },
            onMonthChange: function(month) {
            	
            	calendarElement.find('.calendar-month').html(month.format('MMMM YYYY').toUcFirst());
            }
        },
        doneRendering: function() {
        	
        	if(adjacentMonthDay) {
        		adjacentMonthDay.trigger("click");
        		adjacentMonthDay = undefined;
        	}
        	
        	if (this.options.extras.noFreeSlots) {
        		
        		calendarElement.find('.no-free-slots').show();
        	}
        	
        	if (this.options.extras.weekView) showWeekView(id);

            calendarElement.find(".managerfilter").val(bookingQueryCalendarManager[id]);
            calendarElement.find(".locationfilter").val(bookingQueryCalendarLocation[id]);
        },
        extras: {
        	showFullyBookedDaysInCalendar: showFullyBookedDaysInCalendar,
        	fullyBookedCalendarDayText: fullyBookedCalendarDayText,
        	noFreeSlots: noFreeSlots,
        	weekView: false,
            showDuration: calendarElement.data("showduration"),
			currentWeek: null
        }
    });
    
    bookingQueryCalendars[id] = cal;
    
    function selectSlot(clickedElement) {
    	
    	var a = $(clickedElement);
    	var selectingDiv = calendarElement.closest(".selecting");
    	var selectedDiv = selectingDiv.parent().find(".selected");
    	var queryDiv = calendarElement.closest('.query');
    	
    	var slotID = a.data("slot-id");
    	var date = moment(a.data("date")).format(DATE_FORMAT);
    	var time = a.data("formatted-time");

    	selectedDiv.find("input.selected-value").val(slotID).trigger("change");
    	selectedDiv.find(".selected-name").html(date + "<br>" + time);
    	selectedDiv.find(".selected-description").html(getSlotDescription(query, slotID));

    	toggleView(query, false);
    	
    	query.find('#view-select').hide();
    	
    	queryDiv[0] && queryDiv[0].scrollIntoView();
    }
    
    query.find(".revert-choice").on("click", function(e) {
    	e.preventDefault();
    	
    	var selectedDiv = $(this).closest(".selected");
    	
    	selectedDiv.find("input.selected-value").val('').trigger("change");
    	selectedDiv.find(".selected-name, .selected-description").text('');
    	
    	toggleView(query, false);
    	
    	query.find('#view-select').show();
	});
    
	query.find("#view-select").on("change", function(e) {
		
		var mode = this.value;
		
		toggleView(query, mode == "LIST");

		if (mode != "LIST") {
			
	   		calendarElement.toggleClass('week-view', mode == "WEEK");
	 	   
	   		cal.options.extras.weekView = mode == "WEEK";
		   
	   		cal.render();		
	   	}
	});
    
    if (calendarWeekView && calendarElement.is(':visible')) {
    	
    	query.find("#view-select").val("WEEK").trigger("change");
	}
	
	if (!noFreeSlots && showFirstFreeSlot) {
		
		var firstFreeSlot = getFirstFreeSlot();
		
		if (firstFreeSlot) {
			
			cal.setYear(firstFreeSlot.year());
			
			cal.setMonth(firstFreeSlot.month());
			
			if (calendarWeekView) {
				
				var count = 0;
				
				while (!calendarElement.find('.calendar-row:visible .day li.slot-active').length) {
					
					calendarElement.find('.next-week-button').trigger("click");
					
					if (count > 4) break;
					count++;
				}
			} 
		}
	}

	function showWeekView(queryID){
    	
    	var calendarRowSelector = ".calendar-row:not(.calendar-row-header)";
    	var	$calendarWeekRows = calendarElement.find(calendarRowSelector); 	

		var $weekRowToShow = getWeekRowToShow($calendarWeekRows);
    	
       	$calendarWeekRows.not($weekRowToShow).hide();

		cal.options.extras.currentWeek = $weekRowToShow.find(".calendar-week").data("week");
    	
       	calendarElement.find('a.btn').toggleClass('hidden');

       	calendarElement.find('.calendar-month').wrapInner('<div class="margintop font-weight-normal"/>').prepend('<div class="week">');
    	
    	function renderDetails() {
    		
    		calendarElement.find('.calendar-month > .week').html(' ' + BookingQueryi18n.Week + ' ' + calendarElement.find('.calendar-row:visible .calendar-week').html());
    	}
    	
    	renderDetails();

    	calendarElement.find('.previous-week-button').on("click", function(e){
    		
    		e.preventDefault();

    		var $currentWeekRow = calendarElement.find(calendarRowSelector).filter(':visible');
    		var	$previousWeekRow = $currentWeekRow.prev(calendarRowSelector);
    		
    		if (!$previousWeekRow.length) {
    			
    			cal.previous();

    			$previousWeekRow = calendarElement.find(calendarRowSelector).last();
    			
    			if ($previousWeekRow.find('.calendar-week').text() === $currentWeekRow.find('.calendar-week').text()) {
    				
    				$previousWeekRow = $previousWeekRow.prev();
    			}
    		}
    		
    		calendarElement.find(calendarRowSelector).not($previousWeekRow).hide();
    		
    		$previousWeekRow.show();

			cal.options.extras.currentWeek = $previousWeekRow.find(".calendar-week").data("week");
    		
    		renderDetails();
    	});     
    	
    	calendarElement.find('.next-week-button').on("click", function(e){
    		
    		e.preventDefault();
    		
    		var $currentWeekRow = calendarElement.find(calendarRowSelector).filter(':visible');
    		var	$nextWeekRow = $currentWeekRow.next(calendarRowSelector);
    		
    		if (!$nextWeekRow.length) {
    			
    			cal.next();

    			$nextWeekRow = calendarElement.find(calendarRowSelector).first();
    			
    			if ($nextWeekRow.find('.calendar-week').text() === $currentWeekRow.find('.calendar-week').text()) {
    				
    				$nextWeekRow = $nextWeekRow.next();
    			}
    		}
    		
    		calendarElement.find(calendarRowSelector).not($nextWeekRow).hide();
    		
    		$nextWeekRow.show();

			cal.options.extras.currentWeek = $nextWeekRow.find(".calendar-week").data("week");
    		
    		renderDetails();
		});     
    	
    	$calendarWeekRows.find("a.day li").on("click", function(){
    		
            var dayElement = $(this).closest("a.day"),
            	date = moment(dayElement.data("date")),
            	events = bookingQueryDetailedEvents[id][date.format('YYYY-MM-DD')],
            	slotID = this.dataset.slotId;

            if (dayElement.hasClass('disabled') || !dayElement.hasClass('activities'))
            	return;
            
            var filteredEvents = events.filter(function(event){
            	
            	return slotID.split(",").indexOf(event.id.toString()) != -1; 
            });
        
            openDay(filteredEvents, date);
    	});
    	
    }
	
	function getFirstFreeSlot() {
		
		for (var i = 0; i < titleEvents.length; i++) {

			var event = titleEvents[i];
			
			if (event.freeSlots > 0 && !event.passed && event.bookable) {
				
				return moment(event.start, 'YYYY-MM-DD HH:mm');
			}
		}
		
		return null;
	}
	
	function getWeekRowToShow($calendarWeekRows) {
		
		if (cal.options.extras.currentWeek) {
			
			var $currentWeekRow = $calendarWeekRows.has(".calendar-week[data-week='" + cal.options.extras.currentWeek + "']")
			
			if ($currentWeekRow.length) {
				
				return $currentWeekRow;
			}
		}
		
		var	$rowWithToday = $calendarWeekRows.has('.today');

		if ($rowWithToday.length) {
			
	       	return $rowWithToday;
			
		}	

       	return $calendarWeekRows.first();
	}
	
}

Handlebars.registerHelper('BookingQuery-Week', function(items, options) {

    var x = 0;
    var out = '<div class="calendar-row">';

    for (var i = 0, il = items.length; i < il; i++) {

        var data = items[i];
		var weekNumber = new Date(data.date._d).getWeekNumber();

        if (x === 0) {
            if (i !== 0) {
                out += '<div class="calendar-row">';
            }
            out += '<div><span class="calendar-week" data-week="' + weekNumber + '">' + weekNumber + '</<span></div>';
        }
        
        var eventsWithFreeSlots = [];
    	
    	for (var y = 0; y < data.events.length; y++) {
    		
    		var event = data.events[y];
    		
    		if (event.freeSlots > 0) {
    			
    			eventsWithFreeSlots.push(event);
    		} 
    	}

        if (eventsWithFreeSlots.length > 0) {
         
        	data.classes += ' activities';
        }

        out += '<a data-date="' + data.date.format('YYYY-MM-DD') + '" class="' + data.classes + '" href="#" onclick="return false;"><div class="day-of-month">' + data.day + '</div>';
        
        if (eventsWithFreeSlots.length > 0) {
        	
            out += BookingQueryGetEventHtml(eventsWithFreeSlots, options);
            
        } else if (this.extras.showFullyBookedDaysInCalendar && data.events.length > 0) {
        	
        	out += '<div class="day-fully-booked">' + this.extras.fullyBookedCalendarDayText + '</div>';
        }

        out += '</a>';

        if (x === 6) {
            out += '</div>';
            x = 0;
        } else {
            x++;
        }
    }

    return out + '</div>';
});

function BookingQueryGetEventHtml(events, options) {

    var response = '<ul>';

	if (!options.data.root.extras.weekView) {
		
		events = groupEventsByTime(events);
	}
	
    var eventCountThreshold = options.data.root.extras.weekView ? 8 : 5;
    var eventsToShow = Math.min(events.length, events.length > eventCountThreshold ? eventCountThreshold - 1 : eventCountThreshold);
    
    for (var i = 0; i < eventsToShow; i++) {

    	var event = events[i];
    	
        response += '<li data-slot-id="' + event.id + '"';
        
        if (event.passed) {
        	
        	response += ' class="passed"';
        	
        } else if (!event.bookable) {
        	
        	response += ' class="not-bookable"';
        	
        } else if (event.freeSlots <= 0) {
        	
    		response += ' class="slot-inactive"';
    		
        } else {
    		
    		response += ' class="slot-active"';
        }
        
        response += '>';
	        response += '<span>' + event.startTime + ' - </span>';
	        response += '<span class="month-mode-hide-mobile">';
    		response += options.data.root.extras.showDuration ? event.duration + 'min' : event.endTime;
	        response += '</span>';
	        response += '<span class="hide-tablet floatright" data-icon-before="u">';
	        
	        if(event.showFreeSlotsAsBoolean) {
	        
	        	 response += (event.freeSlots > 0 ? BookingQueryi18n.Free : BookingQueryi18n.Booked)  + '</span>';
	        
	        } else {
	        
	        	response += (event.freeSlots > 0 ? event.freeSlots : '')  + '</span>';
	        }
	        
	        if (event.location) {
	        	response += '<div class="hide-month hide-mobile font-weight-normal">' + Handlebars.escapeExpression(event.location)  + '</div>';
	        }

	    response += '</li>';
    }
    
    if (events.length > eventCountThreshold) {
    	
    	var slotIDs = "";
    	for (var y = eventsToShow; y < events.length; y++) {
    		
    		slotIDs += events[y].id;
    		
    		if (events[y + 1]) {
    			
    			slotIDs += ",";
    		}
    	}
    	
    	response += '<li class="moreevents" data-slot-id="' + slotIDs +'">';
	    	response += '<span>' + (events.length - (eventCountThreshold - 1)) + ' <span class="hide-mobile"> ' + BookingQueryi18n.Slots +  '</span> ' + BookingQueryi18n.More + '</span>';
	    response += '</li>';
    }

    return response + '</ul>';
}

function groupEventsByTime(events) {
	
	return $.extend(true, [], events).reduce(function(acc, cur, idx) {
		
	    var lastAdded = acc[acc.length - 1];

	    if (lastAdded && lastAdded["start"] == cur["start"] && lastAdded["end"] == cur["end"]) {
		
	        lastAdded["freeSlots"] += cur["freeSlots"];

	    } else {
		
	        acc.push(cur);
	    }

	    return acc;

	}, []);
}

// Modal view
Handlebars.registerHelper('BookingQuery-Day', function(events, options) {

    var out = '<ul class="event-list list-style-type-none">';

    if (events != undefined) {
    	
    	var eventsWithFreeSlots = [];
        
        for (var i = 0; i < events.length; i++) {
        	
        	var event = events[i];
        	
        	if (event.freeSlots > 0) {
        		
        		eventsWithFreeSlots.push(event);
        	} 
        }
        
        if (eventsWithFreeSlots.length > 0) {
        	
            for (var i = 0, il = eventsWithFreeSlots.length; i < il; i++) {
            	
            	var event = eventsWithFreeSlots[i];
            	
            	out += '<li class="';
            	var bookable = false;
            	
            	if (event.passed) {
            		
            		out += ' passed';
            		
            	} else if (!event.bookable) {

            		out += ' not-bookable';
            		
            	} else if (event.freeSlots <= 0) {

            		out += ' detail-slot-inactive';
            		
        		} else {
        			
        			out += ' detail-slot-active';
        			bookable = true;
        		}
            	
            	out += '">';

				var eventTime = event.startTime + ' - ' + (options.data.root.showDuration ? event.duration + 'min' : event.endTime); 
            	
            	out += '<a href="#" onclick="return false;" data-slot-id="' + event.id + '" data-date="' + event.startDate + '" data-formatted-time="' + eventTime + '"';
            	
            	if (!bookable) {
            		
            		out += ' tabindex="-1"';
            	}

            	out += '>';
            	
            	out += '<div class="event-info">';
            	
            	out += '<span class="event-time">';
            	
				out += eventTime;
            	
            	out += '</span>';

            	out += '<span class="event-time-info bigmarginleft">';
            	
	        	if (event.passed) {

	        		out += BookingQueryi18n.SlotPassed;
            		
	        	} else if (!event.bookable) {

	        		out += '<span>' + event.notBookableText + '</span>';
	        		
            	} else if (event.showFreeSlotsAsBoolean) {
            			
        			out += (event.freeSlots > 0 ? BookingQueryi18n.Free : BookingQueryi18n.Booked);
        			
        		} else {
        			
        			out += event.freeSlots + ' ' + (event.freeSlots == 1 ? BookingQueryi18n.OneSlot : BookingQueryi18n.MultipleSlots);
        		}
	        	
            	out += '</span>';
            	
            	if (event.description != undefined || event.location != undefined || (options.data.root.showManagers && event.managers != undefined)) {
            		
            		out += '<p class="meta-line">';
            		
            		if (options.data.root.showManagers && event.managers != undefined) {
            			
            			out += "<p><strong>" + BookingQueryi18n.ManagersTitle + "</strong>";
        				out += '<br>';
            			
            			$.each(event.managers, function(index, manager) {
            				
            				if (index > 0) {
            					out += ', ';
            				}
            				
            				out += Handlebars.escapeExpression(manager.name);
            			});
            			
            			out += '</p>';
            		}
            		
            		if (event.location != undefined) {
            			
            			out += "<p><strong>" + BookingQueryi18n.LocationTitle + ": </strong>";
            			out += Handlebars.escapeExpression(event.location) + "</p>";
            		}
            		
            		if (event.description != undefined) {
            			
            			out += "<p><strong>" + BookingQueryi18n.DescriptionTitle + "</strong>";
            			
            			var descriptionRows = Handlebars.escapeExpression(event.description).split('\r');
            			
            			$.each(descriptionRows, function(index, text) {
            				out += '<br>' + text;
            			});
            			
            			out += '</p>';
            		}
            		
            		out += '</p>';
            	}
            	
            	out += '</div>';
            	
            	if (bookable) {
            		out += '<span class="choose-event">' + BookingQueryi18n.Choose + '</span>';
            	}
            	
            	out += '</a>';
            	out += '</li>';
            }
            
        }    	
    }

    return out + '</ul>';
});

Handlebars.registerHelper('bookingQuerySafe_string', function(context) {
    var html = context;
    // context variable is the HTML you will pass into the helper
    // Strip the script tags from the html, and return it as a Handlebars.SafeString
    return new Handlebars.SafeString(html);
});

Date.prototype.getWeekNumber = function() {
	return moment.unix(this.getTime() / 1000).format('W');
}

String.prototype.toUcFirst = function() {
    return this.charAt(0).toUpperCase() + this.slice(1);
}
