package com.nordicpeak.flowengine.evaluators.childquery.childcount;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "child_count_weight_evaluators")
@XMLElement
public class ChildCountWeightEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = 2722817930085002654L;

	@DAOManaged
	@WebPopulate(required = true, minLength = 0)
	@XMLElement
	private Integer fromValue;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer toValue;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer weight;

	public Integer getFromValue() {

		return fromValue;
	}

	public void setFromValue(Integer fromValue) {

		this.fromValue = fromValue;
	}

	public Integer getToValue() {

		return toValue;
	}

	public void setToValue(Integer toValue) {

		this.toValue = toValue;
	}

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		fromValue = XMLValidationUtils.validateParameter("fromValue", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		toValue = XMLValidationUtils.validateParameter("toValue", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		
		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

}
