package com.nordicpeak.flowengine.evaluators.childquery.duplicates;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.BooleanQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.childquery.beans.ChildQuery;
import com.nordicpeak.flowengine.queries.childquery.beans.ChildQueryInstance;

public class ChildDuplicateFlownstanceWeightEvaluationProviderModule extends BaseEvaluationProviderModule<ChildDuplicateFlownstanceWeightEvaluator> {

	protected ChildDuplicateFlownstanceWeightEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<ChildDuplicateFlownstanceWeightEvaluator> evaluatorDAO;

	protected QueryParameterFactory<ChildDuplicateFlownstanceWeightEvaluator, Integer> evaluatorIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ChildDuplicateFlownstanceWeightEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(ChildDuplicateFlownstanceWeightEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new ChildDuplicateFlownstanceWeightEvaluatorCRUD(ChildDuplicateFlownstanceWeightEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, ChildDuplicateFlownstanceWeightEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (!(queryInstance instanceof ChildQueryInstance)) {

			log.warn("Query instance " + queryInstance + " is not of the type " + ChildQueryInstance.class.getName());

			return null;
		}

		if(user == null || callback.getFlowInstance().getFirstSubmitted() != null){
			
			return null;
		}
		
		BigDecimal weight = BigDecimal.ZERO;

		String citizenIdentifier = ((ChildQueryInstance) queryInstance).getCitizenIdentifier();

		if (citizenIdentifier != null) {
			
			try{
				log.info("Checking for duplicate flow instances in family of flow " + callback.getFlowInstance().getFlow() + " for child " + citizenIdentifier + " and poster " + user);
			
				if(hasDuplicates(user, citizenIdentifier, callback.getFlowInstance())){
					
					weight = BigDecimal.valueOf(evaluator.getWeight());
				}
					
			} catch (Exception e) {

				log.error("Error getting duplicates for citizen identifier " + citizenIdentifier + " for flow instance " + callback.getFlowInstance(), e);
			}
		}

		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), weight, attributeHandler), true);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}

		return null;
	}

	private boolean hasDuplicates(User user, String citizenIdentifier, ImmutableFlowInstance flowInstance) throws SQLException {

		BooleanQuery query;
		
		if(flowInstance.getFlowInstanceID() == null){
			
			query = new se.unlogic.standardutils.dao.querys.BooleanQuery(dataSource, "SELECT flowengine_flow_instances.flowInstanceID " + 
					"FROM flowengine_flow_instances " + 
					"INNER JOIN flowengine_flows ON flowengine_flows.flowID = flowengine_flow_instances.flowID " + 
					"INNER JOIN flowengine_flow_instance_attributes ON flowengine_flow_instance_attributes.flowInstanceID = flowengine_flow_instances.flowInstanceID " + 
					"WHERE " + 
					"flowengine_flow_instances.firstSubmitted IS NULL " + 
					"AND flowengine_flow_instances.poster = ? " + 
					"AND flowengine_flows.flowFamilyID = ? " + 
					"AND flowengine_flow_instance_attributes.name = ? " + 
					"AND flowengine_flow_instance_attributes.value = ? " +
					"LIMIT 1");
		}else{
			
			query = new se.unlogic.standardutils.dao.querys.BooleanQuery(dataSource, "SELECT flowengine_flow_instances.flowInstanceID " + 
					"FROM flowengine_flow_instances " + 
					"INNER JOIN flowengine_flows ON flowengine_flows.flowID = flowengine_flow_instances.flowID " + 
					"INNER JOIN flowengine_flow_instance_attributes ON flowengine_flow_instance_attributes.flowInstanceID = flowengine_flow_instances.flowInstanceID " + 
					"WHERE " + 
					"flowengine_flow_instances.firstSubmitted IS NULL " +
					"AND flowengine_flow_instances.poster = ? " + 
					"AND flowengine_flows.flowFamilyID = ? " + 
					"AND flowengine_flow_instance_attributes.name = ? " + 
					"AND flowengine_flow_instance_attributes.value = ? " +
					"AND flowengine_flow_instances.flowInstanceID != ? " + 
					"LIMIT 1");
			
			query.setInt(5, flowInstance.getFlowInstanceID());
		}
		
		query.setInt(1, user.getUserID());
		query.setInt(2, flowInstance.getFlow().getFlowFamily().getFlowFamilyID());
		query.setString(3, "childCitizenIdentifier");
		query.setString(4, citizenIdentifier);
		
		return query.executeQuery();
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ChildDuplicateFlownstanceWeightEvaluator evaluator = new ChildDuplicateFlownstanceWeightEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return ChildQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		ChildDuplicateFlownstanceWeightEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<ChildDuplicateFlownstanceWeightEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	protected Class<ChildDuplicateFlownstanceWeightEvaluator> getEvaluatorClass() {

		return ChildDuplicateFlownstanceWeightEvaluator.class;
	}

	protected AnnotatedDAO<ChildDuplicateFlownstanceWeightEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<ChildDuplicateFlownstanceWeightEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		ChildDuplicateFlownstanceWeightEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<ChildDuplicateFlownstanceWeightEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<ChildDuplicateFlownstanceWeightEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	@Override
	public ChildDuplicateFlownstanceWeightEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		ChildDuplicateFlownstanceWeightEvaluator evaluator = getEvaluatorClass().newInstance();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}
}
