package com.nordicpeak.flowengine.evaluators.childquery.duplicates;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "child_duplicate_flow_instances_weight_evaluators")
@XMLElement
public class ChildDuplicateFlownstanceWeightEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -336493853396860255L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer weight;
	
	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}
}
