<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/colorpickerqueryadmin.js
	</xsl:variable>

	<xsl:variable name="rawPreferedFormatOptions">
		<option>
			<name>HEX: #RRGGBB</name>
			<value>HEX</value>
		</option>
		<option>
			<name>HEX8: #AARRGGBB</name>
			<value>HEX8</value>
		</option>
		<option>
			<name>RGB: rgb(0, 123, 255) / rgba(0, 123, 255, 0.5)</name>
			<value>RGB</value>
		</option>
		<option>
			<name>HSL: hsl(123, 45%, 67%) / hsla(123, 45%, 67%, 0.5)</name>
			<value>HSL</value>
		</option>
		<option>
			<name>HSV: hsv(123, 45%, 67%) / hsva(123, 45%, 67%, 0.5)</name>
			<value>HSV</value>
		</option>
	</xsl:variable>

	<xsl:variable name="preferedFormatOptions" select="exsl:node-set($rawPreferedFormatOptions)/option" />

	<xsl:template match="Document">

		<div id="ColorPickerQueryProvider" class="contentitem">

			<xsl:apply-templates select="validationError" />
			<xsl:apply-templates select="UpdateColorPickerQuery" />

		</div>

	</xsl:template>

	<xsl:template match="UpdateColorPickerQuery">

		<h1>
			<xsl:value-of select="$i18n.UpdateQuery" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="ColorPickerQuery/QueryDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form id="updateColorPickerQueryForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="ColorPickerQuery" />
			</xsl:call-template>

			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'useAlpha'" />
						<xsl:with-param name="name" select="'useAlpha'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="ColorPickerQuery" />
					</xsl:call-template>
					
					<label for="useAlpha">
						<xsl:value-of select="$i18n.UseAlpha" />
					</label>
				</div>

			</div>
			
			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'usePalette'" />
						<xsl:with-param name="name" select="'usePalette'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="ColorPickerQuery" />
					</xsl:call-template>
					
					<label for="usePalette">
						<xsl:value-of select="$i18n.UsePalette" />
					</label>
				</div>

			</div>
			
			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'useComparison'" />
						<xsl:with-param name="name" select="'useComparison'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="ColorPickerQuery" />
					</xsl:call-template>
					
					<label for="useComparison">
						<xsl:value-of select="$i18n.UseComparison" />
					</label>
				</div>

			</div>
			
			<div class="floatleft full bigmarginbottom">
				
				<label class="floatleft clearboth" for="preferedFormat">
					<xsl:value-of select="$i18n.PreferedFormat" />
				</label>
				
				<div class="clearboth floatleft marginbottom"><xsl:value-of select="$i18n.PreferedFormatDescription" /></div>
				
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="width" select="'99%'"/>
						<xsl:with-param name="id" select="'preferedFormat'" />
						<xsl:with-param name="name" select="'preferedFormat'" />
						<xsl:with-param name="element" select="$preferedFormatOptions" />
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="valueElementName" select="'value'" />
						<xsl:with-param name="selectedValue" select="ColorPickerQuery/ColorPickerFormat/Value" />
					</xsl:call-template>
				</div>
			</div>

			<div class="floatleft full bigmargintop">
				<h2><xsl:value-of select="$i18n.AdvancedSettings" /></h2>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'setAsAttribute'" />
						<xsl:with-param name="name" select="'setAsAttribute'" />
						<xsl:with-param name="element" select="ColorPickerQuery" /> 
						<xsl:with-param name="class" select="'vertical-align-middle'" />
					</xsl:call-template>
						
					<label for="setAsAttribute">
						<xsl:value-of select="$i18n.setAsAttribute" />
					</label>
			    </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="attributeName" class="floatleft clearboth"><xsl:value-of select="$i18n.attributeName" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'attributeName'"/>
						<xsl:with-param name="name" select="'attributeName'"/>
						<xsl:with-param name="title" select="$i18n.attributeName"/>
						<xsl:with-param name="element" select="ColorPickerQuery" />
						<xsl:with-param name="maxlength" select="'255'"/>
					</xsl:call-template>
			    </div>
			</div>	

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'UpdateFailedColorPickerQueryNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.ColorPickerQueryNotFound" />
		</p>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'AlphaNotSupportedWithHEX']">

		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.AlphaNotSupportedWithHEX" />
		</p>

	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'preferedFormat'">
				<xsl:value-of select="$i18n.PreferedFormat" />
			</xsl:when>
			<xsl:when test="$fieldName = 'attributeName'">
				<xsl:value-of select="$i18n.attributeName" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>