package com.nordicpeak.flowengine.queries.colorpickerquery.populator;

import java.util.regex.Pattern;

import se.unlogic.standardutils.populators.BaseStringPopulator;
import se.unlogic.standardutils.string.StringUtils;

public class ColorPickerPopulator extends BaseStringPopulator<String> {
	
	private static final Pattern pattern = Pattern.compile("(?im)^(?:#[\\da-f]{6}|rgb\\(\\d{1,3},[\t ]?\\d{1,3},[\t ]?\\d{1,3}\\)|(?:hsl\\(|hsv\\()\\d{1,3},[\t ]?(?:100|[1-9]?[0-9]){1,3}%,[\t ]?(?:100|[1-9]?[0-9]){1,3}%\\))$");
	private static final Pattern patternAlpha = Pattern.compile("(?im)^(?:#[\\da-f]{8}|rgba\\(\\d{1,3},[\t ]?\\d{1,3},[\t ]?\\d{1,3},[\t ]?(?:\\.\\d{1,3}|[01](?:\\.\\d{1,3})?)\\)|(?:hsla\\(|hsva\\()\\d{1,3},[\t ]?(?:100|[1-9]?[0-9]){1,3}%,[\t ]?(?:100|[1-9]?[0-9]){1,3}%,[\t ]?(?:[,.]\\d{1,3}|[01](?:[,.]\\d{1,3})?)\\))$");
	
	private boolean useAlpha;
	
	public ColorPickerPopulator(boolean useAlpha) {
		super();
		
		this.useAlpha = useAlpha;
	}

	@Override
	public String getValue(String value) {
		
		if (StringUtils.isEmpty(value)) {
			
			return null;
		}
		
		return value.trim();
	}
	
	@Override
	public Class<? extends String> getType() {
		return String.class;
	}
	
	@Override
	protected boolean validateDefaultFormat(String value) {
		
		if (StringUtils.isEmpty(value)) {
			return false;
		}
		
		value = value.trim();
		
		boolean valid = pattern.matcher(value).matches();
		
		if (!valid && useAlpha) {

			valid = patternAlpha.matcher(value).matches();
		}
		
		return valid;
	}
	
}
