package com.nordicpeak.flowengine.coloredtag.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

@Table(name = "flowengine_colored_tags")
@XMLElement(name = "Tag")
public class ColoredTag extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = 3268646669132872847L;

	public static final Field FIELD_TAG_ID = ReflectionUtils.getField(ColoredTag.class, "tagID");
	public static final Field FIELD_SETTINGS = ReflectionUtils.getField(ColoredTag.class, "settings");
	public static final Field FIELD_MAPPINGS = ReflectionUtils.getField(ColoredTag.class, "mappings");

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer tagID;

	@DAOManaged(columnName = "settingsID")
	@ManyToOne
	@XMLElement(fixCase = true)
	private ColoredTagSettings settings;

	@DAOManaged
	@XMLElement
	@WebPopulate
	@OrderBy
	private String name;

	@DAOManaged
	@XMLElement
	@WebPopulate
	private String hexColor;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<ColoredTagMapping> mappings;

	@Override
	public String toString() {

		return name + " (tagID=" + tagID + ")";
	}

	public JsonNode toJson() {

		JsonObject json = new JsonObject();

		json.putField("tagID", tagID);
		json.putField("name", name);
		json.putField("hexColor", hexColor);

		return json;
	}

	public Integer getTagID() {

		return tagID;
	}

	public void setTagID(Integer tagID) {

		this.tagID = tagID;
	}

	public ColoredTagSettings getSettings() {

		return settings;
	}

	public void setSettings(ColoredTagSettings settings) {

		this.settings = settings;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getHexColor() {

		return hexColor;
	}

	public void setHexColor(String hexColor) {

		this.hexColor = hexColor;
	}

	public List<ColoredTagMapping> getMappings() {

		return mappings;
	}

	public void setMappings(List<ColoredTagMapping> mappings) {

		this.mappings = mappings;
	}

	public boolean hasMapping(Integer flowInstanceID) {

		if (mappings != null) {

			for (ColoredTagMapping mapping : mappings) {

				if (mapping.getFlowInstanceID().equals(flowInstanceID)) {

					return true;
				}
			}
		}

		return false;
	}

}
