package com.nordicpeak.flowengine.queries.constructionpermit;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.constructionpermit.beans.ConstructionPermitQuery;
import com.nordicpeak.flowengine.queries.constructionpermit.beans.ConstructionPermitQueryInstance;
import com.nordicpeak.flowengine.queries.constructionpermit.beans.Model;
import com.nordicpeak.flowengine.queries.constructionpermit.beans.PermitNode;
import com.nordicpeak.flowengine.queries.constructionpermit.beans.PermitNodeValue;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class ConstructionPermitQueryProviderModule extends BaseQueryProviderModule<ConstructionPermitQueryInstance> implements BaseQueryCRUDCallback {

	private static final RelationQuery SAVE_QUERY_INSTANCE_RELATIONS = new RelationQuery(ConstructionPermitQueryInstance.VALUES_RELATION); 
	
	private AnnotatedDAO<ConstructionPermitQuery> queryDAO;
	private AnnotatedDAO<ConstructionPermitQueryInstance> queryInstanceDAO;
	private AnnotatedDAO<Model> modelDAO;

	private ConstructionPermitQueryCRUD queryCRUD;

	private QueryParameterFactory<ConstructionPermitQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<ConstructionPermitQueryInstance, Integer> queryInstanceIDParamFactory;

	private QueryParameterFactory<Model, String> modelIDParamFactory;
	
	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ConstructionPermitQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(ConstructionPermitQuery.class);
		queryInstanceDAO = daoFactory.getDAO(ConstructionPermitQueryInstance.class);
		modelDAO = daoFactory.getDAO(Model.class);

		AnnotatedDAOWrapper<ConstructionPermitQuery, Integer> queryDAOWrapper = queryDAO.getWrapper(Integer.class);
		
		queryCRUD = new ConstructionPermitQueryCRUD(queryDAOWrapper, new AnnotatedRequestPopulator<>(ConstructionPermitQuery.class), "ConstructionPermitQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		
		modelIDParamFactory = modelDAO.getParamFactory("modelID", String.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		ConstructionPermitQuery query = new ConstructionPermitQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		ConstructionPermitQuery query = new ConstructionPermitQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		if(query.getImportModelID() != null){
			
			query.setModel(getModel(query.getImportModelID()));
		}
		
		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	public Model getModel(String modelID) throws SQLException {

		HighLevelQuery<Model> query = new HighLevelQuery<>(modelIDParamFactory.getParameter(modelID));
		
		return modelDAO.get(query);
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		ConstructionPermitQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ConstructionPermitQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		ConstructionPermitQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new ConstructionPermitQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private ConstructionPermitQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<ConstructionPermitQuery> query = new HighLevelQuery<>(ConstructionPermitQuery.MODEL_RELATION, Model.ROOT_NODES_RELATION, PermitNode.CHILD_NODES_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private ConstructionPermitQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<ConstructionPermitQuery> query = new HighLevelQuery<>(ConstructionPermitQuery.MODEL_RELATION, Model.ROOT_NODES_RELATION, PermitNode.CHILD_NODES_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private ConstructionPermitQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<ConstructionPermitQueryInstance> query = new HighLevelQuery<>(ConstructionPermitQueryInstance.QUERY_RELATION, ConstructionPermitQuery.MODEL_RELATION, Model.ROOT_NODES_RELATION, PermitNode.CHILD_NODES_RELATION, ConstructionPermitQueryInstance.VALUES_RELATION, PermitNodeValue.PERMIT_NODE_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(ConstructionPermitQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATIONS);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATIONS);
		}
	}

	@Override
	public void populate(ConstructionPermitQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		if(queryInstance.getQuery().getModel() == null){

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			return;
		}
		
		Integer queryID = queryInstance.getQuery().getQueryID();

		List<Integer> permitIDs = NumberUtils.toInt(req.getParameterValues("q" + queryID + "_permitid[]"));
		
		if(permitIDs == null){
			
			if(!allowPartialPopulation){
				
				throw new ValidationException(new ValidationError("NoPermitsSelected"));
				
			}else{
				
				queryInstance.reset(attributeHandler);
				queryInstance.getQueryInstanceDescriptor().setPopulated(false);
				
				return;
			}
		}

		List<PermitNode> permitNodes = getPermitNodes(permitIDs, queryInstance);
		
		if(permitNodes == null && !allowPartialPopulation){
			
			queryInstance.reset(attributeHandler);
			
			throw new ValidationException(new ValidationError("SelectedPermitsNotFound"));
		}
		
		List<ValidationError> validationErrors = new ArrayList<>();
		
		List<PermitNodeValue> permitNodeValues = new ArrayList<>(permitNodes.size());
		
		for(PermitNode permitNode : permitNodes){
			
			PermitNodeValue nodeValue = new PermitNodeValue();
			
			if(permitNode.requiresCount()){
				
				Integer count = ValidationUtils.validateParameter("q" + queryID + "_node_" + permitNode.getPermitNodeID() + "_count", req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
				
				nodeValue.setCount(count);
			}
			
			if(permitNode.requiresDescription()){
				
				String description = ValidationUtils.validateParameter("q" + queryID + "_node_" + permitNode.getPermitNodeID() + "_description", req, true, 1, 255, validationErrors);
				
				nodeValue.setDescription(description);
			}
			
			if(permitNode.requiresFromToDescription()){
				
				String fromDescription = ValidationUtils.validateParameter("q" + queryID + "_node_" + permitNode.getPermitNodeID() + "_from", req, true, 1, 255, validationErrors);
				String toDescription = ValidationUtils.validateParameter("q" + queryID + "_node_" + permitNode.getPermitNodeID() + "_to", req, true, 1, 255, validationErrors);
			
				nodeValue.setFromDescription(fromDescription);
				nodeValue.setToDescription(toDescription);
			}
			
			nodeValue.setPermitNode(permitNode);
			
			permitNodeValues.add(nodeValue);
		}
		
		queryInstance.setValues(permitNodeValues);
		queryInstance.getQueryInstanceDescriptor().setPopulated(true);
		queryInstance.setAttribute(attributeHandler);
		
		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}
	}

	private List<PermitNode> getPermitNodes(List<Integer> permitIDs, ConstructionPermitQueryInstance queryInstance) {

		List<PermitNode> permitNodes = queryInstance.getQuery().getAlternatives();
		
		if(permitNodes != null){
			
			Iterator<PermitNode> iterator = permitNodes.iterator();
			
			while(iterator.hasNext()){
				
				PermitNode permitNode = iterator.next();
				
				if(!permitIDs.contains(permitNode.getPermitNodeID())){
					
					iterator.remove();
				}
			}
			
			if(!permitNodes.isEmpty()){
				
				return permitNodes;
			}
		}
		
		return null;
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		ConstructionPermitQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ConstructionPermitQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		ConstructionPermitQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, ConstructionPermitQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<ConstructionPermitQueryInstance> getQueryInstanceClass() {

		return ConstructionPermitQueryInstance.class;
	}

	public List<Model> getModels() throws SQLException {

		return modelDAO.getAll();
	}
}
