package com.nordicpeak.flowengine.culling;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.cron4jutils.CronStringValidator;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.UserMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.SimpleViewFragment;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.enums.ResponseType;
import se.unlogic.hierarchy.core.enums.SystemStatus;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.HTMLContentFilter;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.listeners.ServerStartupListener;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryOperators;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.QueryResultsStreamer;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.datatypes.SimpleEntry;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.DBUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.enums.Order;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.QueryInstanceDescriptor;
import com.nordicpeak.flowengine.beans.Status;
import com.nordicpeak.flowengine.culling.beans.AutomaticCullingMessage;
import com.nordicpeak.flowengine.culling.beans.AutomaticCullingSetting;
import com.nordicpeak.flowengine.culling.beans.CullingComment;
import com.nordicpeak.flowengine.culling.beans.DeletedFlowInstance;
import com.nordicpeak.flowengine.culling.enums.CullingCategory;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.events.DeletedByManagerEvent;
import com.nordicpeak.flowengine.events.DeletedByOwnerEvent;
import com.nordicpeak.flowengine.interfaces.FlowAdminFragmentExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.FlowSubmitSurveyProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StatisticsExtensionProvider;
import com.nordicpeak.flowengine.savedmessagehandler.SavedFlowInstanceMessageHandler;
import com.nordicpeak.flowengine.savedmessagehandler.SavedFlowInstanceMessageProvider;
import com.nordicpeak.flowengine.statistics.FlowFamilyStatistics;
import com.nordicpeak.flowengine.statistics.IntegerEntry;
import com.nordicpeak.flowengine.statistics.StatisticsModule;
import com.nordicpeak.flowengine.statistics.beans.FlowInstanceStatistic;
import com.nordicpeak.flowengine.statistics.interfaces.StatisticsAPIExtensionProvider;
import com.nordicpeak.flowengine.statistics.interfaces.StatisticsExtensionConsumer;

import it.sauronsoftware.cron4j.Scheduler;

public class CullingModule extends AnnotatedForegroundModule implements StatisticsExtensionProvider, StatisticsAPIExtensionProvider, FlowAdminFragmentExtensionViewProvider, ViewFragmentModule<ForegroundModuleDescriptor>, ServerStartupListener, Runnable, AccessInterface, SavedFlowInstanceMessageProvider {

	private static final ContentType[] ALL_CULLING_CONTENT_TYPES = {ContentType.NEW, ContentType.WAITING_FOR_MULTISIGN, ContentType.WAITING_FOR_PAYMENT, ContentType.ARCHIVED};
	private static final ContentType[] UNSUBMITTED_CULLING_CONTENT_TYPES = {ContentType.NEW, ContentType.WAITING_FOR_MULTISIGN, ContentType.WAITING_FOR_PAYMENT};

	private static final RelationQuery COMMENT_RELATION_QUERY = new RelationQuery(DeletedFlowInstance.COMMENT);
	
	//@formatter:off
	private static final String FLOW_INSTANCE_COUNT_SQL = "SELECT COUNT(fi.flowInstanceID) FROM flowengine_flow_instances fi"
											+ " INNER JOIN flowengine_flows f ON f.flowID = fi.flowID AND f.flowFamilyID = ?";
	
	private static final String MANUAL_CULLING_STATUSES_SQL = "SELECT fs.* FROM flowengine_flow_statuses fs"
											+ " INNER JOIN flowengine_flows f ON f.flowID = fs.flowID AND f.flowFamilyID = ?";
	
	private static final String AUTOMATIC_CULLING_STATUSES_SQL = "SELECT fs.* FROM flowengine_flow_statuses fs"
											+ " INNER JOIN flowengine_flows f ON f.flowID = fs.flowID AND f.flowFamilyID = ?"
											+ " AND fs.contentType IN (" + StringUtils.toQuotedCommaSeparatedString(ALL_CULLING_CONTENT_TYPES) + ")";
	
	private static final String AUTOMATIC_CULLING_ARCHIVED_FLOW_INSTANCES_SQL = "SELECT flowInstanceID FROM flowengine_flow_instances fi"
											+ " INNER JOIN flowengine_flows f ON f.flowID = fi.flowID AND f.flowFamilyID = ?"
											+ " INNER JOIN flowengine_flow_statuses fs ON fs.statusID = fi.statusID AND fs.contentType = '" + ContentType.ARCHIVED + "'"
	                                        + " INNER JOIN flowengine_culling_settings cs ON cs.flowFamilyID = f.flowFamilyID AND cs.name = fs.name"
	                                        + " WHERE cs.name = ? AND fi.lastStatusChange IS NOT NULL AND DATEDIFF(NOW(), fi.lastStatusChange) > cs.lifetimeDays";

	private static final String AUTOMATIC_CULLING_UNSUBMITTED_FLOW_INSTANCES_SQL = "SELECT flowInstanceID FROM flowengine_flow_instances fi INNER JOIN"
											+ " flowengine_flows f ON f.flowID = fi.flowID AND f.flowFamilyID = ? INNER JOIN"
											+ " flowengine_flow_statuses fs ON fs.statusID = fi.statusID AND fs.contentType IN"
											+ " (" + StringUtils.toQuotedCommaSeparatedString(UNSUBMITTED_CULLING_CONTENT_TYPES) + ") INNER JOIN"
											+ " flowengine_culling_settings cs ON cs.flowFamilyID = f.flowFamilyID AND cs.name = fs.name"
											+ " WHERE cs.name = ? AND fi.firstSubmitted IS NULL AND ((fi.updated IS NULL AND fi.lastStatusChange IS NOT NULL"
											+ " AND DATEDIFF(NOW(), fi.lastStatusChange) > cs.lifetimeDays) OR (fi.updated IS NOT NULL AND DATEDIFF(NOW(), fi.updated) > cs.lifetimeDays))";
	
	private static final String STATISTICS_GLOBAL_FLOW_INSTANCE_COUNT_QUERY = "SELECT DISTINCT(YEARWEEK(firstSubmitted, 3)) as id, count(flowInstanceID) as value"
											+ " FROM flowengine_deleted_flow_instances"
	                                        + " WHERE firstSubmitted BETWEEN ? AND ?"
	                                        + " GROUP BY id ORDER BY id ASC;";
	private static final String STATISTICS_FLOW_INSTANCE_COUNT_QUERY = "SELECT DISTINCT(YEARWEEK(firstSubmitted, 3)) as id, count(flowInstanceID) as value"
	                                        + " FROM flowengine_deleted_flow_instances"
	                                        + " WHERE flowFamilyID = ? AND firstSubmitted BETWEEN ? AND ?"
	                                        + " GROUP BY id ORDER BY id ASC;";
	//@formatter:on

	@XSLVariable(prefix = "java.")
	private String extensionViewTitle = "Culling functions";
	
	@XSLVariable(prefix = "java.")
	private String cullingUpdated = "Automatic culling settings updated";
	
	@XSLVariable(prefix = "java.")
	private String cullingInactivated = "Inactivated automatic culling";
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Priority", description = "The priority of this extension provider compared to other providers. A low value means a higher priority. Valid values are 0 - " + Integer.MAX_VALUE + ".", required = true, formatValidator = NonNegativeStringIntegerValidator.class)
	protected int priority = 0;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmentXML;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Check for flow instances to remove interval", description = "How often this module should check for expiring flow instances (specified in crontab format)", required = true, formatValidator = CronStringValidator.class)
	private String deleteFlowInstancesInterval = "0 0 * * *";
	
	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Allowed groups", description = "Groups allowed to remove flow instances")
	protected List<Integer> allowedGroupIDs;

	@ModuleSetting(allowsNull = true)
	@UserMultiListSettingDescriptor(name = "Allowed users", description = "Users allowed to remove flow instances")
	protected List<Integer> allowedUserIDs;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "Default saved culling message", description = "Default culling message shown when flowinstances are saved", required = true)
	@XSLVariable(prefix = "java.")
	private String defaultSavedCullingMessage;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Comment required", description = "Enables comment required during culling")
	private boolean cullingCommentRequired = false;
	
	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@InstanceManagerDependency(required = false)
	private FlowSubmitSurveyProvider flowSubmitSurveyProvider;
	
	@InstanceManagerDependency(required = false)
	private HTMLContentFilter htmlContentFilter;

	private AnnotatedDAO<Status> statusDAO;
	private AnnotatedDAO<DeletedFlowInstance> deletedFlowInstanceDAO;
	protected AnnotatedDAO<AutomaticCullingSetting> settingsDAO;
	protected AnnotatedDAO<AutomaticCullingMessage> messageDAO;
	protected AnnotatedDAO<CullingComment> cullingCommentDAO;
	
	protected QueryParameterFactory<DeletedFlowInstance, Timestamp> deletedFlowInstanceAddedParamFactory;
	protected QueryParameterFactory<DeletedFlowInstance, Integer> deletedFlowInstanceFlowIDParamFactory;
	protected QueryParameterFactory<DeletedFlowInstance, Integer> deletedFlowInstanceFlowFamilyIDParamFactory;

	protected QueryParameterFactory<AutomaticCullingSetting, Integer> settingFlowFamilyIDParamFactory;
	protected QueryParameterFactory<AutomaticCullingSetting, String> settingStatusNameParamFactory;
	protected QueryParameterFactory<AutomaticCullingSetting, CullingCategory> settingdCategoryParamFactory;
	protected QueryParameterFactory<AutomaticCullingMessage, Integer> messageFlowFamilyIDParamFactory;
	protected QueryParameterFactory<QueryInstanceDescriptor, Integer> queryInstanceDescriptorFlowInstanceIDParamFactory;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	private FlowAdminModule flowAdminModule;
	private StatisticsModule statisticsModule;
	private StatisticsExtensionConsumer statisticsExtensionConsumer;
	private SavedFlowInstanceMessageHandler savedFlowInstanceMessageHandler;
	
	private Scheduler scheduler;
	private String deleteFlowInstancesScheduleID;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, sectionInterface.getSystemInterface().getEncoding());
		
		super.init(moduleDescriptor, sectionInterface, dataSource);
		
		systemInterface.addServerStartupListener(this);
	}
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, CullingModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("daos/DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface);

		statusDAO = daoFactory.getDAO(Status.class);
		settingsDAO = daoFactory.getDAO(AutomaticCullingSetting.class);
		messageDAO = daoFactory.getDAO(AutomaticCullingMessage.class);
		deletedFlowInstanceDAO = daoFactory.getDAO(DeletedFlowInstance.class);
		
		cullingCommentDAO = daoFactory.getDAO(CullingComment.class);

		AnnotatedDAO<QueryInstanceDescriptor> queryInstanceDAO = daoFactory.getDAO(QueryInstanceDescriptor.class);

		queryInstanceDescriptorFlowInstanceIDParamFactory = queryInstanceDAO.getParamFactory("flowInstanceID", Integer.class);
		settingFlowFamilyIDParamFactory = settingsDAO.getParamFactory("flowFamilyID", Integer.class);
		settingStatusNameParamFactory = settingsDAO.getParamFactory("name", String.class);
		settingdCategoryParamFactory = settingsDAO.getParamFactory("category", CullingCategory.class);
		messageFlowFamilyIDParamFactory = messageDAO.getParamFactory("flowFamilyID", Integer.class);
		
		deletedFlowInstanceAddedParamFactory = deletedFlowInstanceDAO.getParamFactory("added", Timestamp.class);
		deletedFlowInstanceFlowIDParamFactory = deletedFlowInstanceDAO.getParamFactory("flowID", Integer.class);
		deletedFlowInstanceFlowFamilyIDParamFactory = deletedFlowInstanceDAO.getParamFactory("flowFamilyID", Integer.class);
		
	}
	
	@Override
	public void serverStarted() throws Exception {
		
		initScheduler();
	}
	
	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if (flowAdminModule == null && this.flowAdminModule != null) {

			this.flowAdminModule.removeFragmentExtensionViewProvider(this);
		}

		this.flowAdminModule = flowAdminModule;

		if (this.flowAdminModule != null) {

			this.flowAdminModule.addFragmentExtensionViewProvider(this);
		}
	}
	
	@InstanceManagerDependency
	public void setStatisticsModule(StatisticsModule statisticsModule) {
		
		if (statisticsModule != null) {
			
			statisticsModule.addStatisticsExtensionProvider(this);
			
		} else {
			
			this.statisticsModule.removeStatisticsExtensionProvider(this);
		}
		
		this.statisticsModule = statisticsModule;
	}
	
	@InstanceManagerDependency
	public void setStatisticsAPIModule(StatisticsExtensionConsumer statisticsExtensionConsumer) {
		
		if (statisticsExtensionConsumer != null) {
			
			statisticsExtensionConsumer.addStatisticsExtensionProvider(this);
			
		} else {
			
			this.statisticsExtensionConsumer.removeStatisticsExtensionProvider(this);
		}
		
		this.statisticsExtensionConsumer = statisticsExtensionConsumer;
	}
	
	@InstanceManagerDependency(required = true)
	public void setSavedFlowInstanceMessageHandler(SavedFlowInstanceMessageHandler savedFlowInstanceMessageHandler) {
		
		if (savedFlowInstanceMessageHandler != null) {
			
			savedFlowInstanceMessageHandler.addSavedFlowInstanceMessageProvider(this);
			
		} else {
			
			this.savedFlowInstanceMessageHandler.removeSavedFlowInstanceMessageProvider(this);
		}
		
		this.savedFlowInstanceMessageHandler = savedFlowInstanceMessageHandler;
		
	}
	
	@Override
	public void update(ForegroundModuleDescriptor descriptor, DataSource dataSource) throws Exception {
		
		super.update(descriptor, dataSource);
		
		scheduler.reschedule(deleteFlowInstancesScheduleID, deleteFlowInstancesInterval);
	}
	
	@Override
	protected void moduleConfigured() throws Exception {
		
		viewFragmentTransformer.setDebugXML(debugFragmentXML);
		viewFragmentTransformer.modifyScriptsAndLinks(true, null);
	}

	@Override
	public void unload() throws Exception {

		stopScheduler();
		
		if (flowAdminModule != null) {

			setFlowAdminModule(null);
		}
		
		if (statisticsModule != null) {

			setStatisticsModule(null);
		}
		
		if (statisticsExtensionConsumer != null) {

			setStatisticsAPIModule(null);
		}

		if (savedFlowInstanceMessageHandler != null) {

			setSavedFlowInstanceMessageHandler(null);
		}
		
		super.unload();
	}
	
	protected synchronized void initScheduler() {
		
		if (scheduler != null) {
			
			log.warn("Invalid state, scheduler already running!");
			stopScheduler();
		}
		
		scheduler = new Scheduler(systemInterface.getApplicationName() + " - " + moduleDescriptor.toString());
		scheduler.setDaemon(true);
		
		deleteFlowInstancesScheduleID = scheduler.schedule(deleteFlowInstancesInterval, this);
		
		scheduler.start();
	}
	
	protected synchronized void stopScheduler() {
		
		try {
			if (scheduler != null) {
				
				scheduler.stop();
				scheduler = null;
			}
			
		} catch (IllegalStateException e) {
			log.error("Error stopping scheduler", e);
		}
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException {

		Document doc = createDocument(req, uriParser);
		Element cullFamilyElement = doc.createElement("ListCullings");
		doc.getFirstChild().appendChild(cullFamilyElement);

		log.info("User " + user + " listing all cullings");

		return new SimpleForegroundModuleResponse(doc);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse getAllInstances(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException, IOException {

		HighLevelQuery<DeletedFlowInstance> query = new HighLevelQuery<>();
		query.addRelation(DeletedFlowInstance.COMMENT);
		
		query.addCachedRelation(DeletedFlowInstance.COMMENT);
		
		List<DeletedFlowInstance> deletedInstances = deletedFlowInstanceDAO.getAll(query);

		JsonObject jsonObject = new JsonObject(1);

		if (!CollectionUtils.isEmpty(deletedInstances)) {
			JsonArray instancesArray = new JsonArray(deletedInstances.size());

			for (DeletedFlowInstance deletedInstance : deletedInstances) {
				JsonArray instanceArray = new JsonArray(10);

				instanceArray.addNode(deletedInstance.getFlowInstanceID());
				instanceArray.addNode(deletedInstance.getFlowFamilyID());
				instanceArray.addNode(deletedInstance.getFlowID());
				instanceArray.addNode(deletedInstance.getFlowName());
				instanceArray.addNode(DateUtils.DATE_TIME_FORMATTER.format(deletedInstance.getAdded()));
				instanceArray.addNode(deletedInstance.getFirstSubmitted() != null ? DateUtils.DATE_TIME_FORMATTER.format(deletedInstance.getFirstSubmitted()) : "");
				instanceArray.addNode(deletedInstance.getDeletedBy() == null ? "" : deletedInstance.getDeletedBy());
				instanceArray.addNode(deletedInstance.getDeletedByName() == null ? "" : deletedInstance.getDeletedByName());
				instanceArray.addNode(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(deletedInstance.getDeleted()));
				instanceArray.addNode(deletedInstance.getComment() == null ? "" : deletedInstance.getComment().getComment());

				instancesArray.addNode(instanceArray);
			}

			jsonObject.putField("data", instancesArray);
			
		} else {
			jsonObject.putField("data", new JsonArray());
		}

		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

		return null;
	}

	public ForegroundModuleResponse cullFamily(String extensionRequestURL, Flow flow, HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws SQLException {

		FlowFamily flowFamily = flow.getFlowFamily();
		
		log.info("User " + user + " viewing culling options for flow family " + flowFamily);

		Document doc = createDocument(req, uriParser);
		Element cullFamilyElement = doc.createElement("CullFamily");
		doc.getFirstChild().appendChild(cullFamilyElement);

		XMLUtils.append(doc, cullFamilyElement, flowFamily);

		LowLevelQuery<Status> statusQuery = new LowLevelQuery<>(MANUAL_CULLING_STATUSES_SQL);
		statusQuery.addParameter(flowFamily.getFlowFamilyID());

		List<Status> statuses = statusDAO.getAll(statusQuery);

		if (!CollectionUtils.isEmpty(statuses)) {
			TreeSet<String> statusNames = new TreeSet<>();

			for (Status status : statuses) {
				statusNames.add(status.getName());
			}

			XMLUtils.append(doc, cullFamilyElement, statuses);
			XMLUtils.append(doc, cullFamilyElement, "StatusNames", "Name", statusNames);

		}
		
		if(!CollectionUtils.isEmpty(validationErrors)) {
			XMLUtils.append(doc, cullFamilyElement, "ValidationErrors", validationErrors);
		}

		XMLUtils.appendNewElement(doc, cullFamilyElement, "VersionName", flow.getName());
		XMLUtils.appendNewElement(doc, cullFamilyElement, "extensionRequestURL", extensionRequestURL);

		return new SimpleForegroundModuleResponse(doc);
	}

	public ForegroundModuleResponse getInstances(Flow flow, HttpServletRequest req, HttpServletResponse res) throws SQLException, IOException {

		FlowFamily flowFamily = flow.getFlowFamily();
		
		String fromDateText = req.getParameter("fromDate");
		String toDateText = req.getParameter("toDate");

		Date fromDate = DateUtils.getDate(DateUtils.DATE_FORMATTER, fromDateText);
		Date toDate = DateUtils.getDate(DateUtils.DATE_FORMATTER, toDateText);

		boolean submitted = req.getParameter("submitted") != null;
		boolean notSubmitted = req.getParameter("notSubmitted") != null;
		
		JsonObject jsonObject = new JsonObject(1);
		
		if ((!StringUtils.isEmpty(fromDateText) && fromDate == null) || (!StringUtils.isEmpty(toDateText) && toDate == null)) {
			jsonObject.putField("error", "dateformat");
			
			HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

			return null;
		}

		if ((fromDate != null && toDate != null && fromDate.after(toDate))) {
			jsonObject.putField("error", "dateinterval");
			
			HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

			return null;
		}
		
		if ((!submitted && !notSubmitted)) {
			jsonObject.putField("error", "submittederror");
			
			HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

			return null;
		}

		if (fromDate != null) {
			fromDate = DateUtils.setTimeToMidnight(fromDate);
		}

		if (toDate != null) {
			toDate = DateUtils.setTimeToMaximum(toDate);
		}

		StringBuilder sql = new StringBuilder("SELECT * FROM ");
		sql.append(flowAdminModule.getDAOFactory().getFlowInstanceDAO().getTableName());
		sql.append(" fi INNER JOIN ");
		sql.append(flowAdminModule.getDAOFactory().getFlowDAO().getTableName());
		sql.append(" f ON (fi.flowID = f.flowID) ");
		sql.append("WHERE f.flowFamilyID = ");
		sql.append(flowFamily.getFlowFamilyID());

		List<Integer> statusIDs = NumberUtils.toInt(req.getParameterValues("statusID"));

		if (!CollectionUtils.isEmpty(statusIDs)) {
			sql.append(" AND fi.statusID IN (");
			sql.append(StringUtils.toCommaSeparatedString(statusIDs));
			sql.append(")");
		}

		if (submitted && notSubmitted) {

			if (fromDate != null && toDate != null) {
				sql.append(" AND ((fi.firstSubmitted >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("' AND fi.firstSubmitted <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("') OR (fi.firstSubmitted IS NULL AND fi.added >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("' AND fi.added <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("'))");

			} else if (fromDate != null) {
				sql.append(" AND ((fi.firstSubmitted >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("') OR (fi.firstSubmitted IS NULL AND fi.added >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("'))");

			} else if (toDate != null) {
				sql.append(" AND ((fi.firstSubmitted IS NOT NULL AND fi.firstSubmitted <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("') OR (fi.firstSubmitted IS NULL AND fi.added <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("'))");
			}

		} else if (submitted) {

			if (fromDate != null && toDate != null) {
				sql.append(" AND ((fi.firstSubmitted >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("' AND fi.firstSubmitted <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("'))");

			} else if (fromDate != null) {
				sql.append(" AND fi.firstSubmitted >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("'");

			} else if (toDate != null) {
				sql.append(" AND fi.firstSubmitted <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("'");

			} else {
				sql.append(" AND fi.firstSubmitted IS NOT NULL");
			}

		} else {

			if (fromDate != null && toDate != null) {
				sql.append(" AND ((fi.added >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("' AND fi.added <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("')");

			} else if (fromDate != null) {
				sql.append(" AND (fi.added >= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate));
				sql.append("'");

			} else if (toDate != null) {
				sql.append(" AND (fi.added <= '");
				sql.append(DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate));
				sql.append("'");
			}

			sql.append(" AND fi.firstSubmitted IS NULL");

			if (fromDate != null || toDate != null) {
				sql.append(")");
			}
		}

		LowLevelQuery<FlowInstance> query = new LowLevelQuery<>(sql.toString());
		query.addRelation(FlowInstance.FLOW_RELATION);

		List<FlowInstance> flowInstances = flowAdminModule.getDAOFactory().getFlowInstanceDAO().getAll(query);

		JsonArray instanceArray = new JsonArray(CollectionUtils.getSize(flowInstances));

		if (!CollectionUtils.isEmpty(flowInstances)) {
			for (FlowInstance flowInstance : flowInstances) {
				Date addedDate = flowInstance.getFirstSubmitted() == null ? flowInstance.getAdded() : flowInstance.getFirstSubmitted();

				JsonArray instance = new JsonArray(5);
				instance.addNode("");
				instance.addNode(flowInstance.getFlowInstanceID());
				instance.addNode(flowInstance.getFlow().getVersion());
				instance.addNode(DateUtils.DATE_FORMATTER.format(addedDate));
				instance.addNode(flowInstance.getFirstSubmitted() != null);

				instanceArray.addNode(instance);
			}
		}

		jsonObject.putField("instances", instanceArray);

		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);

		return null;
	}

	public synchronized ForegroundModuleResponse removeInstances(String extensionRequestURL, Flow flow, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException, ValidationException {

		if (req.getMethod().equals("POST")) {
			
			FlowFamily flowFamily = flow.getFlowFamily();

			List<Integer> flowInstanceIDs = NumberUtils.toInt(req.getParameterValues("flowInstanceID"));

			if (!CollectionUtils.isEmpty(flowInstanceIDs)) {
				
				List<ValidationError> validationErrors = new ArrayList<ValidationError>(1);
				
				String comment = ValidationUtils.validateParameter("comment", req, cullingCommentRequired, 0, 65536, StringPopulator.getPopulator(), validationErrors);
				
				if(!validationErrors.isEmpty()) {
					
					return cullFamily(extensionRequestURL,flow, req, user, uriParser, validationErrors);
				}

				flowInstanceIDs = CollectionUtils.removeDuplicates(flowInstanceIDs);

				log.info("User " + user + " deleting all flow instances with ID's: " + flowInstanceIDs);

				deleteFlowInstances(flowFamily.getFlowFamilyID(), flowInstanceIDs, comment, user);
			}

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	public ForegroundModuleResponse updateAutomaticCulling(Flow flow, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException, IOException {

		FlowFamily flowFamily = flow.getFlowFamily();
		Integer flowFamilyID = flowFamily.getFlowFamilyID();

		HighLevelQuery<AutomaticCullingSetting> query = new HighLevelQuery<>();
		query.addParameter(settingFlowFamilyIDParamFactory.getParameter(flowFamilyID));

		List<AutomaticCullingSetting> settings = settingsDAO.getAll(query);

		HighLevelQuery<AutomaticCullingMessage> messageQuery = new HighLevelQuery<>();
		messageQuery.addParameter(messageFlowFamilyIDParamFactory.getParameter(flowFamilyID));

		AutomaticCullingMessage message = messageDAO.get(messageQuery);
		
		//Get all statuses for the current FlowFamily
		LowLevelQuery<Status> statusQuery = new LowLevelQuery<>(AUTOMATIC_CULLING_STATUSES_SQL);
		statusQuery.addParameter(flowFamilyID);

		List<Status> statuses = statusDAO.getAll(statusQuery);
		List<String> archivedStatusNames = new ArrayList<>();
		List<String> unsubmittedStatusNames = new ArrayList<>();

		if (!CollectionUtils.isEmpty(statuses)) {

			for (Status status : statuses) {
				
				if(status.getContentType().equals(ContentType.ARCHIVED)) {
					
					archivedStatusNames.add(status.getName());
					
				} else {
					
					unsubmittedStatusNames.add(status.getName());
				}
			}

			CollectionUtils.removeDuplicates(archivedStatusNames, String.CASE_INSENSITIVE_ORDER);
			CollectionUtils.removeDuplicates(unsubmittedStatusNames, String.CASE_INSENSITIVE_ORDER);
		}

		List<ValidationError> validationErrors = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			validationErrors = new ArrayList<>();

			// Archived statuses
			Map<String, Integer> archivedStatusLifetimes = new HashMap<>();

			String[] culledArchivedStatuses = req.getParameterValues("culledStatus-archived");
			
			if (culledArchivedStatuses != null) {
				
				for (String statusName : culledArchivedStatuses) {

					if (archivedStatusNames.contains(statusName)) {

						Integer lifetime = ValidationUtils.validateParameter("days-" + statusName, req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);

						if (lifetime != null) {

							archivedStatusLifetimes.put(statusName, lifetime);
						}

					} else {

						validationErrors.add(new ValidationError("culledStatus-archived", ValidationErrorType.InvalidFormat));
					}
				}
			}

			// Unsubmitted statuses
			Map<String, Integer> unsubmittedStatusLifetimes = new HashMap<>();

			String[] culledUnsubmittedStatuses = req.getParameterValues("culledStatus-unsubmitted");
			
			if (culledUnsubmittedStatuses != null) {
				
				for (String statusName : culledUnsubmittedStatuses) {

					if (unsubmittedStatusNames.contains(statusName)) {

						Integer lifetime = ValidationUtils.validateParameter("days-" + statusName, req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);

						if (lifetime != null) {

							unsubmittedStatusLifetimes.put(statusName, lifetime);
						}

					} else {

						validationErrors.add(new ValidationError("culledStatus-unsubmitted", ValidationErrorType.InvalidFormat));
					}
				}
			}
			
			String customSavedInstanceCullingMessage = null;

			//Check custom message
			if("true".equals(req.getParameter("enableCustomSavedCullingMessage"))) {
				
				if(!StringUtils.isEmpty(req.getParameter("customSavedCullingMessage"))) {
					
					customSavedInstanceCullingMessage = req.getParameter("customSavedCullingMessage");
					
				} else {
					
					validationErrors.add(new ValidationError("customSavedCullingMessage", ValidationErrorType.RequiredField));
				}
			}

			// Store settings
			if (validationErrors.isEmpty()) {

				if (archivedStatusLifetimes.isEmpty() && unsubmittedStatusLifetimes.isEmpty()) {

					if (settings != null) {
						
						settingsDAO.delete(settings);

						flowAdminModule.addFlowFamilyEvent(cullingInactivated, flowAdminModule.getLatestFlowVersion(flowFamily), user);
					}
					
					res.sendRedirect(uriParser.getFullContextPath() + flowAdminModule.getFullAlias() + "/showflow/" + flowAdminModule.getLatestFlowVersion(flowFamily).getFlowID() + "#culling");
					return null;

				} else if (!"true".equals(req.getParameter("confirm"))) {

					validationErrors.add(new ValidationError("confirm", ValidationErrorType.RequiredField));

				} else {

					TransactionHandler transaction = null;

					try {

						transaction = new TransactionHandler(dataSource);

						HighLevelQuery<AutomaticCullingSetting> deleteQuery = new HighLevelQuery<>();
						deleteQuery.addParameter(settingFlowFamilyIDParamFactory.getParameter(flowFamilyID));
						settingsDAO.delete(deleteQuery, transaction);

						// Archived
						for (Entry<String, Integer> a : archivedStatusLifetimes.entrySet()) {

							log.info("User " + user + " added automatic culling settings for status " + a.getKey() + " with lifespan " + a.getValue() + "  for flow family " + flowFamily);

							settingsDAO.add(new AutomaticCullingSetting(a.getKey(), a.getValue(), flowFamilyID, CullingCategory.ARCHIVED), transaction, null);
						}
						
						// Unsubmitted
						for (Entry<String, Integer> a : unsubmittedStatusLifetimes.entrySet()) {

							log.info("User " + user + " added automatic culling settings for status " + a.getKey() + " with lifespan " + a.getValue() + "  for flow family " + flowFamily);

							settingsDAO.add(new AutomaticCullingSetting(a.getKey(), a.getValue(), flowFamilyID, CullingCategory.UNSUBMITTED), transaction, null);
						}
						
						if(!StringUtils.isEmpty(customSavedInstanceCullingMessage)) {
							
							if(htmlContentFilter != null) {
								
								customSavedInstanceCullingMessage = htmlContentFilter.filterHTML(customSavedInstanceCullingMessage);
							}
							
							if(message == null) {
								
								message = new AutomaticCullingMessage(flowFamilyID, customSavedInstanceCullingMessage);
								messageDAO.add(message);
								
							} else {
								
								message.setMessage(customSavedInstanceCullingMessage);
								messageDAO.update(message);
							}
						} else {
							
							if(message != null) {
								
								messageDAO.delete(message);
							}
						}
						
						transaction.commit();

						flowAdminModule.addFlowFamilyEvent(cullingUpdated, flowAdminModule.getLatestFlowVersion(flowFamily), user);

					} finally {
						TransactionHandler.autoClose(transaction);
					}

					res.sendRedirect(uriParser.getFullContextPath() + flowAdminModule.getFullAlias() + "/showflow/" + flowAdminModule.getLatestFlowVersion(flowFamily).getFlowID() + "#culling");
					return null;
				}
			}
		}

		log.info("User " + user + " viewing automatic culling settings for flow family " + flowFamily);

		Document doc = createDocument(req, uriParser);
		Element automaticCullingElement = doc.createElement("AutomaticCulling");
		doc.getFirstChild().appendChild(automaticCullingElement);

		XMLUtils.append(doc, automaticCullingElement, flowFamily);

		if (!CollectionUtils.isEmpty(archivedStatusNames)) {

			XMLUtils.append(doc, automaticCullingElement, "ArchivedStatusNames", "Name", archivedStatusNames);
		}

		if (!CollectionUtils.isEmpty(unsubmittedStatusNames)) {

			XMLUtils.append(doc, automaticCullingElement, "UnsubmittedStatusNames", "Name", unsubmittedStatusNames);
		}

		XMLUtils.appendNewElement(doc, automaticCullingElement, "VersionName", flow.getName());

		if(message != null) {
			
			XMLUtils.appendNewElement(doc, automaticCullingElement, "CustomSavedCullingMessage", message.getMessage());
			
		} else {
			
			XMLUtils.appendNewElement(doc, automaticCullingElement, "DefaultSavedCullingMessage", this.defaultSavedCullingMessage);
		}
		
		if (settings != null) {

			XMLUtils.append(doc, automaticCullingElement, "Settings", settings);
		}
		
		if (!CollectionUtils.isEmpty(validationErrors)) {

			automaticCullingElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			XMLUtils.append(doc, automaticCullingElement, "ValidationErrors", validationErrors);
		}

		return new SimpleForegroundModuleResponse(doc);
	}
	
	@Override
	public void run() {

		try {

			// Archived
			HighLevelQuery<AutomaticCullingSetting> archivedQuery = new HighLevelQuery<>();
			archivedQuery.addParameter(settingdCategoryParamFactory.getParameter(CullingCategory.ARCHIVED));
		
			List<AutomaticCullingSetting> settingsArchived = settingsDAO.getAll(archivedQuery);
			
			if (settingsArchived != null) {
				
				log.info("Checking for flow instances to delete for " + settingsArchived.size() + " archived statuses");
	
				for (AutomaticCullingSetting setting : settingsArchived) {
					
					ArrayListQuery<Integer> query = new ArrayListQuery<>(dataSource, AUTOMATIC_CULLING_ARCHIVED_FLOW_INSTANCES_SQL, IntegerPopulator.getPopulator());
					query.setInt(1, setting.getFlowFamilyID());
					query.setString(2, setting.getName());
					
					List<Integer> flowInstanceIDs = query.executeQuery();
					
					if (flowInstanceIDs != null) {
						
						log.info("Found " + flowInstanceIDs.size() + " flow instances to delete for flow family " + setting.getFlowFamilyID() + " with status " + setting.getName() + ". Deleting flow instances with ID's: " + flowInstanceIDs);
						
						deleteFlowInstances(setting.getFlowFamilyID(), flowInstanceIDs, null, null);
					}
				}
			}
			
			// Unsubmitted
			HighLevelQuery<AutomaticCullingSetting> unsubmittedQuery = new HighLevelQuery<>();
			unsubmittedQuery.addParameter(settingdCategoryParamFactory.getParameter(CullingCategory.UNSUBMITTED));
		
			List<AutomaticCullingSetting> settingsUnsubmitted = settingsDAO.getAll(unsubmittedQuery);
			
			if (settingsUnsubmitted != null) {
				
				log.info("Checking for flow instances to delete for " + settingsUnsubmitted.size() + " unsubmitted statuses");
	
				for (AutomaticCullingSetting setting : settingsUnsubmitted) {
					
					ArrayListQuery<Integer> query = new ArrayListQuery<>(dataSource, AUTOMATIC_CULLING_UNSUBMITTED_FLOW_INSTANCES_SQL, IntegerPopulator.getPopulator());
					query.setInt(1, setting.getFlowFamilyID());
					query.setString(2, setting.getName());
					
					List<Integer> flowInstanceIDs = query.executeQuery();
					
					if (flowInstanceIDs != null) {
						
						log.info("Found " + flowInstanceIDs.size() + " flow instances to delete for flow family " + setting.getFlowFamilyID() + " with status " + setting.getName() + ". Deleting flow instances with ID's: " + flowInstanceIDs);
						
						deleteFlowInstances(setting.getFlowFamilyID(), flowInstanceIDs, null, null);
					}
				}
			}
			
		
		} catch (Throwable t) {
			log.error("Error auto deleting flow instances", t);
		}
	}
	
	public void deleteFlowInstances(Integer flowFamilyID, List<Integer> flowInstanceIDs, String comment, User user) throws SQLException {
		
		if (!CollectionUtils.isEmpty(flowInstanceIDs)) {
			
			TransactionHandler transactionHandler = null;

			try {
				transactionHandler = new TransactionHandler(dataSource);

				LowLevelQuery<FlowInstance> query = new LowLevelQuery<>("SELECT * FROM " + flowAdminModule.getDAOFactory().getFlowInstanceDAO().getTableName() +
				 " fi INNER JOIN " + flowAdminModule.getDAOFactory().getFlowDAO().getTableName() + " f ON (fi.flowID = f.flowID) WHERE f.flowFamilyID = ? AND fi.flowInstanceID IN (" + (StringUtils.toCommaSeparatedString(flowInstanceIDs)) + ")");
				query.addParameter(flowFamilyID);

				query.addRelations(FlowInstance.FLOW_RELATION);
				query.addCachedRelations(FlowInstance.FLOW_RELATION);

				List<FlowInstance> flowInstances = flowAdminModule.getDAOFactory().getFlowInstanceDAO().getAll(query, transactionHandler);

				if (flowInstances != null) {

					Timestamp deletedTime = TimeUtils.getCurrentTimestamp();
					
					CullingComment cullingComment = null;
					
					if(comment != null) {
						
						cullingComment = new CullingComment();
						cullingComment.setComment(comment);
					}

					for (FlowInstance flowInstance : flowInstances) {

						DeletedFlowInstance deletedInstance = createDeletedFlowInstance(flowInstance, flowFamilyID, deletedTime, user);
						
						deletedInstance.setComment(cullingComment);
						
						deletedFlowInstanceDAO.add(deletedInstance, transactionHandler, COMMENT_RELATION_QUERY);

						HighLevelQuery<QueryInstanceDescriptor> descriptorQuery = new HighLevelQuery<>(queryInstanceDescriptorFlowInstanceIDParamFactory.getParameter(flowInstance.getFlowInstanceID()));

						descriptorQuery.addRelation(QueryInstanceDescriptor.QUERY_DESCRIPTOR_RELATION);

						List<QueryInstanceDescriptor> queryInstanceDescriptors = flowAdminModule.getDAOFactory().getQueryInstanceDescriptorDAO().getAll(descriptorQuery, transactionHandler);

						if (queryInstanceDescriptors != null) {
							for (QueryInstanceDescriptor queryInstanceDescriptor : queryInstanceDescriptors) {
								
								log.info("Deleting query instance: " + queryInstanceDescriptor);

								try {
									queryHandler.deleteQueryInstance(queryInstanceDescriptor, transactionHandler);
									
								} catch (Exception e) {
									log.warn("Error deleting query instance " + queryInstanceDescriptor);
								}
							}
						}

						log.info("Deleting flow instance " + flowInstance);

						flowAdminModule.getDAOFactory().getFlowInstanceDAO().delete(flowInstance, transactionHandler);
					}
					
					transactionHandler.commit();
					
					systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(FlowInstance.class, CRUDAction.DELETE, flowInstances), EventTarget.ALL);
				}
				
			} finally {
				TransactionHandler.autoClose(transactionHandler);
			}
		}
	}

	public DeletedFlowInstance createDeletedFlowInstance(FlowInstance flowInstance, Integer flowFamilyID, Timestamp deleted, User user) {

		DeletedFlowInstance deletedInstance = new DeletedFlowInstance();

		deletedInstance.setFlowInstanceID(flowInstance.getFlowInstanceID());
		deletedInstance.setFlowFamilyID(flowFamilyID);
		deletedInstance.setFlowID(flowInstance.getFlow().getFlowID());
		deletedInstance.setAdded(flowInstance.getAdded());
		deletedInstance.setFirstSubmitted(flowInstance.getFirstSubmitted());
		deletedInstance.setFlowName(flowInstance.getFlow().getName());
		
		if (user != null) {
			
			deletedInstance.setDeletedBy(user.getUserID());
			deletedInstance.setDeletedByName(user.getFirstname() + " " + user.getLastname());
		}

		deletedInstance.setDeleted(deleted);

		return deletedInstance;
	}

	@Override
	public int getPriority() {

		return priority;
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@Override
	public FlowAdminExtensionShowView getShowView(String extensionRequestURL, Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerException, SQLException {

		if (!AccessUtils.checkAccess(user, this) && !AccessUtils.checkRecursiveModuleAccess(user, moduleDescriptor, systemInterface)) {
			return null;
		}

		Document doc = createDocument(req, uriParser);

		Element showViewElement = doc.createElement("ShowCulling");
		doc.getDocumentElement().appendChild(showViewElement);

		ObjectQuery<Integer> flowInstancesCountQuery = new ObjectQuery<>(dataSource, FLOW_INSTANCE_COUNT_SQL, IntegerPopulator.getPopulator());
		flowInstancesCountQuery.setInt(1, flow.getFlowFamily().getFlowFamilyID());
		Integer flowInstanceCount = flowInstancesCountQuery.executeQuery();

		if (flowInstanceCount > 0) {
			XMLUtils.appendNewElement(doc, showViewElement, "FlowInstanceCount", flowInstanceCount);
		}

		XMLUtils.append(doc, showViewElement, flow);
		
		HighLevelQuery<AutomaticCullingSetting> query = new HighLevelQuery<>();
		query.addParameter(settingFlowFamilyIDParamFactory.getParameter(flow.getFlowFamily().getFlowFamilyID()));

		if (settingsDAO.getBoolean(query)) {
			
			XMLUtils.appendNewElement(doc, showViewElement, "IsAutomatic");
		}
		
		XMLUtils.appendNewElement(doc, showViewElement, "extensionRequestURL", extensionRequestURL);

		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), true);
	}
	
	@Override
	public ViewFragment processRequest(String extensionRequestURL, Flow flow, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		if (!AccessUtils.checkAccess(user, this) && !AccessUtils.checkRecursiveModuleAccess(user, moduleDescriptor, systemInterface)) {
			throw new AccessDeniedException("User is not allowed to remove flow instances");
		}
		
		String method = uriParser.get(4);

		if ("cullfamily".equals(method)) {

			return getViewFragmentResponse(cullFamily(extensionRequestURL, flow, req, user, uriParser, null));

		} else if ("updateautomaticculling".equals(method)) {

			return getViewFragmentResponse(updateAutomaticCulling(flow, req, res, user, uriParser));

		} else if ("removeinstances".equals(method)) {

			return getViewFragmentResponse(removeInstances(extensionRequestURL, flow, req, res, user, uriParser));

		} else if ("getinstances".equals(method)) {

			return getViewFragmentResponse(getInstances(flow, req, res));

		} else if ("toflow".equals(method)) {

			return null;
		}

		throw new URINotFoundException(uriParser);
	}

	private ViewFragment getViewFragmentResponse(ForegroundModuleResponse foregroundModuleResponse) throws TransformerException {

		if (foregroundModuleResponse != null) {

			if (foregroundModuleResponse.getResponseType() == ResponseType.XML_FOR_SEPARATE_TRANSFORMATION) {

				return viewFragmentTransformer.createViewFragment(foregroundModuleResponse.getDocument());

			} else {

				log.warn("Scripts and links have not been modified for FlowAdminFragmentExtensionViewProviderProcessRequest");
				return new SimpleViewFragment(foregroundModuleResponse.getHtml(), debugFragmentXML ? foregroundModuleResponse.getDocument() : null, foregroundModuleResponse.getScripts(), foregroundModuleResponse.getLinks());
			}
		}

		return null;
	}

	@Override
	public String getExtensionViewTitle() {

		return extensionViewTitle;
	}

	@Override
	public String getExtensionViewLinkName() {
		return "culling";
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");
		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		document.appendChild(this.moduleDescriptor.toXML(doc));

		XMLUtils.appendNewElement(doc, document, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, document, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));
		
		XMLUtils.appendNewElement(doc, document, "CullingCommentRequired", cullingCommentRequired);

		doc.appendChild(document);

		return doc;
	}

	@EventListener(channel = FlowInstance.class)
	public void processEvent(DeletedByManagerEvent event, EventSource source) throws SQLException {

		DeletedFlowInstance deletedInstance = createDeletedFlowInstance(event.getFlowInstance(), event.getFlowInstance().getFlow().getFlowFamily().getFlowFamilyID(), TimeUtils.getCurrentTimestamp(), event.getUser());

		deletedFlowInstanceDAO.add(deletedInstance);
	}
	
	@EventListener(channel = FlowInstance.class)
	public void processEvent(DeletedByOwnerEvent event, EventSource source) throws SQLException {

		User user;
		
		if(event.getFlowInstance().getFirstSubmitted() != null) {
			
			user = event.getUser();
			
		}else {
			
			user = null;
		}
		
		DeletedFlowInstance deletedInstance = createDeletedFlowInstance(event.getFlowInstance(), event.getFlowInstance().getFlow().getFlowFamily().getFlowFamilyID(), TimeUtils.getCurrentTimestamp(), user);

		deletedFlowInstanceDAO.add(deletedInstance);
	}
	
	@WebPublic(requireLogin = true)
	public ForegroundModuleResponse checkCulling(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException {
		
		if (user.isAdmin()) {
			
			run();
			return new SimpleForegroundModuleResponse("See log", getDefaultBreadcrumb());
		}
		
		throw new URINotFoundException(uriParser);
	}
	
	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

	@Override
	public boolean allowsAdminAccess() {
		return false;
	}

	@Override
	public boolean allowsUserAccess() {
		return false;
	}

	@Override
	public boolean allowsAnonymousAccess() {
		return false;
	}

	@Override
	public Collection<Integer> getAllowedGroupIDs() {
		return allowedGroupIDs;
	}

	@Override
	public Collection<Integer> getAllowedUserIDs() {
		return allowedUserIDs;
	}
	
	@Override
	public void getFlowFamilyStatistics(FlowFamilyStatistics familyStatistics, Timestamp startDate, Timestamp endDate, boolean detailed) throws SQLException {

		List<IntegerEntry> deletedFlowInstanceCountByWeek = getFlowFamilyDeletedInstanceCount(familyStatistics.getFlowFamilyID(), startDate, endDate);

		if (deletedFlowInstanceCountByWeek != null) {

			familyStatistics.setFlowInstanceCount(StatisticsModule.mergeIntegerEntryLists(familyStatistics.getFlowInstanceCount(), deletedFlowInstanceCountByWeek));

			if (detailed) {

				List<Entry<String, Integer>> deletedFlowInstanceCountByWeekAsStringEntries = new ArrayList<>(deletedFlowInstanceCountByWeek.size());
				List<Entry<String, Integer>> dummyGenderEntries = new ArrayList<>(deletedFlowInstanceCountByWeek.size());

				for (IntegerEntry entry : deletedFlowInstanceCountByWeek) {

					String weekInt = entry.getId().toString();
					String week = weekInt.substring(0, 4) + " v." + weekInt.substring(4);

					deletedFlowInstanceCountByWeekAsStringEntries.add(new SimpleEntry<>(week, entry.getValue()));
					dummyGenderEntries.add(new SimpleEntry<>(week, 0));
				}

				familyStatistics.setTotalFlowInstancesCountByWeek(StatisticsModule.mergeStringEntryLists(familyStatistics.getTotalFlowInstancesCountByWeek(), deletedFlowInstanceCountByWeekAsStringEntries));
				familyStatistics.setUnkownFlowInstancesCountByWeek(StatisticsModule.mergeStringEntryLists(familyStatistics.getUnkownFlowInstancesCountByWeek(), deletedFlowInstanceCountByWeekAsStringEntries));

				familyStatistics.setFemaleFlowInstancesCountByWeek(StatisticsModule.mergeStringEntryLists(familyStatistics.getFemaleFlowInstancesCountByWeek(), dummyGenderEntries));
				familyStatistics.setMaleFlowInstancesCountByWeek(StatisticsModule.mergeStringEntryLists(familyStatistics.getMaleFlowInstancesCountByWeek(), dummyGenderEntries));
			}
		}
	}
	
	@Override
	public List<IntegerEntry> getGlobalFlowInstanceCount(Timestamp startDate, Timestamp endDate) throws SQLException {
		
		ArrayListQuery<IntegerEntry> query = new ArrayListQuery<>(dataSource, STATISTICS_GLOBAL_FLOW_INSTANCE_COUNT_QUERY, StatisticsModule.INTEGER_ENTRY_POPULATOR);

		query.setTimestamp(1, startDate);
		query.setTimestamp(2, endDate);

		return query.executeQuery();
	}
	
	private List<IntegerEntry> getFlowFamilyDeletedInstanceCount(Integer flowFamilyID, Timestamp startDate, Timestamp endDate) throws SQLException {

		ArrayListQuery<IntegerEntry> query = new ArrayListQuery<>(dataSource, STATISTICS_FLOW_INSTANCE_COUNT_QUERY, StatisticsModule.INTEGER_ENTRY_POPULATOR);

		query.setInt(1, flowFamilyID);
		query.setTimestamp(2, startDate);
		query.setTimestamp(3, endDate);

		return query.executeQuery();
	}

	@Override
	public List<FlowInstanceStatistic> getFlowInstanceAPIStatistics(Timestamp fromTimestamp, Timestamp toTimestamp, Integer filterFlowID, Integer filterFlowFamilyID, int rowLimit) {

		Connection connection = null;

		try {
			connection = dataSource.getConnection();

			HighLevelQuery<DeletedFlowInstance> flowInstancesQuery = new HighLevelQuery<>();
			flowInstancesQuery.setRowLimiter(new MySQLRowLimiter(rowLimit));

			if (fromTimestamp != null) {
				flowInstancesQuery.addParameter(deletedFlowInstanceAddedParamFactory.getParameter(fromTimestamp, QueryOperators.BIGGER_THAN_OR_EQUALS));
			}

			if (toTimestamp != null) {
				flowInstancesQuery.addParameter(deletedFlowInstanceAddedParamFactory.getParameter(toTimestamp, QueryOperators.SMALLER_THAN_OR_EQUALS));
			}

			if (filterFlowID != null) {
				flowInstancesQuery.addParameter(deletedFlowInstanceFlowIDParamFactory.getParameter(filterFlowID));
			}

			if (filterFlowFamilyID != null) {
				flowInstancesQuery.addParameter(deletedFlowInstanceFlowFamilyIDParamFactory.getParameter(filterFlowFamilyID));
			}

			QueryResultsStreamer<DeletedFlowInstance, Integer> resultsStreamer = new QueryResultsStreamer<>(deletedFlowInstanceDAO, DeletedFlowInstance.ID_FIELD, Integer.class, Order.ASC, flowInstancesQuery, connection);

			List<DeletedFlowInstance> flowInstances = resultsStreamer.getBeans();
			
			if (flowInstances != null) {
				
				List<FlowInstanceStatistic> flowInstanceStatistics = new ArrayList<>(flowInstances.size());
	
				while (flowInstances != null) {
					
					if (systemInterface.getSystemStatus() == SystemStatus.STOPPING) {
						log.info("List flowinstance statistics aborted...");
						return null;
					}
					
					for (DeletedFlowInstance flowInstance : flowInstances) {
	
						FlowInstanceStatistic statistic = new FlowInstanceStatistic(flowInstance.getFlowInstanceID(), flowInstance.getFlowID(), flowInstance.getAdded());
	
						if (flowInstance.getFirstSubmitted() != null) {
							
							statistic.setSubmitted(flowInstance.getFirstSubmitted());
							
							if (flowSubmitSurveyProvider != null) {
								statistic.setSurveyAnswer(flowSubmitSurveyProvider.getFlowInstanceSurveyResult(flowInstance.getFlowInstanceID(), connection));
							}
						}
						
						flowInstanceStatistics.add(statistic);
					}
	
					if (flowInstances.size() == rowLimit) {
						flowInstances = resultsStreamer.getBeans();
	
					} else {
						break;
					}
				}
				
				return flowInstanceStatistics;
			}
			
			return null;

		} catch (SQLException e) {
			
			log.error("Error getting deleted flow instances for statistics", e);
			return null;
			
		} finally {
			DBUtils.closeConnection(connection);
		}
	}
	
	@Override
	public String getSavedInstanceMessage(ImmutableFlowInstance flowInstance) throws SQLException {

		Integer flowFamilyID = flowInstance.getFlow().getFlowFamily().getFlowFamilyID();
		
		HighLevelQuery<AutomaticCullingSetting> query = new HighLevelQuery<>();
		query.addParameter(settingFlowFamilyIDParamFactory.getParameter(flowFamilyID));
		query.addParameter(settingStatusNameParamFactory.getParameter(flowInstance.getStatus().getName()));
		
		AutomaticCullingSetting setting = settingsDAO.get(query);
		
		if(setting != null) {
			
			HighLevelQuery<AutomaticCullingMessage> messageQuery = new HighLevelQuery<>();
			messageQuery.addParameter(messageFlowFamilyIDParamFactory.getParameter(flowFamilyID));
			
			AutomaticCullingMessage message = messageDAO.get(messageQuery);
			
			if(message != null && !StringUtils.isEmpty(message.getMessage())) {
			
				return message.getMessage().replace("$autodeletion.days", setting.getLifetimeDays().toString());
				
			} else {
				
				return this.defaultSavedCullingMessage.replace("$autodeletion.days", setting.getLifetimeDays().toString());
			}
		}
		
		return null;
	}
}