<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/CKEditor.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/featherlight/js/featherlight.min.js
		/ckeditor/init.js			
	</xsl:variable>

	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/datatables.min.js
		/js/jquery.ui.datepicker-sv.js
		/js/cullingmodule.js?v=2
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/datatables.css
		/css/cullingmodule.css
	</xsl:variable>
	
	<xsl:template match="Document">
	
		<xsl:apply-templates select="ListCullings"/>
		<xsl:apply-templates select="ShowCulling" />
		<xsl:apply-templates select="CullFamily" />
		<xsl:apply-templates select="AutomaticCulling" />
			
	</xsl:template>
	
	<xsl:template match="ListCullings">
	
		<script>
			i18nCullingModule.EMPTY_TABLE = '<xsl:value-of select="$i18n.EmptyTable"/>';
			i18nCullingModule.NEXT = '<xsl:value-of select="$i18n.Next"/>';
			i18nCullingModule.PREVIOUS =  '<xsl:value-of select="$i18n.Previous"/>';
			i18nCullingModule.SEARCH =  '<xsl:value-of select="$i18n.Search"/>';
			i18nCullingModule.ZERO_RECORDS =  '<xsl:value-of select="$i18n.ZeroRecords"/>';
			i18nCullingModule.LENGTH_MENU = '<xsl:value-of select="$i18n.LengthMenu"/>';
			i18nCullingModule.INFO = '<xsl:value-of select="$i18n.DataTable.info"/>';
			i18nCullingModule.INFO_EMPTY = '<xsl:value-of select="$i18n.DataTable.infoEmpty"/>';
			i18nCullingModule.INFO_FILTERED = '<xsl:value-of select="$i18n.DataTable.infoFiltered"/>';
			i18nCullingModule.COMMENT = '<xsl:value-of select="$i18n.presentedComment"/>';
		</script>
	
		<div id="CullingModule">
			<div class="contentitem">
				<h1>
					<xsl:value-of select="$i18n.ListCullings"/>
				</h1>
				
				<table id="all-instances-table" class="stripe display" data-fetchurl="{/Document/ModuleURI}/getallinstances">
					<thead>
						<tr>
							<th class="five"><xsl:value-of select="$i18n.Instance"/></th>
							<th class="five"><xsl:value-of select="$i18n.Family"/></th>
							<th><xsl:value-of select="$i18n.Version"/></th>
							<th class="nowrap"><xsl:value-of select="$i18n.AddedDate"/></th>
							<th class="nowrap"><xsl:value-of select="$i18n.SubmittedDate"/></th>
							<th><xsl:value-of select="$i18n.DeletedBy"/></th>
							<th class="nowrap"><xsl:value-of select="$i18n.Deleted"/></th>
							<th></th>
						</tr>
					</thead>
				</table>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="ShowCulling">
		
		<xsl:variable name="extensionImgPath"><xsl:value-of select="/Document/requestinfo/contextpath" /><xsl:value-of select="extensionRequestURL" />/static/pics</xsl:variable>
		
		<a name="culling" />
		
		<xsl:choose>
			<xsl:when test="FlowInstanceCount">
				<xsl:value-of select="$i18n.CullingText.Part1" />
				<xsl:text> </xsl:text>
				<xsl:value-of select="FlowInstanceCount" />
				<xsl:text> </xsl:text>
				<xsl:value-of select="$i18n.Cullingtext.Part2" />
				
				<a class="floatright" href="{/Document/requestinfo/contextpath}{extensionRequestURL}/cullfamily">
					<xsl:value-of select="$i18n.ShowManualCullingOptions" />
					<img class="marginleft" src="{$extensionImgPath}/pen.png" alt="{$i18n.ShowManualCullingOptions}" />
				</a>
				
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoFlowInstances" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a class="floatright clearboth" href="{/Document/requestinfo/contextpath}{extensionRequestURL}/updateautomaticculling">
			<xsl:value-of select="$i18n.ShowAutomaticCullingOptions" />
			<img class="marginleft" src="{$extensionImgPath}/pen.png" alt="{$i18n.ShowAutomaticCullingOptions}" />
		</a>
		
		<xsl:if test="IsAutomatic">
			<br/>
			<xsl:value-of select="$i18n.AutomaticCullingInUse" />
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="CullFamily">
		
		<xsl:variable name="extensionImgPath"><xsl:value-of select="/Document/requestinfo/contextpath" /><xsl:value-of select="extensionRequestURL" />/static/pics</xsl:variable>
		
		<script>
			$(function() {
				$( "#fromDate" ).datepicker({
					showOn: "button",
					buttonImage: '<xsl:value-of select="$extensionImgPath"/>/calendar_grid.png',
					buttonImageOnly: true,
					buttonText: '<xsl:value-of select="$i18n.FromDate"/>',
					changeMonth: true,
            		changeYear: true,
            		showWeek: true,
				});
				
				$( "#toDate" ).datepicker({
					showOn: "button",
					buttonImage: '<xsl:value-of select="$extensionImgPath"/>/calendar_grid.png',
					buttonImageOnly: true,
					buttonText: '<xsl:value-of select="$i18n.ToDate"/>',
					changeMonth: true,
            		changeYear: true,
            		showWeek: true,
				});
			});
			
			i18nCullingModule.EMPTY_TABLE = '<xsl:value-of select="$i18n.EmptyTable"/>';
			i18nCullingModule.NEXT = '<xsl:value-of select="$i18n.Next"/>';
			i18nCullingModule.PREVIOUS =  '<xsl:value-of select="$i18n.Previous"/>';
			i18nCullingModule.YES = '<xsl:value-of select="$i18n.Yes"/>';
			i18nCullingModule.NO = '<xsl:value-of select="$i18n.No"/>';
			i18nCullingModule.CONFIRM_DELETE_PART1 = '<xsl:value-of select="$i18n.ConfirmDelete.Part1"/>';
			i18nCullingModule.CONFIRM_DELETE_PART2 = '<xsl:value-of select="$i18n.ConfirmDelete.Part2"/>';
			i18nCullingModule.LENGTH_MENU = '<xsl:value-of select="$i18n.LengthMenu"/>';
		</script>
	
		<div id="CullingModule">
			<div class="contentitem">
				<h1>
					<xsl:value-of select="$i18n.CullFamily"/>
					<xsl:text> - </xsl:text>
					<xsl:value-of select="VersionName"/>
				</h1>
				
				<div id="validationerrors"></div>
				<xsl:apply-templates select="ValidationErrors/validationError" />
				
				<fieldset class="floatleft full marginbottom">
					<legend><xsl:value-of select="$i18n.Status" /></legend>
					
					<xsl:apply-templates select="StatusNames/Name" mode="button-list"/>
				</fieldset>
				
				<fieldset class="floatleft full marginbottom">
					<legend><xsl:value-of select="$i18n.Period" /></legend>
					
					<div class="floatleft full">
						<p class="description"><xsl:value-of select="$i18n.PeriodHelpText"/></p>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<div class="floatleft">
							<label for="publishDate" class="floatleft">
								<xsl:value-of select="$i18n.FromDate" />
							</label>
							
							<div class="floatleft clearboth">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="'fromDate'" />
									<xsl:with-param name="id" select="'fromDate'" />
									<xsl:with-param name="size" select="10" />
									<xsl:with-param name="class" select="'marginright'" />
								</xsl:call-template>
							</div>
						</div>
						
						<div class="bigmarginleft floatleft">
							<label for="unPublishDate" class="floatleft">
								<xsl:value-of select="$i18n.ToDate" />
							</label>
							
							<div class="floatleft clearboth">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="'toDate'" />
									<xsl:with-param name="id" select="'toDate'" />
									<xsl:with-param name="size" select="10" />
									<xsl:with-param name="class" select="'marginright'" />
								</xsl:call-template>
							</div>
						</div>
					</div>
				</fieldset>
				
				<fieldset class="floatleft full bigmarginbottom">
					<legend><xsl:value-of select="$i18n.SubmittedNotSubmitted"/></legend>
					
					<div class="floatleft full">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="'submitted'" />
							<xsl:with-param name="name" select="'submitted'" />
						</xsl:call-template>
						
						<label for="submitted">
							<xsl:value-of select="$i18n.Submitted" />
						</label>
					</div>
					
					<div class="floatleft full">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="'notSubmitted'" />
							<xsl:with-param name="name" select="'notSubmitted'" />
						</xsl:call-template>
						
						<label for="notSubmitted">
							<xsl:value-of select="$i18n.NotSubmitted" />
						</label>
					</div>
				</fieldset>
				
				<div class="floatright clearboth bigmargintop">
					<input type="button" id="showInstances" value="{$i18n.ShowInstances}" data-submittederror="{$i18n.SubmittedError}" data-dateerror="{$i18n.DateError}" data-dateformaterror="{$i18n.DateFormatError}" data-unknownerror="{$i18n.UnknownError}" data-fetchurl="{/Document/requestinfo/contextpath}{extensionRequestURL}/getinstances" />
				</div>
			</div>
			
			<div class="contentitem" id="instances-wrapper">
				<h1>
					<xsl:value-of select="$i18n.Found"/>
					<xsl:text> </xsl:text>
					<span id="instance-count">0</span>
					<xsl:text> </xsl:text>
					<xsl:value-of select="$i18n.Instances"/>
				</h1>
				
				<table id="instances-table" class="full stripe display">
					<thead>
						<tr>
							<th><input type="checkbox" id="select-all"/></th>
							<th class="twenty"><xsl:value-of select="$i18n.Instance"/></th>
							<th class="twenty"><xsl:value-of select="$i18n.Version"/></th>
							<th><xsl:value-of select="$i18n.Date"/></th>
							<th class="twenty"><xsl:value-of select="$i18n.IsSubmitted"/></th>
						</tr>
					</thead>
				</table>
				
				<label for="comment">
					<xsl:attribute name="class">
						<xsl:if test="../CullingCommentRequired='true'">required</xsl:if>
					</xsl:attribute>
						<xsl:value-of select="$i18n.CullingComment" />
					</label>
					
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'comment'"/>
						<xsl:with-param name="name" select="'comment'"/>
						<xsl:with-param name="rows" select="4"/>
					</xsl:call-template>
				
				<div class="floatright clearboth bigmargintop">
					<a href="{/Document/requestinfo/contextpath}{extensionRequestURL}/removeinstances" onclick="return false;" class="btn btn-light" disabled="disabled" id="deleteInstances"><xsl:value-of select="$i18n.CullInstances"/></a>
				</div>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="Name" mode="button-list">
	
		<xsl:variable name="statusName">
			<xsl:value-of select="."/>
		</xsl:variable>
		
		<span class="status-button">
			<xsl:value-of select="$statusName"/>
			
			<xsl:apply-templates select="../../Status[name = $statusName]" mode="hidden-inputs"/>
		</span>
		
	</xsl:template>
	
	<xsl:template match="Status" mode="hidden-inputs">
		
		<input type="checkbox" value="{statusID}" name="statusID"/>
		
	</xsl:template>
	
	<xsl:template match="AutomaticCulling">
		
		<div id="CullingModule">
			<div class="contentitem errands-wrapper">
			
				<h1>
					<xsl:value-of select="$i18n.AutomaticCulling.title"/>
					<xsl:text> - </xsl:text>
					<xsl:value-of select="VersionName"/>
				</h1>
				
				<xsl:apply-templates select="ValidationErrors/validationError[not(starts-with(fieldName, 'days-'))]"/>
				
					<p>
					<xsl:value-of select="$i18n.AutomaticCulling.information"/>
				</p>
				
					
				<form id="autoculling" method="POST" action="{/Document/requestinfo/uri}">
						
					<fieldset id="archived-instances">
						<legend><xsl:value-of select="$i18n.AutomaticCulling.archived.heading"/></legend>
										
						<p>
							<xsl:value-of select="$i18n.AutomaticCulling.archived.description1"/>
						</p>
						<p>
							<xsl:value-of select="$i18n.AutomaticCulling.archived.description2"/>
						</p>
				
						<xsl:choose>
							<xsl:when test="ArchivedStatusNames/Name">
							
								<table class="full coloredtable sortabletable oep-table" cellspacing="0">
									<thead>
										<tr>
											<th style="width: 1%; min-width: 150px;"><xsl:value-of select="$i18n.Status" /></th>
											<th><xsl:value-of select="$i18n.AutomaticCulling.Column.lifetime" /></th>
										</tr>
									</thead>
									<tbody>
										<xsl:apply-templates select="ArchivedStatusNames/Name" mode="automatic-culling" >
											<xsl:with-param name="category">archived</xsl:with-param>
										</xsl:apply-templates>
									</tbody>
								</table>
							
							</xsl:when>
							<xsl:otherwise>
								<p>
									<xsl:value-of select="$i18n.AutomaticCulling.noArchivedStatuses"/>
								</p>
							</xsl:otherwise>
						</xsl:choose>
						
					</fieldset>
					
					<fieldset id="unsubmitted-instances">
						<legend><xsl:value-of select="$i18n.AutomaticCulling.unsubmitted.heading"/></legend>
					
						<p>
							<xsl:value-of select="$i18n.AutomaticCulling.unsubmitted.description1"/>
						</p>
						<p>
							<xsl:value-of select="$i18n.AutomaticCulling.unsubmitted.description2"/>
						</p>
				
						<xsl:choose>
							<xsl:when test="UnsubmittedStatusNames/Name">
							
								<table class="full coloredtable sortabletable oep-table" cellspacing="0">
									<thead>
										<tr>
											<th style="width: 1%; min-width: 150px;"><xsl:value-of select="$i18n.Status" /></th>
											<th><xsl:value-of select="$i18n.AutomaticCulling.Column.lifetime" /></th>
										</tr>
									</thead>
									<tbody>
										<xsl:apply-templates select="UnsubmittedStatusNames/Name" mode="automatic-culling" >
											<xsl:with-param name="category">unsubmitted</xsl:with-param>
										</xsl:apply-templates>
									</tbody>
								</table>
							
							</xsl:when>
							<xsl:otherwise>
								<p>
									<xsl:value-of select="$i18n.AutomaticCulling.noUnsubmittedStatuses"/>
								</p>
							</xsl:otherwise>
						</xsl:choose>
					
						<p><xsl:value-of select="$i18n.AutomaticCulling.SavedCullingMessage.description1"/></p>
					
						<div class="d-flex bigmarginbottom bigmargintop">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="name" select="'enableCustomSavedCullingMessage'" />
								<xsl:with-param name="id" select="'enableCustomSavedCullingMessage'" />
								<xsl:with-param name="checked">
									<xsl:if test="CustomSavedCullingMessage">true</xsl:if>
								</xsl:with-param>
								<xsl:with-param name="requestparameters" select="requestparameters" />
								<xsl:with-param name="class" select="'margintop'"/>
							</xsl:call-template>
							
							<label for="enableCustomSavedCullingMessage" style="margin-top: 0;">
								<xsl:value-of select="$i18n.AutomaticCulling.EnableCustomSavedCullingMessage" />
							</label>
						</div>
						
						<div id="customSavedCullingMessageContainer" class="floatleft full">
							<xsl:call-template name="createTextArea">
								<xsl:with-param name="id" select="'customSavedCullingMessage'"/>
								<xsl:with-param name="name" select="'customSavedCullingMessage'"/>
								<xsl:with-param name="class" select="'fckeditor'" />
								<xsl:with-param name="value">
									<xsl:choose>
										<xsl:when test="CustomSavedCullingMessage"> 
											<xsl:value-of select="CustomSavedCullingMessage" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="DefaultSavedCullingMessage" />
										</xsl:otherwise>
									</xsl:choose>
								</xsl:with-param>
							</xsl:call-template>
							
							<xsl:call-template name="addTagsTable"/>
						</div>
						
						<script type="text/javascript">
				
							$(function() {
								var checkbox = $("#enableCustomSavedCullingMessage");
								var div = $("#customSavedCullingMessageContainer");
								
								div.toggle(checkbox[0].checked);
								
								checkbox.on("click", function(){
									var checked = checkbox[0].checked;
									div.toggle(checked).find('textarea').prop('disabled', !checked);
								});
							});
						
						</script>
					
					</fieldset>

					<xsl:call-template name="initializeFCKEditor">
						<xsl:with-param name="basePath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/ckeditor/</xsl:with-param>
						<xsl:with-param name="customConfig">config.js</xsl:with-param>
						<xsl:with-param name="editorContainerClass">fckeditor</xsl:with-param>
						<xsl:with-param name="editorHeight">150</xsl:with-param>
						<xsl:with-param name="contentsCss">
							<xsl:if test="/Document/cssPath">
								<xsl:value-of select="/Document/cssPath"/>
							</xsl:if>
						</xsl:with-param>
					</xsl:call-template>
					
					<div class="clearboth"></div>

					<div class="d-flex bigmarginbottom bigmargintop">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="name" select="'confirm'" />
							<xsl:with-param name="id" select="'confirm'" />
							<xsl:with-param name="requestparameters" select="null" />
							<xsl:with-param name="class" select="'margintop'"/>
						</xsl:call-template>
						
						<label for="confirm" style="margin-top: 0;">
							<xsl:value-of select="$i18n.AutomaticCulling.confirm" />
						</label>
					</div>
										
					<div class="clearfix">
						<div class="floatright">
							<input type="submit" value="{$i18n.AutomaticCulling.save}" />
						</div>
					</div>
				</form>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="Name" mode="automatic-culling">
		<xsl:param name="category" />
		
		<xsl:variable name="statusName">
			<xsl:value-of select="."/>
		</xsl:variable>
		
		<tr>
			<td>
				<div class="floatleft full bigmarginbottom margintop">
				
					<xsl:variable name="id">
						<xsl:text>culledStatus-</xsl:text>
						<xsl:value-of select="."/>
					</xsl:variable>
					
					<div class="floatleft">
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="$id" />
							<xsl:with-param name="name" select="concat('culledStatus-', $category)" />
							<xsl:with-param name="value" select="$statusName" />
							<xsl:with-param name="checked">
								<xsl:if test="../../Settings/Setting[name = $statusName]/lifetimeDays">true</xsl:if>
							</xsl:with-param>
							<xsl:with-param name="requestparameters" select="../../requestparameters" />
						</xsl:call-template>
						
						<label for="{$id}">
							<xsl:choose>
								<xsl:when test="string-length($statusName) > 70">
									<xsl:value-of select="concat(substring($statusName,0,67),'...')"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$statusName" />
								</xsl:otherwise>
							</xsl:choose>
						</label>
					</div>
					
				</div>
			</td>
			<td>
				<xsl:variable name="daysID">
					<xsl:text>days-</xsl:text>
					<xsl:value-of select="."/>
				</xsl:variable>
				
				<xsl:variable name="class">
					<xsl:if test="../../ValidationErrors/validationError[fieldName = $daysID]">invalid input-error</xsl:if>
				</xsl:variable>
				
				<div class="{$class}">
				
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="$daysID"/>
						<xsl:with-param name="name" select="$daysID"/>
						<xsl:with-param name="value" select="../../Settings/Setting[name = $statusName]/lifetimeDays" />
						<xsl:with-param name="class" select="$class" />
						<xsl:with-param name="requestparameters" select="../../requestparameters" />
					</xsl:call-template>
					
					<xsl:apply-templates select="../../ValidationErrors/validationError[fieldName = $daysID]" mode="textfield"/>
				</div>
			</td>
		</tr>
		
	</xsl:template>

	<xsl:template name="addTagsTable">
		<div class="floatleft margintop full">

			<p><xsl:value-of select="$i18n.TagsTable.description"/></p>
		
			<table class="full border">
				<tr>
					<th>
						<xsl:value-of select="$i18n.Tag"/>
					</th>
					<th>
						<xsl:value-of select="$i18n.Description"/>
					</th>
				</tr>
				
				<tr>
					<td>
						<xsl:text>$autodeletion.days</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.AutodeletionDaysTag"/>
					</td>
				</tr>
				
			</table>
		</div>
	</xsl:template>
		
	<xsl:template match="validationError[validationErrorType = 'RequiredField']" mode="textfield">
		
		<i data-icon-after="!" title="{$i18n.Validation.Inline.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'TooLong']" mode="textfield">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.Validation.Inline.TooLongFieldContent.part1"/>
				<xsl:value-of select="currentLength"/>
				<xsl:value-of select="$i18n.Validation.Inline.TooLongFieldContent.part2"/>
				<xsl:value-of select="maxLength"/>
				<xsl:value-of select="$i18n.Validation.Inline.TooLongFieldContent.part3"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']" mode="textfield">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:choose>
					<xsl:when test="invalidFormatMessage">
						<xsl:value-of select="invalidFormatMessage"/>
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="$i18n.Validation.Inline.InvalidFormat"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError" mode="textfield">
		
		<i data-icon-after="!" title="{$i18n.Validation.Inline.UnknownValidationError}"></i>
		
	</xsl:template>
		
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="fieldName = 'confirm'">
						<xsl:value-of select="$i18n.AutomaticCulling.confirm" />
					</xsl:when>
					<xsl:when test="fieldName = 'customSavedCullingMessage'">
						<xsl:value-of select="$i18n.AutomaticCulling.customSavedCullingMessage" />
					</xsl:when>
					<xsl:when test="fieldName = 'comment'">
						<xsl:value-of select="$i18n.CullingComment" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:choose>
					<xsl:when test="messageKey='DummyKey'">
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="$i18n.validation.unknownFault" />
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>