package com.nordicpeak.flowengine.flowfinder;

import java.io.IOException;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLinkProvider;
import se.unlogic.hierarchy.core.utils.xml.UserAttributeElementableListener;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.enums.ContentType;


public class FlowFinderModule extends AnnotatedForegroundModule {

	private static final UserAttributeElementableListener ELEMENTABLE_LISTENER = new UserAttributeElementableListener();
	
	@InstanceManagerDependency(required=true)
	protected FlowInstanceAdminModule flowInstanceAdminModule;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable flow test module support", description = "Controls if support for the manager flow test module should be enabled")
	private boolean enableFlowTestModuleSupport = false;
	
	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Document doc = this.createDocument(req, uriParser);
		Element searchFlowInstance = doc.createElement("SearchFlowInstance");
		doc.getDocumentElement().appendChild(searchFlowInstance);
		
		if(enableFlowTestModuleSupport) {
			
			for(ExtensionLinkProvider extensionLinkProvider : this.flowInstanceAdminModule.getListOverviewExtensionLinkProviders()) {
				
				if(extensionLinkProvider.getClass().getName().equals("com.nordicpeak.flowengine.managerflowtest.ManagerFlowTestModule") && AccessUtils.checkAccess(user, extensionLinkProvider.getAccessInterface())) {
					
					searchFlowInstance.appendChild(extensionLinkProvider.getExtensionLink(user).toXML(doc));
				}
			}
		}
		
		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), this.getDefaultBreadcrumb());
	}
	
	@WebPublic(alias = "search")
	public ForegroundModuleResponse search(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws IOException {

		flowInstanceAdminModule.search(req, res, user, false, false);

		return null;
	}

	@WebPublic(alias = "overview")
	public ForegroundModuleResponse showFlowInstanceOverview(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException,URINotFoundException {
		
		Integer flowInstanceID;
		FlowInstance flowInstance;
		
		if (uriParser.size() == 3 && (flowInstanceID = uriParser.getInt(2)) != null && (flowInstance = flowInstanceAdminModule.getFlowInstance(flowInstanceID, null, FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION, FlowInstance.STATUS_RELATION, FlowInstance.FLOW_RELATION, FlowInstance.EVENTS_RELATION, Flow.FLOW_FAMILY_RELATION, FlowInstance.ATTRIBUTES_RELATION)) != null && !flowInstance.getStatus().getContentType().equals(ContentType.NEW)) {
			
			Document doc = this.createDocument(req, uriParser);
			Element showFlowInstanceOverviewElement = doc.createElement("ShowFlowInstanceOverview");
			doc.getDocumentElement().appendChild(showFlowInstanceOverviewElement);
			
			XMLGeneratorDocument generatorDocument = new XMLGeneratorDocument(doc);
			generatorDocument.addAssignableRootElementableListener(User.class, ELEMENTABLE_LISTENER);
			
			showFlowInstanceOverviewElement.appendChild(flowInstance.toXML(generatorDocument));

			return new SimpleForegroundModuleResponse(doc, flowInstance.getFlow().getName(), this.getDefaultBreadcrumb());
		}
		
		throw new URINotFoundException(uriParser);
	}
	
	@WebPublic(alias = "icon")
	public ForegroundModuleResponse getFlowIcon(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException{
		
		return flowInstanceAdminModule.getFlowIcon(req, res, user, uriParser);
	}
	
	public Document createDocument(HttpServletRequest req, URIParser uriParser) {

		Document doc = XMLUtils.createDomDocument();
		Element documentElement = doc.createElement("Document");
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));
		doc.appendChild(documentElement);
		
		return doc;
	}
}
