package com.nordicpeak.flowengine.documentsigner.beans;

import java.sql.Timestamp;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.string.StringTag;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.documentsigner.enums.SignerType;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;

@Table(name = "document_signer_document_signers")
@XMLElement
public class DocumentSigner extends GeneratedElementable {

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer signerID;

	@DAOManaged(columnName = "documentID")
	@ManyToOne
	@XMLElement
	private SigningDocument document;

	@DAOManaged
	@XMLElement
	private SignerType type;

	@DAOManaged
	@OrderBy
	@XMLElement
	private int sortIndex;

	@DAOManaged(columnName = "userID", dontUpdateIfNull = true)
	@XMLElement
	private User user;

	@DAOManaged
	private String citizenID;

	@DAOManaged
	@XMLElement
	private String unsecureKey; // unsecureKey = RandomUtils.getRandomString(16, RandomUtils.MIXED_CASE_CHARACTERS_AND_DIGITS);

	@DAOManaged
	private String firstname;

	@DAOManaged
	private String lastname;

	@DAOManaged
	private String email;

	@DAOManaged
	private String mobilePhone;

	@DAOManaged
	@XMLElement
	private Timestamp signedDate;

	@DAOManaged
	@XMLElement
	private String signingChecksum;

	@DAOManaged
	private String signatureData;

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);
		
		if (user != null && type != SignerType.EXTERNAL_USER) {
			
			XMLUtils.appendNewElement(doc, element, "email", user.getEmail());
			XMLUtils.appendNewElement(doc, element, "mobilePhone", user.getAttributeHandler().getString("mobilePhone"));
			
		} else {
			
			XMLUtils.appendNewElement(doc, element, "email", email);
			XMLUtils.appendNewElement(doc, element, "mobilePhone", mobilePhone);
		}

		if (user != null) {

			XMLUtils.appendNewElement(doc, element, "firstname", user.getFirstname());
			XMLUtils.appendNewElement(doc, element, "lastname", user.getLastname());
			XMLUtils.appendNewElement(doc, element, "citizenID", CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user));

		} else {

			XMLUtils.appendNewElement(doc, element, "firstname", firstname);
			XMLUtils.appendNewElement(doc, element, "lastname", lastname);
			XMLUtils.appendNewElement(doc, element, "citizenID", citizenID);
		}

		return element;
	}

	public Integer getSignerID() {

		return signerID;
	}

	public void setSignerID(Integer signerID) {

		this.signerID = signerID;
	}

	public SigningDocument getDocument() {

		return document;
	}

	public void setDocument(SigningDocument document) {

		this.document = document;
	}

	public SignerType getType() {

		return type;
	}

	public void setType(SignerType type) {

		this.type = type;
	}

	public User getUser() {

		return user;
	}

	public int getSortIndex() {

		return sortIndex;
	}

	public void setSortIndex(int sortIndex) {

		this.sortIndex = sortIndex;
	}

	public void setUser(User user) {

		this.user = user;
	}

	public String getCitizenID() {

		return citizenID;
	}

	public void setCitizenID(String citizenID) {

		this.citizenID = citizenID;
	}

	public String getUnsecureKey() {

		return unsecureKey;
	}

	public void setUnsecureKey(String unsecureKey) {

		this.unsecureKey = unsecureKey;
	}

	@StringTag(name = "firstname")
	public String getFirstname() {

		if (user != null) {
			return user.getFirstname();
		}

		return firstname;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	@StringTag(name = "lastname")
	public String getLastname() {

		if (user != null) {
			return user.getLastname();
		}

		return lastname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public String getEmail() {

		if (user != null) {
			return user.getEmail();
		}

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getMobilePhone() {

		if (user != null) {
			return user.getAttributeHandler().getString("mobilePhone");
		}

		return mobilePhone;
	}

	public void setMobilePhone(String mobilePhone) {

		this.mobilePhone = mobilePhone;
	}

	public Timestamp getSignedDate() {

		return signedDate;
	}

	public void setSignedDate(Timestamp signedDate) {

		this.signedDate = signedDate;
	}

	public String getSigningChecksum() {

		return signingChecksum;
	}

	public void setSigningChecksum(String signingChecksum) {

		this.signingChecksum = signingChecksum;
	}

	public String getSignatureData() {

		return signatureData;
	}

	public void setSignatureData(String signatureData) {

		this.signatureData = signatureData;
	}

	@Override
	public String toString() {

		return getClass().getSimpleName() + " (signerID=" + signerID + ", document=" + document + ", type=" + type + ")";
	}

	public Contact getContact() {

		Contact contact = new Contact();

		contact.setFirstname(getFirstname());
		contact.setLastname(getLastname());
		contact.setEmail(getEmail());
		contact.setMobilePhone(getMobilePhone());

		return contact;
	}

	public boolean isUnsecure() {

		return unsecureKey != null;
	}

}
