package com.nordicpeak.flowengine.documentsigner.beans;

import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.io.BinarySizeFormater;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringTag;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.documentsigner.enums.SigningDocumentStatus;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;

@Table(name = "document_signer_documents")
@XMLElement(name = "Document")
public class SigningDocument extends GeneratedElementable implements Comparable<SigningDocument> {

	public static final Field SIGNERS_RELATION = ReflectionUtils.getField(SigningDocument.class, "signers");

	@DAOManaged(autoGenerated = true)
	@Key
	@StringTag
	@XMLElement
	private Integer documentID;

	@DAOManaged
	@StringTag
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String name;
	
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String description;

	@DAOManaged
	@XMLElement
	private String filename;

	@DAOManaged
	@XMLElement
	private long filesize;

	@DAOManaged(columnName = "posterID", dontUpdateIfNull = true)
	private User poster;

	@DAOManaged
	@XMLElement
	private Timestamp added;

	@DAOManaged
	@XMLElement
	private Timestamp startDate;

	@DAOManaged
	@XMLElement
	private Timestamp completedDate;

	@DAOManaged
	@XMLElement
	private Integer profileID;

	@DAOManaged
	@XMLElement
	private SigningDocumentStatus status;
	
	@DAOManaged
	@XMLElement
	private String hashAlgorithm;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class, required = true)
	@XMLElement
	private Integer cancelInXDays;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class, required = true)
	@XMLElement
	private Integer sendReminderInXDays;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class, required = true)
	@XMLElement
	private Integer deleteAfterXDays;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useSequentialSigning;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showPreviousSignaturesToSigners;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showSignedDocumentInUserModule;

	@DAOManaged
	@XMLElement
	private boolean reminderSent;
	
	@DAOManaged
	@XMLElement
	private boolean expiringSent;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean publishOriginal;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<DocumentSigner> signers;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendEmailWhenSigningPartiallyDone;

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		XMLUtils.appendNewElement(doc, element, "FormattedFilesize", BinarySizeFormater.getFormatedSize(filesize));
		XMLUtils.appendNewElement(doc, element, "FormattedHashAlgorithm", hashAlgorithm);

		if (poster != null) {

			Element posterElement = poster.toXML(doc);
			posterElement = (Element) doc.renameNode(posterElement, null, "poster");

			AttributeHandler attributeHandler = poster.getAttributeHandler();

			if (attributeHandler != null && !attributeHandler.isEmpty()) {

				posterElement.appendChild(attributeHandler.toXML(doc));
			}

			element.appendChild(posterElement);
		}

		return element;
	}

	public Integer getDocumentID() {

		return documentID;
	}

	public void setDocumentID(Integer documentID) {

		this.documentID = documentID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getFilename() {

		return filename;
	}

	public void setFilename(String filename) {

		this.filename = filename;
	}

	public long getFilesize() {

		return filesize;
	}

	public void setFilesize(long filesize) {

		this.filesize = filesize;
	}

	public User getPoster() {

		return poster;
	}

	public void setPoster(User poster) {

		this.poster = poster;
	}

	public Timestamp getAdded() {

		return added;
	}

	public void setAdded(Timestamp added) {

		this.added = added;
	}

	public Timestamp getStartDate() {

		return startDate;
	}

	public void setStartDate(Timestamp startDate) {

		this.startDate = startDate;
	}

	public Timestamp getCompletedDate() {

		return completedDate;
	}

	public void setCompletedDate(Timestamp completedDate) {

		this.completedDate = completedDate;
	}

	public Integer getProfileID() {

		return profileID;
	}

	public void setProfileID(Integer profileID) {

		this.profileID = profileID;
	}

	public SigningDocumentStatus getStatus() {

		return status;
	}

	public void setStatus(SigningDocumentStatus status) {

		this.status = status;
	}

	public Integer getCancelInXDays() {

		return cancelInXDays;
	}

	public void setCancelInXDays(Integer cancelInXDays) {

		this.cancelInXDays = cancelInXDays;
	}

	public Integer getSendReminderInXDays() {

		return sendReminderInXDays;
	}

	public void setSendReminderInXDays(Integer sendReminderInXDays) {

		this.sendReminderInXDays = sendReminderInXDays;
	}

	public Integer getDeleteAfterXDays() {

		return deleteAfterXDays;
	}

	public void setDeleteAfterXDays(Integer deleteAfterXDays) {

		this.deleteAfterXDays = deleteAfterXDays;
	}

	public boolean isUseSequentialSigning() {

		return useSequentialSigning;
	}

	public void setUseSequentialSigning(boolean useSequentialSigning) {

		this.useSequentialSigning = useSequentialSigning;
	}

	public boolean isShowPreviousSignaturesToSigners() {

		return showPreviousSignaturesToSigners;
	}

	public void setShowPreviousSignaturesToSigners(boolean showPreviousSignaturesToSigners) {

		this.showPreviousSignaturesToSigners = showPreviousSignaturesToSigners;
	}

	public boolean isShowSignedDocumentInUserModule() {

		return showSignedDocumentInUserModule;
	}

	public void setShowSignedDocumentInUserModule(boolean showSignedDocumentInUserModule) {

		this.showSignedDocumentInUserModule = showSignedDocumentInUserModule;
	}

	public List<DocumentSigner> getSigners() {

		return signers;
	}

	public void setSigners(List<DocumentSigner> signers) {

		this.signers = signers;
	}

	
	public boolean sendEmailWhenSigningPartiallyDone() {
	
		return sendEmailWhenSigningPartiallyDone;
	}

	
	public void setSendEmailWhenSigningDone(boolean sendEmailWhenSigningDone) {
	
		this.sendEmailWhenSigningPartiallyDone = sendEmailWhenSigningDone;
	}

	@Override
	public String toString() {

		return StringUtils.toLogFormat(name, 50) + " (documentID:" + documentID + ")";
	}

	public Contact getPosterContact() {

		Contact contact = new Contact();

		contact.setFirstname(poster.getFirstname());
		contact.setLastname(poster.getLastname());
		contact.setEmail(poster.getEmail());
		contact.setMobilePhone(poster.getAttributeHandler().getString("mobilePhone"));

		return contact;
	}

	public boolean isFullySigned() {

		for (DocumentSigner signer : signers) {

			if (signer.getSignedDate() == null) {
				return false;
			}
		}

		return true;
	}

	@Override
	public int compareTo(SigningDocument o) {

		int diff = status.compareTo(o.getStatus());

		if (diff == 0) {
			diff = -added.compareTo(o.getAdded());
		}

		return diff;
	}

	public boolean isReminderSent() {
		return reminderSent;
	}

	public void setReminderSent(boolean reminderSent) {
		this.reminderSent = reminderSent;
	}

	public boolean isExpiringSent() {
		return expiringSent;
	}

	public void setExpiringSent(boolean expiringSent) {
		this.expiringSent = expiringSent;
	}

	public boolean isPublishOriginal() {
		return publishOriginal;
	}

	public void setPublishOriginal(boolean publishOriginal) {
		this.publishOriginal = publishOriginal;
	}

	public DocumentSigner getSigner(User user, String unsecureKey) {

		for (DocumentSigner signer : signers) {

			if (signer.getUser() != null) {

				if (signer.getUser().equals(user)) {
					return signer;
				}

			} else if (signer.isUnsecure()) {

				if (signer.getUnsecureKey().equals(unsecureKey)) {
					return signer;
				}

			} else if (signer.getCitizenID().equals(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user))) {

				return signer;
			}
		}

		return null;
	}

	public String getHashAlgorithm() {
		return hashAlgorithm;
	}

	public void setHashAlgorithm(String hashAlgorithm) {
		this.hashAlgorithm = hashAlgorithm;
	}

}
