<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:template match="Document">	
		
		<div id="DynamicTableQueryProvider" class="contentitem">
		
			<xsl:apply-templates select="validationError"/>
			<xsl:apply-templates select="ShowDynamicTableQuery"/>
			<xsl:apply-templates select="UpdateDynamicTableQuery"/>
			<xsl:apply-templates select="AddDynamicTableColumn"/>
			<xsl:apply-templates select="UpdateDynamicTableColumn"/>
			<xsl:apply-templates select="SortColumns"/>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="AddDynamicTableColumn">
		<xsl:call-template name="AddTableColumn"/>
	</xsl:template>
	
	<xsl:template match="UpdateDynamicTableColumn">
		<xsl:call-template name="UpdateTableColumn"/>
	</xsl:template>
	
	<xsl:template match="ShowDynamicTableQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateQuery" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="DynamicTableQuery/QueryDescriptor/name" /></h1>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.BaseInfo"/></legend>
			
			<div class="floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatequery/{DynamicTableQuery/queryID}" title="{$i18n.UpdateBaseInformation}">
					<img class="alignbottom" src="{$imagePath}/pen.png"/>
				</a>
			</div>
			
			<xsl:call-template name="createCommonShowFields">
				<xsl:with-param name="element" select="DynamicTableQuery" />
			</xsl:call-template>
			
			<div class="clearboth floatleft bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.MaxRows" /></label>
				<div class="floatleft full">
					<xsl:value-of select="DynamicTableQuery/maxRows" />
				</div>
			</div>
			
			<div class="clearboth floatleft bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.Query.decimalsToShow.show" /></label>
				<div class="floatleft full">
					<xsl:choose>
						<xsl:when test="DynamicTableQuery/decimalsToShow">
							<xsl:value-of select="DynamicTableQuery/decimalsToShow" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.Query.decimalsToShow.show.automatic" />
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
			<div class="clearboth floatleft bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.HideTitle" /></label>
				<div class="floatleft full">
					<xsl:choose>
						<xsl:when test="DynamicTableQuery/hideTitle = 'true'">
							<xsl:value-of select="$i18n.Yes" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.No" />
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Columns"/></legend>
			
			<xsl:if test="DynamicTableQuery/Columns/Column">
				
				<div class="floatleft full">
				
					<xsl:apply-templates select="DynamicTableQuery/Columns/Column" mode="list" />
			
				</div>
			
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sortcolumns/{DynamicTableQuery/queryID}" title="{$i18n.SortColumns.Title}">
						<xsl:value-of select="$i18n.SortColumns.Title"/>
						<img class="alignbottom marginleft" src="{$commonImagePath}/move.png"/>
					</a>
				</div>
				
			</xsl:if>
			
			<div class="clearboth floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addcolumn/{DynamicTableQuery/queryID}" title="{$i18n.AddColumn}">
					<xsl:value-of select="$i18n.AddColumn"/>
					<img class="alignbottom marginleft" src="{$commonImagePath}/add.png"/>
				</a>
			</div>
			
		</fieldset>
		
		<div class="floatright margintop clearboth">
			<input type="button" value="{$i18n.Done}" onclick="window.location = '{showFlowURL}'" />
		</div>
		
	</xsl:template>
	
	<xsl:template match="UpdateDynamicTableQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateBaseInformation" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="updateDynamicTableQueryForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="DynamicTableQuery" />
				<xsl:with-param name="flowTypeID" select="FlowType/flowTypeID" />
			</xsl:call-template>
			
			<div class="floatleft full bigmarginbottom">
				<label for="maxRows" class="floatleft clearboth"><xsl:value-of select="$i18n.MaxRows" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'maxRows'"/>
						<xsl:with-param name="name" select="'maxRows'"/>
						<xsl:with-param name="title" select="$i18n.MaxRows"/>
						<xsl:with-param name="element" select="DynamicTableQuery" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="decimalsToShow" class="floatleft clearboth"><xsl:value-of select="$i18n.Query.decimalsToShow.update" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'decimalsToShow'"/>
						<xsl:with-param name="name" select="'decimalsToShow'"/>
						<xsl:with-param name="title" select="$i18n.Query.decimalsToShow.update"/>
						<xsl:with-param name="element" select="DynamicTableQuery" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'hideTitle'" />
						<xsl:with-param name="name" select="'hideTitle'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="DynamicTableQuery" />
					</xsl:call-template>
					
					<label for="hideTitle">
						<xsl:value-of select="$i18n.HideTitle" />
					</label>
				</div>

			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
		
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'UpdateFailedDynamicTableQueryNotFound']">
		
		<p class="error">
			<xsl:value-of select="$i18n.TableQueryNotFound" />
		</p>
		
	</xsl:template>

	<xsl:template name="fieldName">
		<xsl:param name="fieldName"/>
	
		<xsl:choose>
			<xsl:when test="$fieldName = 'maxRows'">
				<xsl:value-of select="$i18n.MaxRows" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

</xsl:stylesheet>