<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:variable name="scripts">
		/common/js/query.js
		/js/jquery.ui.datepicker-sv.js
		/js/tablequery.js?v=1
		/js/dynamictablequery.js?v=1
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/tablequery.css?v=2
		/css/dynamictablequery.css
	</xsl:variable>
	
	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
		
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="DynamicTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article>
				
				<div class="heading-wrapper tablequery-heading">
				
					<xsl:if test="not(DynamicTableQueryInstance/DynamicTableQuery/hideTitle = 'true')">
						<h2>
						<xsl:attribute name="class">
							<xsl:if test="DynamicTableQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="DynamicTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					</xsl:if>	
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="DynamicTableQueryInstance/DynamicTableQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="DynamicTableQueryInstance/DynamicTableQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<table class="tablequery full">
				
					<thead>
					
						<tr>
							
							<xsl:apply-templates select="DynamicTableQueryInstance/DynamicTableQuery/Columns/Column" mode="thead-show" />
							
						</tr>
						
					</thead>
					
					<tbody>
					
						<xsl:apply-templates select="DynamicTableQueryInstance/Rows/Row" mode="tbody-show" />
						
					</tbody>
					
					<tfoot>
					
						<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/Columns/Column[summarise = 'true']">
							<tr data-title="{$i18n.Summary}">
							
								<xsl:for-each select="DynamicTableQueryInstance/DynamicTableQuery/Columns/Column">
									
									<xsl:variable name="columnIndex"><xsl:number/></xsl:variable>
									
									<xsl:choose>
										<xsl:when test="$columnIndex = 1 and not(summarise = 'true')">
										
											<td class="padding">
												<strong class="floatleft"><xsl:value-of select="$i18n.Summary"/></strong>
											</td>
										
										</xsl:when>
										<xsl:when test="$columnIndex = 1 and summarise = 'true'">
										
											<td class="padding" data-summarise="{summarise}" data-title="{label}">
												<strong class="floatleft"><xsl:value-of select="$i18n.Summary"/></strong>
												
												<xsl:if test="summarise = 'true'">
													<span class="floatright">
														<xsl:value-of select="sum"/>
													</span>
												</xsl:if>
											</td>
										
										</xsl:when>
										<xsl:otherwise>
										
											<td class="padding" data-summarise="{summarise}" data-title="{label}">
												<xsl:if test="summarise = 'true'">
													<span class="floatright">
														<xsl:value-of select="sum"/>
													</span>
												</xsl:if>
											</td>
											
										</xsl:otherwise>
									</xsl:choose>
									
								</xsl:for-each>
								
							</tr>
						</xsl:if>
						
					</tfoot>
					
				</table>
				
			</article>
			
		</div>
		
	</xsl:template>		
		
	<xsl:template match="ShowQueryForm">
	
		<script type="text/javascript">DynamicTableQuery = {"i18nAddRow": "<xsl:value-of select="$i18n.AddRow"/>", "i18nMaxRowsReached": "<xsl:value-of select="$i18n.MaxRowsReached"/>"};</script>
		
		<xsl:variable name="queryID" select="concat('query_', DynamicTableQueryInstance/DynamicTableQuery/queryID)" />
		<xsl:variable name="qID" select="concat('q', DynamicTableQueryInstance/DynamicTableQuery/queryID)" />
		
		<div class="query dynamictablequery tablequery" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query dynamictablequery tablequery</xsl:text>
				<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="DynamicTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<xsl:if test="EnableAjaxPosting">
				<xsl:call-template name="loadAjaxValidationErrors"/>
			</xsl:if>
			
			<a name="{$queryID}" />
			
			<input type="hidden" name="summaryURL" disabled="disabled" value="{../ModuleURL}/getcolumnsum"/>
			<input type="hidden" name="rowsumURL" disabled="disabled" value="{../ModuleURL}/getrowsum"/>
			
			<xsl:if test="ValidationErrors/validationError">
			
				<div id="{$queryID}-validationerrors" class="validationerrors">
			
					<div class="info-box error">
							
						<xsl:variable name="fieldValidationErrors" select="ValidationErrors/validationError[fieldName]"/>
						
						<xsl:choose>
							<xsl:when test="$fieldValidationErrors">
								
								<xsl:call-template name="fieldValidationErrorsMessage" />
								
							</xsl:when>
							<xsl:otherwise>
								
								<span />
								
							</xsl:otherwise>
						</xsl:choose>
						<div class="marker"></div>
					</div>
			
				</div>
			
			</xsl:if>
			
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper tablequery-heading">
					
					<xsl:if test="not(DynamicTableQueryInstance/DynamicTableQuery/hideTitle = 'true')">
						<h2 id="{$queryID}-title">
							<xsl:attribute name="class">
								<xsl:if test="DynamicTableQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="DynamicTableQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
					
					<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/helpText">
						<xsl:apply-templates select="DynamicTableQueryInstance/DynamicTableQuery/helpText" />
					</xsl:if>
					
				</div>
				
				<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="DynamicTableQueryInstance/DynamicTableQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>				
				
				<table aria-describedby="{$queryID}-title">
				
					<xsl:attribute name="class">
						<xsl:text>tablequery dynamictablequery full</xsl:text>
						
						<xsl:if test="DynamicTableQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
							<xsl:text> required</xsl:text>
						</xsl:if>
					</xsl:attribute>
					
					<thead>
					
						<tr>
						
							<xsl:apply-templates select="DynamicTableQueryInstance/DynamicTableQuery/Columns/Column" mode="thead" />
							
							<th width="85px"/>
						
						</tr>
						
					</thead>
					
					<tbody id="{$queryID}_maintable" data-maxrows="{DynamicTableQueryInstance/DynamicTableQuery/maxRows}">
						<xsl:variable name="rowParam">
							<xsl:value-of select="$qID"/>
							<xsl:value-of select="'-row'"/>
						</xsl:variable>
					
						<xsl:choose>
							<xsl:when test="requestparameters/parameter[name=$rowParam]">
								
								<xsl:for-each select="requestparameters/parameter[name=$rowParam]/value">
								
									<xsl:call-template name="Row-tbody" >
										<xsl:with-param name="rowID" select="."/>
										<xsl:with-param name="queryID" select="$qID"/>
										<xsl:with-param name="columns" select="../../../DynamicTableQueryInstance/DynamicTableQuery/Columns"/>
									</xsl:call-template>
									
								</xsl:for-each>
								
							</xsl:when>
							<xsl:when test="DynamicTableQueryInstance/Rows">
								
								<xsl:apply-templates select="DynamicTableQueryInstance/Rows/Row" mode="tbody" >
									<xsl:with-param name="queryID" select="$qID"/>
								</xsl:apply-templates>
								
							</xsl:when>
							<xsl:otherwise>
							
									<xsl:call-template name="Row-tbody" >
										<xsl:with-param name="rowID" select="'newRow'"/>
										<xsl:with-param name="queryID" select="$qID"/>
									</xsl:call-template>
							
							</xsl:otherwise>
						</xsl:choose>
						
					</tbody>
					
					<tfoot>
					
						<xsl:if test="DynamicTableQueryInstance/DynamicTableQuery/Columns/Column[summarise = 'true']">
							<tr data-title="{$i18n.Summary}">
							
								<xsl:for-each select="DynamicTableQueryInstance/DynamicTableQuery/Columns/Column">
									
									<xsl:variable name="columnIndex"><xsl:number/></xsl:variable>
									
									<xsl:variable name="columnIndex2">
										<xsl:value-of select="$columnIndex - 1"/>
									</xsl:variable>
									
									<xsl:choose>
										<xsl:when test="$columnIndex = 1 and not(summarise = 'true')">
										
											<td class="padding">
												<strong class="floatleft"><xsl:value-of select="$i18n.Summary"/></strong>
											</td>
										
										</xsl:when>
										<xsl:when test="$columnIndex = 1 and summarise = 'true'">
										
											<td class="padding bigpaddingright text-align-right" id="summary-{columnID}" data-summarise="{summarise}" data-columnindex="{$columnIndex2}" data-title="{label}">
												<strong class="floatleft"><xsl:value-of select="$i18n.Summary"/></strong>
												
												<xsl:if test="summarise = 'true'">
													<span>
														<xsl:text>0</xsl:text>
													</span>
												</xsl:if>
											</td>
										
										</xsl:when>
										<xsl:otherwise>
										
											<td class="padding bigpaddingright text-align-right" id="summary-{columnID}" data-summarise="{summarise}" data-columnindex="{$columnIndex2}" data-title="{label}">
												<xsl:if test="summarise = 'true'">
													<span>
														<xsl:text>0</xsl:text>
													</span>
												</xsl:if>
											</td>
											
										</xsl:otherwise>
									</xsl:choose>
									
								</xsl:for-each>
								
								</tr>
							</xsl:if>
							
					</tfoot>
					
				</table>
				
				<div class="newDynamicTableRow">
					<a class="btn btn-green margintop floatright" href="#">
						<xsl:value-of select="$i18n.AddRow"/>
					</a>
				</div>
				
				<table style="display: none;">
					<tbody id="{$queryID}-rowTemplate" class="tableRowTemplate">
					
					<xsl:call-template name="Row-tbody" >
						<xsl:with-param name="rowID" select="'template'"/>
						<xsl:with-param name="queryID" select="$qID"/>
					</xsl:call-template>
					
					</tbody>
				</table>
				
			</article>
			
		</div>
		
		<script type="text/javascript">
			$(function(){
				initDynamicTableQuery(
					'<xsl:value-of select="DynamicTableQueryInstance/DynamicTableQuery/queryID" />',
					'<xsl:value-of select="queryRequestURL" />'
				);
			});
		</script>
		
	</xsl:template>
	
	<xsl:template name="Row-tbody">
		<xsl:param name="rowID" select="null"/>
		<xsl:param name="queryID" select="null"/>
		<xsl:param name="columns" select="DynamicTableQueryInstance/DynamicTableQuery/Columns"/>
		
		<tr>
			
			<xsl:apply-templates select="$columns/Column" mode="tbody" >
				<xsl:with-param name="rowID" select="$rowID"/>
			</xsl:apply-templates>
			
			<td>
				<a class="btn btn-red vertical-align-middle" href="javascript:;" onclick="removeTableRow(this);">
					<xsl:value-of select="$i18n.DeleteRow"/>
				</a>
				
				<input name="{$queryID}-row" value="{$rowID}" style="display: none;" />
			</td>
			
		</tr>
		
	</xsl:template>
	
	<xsl:template match="Row" mode="tbody">
		<xsl:param name="queryID" select="null"/>
		<xsl:param name="columns" select="../../DynamicTableQuery/Columns"/>
		
		<tr>
			
			<xsl:apply-templates select="$columns/Column" mode="tbody" >
				<xsl:with-param name="rowID" select="rowID"/>
			</xsl:apply-templates>
			
			<td>
				<a class="btn btn-red vertical-align-middle" href="javascript:;" onclick="removeTableRow(this);">
					<xsl:value-of select="$i18n.DeleteRow"/>
				</a>
				
				<input name="{$queryID}-row" value="{rowID}" style="display: none;" />
			</td>
			
		</tr>
		
	</xsl:template>
	
	<xsl:template match="Row" mode="tbody-show">
		<xsl:param name="columns" select="../../DynamicTableQuery/Columns"/>
		
		<tr>
			
			<xsl:apply-templates select="$columns/Column" mode="tbody-show" >
				<xsl:with-param name="rowID" select="rowID"/>
			</xsl:apply-templates>
			
		</tr>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'NoRows']">
		
		<xsl:value-of select="$i18n.NoRows" />
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TooManyRows']">
		
		<xsl:value-of select="$i18n.TooManyRows" />
		
	</xsl:template>
	
</xsl:stylesheet>