package com.nordicpeak.flowengine.easitgetitemsalternativelist;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.List;

import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeList;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;

@Table(name = "easit_alternative_list_sources")
@XMLElement(name = "AlternativeList")
public class ListSource extends BaseAlternativeList<ListSource, ListSourceEvent> implements Serializable {

	public static final Field EVENTS_RELATION = ReflectionUtils.getField(ListSource.class, "events");

	private static final long serialVersionUID = 3296400205039536096L;

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement(name = "listID")
	private Integer sourceID;

	@DAOManaged
	@OrderBy
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String viewIdentifier;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String idProperty;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String nameProperty;

	@DAOManaged(columnName = "endPointID")
	@ManyToOne(autoGet = true)
	@XMLElement
	private EasitGetItemsEndpoint endPoint;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer cacheRefreshTime;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean keepOldListOnRefreshFail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setIdAsAttribute;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ListSourceEvent> events;

	public Integer getSourceID() {

		return sourceID;
	}

	public void setSourceID(Integer sourceID) {

		this.sourceID = sourceID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getViewIdentifier() {

		return viewIdentifier;
	}

	public void setViewIdentifier(String viewIdentifier) {

		this.viewIdentifier = viewIdentifier;
	}

	public String getIdProperty() {

		return idProperty;
	}

	public void setIdProperty(String idProperty) {

		this.idProperty = idProperty;
	}

	public String getNameProperty() {

		return nameProperty;
	}

	public void setNameProperty(String nameProperty) {

		this.nameProperty = nameProperty;
	}

	// In minutes
	public Integer getCacheRefreshTime() {

		return cacheRefreshTime;
	}

	public void setCacheRefreshTime(Integer cacheRefreshTime) {

		this.cacheRefreshTime = cacheRefreshTime;
	}

	@Override
	public String toString() {

		return StringUtils.toLogFormat(name, 30) + " (ID: " + sourceID + ", Endpoint: " + endPoint + ")";
	}

	@Override
	public String getListName() {

		return name;
	}

	@Override
	public Integer getListID() {

		return sourceID;
	}

	@Override
	public List<ListSourceEvent> getEvents() {

		return events;
	}

	@Override
	public void setEvents(List<ListSourceEvent> events) {

		this.events = events;
	}

	public boolean isKeepOldListOnRefreshFail() {

		return keepOldListOnRefreshFail;
	}

	public void setKeepOldListOnRefreshFail(boolean keepOldListOnRefreshFail) {

		this.keepOldListOnRefreshFail = keepOldListOnRefreshFail;
	}

	public boolean isSetIdAsAttribute() {

		return setIdAsAttribute;
	}

	public void setSetIdAsAttribute(boolean setIdAsAttribute) {

		this.setIdAsAttribute = setIdAsAttribute;
	}

	public EasitGetItemsEndpoint getEndPoint() {

		return endPoint;
	}

	public void setEndPoint(EasitGetItemsEndpoint endPoint) {

		this.endPoint = endPoint;
	}
}
