package com.nordicpeak.flowengine.easitgetitemsalternativelist;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeListCRUD;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.utils.crud.IntegerBeanIDParser;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

public class ListSourceCRUD extends BaseAlternativeListCRUD<ListSource, EasitGetItemsAlternativeListInstance, ListSourceCRUD, EasitGetItemsAlternativeListProviderModule, ListSourceEvent> {

	public ListSourceCRUD(AnnotatedDAOWrapper<ListSource, Integer> queryDAO, BeanRequestPopulator<ListSource> populator, EasitGetItemsAlternativeListProviderModule callback) {

		super(IntegerBeanIDParser.getInstance(), queryDAO, populator, "AlternativeList", "list source", "/", callback);
	}

	@Override
	protected void appendShowFormData(ListSource source, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {

		super.appendShowFormData(source, doc, showTypeElement, user, req, res, uriParser);

		EasitGetItemsAlternativeListInstance instance = callback.getAlternativeInstance(source, null, null);

		if (instance != null && !CollectionUtils.isEmpty(instance.getAlternatives())) {

			XMLUtils.append(doc, showTypeElement, "Alternatives", instance.getAlternatives());
		}
	}

	@Override
	protected void appendListFormData(Document doc, Element listTypeElement, User user, HttpServletRequest req, URIParser uriParser, List<ValidationError> validationError) throws SQLException {

		super.appendListFormData(doc, listTypeElement, user, req, uriParser, validationError);

		Collection<EasitGetItemsAlternativeListInstance> alternativeInstances = callback.getAlternativeInstances();

		if (!CollectionUtils.isEmpty(alternativeInstances)) {
			for (AlternativeListInstance alternativeList : alternativeInstances) {

				if (alternativeList.getAlternatives() != null) {

					Element sizeElement = XMLUtils.appendNewElement(doc, listTypeElement, "ListSize");

					XMLUtils.appendNewElement(doc, sizeElement, "Name", alternativeList.getDescriptor().getName());
					XMLUtils.appendNewElement(doc, sizeElement, "Value", alternativeList.getAlternatives().size());
				}
			}
		}
	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		appendFormData(null, doc, addTypeElement, user, req, uriParser);
	}

	@Override
	protected void appendUpdateFormData(ListSource bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		appendFormData(bean, doc, updateTypeElement, user, req, uriParser);
	}

	protected void appendFormData(ListSource bean, Document doc, Element element, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		XMLUtils.append(doc, element, "Endpoints", callback.getEndPointDAO().getAll());

	}

	@Override
	protected void validateAddPopulation(ListSource source, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateAddPopulation(source, req, user, uriParser);
		validate(source, req, user, uriParser);
	}

	@Override
	protected void validateUpdatePopulation(ListSource source, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateUpdatePopulation(source, req, user, uriParser);
		validate(source, req, user, uriParser);
	}

	protected void validate(ListSource source, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		super.validate(source, req, user, uriParser, validationErrors);

		Integer endpointID = ValidationUtils.validateParameter("endpointID", req, true, IntegerPopulator.getPopulator(), validationErrors);
		
		if(endpointID != null) {
			
			EasitGetItemsEndpoint endPoint = callback.getEndpoint(endpointID);
			
			if(endPoint == null) {
				
				validationErrors.add(new ValidationError("EndpointNotFound"));
				
			} else {
				
				source.setEndPoint(endPoint);
			}
			
		}
		
		if (!callback.testSource(source)) {
			validationErrors.add(new ValidationError("InvalidSource", "", "url"));
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
	}

}
