<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/easitintegration.js?v2
	</xsl:variable>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">
	
		<xsl:choose>
			<xsl:when test="ShowViewIntegrationSettings">
				<xsl:apply-templates select="ShowViewIntegrationSettings" />	
			</xsl:when>
			<xsl:otherwise>
				
				<div id="EasitIntegrationProvider" class="contentitem errands-wrapper border-box">
					
					<xsl:apply-templates select="UpdateIntegrationSettings" />
					<xsl:apply-templates select="ShowIntegrationSettings" />
					<xsl:apply-templates select="AddIntegrationParameterGroup" />
					<xsl:apply-templates select="UpdateIntegrationParameterGroup" />
					<xsl:apply-templates select="SortIntegrationParameterGroup" />
				
				</div>
				
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="ShowViewIntegrationSettings">
	
		<a name="easitintegrationsettings" />
		
		<xsl:choose>
			<xsl:when test="IntegrationSettings">
					
				<xsl:value-of select="$i18n.IntegrationEnabled"/>
				<xsl:text>:&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="IntegrationSettings/enabled = 'true'">
						<xsl:value-of select="$i18n.Yes"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No"/>
					</xsl:otherwise>
				</xsl:choose>
				
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoIntegrationSettings" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/settings/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateIntegrationSettings" />
			<img class="alignbottom" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="IntegrationSettings">
			<a href="{/Document/ModuleURI}/delete/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteIntegrationSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteIntegrationSettings" />
				<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="UpdateIntegrationSettings">
		
		<h1>
			<xsl:value-of select="$i18n.UpdateIntegrationSettingsTitle"/>
			<xsl:text>:&#x20;</xsl:text>
			<xsl:value-of select="Flow/name"/>
			<xsl:text>&#x20;(</xsl:text>
			<xsl:value-of select="$i18n.flowVersion"/>
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="Flow/version"/>
			<xsl:text>)</xsl:text>
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>

		<form method="post" action="{/Document/ModuleURI}/update/{Flow/flowID}">
		
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'enabled'" />
					<xsl:with-param name="id" select="'enabled'" />
					<xsl:with-param name="element" select="IntegrationSettings" />       
				</xsl:call-template>
				
				<label for="enabled">
					<xsl:value-of select="$i18n.EnableIntegration" />
				</label>
			</div>
		
			<label for="endpointID" class="floatleft full">
				<xsl:value-of select="$i18n.Endpoint" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="id" select="'endpointID'"/>
					<xsl:with-param name="name" select="'endpointID'"/>
					<xsl:with-param name="element" select="Endpoints/Endpoint" /> 
					<xsl:with-param name="labelElementName" select="'name'" />        
					<xsl:with-param name="valueElementName" select="'endpointID'" />
					<xsl:with-param name="selectedValue" select="IntegrationSettings/Endpoint/endpointID"/>
				</xsl:call-template>
			</div>
		
			<div class="floatright clearboth bigmargintop">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
			
		</form>
			
	</xsl:template>
	
	<xsl:template match="ShowIntegrationSettings">
		
		<h1>
			<xsl:value-of select="$i18n.UpdateIntegrationSettingsTitle"/>
			<xsl:text>:&#x20;</xsl:text>
			<xsl:value-of select="Flow/name"/>
			<xsl:text>&#x20;(</xsl:text>
			<xsl:value-of select="$i18n.flowVersion"/>
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="Flow/version"/>
			<xsl:text>)</xsl:text>
		</h1>

		<div class="full floatleft">
			<h3>
				<xsl:value-of select="$i18n.BaseSettings" />
				<div class="floatright">
					<a href="{/Document/ModuleURI}/update/{Flow/flowID}">
						<img class="alignbottom" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
					</a>
				</div>
			</h3>
		</div>

		<div class="thirty floatleft">

			<label for="endpointID" class="floatleft full">
				<xsl:value-of select="$i18n.IntegrationEnabled" />
			</label>
		
			<div class="floatleft full">
				<xsl:choose>
					<xsl:when test="IntegrationSettings/enabled = 'true'">
						<img src="{/Document/StaticContentURL}/pics/check.png" alt="" class="vertical-align-middle marginright" />
						<xsl:value-of select="$i18n.Yes" />
					</xsl:when>
					<xsl:otherwise>
						<img src="{/Document/StaticContentURL}/pics/disabled.png" alt="" class="vertical-align-middle marginright" />
						<xsl:value-of select="$i18n.No" />
					</xsl:otherwise>
				</xsl:choose>
			</div>
		
		</div>
		
		<div class="thirty floatleft">
		
			<label for="endpointID" class="floatleft full">
				<xsl:value-of select="$i18n.Endpoint" />
			</label>
		
			<div class="floatleft full">
				<xsl:value-of select="IntegrationSettings/Endpoint/name" />
			</div>
		
		</div>
		
		<div class="full floatleft bigmargintop">
			<h3><xsl:value-of select="$i18n.ParameterGroups" /></h3>
		</div>
	
		<table class="clearboth full coloredtable oep-table">
			
			<thead>
				<tr>
					<th width="16" />
					<th><xsl:value-of select="$i18n.ParameterGroupName" /></th>
					<th width="20"><xsl:value-of select="$i18n.ParameterGroupEnabled" /></th>
					<th width="100"><xsl:value-of select="$i18n.ParameterCount" /></th>
					<th width="100"><xsl:value-of select="$i18n.ImportWithPrevious" /></th>
					<th width="40" />
				</tr>
			</thead>
			<tbody>
				<xsl:choose>
					<xsl:when test="IntegrationSettings/parameterGroups/IntegrationParameterGroup">
						<xsl:apply-templates select="IntegrationSettings/parameterGroups/IntegrationParameterGroup" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td />
							<td colspan="5"><xsl:value-of select="$i18n.NoIntegrationParameterGroups" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
			</tbody>
			
		</table>

		<div class="floatright clearboth bigmargintop">
			<a href="{/Document/ModuleURI}/addparametergroup/{Flow/flowID}">
				<xsl:value-of select="$i18n.AddIntegrationGroupTitle" />
				<img class="alignbottom" src="{/Document/StaticContentURL}/pics/add.png" alt="" />
			</a>
		</div>	
	
		<div class="floatright clearboth margintop">
			<a href="{/Document/ModuleURI}/sortparametergroups/{Flow/flowID}">
				<xsl:value-of select="$i18n.SortIntegrationGroups" />
				<img class="alignbottom" src="{/Document/StaticContentURL}/pics/cog_move.png" alt="" />
			</a>
		</div>
	
	</xsl:template>
	
	<xsl:template match="IntegrationParameterGroup" mode="list">
	
		<tr>
			<td>
				<xsl:choose>
					<xsl:when test="enabled = 'true'"><img src="{/Document/StaticContentURL}/pics/cog_check.png" alt="" /></xsl:when>
					<xsl:otherwise><img src="{/Document/StaticContentURL}/pics/cog_disabled.png" alt="" /></xsl:otherwise>
				</xsl:choose>
			</td>
			<td><xsl:value-of select="name" /></td>
			<td>
				<xsl:choose>
					<xsl:when test="enabled = 'true'"><xsl:value-of select="$i18n.Yes" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="$i18n.No" /></xsl:otherwise>
				</xsl:choose>
			</td>
			<td><xsl:value-of select="count(Parameters/IntegrationParameter)" /></td>
			<td>
				<xsl:choose>
					<xsl:when test="importWithPrevious = 'true'"><xsl:value-of select="$i18n.ImportWithPrevious.Yes" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="$i18n.ImportWithPrevious.No" /></xsl:otherwise>
				</xsl:choose>
			</td>
			<td>
				<a href="{/Document/ModuleURI}/deleteparametergroup/{../../../Flow/flowID}/{parameterGroupID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.DeleteParameterGroupConfirm.Title}: {name}" class="floatright">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>
				<a href="{/Document/ModuleURI}/updateparametergroup/{../../../Flow/flowID}/{parameterGroupID}" class="floatright" title="{$i18n.UpdateParameterGroupConfirm.Title}: {name}">
					<img class="alignbottom marginright" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
				</a>
			</td>
		</tr>

	</xsl:template>
	
	<xsl:template match="AddIntegrationParameterGroup">
	
		<h1>
			<xsl:value-of select="$i18n.AddIntegrationGroupTitle"/>
		</h1>

		<xsl:call-template name="createIntegrationParameterGroupForm">
			<xsl:with-param name="formAction"><xsl:value-of select="/Document/ModuleURI" />/addparametergroup/<xsl:value-of select="Flow/flowID" /></xsl:with-param>
			<xsl:with-param name="submitButton" select="$i18n.Add" />
		</xsl:call-template>
			
	</xsl:template>
	
	<xsl:template match="UpdateIntegrationParameterGroup">
	
		<h1>
			<xsl:value-of select="$i18n.UpdateIntegrationGroupTitle" />
			<xsl:text>:&#x20;</xsl:text>
			<xsl:value-of select="IntegrationParameterGroup/name"/>
		</h1>

		<xsl:call-template name="createIntegrationParameterGroupForm">
			<xsl:with-param name="formAction"><xsl:value-of select="/Document/ModuleURI" />/updateparametergroup/<xsl:value-of select="Flow/flowID" />/<xsl:value-of select="IntegrationParameterGroup/parameterGroupID" /></xsl:with-param>
			<xsl:with-param name="submitButton" select="$i18n.SaveChanges" />
		</xsl:call-template>
	
	</xsl:template>
	
	<xsl:template name="createIntegrationParameterGroupForm">
		
		<xsl:param name="formAction" />
		<xsl:param name="submitButton" />
		
		<xsl:apply-templates select="validationException/validationError"/>

		<form method="post" action="{$formAction}">
	
			<label for="name" class="floatleft full">
				<xsl:value-of select="$i18n.Name" />
			</label>				
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'name'"/>
					<xsl:with-param name="name" select="'name'"/>
					<xsl:with-param name="element" select="IntegrationParameterGroup" />          
				</xsl:call-template>
			</div>
			
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'enabled'" />
					<xsl:with-param name="id" select="'enabled'" />
					<xsl:with-param name="element" select="IntegrationParameterGroup" />       
				</xsl:call-template>
				
				<label for="enabled">
					<xsl:value-of select="$i18n.EnableParameterGroup" />
				</label>
			</div>
			
			<label for="importHandlerIdentifier" class="floatleft full">
				<xsl:value-of select="$i18n.ImportHandlerIdentifier" />
			</label>				
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'importHandlerIdentifier'"/>
					<xsl:with-param name="name" select="'importHandlerIdentifier'"/>
					<xsl:with-param name="element" select="IntegrationParameterGroup" />          
				</xsl:call-template>
			</div>
	
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'importWithPrevious'" />
					<xsl:with-param name="id" select="'importWithPrevious'" />
					<xsl:with-param name="element" select="IntegrationParameterGroup" />       
				</xsl:call-template>
				
				<label for="importWithPrevious">
					<xsl:value-of select="$i18n.ImportWithPrevious" />
				</label>
			</div>
			
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'importParametersSeparately'" />
					<xsl:with-param name="id" select="'importParametersSeparately'" />
					<xsl:with-param name="element" select="IntegrationParameterGroup" />       
				</xsl:call-template>
				
				<label for="importParametersSeparately">
					<xsl:value-of select="$i18n.ImportParametersSeparately" />
				</label>
			</div>

			<div class="full floatleft bigmargintop">
				<h3><xsl:value-of select="$i18n.AttributeRule" /></h3>
			</div>
			
			<p><xsl:value-of select="$i18n.AttributeRuleDescription" /></p>
			
			<label for="name" class="floatleft full">
				<xsl:value-of select="$i18n.AttributeName" />
			</label>
			
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'attributeName'"/>
					<xsl:with-param name="name" select="'attributeName'"/>
					<xsl:with-param name="element" select="IntegrationParameterGroup" />          
				</xsl:call-template>
			</div>
	
			<label for="attributeValues" class="floatleft full">
				<xsl:value-of select="$i18n.AttributeValues" />
			</label>				
			
			<div class="floatleft full">
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="id" select="'attributeValues'"/>
					<xsl:with-param name="name" select="'attributeValues'"/>
					<xsl:with-param name="element" select="IntegrationParameterGroup/AttributeValues/value" />
					<xsl:with-param name="separateListValues" select="'true'"/>  
					<xsl:with-param name="rows" select="4" />        
				</xsl:call-template>
			</div>
	
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'invert'" />
					<xsl:with-param name="id" select="'invert'" />
					<xsl:with-param name="element" select="IntegrationParameterGroup" />       
				</xsl:call-template>
				
				<label for="invert">
					<xsl:value-of select="$i18n.InvertAttributeValues" />
				</label>
			</div>
	
			<div class="full floatleft bigmargintop">
				<h3><xsl:value-of select="$i18n.Parameters" /></h3>
			</div>
		
			<table id="param-table" class="full coloredtable ">
				<thead>
					<tr>
						<th><xsl:value-of select="$i18n.Name" /></th>
						<th><xsl:value-of select="$i18n.Value" /></th>
						<th><img class="alignbottom" src="{/Document/StaticContentURL}/pics/split.png" alt="{$i18n.SplitValues}" title="{$i18n.SplitValues}"/></th>
						<th width="16px"></th>
					</tr>
				</thead>
				<tbody>
				
					<xsl:choose>
						<xsl:when test="requestparameters">
						
							<xsl:apply-templates select="requestparameters/parameter[name = 'parameterID']/value" mode="parameter"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:apply-templates select="IntegrationParameterGroup/Parameters/IntegrationParameter"/>
						
						</xsl:otherwise>
					</xsl:choose>					
				
				</tbody>
			</table>
			
			<div class="floatright clearboth bigmargintop">
				<a onclick="addParameter();">
					<xsl:value-of select="$i18n.AddParameter" />
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/add.png" alt="" />
				</a>
			</div>			
		
			<div class="full floatleft bigmargintop">
				<h3><xsl:value-of select="$i18n.Attachments" /></h3>
			</div>
		
			<label for="attachmentPrefixes" class="floatleft full">
				<xsl:value-of select="$i18n.AttachmentPrefixes" />
			</label>
		
			<div class="floatleft full">
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="id" select="'attachmentPrefixes'"/>
					<xsl:with-param name="name" select="'attachmentPrefixes'"/>
					<xsl:with-param name="element" select="IntegrationParameterGroup/AttachmentPrefixes/value" />
					<xsl:with-param name="separateListValues" select="'true'"/>  
					<xsl:with-param name="rows" select="4" />        
				</xsl:call-template>
			</div>
		
			<div class="floatright clearboth bigmargintop">
		
				<div class="floatright">
					<input type="submit" value="{$submitButton}" />
				</div>
			
			</div>
		
		</form>
		
		<div style="display: none;">
			
			<table>
				<tr id="parameter-template">
					<td>
						<input type="text" name="name" class="full"/>
					</td>
					<td>
						<!-- <input type="text" name="value" class="full"/> -->
						<textarea rows="2" name="value" class="full" />
					</td>
					<td>
						<input type="checkbox" name="split"/>
					</td>						
					<td>
						<input type="hidden" name="parameterID"/>
						
						<a onclick="deleteParameter(this)">
							<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
						</a>				
					</td>
				</tr>
			</table>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="IntegrationParameter">
		
		<tr id="parameter-{parameterID}">
			<td>
				<input type="text" name="name-{parameterID}" value="{name}" class="full"/>
			</td>
			<td>
				<!-- <input type="text" name="value-{parameterID}" value="{value}" class="full"/> -->
				<textarea rows="2" name="value-{parameterID}" class="full"><xsl:value-of select="value" /></textarea>
			</td>
			<td>
				<input type="checkbox" name="split-{parameterID}" value="true">
					
					<xsl:if test="splitValue = 'true'">
						<xsl:attribute name="checked">true</xsl:attribute>
					</xsl:if>
					
				</input>
			</td>			
			<td>
				<input type="hidden" name="parameterID" value="{parameterID}"/>
				
				<a onclick="deleteParameter(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="value" mode="parameter">
		
 		<xsl:variable name="parameterID" select="."/>
		
		<xsl:variable name="nameParam" select="concat('name-', $parameterID)"/>
		
		<xsl:variable name="valueParam" select="concat('value-', $parameterID)"/>
		
		<xsl:variable name="splitParam" select="concat('split-', $parameterID)"/>
		
		<tr id="parameter-{$parameterID}">
 			<td>
				<input type="text" name="name-{$parameterID}" value="{../../parameter[name = $nameParam]/value}" class="full"/>
			</td>
			<td>
<!-- 			<input type="text" name="value-{$parameterID}" value="{../../parameter[name = $valueParam]/value}" class="full"/> -->
				<textarea rows="2" name="value-{parameterID}" class="full"><xsl:value-of select="../../parameter[name = $valueParam]/value" /></textarea>
			</td>
			<td>
				<input type="checkbox" name="split-{$parameterID}" value="true">
				
					<xsl:if test="../../parameter[name = $splitParam]/value = 'true'">
						<xsl:attribute name="checked">true</xsl:attribute>
					</xsl:if>
				
				</input>
			</td>				
			<td>
				<input type="hidden" name="parameterID" value="{$parameterID}"/>
				
				<a onclick="deleteParameter(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>
		</tr>
	
	</xsl:template>	
	
	<xsl:template match="SortIntegrationParameterGroup">
	
		<h1>
			<xsl:value-of select="$i18n.SortIntegrationGroupTitle" />
		</h1>
	
		<form id="sortParameterGroupForm" method="post" action="{/Document/ModuleURI}/sortparametergroups/{Flow/flowID}">
		
			<div class="floatleft full sortable">
							
				<xsl:apply-templates select="IntegrationSettings/parameterGroups/IntegrationParameterGroup" mode="sort" />
							
			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>
		
	</xsl:template>
	
	<xsl:template match="IntegrationParameterGroup" mode="sort">
	
		<div id="parametergroup_{parameterGroupID}" class="floatleft hover border full marginbottom lightbackground cursor-move border-radius">
			<div class="padding">
				<img class="vertical-align-middle marginright" src="{/Document/StaticContentURL}/pics/move.png" title="{$i18n.MoveParameterGroup}"/>
				<xsl:value-of select="name" />
				<xsl:call-template name="createHiddenField">
					<xsl:with-param name="id" select="concat('sortorder_', parameterGroupID)" />
					<xsl:with-param name="name" select="concat('sortorder_', parameterGroupID)" />
					<xsl:with-param name="value" select="sortIndex" />
					<xsl:with-param name="class" select="'sortorder'" />
				</xsl:call-template>
			</div>
		</div>
	
	</xsl:template>
	
	
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="displayName">
						<xsl:value-of select="displayName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'name'">
						<xsl:value-of select="$i18n.Name"/>
					</xsl:when>
					<xsl:when test="fieldName = 'value'">
						<xsl:value-of select="$i18n.Value"/>
					</xsl:when>
					<xsl:when test="fieldName = 'endpointID'">
						<xsl:value-of select="$i18n.Endpoint"/>
					</xsl:when>
					<xsl:when test="fieldName = 'attributeValues'">
						<xsl:value-of select="$i18n.AttributeValues"/>
					</xsl:when>
					<xsl:when test="fieldName = 'attributeName'">
						<xsl:value-of select="$i18n.AttributeName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'importHandlerIdentifier'">
						<xsl:value-of select="$i18n.ImportHandlerIdentifier"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>