package com.nordicpeak.flowengine.externalmessageoffer.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.time.LocalDateTime;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.externalmessageoffer.enums.ExternalMessageOfferStatus;

@Table(name = "flowengine_external_message_offers")
@XMLElement(name = "Offer")
public class ExternalMessageOffer extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = -5166683291745202612L;

	public static final Field FIELD_OFFER_ID = ReflectionUtils.getField(ExternalMessageOffer.class, "offerID");
	public static final Field FIELD_FLOW_INSTANCE_ID = ReflectionUtils.getField(ExternalMessageOffer.class, "flowInstanceID");
	public static final Field FIELD_MESSAGE_ID = ReflectionUtils.getField(ExternalMessageOffer.class, "messageID");
	public static final Field FIELD_SETTINGS = ReflectionUtils.getField(ExternalMessageOffer.class, "settings");
	public static final Field FIELD_STATUS = ReflectionUtils.getField(ExternalMessageOffer.class, "status");

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer offerID;

	@DAOManaged
	@XMLElement
	private Integer messageID;

	@DAOManaged
	@XMLElement
	private Integer flowInstanceID;

	@DAOManaged(columnName = "settingsID")
	@ManyToOne
	@XMLElement
	private ExternalMessageOfferSettings settings;

	@DAOManaged
	@XMLElement
	private ExternalMessageOfferStatus status;

	@DAOManaged
	@XMLElement
	private Timestamp created;

	@Override
	public String toString() {

		return "(offerID=" + offerID + ", messageID=" + messageID + ", flowInstanceID=" + getFlowInstanceID() + ", settings=" + settings + ", status=" + status + ", created=" + created + ")";
	}

	public Integer getOfferID() {

		return offerID;
	}

	public void setOfferID(Integer offerID) {

		this.offerID = offerID;
	}

	public Integer getMessageID() {

		return messageID;
	}

	public void setMessageID(Integer messageID) {

		this.messageID = messageID;
	}

	public Integer getFlowInstanceID() {

		return flowInstanceID;
	}

	public void setFlowInstanceID(Integer flowInstanceID) {

		this.flowInstanceID = flowInstanceID;
	}

	public ExternalMessageOfferSettings getSettings() {

		return settings;
	}

	public void setSettings(ExternalMessageOfferSettings settings) {

		this.settings = settings;
	}

	public ExternalMessageOfferStatus getStatus() {

		return status;
	}

	public void setStatus(ExternalMessageOfferStatus status) {

		this.status = status;
	}

	public Timestamp getCreated() {

		return created;
	}

	public void setCreated(Timestamp created) {

		this.created = created;
	}

	public LocalDateTime calculateExpirationTime() {

		if (!settings.isSetTimeLimit()) {

			return null;
		}

		return created.toLocalDateTime().plusDays(settings.getDurationDays());
	}

}
