package com.nordicpeak.flowengine.externalmessageoffer.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

@Table(name = "flowengine_flow_status_offer_settings")
@XMLElement(name = "Settings")
public class ExternalMessageOfferSettings extends GeneratedElementable implements Serializable, XMLParserPopulateable {

	private static final long serialVersionUID = 3268646669132872847L;

	public static final Field FIELD_SETTINGS_ID = ReflectionUtils.getField(ExternalMessageOfferSettings.class, "settingsID");
	public static final Field FIELD_STATUS_ID = ReflectionUtils.getField(ExternalMessageOfferSettings.class, "statusID");

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer settingsID;

	@DAOManaged
	@XMLElement
	private Integer statusID;

	@DAOManaged
	@XMLElement(name = "offerEnabled")
	@WebPopulate(paramName = "offerEnabled")
	private boolean enabled;

	@DAOManaged
	@XMLElement(name = "setOfferAsAttribute")
	@WebPopulate(paramName = "setOfferAsAttribute")
	private boolean setAsAttribute;

	@DAOManaged
	@XMLElement(name = "offerAttributeName")
	@WebPopulate(paramName = "offerAttributeName", maxLength = 255)
	@RequiredIfSet(paramNames = "setOfferAsAttribute")
	private String attributeName;

	@DAOManaged
	@XMLElement(name = "setOfferTimeLimit")
	@WebPopulate(paramName = "setOfferTimeLimit")
	private boolean setTimeLimit;

	@DAOManaged
	@XMLElement(name = "offerDurationDays")
	@WebPopulate(paramName = "offerDurationDays")
	@RequiredIfSet(paramNames = "setOfferTimeLimit")
	private Integer durationDays;

	@DAOManaged
	@XMLElement(name = "offerExpiredStatus")
	@WebPopulate(paramName = "offerExpiredStatus", maxLength = 255)
	@RequiredIfSet(paramNames = "setOfferTimeLimit")
	private Integer expiredStatusID;

	@XMLElement(name = "offerExpiredStatusName")
	private String expiredStatusName;

	@DAOManaged
	@XMLElement(name = "offerAcceptedStatus")
	@WebPopulate(paramName = "offerAcceptedStatus", maxLength = 255, required = true)
	private Integer acceptedStatusID;

	@XMLElement(name = "offerAcceptedStatusName")
	private String acceptedStatusName;

	@DAOManaged
	@XMLElement(name = "offerAcceptedButtonText")
	@WebPopulate(paramName = "offerAcceptedButtonText", maxLength = 40, required = true)
	private String acceptedButtonText;

	@DAOManaged
	@XMLElement(name = "offerAcceptedMessage")
	@WebPopulate(paramName = "offerAcceptedMessage", maxLength = 255, required = true)
	private String acceptedMessage;

	@DAOManaged
	@XMLElement(name = "offerDeclinedStatus")
	@WebPopulate(paramName = "offerDeclinedStatus", maxLength = 255, required = true)
	private Integer declinedStatusID;

	@XMLElement(name = "offerDeclinedStatusName")
	private String declinedStatusName;

	@DAOManaged
	@XMLElement(name = "offerDeclinedButtonText")
	@WebPopulate(paramName = "offerDeclinedButtonText", maxLength = 40, required = true)
	private String declinedButtonText;

	@DAOManaged
	@XMLElement(name = "offerDeclinedMessage")
	@WebPopulate(paramName = "offerDeclinedMessage", maxLength = 255, required = true)
	private String declinedMessage;

	@DAOManaged(columnName = "offerID")
	@OneToMany
	@XMLElement(name = "Offers")
	private List<ExternalMessageOffer> offers;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		settingsID = XMLValidationUtils.validateParameter("settingsID", xmlParser, true, IntegerPopulator.getPopulator(), errors);
		statusID = XMLValidationUtils.validateParameter("statusID", xmlParser, true, IntegerPopulator.getPopulator(), errors);
		enabled = xmlParser.getPrimitiveBoolean("enabled");

		setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		if (setAsAttribute) {

			attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		}

		setTimeLimit = xmlParser.getPrimitiveBoolean("setTimeLimit");
		if (setTimeLimit) {

			durationDays = XMLValidationUtils.validateParameter("durationDays", xmlParser, true, IntegerPopulator.getPopulator(), errors);
			expiredStatusID = XMLValidationUtils.validateParameter("expiredStatusID", xmlParser, true, 1, 255, IntegerPopulator.getPopulator(), errors);
		}

		acceptedStatusID = XMLValidationUtils.validateParameter("acceptedStatusID", xmlParser, true, 1, 255, IntegerPopulator.getPopulator(), errors);
		acceptedButtonText = XMLValidationUtils.validateParameter("acceptedButtonText", xmlParser, true, 1, 40, StringPopulator.getPopulator(), errors);
		acceptedMessage = XMLValidationUtils.validateParameter("acceptedMessage", xmlParser, true, 1, 65535, StringPopulator.getPopulator(), errors);

		declinedStatusID = XMLValidationUtils.validateParameter("declinedStatusID", xmlParser, true, 1, 255, IntegerPopulator.getPopulator(), errors);
		declinedButtonText = XMLValidationUtils.validateParameter("declinedButtonText", xmlParser, true, 1, 40, StringPopulator.getPopulator(), errors);
		declinedMessage = XMLValidationUtils.validateParameter("declinedMessage", xmlParser, true, 1, 65535, StringPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return "(settingsID=" + settingsID + ", statusID=" + statusID + ")";
	}

	public Integer getSettingsID() {

		return settingsID;
	}

	public void setSettingsID(Integer settingsID) {

		this.settingsID = settingsID;
	}

	public Integer getStatusID() {

		return statusID;
	}

	public void setStatusID(Integer statusID) {

		this.statusID = statusID;
	}

	public boolean isEnabled() {

		return enabled;
	}

	public void setEnabled(boolean enabled) {

		this.enabled = enabled;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public boolean isSetTimeLimit() {

		return setTimeLimit;
	}

	public void setSetTimeLimit(boolean setTimeLimit) {

		this.setTimeLimit = setTimeLimit;
	}

	public Integer getDurationDays() {

		return durationDays;
	}

	public void setDurationDays(Integer durationDays) {

		this.durationDays = durationDays;
	}

	public Integer getExpiredStatusID() {

		return expiredStatusID;
	}

	public void setExpiredStatusID(Integer expiredStatusID) {

		this.expiredStatusID = expiredStatusID;
	}

	public String getExpiredStatusName() {

		return expiredStatusName;
	}

	public void setExpiredStatusName(String expiredStatusName) {

		this.expiredStatusName = expiredStatusName;
	}

	public Integer getAcceptedStatusID() {

		return acceptedStatusID;
	}

	public void setAcceptedStatusID(Integer acceptedStatusID) {

		this.acceptedStatusID = acceptedStatusID;
	}

	public String getAcceptedStatusName() {

		return acceptedStatusName;
	}

	public void setAcceptedStatusName(String acceptedStatusName) {

		this.acceptedStatusName = acceptedStatusName;
	}

	public String getAcceptedButtonText() {

		return acceptedButtonText;
	}

	public void setAcceptedButtonText(String acceptedButtonText) {

		this.acceptedButtonText = acceptedButtonText;
	}

	public String getAcceptedMessage() {

		return acceptedMessage;
	}

	public void setAcceptedMessage(String acceptedMessage) {

		this.acceptedMessage = acceptedMessage;
	}

	public Integer getDeclinedStatusID() {

		return declinedStatusID;
	}

	public void setDeclinedStatusID(Integer declinedStatusID) {

		this.declinedStatusID = declinedStatusID;
	}

	public String getDeclinedStatusName() {

		return declinedStatusName;
	}

	public void setDeclinedStatusName(String declinedStatusName) {

		this.declinedStatusName = declinedStatusName;
	}

	public String getDeclinedButtonText() {

		return declinedButtonText;
	}

	public void setDeclinedButtonText(String declinedButtonText) {

		this.declinedButtonText = declinedButtonText;
	}

	public String getDeclinedMessage() {

		return declinedMessage;
	}

	public void setDeclinedMessage(String declinedMessage) {

		this.declinedMessage = declinedMessage;
	}

	public List<ExternalMessageOffer> getOffers() {

		return offers;
	}

	public void setOffers(List<ExternalMessageOffer> offers) {

		this.offers = offers;
	}

}
