package com.nordicpeak.flowengine.externalmessageoffer.daos;

import java.lang.reflect.Field;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOFactory;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;

import com.nordicpeak.flowengine.externalmessageoffer.beans.ExternalMessageOffer;
import com.nordicpeak.flowengine.externalmessageoffer.enums.ExternalMessageOfferStatus;

public class ExternalMessageOfferDAO extends AnnotatedDAO<ExternalMessageOffer> {

	private final QueryParameterFactory<ExternalMessageOffer, Integer> offerIDParamFactory;
	private final QueryParameterFactory<ExternalMessageOffer, Integer> flowInstanceIDParamFactory;
	private final QueryParameterFactory<ExternalMessageOffer, Integer> messageIDParamFactory;
	private final QueryParameterFactory<ExternalMessageOffer, ExternalMessageOfferStatus> statusParamFactory;

	public ExternalMessageOfferDAO(DataSource dataSource, Class<ExternalMessageOffer> beanClass, AnnotatedDAOFactory daoFactory) {

		super(dataSource, beanClass, daoFactory);

		offerIDParamFactory = this.getParamFactory(ExternalMessageOffer.FIELD_OFFER_ID, Integer.class);
		flowInstanceIDParamFactory = this.getParamFactory(ExternalMessageOffer.FIELD_FLOW_INSTANCE_ID, Integer.class);
		messageIDParamFactory = this.getParamFactory(ExternalMessageOffer.FIELD_MESSAGE_ID, Integer.class);
		statusParamFactory = this.getParamFactory(ExternalMessageOffer.FIELD_STATUS, ExternalMessageOfferStatus.class);
	}

	public ExternalMessageOffer get(Integer offerID, Field... relations) throws SQLException {

		HighLevelQuery<ExternalMessageOffer> query = new HighLevelQuery<>();

		query.addParameter(offerIDParamFactory.getParameter(offerID));

		if (relations != null) {

			query.addRelations(relations);
		}

		return this.get(query);
	}

	public ExternalMessageOffer getByMessageID(Integer messageID, Field... relations) throws SQLException {

		HighLevelQuery<ExternalMessageOffer> query = new HighLevelQuery<>();

		query.addParameter(messageIDParamFactory.getParameter(messageID));

		if (relations != null) {

			query.addRelations(relations);
		}

		return this.get(query);
	}

	public Integer deleteByFlowInstanceID(Integer flowInstanceID, Field... relations) throws SQLException {

		HighLevelQuery<ExternalMessageOffer> query = new HighLevelQuery<>();

		query.addParameter(flowInstanceIDParamFactory.getParameter(flowInstanceID));

		if (relations != null) {

			query.addRelations(relations);
		}

		return this.delete(query);
	}

	public List<ExternalMessageOffer> getByStatus(ExternalMessageOfferStatus status, Field... relations) throws SQLException {

		HighLevelQuery<ExternalMessageOffer> query = new HighLevelQuery<>();

		query.addParameter(statusParamFactory.getParameter(status));

		if (relations != null) {

			query.addRelations(relations);
		}

		return this.getAll(query);
	}

}
