package com.nordicpeak.flowengine.queries.fixedtablequery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basetablequery.TableRow;

@Table(name = "fixed_table_query_rows")
@XMLElement(name = "Row")
public class FixedTableRow extends TableRow<DropDownAlternative, FixedTableColumn, FixedTableQuery> implements XMLParserPopulateable {

	private static final long serialVersionUID = -2724650199031970214L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(FixedTableRow.class, "query");

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String label;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean required;

	@DAOManaged
	@OrderBy
	@XMLElement
	private Integer sortIndex;

	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String helpText;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private FixedTableQuery query;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<FixedTableCellValue> values;

	public FixedTableQuery getQuery() {

		return query;
	}

	public void setQuery(FixedTableQuery query) {

		this.query = query;
	}

	public String getLabel() {

		return label;
	}

	public void setLabel(String name) {

		this.label = name;
	}

	@Override
	public boolean isRequired() {

		return required;
	}

	public void setRequired(boolean required) {

		this.required = required;
	}

	public List<FixedTableCellValue> getValues() {

		return values;
	}

	public void setValues(List<FixedTableCellValue> values) {

		this.values = values;
	}

	@Override
	public Integer getSortIndex() {

		return sortIndex;
	}

	@Override
	public void setSortIndex(Integer sortIndex) {

		this.sortIndex = sortIndex;
	}

	public String getHelpText() {

		return helpText;
	}

	public void setHelpText(String helpText) {

		this.helpText = helpText;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		label = XMLValidationUtils.validateParameter("label", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		required = xmlParser.getPrimitiveBoolean("required");

		setSortIndex(XMLValidationUtils.validateParameter("sortIndex", xmlParser, true, NonNegativeStringIntegerPopulator.getPopulator(), errors));

		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}

	}

	@Override
	public String getRowIdentifier() {

		return getRowID().toString();
	}

	@Override
	public String getName() {

		return label;
	}

}
