package com.nordicpeak.flowengine.paymentprovider.beans;

import java.util.List;

import se.unlogic.emailutils.populators.LowerCaseEmailPopulator;
import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CaseInsensitiveStringComparator;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

@Table(name = "flow_generic_payment_settings")
@XMLElement
public class FlowFamilyPaymentSettings extends GeneratedElementable {
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer flowFamilyID;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private String invoiceDescriptionSuffix;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "allowsInvoicePayment")
	@XMLElement
	private String invoicePaymentEventDescription;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "flow_generic_payment_invoice_notification_addresses", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = LowerCaseEmailPopulator.class)
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> invoicePaymentNotificationAddresses;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "flow_generic_payment_direct_notification_addresses", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = LowerCaseEmailPopulator.class)
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> directPaymentNotificationAddresses;
	
	public Integer getFlowFamilyID() {
		
		return flowFamilyID;
	}
	
	public void setFlowFamilyID(Integer flowFamilyID) {
		
		this.flowFamilyID = flowFamilyID;
	}
	
	public String getInvoiceDescriptionSuffix() {
		return invoiceDescriptionSuffix;
	}
	
	public void setInvoiceDescriptionSuffix(String invoiceDescriptionSuffix) {
		this.invoiceDescriptionSuffix = invoiceDescriptionSuffix;
	}
	
	public String getInvoicePaymentEventDescription() {
		return invoicePaymentEventDescription;
	}
	
	public void setInvoicePaymentEventDescription(String invoicePaymentEventDescription) {
		this.invoicePaymentEventDescription = invoicePaymentEventDescription;
	}
	
	public List<String> getInvoicePaymentNotificationAddresses() {
		
		return invoicePaymentNotificationAddresses;
	}
	
	public void setInvoicePaymentNotificationAddresses(List<String> invoicePaymentNotificationAddresses) {
		
		this.invoicePaymentNotificationAddresses = invoicePaymentNotificationAddresses;
	}
	
	public List<String> getDirectPaymentNotificationAddresses() {
		
		return directPaymentNotificationAddresses;
	}
	
	public void setDirectPaymentNotificationAddresses(List<String> directPaymentNotificationAddresses) {
		
		this.directPaymentNotificationAddresses = directPaymentNotificationAddresses;
	}
	
}
