package com.nordicpeak.flowengine.queries.iconmapquery;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.sql.Blob;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.sql.rowset.serial.SerialBlob;

import org.apache.commons.fileupload.FileUploadException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.fileuploadutils.MultipartRequest;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.DropDownSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EnumDropDownSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.PathType;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.log4jutils.levels.LogLevel;
import se.unlogic.standardutils.arrays.ArrayUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.io.BinarySizes;
import se.unlogic.standardutils.io.CloseUtils;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.mime.MimeUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.streams.StreamUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.PDFAttachment;
import com.nordicpeak.flowengine.interfaces.PDFResourceProvider;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.interfaces.ReservationProvider;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.iconmapquery.MapImageDataGenerator.MapImageData;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.Geometry;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.IconMapQuery;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.IconMapQueryInstance;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.IconMapQueryInstancePrint;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.MapIcon;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.SelectedRemoteGeometry;
import com.nordicpeak.flowengine.queries.iconmapquery.requestprocessors.GeometryReservationRequestProcessor;
import com.nordicpeak.flowengine.queries.iconmapquery.requestprocessors.MapImageRequestProcessor;
import com.nordicpeak.flowengine.queries.iconmapquery.validationerrors.MapIconValidationError;
import com.nordicpeak.flowengine.queries.iconmapquery.validationerrors.SelectedRemoteGeometryValidationError;
import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometry;
import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometryEndpoint;
import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometryEndpointProvider;
import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometryQuery;
import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometryQueryProvider;
import com.nordicpeak.flowengine.utils.BlobPDFAttachment;
import com.nordicpeak.flowengine.utils.BlobResourceProvider;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;
import com.nordicpeak.lmsearch.LMSearchProvider;
import com.nordicpeak.lmsearch.beans.UnitArea;
import com.nordicpeak.origomap.OrigoMapProvider;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.io.WKTReader;

public class IconMapQueryProviderModule extends BaseQueryProviderModule<IconMapQueryInstance> implements RemoteGeometryQueryProvider {

	private static final RelationQuery GET_QUERY_INSTANCE_RELATION_QUERY = new RelationQuery(IconMapQueryInstance.QUERY_RELATION, IconMapQueryInstance.MAPPRINTS_RELATION, IconMapQueryInstance.GEOMETRIES_RELATION, IconMapQueryInstance.SELECTED_GEOMETRIES_RELATION, IconMapQueryInstance.RESERVATIONS_RELATION);
	private static final RelationQuery SAVE_QUERY_INSTANCE_RELATION_QUERY = new RelationQuery(IconMapQueryInstance.MAPPRINTS_RELATION, IconMapQueryInstance.GEOMETRIES_RELATION, IconMapQueryInstance.SELECTED_GEOMETRIES_RELATION, IconMapQueryInstance.RESERVATIONS_RELATION);

	@XSLVariable(prefix = "java.")
	private String pdfAttachmentDescriptionPrefix = "A file from query:";

	@XSLVariable(prefix = "java.")
	private String pdfAttachmentFilename = "Map $scale";

	@XSLVariable(prefix = "java.")
	private String pdfAttachmentFilenameWithoutScale = "Map";

	@XSLVariable(prefix = "java.")
	private String missingEndpoint = "Missing endpoint";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Origo base URL", description = "The base URL to the origo client", required = true)
	protected String origoBaseURL = "/origo-map";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "MapFish connection timeout", description = "MapFish connection timeout")
	protected Integer mapFishConnectionTimeout = 5000;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "MapFish read timeout", description = "MapFish read timeout")
	protected Integer mapFishReadTimeout = 10000;

	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "MapFish timeout log level", description = "The log level used when there is a socket timeout while waiting for an image from MapFish", required = true)
	protected LogLevel mapFishTimeoutLogLevel = LogLevel.WARN;

	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "MapFish error log level", description = "The log level used when there is an error in the communication with MapFish", required = true)
	protected LogLevel mapFishErrorLogLevel = LogLevel.WARN;

	@ModuleSetting(allowsNull = true)
	@EnumDropDownSettingDescriptor(name = "Map configuration path type", description = "The pathtype of the map configuration file", required = false)
	protected PathType mapConfigurationPathType = PathType.Filesystem;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "Map configuration path", description = "The path to the map configuration file", required = false)
	protected String mapConfigurationPath;

	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Print configuration path type", description = "The pathtype of the print configuration file", required = true)
	protected PathType printConfigurationPathType = PathType.Filesystem;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Print configuration path", description = "The path to the print configuration file", required = true)
	protected String printConfigurationPath;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Print service adress", description = "The address to the print service", required = true)
	protected String printServiceAddress = "http://not.set/print";

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Log print configuration", description = "Log generated print configuration")
	protected boolean logPrintConfig;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Disk threshold", description = "Maximum size of files in MB to be buffered on disk during file uploads.", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer diskThreshold = 500;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "RAM threshold", description = "Maximum size of files in KB to be buffered in RAM during file uploads. Files exceeding the threshold are written to disk instead.", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer ramThreshold = 500;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max file size", description = "Maximum size of individual files in MB for file uploads.", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer maxFileSize = 20;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "Temp dir", description = "Directory for temporary files. Should be on the same filesystem as the file store for best performance. If not set system default temp directory will be used")
	protected String tempDir;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max map icon width", description = "Max allowed map icon width.")
	protected int maxMapIconWidth = 100;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max map icon height", description = "Max allowed map icon height.")
	protected int maxMapIconHeight = 100;

	/* PNG map image settings */

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "PNG map image layout", description = "The layout to use when generating png map image", required = true)
	protected String pngMapImageLayout = "OH_PNG";

	@ModuleSetting(allowsNull = true)
	@DropDownSettingDescriptor(name = "PNG map image scale", description = "The scale to use when generating png map image (do not set if calculated scale should be used)", required = false, valueDescriptions = { "1:400", "1:500", "1:750", "1:1000", "1:2000", "1:5000", "1:10000" }, values = { "400", "500", "750", "1000", "2000", "5000", "10000" })
	protected Integer pngMapImageScale;

	@ModuleSetting
	@DropDownSettingDescriptor(name = "PNG map image resolution", description = "The resolution to use when generating png map image", required = true, valueDescriptions = { "56 dpi", "127 dpi", "190 dpi", "254 dpi" }, values = { "56", "127", "190", "254" })
	protected Integer pngMapImageResolution = 56;

	/* PDF map image settings */

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "PDF map image layout", description = "The layout to use when generating pdf map image", required = false)
	protected String pdfMapImageLayout;

	@ModuleSetting(allowsNull = true)
	@DropDownSettingDescriptor(name = "PDF image scale", description = "The scale to use when generating pdf map image (do not set if calculated scale should be used)", required = false, valueDescriptions = { "1:400", "1:500", "1:750", "1:1000", "1:2000", "1:5000", "1:10000" }, values = { "400", "500", "750", "1000", "2000", "5000", "10000" })
	protected Integer pdfMapImageScale;

	@ModuleSetting(allowsNull = true)
	@DropDownSettingDescriptor(name = "PDF map image resolution", description = "The resolution to use when pdf map image", required = false, valueDescriptions = { "56 dpi", "127 dpi", "190 dpi", "254 dpi" }, values = { "56", "127", "190", "254" })
	protected Integer pdfMapImageResolution;

	protected String mapConfiguration;
	protected String printConfiguration;

	private AnnotatedDAO<MapIcon> mapIconDAO;
	private AnnotatedDAO<IconMapQuery> queryDAO;
	private AnnotatedDAO<IconMapQueryInstance> queryInstanceDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;

	private IconMapQueryCRUD queryCRUD;

	private QueryParameterFactory<MapIcon, Integer> mapIconIDParamFactory;
	private QueryParameterFactory<IconMapQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<IconMapQuery, String> geometryEndpointNameParamFactory;
	private QueryParameterFactory<IconMapQueryInstance, Integer> queryInstanceIDParamFactory;

	@InstanceManagerDependency
	private RemoteGeometryEndpointProvider remoteGeometryEndpointProvider;

	private OrigoMapProvider origoMapProvider;

	@InstanceManagerDependency
	private LMSearchProvider lmSearchProvider;

	private IconMapQueryReservationHandler reservationHandler;

	@Override
	protected synchronized void moduleConfigured() throws Exception {

		super.moduleConfigured();

		mapConfiguration = getConfiguration(mapConfigurationPath, mapConfigurationPathType, "map");

		if (origoMapProvider != null && mapConfiguration != null) {

			origoMapProvider.updateModuleSettings(mapConfiguration);
		}

		printConfiguration = getConfiguration(printConfigurationPath, printConfigurationPathType, "print");

	}

	private String getConfiguration(String path, PathType pathType, String configurationLogName) {

		if (pathType != null && path != null) {

			File file = null;

			try {

				if (pathType == PathType.Filesystem) {

					file = new File(path);

				} else if (pathType == PathType.RealtiveFilesystem) {

					file = new File(systemInterface.getApplicationFileSystemPath() + path);

				} else if (pathType == PathType.Classpath) {

					URL url = this.getClass().getResource(path);

					if (url == null) {

						log.error(configurationLogName + " configuration " + path + " not found, please check modulesettings");

						return null;

					}

					return StringUtils.readStreamAsString(url.openStream());
				}

				if (file == null) {

					log.error("Unable to found " + configurationLogName + " configuration " + path);

					return null;

				}

				return StringUtils.readFileAsString(file);

			} catch (Exception e) {

				log.error("Unable to get " + configurationLogName + " configuration " + path, e);

			}

		}

		return null;

	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, IconMapQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		mapIconDAO = daoFactory.getDAO(MapIcon.class);
		queryDAO = daoFactory.getDAO(IconMapQuery.class);
		queryInstanceDAO = daoFactory.getDAO(IconMapQueryInstance.class);

		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());

		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);

		mapIconIDParamFactory = mapIconDAO.getParamFactory("mapIconID", Integer.class);
		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		geometryEndpointNameParamFactory = queryDAO.getParamFactory("geometryEndpointName", String.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);

		queryCRUD = new IconMapQueryCRUD(queryDAO.getWrapper(Integer.class), this);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		IconMapQuery query = new IconMapQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		IconMapQuery query = new IconMapQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));

		contentFilter.filterHTML(query);

		List<Integer> oldMapIconIDs = new ArrayList<>();

		if (query.getIcons() != null) {

			for (MapIcon mapIcon : query.getIcons()) {

				oldMapIconIDs.add(mapIcon.getMapIconID());

				mapIcon.setMapIconID(null);
			}
		}

		this.queryDAO.add(query, transactionHandler, null);

		if (query.getIcons() != null) {

			Map<Integer, Integer> mapIconsConversionMap = new HashMap<>(oldMapIconIDs.size());

			int index = 0;

			for (Integer oldMapIconID : oldMapIconIDs) {

				mapIconsConversionMap.put(oldMapIconID, query.getIcons().get(index).getMapIconID());

				index++;
			}

			query.setMapIconsConversionMap(mapIconsConversionMap);
		}

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		IconMapQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		IconMapQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		IconMapQueryInstance queryInstance = null;

		// Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new IconMapQueryInstance();

			queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

			if (queryInstance.getQuery() == null) {

				return null;
			}

			queryInstance.set(descriptor);
			queryInstance.copyQueryValues();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}

			queryInstance.setupPrintableGeometries();

			queryInstance.set(descriptor);

		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		return queryInstance;
	}

	@Override
	public QueryResponse getFormHTML(IconMapQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		String geometryEndpointName = queryInstance.getQuery().getGeometryEndpointName();

		if (remoteGeometryEndpointProvider != null && !StringUtils.isEmpty(geometryEndpointName)) {

			List<RemoteGeometry> geometries = remoteGeometryEndpointProvider.getGeometries(geometryEndpointName, attributeHandler, poster);

			queryInstance.setFetchedGeometries(geometries);

			String endpointHash = remoteGeometryEndpointProvider.getEndpointHash(geometryEndpointName, attributeHandler, poster);

			if (queryInstance.getLastUsedEndpointHash() == null || !queryInstance.getLastUsedEndpointHash().equals(endpointHash)) {

				removeUnavailableSelectedRemoteGeometries(queryInstance, poster);
			}

			queryInstance.setLastUsedEndpointHash(endpointHash);

			if (CollectionUtils.isEmpty(geometries)) {

				validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("NoRemoteGeometries", queryInstance.getQuery().getNoGeometriesMessage()));
			}

		}

		if (isReservationEnabled(queryInstance)) {

			try {

				reservationHandler.validateForReservation(queryInstance, poster, requestMetadata);

			} catch (ValidationException e) {

				validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, e.getErrors());
			}

			reservationHandler.deleteNoLongerSelectedReservations(queryInstance);

			reservationHandler.setCurrentlyReservedGeometryColors(queryInstance);
		}

		if (queryInstance.getSubmitValidationErrors() != null) {

			validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, queryInstance.getSubmitValidationErrors());
			queryInstance.setSubmitValidationErrors(null);
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	private void removeUnavailableSelectedRemoteGeometries(IconMapQueryInstance queryInstance, User poster) {

		if (queryInstance.getSelectedGeometries() == null || queryInstance.getFetchedGeometries() == null) {

			return;
		}

		for (Iterator<SelectedRemoteGeometry> it = queryInstance.getSelectedGeometries().iterator(); it.hasNext();) {

			SelectedRemoteGeometry selectedRemoteGeometry = it.next();

			boolean selectedRemoteGeometryIsStillAvailable = CollectionUtils.find(queryInstance.getFetchedGeometries(), remoteGeometry -> remoteGeometry.getObjectID().equals(selectedRemoteGeometry.getObjectID())) == null;

			if (selectedRemoteGeometryIsStillAvailable) {

				log.info("Removing no longer available selectedRemoteGeometry " + selectedRemoteGeometry + " after user " + poster + " fetched new geometries");
				it.remove();
			}
		}
	}

	@Override
	protected void appendQueryInstance(IconMapQueryInstance queryInstance, Document doc, Element targetElement, AttributeHandler attributeHandler) {

		super.appendQueryInstance(queryInstance, doc, targetElement, attributeHandler);

		if (isReservationEnabled(queryInstance)) {

			XMLUtils.appendNewElement(doc, targetElement, "ReservationEnabled", true);
		}

		if (isPaymentEnabled(queryInstance)) {

			XMLUtils.appendNewElement(doc, targetElement, "PaymentEnabled", true);
		}
	}

	private boolean isPaymentEnabled(IconMapQueryInstance queryInstance) {

		if (!queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getStep().getFlow().isPaymentSupportEnabled()) {

			return false;
		}

		boolean noFetchedGeometriesHasPrice = CollectionUtils.find(queryInstance.getFetchedGeometries(), geometry -> geometry.getPrice() != null) == null;

		if (noFetchedGeometriesHasPrice) {

			boolean noSelectedGeometriesHasPrice = CollectionUtils.find(queryInstance.getSelectedGeometries(), geometry -> geometry.getPrice() != null) == null;

			if (noSelectedGeometriesHasPrice) {

				return false;
			}
		}

		return true;
	}

	private IconMapQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<IconMapQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private IconMapQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<IconMapQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private IconMapQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<IconMapQueryInstance> query = new HighLevelQuery<>();

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		query.addRelations(GET_QUERY_INSTANCE_RELATION_QUERY);

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(IconMapQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (isReservationEnabled(queryInstance)) {

			try {

				reservationHandler.reserveSelectedGeometries(queryInstance, requestMetadata.getFlowInstanceManagerID());

			} catch (ValidationException e) {

				log.info("Validation error when creating reservations while saving query instance " + queryInstance + ": " + e);
			}
		}

		//Check if the query instance has an ID set and if the ID of the descriptor has changed
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
		}
	}

	@Override
	public void populate(IconMapQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		if (isReservationEnabled(queryInstance)) {

			try {

				reservationHandler.validateForReservation(queryInstance, poster, requestMetadata);

			} catch (ValidationException e) {

				return;
			}
		}

		IconMapQuery query = queryInstance.getQuery();
		String parmPrefix = "q" + query.getQueryID() + "_";

		String extent = req.getParameter(parmPrefix + "extent");
		String epsg = req.getParameter(parmPrefix + "epsg");
		String baseLayer = req.getParameter(parmPrefix + "baseLayer");
		String propertyUnitDesignation = req.getParameter(parmPrefix + "propertyUnitDesignation");
		String pudObjectID = req.getParameter(parmPrefix + "pudObjectID");

		String[] geometries = req.getParameterValues(parmPrefix + "geometry");

		GeometryFactory geometryFactory = new GeometryFactory();

		WKTReader reader = new WKTReader(geometryFactory);

		List<Geometry> iconGeometries = null;
		List<SelectedRemoteGeometry> selectedGeometries = null;

		if (!ArrayUtils.isEmpty(geometries)) {

			if (query.isUseSelectableGeometries()) {

				selectedGeometries = populateSelectedGeometries(queryInstance, geometries);

			} else {

				iconGeometries = populateIconGeometries(query, geometries, reader, req);

			}

		}

		queryInstance.setGeometries(iconGeometries);
		queryInstance.setSelectedGeometries(selectedGeometries);
		queryInstance.setExtent(extent);
		queryInstance.setEpsg(epsg);
		queryInstance.setVisibleBaseLayer(baseLayer);

		if (query.isRequirePUD()) {
			queryInstance.setPropertyUnitDesignation(propertyUnitDesignation);
			queryInstance.setPudObjectID(pudObjectID);
		}

		if (isReservationEnabled(queryInstance)) {

			reservationHandler.reserveSelectedGeometries(queryInstance, requestMetadata.getFlowInstanceManagerID());
		}

		boolean requireFields = queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && !allowPartialPopulation;

		if (requireFields) {

			if (query.isRequirePUD() && (StringUtils.isEmpty(propertyUnitDesignation) || StringUtils.isEmpty(pudObjectID))) {

				throw new ValidationException(new ValidationError("PUDRequired"));

			}
			if (query.getIcons() != null && CollectionUtils.isEmpty(iconGeometries) || query.isUseSelectableGeometries() && CollectionUtils.isEmpty(selectedGeometries) && query.getMinSelectableGeometries() != null) {

				boolean geometryRequired = false;
				
				if(query.getIcons() != null) {
					for (MapIcon mapIcon : query.getIcons()) {
						if(mapIcon.getMinIcons() == null || mapIcon.getMinIcons() > 0) {
							geometryRequired = true;
							break;
						}
					}
				} else {
					geometryRequired = true;
				}
				
				if(geometryRequired) {
					throw new ValidationException(new ValidationError("GeometryRequired"));
				}
			}
		}

		if (CollectionUtils.isEmpty(iconGeometries) && CollectionUtils.isEmpty(selectedGeometries) && StringUtils.isEmpty(propertyUnitDesignation) && !query.isForceQueryPopulation()) {

			queryInstance.reset(attributeHandler);

			return;
		}

		if (!allowPartialPopulation) {

			List<ValidationError> errors = new ArrayList<>();

			if (iconGeometries != null) {

				for (MapIcon mapIcon : query.getIcons()) {

					if (mapIcon.getMinIcons() != null || mapIcon.getMaxIcons() != null) {

						int count = 0;

						for (Geometry geometry : iconGeometries) {

							if (mapIcon.getMapIconID().equals(geometry.getMapIcon().getMapIconID())) {

								count++;
							}

						}

						if (mapIcon.getMinIcons() != null && count < mapIcon.getMinIcons()) {

							errors.add(new MapIconValidationError("ToFewIcons", mapIcon));
						}

						if (mapIcon.getMaxIcons() != null && count > mapIcon.getMaxIcons()) {

							errors.add(new MapIconValidationError("ToManyIcons", mapIcon));
						}

					}

				}

			} else if (selectedGeometries != null && (query.getMinSelectableGeometries() != null || query.getMaxSelectableGeometries() != null)) {

				if (query.getMinSelectableGeometries() != null && selectedGeometries.size() < query.getMinSelectableGeometries()) {

					errors.add(new SelectedRemoteGeometryValidationError("ToFewSelected", query.getMinSelectableGeometries(), query.getMaxSelectableGeometries()));
				}

				if (query.getMaxSelectableGeometries() != null && selectedGeometries.size() > query.getMaxSelectableGeometries()) {

					errors.add(new SelectedRemoteGeometryValidationError("ToManySelected", query.getMinSelectableGeometries(), query.getMaxSelectableGeometries()));
				}

			} else if (CollectionUtils.isEmpty(selectedGeometries) && query.getMinSelectableGeometries() != null && query.getMinSelectableGeometries() > 0) {

				errors.add(new SelectedRemoteGeometryValidationError("ToFewSelected", query.getMinSelectableGeometries(), query.getMaxSelectableGeometries()));

			}

			if (!errors.isEmpty()) {

				throw new ValidationException(errors);
			}

		}

		if (queryInstance.getQuery().isRequirePUD() && !StringUtils.isEmpty(queryInstance.getPudObjectID())) {

			List<UnitArea> unitAreas = lmSearchProvider.getLmConnection().getUnitAreasByRegisterUnitID(queryInstance.getPudObjectID());

			if (unitAreas == null) {

				throw new ValidationException(new ValidationError("InCompleteMapQuerySubmit"));
			}
			List<com.vividsolutions.jts.geom.Geometry> pudGeometries = new ArrayList<>(unitAreas.size());

			for (UnitArea unitArea : unitAreas) {

				if (unitArea.getGeometry() != null) {

					pudGeometries.add(unitArea.getGeometry());
				}

			}

			queryInstance.setPudGeometries(pudGeometries);

		}

		if (StringUtils.isEmpty(queryInstance.getExtent()) || StringUtils.isEmpty(queryInstance.getEpsg())) {

			throw new ValidationException(new ValidationError("InCompleteMapQuerySubmit"));

		}

		queryInstance.getQueryInstanceDescriptor().setPopulated(true);

		if (query.isSetAsAttribute()) {

			queryInstance.resetAttribute(attributeHandler);
			queryInstance.setAttribute(attributeHandler);
		}

		generateMapImages(queryInstance, req, user);
	}

	@WebPublic(alias = "icon")
	public ForegroundModuleResponse getMapIcon(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, AccessDeniedException, ModuleConfigurationException, SQLException, IOException {

		Integer mapIconID = null;

		if (uriParser.size() == 3 && (mapIconID = NumberUtils.toInt(uriParser.get(2))) != null) {

			MapIcon mapIcon = getMapIcon(mapIconID);

			if (mapIcon != null && mapIcon.getIconImage() != null) {

				InputStream in = null;
				OutputStream out = null;

				try {

					HTTPUtils.setContentLength(mapIcon.getIconImage().length(), res);

					res.setContentType(MimeUtils.getMimeType(mapIcon.getIconImageFilename()));
					res.setHeader("Content-Disposition", "inline; filename=\"" + FileUtils.toValidHttpFilename(mapIcon.getIconImageFilename()) + "\"");

					in = mapIcon.getIconImage().getBinaryStream();
					out = res.getOutputStream();

					StreamUtils.transfer(in, out);

				} catch (RuntimeException | IOException e) {

					log.debug("Caught exception " + e + " while sending image " + mapIcon.getIconImageFilename() + " to " + user);

				} finally {

					CloseUtils.close(in);
					CloseUtils.close(out);
				}

				return null;
			}
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		IconMapQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		IconMapQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		IconMapQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		if (query.getIcons() != null) {

			for (MapIcon mapIcon : query.getIcons()) {

				mapIcon.setMapIconID(null);
			}
		}

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, IconMapQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected List<PDFAttachment> getPDFAttachments(IconMapQueryInstance queryInstance) {

		List<PDFAttachment> attachments = new ArrayList<>();

		if (queryInstance.getMapPrints() != null) {

			for (IconMapQueryInstancePrint queryInstancePrint : queryInstance.getMapPrints()) {

				if (queryInstancePrint.getMapImage() != null) {

					String namePrefix;

					if (queryInstancePrint.getScale() != null) {
						namePrefix = this.pdfAttachmentFilename.replace("$scale", queryInstancePrint.getScale() + "");
					} else {
						namePrefix = this.pdfAttachmentFilenameWithoutScale;
					}

					attachments.add(new BlobPDFAttachment(queryInstancePrint.getMapImage(), namePrefix + "." + queryInstancePrint.getFormat(), this.pdfAttachmentDescriptionPrefix + " " + queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getName()));
				}

			}

		}

		if (!attachments.isEmpty()) {

			return attachments;
		}

		return null;
	}

	@Override
	protected PDFResourceProvider getPDFResourceProvider(IconMapQueryInstance queryInstance) {

		if (queryInstance.getMapPrints() != null) {

			for (IconMapQueryInstancePrint queryInstancePrint : queryInstance.getMapPrints()) {

				if (queryInstancePrint.isUseInPreview() && queryInstancePrint.getMapImage() != null) {

					return new BlobResourceProvider(queryInstancePrint.getMapImage());
				}

			}

		}

		return null;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(IconMapQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws Exception {

		String action = req.getParameter("action");

		if ("mapimage".equals(action) && queryInstance.getMapPrints() != null) {

			for (IconMapQueryInstancePrint queryInstancePrint : queryInstance.getMapPrints()) {

				if (queryInstancePrint.isUseInPreview() && queryInstancePrint.getMapImage() != null) {

					return new MapImageRequestProcessor("mapimage", queryInstancePrint.getMapImage());
				}
			}

		} else if (isReservationEnabled(queryInstance) && GeometryReservationRequestProcessor.isGeometryReservationRequest(action)) {

			return new GeometryReservationRequestProcessor(reservationHandler, queryInstance, requestMetadata, poster, action);
		}

		return null;

	}

	@Override
	public Document createDocument(HttpServletRequest req, User poster) {

		Document doc = super.createDocument(req, poster);

		Element document = doc.getDocumentElement();

		if (origoMapProvider != null) {

			XMLUtils.appendNewElement(doc, document, "origoBaseURL", origoMapProvider.getStaticContentURL());
			XMLUtils.appendNewElement(doc, document, "mapConfiguration", origoMapProvider.getMapConfiguration());

		} else {

			log.error("Missing OrigoMapProvider");
		}

		return doc;
	}

	private List<Geometry> populateIconGeometries(IconMapQuery query, String[] geometries, WKTReader reader, HttpServletRequest req) throws ValidationException {

		List<Geometry> populatedGeometries = new ArrayList<>(geometries.length);

		for (String value : geometries) {

			String[] valueParts = value.split("#");

			if (valueParts.length > 2) {

				Integer mapIconID = NumberUtils.toInt(valueParts[2].trim());

				if (mapIconID != null && query.getIcons() != null) {

					MapIcon mapIcon = null;

					for (MapIcon icon : query.getIcons()) {

						if (icon.getMapIconID().equals(mapIconID)) {

							mapIcon = icon;
						}

					}

					if (mapIcon != null) {

						String geometryStr = valueParts[1].trim();

						com.vividsolutions.jts.geom.Geometry geometry = populateGeometry(reader, geometryStr);

						Integer index = NumberUtils.toInt(valueParts[0].trim());

						Geometry iconGeometry = new Geometry(mapIcon, geometryStr, geometry, index);

						if (query.isUseDetailedIconInfo()) {

							String comment = req.getParameter("q" + query.getQueryID() + "_comment_" + index);

							if (!StringUtils.isEmpty(comment)) {
								iconGeometry.setComment(StringUtils.substring(comment, 1024, ".."));
							}

							if (query.isShowIconPUDInfo()) {

								String propertyUnitDesignation = req.getParameter("q" + query.getQueryID() + "_pud_" + index);

								if (!StringUtils.isEmpty(propertyUnitDesignation)) {
									iconGeometry.setPropertyUnitDesignation(propertyUnitDesignation);
								}

							}

						}

						populatedGeometries.add(iconGeometry);

					}

				}

			}

		}

		return populatedGeometries;

	}

	private com.vividsolutions.jts.geom.Geometry populateGeometry(WKTReader reader, String geometry) throws ValidationException {

		try {

			return reader.read(geometry);

		} catch (Exception e) {

			throw new ValidationException(new ValidationError("GeometryNotValid"));

		}

	}

	private List<SelectedRemoteGeometry> populateSelectedGeometries(IconMapQueryInstance queryInstance, String[] geometries) throws ValidationException {

		if (queryInstance.getFetchedGeometries() == null) {

			return null;
		}

		List<SelectedRemoteGeometry> populatedGeometries = new ArrayList<>(geometries.length);

		for (String value : geometries) {

			String[] valueParts = value.split("#");

			if (valueParts.length == 2) {

				String objectID = valueParts[1];
				Integer index = NumberUtils.toInt(valueParts[0].trim());

				RemoteGeometry remoteGeometry = null;

				for (RemoteGeometry geometry : queryInstance.getFetchedGeometries()) {

					if (geometry.getObjectID().equals(objectID)) {

						remoteGeometry = geometry;
					}

				}

				if (remoteGeometry == null || !remoteGeometry.isSelectable()) {

					throw new ValidationException(new ValidationError("GeometryNotValid"));
				}

				populatedGeometries.add(new SelectedRemoteGeometry(remoteGeometry, index));
			}
		}

		return populatedGeometries;
	}

	public void generateMapImages(IconMapQueryInstance queryInstance, HttpServletRequest req, User user) throws ValidationException {

		log.info("Generating map images for queryInstance " + queryInstance + " for user " + user);

		String fullModuleURL = RequestUtils.getFullContextPathURL(req) + getFullAlias();
		MapImageDataGenerator mapImageDataGenerator = new MapImageDataGenerator(fullModuleURL);
		MapImageData mapImageData = mapImageDataGenerator.generateMapImageData(queryInstance);

		List<IconMapQueryInstancePrint> queryInstancePrints = new ArrayList<>(2);

		queryInstancePrints.add(generateMapImage(queryInstance, user, mapImageData, pngMapImageResolution, pngMapImageScale, pngMapImageLayout, "png", true));

		if (pdfMapImageResolution != null && pdfMapImageLayout != null) {

			queryInstancePrints.add(generateMapImage(queryInstance, user, mapImageData, pdfMapImageResolution, pdfMapImageScale, pdfMapImageLayout, "pdf", false));
		}

		queryInstance.setMapPrints(queryInstancePrints);
	}

	private IconMapQueryInstancePrint generateMapImage(IconMapQueryInstance queryInstance, User user, MapImageData mapImageData, Integer resolution, Integer scale, String layout, String outputFormat, boolean useInPreview) throws ValidationException {

		String templateConfig = printConfiguration;

		if (scale == null) {
			templateConfig = templateConfig.replace("$bbox,", "bbox: [" + mapImageData.getExtent().getMinX() + "," + mapImageData.getExtent().getMinY() + "," + mapImageData.getExtent().getMaxX() + "," + mapImageData.getExtent().getMaxY() + "],");
			templateConfig = templateConfig.replace("$scale,", "").replace("$center,", "");
		} else {
			templateConfig = templateConfig.replace("$center,", "center: [" + mapImageData.getCenterCoordinate().x + "," + mapImageData.getCenterCoordinate().y + "],");
			templateConfig = templateConfig.replace("$scale,", "scale: " + scale + ",");
			templateConfig = templateConfig.replace("$bbox,", "");
		}

		templateConfig = templateConfig.replace("$styles", mapImageData.getStyles().toJson());
		templateConfig = templateConfig.replace("$features", mapImageData.getFeatures().toJson());
		templateConfig = templateConfig.replace("$srs", queryInstance.getEpsg());
		templateConfig = templateConfig.replace("$dpi", resolution + "");
		templateConfig = templateConfig.replace("$layout", layout);
		templateConfig = templateConfig.replace("$outputFormat", outputFormat);

		if (logPrintConfig) {

			log.info("Generated print config (png) for queryInstance " + queryInstance + ": " + templateConfig);
		}

		Blob mapImage = getMapImageFromMapFish(queryInstance, user, templateConfig);

		try {

			if (mapImage == null || mapImage.length() == 0) {

				throw new ValidationException(new ValidationError("UnableToGeneratePNG"));
			}

		} catch (SQLException e) {

			throw new ValidationException(new ValidationError("UnableToGeneratePNG"));
		}

		IconMapQueryInstancePrint print = new IconMapQueryInstancePrint();

		print.setMapImage(mapImage);
		print.setFormat(outputFormat);
		print.setLayout(layout);
		print.setResolution(resolution);
		print.setScale(scale);
		print.setUseInPreview(useInPreview);

		return print;

	}

	private SerialBlob getMapImageFromMapFish(IconMapQueryInstance queryInstance, User user, String config) throws ValidationException {

		ByteArrayOutputStream outputStream = null;

		try {

			StringReader reader = new StringReader(config);

			StringWriter writer = new StringWriter();

			String url = printServiceAddress + "/pdf/create.json";

			HTTPUtils.sendHTTPPostRequest(reader, new URL(url), writer, "UTF-8", mapFishConnectionTimeout, mapFishReadTimeout);

			String mapImageURL = writer.toString();

			if (mapImageURL != null) {

				mapImageURL = mapImageURL.substring(11, mapImageURL.length() - 2);

				if (HTTPUtils.isValidURL(mapImageURL)) {

					log.info("Generatated map image: " + mapImageURL + " for queryInstance " + queryInstance + " for user " + user);

					outputStream = new ByteArrayOutputStream();

					HTTPUtils.sendHTTPGetRequest(mapImageURL, null, outputStream);

					return new SerialBlob(outputStream.toByteArray());

				}

			}

			log.error("Invalid response from print service whenn generating png for queryInstance " + queryInstance + " for user " + user);

		} catch (SocketTimeoutException e) {

			log.log(mapFishTimeoutLogLevel.getLevel(), "Unable to generate png for queryInstance " + queryInstance + " for user " + user, e);

		} catch (Exception e) {

			log.log(mapFishErrorLogLevel.getLevel(), "Unable to generate png for queryInstance " + queryInstance + " for user " + user, e);

		} finally {

			CloseUtils.close(outputStream);

		}

		throw new ValidationException(new ValidationError("UnableToGeneratePNG"));

	}

	protected MapIcon getMapIcon(Integer mapIconID) throws SQLException {

		HighLevelQuery<MapIcon> query = new HighLevelQuery<>();

		query.addParameter(mapIconIDParamFactory.getParameter(mapIconID));

		return mapIconDAO.get(query);
	}

	@Override
	protected Class<IconMapQueryInstance> getQueryInstanceClass() {

		return IconMapQueryInstance.class;
	}

	public MultipartRequest parseMultipartRequest(HttpServletRequest req, User user) throws FileUploadException {

		return MultipartRequest.getMultipartRequest(ramThreshold * BinarySizes.KiloByte, diskThreshold * BinarySizes.MegaByte, (long) (maxFileSize * BinarySizes.MegaByte), tempDir, req);
	}

	public Integer getMaxFileSize() {

		return maxFileSize;
	}

	public int getMaxMapIconHeight() {

		return maxMapIconHeight;
	}

	public int getMaxMapIconWidth() {

		return maxMapIconWidth;
	}

	public RemoteGeometryEndpointProvider getRemoteGeometryEndpointProvider() {

		return remoteGeometryEndpointProvider;
	}

	@Override
	public boolean hasQueriesUsingEndpoint(RemoteGeometryEndpoint endpoint) throws SQLException {

		HighLevelQuery<IconMapQuery> query = new HighLevelQuery<>();
		query.addParameter(geometryEndpointNameParamFactory.getParameter(endpoint.getName()));
		query.setRowLimiter(MySQLRowLimiter.SINGLE_ROW);
		return queryDAO.getBoolean(query);
	}	
	
	@Override
	public List<RemoteGeometryQuery> getQueriesUsingEndpoint(RemoteGeometryEndpoint endpoint) throws SQLException {

		HighLevelQuery<IconMapQuery> getAllQuery = new HighLevelQuery<>();

		getAllQuery.addParameter(geometryEndpointNameParamFactory.getParameter(endpoint.getName()));

		List<IconMapQuery> queries = queryDAO.getAll(getAllQuery);

		if (queries == null) {

			return null;
		}

		for (IconMapQuery query : queries) {

			QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(query.getQueryID());

			if (queryDescriptor != null) {

				query.init(queryDescriptor, getFullAlias() + "/config/" + query.getQueryID());
			}
		}

		return Collections.unmodifiableList(queries);
	}

	public String getMissingEndpoint() {

		return missingEndpoint;
	}

	@InstanceManagerDependency(required = true)
	public void setOrigoMapProvider(OrigoMapProvider origoMapProvider) throws Exception {

		this.origoMapProvider = origoMapProvider;

		if (origoMapProvider != null && mapConfiguration != null) {

			origoMapProvider.updateModuleSettings(mapConfiguration);
		}
	}

	@InstanceManagerDependency
	public void setReservationProvider(ReservationProvider reservationProvider) throws Exception {

		if (reservationProvider != null) {

			reservationHandler = new IconMapQueryReservationHandler(reservationProvider);

		} else {

			reservationHandler = null;
		}
	}

	public IconMapQueryReservationHandler getReservationHandler() {

		return reservationHandler;
	}

	public boolean isReservationEnabled(IconMapQueryInstance queryInstance) {

		return reservationHandler != null && queryInstance.getQuery().isSelectableGeometryReservationEnabled();
	}

}
