<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/iconmapquery.js?v19
	</xsl:variable>

	<xsl:variable name="links">
		/css/iconmapquery.css?v13
	</xsl:variable>

	<xsl:template match="Document">	
	
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
	
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
	
		<xsl:variable name="shortQueryID" select="concat('q', IconMapQueryInstance/IconMapQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', IconMapQueryInstance/IconMapQuery/queryID)" />
		
		<div class="query icon-map-query loaded" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query icon-map-query loaded</xsl:text>
				<xsl:if test="IconMapQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<xsl:variable name="webMapPreview" select="not(IconMapQueryInstance/mapPrints/IconMapQueryInstancePrint[useInPreview = 'true'])" />
		
			<article>
				
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="IconMapQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="IconMapQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="IconMapQueryInstance/IconMapQuery/queryID" />
					</xsl:call-template>
				
				</div>

				<xsl:if test="IconMapQueryInstance/IconMapQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="IconMapQueryInstance/IconMapQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>
		
				<img src="{queryRequestURL}?action=mapimage" class="full bigmargintop" alt="" />
				
				<xsl:if test="IconMapQueryInstance/propertyUnitDesignation">
					<div class="pudinfo-wrapper">
						<div><xsl:value-of select="$i18n.PropertyUnitDesignation" /><xsl:text>:&#160;</xsl:text><span><xsl:value-of select="IconMapQueryInstance/propertyUnitDesignation" /></span></div>
					</div>
				</xsl:if>

				<xsl:if test="IconMapQueryInstance/IconMapQuery/useDetailedIconInfo = 'true' and IconMapQueryInstance/geometries/Geometry">
				
					<table class="geometry-table oep-table">
						<xsl:if test="IconMapQueryInstance/IconMapQuery/showIconPUDInfo = 'true'">
							<xsl:attribute name="class">geometry-table oep-table show-pud</xsl:attribute>
						</xsl:if>
						<thead>
							<tr>
								<th width="20px">#</th>
								<th width="100px"><xsl:value-of select="$i18n.MapIcon" /></th>
								<th width="300px">
									<span class="marginright"><xsl:value-of select="$i18n.Coordinates" /></span>
									<span class="hide-mobile"><xsl:text> </xsl:text><xsl:value-of select="$i18n.CoordinatesLonLat" /></span>
								</th>
								<xsl:if test="IconMapQueryInstance/IconMapQuery/showIconPUDInfo = 'true'">
									<th width="300px"><xsl:value-of select="$i18n.PropertyUnitDesignation" /></th>
								</xsl:if>
								<th width="300px"><xsl:value-of select="$i18n.Comment" /></th>
								<th width="16px" />
							</tr>
						</thead>
						<tbody>
							<xsl:apply-templates select="IconMapQueryInstance/geometries/Geometry" mode="table" />
						</tbody>
					</table>
					
				</xsl:if>
				
				<xsl:if test="IconMapQueryInstance/IconMapQuery/useSelectableGeometries = 'true' and IconMapQueryInstance/selectedGeometries/SelectedRemoteGeometry">
					
					<table class="geometry-table oep-table full">
						<thead>
							<tr>
								<th width="20px">#</th>
								<th width="300px"><xsl:value-of select="$i18n.Name" /></th>
								<th><xsl:value-of select="$i18n.Description" /></th>
								<xsl:if test="PaymentEnabled">
									<th width="50px"><xsl:value-of select="$i18n.Price" /></th>
								</xsl:if>
								<th width="16px" />
							</tr>
						</thead>
						<tbody>
							<xsl:apply-templates select="IconMapQueryInstance/selectedGeometries/SelectedRemoteGeometry" mode="table">
								<xsl:with-param name="paymentEnabled" select="PaymentEnabled" />
							</xsl:apply-templates>
						</tbody>					
					</table>
					
				</xsl:if>
				
			</article>
				
		</div>
		
	</xsl:template>	
		
	<xsl:template match="Geometry" mode="table">
	
		<tr>
			<td><xsl:value-of select="index" /></td>
			<td data-title="{$i18n.MapIcon}"><xsl:value-of select="MapIcon/name" /></td>
			<td data-title="{$i18n.Coordinates}"><xsl:value-of select="coordinates" /></td>
			<xsl:if test="../../IconMapQuery/showIconPUDInfo = 'true'">
				<td data-title="{$i18n.PropertyUnitDesignation}">
					<xsl:choose>
						<xsl:when test="propertyUnitDesignation"><xsl:value-of select="propertyUnitDesignation" /></xsl:when>
						<xsl:otherwise>-</xsl:otherwise>
					</xsl:choose>
				</td>
			</xsl:if>
			<td data-title="{$i18n.Comment}">
				<xsl:choose>
					<xsl:when test="comment"><xsl:value-of select="comment" /></xsl:when>
					<xsl:otherwise>-</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="SelectedRemoteGeometry" mode="table">
		<xsl:param name="paymentEnabled" select="false()" />
	
		<tr>
			<td><xsl:value-of select="index" /></td>
			<td data-title="{$i18n.Name}"><xsl:value-of select="name" disable-output-escaping="yes" /></td>
			<td data-title="{$i18n.Description}"><xsl:value-of select="description" disable-output-escaping="yes" /></td>
			<xsl:if test="$paymentEnabled and price">
				<td data-title="{$i18n.Price}"><xsl:value-of select="price" disable-output-escaping="yes" /><xsl:value-of select="$i18n.CurrencySymbol" /></td>
			</xsl:if>
		</tr>
	
	</xsl:template>	
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', IconMapQueryInstance/IconMapQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', IconMapQueryInstance/IconMapQuery/queryID)" />
	
		<div class="query icon-map-query" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query icon-map-query</xsl:text>
				<xsl:if test="IconMapQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<a name="{$queryID}" />
	
			<div id="{$shortQueryID}-validationerrors">
				<xsl:choose>
					<xsl:when test="ValidationErrors/validationError">
						<div id="{$queryID}-validationerrors" class="validationerrors">
							<div class="info-box error">
								<xsl:apply-templates select="ValidationErrors/validationError" />
								<div class="marker"></div>
							</div>
						</div>
					</xsl:when>
					<xsl:when test="IconMapQueryInstance/initializeErrorMessage">
						<div id="{$queryID}-validationerrors" class="validationerrors">
							<div class="info-box error">
								<span>
									<strong data-icon-before="!">
										<xsl:value-of select="IconMapQueryInstance/initializeErrorMessage" />
									</strong>
								</span>
								<div class="marker"></div>
							</div>
						</div>
					</xsl:when>
				</xsl:choose>
			</div>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError or IconMapQueryInstance/initializeErrorMessage">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="IconMapQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="IconMapQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="IconMapQueryInstance/IconMapQuery/helpText">		
						<xsl:apply-templates select="IconMapQueryInstance/IconMapQuery/helpText" />
					</xsl:if>
				
				</div>

				<xsl:if test="IconMapQueryInstance/IconMapQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="IconMapQueryInstance/IconMapQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
		
				<div id="{$shortQueryID}" class="mapquery">
				
					<xsl:if test="IconMapQueryInstance/IconMapQuery/Icons/MapIcon">
				
						<div class="draw-icon-toolbar">
						    <xsl:apply-templates select="IconMapQueryInstance/IconMapQuery/Icons/MapIcon" mode="tool" />
						    <button id="modify" type="button"  title="{$i18n.Move}" data-id="modify">
						    	<span class="icon"><img src="{$imagePath}/arrow.png" /></span>
						    	<span class="text"><xsl:value-of select="$i18n.Move" /></span>
						    </button>
						    <button id="select" type="button" title="{$i18n.Delete}" data-id="select">
						    	<span class="icon"><img src="{$imagePath}/trash.png" /></span>
						    	<span class="text"><xsl:value-of select="$i18n.Delete" /></span>
						    </button>
						</div>
					
					</xsl:if>
					
					<div id="app-wrapper" class="o-map app-wrapper">
											
					</div>

					<div class="popup">
						<div class="arrow" />
						<h3 class="popup-title"><span /><a class="close" href="#" data-icon-after="x"></a></h3>
						<div class="popup-content">
							<span />
							<xsl:if test="IconMapQueryInstance/IconMapQuery/useSelectableGeometries = 'true'">
								<input type="button" class="select-btn btn btn-blue" value="{$i18n.Select}" /><input type="button" class="unselect-btn btn btn-blue" style="display:none;" value="{$i18n.UnSelect}" />
								<div class="clearboth" />
							</xsl:if>
						</div>
					</div>

				</div>
		
				<xsl:if test="IconMapQueryInstance/IconMapQuery/useDetailedIconInfo = 'true'">
				
					<table class="geometry-table oep-table">
						<xsl:if test="IconMapQueryInstance/IconMapQuery/showIconPUDInfo = 'true'">
							<xsl:attribute name="class">geometry-table oep-table show-pud</xsl:attribute>
						</xsl:if>
						<thead>
							<tr>
								<th width="20px">#</th>
								<th width="100px"><xsl:value-of select="$i18n.MapIcon" /></th>
								<th width="300px">
									<span class="marginright"><xsl:value-of select="$i18n.Coordinates" /></span>
									<span class="hide-mobile"><xsl:text> </xsl:text><xsl:value-of select="$i18n.CoordinatesLonLat" /></span>
									<a class="open-help" href="#" data-icon-after="?" data-help-box="query_{$queryID}_coordinates"><span /></a>
									<div class="help-box" data-help-box="query_{$queryID}_coordinates">
										<div>
								  			<div> 
								  				<a class="close" href="#" data-icon-after="x"></a> 
								  				<xsl:value-of select="$i18n.CoordinatesDescription" />
								  			</div> 
										</div>
									</div>
									<div class="help-backdrop" data-help-box="query_{$queryID}_coordinates" />
								</th>
								<xsl:if test="IconMapQueryInstance/IconMapQuery/showIconPUDInfo = 'true'">
									<th width="300px"><xsl:value-of select="$i18n.PropertyUnitDesignation" /></th>
								</xsl:if>
								<th width="500px"><xsl:value-of select="$i18n.Comment" /></th>
								<th width="16px" />
							</tr>
						</thead>
						<tbody>
							<tr class="empty-row"><td colspan="5"><xsl:value-of select="$i18n.NoDrawnGeometries" /></td></tr>
						</tbody>
					</table>
					
				</xsl:if>
				
				<xsl:if test="IconMapQueryInstance/IconMapQuery/useSelectableGeometries = 'true'">
					
					<table class="geometry-table oep-table hidden full">
						<thead>
							<tr>
								<th width="20px">#</th>
								<th width="300px"><xsl:value-of select="$i18n.Name" /></th>
								<th><xsl:value-of select="$i18n.Description" /></th>
								<xsl:if test="PaymentEnabled">
									<th width="50px"><xsl:value-of select="$i18n.Price" /></th>
								</xsl:if>
								<th width="16px" />
							</tr>
						</thead>
						<tbody>
						</tbody>					
					</table>
					
				</xsl:if>
		
				<xsl:if test="IconMapQueryInstance/geometries/Geometry">
					<xsl:for-each select="IconMapQueryInstance/geometries/Geometry">
						<input type="hidden" id="{$shortQueryID}_geometry_{index}" name="{$shortQueryID}_geometry" disabled="disabled" value="{index}#{geometry}#{MapIcon/mapIconID}" />
						<xsl:if test="comment">
							<input type="hidden" id="{$shortQueryID}_geometry_{index}_comment" name="{$shortQueryID}_comment" disabled="disabled" value="{comment}" />
						</xsl:if>
					</xsl:for-each>
				</xsl:if>
			
				<input id="{$shortQueryID}_extent" type="hidden" name="{$shortQueryID}_extent" value="{IconMapQueryInstance/extent}" />
				<input id="{$shortQueryID}_epsg" type="hidden" name="{$shortQueryID}_epsg" value="{IconMapQueryInstance/epsg}" />
				<input id="{$shortQueryID}_baseLayer" type="hidden" name="{$shortQueryID}_baseLayer" value="{IconMapQueryInstance/visibleBaseLayer}" />
				
				<xsl:if test="IconMapQueryInstance/IconMapQuery/requirePUD = 'true'">
					<input id="{$shortQueryID}_propertyUnitDesignation" type="hidden" name="{$shortQueryID}_propertyUnitDesignation" value="{IconMapQueryInstance/propertyUnitDesignation}" />
					<input id="{$shortQueryID}_pudObjectID" type="hidden" name="{$shortQueryID}_pudObjectID" value="{IconMapQueryInstance/pudObjectID}" />
				</xsl:if>				
				
				<div class="geometry-data">
					
				</div>
				
				<link href="{/Document/origoBaseURL}/css/customstyle.css" rel="stylesheet" />
				
			</article>
		
			<script type="text/javascript">
			
				iconMapQueryLanguage = {
					'MAXGEOMETRIES_MESSAGE' : '<xsl:value-of select="$i18n.MaxGeometriesMessage" />',
					'MAXSELECTED_MESSAGE' : '<xsl:value-of select="$i18n.MaxSelectedMessage" />',
					'GETTING_PUD': '<xsl:value-of select="$i18n.GettingPUD" />',
					'PUD_NOT_FOUND': '<xsl:value-of select="$i18n.PUDNotFound" />',
					'PUD_REQUEST_ERROR': '<xsl:value-of select="$i18n.PUDRequestError" />',
					'TITLE': {
						'MAP_ICON': '<xsl:value-of select="$i18n.MapIcon" />',
						'COORDINATES': '<xsl:value-of select="$i18n.Coordinates" />',
						'PROPERTY_UNIT_DESIGNATION': '<xsl:value-of select="$i18n.PropertyUnitDesignation" />',
						'COMMENT': '<xsl:value-of select="$i18n.Comment" />',
						'NAME': '<xsl:value-of select="$i18n.Name" />',
						'DESCRIPTION': '<xsl:value-of select="$i18n.Description" />',
						'PRICE': '<xsl:value-of select="$i18n.Price" />'
					},
					'SELECT': '<xsl:value-of select="$i18n.Select" />',
					'CURRENCY_SYMBOL': '<xsl:value-of select="$i18n.CurrencySymbol" />',
					'HITS_CHOOSE_OBJECT': '<xsl:value-of select="$i18n.HitsChooseSymbol" />'
				};
			
				$(function(){
				
					var configuration = <xsl:value-of select="/Document/mapConfiguration" />;
					
					<xsl:if test="IconMapQueryInstance/IconMapQuery/requirePUD = 'true'">
						if(configuration.controls) {
							var tool = $.grep(configuration.controls, function(obj) {
								return obj.name == "getpudsearch";
							});
							
							if(tool[0]) {
								tool[0].options.persistPUD = true;
							}
						}
					</xsl:if>
				
					var queryOptions = {
						requirePUD: <xsl:value-of select="IconMapQueryInstance/IconMapQuery/requirePUD" />,
						useDetailedIconInfo: <xsl:value-of select="IconMapQueryInstance/IconMapQuery/useDetailedIconInfo" />,
						showIconPUDInfo: <xsl:value-of select="IconMapQueryInstance/IconMapQuery/showIconPUDInfo" />,
						mapconfig: configuration,
						useSelectableGeometries: <xsl:value-of select="IconMapQueryInstance/IconMapQuery/useSelectableGeometries" />,
						zoomToGeometries: <xsl:value-of select="IconMapQueryInstance/IconMapQuery/zoomToGeometries" />,
						missingGeometryMessage: '<xsl:value-of select="IconMapQueryInstance/IconMapQuery/missingGeometryMessage" />',
						origoScriptURL: '<xsl:value-of select="/Document/origoBaseURL"/>/js/origo.min.js',
						baseUrl: '<xsl:value-of select="/Document/origoBaseURL" />/',
						reservationEnabled: <xsl:value-of select="ReservationEnabled = 'true'" />,
						reservationFailedMessage: '<xsl:value-of select="IconMapQueryInstance/IconMapQuery/geometryAlreadyReservedMessage" />',
						createReservationURL: '<xsl:value-of select="queryRequestURL" />?action=createreservation&amp;objectid=',
						deleteReservationURL: '<xsl:value-of select="queryRequestURL" />?action=deletereservation&amp;objectid=',
						paymentEnabled: <xsl:value-of select="PaymentEnabled = 'true'" />,
					};
					
					<xsl:if test="IconMapQueryInstance/FetchedGeometries">
						queryOptions.fetchedGeometries = <xsl:value-of select="IconMapQueryInstance/FetchedGeometries" />;
					</xsl:if>
					
					<xsl:if test="IconMapQueryInstance/SelectedGeometries">
						queryOptions.selectedGeometries = <xsl:value-of select="IconMapQueryInstance/SelectedGeometries" />;
					</xsl:if>
					
					<xsl:if test="IconMapQueryInstance/IconMapQuery/maxSelectableGeometries">
						queryOptions.maxSelectableGeometries = <xsl:value-of select="IconMapQueryInstance/IconMapQuery/maxSelectableGeometries" />;
					</xsl:if>
					
					initIconMapQuery('<xsl:value-of select="IconMapQueryInstance/IconMapQuery/queryID" />', queryOptions);
				});
				
			</script>
						
		</div>
		
	</xsl:template>
	
	<xsl:template match="MapIcon" mode="tool">
		
		<button id="mapicon_{mapIconID}" type="button" title="{name}" class="mapicon" data-id="{mapIconID}">
			<xsl:if test="maxIcons"><xsl:attribute name="data-max"><xsl:value-of select="maxIcons" /></xsl:attribute></xsl:if>
	    	<span class="icon"><img src="{/Document/requestinfo/contextpath}{/Document/fullAlias}/icon/{mapIconID}" /></span>
	    	<span class="text"><xsl:value-of select="name" /></span>
	    </button>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='GeometriesRequired']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="../../IconMapQueryInstance/IconMapQuery/requiredQueryMessage" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='GeometryRequired']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="../../IconMapQueryInstance/IconMapQuery/requiredQueryMessage" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='GeometryNotValid']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.GeometryNotValid" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='RequiredQuery']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="../../IconMapQueryInstance/requiredQueryMessage" />
			</strong>
		</span>
		
	</xsl:template>
	
		<xsl:template match="validationError[messageKey='InCompleteMapQuerySubmit']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.InCompleteMapQuerySubmit" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToGeneratePNG']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnableToGeneratePNG" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ToFewIcons']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ToFewIcons.Part1" /><xsl:text> </xsl:text><xsl:value-of select="MapIcon/minIcons" /><xsl:text> </xsl:text><xsl:value-of select="$i18n.ToFewIcons.Part2" /><xsl:text> </xsl:text><xsl:value-of select="MapIcon/name" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ToFewSelected']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ToFewSelected.Part1" /><xsl:text> </xsl:text><xsl:value-of select="minSelectableGeometries" /><xsl:text> </xsl:text><xsl:value-of select="$i18n.ToFewSelected.Part2" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ToManySelected']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ToManySelected.Part1" /><xsl:text> </xsl:text><xsl:value-of select="maxSelectableGeometries" /><xsl:text> </xsl:text><xsl:value-of select="$i18n.ToManySelected.Part2" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ReservationError']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ReservationError" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ReservationFailed']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="displayName" /><xsl:text> - #</xsl:text><xsl:value-of select="SelectedRemoteGeometry/index" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='RequiresLogin']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.RequiresLogin" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='SubmittedWithReservations']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.SubmittedWithReservations" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='PUDRequired']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.PUDRequired" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='NoRemoteGeometries']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="displayName" />
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>
	
</xsl:stylesheet>