package com.nordicpeak.flowengine.queries.iconmapquery.beans;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.base64.Base64;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.BaseInvoiceLine;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.interfaces.InvoiceLine;
import com.nordicpeak.flowengine.interfaces.PaymentQueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.interfaces.ReservationQueryInstance;
import com.nordicpeak.flowengine.interfaces.SubmitCheck;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryUtils;
import com.nordicpeak.flowengine.queries.iconmapquery.IconMapQueryProviderModule;
import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;

@Table(name = "icon_map_query_instances")
@XMLElement
public class IconMapQueryInstance extends BaseQueryInstance implements SubmitCheck, ReservationQueryInstance, PaymentQueryInstance {

	private static final long serialVersionUID = 5366400543122031447L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(IconMapQueryInstance.class, "query");
	public static final Field MAPPRINTS_RELATION = ReflectionUtils.getField(IconMapQueryInstance.class, "mapPrints");
	public static final Field GEOMETRIES_RELATION = ReflectionUtils.getField(IconMapQueryInstance.class, "geometries");
	public static final Field SELECTED_GEOMETRIES_RELATION = ReflectionUtils.getField(IconMapQueryInstance.class, "selectedGeometries");
	public static final Field RESERVATIONS_RELATION = ReflectionUtils.getField(IconMapQueryInstance.class, "reservations");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private IconMapQuery query;

	@DAOManaged
	@XMLElement
	private String extent;

	@DAOManaged
	@XMLElement
	private String epsg;

	@DAOManaged
	@XMLElement
	private String visibleBaseLayer;

	@DAOManaged
	@XMLElement
	private String propertyUnitDesignation;

	@DAOManaged
	@XMLElement
	private String pudObjectID;

	@DAOManaged
	@XMLElement
	private String lastUsedEndpointHash;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<Geometry> geometries;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<SelectedRemoteGeometry> selectedGeometries;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<IconMapQueryInstancePrint> mapPrints;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<IconMapQueryInstanceReservation> reservations;

	@XMLElement
	private String initializeErrorMessage;

	private List<com.vividsolutions.jts.geom.Geometry> pudGeometries;

	private List<RemoteGeometry> fetchedGeometries;

	private List<ValidationError> submitValidationErrors;

	@Override
	public String toString() {

		return "(queryInstanceID=" + queryInstanceID + ")";
	}

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public IconMapQuery getQuery() {

		return query;
	}

	public void setQuery(IconMapQuery query) {

		this.query = query;
	}

	public String getExtent() {

		return extent;
	}

	public void setExtent(String extent) {

		this.extent = extent;
	}

	public String getEpsg() {

		return epsg;
	}

	public void setEpsg(String epsg) {

		this.epsg = epsg;
	}

	public String getVisibleBaseLayer() {

		return visibleBaseLayer;
	}

	public void setVisibleBaseLayer(String visibleBaseLayer) {

		this.visibleBaseLayer = visibleBaseLayer;
	}

	public String getPropertyUnitDesignation() {

		return propertyUnitDesignation;
	}

	public void setPropertyUnitDesignation(String propertyUnitDesignation) {

		this.propertyUnitDesignation = propertyUnitDesignation;
	}

	public String getPudObjectID() {

		return pudObjectID;
	}

	public void setPudObjectID(String pudObjectID) {

		this.pudObjectID = pudObjectID;
	}

	public String getLastUsedEndpointHash() {

		return lastUsedEndpointHash;
	}

	public void setLastUsedEndpointHash(String lastUsedEndpointHash) {

		this.lastUsedEndpointHash = lastUsedEndpointHash;
	}

	public List<Geometry> getGeometries() {

		return geometries;
	}

	public void setGeometries(List<Geometry> geometries) {

		this.geometries = geometries;
	}

	public List<SelectedRemoteGeometry> getSelectedGeometries() {

		return selectedGeometries;
	}

	public void setSelectedGeometries(List<SelectedRemoteGeometry> selectedGeometries) {

		this.selectedGeometries = selectedGeometries;
	}

	public String getInitializeErrorMessage() {

		return initializeErrorMessage;
	}

	public void setInitializeErrorMessage(String initializeErrorMessage) {

		this.initializeErrorMessage = initializeErrorMessage;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		if (query.isSetAsAttribute()) {

			resetAttribute(attributeHandler);
		}

		submitValidationErrors = null;

		super.reset(attributeHandler);
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = super.getBaseExportXML(doc);

		XMLUtils.appendNewElement(doc, element, "Extent", extent);
		XMLUtils.appendNewElement(doc, element, "EPSG", epsg);
		XMLUtils.appendNewElement(doc, element, "VisibleBaseLayer", visibleBaseLayer);

		if (mapPrints != null) {

			Element printsElement = XMLUtils.appendNewElement(doc, element, "InstancePrints");

			for (IconMapQueryInstancePrint print : mapPrints) {

				Element printElement = XMLUtils.appendNewElement(doc, printsElement, "InstancePrint");

				XMLUtils.appendNewElement(doc, printElement, "MapImage", Base64.encodeBytes(print.getMapImage().getBytes(1, (int) print.getMapImage().length())));
				XMLUtils.appendNewElement(doc, printElement, "DPI", print.getResolution());
				XMLUtils.appendNewElement(doc, printElement, "Scale", print.getScale());
				XMLUtils.appendNewElement(doc, printElement, "Layout", print.getLayout());
				XMLUtils.appendNewElement(doc, printElement, "Format", print.getFormat());

			}
		}

		if (geometries != null) {

			Element geometriesElement = XMLUtils.appendNewElement(doc, element, "Geometries");

			for (Geometry geometry : geometries) {

				Element geometryElement = XMLUtils.appendNewElement(doc, geometriesElement, "Geometry");

				XMLUtils.appendNewElement(doc, geometryElement, "Type", geometry.getMapIcon().getName());
				XMLUtils.appendNewElement(doc, geometryElement, "Comment", geometry.getComment());
				XMLUtils.appendNewElement(doc, geometryElement, "PropertyUnitDesignation", geometry.getPropertyUnitDesignation());
				XMLUtils.appendNewElement(doc, geometryElement, "WKT", geometry.getGeometry());
			}
		}

		if (selectedGeometries != null) {

			Element selectedGeometriesElement = XMLUtils.appendNewElement(doc, element, "SelectedGeometries");

			for (SelectedRemoteGeometry selectedGeometery : selectedGeometries) {

				Element selectedGeometryElement = XMLUtils.appendNewElement(doc, selectedGeometriesElement, "SelectedGeometry");

				XMLUtils.appendNewElement(doc, selectedGeometryElement, "ObjectID", selectedGeometery.getObjectID());
				XMLUtils.appendNewElement(doc, selectedGeometryElement, "Name", selectedGeometery.getName());
				XMLUtils.appendNewElement(doc, selectedGeometryElement, "Description", selectedGeometery.getDescription());
				XMLUtils.appendNewElement(doc, selectedGeometryElement, "WKT", selectedGeometery.getWKTGeometry());
			}
		}

		if (propertyUnitDesignation != null) {

			XMLUtils.appendNewCDATAElement(doc, element, "PropertyUnitDesignation", propertyUnitDesignation);
		}

		if (reservations != null) {

			XMLUtils.append(doc, element, "Reservations", reservations);
		}

		return element;

	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		if (fetchedGeometries != null) {

			JsonArray geometries = new JsonArray();

			for (RemoteGeometry geometry : fetchedGeometries) {

				geometries.addNode(geometry.toJson());
			}

			XMLUtils.appendNewElement(doc, element, "FetchedGeometries", geometries.toJson());

		}

		if (selectedGeometries != null) {

			JsonArray geometries = new JsonArray();

			for (SelectedRemoteGeometry geometry : selectedGeometries) {

				geometries.addNode(geometry.toJson());
			}

			XMLUtils.appendNewElement(doc, element, "SelectedGeometries", geometries.toJson());

		}

		return element;
	}

	public void copyQueryValues() {

	}

	public void setupPrintableGeometries() {

		GeometryFactory geometryFactory = new GeometryFactory();

		WKTReader reader = new WKTReader(geometryFactory);

		if (geometries != null) {

			for (Geometry geometry : geometries) {

				try {
					geometry.setPrintableGeometry(reader.read(geometry.getGeometry()));
				} catch (ParseException e) {
					// This should never happen because geometry is already validated
				}

			}
		}

	}

	public List<com.vividsolutions.jts.geom.Geometry> getPrintableGeometries() {

		List<com.vividsolutions.jts.geom.Geometry> printableGeometries = new ArrayList<>();

		if (geometries != null) {

			for (Geometry geometry : geometries) {

				if (geometry.getPrintableGeometry() != null) {

					printableGeometries.add(geometry.getPrintableGeometry());

				}

			}
		}

		return printableGeometries;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(HttpServletRequest req, User user, User poster, URIParser uriParser, QueryHandler queryHandler, InstanceRequestMetadata requestMetadata) throws Exception {

		return BaseQueryUtils.getGenericQueryInstanceProvider(this.getClass(), queryHandler, queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID()).getQueryRequestProcessor(this, req, user, poster, uriParser, requestMetadata);
	}

	public List<IconMapQueryInstancePrint> getMapPrints() {

		return mapPrints;
	}

	public void setMapPrints(List<IconMapQueryInstancePrint> mapPrints) {

		this.mapPrints = mapPrints;
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {

		if (geometries != null || selectedGeometries != null) {

			StringBuilder wktBuilder = new StringBuilder();
			StringBuilder objectIDBuilder = new StringBuilder();
			StringBuilder coordinatesBuilder = new StringBuilder();
			StringBuilder propertyUnitBuilder = new StringBuilder();

			if (geometries != null) {

				for (Geometry geometry : geometries) {

					appendDelimited(wktBuilder, geometry.getGeometry());
					appendDelimited(coordinatesBuilder, geometry.getGeometry().substring("POINT(".length(), geometry.getGeometry().length() - ")".length()).replace(" ", ","));
					appendDelimited(propertyUnitBuilder, geometry.getPropertyUnitDesignation());
				}
			}

			if (selectedGeometries != null) {

				for (SelectedRemoteGeometry selectedGeometry : selectedGeometries) {

					appendDelimited(wktBuilder, selectedGeometry.getWKTGeometry());
					appendDelimited(objectIDBuilder, selectedGeometry.getObjectID());
				}
			}

			attributeHandler.setAttribute(getMainPropertyUnitDesignationAttributeName(), this.propertyUnitDesignation);
			attributeHandler.setAttribute(getWKTAttributeName(), wktBuilder.toString());
			attributeHandler.setAttribute(getObjectIDAttributeName(), objectIDBuilder.toString());
			attributeHandler.setAttribute(getCoordinatesAttributeName(), coordinatesBuilder.toString());
			attributeHandler.setAttribute(getPropertyUnitDesignationAttributeName(), propertyUnitBuilder.toString());

			// Backwards compatibility
			attributeHandler.setAttribute(query.getAttributeName(), coordinatesBuilder.toString());
		}

	}

	private void appendDelimited(StringBuilder builder, String text) {

		if (builder.length() > 0) {

			builder.append(";");
		}

		builder.append(text);
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler) {

		attributeHandler.removeAttribute(getMainPropertyUnitDesignationAttributeName());
		attributeHandler.removeAttribute(query.getAttributeName());
		attributeHandler.removeAttribute(getWKTAttributeName());
		attributeHandler.removeAttribute(getObjectIDAttributeName());
		attributeHandler.removeAttribute(getCoordinatesAttributeName());
		attributeHandler.removeAttribute(getPropertyUnitDesignationAttributeName());
	}

	String getMainPropertyUnitDesignationAttributeName() {

		return query.getAttributeName() + ".mainPropertyUnitDesignation";
	}

	String getPropertyUnitDesignationAttributeName() {

		return query.getAttributeName() + ".propertyUnitDesignation";
	}

	private String getWKTAttributeName() {

		return query.getAttributeName() + ".wkt";
	}

	private String getObjectIDAttributeName() {

		return query.getAttributeName() + ".objectID";
	}

	private String getCoordinatesAttributeName() {

		return query.getAttributeName() + ".coordinates";
	}

	public List<RemoteGeometry> getFetchedGeometries() {

		return fetchedGeometries;
	}

	public void setFetchedGeometries(List<RemoteGeometry> fetchedGeometries) {

		this.fetchedGeometries = fetchedGeometries;
	}

	public List<ValidationError> getSubmitValidationErrors() {

		return submitValidationErrors;
	}

	public void setSubmitValidationErrors(List<ValidationError> submitValidationErrors) {

		this.submitValidationErrors = submitValidationErrors;
	}

	public List<com.vividsolutions.jts.geom.Geometry> getPudGeometries() {

		return pudGeometries;
	}

	public void setPudGeometries(List<com.vividsolutions.jts.geom.Geometry> pudGeometries) {

		this.pudGeometries = pudGeometries;
	}

	@Override
	public boolean isValidForSubmit(MutableAttributeHandler attributeHandler, User poster, QueryHandler queryHandler, FlowInstanceManager flowInstanceManager) throws Exception {

		IconMapQueryProviderModule iconMapQueryProviderModule = queryHandler.getQueryProvider(queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID(), IconMapQueryProviderModule.class);

		if (iconMapQueryProviderModule.isReservationEnabled(this)) {

			try {

				iconMapQueryProviderModule.getReservationHandler().reserveSelectedGeometries(this, flowInstanceManager.getInstanceManagerID());

			} catch (ValidationException e) {

				submitValidationErrors = e.getErrors();
				return false;
			}
		}

		submitValidationErrors = null;
		return true;
	}

	@Override
	public List<String> getReservationIDs() {

		return CollectionUtils.map(reservations, IconMapQueryInstanceReservation::getReservationID);
	}

	public List<IconMapQueryInstanceReservation> getReservations() {

		return reservations;
	}

	public void setReservations(List<IconMapQueryInstanceReservation> reservations) {

		this.reservations = reservations;
	}

	public void addReservation(IconMapQueryInstanceReservation reservation) {

		if (!CollectionUtils.contains(reservations, reservation)) {

			reservations = CollectionUtils.addAndInstantiateIfNeeded(reservations, reservation);
		}

	}

	@Override
	public List<? extends InvoiceLine> getInvoiceLines() {

		if (selectedGeometries == null) {

			return null;
		}

		List<BaseInvoiceLine> invoiceLines = null;

		for (SelectedRemoteGeometry selectedGeometry : selectedGeometries) {

			String name = selectedGeometry.getName();
			Integer price = selectedGeometry.getPrice();

			if (name != null && price != null) {

				invoiceLines = CollectionUtils.addAndInstantiateIfNeeded(invoiceLines, new BaseInvoiceLine(1, price, name, ""));
			}
		}

		return invoiceLines;
	}

}
