package com.nordicpeak.flowengine.queries.iconmapquery.beans;

import java.io.Serializable;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.remotegeometryendpointprovider.RemoteGeometry;

@Table(name = "icon_map_query_instance_selectedgeometries")
@XMLElement
public class SelectedRemoteGeometry extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = 3862696357744155873L;

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement
	private Integer geometryID;

	@DAOManaged
	@XMLElement
	private String objectID;

	@DAOManaged
	@XMLElement
	private String name;

	@DAOManaged
	@XMLElement
	private String description;

	@DAOManaged
	@XMLElement
	private String wktGeometry;

	@DAOManaged
	@XMLElement
	private String iconURL;

	@DAOManaged
	@XMLElement
	private String color;

	@DAOManaged
	@XMLElement(name = "index")
	private Integer labelIndex;

	@DAOManaged
	@XMLElement
	private Integer price;

	@DAOManaged(columnName = "queryInstanceID")
	@ManyToOne
	private IconMapQueryInstance queryInstance;

	private com.vividsolutions.jts.geom.Geometry printableGeometry;

	public SelectedRemoteGeometry() {

	}

	public SelectedRemoteGeometry(RemoteGeometry remoteGeometry, Integer labelIndex) {

		this.objectID = remoteGeometry.getObjectID();
		this.name = remoteGeometry.getName();
		this.description = remoteGeometry.getDescription();
		this.wktGeometry = remoteGeometry.getWktGeometry();
		this.iconURL = remoteGeometry.getSelectedIconURL();
		this.color = remoteGeometry.getSelectedColor();
		this.price = remoteGeometry.getPrice();
		this.printableGeometry = remoteGeometry.getGeometry();
		this.labelIndex = labelIndex;
	}

	public Integer getGeometryID() {

		return geometryID;
	}

	public void setGeometryID(Integer geometryID) {

		this.geometryID = geometryID;
	}

	public String getObjectID() {

		return objectID;
	}

	public void setObjectID(String objectID) {

		this.objectID = objectID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getDescription() {

		return description;
	}

	public void setDescription(String description) {

		this.description = description;
	}

	public String getWKTGeometry() {

		return wktGeometry;
	}

	public void setWKTGeometry(String wktGeometry) {

		this.wktGeometry = wktGeometry;
	}

	public String getIconURL() {

		return iconURL;
	}

	public void setIconURL(String iconURL) {

		this.iconURL = iconURL;
	}

	public String getColor() {

		return color;
	}

	public void setColor(String color) {

		this.color = color;
	}

	public Integer getIndex() {

		return labelIndex;
	}

	public void setIndex(Integer labelIndex) {

		this.labelIndex = labelIndex;
	}

	public Integer getPrice() {

		return price;
	}

	public void setPrice(Integer price) {

		this.price = price;
	}

	public IconMapQueryInstance getQueryInstance() {

		return queryInstance;
	}

	public void setQueryInstance(IconMapQueryInstance queryInstance) {

		this.queryInstance = queryInstance;
	}

	public com.vividsolutions.jts.geom.Geometry getPrintableGeometry() {

		return printableGeometry;
	}

	public void setPrintableGeometry(com.vividsolutions.jts.geom.Geometry printableGeometry) {

		this.printableGeometry = printableGeometry;
	}

	public JsonObject toJson() {

		JsonObject geometry = new JsonObject();
		geometry.putField("objectID", objectID);
		geometry.putField("name", name);
		geometry.putField("geometry", wktGeometry);

		if (!StringUtils.isEmpty(description)) {
			geometry.putField("description", description);
		}

		if (!StringUtils.isEmpty(iconURL)) {
			geometry.putField("iconURL", iconURL);
		}

		if (!StringUtils.isEmpty(color)) {
			geometry.putField("color", color);
		}

		if (price != null) {
			geometry.putField("price", price);
		}

		return geometry;
	}

	@Override
	public String toString() {

		return "(geometryID=" + geometryID + ", objectID=" + objectID + ", name=" + name + ")";
	}

}
