var iconMapQueryLanguage = {
	"MAXGEOMETRIES_MESSAGE": "You cannot draw more symbols of this type",
	"MAXSELECTED_MESSAGE": "You cannot select more objects",
	"GETTING_PUD": "Getting property unit designation...",
	"PUD_NOT_FOUND": "No pud found",
	"PUD_REQUEST_ERROR": "Problem getting pud",
	"TITLE": {
		"MAP_ICON": "Map Icon",
		"COORDINATES": "Coordinates",
		"PROPERTY_UNIT_DESIGNATION": "Property unit designation",
		"COMMENT": "Comment",
		"NAME": "Name",
		"DESCRIPTION": "Description",
		"PRICE": "Price"
	},
	"SELECT": "Select",
	"CURRENCY_SYMBOL": "Currency Symbol",
	"HITS_CHOOSE_OBJECT": "hits. Choose a object"
};

$(function() {

	setQueryRequiredFunctions["IconMapQueryInstance"] = makeIconMapQueryRequired;
});

function makeIconMapQueryRequired(queryID) {

	$("#query_" + queryID).find(".heading-wrapper h2").addClass("required");

}

function initIconMapQuery(queryID, queryOptions, origoInstance) {

	if (typeof Origo == "undefined") {

		$.getScript(queryOptions.origoScriptURL, function() {

			initIconMapQuery(queryID, queryOptions);
		});

		return;
	}

	if (!origoInstance) {

		origoInstance = Origo(queryOptions.mapconfig, {
			"baseUrl": queryOptions.baseUrl
		}, function() {

			initIconMapQuery(queryID, queryOptions, origoInstance);
		});

		return;
	}

	var shortQueryID = "q" + queryID;

	var $queryWrapper = $("#query_" + queryID);
	var $mapWrapper = $("#" + shortQueryID);
	var $geometryData = $queryWrapper.find(".geometry-data");
	var $geometryTable = $queryWrapper.find(".geometry-table");

	var $iconToolBar = $mapWrapper.find(".draw-icon-toolbar");
	$iconToolBar.appendTo($("#o-map"));

	var searchPUDURL = null;
	var searchPUDAreaURL = null;

	if (queryOptions.mapconfig.controls) {
		var tool = $.grep(queryOptions.mapconfig.controls, function(obj) {
			return obj.name == "getpudsearch";
		});

		if (tool[0]) {
			searchPUDURL = tool[0].options.url_fastighet;
			searchPUDAreaURL = tool[0].options.url_yta;
		}
	}

	var ol = Origo.ol;
	var map = origoInstance.getMap();

	var getWKTRepresentation = function(feature) {

		var format = new ol.format.WKT();
		return format.writeGeometry(feature.getGeometry());

	};

	var getCenterCoordinates = function(feature) {

		return ol.extent.getCenter(feature.getGeometry().getExtent());

	}

	var getCoordinatesAsString = function(coordinates) {

		return Math.round(coordinates[0]) + ", " + Math.round(coordinates[1]);
	}

	var getIconToolName = function(feature) {

		return $iconToolBar.find("[data-id='" + feature.getGeometryName() + "'] span.text").text();

	}

	var getPUDByCoordinates = function(coordinates, $pudHolder) {

		$pudHolder.find(".text").text(iconMapQueryLanguage.GETTING_PUD);

		$.ajax({
			url: searchPUDURL,
			dataType: "json",
			contentType: "application/x-www-form-urlencoded;charset=UTF-8",
			data: {
				x: coordinates[0],
				y: coordinates[1]
			},
			success: function(response) {

				if (response.name) {

					$pudHolder.find(".text").text(response.name);
					$pudHolder.find("input").val(response.name);

				} else {

					$pudHolder.find(".text").text(iconMapQueryLanguage.PUD_NOT_FOUND).addClass("error");

				}

			},
			error: function() {

				$pudHolder.find(".text").text(iconMapQueryLanguage.PUD_REQUEST_ERROR).addClass("error");

			}
		});

	};

	var getPUDByObjectID = function(objectID, callback) {

		var url = searchPUDAreaURL.replace("objectId", objectID);

		return $.ajax({
			url: url,
			dataType: 'json',
			success: callback,
		});
	}

	var createReservation = function(objectID, queryOptions, callback) {

		var url = queryOptions.createReservationURL + objectID;

		return $.ajax({
			url: url,
			dataType: 'json',
			success: function(data) {

				console.log("Successfully created reservation for object " + objectID);
				callback();
			},
			error: function(jqXHR) {

				if (jqXHR.status === 403) {

					location.reload();
				}

				var error = jqXHR.responseJSON.error;

				console.log("Failed to create reservation for object " + objectID + ": " + error);

				if (error == "reservationFailed") {

					alert(queryOptions.reservationFailedMessage);
				}
			}
		});
	}

	var deleteReservation = function(objectID, queryOptions, callback) {

		var url = queryOptions.deleteReservationURL + objectID;

		return $.ajax({
			url: url,
			dataType: 'json',
			success: function(data) {

				console.log("Successfully deleted reservation for object " + objectID);
				callback();
			},
			error: function(jqXHR) {

				if (jqXHR.status === 403) {

					location.reload();
				}

				var error = jqXHR.responseJSON.error;

				console.log("Failed to delete reservation for object " + objectID + ": " + error);
			}
		});
	}

	if ($iconToolBar.length > 0) {

		var drawnFeaturesCount = {};
		var maxGeometries = {};
		var styles = {};
		var interactions = {};
		var featureCounter = 0;

		var iconsSource = new ol.source.Vector();
		var iconsLayer = new ol.layer.Vector({
			id: "iconsLayer",
			source: iconsSource,
			style: function(feature, resolution) {

				var style = styles[feature.getGeometryName()];

				if (queryOptions.useDetailedIconInfo) {

					var labelID = feature.getId().split("_")[1];

					style.getText().setText("#" + labelID);

				}

				return style;
			}
		});

		var addMapIcon = function(feature) {

			var labelID = feature.getId().split("_")[1];

			$geometryData.append($("<input id='" + feature.getId() + "' type='hidden' name='" + shortQueryID + "_geometry' value='" + labelID + "#" + getWKTRepresentation(feature) + "#" + feature.getGeometryName() + "' />"));

			if (queryOptions.useDetailedIconInfo) {

				var coordinates = getCenterCoordinates(feature);

				var comment = feature.get("comment");

				if (!comment) {
					comment = "";
				}

				var $row = $("<tr data-id='" + feature.getId() + "'>" +
					"<td class='id'>" + labelID + "</td>" +
					"<td data-title=" + iconMapQueryLanguage.TITLE.MAP_ICON + " class='type'>" + getIconToolName(feature) + "</td>" +
					"<td data-title=" + iconMapQueryLanguage.TITLE.COORDINATES + " class='coordinates'>" + getCoordinatesAsString(coordinates) + "</td>" +
					"<td data-title=" + iconMapQueryLanguage.TITLE.COMMENT + " class='comment'><input type='text' name='" + shortQueryID + "_comment_" + labelID + "' value='" + comment + "' /></td>" +
					"<td class='delete'><a href='#' data-icon-after='t'></a></td>" +
					"</tr>");

				if (queryOptions.showIconPUDInfo) {

					var $pudCol = $("<td data-title=" + iconMapQueryLanguage.TITLE.PROPERTY_UNIT_DESIGNATION + " class='pud'><span class='text'></span><input type='hidden' name='" + shortQueryID + "_pud_" + labelID + "' value='' /></td>");
					$pudCol.insertAfter($row.find(".coordinates"));

					getPUDByCoordinates(coordinates, $pudCol);

				}

				$row.find(".delete a").on("click", function(e) {
					e.preventDefault();

					iconsLayer.getSource().removeFeature(feature);

				});

				$row.on("hover", function() {}, function() {});

				$geometryTable.find("tbody").append($row);
				$geometryTable.find(".empty-row").hide();

			}

		};

		var removeMapIcon = function(feature) {

			$("#" + feature.getId()).remove();

			if (queryOptions.useDetailedIconInfo) {

				$geometryTable.find("tbody tr[data-id='" + feature.getId() + "']").remove();

				if ($geometryTable.find("tbody tr").length == 1) {

					$geometryTable.find(".empty-row").show();
					featureCounter = 0;

				}

			}

		};

		var updateMapIcon = function(feature) {

			var labelID = feature.getId().split("_")[1];

			$("#" + feature.getId()).val(labelID + "#" + getWKTRepresentation(feature) + "#" + feature.getGeometryName());

			if (queryOptions.useDetailedIconInfo) {

				var coordinates = getCenterCoordinates(feature);

				$geometryTable.find("tbody tr[data-id='" + feature.getId() + "'] td.id").text(labelID);
				$geometryTable.find("tbody tr[data-id='" + feature.getId() + "'] td.comment input").attr("name", shortQueryID + "_comment_" + labelID);
				$geometryTable.find("tbody tr[data-id='" + feature.getId() + "'] td.coordinates").text(getCoordinatesAsString(coordinates));

			}

		};

		iconsSource.on('addfeature', function(evt) {

			var feature = evt.feature;
			var geometryType = feature.getGeometryName();

			drawnFeaturesCount[geometryType]++;

			if (maxGeometries[geometryType] != undefined && drawnFeaturesCount[geometryType] > maxGeometries[geometryType]) {

				iconsLayer.getSource().removeFeature(feature);
				alert(iconMapQueryLanguage.MAXGEOMETRIES_MESSAGE);

			} else {

				var index = feature.get("index");

				if (index) {
					featureCounter = index;
				} else {
					featureCounter++;
				}

				feature.setId("geometry_" + featureCounter);

				addMapIcon(feature);

			}

		});

		iconsSource.on('removefeature', function(evt) {

			var feature = evt.feature;

			drawnFeaturesCount[feature.getGeometryName()]--;

			if (drawnFeaturesCount[feature.getGeometryName()] < 0) {
				drawnFeaturesCount[feature.getGeometryName()] = 0;
			}

			featureCounter--;

			if (featureCounter < 0) {
				featureCounter = 0;
			}

			if (feature.getId() != undefined) {

				var $feature = $("#" + feature.getId());

				var $geometries = $feature.nextAll();

				removeMapIcon(feature);

				$geometries.each(function() {

					var id = $(this).attr("id");

					var index = id.split("_")[1];

					var ftr = iconsLayer.getSource().getFeatureById(id)

					if (ftr.getGeometryName()) {

						ftr.setStyle(function(f, resolution) {

							var style = styles[ftr.getGeometryName()];

							if (queryOptions.useDetailedIconInfo) {
								style.getText().setText("#" + (index - 1));
							}

							return style;

						});

					}

					var newID = "geometry_" + (index - 1);

					$("#" + ftr.getId()).attr("id", newID);

					$geometryTable.find("tbody tr[data-id='" + ftr.getId() + "']").attr("data-id", newID);

					ftr.setId(newID);

					updateMapIcon(ftr);

				});

			}

		});

		iconsSource.on('changefeature', function(evt) {

			var feature = evt.feature;

			if (feature.getId() != undefined) {
				updateMapIcon(feature);
			}

		});

		$iconToolBar.find(".mapicon").each(function() {

			var $this = $(this);

			var id = $this.data("id");

			if ($this.data("max") != undefined) {
				maxGeometries[id] = $this.data("max");
			}

			var $img = $this.find("img")

			styles[id] = new ol.style.Style({
				image: new ol.style.Icon({
					src: $img.attr("src"),
					scale: 0.3
				}),
				text: new ol.style.Text({
					text: "",
					fill: new ol.style.Fill({
						color: 'black'
					}),
					offsetX: ($img.width() * 0.3) + 27,
					offsetY: (($img.height() * 0.3) / 2) - 1,
					font: '14px sans-serif'
				})
			});

			var interaction = new ol.interaction.Draw({
				type: 'Point',
				source: iconsLayer.getSource(),
				style: function(feature, resolution) {
					var style = styles[id];
					style.getText().setText("");
					return style;
				},
				geometryName: id
			});

			interaction.on('drawstart', function(evt) {});

			interaction.on('drawend', function(evt) {});

			interactions[id] = interaction;

			drawnFeaturesCount[id] = 0;

		});

		styles["modify"] = new ol.style.Style({});
		styles["select"] = new ol.style.Style({});

		map.addLayer(iconsLayer);

		$('#draw-icon-toolbar button').buttonset();

		var currentInteraction = null;

		$iconToolBar.find("button").on('click', function(event) {

			var id = $(this).data("id");

			$iconToolBar.find("button").removeClass("selected");
			$("button[data-id = '" + id + "']").addClass("selected");

			map.removeInteraction(currentInteraction);

			if (id == "modify") {

				currentInteraction = new ol.interaction.Modify({
					features: new ol.Collection(iconsLayer.getSource().getFeatures()),
					style: styles["modify"],
					pixelTolerance: 20
				});

			} else if (id == "select") {

				var selectInteraction = new ol.interaction.Select({
					style: styles["select"],
					multi: true
				});

				selectInteraction.on("select", function(event) {

					for (var i = 0; i < event.selected.length; i++) {
						if (event.selected[i].getId() != undefined) {
							iconsLayer.getSource().removeFeature(event.selected[i]);
						}
					}

					selectInteraction.getFeatures().clear();

				});

				currentInteraction = selectInteraction;

			} else {

				currentInteraction = interactions[id];

			}

			map.addInteraction(currentInteraction);

		});

		var addGeometry = function($geometry) {

			var valParts = $geometry.val().split("#");

			var index = valParts[0]
			var wkt = valParts[1];
			var mapIcon = valParts[2];

			var wktFormat = new ol.format.WKT();

			var geometry = wktFormat.readGeometry(wkt);
			geometry.set("name", mapIcon);

			var featureConfig = {};

			featureConfig[mapIcon] = geometry;

			var feature = new ol.Feature(featureConfig);

			var style = styles[mapIcon];

			if (index != "") {

				feature.set("index", index);

				if (queryOptions.useDetailedIconInfo) {

					var comment = $("#" + $geometry.attr("id") + "_comment").val();

					if (comment) {

						feature.set("comment", comment);
					}

				}

			}

			feature.setGeometryName(mapIcon);

			feature.setStyle(function(feature, resolution) {

				var style = styles[mapIcon];

				if (queryOptions.useDetailedIconInfo) {
					style.getText().setText("#" + index);
				}

				return style;
			});

			iconsSource.addFeature(feature);

		};

		var $geometries = $("input[name='" + shortQueryID + "_geometry']");

		if ($geometries.length > 0) {

			$geometries.each(function() {
				addGeometry($(this));
			});

		}

	}

	if (queryOptions.fetchedGeometries) {

		var geometriesLayer = new ol.layer.Vector({
			id: "geometriesLayer",
			source: new ol.source.Vector()
		});

		map.addLayer(geometriesLayer);

		var defaultStyles = {};
		var textStyle = new ol.style.Text({
			text: "",
			fill: new ol.style.Fill({
				color: 'black'
			}),
			font: '14px sans-serif',
			textBaseline: "middle"
		});

		defaultStyles["Polygon"] = new ol.style.Style({
			stroke: new ol.style.Stroke({
				color: "rgba(0, 0, 255, 0.8)",
				width: 2
			}),
			fill: new ol.style.Fill({
				color: "rgba(0, 0, 255, 0.4)"
			}),
			text: $.extend({}, textStyle)
		});

		defaultStyles["MultiPolygon"] = defaultStyles["Polygon"];
		defaultStyles["LineString"] = defaultStyles["Polygon"];
		defaultStyles["MultiLineString"] = defaultStyles["Polygon"];

		defaultStyles["Point"] = new ol.style.Style({
			image: new ol.style.Circle({
				radius: 5,
				stroke: new ol.style.Stroke({
					color: "rgba(0, 0, 255, 0.8)",
					width: 2
				}),
				fill: new ol.style.Fill({
					color: "rgba(0, 0, 255, 0.4)"
				}),
				text: $.extend({}, textStyle)
			})
		});

		defaultStyles["MultiPoint"] = defaultStyles["Point"];

		var selectedStyles = {};

		selectedStyles["Polygon"] = new ol.style.Style({
			stroke: new ol.style.Stroke({
				color: "rgba(8, 135, 39, 0.8)",
				width: 2
			}),
			fill: new ol.style.Fill({
				color: "rgba(8, 135, 39, 0.4)"
			}),
			text: $.extend({}, textStyle)
		});

		selectedStyles["MultiPolygon"] = selectedStyles["Polygon"];
		selectedStyles["LineString"] = selectedStyles["Polygon"];
		selectedStyles["MultiLineString"] = selectedStyles["Polygon"];

		selectedStyles["Point"] = new ol.style.Style({
			image: new ol.style.Circle({
				radius: 5,
				stroke: new ol.style.Stroke({
					color: "rgba(8, 135, 39, 0.8)",
					width: 2
				}),
				fill: new ol.style.Fill({
					color: "rgba(8, 135, 39, 0.4)"
				})
			}),
			text: $.extend({}, textStyle)
		});

		selectedStyles["MultiPoint"] = selectedStyles["Point"];

		var addFetchedGeometry = function(item) {

			var wktFormat = new ol.format.WKT();

			var parsedGeometry = wktFormat.readGeometry(item.wktGeometry);

			var feature = new ol.Feature({
				geometry: parsedGeometry
			});

			feature.setId(item.objectID);

			feature.set("name", item.name);
			feature.set("isFetchedGeometry", true);
			feature.set("isSelectable", item.selectable);
			
			if (item.description && !item.hideDescription) {
				feature.set("description", item.description);
			}

			if (queryOptions.paymentEnabled && item.price) {
				feature.set("price", item.price);
			}

			if (item.iconURL && parsedGeometry.getType() == "Point") {

				feature.defaultStyle = new ol.style.Style({
					image: new ol.style.Icon({
						src: item.iconURL,
						scale: 1
					}),
					text: $.extend({}, textStyle)
				});

				feature.setStyle(feature.defaultStyle);

			} else if (item.color && parsedGeometry.getType() == "Point") {

				var colorString = item.color.red + ", " + item.color.green + ", " + item.color.blue;

				feature.defaultStyle = new ol.style.Style({
					image: new ol.style.Circle({
						radius: 5,
						stroke: new ol.style.Stroke({
							color: "rgba(" + colorString + ", 0.8)",
							width: 2
						}),
						fill: new ol.style.Fill({
							color: "rgba(" + colorString + ", 0.4)",
						}),
						text: $.extend({}, textStyle)
					})
				});

				feature.setStyle(feature.defaultStyle);

			} else if (item.color) {

				var colorString = item.color.red + ", " + item.color.green + ", " + item.color.blue;

				feature.defaultStyle = new ol.style.Style({
					stroke: new ol.style.Stroke({
						color: "rgba(" + colorString + ", 0.8)",
						width: 2
					}),
					fill: new ol.style.Fill({
						color: "rgba(" + colorString + ", 0.4)",
					}),
					text: $.extend({}, textStyle)
				});

				feature.setStyle(feature.defaultStyle);

			} else {

				var style = $.extend(true, {}, defaultStyles[parsedGeometry.getType()]);

				feature.setStyle(style);
			}

			if (item.selectedIconURL && parsedGeometry.getType() == "Point") {

				feature.selectedStyle = new ol.style.Style({
					image: new ol.style.Icon({
						src: item.selectedIconURL,
						scale: 1
					}),
					text: $.extend({}, textStyle)
				});

			} else if (item.selectedColor && parsedGeometry.getType() == "Point") {

				var colorString = item.selectedColor.red + ", " + item.selectedColor.green + ", " + item.selectedColor.blue;

				feature.selectedStyle = new ol.style.Style({
					image: new ol.style.Circle({
						radius: 5,
						stroke: new ol.style.Stroke({
							color: "rgba(" + colorString + ", 0.8)",
							width: 2
						}),
						fill: new ol.style.Fill({
							color: "rgba(" + colorString + ", 0.4)",
						}),
						text: $.extend({}, textStyle)
					})
				});

			} else if (item.selectedColor) {

				var colorString = item.selectedColor.red + ", " + item.selectedColor.green + ", " + item.selectedColor.blue;

				feature.selectedStyle = new ol.style.Style({
					stroke: new ol.style.Stroke({
						color: "rgba(" + colorString + ", 0.8)",
						width: 2
					}),
					fill: new ol.style.Fill({
						color: "rgba(" + colorString + ", 0.4)",
					}),
					text: $.extend({}, textStyle)
				});
			}

			geometriesLayer.getSource().addFeature(feature);

			return feature;

		};

		$.each(queryOptions.fetchedGeometries, function(i, item) {

			addFetchedGeometry(item);

		});

		var $popup = $queryWrapper.find(".popup");

		$popup.find("a.close").on("click", function(e) {
			e.preventDefault();
			popup.setPosition(undefined);
		});

		var popup = new ol.Overlay({
			element: $popup.get(0),
			positioning: 'bottom-center',
			stopEvent: true,
			offset: [0, -7],
			autoPan: true,
			autoPanAnimation: {
				duration: 0
			}
		});

		map.addOverlay(popup);

		$mapWrapper.find("#o-search-button-close").on("click", function(e) {

			(function readdOverlaysAfterClear() {

				if (map.getOverlays().getLength() === 0) {

					map.addOverlay(popup);

					map.renderSync();

				} else {

					setTimeout(function() {

						readdOverlaysAfterClear();

					}, 100);
				}

			})();
		});

		var selectInteraction = new ol.interaction.Select({
			multi: true
		});

		selectInteraction.on("select", function(event) {

			var selectedFeatures = new Array();

			for (var i = 0; i < event.selected.length; i++) {

				var feature = event.selected[i];

				if (feature.getId() != undefined && feature.get("isFetchedGeometry")) {

					selectedFeatures.push(feature);
				}

			}
			
			if (selectedFeatures.length == 1) {

				initPopup(selectedFeatures[0]);

			} else if (selectedFeatures.length > 1) {

				initMultiFeaturePopop(selectedFeatures);
			}

			selectInteraction.getFeatures().clear();

		});

		var initMultiFeaturePopop = function(features) {

			$popup.data("id", "multipopup");
			$popup.find(".popup-title span").html(features.length + " " + iconMapQueryLanguage.HITS_CHOOSE_OBJECT + ":");
			$popup.find(".select-btn").hide();
			$popup.find(".unselect-btn").hide();

			var $content = $popup.find(".popup-content span").html("");

			$.each(features, function(index, feature) {

				var $link = $("<a href='#'>" + feature.get("name") + "</a><br/>");

				$link.on("click", function(e) {
					e.stopPropagation();
					e.preventDefault();
					initPopup(feature);
				});

				$content.append($link);
			});

			var centerCord = getCenterCoordinates(features[0]);

			popup.setPosition(centerCord);

			setTimeout(function() {
				if ($popup.parent().position().left < 0 && map.getSize()[0] < 400) {
					map.getView().setCenter(centerCord);
				}
			}, 10);

		};

		var initPopup = function(feature) {

			if (feature.getId() != undefined && feature.get("isFetchedGeometry")) {

				if (!feature.get("isSelectable")) {

					$popup.find(".select-btn").hide();
					$popup.find(".unselect-btn").hide();

				} else if (feature.get("isSelected")) {

					$popup.find(".select-btn").hide();
					$popup.find(".unselect-btn").show();
				} else {
					$popup.find(".select-btn").show();
					$popup.find(".unselect-btn").hide();
				}

				$popup.data("id", feature.getId());

				$popup.find(".popup-title span").html(feature.get("name"));
				
				if (feature.get("description") != undefined) {
					$popup.find(".popup-content span").html(feature.get("description"));					
				} else {
					$popup.find(".popup-content span").html("");
				}

				if (queryOptions.paymentEnabled) {

					var price = feature.get("price");
					var selectButtonText = iconMapQueryLanguage.SELECT;

					if (price) {

						selectButtonText += " (" + price + iconMapQueryLanguage.CURRENCY_SYMBOL + ")";
					}

					$popup.find(".popup-content .select-btn").val(selectButtonText);
				}

				var centerCord = getCenterCoordinates(feature);

				popup.setPosition(centerCord);

				setTimeout(function() {
					if ($popup.parent().position().left < 0 && map.getSize()[0] < 400) {
						map.getView().setCenter(centerCord);
					}
				}, 10);

			}

		};

		var getFeaturePixelWidth = function(feature) {

			var extent = feature.getGeometry().getExtent();

			var minPixel = map.getPixelFromCoordinate([extent[0], extent[1]]);

			var maxPixel = map.getPixelFromCoordinate([extent[2], extent[3]]);

			if (maxPixel == null) {

				return 0;
			}

			return Math.round(maxPixel[0]) - Math.round(minPixel[0]);

		};

		map.addInteraction(selectInteraction);

		if (queryOptions.useSelectableGeometries) {

			var selectedCount = 0;

			function selectGeometryInModal(objectID, $selectButton) {

				$selectButton.hide();
				$popup.find(".unselect-btn").show();

				var feature = geometriesLayer.getSource().getFeatureById(objectID);

				selectGeometry(feature);
			}

			$popup.find(".select-btn").on("click", function(e) {
				e.stopPropagation();
				e.preventDefault();
				if (queryOptions.maxSelectableGeometries && selectedCount >= queryOptions.maxSelectableGeometries) {

					alert(iconMapQueryLanguage.MAXSELECTED_MESSAGE.replace("$objects", queryOptions.maxSelectableGeometries));
					return;
				}

				var objectID = $popup.data("id");

				var $selectButton = $(this);

				if (queryOptions.reservationEnabled) {

					createReservation(objectID, queryOptions, function() {

						selectGeometryInModal(objectID, $selectButton);
					});

				} else {

					selectGeometryInModal(objectID, $selectButton);
				}

			});

			$popup.find(".unselect-btn").on("click", function(e) {
				e.stopPropagation();
				e.preventDefault();

				$geometryTable.find("tr[data-id='" + $popup.data("id") + "'] .delete a").trigger("click");

			});

			var updateGeometryIndex = function() {

				$.each(geometriesLayer.getSource().getFeatures(), function(i, feature) {

					if (feature.getStyle().getText() != null) {

						var $row = $geometryTable.find("tbody tr[data-id='" + feature.getId() + "']");

						if ($row.length > 0) {
							var index = $row.index() + 1;
							feature.getStyle().getText().setText("#" + index);

							var offset = (getFeaturePixelWidth(feature) / 2);

							if (offset == 0) {
								offset = 25;
							}

							feature.getStyle().getText().setOffsetX(offset);

							$row.find(".index").text(index);
							$geometryData.find("#geometry_" + feature.getId()).val(index + "#" + feature.getId());
						} else {
							feature.getStyle().getText().setText("");
						}

					}

				});

			};

			var selectGeometry = function(feature) {

				feature.set("isSelected", true);

				if (feature.selectedStyle) {

					feature.setStyle(feature.selectedStyle);

				} else {

					var style = $.extend(true, {}, selectedStyles[feature.getGeometry().getType()]);
					feature.setStyle(style);
				}

				$geometryData.append($("<input id='geometry_" + feature.getId() + "' type='hidden' name='" + shortQueryID + "_geometry' value='" + feature.getId() + "' />"));

				var $row = $("<tr data-id='" + feature.getId() + "'>" +
					"<td class='index'></td>" +
					"<td data-title=" + iconMapQueryLanguage.TITLE.NAME + " class='name'>" + feature.get("name") + "</td>" +
					"<td data-title=" + iconMapQueryLanguage.TITLE.DESCRIPTION + " class='description'></td>" + (queryOptions.paymentEnabled ? "<td data-title=" + iconMapQueryLanguage.TITLE.PRICE + " class='price'></td>" : "") +
					"<td class='delete'><a href='#' data-icon-after='t'></a></td>" +
					"</tr>");

				function deleteSelectedGeometry(objectID) {

					var feature = geometriesLayer.getSource().getFeatureById(objectID);

					feature.set("isSelected", false);

					if (feature.defaultStyle) {

						feature.setStyle(feature.defaultStyle);

					} else {

						var style = $.extend(true, {}, defaultStyles[feature.getGeometry().getType()]);
						feature.setStyle(style);
					}

					$("#geometry_" + feature.getId()).remove();

					$popup.find(".select-btn").show();
					$popup.find(".unselect-btn").hide();

					popup.setPosition(undefined);

					$geometryTable.find("tr[data-id='" + feature.getId() + "']").remove();

					if ($geometryTable.find("tbody tr").length == 0) {
						$geometryTable.hide();
					}

					updateGeometryIndex();

					selectedCount--;
				}

				$row.find(".delete a").on("click", function(e) {
					e.preventDefault();

					var objectID = $row.data("id");

					if (queryOptions.reservationEnabled) {

						deleteReservation(objectID, queryOptions, function() {

							deleteSelectedGeometry(objectID);
						});

					} else {

						deleteSelectedGeometry(objectID);
					}
				});

				$row.find(".description").html(feature.get("description"));

				if (queryOptions.paymentEnabled && feature.get("price")) {

					$row.find(".price").html(feature.get("price") + iconMapQueryLanguage.CURRENCY_SYMBOL);
				}

				$geometryTable.children("tbody").append($row);
				$geometryTable.show();

				updateGeometryIndex();

				selectedCount++;

			};

			if (queryOptions.selectedGeometries) {

				var hasMissingGeometries = false;

				$.each(queryOptions.selectedGeometries, function(i, item) {

					var wktFormat = new ol.format.WKT();

					var feature = geometriesLayer.getSource().getFeatureById(item.objectID);

					if (feature) {

						selectGeometry(feature);

					} else {

						if (!hasMissingGeometries) {

							$queryWrapper.children("article").addClass("error").before(
								'<div id="query_' + queryID + '-validationerrors" class="validationerrors">' +
								'<div class="info-box error">' +
								'<span><strong data-icon-before="!">' + queryOptions.missingGeometryMessage +
								'</strong></span>' +
								'<div class="marker"></div>' +
								'</div>' +
								'</div>');

							hasMissingGeometries = true;

						}
					}

				});

			}

		}

		var mouseInPopup = false;

		$(".ol-overlay-container").on("mouseover", ".popup", function() {
			mouseInPopup = true;
		}).on("mouseout", ".popup", function() {
			mouseInPopup = false;
		});

		map.on('click', function(e) {

			if (!e.dragging && !mouseInPopup) {
				popup.setPosition(undefined);
			}
		});
	}

	var $extent = $("#" + shortQueryID + "_extent");

	if (queryOptions.fetchedGeometries && queryOptions.zoomToGeometries) {

		map.getView().fit(geometriesLayer.getSource().getExtent(), map.getSize());

	} else if ($extent.val() != "") {

		var extent = $extent.val().split(",");
		map.getView().fit(extent, map.getSize());
	}

	map.on("moveend", function(e) {
		$extent.val(map.getView().calculateExtent(map.getSize()));
	});

	$("#" + shortQueryID + "_epsg").val(queryOptions.mapconfig.projectionCode);
	$extent.val(map.getView().calculateExtent(map.getSize()));

	if (queryOptions.requirePUD && Origo.controls.Getpudsearch) {

		Origo.controls.Getpudsearch.addListener(function(pud, objectID) {

			var selectedPud = pud;

			if (selectedPud == null || objectID == null) {
				selectedPud = "";
				objectID = "";
			}

			$("#" + shortQueryID + "_propertyUnitDesignation").val(selectedPud);
			$("#" + shortQueryID + "_pudObjectID").val(objectID);

		});

		var pud = $("#" + shortQueryID + "_propertyUnitDesignation").val();
		var objectID = $("#" + shortQueryID + "_pudObjectID").val();

		if (pud && objectID) {

			getPUDByObjectID(objectID, function(pudResponse) {

				if (pudResponse && pudResponse.geometry) {

					Origo.controls.Getpudsearch.setPUD(objectID, pud, pudResponse.geometry);
				}
			});

		}
	}

	var $target = $("#" + map.getTarget());

	map.on("pointermove", function(event) {

		var mouseCoordInMapPixels = [event.originalEvent.offsetX, event.originalEvent.offsetY];

		var hit = map.forEachFeatureAtPixel(mouseCoordInMapPixels, function(feature, layer) {

			if (layer == null) return false;

			var layerID = layer.get("id");

			if ((layerID == "iconsLayer" && currentInteraction != null) || layerID == "geometriesLayer") {
				return true;
			}

			return false;
		});

		if (hit) {
			$target.css("cursor", "pointer");
		} else {
			$target.css("cursor", "");
		}

	});

	$queryWrapper.addClass("loaded");
}
