<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="InfoQueryInstance/InfoQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="InfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article class="infoquery show-mode">
				
				<xsl:if test="InfoQueryInstance/InfoQuery/hideBackground = 'true'">
					<xsl:attribute name="class">infoquery show-mode nobackground</xsl:attribute>
				</xsl:if>
				
				<div class="heading-wrapper">
				
					<xsl:if test="not(InfoQueryInstance/InfoQuery/hideTitle = 'true')">
						<h2>
							<xsl:value-of select="InfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
				</div>
				
				<xsl:if test="InfoQueryInstance/InfoQuery/description">
					<span class="italic">
						<xsl:value-of select="InfoQueryInstance/InfoQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
							
			</article>
				
		</div>
		
	</xsl:template>	
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', InfoQueryInstance/InfoQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', InfoQueryInstance/InfoQuery/queryID)" />
	
		<div class="query infoquery" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query infoquery</xsl:text>
				<xsl:if test="InfoQueryInstance/InfoQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="InfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<xsl:if test="ValidationErrors/validationError[messageKey]">
						<div class="info-box error">
							<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
							<div class="marker"></div>
						</div>
					</xsl:if>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:attribute name="class">
					<xsl:if test="InfoQueryInstance/InfoQuery/hideBackground = 'true'">
						<xsl:text> nobackground</xsl:text>
					</xsl:if>
					
					<xsl:if test="ValidationErrors/validationError[messageKey]">
						<xsl:text> error</xsl:text>
					</xsl:if>
					
				</xsl:attribute>
			
				<div class="heading-wrapper">
					
					<xsl:if test="not(InfoQueryInstance/InfoQuery/hideTitle = 'true')">
						<h2>
							<xsl:value-of select="InfoQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
					
					<xsl:if test="InfoQueryInstance/InfoQuery/helpText">		
						<xsl:apply-templates select="InfoQueryInstance/InfoQuery/helpText" />
					</xsl:if>
				</div>
				
				<xsl:if test="InfoQueryInstance/InfoQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="InfoQueryInstance/InfoQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>				
				
			</article>			
		
		</div>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<i data-icon-after="!" title="{$i18n.InvalidFormat}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
		
</xsl:stylesheet>