package com.nordicpeak.flowengine.internalalternativelist;

import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URLDecoder;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.WorkbookUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.Breadcrumb;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.handlers.GroupHandler;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.crud.MultipartLimitProvider;
import se.unlogic.hierarchy.core.utils.crud.MultipartRequestFilter;
import se.unlogic.hierarchy.core.utils.usergrouplist.UserGroupListConnector;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOFactory;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.io.CloseUtils;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.threads.ExeptionHandlingRunnable;
import se.unlogic.standardutils.threads.ThreadUtils;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.exceptions.UnableToDeleteListInUseException;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeListProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableStep;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternative;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternativeList;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternativeListEvent;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternativeListInstance;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalCombindedListOrdering;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalCombinedList;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalCombinedListEvent;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalCombinedListInstance;

import it.sauronsoftware.cron4j.Scheduler;

public class InternalAlternativeListProviderModule extends BaseAlternativeListProvider<InternalAlternativeList, InternalAlternativeListInstance, InternalAlternativeListCRUD, InternalAlternativeListProviderModule, InternalAlternativeListEvent> implements Runnable, MultipartLimitProvider {

	public static final InternalAlternativeNameComparator INTERNAL_ALTERNATIVE_NAME_COMPARATOR = new InternalAlternativeNameComparator();

	@XSLVariable(prefix = "i18n.", name = "Show.Alternative.Name")
	private String exportName = "name";
	
	@XSLVariable(prefix = "i18n.", name = "AlternativeXMLValue")
	private String exportXML = "xml";
	
	@XSLVariable(prefix = "i18n.", name = "AlternativeAttributeValue")
	private String exportAttribute = "attribute";
	
	@XSLVariable(prefix = "i18n.", name = "Alternative.ValidFrom")
	private String exportValidFrom = "from";
	
	@XSLVariable(prefix = "i18n.", name = "Alternative.ValidTo")
	private String exportValidTo = "to";
	
	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Default admin groups", description = "These groups have admin access to all groups")
	private List<Integer> defaultAdminGroups;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max request size", description = "Maxmium allowed request size in megabytes", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer maxRequestSize = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "RAM threshold", description = "Maximum size of files in KB to be buffered in RAM during file uploads. Files exceeding the threshold are written to disk instead.", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer ramThreshold = 500;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Temp dir", description = "Directory for temporary files. Should be on the same filesystem as the file store for best performance. If not set system default temp directory will be used", required = true)
	protected String tempDir;

	protected AnnotatedDAO<InternalCombinedList> combinedListDAO;
	protected AnnotatedDAOWrapper<InternalCombinedList, Integer> combinedListDAOWrapper;
	protected AnnotatedDAO<InternalCombinedListEvent> combinedListEventDAO;

	protected QueryParameterFactory<InternalCombinedList, String> combinedListNameParamFactory;
	protected QueryParameterFactory<InternalCombinedListEvent, InternalCombinedList> combinedListEventCombinedListParamFactory;

	protected InternalCombinedListCRUD combinedListCRUD;

	private Scheduler scheduler;

	protected UserGroupListConnector userGroupListConnector;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, InternalAlternativeListProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		AnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface);

		listDAO = daoFactory.getDAO(InternalAlternativeList.class).getWrapper(Integer.class);
		listDAO.addRelations(InternalAlternativeList.ALTERNATIVES_RELATION);
		listDAO.setUseRelationsOnGet(true);
		listDAO.setUseRelationsOnGetAll(true);
		listDAO.setUseRelationsOnAdd(true);
		listDAO.setUseRelationsOnUpdate(true);

		listEventDAO = daoFactory.getDAO(InternalAlternativeListEvent.class);
		listEventListParamFactory = listEventDAO.getParamFactory(InternalAlternativeListEvent.LIST_RELATION, InternalAlternativeList.class);

		combinedListDAO = daoFactory.getDAO(InternalCombinedList.class);

		combinedListDAOWrapper = combinedListDAO.getWrapper(Integer.class);
		combinedListDAOWrapper.addRelations(InternalCombinedList.LISTS_RELATION);
		combinedListDAOWrapper.setUseRelationsOnGet(true);
		combinedListDAOWrapper.setUseRelationsOnGetAll(true);
		combinedListDAOWrapper.setUseRelationsOnAdd(true);
		combinedListDAOWrapper.setUseRelationsOnUpdate(true);

		combinedListNameParamFactory = combinedListDAO.getParamFactory("name", String.class);

		combinedListEventDAO = daoFactory.getDAO(InternalCombinedListEvent.class);
		combinedListEventCombinedListParamFactory = combinedListEventDAO.getParamFactory(InternalCombinedListEvent.LIST_RELATION, InternalCombinedList.class);

		listCRUD = new InternalAlternativeListCRUD(listDAO, new AnnotatedRequestPopulator<InternalAlternativeList>(InternalAlternativeList.class), this);
		listCRUD.addRequestFilter(new MultipartRequestFilter(this));
		combinedListCRUD = new InternalCombinedListCRUD(combinedListDAOWrapper, new AnnotatedRequestPopulator<InternalCombinedList>(InternalCombinedList.class), this);
	}

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		scheduler = new Scheduler(systemInterface.getApplicationName() + " - " + moduleDescriptor.toString());
		scheduler.setDaemon(true);
		scheduler.schedule("0 0 * * *", this);
		scheduler.start();

		this.userGroupListConnector = new UserGroupListConnector(systemInterface);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR);
	}	
	
	@Override
	public void unload() throws Exception {

		try {

			if (scheduler != null) {
				scheduler.stop();
				scheduler = null;
			}

		} catch (IllegalStateException e) {

			log.error("Error stopping scheduler", e);
		}

		super.unload();
	}

	@Override
	public AlternativeListDescriptor getAlternativeList(String listName) {

		AlternativeListDescriptor descriptor = super.getAlternativeList(listName);

		if (descriptor == null) {

			try {
				InternalCombinedList combinedList = getCombinedList(listName);

				if (combinedList != null) {

					return getNewCombinedDescriptor(combinedList);
				}

			} catch (SQLException e) {
				//TODO move error handling to AlternativeListHandler
				log.error("Error getting combined lists", e);
			}
		}

		return descriptor;
	}

	@Override
	public AlternativeListInstance getAlternativeListInstance(String name, AttributeHandler attributeHandler, User poster) {

		AlternativeListInstance listInstance = getCachedInstance(name);

		if (listInstance == null) {

			try {
				InternalCombinedList combinedList = getCombinedList(name, InternalCombinedList.LISTS_RELATION);

				if (combinedList != null) {

					return getCombinedInstance(combinedList, attributeHandler);
				}

			} catch (SQLException e) {

				//TODO move error handling to AlternativeListHandler
				log.error("Error getting combined list " + name, e);
			}
		}

		return listInstance;
	}

	@Override
	public List<AlternativeListDescriptor> getAlternativeLists() {

		List<AlternativeListDescriptor> descriptors = super.getAlternativeLists();

		if (descriptors == null) {
			return null;
		}

		try {
			List<InternalCombinedList> combinedLists = getAllCombinedLists();

			if (!CollectionUtils.isEmpty(combinedLists)) {
				for (InternalCombinedList combinedList : combinedLists) {

					descriptors.add(getNewCombinedDescriptor(combinedList));
				}
			}

			Collections.sort(descriptors);

		} catch (SQLException e) {
			log.error("Error getting combined lists", e);
		}

		return descriptors;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse addCombined(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return combinedListCRUD.add(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse showCombined(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return combinedListCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateCombined(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return combinedListCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse deleteCombined(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		try {
			return combinedListCRUD.delete(req, res, user, uriParser);

		} catch (UnableToDeleteListInUseException e) {

			return combinedListCRUD.show(req, res, user, uriParser, Collections.singletonList(LIST_IN_USE_VALIDATION_ERROR));
		}
	}

	@Override
	protected InternalAlternativeListInstance getNewInstance(AlternativeListDescriptor descriptor, InternalAlternativeList list) {

		if (list.getAlternatives() != null && list.isUseDateLimits()) {

			Date now = new Date(System.currentTimeMillis());
			Date startOfDay = DateUtils.setTimeToMidnight(new Date(System.currentTimeMillis()));

			for (Iterator<InternalAlternative> iterator = list.getAlternatives().iterator(); iterator.hasNext();) {

				InternalAlternative alternative = iterator.next();

				if (alternative.getValidFrom() != null && now.before(alternative.getValidFrom())) {
					iterator.remove();
				}

				if (alternative.getValidTo() != null && startOfDay.after(alternative.getValidTo())) {
					iterator.remove();
				}
			}
		}

		return new InternalAlternativeListInstance(descriptor, list.getAlternatives());
	}

	@Override
	protected InternalAlternativeListEvent getNewListEvent(InternalAlternativeList list, Timestamp added, User poster, String message) {

		return new InternalAlternativeListEvent(list, added, poster, message);
	}

	@SuppressWarnings("unchecked")
	public InternalCombinedListInstance getCombinedInstance(InternalCombinedList combinedList, AttributeHandler attributeHandler) {

		List<InternalAlternative> alternatives = new ArrayList<InternalAlternative>();

		if (!CollectionUtils.isEmpty(combinedList.getLists())) {

			for (InternalAlternativeList list : combinedList.getLists()) {

				InternalAlternativeListInstance instance = getAlternativeInstance(list, attributeHandler, null);

				if (instance != null && !CollectionUtils.isEmpty(instance.getAlternatives())) {

					alternatives.addAll((Collection<InternalAlternative>) instance.getAlternatives());
				}
			}
		}

		if (combinedList.getOrdering() == InternalCombindedListOrdering.AZ) {

			Collections.sort(alternatives, INTERNAL_ALTERNATIVE_NAME_COMPARATOR);

		} else if (combinedList.getOrdering() == InternalCombindedListOrdering.ZA) {

			Collections.sort(alternatives, Collections.reverseOrder(INTERNAL_ALTERNATIVE_NAME_COMPARATOR));
		}

		return new InternalCombinedListInstance(getNewCombinedDescriptor(combinedList), alternatives);
	}

	public AlternativeListDescriptor getNewCombinedDescriptor(InternalCombinedList combinedList) {

		return new AlternativeListDescriptor(providerID, combinedList.getName());
	}

	public InternalAlternativeList getList(Integer listID) throws SQLException {

		return listDAO.get(listID);
	}

	public void addCombinedListEvent(String message, InternalCombinedList list, User user) {

		message = StringUtils.substring(message, 255, "...");

		InternalCombinedListEvent event = new InternalCombinedListEvent(list, new Timestamp(System.currentTimeMillis()), user, message);

		try {
			combinedListEventDAO.add(event);

		} catch (SQLException e) {
			log.error("Error adding new combined list event " + event, e);
		}
	}

	public List<InternalCombinedListEvent> getCombinedListEvents(InternalCombinedList list) throws SQLException {

		HighLevelQuery<InternalCombinedListEvent> query = new HighLevelQuery<InternalCombinedListEvent>();

		query.addParameter(combinedListEventCombinedListParamFactory.getParameter(list));

		return combinedListEventDAO.getAll(query);
	}

	public List<InternalCombinedListEvent> getRecentCombinedListEvents(InternalCombinedList list) throws SQLException {

		HighLevelQuery<InternalCombinedListEvent> query = new HighLevelQuery<InternalCombinedListEvent>();

		query.addParameter(combinedListEventCombinedListParamFactory.getParameter(list));
		query.setRowLimiter(new MySQLRowLimiter(recentListEventsCount));

		return combinedListEventDAO.getAll(query);
	}

	@WebPublic(alias = "combinedevents")
	public ForegroundModuleResponse showCombinedListEvents(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Integer listID;
		InternalCombinedList combinedList;

		if (uriParser.size() >= 3 && PositiveStringIntegerPopulator.getPopulator().validateFormat(uriParser.get(2)) && (listID = PositiveStringIntegerPopulator.getPopulator().getValue(uriParser.get(2))) != null && (combinedList = combinedListDAOWrapper.get(listID)) != null) {

			Document doc = createDocument(req, uriParser, user);
			Element showAlternativeListEventsElement = doc.createElement("ShowAlternativeListEvents");
			doc.getDocumentElement().appendChild(showAlternativeListEventsElement);

			showAlternativeListEventsElement.appendChild(combinedList.toXML(doc));
			XMLUtils.append(doc, showAlternativeListEventsElement, "Events", getCombinedListEvents(combinedList));

			return new SimpleForegroundModuleResponse(doc, getTitlePrefix(), getDefaultBreadcrumb(), new Breadcrumb(this, combinedList.getListName(), "/show/" + combinedList.getListID()));
		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "combinedusage")
	public ForegroundModuleResponse showCombinedListUsage(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Integer listID;
		InternalCombinedList combinedList;

		if (uriParser.size() >= 3 && PositiveStringIntegerPopulator.getPopulator().validateFormat(uriParser.get(2)) && (listID = PositiveStringIntegerPopulator.getPopulator().getValue(uriParser.get(2))) != null && (combinedList = combinedListDAOWrapper.get(listID)) != null) {

			if(!hasListAccess(user, combinedList)) {
				
				throw new AccessDeniedException("User " + user + " is not an administrator for list " + combinedList);
			}
			
			Document doc = createDocument(req, uriParser, user);
			Element showAlternativeListUsageElement = doc.createElement("ShowAlternativeListFlowUsage");
			doc.getDocumentElement().appendChild(showAlternativeListUsageElement);

			showAlternativeListUsageElement.appendChild(combinedList.toXML(doc));

			List<AlternativeListQuery> queries = alternativeListHandlerModule.getQueriesUsingAlternativeList(getNewCombinedDescriptor(combinedList));

			if (!CollectionUtils.isEmpty(queries)) {

				Element queriesElement = XMLUtils.appendNewElement(doc, showAlternativeListUsageElement, "Queries");

				Map<Integer, Element> flowElements = new HashMap<Integer, Element>();
				Map<Integer, Element> stepElements = new HashMap<Integer, Element>();

				for (AlternativeListQuery query : queries) {

					ImmutableFlow flow = query.getQueryDescriptor().getStep().getFlow();
					Element flowElement = flowElements.get(flow.getFlowID());

					if (flowElement == null) {

						flowElement = (Element) queriesElement.appendChild(flow.toXML(doc));
						flowElements.put(flow.getFlowID(), flowElement);
					}

					ImmutableStep step = query.getQueryDescriptor().getStep();
					Element stepElement = stepElements.get(step.getStepID());

					if (stepElement == null) {

						stepElement = (Element) flowElement.appendChild(step.toXML(doc));
						stepElements.put(step.getStepID(), stepElement);
					}

					Element queryElement = XMLUtils.appendNewElement(doc, stepElement, "Query");

					XMLUtils.appendNewElement(doc, queryElement, "ListName", query.getAlternativeListName());
					XMLUtils.appendNewElement(doc, queryElement, "ExtraUsageInfo", query.getExtraUsageInfo());
					queryElement.appendChild(query.getQueryDescriptor().toXML(doc));
				}
			}

			return new SimpleForegroundModuleResponse(doc, getTitlePrefix(), getDefaultBreadcrumb(), new Breadcrumb(this, combinedList.getListName(), "/show/" + combinedList.getListID()));
		}

		throw new URINotFoundException(uriParser);
	}

	private InternalCombinedList getCombinedList(String listName, Field... relations) throws SQLException {

		HighLevelQuery<InternalCombinedList> query = new HighLevelQuery<InternalCombinedList>();

		query.addParameter(combinedListNameParamFactory.getParameter(listName));

		if (relations != null) {
			query.addRelations(relations);
		}

		return combinedListDAO.get(query);
	}

	protected List<InternalCombinedList> getAllCombinedLists() throws SQLException {

		return combinedListDAOWrapper.getAll();
	}

	@WebPublic(alias = "lists")
	public ForegroundModuleResponse searchLists(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		String search = req.getParameter("q");

		if (StringUtils.isEmpty(search)) {

			sendEmptyJSONResponse(res);
			return null;
		}

		if (!systemInterface.getEncoding().equalsIgnoreCase("UTF-8")) {
			search = URLDecoder.decode(search, "UTF-8");
		}

		List<InternalAlternativeList> lists = getAllLists();

		if (lists == null) {

			sendEmptyJSONResponse(res);
			return null;
		}

		String searchLower = search.toLowerCase();

		for (Iterator<InternalAlternativeList> it = lists.iterator(); it.hasNext();) {

			InternalAlternativeList list = it.next();

			if (!list.getName().toLowerCase().contains(searchLower) || !hasListAccess(user, list)) {
				it.remove();
			}
		}

		log.info("User " + user + " searching for internal lists using query " + search + ", found " + CollectionUtils.getSize(lists) + " hits");

		if (lists.isEmpty()) {

			sendEmptyJSONResponse(res);
			return null;
		}

		JsonArray jsonArray = new JsonArray();

		for (InternalAlternativeList list : lists) {

			JsonObject row = new JsonObject();
			row.putField("id", list.getListID());
			row.putField("name", list.getName());
			jsonArray.addNode(row);
		}

		sendJSONResponse(jsonArray, res);
		return null;
	}

	protected static void sendEmptyJSONResponse(HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(1);
		jsonObject.putField("hitCount", "0");
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	protected void sendJSONResponse(JsonArray jsonArray, HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(2);
		jsonObject.putField("hitCount", Integer.toString(jsonArray.size()));
		jsonObject.putField("hits", jsonArray);
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	@WebPublic(alias = "groups")
	public ForegroundModuleResponse getGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return userGroupListConnector.getGroups(req, res, user, uriParser);
	}

	@WebPublic(alias = "users")
	public ForegroundModuleResponse getUsers(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return userGroupListConnector.getUsers(req, res, user, uriParser);
	}

	public UserHandler getUserHandler() {

		return systemInterface.getUserHandler();
	}

	public GroupHandler getGroupHandler() {

		return systemInterface.getGroupHandler();
	}

	public boolean hasFullAccess(User user) {

		return AccessUtils.checkGroupAccess(user, defaultAdminGroups);
	}
	
	public boolean hasListAccess(User user, InternalListAccessInterface listAccess) {
		
		if (AccessUtils.checkGroupAccess(user, defaultAdminGroups)) {
			
			return true;
		}
		
		if (listAccess.isAdminAccessControlEnabled()) {
			
			return  AccessUtils.checkAccess(user, listAccess);
		}

		return false;
	}
	
	@WebPublic(alias = "export")
	public ForegroundModuleResponse exportToExcel(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		Integer listID;
		InternalAlternativeList list;

		if (uriParser.size() >= 3 && PositiveStringIntegerPopulator.getPopulator().validateFormat(uriParser.get(2)) && (listID = PositiveStringIntegerPopulator.getPopulator().getValue(uriParser.get(2))) != null && (list = listDAO.get(listID)) != null) {

			if (!hasListAccess(user, list)) {
				throw new AccessDeniedException("User " + user + " is not an administrator for list " + list);
			}

			//Workaround for thread local usage in XMLBeans used by XSSF
			ExeptionHandlingRunnable runnable = new ExeptionHandlingRunnable() {

				private Exception exception;

				@Override
				public void run() {
					try {
						Workbook workbook = new XSSFWorkbook();

						try {
							String sheetName = StringUtils.substring(list.getName(), 31, "...");
							sheetName = WorkbookUtil.createSafeSheetName(sheetName);
							
							Sheet sheet = workbook.createSheet(sheetName);
							sheet.setColumnWidth(0, 1500);

							Row row = null;

							int rowIndex = 0;
							int cellIndex = 0;
							
							row = sheet.createRow(rowIndex++);

							Font titleFont = workbook.createFont();
							titleFont.setBold(true);
							titleFont.setFontHeightInPoints((short) 16);

							CellStyle titleStyle = workbook.createCellStyle();
							titleStyle.setFont(titleFont);

							Font subTitleFont = workbook.createFont();
							subTitleFont.setBold(true);
							CellStyle subTitleStyle = workbook.createCellStyle();
							subTitleStyle.setFont(subTitleFont);

							createCell(row, 0, list.getName(), titleStyle);
							
							row = sheet.createRow(rowIndex++);
							createCell(row, cellIndex++, exportName, subTitleStyle);
							createCell(row, cellIndex++, exportXML, subTitleStyle);
							createCell(row, cellIndex++, exportAttribute, subTitleStyle);
							createCell(row, cellIndex++, exportValidFrom, subTitleStyle);
							createCell(row, cellIndex++, exportValidTo, subTitleStyle);
							
							if(list.getAlternatives() != null) {
								
								for (InternalAlternative alternative : list.getAlternatives()) {
									row = sheet.createRow(rowIndex++);
									cellIndex = 0;
									
									createCell(row, cellIndex++, alternative.getName(), null);
									
									if (alternative.getExportXMLValue() != null) {
										createCell(row, cellIndex++, alternative.getExportXMLValue(), null);
									} else {
										cellIndex++;
									}
									
									if (alternative.getExportXMLValue() != null) {
										createCell(row, cellIndex++, alternative.getAttributeValue(), null);
									} else {
										cellIndex++;
									}
									
									if (list.isUseDateLimits()) {
										
										if (alternative.getValidFrom() != null) {
											createCell(row, cellIndex++, DateUtils.DATE_FORMATTER.format(alternative.getValidFrom()), null);
										}
										
										if (alternative.getValidTo() != null) {
											createCell(row, cellIndex++, DateUtils.DATE_FORMATTER.format(alternative.getValidTo()), null);
										}
									}
								}							
							}
							
							try {
								res.setHeader("Content-Disposition", "attachment;" + FileUtils.toContentDispositionFilename(list.getName() + ".xlsx"));
								res.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
	
								workbook.write(res.getOutputStream());
	
								res.getOutputStream().flush();
	
							} catch (Exception e) {
	
								log.info("Unable to send excel file to user " + user + ", " + e);
							}

						} finally {
							CloseUtils.close(workbook);
						}

					} catch (Exception e) {
						exception = e;
					}
				}
	
				@Override
				public Exception getException() {
					return exception;
				}
			};
	
			ThreadUtils.runAndWait(runnable, "Internal alternative list export thread for user " + user, true);
	
			if (runnable.getException() != null) {
				throw runnable.getException();
			}
		}

		return null;
	}
	
	private static Cell createCell(Row row, int cellPos, Object value, CellStyle style) {

		Cell cell = row.createCell(cellPos);
		
		if(value != null) {
			
			cell.setCellValue(value.toString());
		
		}else {
			
			cell.setCellValue("");
		}
		
		if(style != null) {
			
			cell.setCellStyle(style);
		}

		return cell;
	}
	
	@Override
	public String getTempDir() {

		return tempDir;
	}

	@Override
	public long getMaxRequestSize() {

		return maxRequestSize;
	}
	
	@Override
	public int getRamThreshold() {

		return ramThreshold;
	}

}
