package com.nordicpeak.flowengine.queries.internaluserquery;

import java.io.IOException;
import java.net.URL;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.Group;
import se.unlogic.hierarchy.core.beans.SimpleAttribute;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.handlers.GroupHandler;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.UserUtils;
import se.unlogic.hierarchy.core.utils.usergrouplist.UserGroupListConnector;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.Stringyfier;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xsl.XSLVariableReader;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesAttributeDescriptor;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesConstants;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesSelectedAttribute;
import com.nordicpeak.flowengine.queries.internaluserqueries.interfaces.CommonInternalUserQueryProvider;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.InternalUserQuery;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.InternalUserQueryInstance;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.ManagerAttributeValue;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.SelectedManagerAttribute;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.SelectedUserAttribute;
import com.nordicpeak.flowengine.queries.internaluserquery.beans.UserAttributeValue;
import com.nordicpeak.flowengine.queries.internaluserquery.exceptions.NoGroupFoundByAttributeException;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class InternalUserQueryProviderModule extends BaseQueryProviderModule<InternalUserQueryInstance> implements BaseQueryCRUDCallback, CommonInternalUserQueryProvider {

	@XSLVariable(prefix = "java.")
	private String exportFirstName;

	@XSLVariable(prefix = "java.")
	private String exportLastName;

	@XSLVariable(prefix = "java.")
	private String exportUsername;

	@XSLVariable(prefix = "java.")
	private String exportEmail;

	@XSLVariable(prefix = "java.")
	private String exportManager;

	protected String noGroupFoundByAttributeErrorDescription;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Search result limit", description = "Maximum number of hits received", required = true, formatValidator = PositiveStringIntegerValidator.class)
	private Integer searchResultLimit = 100;
	
	@ModuleSetting
	@SplitOnLineBreak
	@TextAreaSettingDescriptor(name = "Supported attributes", description = "The format is name:[display_name[:formatterClass]] (without brackets). Only the name is required")
	private List<String> supportedAttributes;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "Username formatter", description = "formatter to use for usernames")
	private String usernameFormatter;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable closest manager", description = "Enable info and multisign support for posters closest manager")
	private boolean enableClosestManager;

	private List<InternalUserQueriesAttributeDescriptor> attributeDescriptors;

	private AnnotatedDAO<InternalUserQuery> queryDAO;
	private AnnotatedDAO<InternalUserQueryInstance> queryInstanceDAO;

	private InternalUserQueryCRUD queryCRUD;

	private QueryParameterFactory<InternalUserQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<InternalUserQueryInstance, Integer> queryInstanceIDParamFactory;

	protected UserGroupListConnector userGroupListConnector;

	protected Stringyfier<String> usernameStringFormatter;
	protected Map<String, Stringyfier<String>> attributeStringFormatter = new HashMap<>();

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		// Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, InternalUserQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(InternalUserQuery.class);
		queryInstanceDAO = daoFactory.getDAO(InternalUserQueryInstance.class);

		queryCRUD = new InternalUserQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<>(InternalUserQuery.class), "ContactDetailQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);

		userGroupListConnector = new UserGroupListConnector(systemInterface);
	}

	@Override
	protected synchronized void moduleConfigured() throws Exception {

		super.moduleConfigured();

		if (ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR)) {

			attributeDescriptors = parseAttributes();

		} else {

			attributeDescriptors = null;
		}
	}

	@Override
	protected XSLVariableReader parseQueryXSLStyleSheet(URL styleSheetURL) {

		XSLVariableReader variableReader = super.parseQueryXSLStyleSheet(styleSheetURL);

		if (variableReader != null) {

			try {

				noGroupFoundByAttributeErrorDescription = variableReader.getValue("i18n.Error.noGroupFoundByAttributeErrorDescription.Default");

			} catch (Exception e) {

				log.error("Unable to get i18n.Error.noGroupFoundByAttributeErrorDescription.Default from query style sheet " + queryStyleSheet, e);
			}
		}

		return variableReader;
	}

	private List<InternalUserQueriesAttributeDescriptor> parseAttributes() {

		usernameStringFormatter = null;

		if (usernameFormatter != null) {
			try {
				@SuppressWarnings("unchecked")
				Stringyfier<String> stringFormatter2 = (Stringyfier<String>) ReflectionUtils.getInstance(usernameFormatter);
				usernameStringFormatter = stringFormatter2;
			} catch (Exception e) {
				log.error("Unable to find stringformatter for username.");
			}
		}

		attributeStringFormatter.clear();
		List<InternalUserQueriesAttributeDescriptor> attributeDescriptors = new ArrayList<>(supportedAttributes == null ? 0 : supportedAttributes.size());

		if (supportedAttributes != null) {
			for (String attribute : supportedAttributes) {

				String[] attributeParts = attribute.replace("\r", "").split(":");

				if (attributeParts.length == 0 || attributeParts.length > 3) {

					log.warn("Unable to parse format of attribute: " + attribute + ", ignoring attribute.");
					continue;
				}

				String name = attributeParts[0];
				
				String displayName;
				Stringyfier<String> stringFormatter = null;

				if (attributeParts.length > 1) {

					displayName = attributeParts[1];

				} else {

					displayName = null;
				}

				if (attributeParts.length >= 3) {

					try {
						@SuppressWarnings("unchecked")
						Stringyfier<String> stringFormatter2 = (Stringyfier<String>) ReflectionUtils.getInstance(attributeParts[2]);
						stringFormatter = stringFormatter2;

						attributeStringFormatter.put(name, stringFormatter);
					} catch (Exception e) {
						log.error("Unable to find stringformatter for attribute: " + attribute + ", adding without formatter.");
					}
				}

				attributeDescriptors.add(new InternalUserQueriesAttributeDescriptor(name, displayName, stringFormatter));
			}
		}

		if (attributeDescriptors.isEmpty()) {

			return null;

		} else {

			return attributeDescriptors;
		}
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		InternalUserQuery query = new InternalUserQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		InternalUserQuery query = new InternalUserQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		if (query.getSelectedUserAttributes() != null) {

			if (getAttributeDescriptors() == null) {

				query.setSelectedUserAttributes(null);

			} else {

				List<SelectedUserAttribute> refreshedSelectedAttributes = new ArrayList<>(query.getSelectedUserAttributes().size());

				for (SelectedUserAttribute selectedAttribute : query.getSelectedUserAttributes()) {

					InternalUserQueriesAttributeDescriptor attributeDescriptor = getAttributeDescriptor(selectedAttribute);

					if (attributeDescriptor != null) {

						refreshedSelectedAttributes.add(new SelectedUserAttribute(attributeDescriptor.getName(), attributeDescriptor.getDisplayName(), selectedAttribute.getSortIndex()));
					}
				}

				query.setSelectedUserAttributes(refreshedSelectedAttributes);
			}
		}

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	private InternalUserQueriesAttributeDescriptor getAttributeDescriptor(SelectedUserAttribute selectedAttribute) {

		for (InternalUserQueriesAttributeDescriptor attributeDescriptor : attributeDescriptors) {
			if (attributeDescriptor.getName().equalsIgnoreCase(selectedAttribute.getName())) {
				return attributeDescriptor;
			}
		}

		return null;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		InternalUserQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		InternalUserQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		InternalUserQueryInstance queryInstance;

		// Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new InternalUserQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private InternalUserQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<InternalUserQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private InternalUserQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<InternalUserQuery> query = new HighLevelQuery<>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private InternalUserQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<InternalUserQueryInstance> query = new HighLevelQuery<>(InternalUserQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public QueryResponse getFormHTML(InternalUserQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		try {

			List<Integer> groupIDs = getGroupIDs(queryInstance.getQuery(), attributeHandler);

			queryInstance.setGroupIDs(groupIDs);

		} catch (NoGroupFoundByAttributeException e) {

			validationErrors = CollectionUtils.addAndInstantiateIfNeeded(validationErrors, new ValidationError("NoGroupFoundByAttribute"));
		}
		
		Document doc = createDocument(req, poster);
		
		if (queryInstance.getQuery().isMaskCitizenIDLastFourDigits()) {
			
			XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
			
			genDoc.addAssignableFieldElementableListener(SimpleAttribute.class, CitizenIdentifierUtils.getMaskCitizenIdentifierAttributeElementableListener());
			
			doc = genDoc;
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler, doc);
	}

	public List<Integer> getGroupIDs(InternalUserQuery query, AttributeHandler attributeHandler) throws NoGroupFoundByAttributeException {

		if (query.isGetGroupsByAttribute()) {

			List<Integer> groupIDs = null;
			List<String> groupNames = null;

			for (String groupGettingAttribute : query.getGroupGettingAttributes()) {

				Integer groupID = attributeHandler.getInt(groupGettingAttribute);

				if (groupID != null) {

					groupIDs = CollectionUtils.addAndInstantiateIfNeeded(groupIDs, groupID);
					continue;
				}

				String groupName = attributeHandler.getString(groupGettingAttribute);

				if (groupName != null) {

					groupNames = CollectionUtils.addAndInstantiateIfNeeded(groupNames, groupName);
				}
			}

			List<Group> groups = null;

			if (groupIDs != null) {

				List<Group> idGroups = getGroupHandler().getGroups(groupIDs, false);

				groups = CollectionUtils.addAndInstantiateIfNeeded(groups, idGroups);
			}

			if (groupNames != null) {

				List<Group> nameGroups = getGroupHandler().getGroupsByName(groupNames, false);

				groups = CollectionUtils.addAndInstantiateIfNeeded(groups, nameGroups);
			}

			if (groups == null) {

				throw new NoGroupFoundByAttributeException();
			}

			return CollectionUtils.map(groups, Group::getGroupID);
		}

		return query.getGroupIDs();
	}

	@Override
	public void save(InternalUserQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public void populate(InternalUserQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		InternalUserQuery query = queryInstance.getQuery();
		Integer queryID = query.getQueryID();

		if (query.isLockForOwnerUpdate() && requestMetadata.flowInstanceIsSubmitted() && !requestMetadata.isManager() ) {
			return;
		}
		
		List<ValidationError> validationErrors = new ArrayList<>();

		String userFieldName = "q" + queryID + "_user";

		Integer userID = ValidationUtils.validateParameter(userFieldName, req, !allowPartialPopulation && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED, IntegerPopulator.getPopulator(), validationErrors);
		User selectedUser = null;

		if (userID != null) {

			List<Integer> groupIDs = queryInstance.getGroupIDs();

			selectedUser = systemInterface.getUserHandler().getUser(userID, groupIDs != null, true);

			if (selectedUser == null) {

				validationErrors.add(new ValidationError(userFieldName, ValidationErrorType.InvalidFormat));

			} else if (groupIDs != null && (selectedUser.getGroups() == null || CollectionUtils.isEmpty(CollectionUtils.conjunction(UserUtils.getUserGroupIDs(selectedUser), groupIDs)))) {

				validationErrors.add(new ValidationError(userFieldName, ValidationErrorType.InvalidFormat));

			} else if (query.isLimitSearchToMyEmployees() && (poster == null || poster.getUsername() == null || !poster.getUsername().equals(selectedUser.getAttributeHandler().getString(InternalUserQueriesConstants.USER_MANAGER_USERNAME_ATTRIBUTE)))) {

				validationErrors.add(new ValidationError(userFieldName, ValidationErrorType.InvalidFormat));

			} else if(query.isExcludeInactiveUsers() && !selectedUser.isEnabled()) {
				
				validationErrors.add(new ValidationError("InactiveAccountSelected"));
				
			} else if (query.isBlockPosterSelection() && selectedUser.equals(poster)) {

				validationErrors.add(new ValidationError("MayNotSelectPoster"));

			} else {

				queryInstance.reset(null);
				populateQueryInstance(queryInstance, selectedUser);

				if (allowPartialPopulation) {

					queryInstance.getQueryInstanceDescriptor().setPopulated(true);

					if (queryInstance.getQuery().isSetAsAttribute()) {

						queryInstance.resetFlowInstanceAttributes(attributeHandler);
						queryInstance.setFlowInstanceAttributes(attributeHandler);
					}

					return;
				}

				if (query.requiresMultipartSigning() && queryInstance.getCitizenIdentifier() == null) {

					validationErrors.add(new ValidationError("NoCitizenIdentifierFound"));
				}

				if(this.enableClosestManager) {

					if ((query.isShowManagerInfo() || query.isRequireManagerSigning()) && !queryInstance.isManagerPopulated()) {
	
						validationErrors.add(new ValidationError("UnableToGetManagerDetails"));
	
					} else if (query.isRequireManagerSigning() && (queryInstance.getManagerCitizenID() == null || queryInstance.getManagerEmail() == null && queryInstance.getManagerMobilePhone() == null)) {
	
						validationErrors.add(new ValidationError("ManagerDoesNotHaveRequiredFieldsForSigning"));
					}
				}
			}
		}

		if (selectedUser == null) {
			queryInstance.reset(attributeHandler);
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		if (selectedUser != null) {

			queryInstance.getQueryInstanceDescriptor().setPopulated(true);

			if (queryInstance.getQuery().isSetAsAttribute()) {

				queryInstance.resetFlowInstanceAttributes(attributeHandler);
				queryInstance.setFlowInstanceAttributes(attributeHandler);
			}

		} else {

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
		}
	}

	private void populateQueryInstance(InternalUserQueryInstance queryInstance, User user) {

		InternalUserQuery query = queryInstance.getQuery();

		queryInstance.setUserID(user.getUserID());
		queryInstance.setCitizenIdentifier(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user));

		if (query.isShowName()) {
			queryInstance.setFirstname(user.getFirstname());
			queryInstance.setLastname(user.getLastname());
		}

		if (query.isShowUsername()) {
			queryInstance.setUsername(formatUsername(user.getUsername()));
		}

		if (query.isShowEmail() || query.isUseForMultipartSigning()) {
			queryInstance.setEmail(user.getEmail());
		}

		if (query.getSelectedUserAttributes() != null) {

			queryInstance.setUserAttributeValues(new ArrayList<>());

			for (SelectedUserAttribute selectedAttribute : queryInstance.getQuery().getSelectedUserAttributes()) {

				String value = user.getAttributeHandler().getString(selectedAttribute.getName());

				if (value != null) {
					queryInstance.getUserAttributeValues().add(new UserAttributeValue(selectedAttribute.getName(), formatAttribute(selectedAttribute, value)));
				}
			}
		}

		if (this.enableClosestManager && (query.isShowManagerInfo() || query.isRequireManagerSigning())) {

			String managerUsername = user.getAttributeHandler().getString(InternalUserQueriesConstants.USER_MANAGER_USERNAME_ATTRIBUTE);

			if (managerUsername != null) {

				User manager = systemInterface.getUserHandler().getUserByUsername(managerUsername, false, true);

				if (manager != null) {

					queryInstance.setManagerUsername(managerUsername);
					queryInstance.setManagerCitizenID(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(manager));
					queryInstance.setManagerEmail(manager.getEmail());
					queryInstance.setManagerFirstname(manager.getFirstname());
					queryInstance.setManagerLastname(manager.getLastname());
					queryInstance.setManagerMobilePhone(ObjectUtils.getFirstNotNull(manager.getAttributeHandler().getString("mobile"), manager.getAttributeHandler().getString("mobilePhone")));

					if (query.isShowManagerInfo() && query.getSelectedManagerAttributes() != null) {

						queryInstance.setManagerAttributeValues(new ArrayList<>());

						for (SelectedManagerAttribute selectedAttribute : queryInstance.getQuery().getSelectedManagerAttributes()) {

							String value = manager.getAttributeHandler().getString(selectedAttribute.getName());

							if (value != null) {
								queryInstance.getManagerAttributeValues().add(new ManagerAttributeValue(selectedAttribute.getName(), formatAttribute(selectedAttribute, value)));
							}
						}
					}
				}
			}
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		InternalUserQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		InternalUserQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public Document createDocument(HttpServletRequest req, User poster) {

		Document doc = super.createDocument(req, poster);

		if (poster != null) {

			Element userElement = poster.toXML(doc);

			doc.getDocumentElement().appendChild(userElement);

			if (poster.getAttributeHandler() != null) {

				XMLUtils.appendNewElement(doc, userElement, "CitizenIdentifier", poster.getAttributeHandler().getString("citizenIdentifier"));
			}
		}

		return doc;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		InternalUserQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, InternalUserQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<InternalUserQueryInstance> getQueryInstanceClass() {

		return InternalUserQueryInstance.class;
	}

	@Override
	public String getExportFirstName() {

		return exportFirstName;
	}

	@Override
	public String getExportLastName() {

		return exportLastName;
	}

	@Override
	public String getExportUsername() {

		return exportUsername;
	}

	@Override
	public String getExportEmail() {

		return exportEmail;
	}

	@Override
	public String getExportManager() {

		return exportManager;
	}

	public List<InternalUserQueriesAttributeDescriptor> getAttributeDescriptors() {

		return attributeDescriptors;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(InternalUserQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws IOException {

		return new InternalUserQueryRequestProcessor(queryInstance, poster, systemInterface.getUserHandler(), systemInterface.getEncoding(), this);
	}

	public UserHandler getUserHandler() {

		return systemInterface.getUserHandler();
	}

	public GroupHandler getGroupHandler() {

		return systemInterface.getGroupHandler();
	}

	@WebPublic(alias = "groups")
	public ForegroundModuleResponse getGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if(this.flowAdminModule == null || !AccessUtils.checkAccess(user, flowAdminModule.getModuleDescriptor())) {
			
			throw new AccessDeniedException("User does not have access to list groups via this module");
		}
		
		return userGroupListConnector.getGroups(req, res, user, uriParser);
	}

	public String formatUsername(String username) {

		if (usernameStringFormatter != null) {
			return usernameStringFormatter.format(username);
		}

		return username;
	}

	public String formatAttribute(InternalUserQueriesSelectedAttribute selectedAttribute, String value) {

		Stringyfier<String> formatter = attributeStringFormatter.get(selectedAttribute.getName());

		if (formatter != null) {
			return formatter.format(value);
		}

		return value;
	}

	public boolean isEnableClosestManager() {

		return enableClosestManager;
	}
	
	public Integer getSearchResultLimit() {
		return searchResultLimit;
	}
	
	public String getContactFieldDisplayName(String name) {

		if (name.equals("firstname")) {

			return exportFirstName;
		}

		if (name.equals("lastname")) {

			return exportLastName;
		}

		if (name.equals("username")) {

			return exportUsername;
		}

		if (name.equals("email")) {

			return exportEmail;
		}

		if (name.equals("manager")) {

			return exportManager;
		}

		if (name.equals("managerFirstname")) {

			return exportManager + " " + exportFirstName.toLowerCase();
		}

		if (name.equals("managerLastname")) {

			return exportManager + " " + exportLastName.toLowerCase();
		}

		if (name.equals("managerUsername")) {

			return exportManager + " " + exportUsername.toLowerCase();
		}

		if (name.equals("managerEmail")) {

			return exportManager + " " + exportEmail.toLowerCase();
		}

		return null;
	}
}
