package com.nordicpeak.lmsearch;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.log4j.Logger;

import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressMemberType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressResponseType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressreferensResponseType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressreferensType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.IdRequestType;
import se.lantmateriet.namespace.distribution.produkter.common.FaultType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.EnhetsomradeType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.EnhetsutrymmeType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.GeometriRequestType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterbeteckningType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterenhetMemberType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterenhetResponseType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterenhetType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterenhetsreferensResponseType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterenhetsreferensType;
import se.lantmateriet.namespace.distribution.produkter.ortnamn.v2.Ortnamn;
import se.lantmateriet.namespace.distribution.produkter.ortnamn.v2.OrtnamnResponse;
import se.lantmateriet.namespace.distribution.produkter.ortnamn.v2.OrtnamnsPlacering;
import se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.BeteckningType;
import se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterbeteckningMemberType;
import se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterbeteckningResponseType;
import se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterbeteckningsreferensResponseType;
import se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterbeteckningsreferensType;
import se.unlogic.hierarchy.core.interfaces.providers.OAuthCredentialsProvider;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.MillisecondTimeUnits;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.SimpleRequest;
import se.unlogic.webutils.http.exceptions.HTTPException;

import com.nordicpeak.lmsearch.beans.HeightResponse;
import com.nordicpeak.lmsearch.beans.PlaceName;
import com.nordicpeak.lmsearch.beans.PointTypeJsonFactory;
import com.nordicpeak.lmsearch.beans.RegisterUnit;
import com.nordicpeak.lmsearch.beans.UnitArea;
import com.nordicpeak.lmsearch.utils.LMTokenHandler;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.PrecisionModel;
import com.vividsolutions.jts.operation.union.UnaryUnionOp;

import flexjson.JSONDeserializer;
import net.opengis.gml._3.AbstractGeometryType;
import net.opengis.gml._3.GeometryArrayPropertyType;
import net.opengis.gml._3.GeometryPropertyType;
import net.opengis.gml._3.ObjectFactory;
import net.opengis.gml._3.PointPropertyType;
import net.opengis.gml._3.PointType;

public class LMConnection {

	protected Logger log = Logger.getLogger(this.getClass());

	private static final Charset UTF8 = Charset.forName("UTF-8");

	private static final String BASICAUTH_LMSERVICE_URL = "https://services.lantmateriet.se";
	private static final String OAUTH_LMSERVICE_URL = "https://api.lantmateriet.se";

	private static final GMLConverter GML_CONVERTER = new GMLConverter();
	private static final GeometryFactory GEOMETRY_FACTORY = new GeometryFactory(new PrecisionModel(PrecisionModel.FLOATING), 3006);
	private static final ObjectFactory objectFactory = new ObjectFactory();

	private static final se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.ObjectFactory addressFactory = new se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.ObjectFactory();
	private static final se.lantmateriet.namespace.distribution.produkter.fastighet.v2.ObjectFactory estateFactory = new se.lantmateriet.namespace.distribution.produkter.fastighet.v2.ObjectFactory();
	private static final se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.ObjectFactory designationFactory = new se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.ObjectFactory();

	private static Map<Integer, Map<Integer, String>> countyMap = new HashMap<Integer, Map<Integer, String>>();

	private JAXBContext addressContext;
	private JAXBContext estateContext;
	private JAXBContext designationContext;

	private final String basicAuthUsername;
	private final String basicAuthPassword;
	private final int connectionTimeout;
	private final int requestTimeout;
	private final int maxResults;
	private final int minimumSearchStringLength;

	private final String lmServiceURL;

	private final LMTokenHandler lmTokenHandler;

	static {
		// For complete county list see https://www.scb.se/hitta-statistik/regional-statistik-och-kartor/regionala-indelningar/lan-och-kommuner/lan-och-kommuner-i-kodnummerordning/

		// 8 Kalmar ln
		HashMap<Integer, String> county08 = new HashMap<Integer, String>();
		county08.put(21, "Hgsby");
		county08.put(34, "Torss");
		county08.put(40, "Mrbylnga");
		county08.put(60, "Hultsfred");
		county08.put(61, "Mnsters");
		county08.put(62, "Emmaboda");
		county08.put(80, "Kalmar");
		county08.put(81, "Nybro");
		county08.put(82, "Oskarshamn");
		county08.put(83, "Vstervik");
		county08.put(84, "Vimmerby");
		county08.put(85, "Borgholm");
		countyMap.put(8, county08);

		// 9 Gotlands ln
		HashMap<Integer, String> county09 = new HashMap<Integer, String>();
		county09.put(80, "Gotland");
		countyMap.put(9, county09);

		// 21 Gvleborgs ln
		HashMap<Integer, String> county21 = new HashMap<Integer, String>();
		county21.put(01, "Ockelbo");
		county21.put(04, "Hofors");
		county21.put(21, "Ovanker");
		county21.put(32, "Nordanstig");
		county21.put(61, "Ljusdal");
		county21.put(80, "Gvle");
		county21.put(81, "Sandviken");
		county21.put(82, "Sderhamn");
		county21.put(83, "Bollns");
		county21.put(84, "Hudiksvall");
		countyMap.put(21, county21);

		// 22 Vsternorrlands ln
		HashMap<Integer, String> county22 = new HashMap<Integer, String>();
		county22.put(60, "nge");
		county22.put(62, "Timr");
		county22.put(80, "Hrnsand");
		county22.put(81, "Sundsvall");
		county22.put(82, "Kramfors");
		county22.put(83, "Sollefte");
		county22.put(84, "rnskldsvik");
		countyMap.put(22, county22);
	}

	public LMConnection(String basicAuthUsername, String basicAuthPassword, OAuthCredentialsProvider oAuthCredentialsProvider, int connectionTimeout, int requestTimeout, int maxResults, int minimumSearchStringLength) throws JAXBException {

		this.basicAuthUsername = basicAuthUsername;
		this.basicAuthPassword = basicAuthPassword;
		this.connectionTimeout = connectionTimeout;
		this.requestTimeout = requestTimeout;
		this.maxResults = maxResults;
		this.minimumSearchStringLength = minimumSearchStringLength;

		if (oAuthCredentialsProvider != null && !StringUtils.isEmpty(oAuthCredentialsProvider.getClientID()) && !StringUtils.isEmpty(oAuthCredentialsProvider.getClientSecret())) {

			this.lmTokenHandler = new LMTokenHandler(oAuthCredentialsProvider, connectionTimeout, requestTimeout);
			this.lmServiceURL = OAUTH_LMSERVICE_URL;

		} else {

			this.lmTokenHandler = null;
			this.lmServiceURL = BASICAUTH_LMSERVICE_URL;

		}

		GML_CONVERTER.setSwapAxes(true);

		addressContext = JAXBContext.newInstance(se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.ObjectFactory.class);
		estateContext = JAXBContext.newInstance(se.lantmateriet.namespace.distribution.produkter.fastighet.v2.ObjectFactory.class);
		designationContext = JAXBContext.newInstance(se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.ObjectFactory.class);
	}

	private SimpleRequest createGetRequest(String url) {

		SimpleRequest request = new SimpleRequest(url);
		request.setConnectionTimeout(connectionTimeout * MillisecondTimeUnits.SECOND);
		request.setReadTimeout(requestTimeout * MillisecondTimeUnits.SECOND);

		if (lmTokenHandler != null) {

			String accessToken = lmTokenHandler.getAccessToken();
			request.addHeader("Authorization", "Bearer " + accessToken);

		} else {

			request.setUsername(basicAuthUsername);
			request.setPassword(basicAuthPassword);

		}

		return request;
	}

	private SimpleRequest createPostRequest(String url) {

		SimpleRequest request = createGetRequest(url);
		request.addHeader("Content-Type", "application/xml");

		return request;
	}

	public List<BelagenhetsadressType> searchAddress(String searchStringWithMunicipalities) {

		String firstWord = searchStringWithMunicipalities.split(" ")[0];
		String[] municipalities = firstWord.split(",");

		return searchAddressInMunicipalities(municipalities, searchStringWithMunicipalities.substring(firstWord.length()));
	}

	public List<BelagenhetsadressType> searchAddressInMunicipalities(String[] municipalities, String searchString) {

		List<BelagenhetsadressType> responses = new ArrayList<BelagenhetsadressType>();

		for (String municipality : municipalities) {

			BelagenhetsadressResponseType addressContainers = searchAddressInMunicipality(municipality.trim(), searchString.trim());

			if (addressContainers != null) {

				for (BelagenhetsadressMemberType addressContainer : addressContainers.getBelagenhetsadressMember()) {

					responses.add(addressContainer.getBelagenhetsadress());
				}
			}
		}

		return responses;
	}

	private IOException parseRemoteError(HTTPException e) {

		try {
			String errorResponseXML = e.getMessage();
			FaultType error = (FaultType) addressContext.createUnmarshaller().unmarshal(new StringReader(errorResponseXML));

			return new IOException(error.toString(), e.getCause());

		} catch (Throwable t) {

			return e;
		}
	}

	public BelagenhetsadressResponseType searchAddressInMunicipality(String municipality, String searchString) {

		String address = municipality + " " + searchString;

		try {
			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/belagenhetsadress/v4.1/referens/fritext/" + URLEncoder.encode(address, "UTF-8").replaceAll("\\+", "%20"));
			request.addHeader("Accept", "application/xml");
			request.addParameter("maxHits", Integer.toString(maxResults));

			String xmlResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Unmarshaller jaxbUnmarshaller = addressContext.createUnmarshaller();
			@SuppressWarnings("unchecked")
			JAXBElement<BelagenhetsadressreferensResponseType> jaxbElement = (JAXBElement<BelagenhetsadressreferensResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			BelagenhetsadressreferensResponseType addressReferences = jaxbElement.getValue();

			if (CollectionUtils.isEmpty(addressReferences.getBelagenhetsadressreferens())) {
				return null;
			}

			request = createPostRequest(lmServiceURL + "/distribution/produkter/belagenhetsadress/v4.1/");
			request.addParameter("includeData", "total");

			IdRequestType idRequestType = new IdRequestType();

			for (BelagenhetsadressreferensType reference : addressReferences.getBelagenhetsadressreferens()) {

				if (reference.getBeteckning().toLowerCase().startsWith(municipality.toLowerCase())) {

					idRequestType.getIDs().add(reference.getObjektidentitet());
				}
			}

			JAXBElement<IdRequestType> requestWrapper = addressFactory.createIdRequest(idRequestType);

			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			addressContext.createMarshaller().marshal(requestWrapper, byteArrayOutputStream);

			xmlResponse = HTTPUtils.sendHTTPPostRequest(request, new ByteArrayInputStream(byteArrayOutputStream.toByteArray()), UTF8);

			@SuppressWarnings("unchecked")
			JAXBElement<BelagenhetsadressResponseType> jaxbElement2 = (JAXBElement<BelagenhetsadressResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			BelagenhetsadressResponseType addresses = jaxbElement2.getValue();
			return addresses;

		} catch (HTTPException e) {

			log.warn("Error searching for address " + address, parseRemoteError(e));

		} catch (Exception e) {

			log.warn("Error searching for address " + address, e);
		}

		return null;
	}

	public Point getAddressPointByObjectID(String objectID) {

		BelagenhetsadressType address = getAddressByObjectID(objectID);

		if (address != null) {

			List<Double> coordinates = address.getAdressplatsattribut().getAdressplatspunkt().getPoint().getPos().getValue();
			double x = coordinates.get(0);
			double y = coordinates.get(1);

			return GEOMETRY_FACTORY.createPoint(new Coordinate(x, y));
		}

		return null;
	}

	public BelagenhetsadressType getAddressByObjectID(String objectID) {

		if (StringUtils.isEmpty(objectID)) {
			throw new NullPointerException();
		}

		try {
			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/belagenhetsadress/v4.1/");
			request.addHeader("Accept", "application/xml");
			request.addParameter("id", objectID);
			request.addParameter("includeData", "total");

			String xmlResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Unmarshaller jaxbUnmarshaller = addressContext.createUnmarshaller();
			@SuppressWarnings("unchecked")
			JAXBElement<BelagenhetsadressResponseType> jaxbElement = (JAXBElement<BelagenhetsadressResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			BelagenhetsadressResponseType addresses = jaxbElement.getValue();

			if (CollectionUtils.isEmpty(addresses.getBelagenhetsadressMember())) {
				return null;
			}

			if (addresses.getBelagenhetsadressMember().size() > 1) {

				log.warn("Got multiple addresses for object id " + objectID);
			}

			return addresses.getBelagenhetsadressMember().get(0).getBelagenhetsadress();

		} catch (HTTPException e) {

			log.warn("Error getting address for objectID = \"" + objectID + "\"", parseRemoteError(e));

		} catch (Exception e) {

			log.warn("Error getting address for objectID = \"" + objectID + "\"", e);
		}

		return null;
	}

	public GMLConverter getGMLConverter() {

		return GML_CONVERTER;
	}

	public GeometryFactory getGeometryFactory() {

		return GEOMETRY_FACTORY;
	}

	public UnitArea searchUnitArea(double x, double y) {

		Point searchPoint = GEOMETRY_FACTORY.createPoint(new Coordinate(x, y));

		return searchUnitArea(searchPoint);
	}

	public UnitArea searchUnitArea(Point searchPoint) {

		if (searchPoint == null) {
			throw new NullPointerException();
		}

		RegisterenhetsreferensType registerenhetsreferens = searchRegisterUnit(searchPoint);

		if (registerenhetsreferens == null) {
			return null;
		}

		String objektidentitet = registerenhetsreferens.getObjektidentitet();

		RegisterenhetResponseType registerenhetResponse = getRegisterUnitByRegisterUnitID(objektidentitet);

		if (!CollectionUtils.isEmpty(registerenhetResponse.getRegisterenhetMember())) {

			RegisterenhetType registerenhet = registerenhetResponse.getRegisterenhetMember().get(0).getRegisterenhet().getValue();

			for (EnhetsomradeType enhetsomrade : registerenhet.getEnhetsomrade()) {

				if (enhetsomrade.getYta() != null) {

					for (JAXBElement<? extends AbstractGeometryType> element : enhetsomrade.getYta().getAbstractGeometry()) {

						Geometry geometry = GML_CONVERTER.toJtsGeometry(element.getValue());

						if (geometry.contains(searchPoint)) {
							return new UnitArea(registerenhetsreferens.getBeteckning(), registerenhetsreferens.getObjektidentitet(), registerenhetsreferens.getFastighetsnyckel(), geometry);
						}
					}

				}

			}
			
			for (EnhetsutrymmeType enhetsutrymme : registerenhet.getEnhetsutrymme()) {

				if(enhetsutrymme.getYta() != null) {
					
					for (JAXBElement<? extends AbstractGeometryType> element : enhetsutrymme.getYta().getAbstractGeometry()) {

						Geometry geometry = GML_CONVERTER.toJtsGeometry(element.getValue());

						if (geometry.contains(searchPoint)) {
							return new UnitArea(registerenhetsreferens.getBeteckning(), registerenhetsreferens.getObjektidentitet(), registerenhetsreferens.getFastighetsnyckel(), geometry);
						}
					}
					
				}
				
			}
		}

		return null;
	}

	public RegisterenhetsreferensType searchRegisterUnit(double x, double y) {

		Point searchPoint = GEOMETRY_FACTORY.createPoint(new Coordinate(x, y));

		return searchRegisterUnit(searchPoint);
	}

	public RegisterenhetsreferensType searchRegisterUnit(Point point) {

		if (point == null) {
			throw new NullPointerException();
		}

		try {
			SimpleRequest request = createPostRequest(lmServiceURL + "/distribution/produkter/fastighet/v2.1/referens/geometri");
			request.addHeader("Accept", "application/xml");
			request.addParameter("includeData", "basinformation,geometri");

			PointType pointType = (PointType) GML_CONVERTER.toVendorGeometry(point);

			GeometryPropertyType geometryProperty = new GeometryPropertyType();
			geometryProperty.setAbstractGeometry(objectFactory.createPoint(pointType));

			GeometriRequestType geometriRequestType = new GeometriRequestType();
			geometriRequestType.setGeometri(geometryProperty);

			JAXBElement<GeometriRequestType> requestWrapper = estateFactory.createGeometriRequest(geometriRequestType);

			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			estateContext.createMarshaller().marshal(requestWrapper, byteArrayOutputStream);

			String xmlResponse = HTTPUtils.sendHTTPPostRequest(request, new ByteArrayInputStream(byteArrayOutputStream.toByteArray()), UTF8);

			Unmarshaller jaxbUnmarshaller = estateContext.createUnmarshaller();

			@SuppressWarnings("unchecked")
			JAXBElement<RegisterenhetsreferensResponseType> jaxbElement = (JAXBElement<RegisterenhetsreferensResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			RegisterenhetsreferensResponseType registerUnitReferences = jaxbElement.getValue();

			if (CollectionUtils.isEmpty(registerUnitReferences.getRegisterenhetsreferens())) {
				return null;
			}

			return registerUnitReferences.getRegisterenhetsreferens().get(0);

		} catch (HTTPException e) {
			e.printStackTrace();
			log.warn("Error searching for register unit at point " + point, parseRemoteError(e));

		} catch (Exception e) {
			e.printStackTrace();
			log.warn("Error searching for register unit at point " + point, e);
		}

		return null;
	}

	public RegisterenhetResponseType getRegisterUnitByRegisterUnitID(String registerUnitID) {

		if (StringUtils.isEmpty(registerUnitID)) {
			throw new NullPointerException();
		}

		try {
			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/fastighet/v2.1/" + URLEncoder.encode(registerUnitID, "UTF-8").replaceAll("\\+", "%20"));
			request.addHeader("Accept", "application/xml");
			request.addParameter("includeData", "basinformation,geometri");

			String xmlResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Unmarshaller jaxbUnmarshaller = estateContext.createUnmarshaller();
			@SuppressWarnings("unchecked")
			JAXBElement<RegisterenhetResponseType> jaxbElement = (JAXBElement<RegisterenhetResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			RegisterenhetResponseType registerUnit = jaxbElement.getValue();

			return registerUnit;

		} catch (HTTPException e) {
			e.printStackTrace();
			log.warn("Error getting register unit by registerUnitID " + registerUnitID, parseRemoteError(e));

		} catch (Exception e) {
			e.printStackTrace();
			log.warn("Error getting register unit by registerUnitID " + registerUnitID, e);
		}

		return null;
	}

	public RegisterenhetResponseType getRegisterUnitByRegisterUnitIDs(List<String> registerUnitIDs) {

		if (CollectionUtils.isEmpty(registerUnitIDs)) {
			throw new NullPointerException();
		}

		try {

			SimpleRequest request = createPostRequest(lmServiceURL + "/distribution/produkter/fastighet/v2.1/");
			request.addHeader("Accept", "application/xml");
			request.addParameter("includeData", "basinformation,geometri");

			se.lantmateriet.namespace.distribution.produkter.fastighet.v2.IdRequestType idRequestType = new se.lantmateriet.namespace.distribution.produkter.fastighet.v2.IdRequestType();
			idRequestType.getId().addAll(registerUnitIDs);

			JAXBElement<se.lantmateriet.namespace.distribution.produkter.fastighet.v2.IdRequestType> requestWrapper = estateFactory.createIdRequest(idRequestType);

			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			estateContext.createMarshaller().marshal(requestWrapper, byteArrayOutputStream);

			String xmlResponse = HTTPUtils.sendHTTPPostRequest(request, new ByteArrayInputStream(byteArrayOutputStream.toByteArray()), UTF8);

			Unmarshaller jaxbUnmarshaller = estateContext.createUnmarshaller();

			@SuppressWarnings("unchecked")
			JAXBElement<RegisterenhetResponseType> jaxbElement = (JAXBElement<RegisterenhetResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			return jaxbElement.getValue();

		} catch (HTTPException e) {
			e.printStackTrace();
			log.warn("Error getting register unit by registerUnitIDs " + StringUtils.toCommaSeparatedString(registerUnitIDs), parseRemoteError(e));

		} catch (Exception e) {
			e.printStackTrace();
			log.warn("Error getting register unit by registerUnitIDs " + StringUtils.toCommaSeparatedString(registerUnitIDs), e);
		}

		return null;
	}

	public RegisterenhetResponseType getRegisterUnitByEstateID(String estateID) {

		if (StringUtils.isEmpty(estateID)) {
			throw new NullPointerException();
		}

		try {

			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/fastighet/v2.1/" + URLEncoder.encode(estateID, "UTF-8").replaceAll("\\+", "%20"));
			request.addHeader("Accept", "application/xml");
			request.addParameter("includeData", "basinformation,geometri");

			String xmlResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Unmarshaller jaxbUnmarshaller = estateContext.createUnmarshaller();
			@SuppressWarnings("unchecked")
			JAXBElement<RegisterenhetResponseType> jaxbElement = (JAXBElement<RegisterenhetResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			RegisterenhetResponseType registerUnit = jaxbElement.getValue();

			return registerUnit;

		} catch (HTTPException e) {
			e.printStackTrace();
			log.warn("Error getting register unit by estateID " + estateID, parseRemoteError(e));

		} catch (Exception e) {
			e.printStackTrace();
			log.warn("Error getting register unit by estateID " + estateID, e);
		}

		return null;
	}

	public List<PlaceName> searchForPlaceName(String municipalityAndCountyCode, String searchString) {

		if (searchString.length() < minimumSearchStringLength) {
			return null;
		}

		if (municipalityAndCountyCode != null) {

			List<PlaceName> features = new ArrayList<PlaceName>();

			String[] municipalityAndCountyCodes = municipalityAndCountyCode.split(",");

			for (String code : municipalityAndCountyCodes) {

				List<PlaceName> places = searchForPlaceName(Integer.parseInt(code.substring(0, 2)), Integer.parseInt(code.substring(2)), searchString);

				if (places != null) {
					features.addAll(places);
				}
			}

			return features;

		} else {

			return searchForPlaceName(null, null, searchString);
		}
	}

	private List<PlaceName> searchForPlaceName(Integer countyCode, Integer municipalityCode, String searchString) {

		if (StringUtils.isEmpty(searchString)) {
			throw new NullPointerException();
		}

		try {
			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/ortnamn/v2.1/kriterier");
			request.addParameter("namn", searchString);
			request.addParameter("match", "contains");
			request.addParameter("sprak", "Svenska");
			request.addParameter("namntyp", "Bebyggelse");
			request.addParameter("namntyp", URLEncoder.encode("Ttort", "UTF-8"));
			request.addParameter("maxHits", Math.min(maxResults, 400));
			request.addParameter("srid", "3006");

			if (countyCode != null) {
				if (municipalityCode != null) {
					request.addParameter("kommunkod", String.format("%02d%02d", countyCode, municipalityCode));

				} else {
					request.addParameter("lankod", String.format("%02d", countyCode));
				}
			}

			request.addHeader("Accept", "application/xml");

			String xmlResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			XMLParser xmlParser = new XMLParser(XMLUtils.parseXML(xmlResponse, "UTF-8", false, false));

			OrtnamnResponse response = new OrtnamnResponse();
			response.populate(xmlParser);

			List<PlaceName> places = new ArrayList<PlaceName>();

			if (response.getOrtnamn() != null) {
				for (Ortnamn ortnamn : response.getOrtnamn()) {

					Long placeNameID = ortnamn.getID();
					String name = ortnamn.getOrtnamn();
					OrtnamnsPlacering ortnamnsplacering = ortnamn.getPlacering();

					List<Double> coordinates = ortnamnsplacering.getPoint().getPos().getValue();
					double x = coordinates.get(0);
					double y = coordinates.get(1);

					Point point = new GeometryFactory().createPoint(new Coordinate(x, y));

					String municipality = null;

					if (countyCode != null && municipalityCode != null) {

						Map<Integer, String> county = countyMap.get(countyCode);

						if (county != null) {
							municipality = county.get(municipalityCode);
						}

						if (municipality == null) {
							log.warn("Found no municipality name for countyCode " + countyCode + " and municipalityCode " + municipalityCode + ". Go and update countyMap");
						}
					}

					places.add(new PlaceName(placeNameID, name, municipality, point));
				}
			}

			return places;

		} catch (HTTPException e) {
			e.printStackTrace();
			log.warn("Error searching for place name with countyCode " + countyCode + " and municipalityCode " + municipalityCode + " searchString " + searchString, parseRemoteError(e));

		} catch (Exception e) {
			e.printStackTrace();
			log.warn("Error searching for place name with countyCode " + countyCode + " and municipalityCode " + municipalityCode + " searchString " + searchString, e);
		}

		return null;
	}

	public List<UnitArea> getUnitAreaByEstateID(String estateID) {

		RegisterenhetResponseType registerenhetResponse = getRegisterUnitByEstateID(estateID);
		return getUnitAreas(registerenhetResponse, false, true);
	}

	public List<UnitArea> getUnitAreasByEstateID(String estateID) {

		RegisterenhetResponseType registerenhetResponse = getRegisterUnitByEstateID(estateID);
		return getUnitAreas(registerenhetResponse, true, false);
	}

	public List<UnitArea> getUnitAreasByRegisterUnitID(String registerUnitID) {

		return getUnitAreasByRegisterUnitID(registerUnitID, true);
	}

	public List<UnitArea> getUnitAreasByRegisterUnitID(String registerUnitID, boolean includeEnhetsomrade) {

		RegisterenhetResponseType registerenhetResponse = getRegisterUnitByRegisterUnitID(registerUnitID);
		return getUnitAreas(registerenhetResponse, includeEnhetsomrade, false);
	}

	public List<UnitArea> getUnitAreasByRegisterUnitIDs(List<String> registerUnitIDs) {

		RegisterenhetResponseType registerenhetResponse = getRegisterUnitByRegisterUnitIDs(registerUnitIDs);
		return getUnitAreas(registerenhetResponse, true, false, false);
	}

	public List<UnitArea> getUnitAreas(String estateIDorRegisterUnitID) {

		RegisterenhetResponseType registerenhetResponse;

		if (estateIDorRegisterUnitID.length() == 36) {

			registerenhetResponse = getRegisterUnitByRegisterUnitID(estateIDorRegisterUnitID);

		} else {

			registerenhetResponse = getRegisterUnitByEstateID(estateIDorRegisterUnitID);
		}

		if (registerenhetResponse == null) {
			return null;
		}

		return getUnitAreas(registerenhetResponse, true, false);
	}

	private String calcBeteckning(RegisterenhetType r) {

		RegisterbeteckningType b = r.getRegisterbeteckning();
		return b.getRegisteromrade() + " " + b.getTraktnamn() + " " + ("*".equals(b.getBlock()) ? "" : (b.getBlock() + ":")) + b.getEnhet();
	}

	private List<UnitArea> getUnitAreas(RegisterenhetResponseType registerenhetResponse, boolean includeEnhetsomrade, boolean singleEstate) {

		return getUnitAreas(registerenhetResponse, includeEnhetsomrade, singleEstate, true);
	}

	private List<UnitArea> getUnitAreas(RegisterenhetResponseType registerenhetResponse, boolean includeEnhetsomrade, boolean singleEstate, boolean singleSamfallighet) {

		if (registerenhetResponse == null) {

			return null;
		}

		List<RegisterenhetMemberType> registerenhetMemberTypes = registerenhetResponse.getRegisterenhetMember();

		if (CollectionUtils.isEmpty(registerenhetMemberTypes)) {
			return null;
		}

		List<UnitArea> unitAreas = new ArrayList<UnitArea>();

		for (RegisterenhetMemberType registerenhetMemberType : registerenhetResponse.getRegisterenhetMember()) {

			RegisterenhetType registerenhet = registerenhetMemberType.getRegisterenhet().getValue();

			for (EnhetsomradeType enhetsomrade : registerenhet.getEnhetsomrade()) {

				UnitArea unitArea = createUnitArea(registerenhet, enhetsomrade.getYta(), enhetsomrade.getOmradesnummer(), includeEnhetsomrade);

				if (unitArea != null) {

					unitAreas.add(unitArea);
					
					if (singleEstate) {
						break;
					}
				}
				
			}

			for (EnhetsutrymmeType enhetsutrymme : registerenhet.getEnhetsutrymme()) {

				UnitArea unitArea = createUnitArea(registerenhet, enhetsutrymme.getYta(), enhetsutrymme.getOmradesnummer(), includeEnhetsomrade);

				if (unitArea != null) {

					unitAreas.add(unitArea);
					
					if (singleEstate) {
						break;
					}
				}

			}

			if (singleSamfallighet) {
				break;
			}
		}

		return unitAreas;
	}

	private UnitArea createUnitArea(RegisterenhetType registerenhet, GeometryArrayPropertyType yta, int omradesNummer, boolean includeEnhetsomrade) {

		if (yta != null && yta.getAbstractGeometry() != null) {

			for (JAXBElement<? extends AbstractGeometryType> element : yta.getAbstractGeometry()) {

				String beteckning = calcBeteckning(registerenhet);

				if (includeEnhetsomrade) {
					beteckning += " Enhetsomrde " + omradesNummer;
				}

				Geometry geometry = GML_CONVERTER.toJtsGeometry(element.getValue());

				return new UnitArea(beteckning, registerenhet.getObjektidentitet(), registerenhet.getFastighetsnyckel(), geometry);

			}
		}

		return null;
	}

	private List<RegisterbeteckningsreferensType> searchRegisterDesignation(String searchString) {

		if (searchString == null || searchString.length() < minimumSearchStringLength) {
			
			return null;
		}

		String firstWord = searchString.split(" ")[0];
		String[] registerAreas = firstWord.split(",");

		List<RegisterbeteckningsreferensType> responses = new ArrayList<RegisterbeteckningsreferensType>();

		for (String registerArea : registerAreas) {

			RegisterbeteckningsreferensResponseType registerDesignationReferences = searchRegisterDesignationInArea(registerArea + searchString.substring(firstWord.length()));

			if (registerDesignationReferences != null) {

				for (RegisterbeteckningsreferensType registerDesignationReference : registerDesignationReferences.getRegisterbeteckningsreferens()) {

					responses.add(registerDesignationReference);
				}
			}
		}

		return responses;
	}

	private RegisterbeteckningsreferensResponseType searchRegisterDesignationInArea(String searchString) {

		if (StringUtils.isEmpty(searchString)) {
			throw new NullPointerException();
		}

		if (searchString.length() > 120) {

			log.warn("Too long search string truncating to 120: " + searchString);
			searchString = searchString.substring(0, 120);
		}

		try {
			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/registerbeteckning/v4/referens/fritext/" + URLEncoder.encode(searchString, "UTF-8").replaceAll("\\+", "%20"));
			request.addHeader("Accept", "application/xml");
			request.addParameter("maxHits", Integer.toString(Math.min(maxResults * 2, 250))); // Get a few extra as some may be filtered away later
			request.addParameter("status", URLEncoder.encode("Gllande", "UTF-8"));

			String xmlResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Unmarshaller jaxbUnmarshaller = designationContext.createUnmarshaller();
			@SuppressWarnings("unchecked")
			JAXBElement<RegisterbeteckningsreferensResponseType> jaxbElement = (JAXBElement<RegisterbeteckningsreferensResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			RegisterbeteckningsreferensResponseType registerDesignationReferences = jaxbElement.getValue();

			if (CollectionUtils.isEmpty(registerDesignationReferences.getRegisterbeteckningsreferens())) {
				return null;
			}

			return registerDesignationReferences;

		} catch (HTTPException e) {

			log.warn("Error searching for register designation with searchString " + searchString, parseRemoteError(e));

		} catch (Exception e) {

			log.warn("Error searching for register designation with searchString " + searchString, e);
		}

		return null;
	}

	public RegisterbeteckningResponseType getRegisterUnitDesignations(List<String> beteckningsIDs) {

		if (CollectionUtils.isEmpty(beteckningsIDs)) {
			throw new NullPointerException();
		}

		try {
			SimpleRequest request = createPostRequest(lmServiceURL + "/distribution/produkter/registerbeteckning/v4/");

			se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.IdRequestType idRequestType = new se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.IdRequestType();
			idRequestType.getIDs().addAll(beteckningsIDs);

			JAXBElement<se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.IdRequestType> requestWrapper = designationFactory.createIdRequest(idRequestType);

			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			designationContext.createMarshaller().marshal(requestWrapper, byteArrayOutputStream);

			String xmlResponse = HTTPUtils.sendHTTPPostRequest(request, new ByteArrayInputStream(byteArrayOutputStream.toByteArray()), UTF8);

			Unmarshaller jaxbUnmarshaller = designationContext.createUnmarshaller();

			@SuppressWarnings("unchecked")
			JAXBElement<RegisterbeteckningResponseType> jaxbElement2 = (JAXBElement<RegisterbeteckningResponseType>) jaxbUnmarshaller.unmarshal(new StringReader(xmlResponse));

			RegisterbeteckningResponseType registerDesignations = jaxbElement2.getValue();
			return registerDesignations;

		} catch (HTTPException e) {

			log.warn("Error getting register unit designations", parseRemoteError(e));

		} catch (Exception e) {

			log.warn("Error getting register unit designations", e);
		}

		return null;
	}

	public List<RegisterUnit> searchRegisterUnit(String searchString) {

		List<RegisterbeteckningsreferensType> registerDesignationReferences = searchRegisterDesignation(searchString);

		if (registerDesignationReferences != null) {

			List<String> beteckningsIDs = new ArrayList<String>();

			HashMap<String, String> nameMap = new HashMap<String, String>();

			for (RegisterbeteckningsreferensType registerDesginationReference : registerDesignationReferences) {

				beteckningsIDs.add(registerDesginationReference.getBeteckningsid());
				nameMap.put(registerDesginationReference.getRegisterenhet(), registerDesginationReference.getBeteckning());
			}

			List<RegisterUnit> registerUnits = new ArrayList<RegisterUnit>();

			if (beteckningsIDs.size() > 0) {

				RegisterbeteckningResponseType registerDesignations = getRegisterUnitDesignations(beteckningsIDs);

				if (registerDesignations == null) {
					return null;
				}

				List<RegisterbeteckningMemberType> registerDesignationMembers = registerDesignations.getRegisterbeteckningMember();

				for (RegisterbeteckningMemberType registerDesignationMember : registerDesignationMembers) {

					BeteckningType designation = registerDesignationMember.getBeteckning();
					se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterenhetsreferensType registerDesignationReference = designation.getRegisterenhetsreferens();
					se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterbeteckningType registerDesignation = designation.getRegisterbeteckning();

					if (registerDesignationReference.getObjektstatus() == se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.RegisterenhetsstatusType.AVREGISTRERAD) {
						continue;
					}

					List<Point> points = new ArrayList<Point>();

					for (se.lantmateriet.namespace.distribution.produkter.registerbeteckning.v4.EnhetsomradeType unitArea : registerDesignationReference.getEnhetsomrade()) {

						PointPropertyType pointPropertyType = unitArea.getCentralpunkt();

						if (pointPropertyType == null) {
							continue;
						}

						Point point = (Point) GML_CONVERTER.toJtsGeometry(pointPropertyType.getPoint());
						points.add(point);
					}

					GeometryFactory factory = new GeometryFactory();
					MultiPoint multiPoint = factory.createMultiPoint(points.toArray(new Point[] {}));

					registerUnits.add(new RegisterUnit(nameMap.get(registerDesignationReference.getObjektidentitet()), registerDesignation.getBeteckningsid(), registerDesignationReference.getObjektidentitet(), multiPoint));

					if (registerUnits.size() >= maxResults) {
						break;
					}
				}
			}

			return registerUnits;
		}

		return null;
	}

	public Geometry getRegisterUnitGeometry(String objectID) {

		RegisterenhetResponseType registerenhetResponse = getRegisterUnitByRegisterUnitID(objectID);

		if (registerenhetResponse != null) {

			RegisterenhetType registerenhet = registerenhetResponse.getRegisterenhetMember().get(0).getRegisterenhet().getValue();

			List<Geometry> geometries = new ArrayList<Geometry>();

			for (EnhetsomradeType enhetsomrade : registerenhet.getEnhetsomrade()) {

				if (enhetsomrade.getYta() != null) {

					for (JAXBElement<? extends AbstractGeometryType> element : enhetsomrade.getYta().getAbstractGeometry()) {

						geometries.add(GML_CONVERTER.toJtsGeometry(element.getValue()));
					}
				}

			}

			return UnaryUnionOp.union(geometries);
		}

		return null;
	}

	protected static Integer getFirstDigit(String address) {

		int index = 0;

		while (index < address.length()) {

			if (Character.isDigit(address.charAt(index))) {

				return index;
			}

			index++;
		}

		return null;
	}

	public HeightResponse getElevation(Double x, Double y) {

		try {
			SimpleRequest request = createGetRequest(lmServiceURL + "/distribution/produkter/hojd/v1/rest/api/hojd/3006/" + x + "/" + y);
			request.addHeader("Accept", "application/json");

			String jsonResponse = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			JSONDeserializer<HeightResponse> deserializer = new JSONDeserializer<HeightResponse>();
			deserializer.use(Geometry.class, new PointTypeJsonFactory());

			HeightResponse response = deserializer.deserialize(jsonResponse, HeightResponse.class);

			return response;

		} catch (Exception e) {

			log.warn("Error getting height for x: " + x + ", y: " + y, e);
		}

		return null;
	}

}
