package com.nordicpeak.lmsearch.beans;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.AdressomradeType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.AdressplatsbeteckningType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressType;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.string.StringUtils;

public class Addresses extends BaseLMResponse {

	private final List<BelagenhetsadressType> addresses;

	public Addresses(List<BelagenhetsadressType> addresses) {

		super();

		this.addresses = addresses;
	}

	public List<BelagenhetsadressType> getAddresses() {

		return addresses;
	}

	@Override
	public JsonNode toJson() {

		List<Address> generatedAdresses = generateAddresses();

		JsonArray rows = new JsonArray(addresses.size());

		for (Address address : generatedAdresses) {

			JsonArray row = new JsonArray();

			row.addNode(address.getObjectIdentity());
			row.addNode(address.getStreetAddress());
			row.addNode(address.getY());
			row.addNode(address.getX());
			row.addNode(address.getReferenceObjectIdentity());
			row.addNode(address.getStreet());
			row.addNode(address.getPostalAddress());
			row.addNode(address.getStreetNumber());

			rows.addNode(row);
		}

		return rows;
	}
	
	public List<Address> generateAddresses() {

		List<Address> generatedAdresses = new ArrayList<Address>(addresses.size());

		for (BelagenhetsadressType addressType : addresses) {

			Address address = new Address();

			address.setObjectIdentity(addressType.getObjektidentitet());

			AdressomradeType addressOmrade = addressType.getAdressomrade();

			if (addressOmrade == null) {

				addressOmrade = addressType.getGardsadressomrade().getAdressomrade();
			}

			StringBuilder streetAddress = new StringBuilder();

			streetAddress.append(addressOmrade.getKommundel().getKommun().getKommunnamn());
			streetAddress.append(" ");
			streetAddress.append(addressOmrade.getFaststalltNamn());

			address.setStreet(addressOmrade.getFaststalltNamn());

			AdressplatsbeteckningType adressplatsbeteckning = addressType.getAdressplatsattribut().getAdressplatsbeteckning();

			if (adressplatsbeteckning.isAvvikerFranStandarden()) {

				if (!StringUtils.isEmpty(adressplatsbeteckning.getAvvikandeAdressplatsbeteckning())) {

					streetAddress.append(" ");
					streetAddress.append(adressplatsbeteckning.getAvvikandeAdressplatsbeteckning());
				}

			} else {

				if (!StringUtils.isEmpty(adressplatsbeteckning.getAdressplatsnummer()) || !StringUtils.isEmpty(adressplatsbeteckning.getBokstavstillagg())) {

					streetAddress.append(" ");

					if (!StringUtils.isEmpty(adressplatsbeteckning.getAdressplatsnummer())) {

						address.setStreetNumber(adressplatsbeteckning.getAdressplatsnummer());

						streetAddress.append(adressplatsbeteckning.getAdressplatsnummer());
					}

					if (!StringUtils.isEmpty(adressplatsbeteckning.getBokstavstillagg())) {

						streetAddress.append(adressplatsbeteckning.getBokstavstillagg());
					}

				}

				if (adressplatsbeteckning.getLagestillagg() != null) {

					streetAddress.append(" ");
					streetAddress.append(adressplatsbeteckning.getLagestillagg().value());

					if (adressplatsbeteckning.getLagestillaggsnummer() != null) {

						streetAddress.append(adressplatsbeteckning.getLagestillaggsnummer());
					}
				}
			}

			if (addressType.getAdressplatsattribut().getPostort() != null) {
				streetAddress.append(", ");
				streetAddress.append(addressType.getAdressplatsattribut().getPostort());

				address.setPostalAddress(addressType.getAdressplatsattribut().getPostort());
			}

			address.setStreetAddress(streetAddress.toString());

			List<Double> coordinates = addressType.getAdressplatsattribut().getAdressplatspunkt().getPoint().getPos().getValue();
			double x = coordinates.get(0);
			double y = coordinates.get(1);

			address.setX(x);
			address.setY(y);

			if (!CollectionUtils.isEmpty(addressType.getRegisterenhetsreferens())) {

				address.setReferenceObjectIdentity(addressType.getRegisterenhetsreferens().get(0).getObjektidentitet());

				if (addressType.getRegisterenhetsreferens().size() > 1) {
					log.warn("Multiple registerUnits linked to address " + streetAddress.toString());
				}

			} else {

				log.warn("No registerUnits linked to address " + streetAddress.toString());
				address.setReferenceObjectIdentity("null");
			}

			generatedAdresses.add(address);

		}

		Collections.sort(generatedAdresses);
		
		return generatedAdresses;
	}

}
