package com.nordicpeak.flowengine.queries.listsearchqueryprovider;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.alternativelist.AlternativeListHandlerModule;
import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListProvider;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQueryProvider;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.checkboxquery.validationerrors.TooFewAlternativesSelectedValidationError;
import com.nordicpeak.flowengine.queries.checkboxquery.validationerrors.TooManyAlternativesSelectedValidationError;
import com.nordicpeak.flowengine.queries.listsearchqueryprovider.beans.ListSearchAlternative;
import com.nordicpeak.flowengine.queries.listsearchqueryprovider.beans.ListSearchQuery;
import com.nordicpeak.flowengine.queries.listsearchqueryprovider.beans.ListSearchQueryInstance;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;


public class ListSearchQueryProviderModule extends BaseQueryProviderModule<ListSearchQueryInstance> implements BaseQueryCRUDCallback, AlternativeListQueryProvider {

	private static final RelationQuery SAVE_QUERY_INSTANCE_RELATION_QUERY = new RelationQuery(ListSearchQueryInstance.ALTERNATIVES_RELATION);

	@XSLVariable(prefix = "java.")
	private String missingList = "Missing";
	
	@XSLVariable(prefix = "java.")
	private String showingPartialList1 = "Showing";
	
	@XSLVariable(prefix = "java.")
	private String showingPartialList2 = "of";
	
	@XSLVariable(prefix = "java.")
	private String showingPartialList3 = "alternatives";
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default max search hits", description = "The default value for max number of search hits returned if no other value is set on the queries.", required = true, formatValidator = PositiveStringIntegerPopulator.class)
	private int maxHits = 20;

	private AnnotatedDAO<ListSearchQuery> queryDAO;
	private AnnotatedDAO<ListSearchQueryInstance> queryInstanceDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;

	private ListSearchQueryCRUD queryCRUD;
	
	private QueryParameterFactory<ListSearchQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<ListSearchQuery, String> listNameParamFactory;
	private QueryParameterFactory<ListSearchQueryInstance, Integer> queryInstanceIDParamFactory;

	@InstanceManagerDependency(required = true)
	private AlternativeListHandlerModule alternativeListHandlerModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ListSearchQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);
		
		queryDAO = daoFactory.getDAO(ListSearchQuery.class);
		queryInstanceDAO = daoFactory.getDAO(ListSearchQueryInstance.class);
		
		queryCRUD = new ListSearchQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<ListSearchQuery>(ListSearchQuery.class), "ListSearchQuery", "query", null, this);
		
		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		listNameParamFactory = queryDAO.getParamFactory("alternativeListName", String.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		
		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());
		
		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}
	
	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ListSearchQuery query = new ListSearchQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, null);
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}

	private ListSearchQuery getQuery(Integer queryID) throws SQLException {
		
		HighLevelQuery<ListSearchQuery> getQuery = new HighLevelQuery<ListSearchQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		ListSearchQuery query = queryDAO.get(getQuery);
		
		if (query != null) {
			query.setAlternativeList(getAlternativeListInstance(query, null, null));
		}
		
		return query;
	}
	
	private ListSearchQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {
		
		HighLevelQuery<ListSearchQuery> getQuery = new HighLevelQuery<ListSearchQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		ListSearchQuery query = queryDAO.get(getQuery, transactionHandler);
		
		if (query != null) {
			query.setAlternativeList(getAlternativeListInstance(query, null, null));
		}
		
		return query;
	}


	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws Throwable {

		ListSearchQueryInstance queryInstance = null;
		
		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance = new ListSearchQueryInstance();
			
		} else {
			
			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());
			
			if (queryInstance == null) {
				
				return null;
			}
		}
		
		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));
		
		if (queryInstance.getQuery() == null) {
			
			return null;
		}
		
		if (req != null) {
			
			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);
			
			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());
		
		queryInstance.getQuery().scanAttributeTags();
		
		queryInstance.set(descriptor);
		
		return queryInstance;
	}

	private ListSearchQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {
		
		HighLevelQuery<ListSearchQueryInstance> query = new HighLevelQuery<ListSearchQueryInstance>(ListSearchQueryInstance.ALTERNATIVES_RELATION, ListSearchQueryInstance.QUERY_RELATION);
		
		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));
		
		return queryInstanceDAO.get(query);
	}
	
	@Override
	public QueryResponse getFormHTML(ListSearchQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if (queryInstance.getQuery().getAlternativeList() == null || queryInstance.getQuery().getAlternativeList().requiresReload(attributeHandler)) {
			
			queryInstance.getQuery().setAlternativeList(getAlternativeListInstance(queryInstance.getQuery(), attributeHandler, poster));
		}
		
		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ListSearchQuery query = getQuery(descriptor.getQueryID());
		
		if (query == null) {
			
			return false;
		}
		
		this.queryDAO.delete(query, transactionHandler);
		
		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ListSearchQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());
		
		if (queryInstance == null) {
			
			return false;
		}
		
		this.queryInstanceDAO.delete(queryInstance, transactionHandler);
		
		return true;
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		ListSearchQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);
		
		query.setQueryID(copyQueryDescriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, new RelationQuery());
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void save(ListSearchQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		//Check if the query instance has an ID set and if the ID of the descriptor has changed
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {
			
			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());
			
			this.queryInstanceDAO.add(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
			
		} else {
			
			this.queryInstanceDAO.update(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
		}
	}

	@Override
	public void populate(ListSearchQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		List<? extends ImmutableAlternative> availableAlternatives = null;

		if (queryInstance.getQuery().getAlternativeList() != null) {

			availableAlternatives = queryInstance.getQuery().getAlternativeList().getAlternatives();
		}

		if (CollectionUtils.isEmpty(availableAlternatives)) { // If the parent query doesn't have any alternatives then there is no population to do
			
			if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && !allowPartialPopulation) {
				
				throw new ValidationException(new ValidationError("RequiredQuery"));
				
			} else {
			
				queryInstance.reset(attributeHandler);
				return;
			}
		}		
		
		Integer queryID = queryInstance.getQuery().getQueryID();
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		List<Integer> alternativeIDs = ValidationUtils.validateParameters("q" + queryID + "_alternative", req, false, IntegerPopulator.getPopulator(), validationErrors);
		
		List<ListSearchAlternative> selectedAlternatives = new ArrayList<ListSearchAlternative>(availableAlternatives.size());

		if (alternativeIDs != null) {
			
			for (ImmutableAlternative alternative : availableAlternatives) {

				if (alternativeIDs.contains(alternative.getAlternativeID())) {

					selectedAlternatives.add(new ListSearchAlternative(alternative.getAlternativeID(), alternative.getName(), alternative.getExportXMLValue(), alternative.getAttributeValue()));
				}
			}
			
			if (selectedAlternatives.size() < alternativeIDs.size()) {
				validationErrors.add(new ValidationError("InvalidAlternative"));
			}
		}

		int alternativesSelected = selectedAlternatives.size();
		
		//If partial population is allowed and the user has not selected any alternatives, skip validation
		if (allowPartialPopulation && alternativesSelected == 0) {

			queryInstance.reset(attributeHandler, false);
			return;
		}

		//Check if this query is required or if the user has selected any alternatives anyway
		if ((!allowPartialPopulation && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED) || alternativesSelected > 0) {

			if (queryInstance.getQuery().getMinChecked() != null && alternativesSelected < queryInstance.getQuery().getMinChecked()) {

				validationErrors.add(new TooFewAlternativesSelectedValidationError(alternativesSelected, queryInstance.getQuery().getMinChecked()));

			} else if (queryInstance.getQuery().getMaxChecked() != null && alternativesSelected > queryInstance.getQuery().getMaxChecked()) {

				validationErrors.add(new TooManyAlternativesSelectedValidationError(alternativesSelected, queryInstance.getQuery().getMaxChecked()));

			} else if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && alternativesSelected == 0) {

				validationErrors.add(new ValidationError("RequiredQuery"));
			}
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		queryInstance.setAlternatives(selectedAlternatives);
		queryInstance.getQueryInstanceDescriptor().setPopulated(!selectedAlternatives.isEmpty());

		if (queryInstance.getQuery().isSetAsAttribute()) {

			queryInstance.resetAttribute(attributeHandler);
			queryInstance.setAttribute(attributeHandler);
		}
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(ListSearchQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws Exception {

		return new ListSearchQueryRequestProcessor(queryInstance, getMaxHits(queryInstance.getQuery()), showingPartialList1, showingPartialList2, showingPartialList3);
	}
	
	private int getMaxHits(ListSearchQuery query) {

		if(query.getMaxHits() != null) {
			
			return query.getMaxHits();
		}
		
		return maxHits;
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.queryCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	protected Class<ListSearchQueryInstance> getQueryInstanceClass() {
		
		return ListSearchQueryInstance.class;
	}

	public String getMissingAlternativeList() {
		
		return missingList;
	}
	
	protected Collection<AlternativeListDescriptor> getAlternativeLists() {
		
		return alternativeListHandlerModule.getAlternativeLists();
	}
	
	protected AlternativeListProvider getAlternativeListProvider(String providerID) {
		
		return alternativeListHandlerModule.getProvider(providerID);
	}
	
	protected AlternativeListDescriptor getAlternativeListDescriptor(AlternativeListQuery query) {
		
		if (!StringUtils.isEmpty(query.getAlternativeListName())) {
			
			return alternativeListHandlerModule.getAlternativeList(query.getAlternativeListName());
		}
		
		return null;
	}
	
	protected AlternativeListInstance getAlternativeListInstance(AlternativeListQuery query, AttributeHandler attributeHandler, User poster) {
		
		AlternativeListDescriptor descriptor = getAlternativeListDescriptor(query);
		
		if (descriptor != null) {
			
			return alternativeListHandlerModule.getListInstance(descriptor, attributeHandler, poster);
		}
		
		return null;
	}
	
	@Override
	public Collection<? extends AlternativeListQuery> getQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) {
		
		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<Integer>(queryDAO.getDataSource(), "SELECT queryID FROM " + queryDAO.getTableName() + " WHERE alternativeListName = ?", IntegerPopulator.getPopulator());
			idQuery.setString(1, descriptor.getName());
			
			List<Integer> queryIDs = idQuery.executeQuery();
			
			if (!CollectionUtils.isEmpty(queryIDs)) {
				
				List<ListSearchQuery> queries = new ArrayList<ListSearchQuery>(queryIDs.size());
				
				for (Integer queryID : queryIDs) {
					
					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(queryID);
					
					if (queryDescriptor != null) {
						
						queries.add((ListSearchQuery) getQuery(queryDescriptor, false));
					}
				}
				
				return queries;
			}
			
		} catch (SQLException e) {
			
			log.error("Unable to get queries using alternative list " + descriptor, e);
		}
		
		return null;
	}
	
	@Override
	public boolean hasQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) throws SQLException {

		HighLevelQuery<ListSearchQuery> query = new HighLevelQuery<>();
		query.addParameter(listNameParamFactory.getParameter(descriptor.getName()));
		query.setRowLimiter(MySQLRowLimiter.SINGLE_ROW);
		return queryDAO.getBoolean(query);
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		ListSearchQuery query = this.getQuery(descriptor.getQueryID());
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ListSearchQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, ListSearchQueryInstance queryInstance, AttributeHandler attributeHandler) {
		
		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);
		
		if (queryInstance.getQuery().getDescription() != null) {
			
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}
	
	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {
		
		ListSearchQuery query = new ListSearchQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);
		
		this.queryDAO.add(query, transactionHandler, null);
		
		return query;
	}
	
}
