package com.nordicpeak.flowengine.queries.listsearchqueryprovider;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.listsearchqueryprovider.beans.ListSearchQueryInstance;

public class ListSearchQueryRequestProcessor implements QueryRequestProcessor {

	private List<? extends ImmutableAlternative> alternatives;

	private final int maxHits;
	private final boolean showAlternativesOnEmptySearch;
	private final boolean showExactMatchesFirst;
	private final String showingPartialList1, showingPartialList2, showingPartialList3;

	public ListSearchQueryRequestProcessor(ListSearchQueryInstance queryInstance, int maxHits, String showingPartialList1, String showingPartialList2, String showingPartialList3) {

		super();

		this.alternatives = queryInstance.getQuery().getAlternatives();
		this.maxHits = maxHits;
		this.showAlternativesOnEmptySearch = queryInstance.getQuery().isShowAlternativesOnEmptySearch();
		this.showExactMatchesFirst = queryInstance.getQuery().isShowExactMatchesFirst();

		if (showAlternativesOnEmptySearch) {

			this.showingPartialList1 = showingPartialList1;
			this.showingPartialList2 = showingPartialList2;
			this.showingPartialList3 = showingPartialList3;

		} else {

			this.showingPartialList1 = null;
			this.showingPartialList2 = null;
			this.showingPartialList3 = null;
		}
	}

	@Override
	public void processRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		String searchString = req.getParameter("q");

		if (StringUtils.isEmpty(searchString)) {

			if (showAlternativesOnEmptySearch && alternatives != null) {

				JsonArray hitList = new JsonArray(maxHits + 1);

				for (ImmutableAlternative alternative : alternatives) {

					JsonObject jsonAlternative = new JsonObject(2);

					jsonAlternative.putField("ID", alternative.getAlternativeID());
					jsonAlternative.putField("Name", alternative.getName());
					hitList.addNode(jsonAlternative);

					if (hitList.size() == maxHits) {
						break;
					}
				}

				if (alternatives.size() > maxHits) {

					JsonObject jsonAlternative = new JsonObject(2);

					jsonAlternative.putField("ID", "null");
					jsonAlternative.putField("Name", showingPartialList1 + " " + hitList.size() + " " + showingPartialList2 + " " + alternatives.size() + " " + showingPartialList3);
					hitList.addNode(jsonAlternative);
				}

				sendJSONResponse(hitList, res);
				return;
			}

			sendEmptyJSONResponse(res);
			return;
		}

		searchString = URLDecoder.decode(searchString, "UTF-8");

		searchString = searchString.toLowerCase();

		JsonArray hitList = new JsonArray(maxHits);

		if (alternatives != null) {

			List<Integer> addedSearchValues = new ArrayList<>();
			if(showExactMatchesFirst) {
				addMatchingSearchMatches(searchString, hitList, addedSearchValues);
			}

			addContainingSearchMatches(searchString, hitList, addedSearchValues);
		}

		sendJSONResponse(hitList, res);
	}

	private void addMatchingSearchMatches(String searchString, JsonArray hitList, List<Integer> addedAlternatives) {

		for (ImmutableAlternative alternative : alternatives) {

			JsonObject jsonAlternative = new JsonObject(2);

			if (alternative.getName().toLowerCase().equals(searchString)) {

				jsonAlternative.putField("ID", alternative.getAlternativeID());
				jsonAlternative.putField("Name", alternative.getName());
				hitList.addNode(jsonAlternative);
				
				addedAlternatives.add(alternative.getAlternativeID());

				if (hitList.size() == maxHits) {

					break;
				}
			}
		}
	}

	private void addContainingSearchMatches(String searchString, JsonArray hitList, List<Integer> alreadyAddedAlternatives) {

		for (ImmutableAlternative alternative : alternatives) {

			JsonObject jsonAlternative = new JsonObject(2);

			if (alternative.getName().toLowerCase().contains(searchString) && 
					!alreadyAddedAlternatives.contains(alternative.getAlternativeID())) {

				jsonAlternative.putField("ID", alternative.getAlternativeID());
				jsonAlternative.putField("Name", alternative.getName());
				hitList.addNode(jsonAlternative);

				if (hitList.size() == maxHits) {

					break;
				}
			}
		}
	}

	protected static void sendEmptyJSONResponse(HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(1);
		jsonObject.putField("hitCount", "0");
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	protected void sendJSONResponse(JsonArray jsonArray, HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(2);
		jsonObject.putField("hitCount", Integer.toString(jsonArray.size()));
		jsonObject.putField("hits", jsonArray);
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	@Override
	public void close() throws Exception {}

}
