package com.nordicpeak.flowengine.queries.listsearchqueryprovider.beans;

import java.lang.reflect.Field;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativeQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

@Table(name = "list_search_query_instances")
@XMLElement
public class ListSearchQueryInstance extends BaseQueryInstance implements FixedAlternativesQueryInstance, StringValueQueryInstance {

	private static final long serialVersionUID = -27311785565846821L;
	
	public static final Field ALTERNATIVES_RELATION = ReflectionUtils.getField(ListSearchQueryInstance.class, "alternatives");
	public static final Field QUERY_RELATION = ReflectionUtils.getField(ListSearchQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName="queryID")
	@ManyToOne
	@XMLElement
	private ListSearchQuery query;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase=true)
	private List<ListSearchAlternative> alternatives;

	@Override
	public ListSearchQuery getQuery() {

		return query;
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		FixedAlternativeQueryUtils.appendExportXMLAlternatives(doc, element, this);

		return element;
	}

	public static long getSerialversionuid() {
		
		return serialVersionUID;
	}
	
	@Override
	public List<ListSearchAlternative> getAlternatives() {
	
		return alternatives;
	}

	public void setAlternatives(List<ListSearchAlternative> alternatives) {
	
		this.alternatives = alternatives;
	}

	public void setQuery(ListSearchQuery query) {
	
		this.query = query;
	}

	public Integer getQueryInstanceID() {
		
		return queryInstanceID;
	}
	
	public void setQueryInstanceID(Integer queryInstanceID) {
	
		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public String getStringValue() {

		if (alternatives == null) {

			return null;
		}

		StringBuilder stringBuilder = new StringBuilder();

		if (alternatives != null) {

			for (ListSearchAlternative alternative : alternatives) {

				if (stringBuilder.length() != 0) {

					stringBuilder.append(", ");
				}

				stringBuilder.append(alternative.getName());
			}
		}

		return stringBuilder.toString();
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {
		
		if (alternatives != null) {
			
			attributeHandler.setAttribute(query.getAttributeName(), getStringValue());
		}
	}
	
	public void resetAttribute(MutableAttributeHandler attributeHandler) {
		
		attributeHandler.removeAttribute(query.getAttributeName());
	}
	
	@Override
	public void reset(MutableAttributeHandler attributeHandler) {
		
		reset(attributeHandler, true);
	}
	
	public void reset(MutableAttributeHandler attributeHandler, boolean resetQueryAlternativeList) {
		
		this.alternatives = null;

		if (query.isSetAsAttribute()) {
			resetAttribute(attributeHandler);
		}
		
		if (resetQueryAlternativeList) {
			
			this.getQuery().setAlternativeList(null);
		}

		super.reset(attributeHandler);
	}

	@Override
	public String toString() {

		return "ListSearchQueryInstance (queryInstanceID=" + queryInstanceID + ")";
	}
	
	@Override
	public QueryRequestProcessor getQueryRequestProcessor(HttpServletRequest req, User user, User poster, URIParser uriParser, QueryHandler queryHandler, InstanceRequestMetadata requestMetadata) throws Exception {
		
		return BaseQueryUtils.getGenericQueryInstanceProvider(this.getClass(), queryHandler, queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID()).getQueryRequestProcessor(this, req, user, poster, uriParser, requestMetadata);
	}

	@Override
	public String getFreeTextAlternativeValue() {

		return null;
	}
}
