package com.nordicpeak.flowengine.queries.manualcheckboxquery;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.interfaces.MutableAlternative;

@Table(name = "manual_checkbox_query_alternatives")
@XMLElement
public class CheckboxAlternative extends GeneratedElementable implements MutableAlternative, Serializable {
	
	private static final long serialVersionUID = 5698552791580194144L;
	
	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer alternativeID;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 1024)
	@XMLElement
	private String name;
	
	@DAOManaged
	@OrderBy
	@XMLElement
	private Integer sortIndex;
	
	@DAOManaged
	@XMLElement
	private boolean checked;
	
	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private ManualCheckboxQuery query;
	
	public CheckboxAlternative() {}
	
	public CheckboxAlternative(String name) {
		
		super();
		this.name = name;
	}
	
	@Override
	public Integer getAlternativeID() {
		
		return alternativeID;
	}
	
	@Override
	public void setAlternativeID(Integer alternativeID) {
		
		this.alternativeID = alternativeID;
	}
	
	public boolean isChecked() {
		return checked;
	}
	
	public void setChecked(boolean checked) {
		this.checked = checked;
	}
	
	public ManualCheckboxQuery getQuery() {
		
		return query;
	}
	
	public void setQuery(ManualCheckboxQuery query) {
		
		this.query = query;
	}
	
	@Override
	public String getName() {
		
		return name;
	}
	
	@Override
	public void setName(String name) {
		
		this.name = name;
	}
	
	@Override
	public Integer getSortIndex() {
		
		return sortIndex;
	}
	
	@Override
	public void setSortIndex(Integer sortIndex) {
		
		this.sortIndex = sortIndex;
	}
	
	@Override
	public String getExportXMLValue() {
		return null;
	}
	
	@Override
	public void setExportXMLValue(String value) {}
	
	@Override
	public String toString() {
		
		return name + " (alternativeID: " + alternativeID + ")";
	}
	
	@Override
	public int hashCode() {
		
		final int prime = 31;
		int result = 1;
		result = prime * result + ((alternativeID == null) ? 0 : alternativeID.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		CheckboxAlternative other = (CheckboxAlternative) obj;
		if (alternativeID == null) {
			if (other.alternativeID != null) {
				return false;
			}
		} else if (!alternativeID.equals(other.alternativeID)) {
			return false;
		}
		return true;
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		alternativeID = XMLValidationUtils.validateParameter("alternativeID", xmlParser, true, PositiveStringIntegerPopulator.getPopulator(), errors);
		name = XMLValidationUtils.validateParameter("name", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		sortIndex = XMLValidationUtils.validateParameter("sortIndex", xmlParser, true, 1, 255, IntegerPopulator.getPopulator(), errors);
		checked = xmlParser.getPrimitiveBoolean("checked");
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
		
	}

	@Override
	public String getAttributeValue() {
		return null;
	}

	@Override
	public void setAttributeValue(String value) {
		
	}
}
