package com.nordicpeak.flowengine.queries.manualsignquery;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLValidationUtils;

@Table(name = "manual_sign_fields")
@XMLElement
public class ManualSignField extends GeneratedElementable implements Serializable, XMLParserPopulateable {

	public static final Field MANUAL_SIGN_QUERY_RELATION = ReflectionUtils.getField(ManualSignField.class, "query");

	private static final long serialVersionUID = 8919797448020843470L;

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer manualSignFieldID;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String label;

	@DAOManaged
	@OrderBy
	@XMLElement
	private Integer sortIndex;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private ManualSignQuery query;

	public ManualSignField() {

	}

	public Integer getManualSignFieldID() {

		return manualSignFieldID;
	}

	public void setManualSignFieldID(Integer textFieldID) {

		this.manualSignFieldID = textFieldID;
	}

	public ManualSignQuery getQuery() {

		return query;
	}

	public void setQuery(ManualSignQuery query) {

		this.query = query;
	}

	public String getLabel() {

		return label;
	}

	public void setLabel(String name) {

		this.label = name;
	}

	@Override
	public String toString() {

		return label + " (manualSignFieldID: " + manualSignFieldID + ")";
	}

	public Integer getSortIndex() {

		return sortIndex;
	}

	public void setSortIndex(Integer sortIndex) {

		this.sortIndex = sortIndex;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		manualSignFieldID = XMLValidationUtils.validateParameter("manualSignFieldID", xmlParser, true, PositiveStringIntegerPopulator.getPopulator(), errors);
		label = XMLValidationUtils.validateParameter("label", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);

		sortIndex = XMLValidationUtils.validateParameter("sortIndex", xmlParser, true, NonNegativeStringIntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

}
