package com.nordicpeak.flowengine.evaluators.maritalstatusquerystateevaluator;

import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.personprovider.MaritalStatus;
import com.nordicpeak.personprovider.Person;
import com.nordicpeak.personprovider.PersonProvider;

public class MaritalStatusQueryStateEvaluationProviderModule extends BaseQueryStateEvaluationProviderModule<MaritalStatusQueryStateEvaluator> {

	protected AnnotatedDAO<MaritalStatusQueryStateEvaluator> evaluatorDAO;

	protected QueryParameterFactory<MaritalStatusQueryStateEvaluator, Integer> evaluatorIDParamFactory;

	protected MaritalStatusQueryStateEvaluatorCRUD evaluatorCRUD;

	@InstanceManagerDependency
	protected PersonProvider personProvider;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, MaritalStatusQueryStateEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(MaritalStatusQueryStateEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new MaritalStatusQueryStateEvaluatorCRUD(MaritalStatusQueryStateEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, MaritalStatusQueryStateEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (evaluator.isInitialized()) {
			
			return null;
		}
		
		if (poster != null) {
			
			evaluator.setInitialized(true);

			if (personProvider == null) {

				log.warn("User " + poster + " tried to evaluate marital status but no person provider is configured!");
				return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
			}

			String citizenIdentifier = poster.getAttributeHandler().getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE);
			Person person;

			if (StringUtils.isEmpty(citizenIdentifier) || (person = personProvider.getPerson(citizenIdentifier)) == null) {

				return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
			}

			boolean married = person.getMaritalStatus() == MaritalStatus.MARRIED;

			if ((married || evaluator.isInverted()) && !(married && evaluator.isInverted())) {

				return applyEvaluatorQueryStates(queryInstance, evaluator, callback, attributeHandler);

			} else {

				return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
			}

		} else {

			return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
		}

	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		MaritalStatusQueryStateEvaluator evaluator = new MaritalStatusQueryStateEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return true;
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		MaritalStatusQueryStateEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<MaritalStatusQueryStateEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	protected Class<MaritalStatusQueryStateEvaluator> getEvaluatorClass() {

		return MaritalStatusQueryStateEvaluator.class;
	}

	@Override
	protected AnnotatedDAO<MaritalStatusQueryStateEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	@Override
	protected QueryParameterFactory<MaritalStatusQueryStateEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return this.evaluatorIDParamFactory;
	}

	@Override
	protected List<Integer> getTargetQueryIDs(MaritalStatusQueryStateEvaluator evaluator, EvaluationCallback callback) {

		return Collections.singletonList(callback.getQueryInstance().getQueryInstanceDescriptor().getQueryDescriptor().getQueryID());
	}

	@Override
	public MaritalStatusQueryStateEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		MaritalStatusQueryStateEvaluator evaluator = importEvaluatorDescriptor(descriptor, transactionHandler, query, statusConversionMap);

		evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

}
