package com.nordicpeak.flowengine.evaluators.internalaccount;

import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.UserUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;


public class MatchingInternalAccountQueryStateEvaluationProviderModule extends BaseQueryStateEvaluationProviderModule<MatchingInternalAccountQueryStateEvaluator> {

	protected AnnotatedDAO<MatchingInternalAccountQueryStateEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<MatchingInternalAccountQueryStateEvaluator, Integer> evaluatorIDParamFactory;
	
	protected MatchingInternalAccountQueryStateEvaluatorCRUD evaluatorCRUD;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, MatchingInternalAccountQueryStateEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(MatchingInternalAccountQueryStateEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new MatchingInternalAccountQueryStateEvaluatorCRUD(MatchingInternalAccountQueryStateEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, MatchingInternalAccountQueryStateEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (poster == null) {
			
			return null;
		}
		
		String citizenIdentifier = UserUtils.getAttribute(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE, poster);
		
		User internalAccount = null;
		
		if (citizenIdentifier != null) {
			
			internalAccount = systemInterface.getUserHandler().getUserByAttribute(Constants.MANAGER_CITIZEN_IDENTIFIER_ATTRIBUTE, citizenIdentifier, false, false);
		}
		
		if (evaluator.isRequireEnabledAccount() && internalAccount != null && !internalAccount.isEnabled()) {
			internalAccount = null;
		}
		
		if (internalAccount == null) {
			
			if (evaluator.isInclusive()) {
				
				return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
				
			} else {
				
				return applyEvaluatorQueryStates(queryInstance, evaluator, callback, attributeHandler);
			}
			
		} else {
			
			if (evaluator.isInclusive()) {
				
				return applyEvaluatorQueryStates(queryInstance, evaluator, callback, attributeHandler);
				
			} else {
				
				return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
			}
		}
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		MatchingInternalAccountQueryStateEvaluator evaluator = new MatchingInternalAccountQueryStateEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return true;
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		MatchingInternalAccountQueryStateEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<MatchingInternalAccountQueryStateEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	protected Class<MatchingInternalAccountQueryStateEvaluator> getEvaluatorClass() {

		return MatchingInternalAccountQueryStateEvaluator.class;
	}

	@Override
	protected AnnotatedDAO<MatchingInternalAccountQueryStateEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	@Override
	protected QueryParameterFactory<MatchingInternalAccountQueryStateEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return this.evaluatorIDParamFactory;
	}

	@Override
	protected List<Integer> getTargetQueryIDs(MatchingInternalAccountQueryStateEvaluator evaluator, EvaluationCallback callback) {

		return Collections.singletonList(callback.getQueryInstance().getQueryInstanceDescriptor().getQueryDescriptor().getQueryID());
	}

	@Override
	public MatchingInternalAccountQueryStateEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		MatchingInternalAccountQueryStateEvaluator evaluator = importEvaluatorDescriptor(descriptor, transactionHandler, query, statusConversionMap);
		
		evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
}
