package com.nordicpeak.flowengine.evaluators.multipartsigningmailevaluator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;

public class MultipartSigningMailEvaluatorEvaluatorCRUD extends BaseEvaluatorCRUD<MultipartSigningMailEvaluatorEvaluator, MultipartSigningMailEvaluatorEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<MultipartSigningMailEvaluatorEvaluator> POPULATOR = new AnnotatedRequestPopulator<MultipartSigningMailEvaluatorEvaluator>(MultipartSigningMailEvaluatorEvaluator.class);

	public MultipartSigningMailEvaluatorEvaluatorCRUD(Class<MultipartSigningMailEvaluatorEvaluator> beanClass, AnnotatedDAOWrapper<MultipartSigningMailEvaluatorEvaluator, Integer> evaluatorDAO, MultipartSigningMailEvaluatorEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "MultipartSigningMailEvaluatorEvaluator", "send signed mail evaluator", null, callback);
	}

	@Override
	protected MultipartSigningMailEvaluatorEvaluator populateFromUpdateRequest(MultipartSigningMailEvaluatorEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		MultipartSigningMailEvaluatorEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);

		String emailRecipientsParam = req.getParameter("emailRecipients");

		if (!StringUtils.isEmpty(emailRecipientsParam)) {

			List<ValidationError> errors = new ArrayList<ValidationError>();

			String[] addresses = emailRecipientsParam.split("[,;:]");

			List<String> recipients = Arrays.asList(addresses);

			List<String> trimmedRecipients = new ArrayList<String>(recipients.size());

			for (String recipient : recipients) {

				recipient = recipient.trim();

				if (!EmailUtils.isValidEmailAddress(recipient) && AttributeTagUtils.getAttributeTags(recipient) == null) {

					errors.add(new ValidationError("emailRecipients", recipient, ValidationErrorType.InvalidFormat));

				} else {

					trimmedRecipients.add(recipient);
				}
			}

			if (!errors.isEmpty()) {

				throw new ValidationException(errors);
			}

			if (!callback.isAllowSignerContactAsSender()) {

				evaluator.setUsePosterContactAsSender(false);
			}

			evaluator.setEmailRecipients(trimmedRecipients);

		} else {

			throw new ValidationException(new ValidationError("emailRecipients", ValidationErrorType.RequiredField));
		}

		return evaluator;
	}

	@Override
	protected void appendUpdateFormData(MultipartSigningMailEvaluatorEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		if (bean.getEmailRecipients() != null) {
			XMLUtils.appendNewElement(doc, updateTypeElement, "EmailRecipients", StringUtils.toCommaSeparatedString(bean.getEmailRecipients()));
		}

		XMLUtils.appendNewElement(doc, updateTypeElement, "AvailableTags", callback.getAvailableTags());

		if (callback.isAllowSignerContactAsSender()) {

			XMLUtils.appendNewElement(doc, updateTypeElement, "allowPosterContactAsSender", true);
		}
	}

}
