package com.nordicpeak.flowengine.evaluators.municipalitycode;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluator;

@Table(name = "municipality_code_query_state_evaluators")
@XMLElement
public class MunicipalityCodeQueryStateEvaluator extends BaseQueryStateEvaluator {

	private static final long serialVersionUID = 7582032280793947029L;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "municipality_code_query_state_codes", remoteValueColumnName = "municipalityCode")
	@WebPopulate(required = true, populator=NonNegativeStringIntegerPopulator.class)
	@SplitOnLineBreak
	@NoDuplicates
	@XMLElement(name = "AllowedMunicipalityCodes", childName = "MunicipalityCode")
	private List<Integer> allowedMunicipalityCodes;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean inclusive;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		super.populate(xmlParser);

		List<ValidationError> errors = new ArrayList<ValidationError>(1);

		inclusive = xmlParser.getPrimitiveBoolean("inclusive");

		allowedMunicipalityCodes = XMLValidationUtils.validateParameters("AllowedMunicipalityCodes/MunicipalityCode", xmlParser, false, IntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public boolean isInclusive() {

		return inclusive;
	}

	public void setInclusive(boolean inclusive) {

		this.inclusive = inclusive;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public List<Integer> getAllowedMunicipalityCodes() {

		return allowedMunicipalityCodes;
	}

	public void setAllowedMunicipalityCodes(List<Integer> allowedMunicipalityCodes) {

		this.allowedMunicipalityCodes = allowedMunicipalityCodes;
	}
}
