<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="imagePath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/common/js/queryadmin.js
	</xsl:variable>

	<xsl:template match="Document">

		<div id="InfoQueryProvider" class="contentitem">

			<xsl:apply-templates select="validationError" />
			
			<xsl:apply-templates select="ShowParagraphInfoQuery"/>
			<xsl:apply-templates select="UpdateParagraphInfoQuery"/>
			<xsl:apply-templates select="AddParagraph"/>
			<xsl:apply-templates select="UpdateParagraph"/>
			<xsl:apply-templates select="SortParagraphs"/>			

		</div>

	</xsl:template>

	<xsl:template match="ShowParagraphInfoQuery">
	
		<h1><xsl:value-of select="$i18n.UpdateQuery" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="ParagraphInfoQuery/QueryDescriptor/name" /></h1>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.BaseInfo"/></legend>
			
			<div class="floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatequery/{ParagraphInfoQuery/queryID}" title="{$i18n.UpdateBaseInformation}">
					<img class="alignbottom" src="{$imagePath}/pen.png"/>
				</a>
			</div>
			
			<xsl:call-template name="createCommonShowFields">
				<xsl:with-param name="element" select="ParagraphInfoQuery" />
			</xsl:call-template>
			
<!-- 			<div class="floatleft full bigmarginbottom">
				<label class="floatleft clearboth"><xsl:value-of select="$i18n.Layout" /></label>
				<div class="floatleft full">
					<xsl:value-of select="FieldLayout[value = $layout]/name" />
			    </div>
			</div> -->
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Paragraphs"/></legend>
			
			<xsl:if test="ParagraphInfoQuery/Paragraphs/Paragraph">
				
				<div class="floatleft full">
				
					<xsl:apply-templates select="ParagraphInfoQuery/Paragraphs/Paragraph" mode="list" />
			
				</div>
			
			</xsl:if>
			
			<xsl:if test="ParagraphInfoQuery/Paragraphs/Paragraph">
			
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sortparagraphs/{ParagraphInfoQuery/queryID}" title="{$i18n.SortParagraphs.Title}">
						<xsl:value-of select="$i18n.SortParagraphs.Title"/>
						<img class="alignbottom marginleft" src="{$commonImagePath}/move.png"/>
					</a>
				</div>
				
			</xsl:if>
			
			<div class="clearboth floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addparagraph/{ParagraphInfoQuery/queryID}" title="{$i18n.AddParagraph}">
					<xsl:value-of select="$i18n.AddParagraph"/>
					<img class="alignbottom marginleft" src="{$commonImagePath}/add.png"/>
				</a>
			</div>
			
		</fieldset>
		
		<div class="floatright margintop clearboth">
			<input type="button" value="{$i18n.Done}" onclick="window.location = '{showFlowURL}'" />
		</div>
		
	</xsl:template>

	<xsl:template match="Paragraph" mode="list">
				
		<div class="floatleft hover border full marginbottom border-radius lightbackground">
			
			<div class="padding floatleft">
				<div class="marginleft">
					<b>
						<xsl:value-of select="name" />
						
						<xsl:if test="visible = 'false'">
							<xsl:text>&#160;(</xsl:text>
							<xsl:value-of select="$i18n.Hidden"></xsl:value-of>
							<xsl:text>)</xsl:text>
						</xsl:if>
					</b>
				</div>
			</div>
			<div class="padding floatright">
				<div class="floatright marginright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deleteparagraph/{paragraphID}" onclick="return confirm('{$i18n.DeleteParagraph.Confirm}: {label}?');" title="{$i18n.DeleteParagraph}: {label}">
						<img src="{$imagePath}/delete.png"/>
					</a>
				</div>
				<div class="floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updateparagraph/{paragraphID}" title="{$i18n.UpdateParagraph}: {label}">
						<img src="{$imagePath}/pen.png"/>
					</a>
				</div>
			</div>
			
		</div>
				
	</xsl:template>

	<xsl:template match="AddParagraph">
	
		<h1><xsl:value-of select="$i18n.AddParagraph" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
	
		<form id="addParagraphForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
			
			<xsl:call-template name="showParagraphForm" />
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.Add}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template match="UpdateParagraph">
	
		<h1><xsl:value-of select="$i18n.UpdateParagraph" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="Paragraph/label" /></h1>
		
		<xsl:apply-templates select="validationException/validationError" />
		
		<form id="udpateParagraphForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<xsl:call-template name="showParagraphForm">
				<xsl:with-param name="element" select="Paragraph" />
			</xsl:call-template>
		
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template name="showParagraphForm">
		
		<xsl:param name="element" select="null" />
		
		<div class="floatleft full bigmarginbottom">
			<label for="name" class="floatleft clearboth"><xsl:value-of select="$i18n.Name" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'name'"/>
					<xsl:with-param name="name" select="'name'"/>
					<xsl:with-param name="title" select="$i18n.Name"/>
					<xsl:with-param name="element" select="$element" />
					<xsl:with-param name="maxlength" select="'255'"/>
				</xsl:call-template>
		    </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'visible'" />
					<xsl:with-param name="name" select="'visible'" />
					<xsl:with-param name="element" select="$element" /> 
					<xsl:with-param name="class" select="'vertical-align-middle'" />
				</xsl:call-template>
					
				<label for="visible">
					<xsl:value-of select="$i18n.Visible" />
				</label>
		    </div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="description" class="floatleft clearboth"><xsl:value-of select="$i18n.Text" /></label>
			<div class="floatleft full">
	
				<xsl:choose>
					<xsl:when test="/Document/useCKEditorForDescription = 'true'">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="'text'" />
							<xsl:with-param name="name" select="'text'" />
							<xsl:with-param name="title" select="$i18n.Text"/>
							<xsl:with-param name="class" select="'common-ckeditor'" />
							<xsl:with-param name="element" select="$element" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="'text'"/>
							<xsl:with-param name="name" select="'text'"/>
							<xsl:with-param name="title" select="$i18n.Text"/>
							<xsl:with-param name="element" select="$element" />
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>			
			
			</div>
		</div>
		
		<xsl:call-template name="initializeCommonCKEditor">
			<xsl:with-param name="element" select="$element" />
			<xsl:with-param name="flowTypeID" select="$element/../flowTypeID"/>
		</xsl:call-template>		
		
	</xsl:template>

	<xsl:template match="SortParagraphs">
		
		<h1><xsl:value-of select="$i18n.SortParagraphs" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="ParagraphInfoQuery/QueryDescriptor/name" /></h1>
		
		<form id="sortParagraphsForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">
		
			<div class="floatleft full sortable">
							
				<xsl:apply-templates select="ParagraphInfoQuery/Paragraphs/Paragraph" mode="sort" />
							
			</div>
			
			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>
		
	</xsl:template>
	
	<xsl:template match="Paragraph" mode="sort">
	
		<div id="paragraph_{paragraphID}" class="floatleft hover border full marginbottom lightbackground cursor-move border-radius">
			<div class="padding">
				<img class="vertical-align-middle marginright" src="{$commonImagePath}/move.png" title="{$i18n.MoveParagraph}"/>
				<xsl:value-of select="name" />
				
				<xsl:if test="visible = 'false'">
					<xsl:text>&#160;(</xsl:text>
					<xsl:value-of select="$i18n.Hidden"></xsl:value-of>
					<xsl:text>)</xsl:text>
				</xsl:if>
				
				<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
				
				<xsl:call-template name="createHiddenField">
					<xsl:with-param name="id" select="concat('sortorder_', paragraphID)" />
					<xsl:with-param name="name" select="concat('sortorder_', paragraphID)" />
					<xsl:with-param name="value" select="sortIndex" />
					<xsl:with-param name="class" select="'sortorder'" />
					<xsl:with-param name="requestparameters" select="//requestparameters" />
				</xsl:call-template>
			</div>
		</div>
	
	</xsl:template>

	<xsl:template match="UpdateParagraphInfoQuery">

		<h1>
			<xsl:value-of select="$i18n.UpdateQuery" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="ParagraphInfoQuery/QueryDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form id="updateParagraphInfoQueryForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="ParagraphInfoQuery" />
			</xsl:call-template>

			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'hideTitle'" />
						<xsl:with-param name="name" select="'hideTitle'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="ParagraphInfoQuery" />
					</xsl:call-template>
					
					<label for="hideTitle">
						<xsl:value-of select="$i18n.HideTitle" />
					</label>
				</div>

			</div>

			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'hideBackground'" />
						<xsl:with-param name="name" select="'hideBackground'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="ParagraphInfoQuery" />
					</xsl:call-template>
					
					<label for="hideBackground">
						<xsl:value-of select="$i18n.HideBackground" />
					</label>
				</div>

			</div>
			
			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'dontSetPopulated'" />
						<xsl:with-param name="name" select="'dontSetPopulated'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="ParagraphInfoQuery" />
					</xsl:call-template>
					
					<label for="dontSetPopulated">
						<xsl:value-of select="$i18n.DontSetPopulated" />
					</label>
				</div>

			</div>			

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'UpdateFailedParagraphInfoQueryNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.ParagraphInfoQueryNotFound" />
		</p>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'DescriptionRequiredWhenTitleHidden']">

		<p class="error">
			<xsl:value-of select="$i18n.DescriptionRequiredWhenTitleHidden" />
		</p>

	</xsl:template>

	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = ''">
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>