package com.nordicpeak.flowengine.paymentlog;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.bool.BooleanUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderException;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowFamily;
import com.nordicpeak.flowengine.interfaces.PaymentProduct;
import com.nordicpeak.flowengine.interfaces.PaymentQuery;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.paymentlog.beans.Account;
import com.nordicpeak.flowengine.paymentlog.beans.ChartOfAccounts;
import com.nordicpeak.flowengine.paymentlog.beans.FlowFamilyTransactionSettings;
import com.nordicpeak.flowengine.paymentlog.beans.PaymentProductTransactionSettings;
import com.nordicpeak.flowengine.paymentlog.beans.Transaction;
import com.nordicpeak.openhierarchy.payment.enums.PaymentType;
import com.nordicpeak.openhierarchy.payment.events.DelayedPaymentEvent;
import com.nordicpeak.openhierarchy.payment.events.PaymentEvent;
import com.nordicpeak.openhierarchy.payment.interfaces.InvoiceLine;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentProvider;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentResponse;

public class TransactionFlowAdminExtensionModule extends AnnotatedForegroundModule implements FlowAdminExtensionViewProvider, ViewFragmentModule<ForegroundModuleDescriptor> {
	
//	private static final AnnotatedRequestPopulator<FlowFamilyTransactionSettings> POPULATOR = new AnnotatedRequestPopulator<FlowFamilyTransactionSettings>(FlowFamilyTransactionSettings.class);
	
	@XSLVariable(prefix = "java.")
	private String extensionViewTitle = "Transaction settings";
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Priority", description = "The priority of this extension provider compared to other providers. A low value means a higher priority. Valid values are 0 - " + Integer.MAX_VALUE + ".", required = true, formatValidator = NonNegativeStringIntegerValidator.class)
	protected int priority = 0;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmententXML;
	
	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;
	
	@InstanceManagerDependency
	private PaymentProvider paymentProvider;
	
	private FlowAdminModule flowAdminModule;
	
	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;
	
	private AnnotatedDAO<FlowFamilyTransactionSettings> settingsDAO;
	private AnnotatedDAOWrapper<Transaction, Integer> transactionDAOWrapper;
	private AnnotatedDAOWrapper<Account, Integer> accountDAOWrapper;
	private AnnotatedDAOWrapper<ChartOfAccounts, Integer> chartDAOWrapper;
	
	private QueryParameterFactory<FlowFamilyTransactionSettings, Integer> flowFamilyIDParamFactory;
	
	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {
		
		this.viewFragmentTransformer = new ModuleViewFragmentTransformer<ForegroundModuleDescriptor>(sectionInterface.getForegroundModuleXSLTCache(), this, sectionInterface.getSystemInterface().getEncoding());
		
		super.init(moduleDescriptor, sectionInterface, dataSource);
	}
	
	@Override
	public void unload() throws Exception {
		
		if (flowAdminModule != null) {
			
			setFlowAdminModule(null);
		}
		
		super.unload();
	}
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		super.createDAOs(dataSource);
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AccountModule.class.getName(), new XMLDBScriptProvider(AccountModule.class.getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface);
		
		settingsDAO = daoFactory.getDAO(FlowFamilyTransactionSettings.class);
		accountDAOWrapper = daoFactory.getDAO(Account.class).getWrapper(Integer.class);
		accountDAOWrapper.addRelations(Account.CHART_RELATION);
		accountDAOWrapper.setUseRelationsOnGet(true);
		
		chartDAOWrapper = daoFactory.getDAO(ChartOfAccounts.class).getWrapper(Integer.class);
		chartDAOWrapper.addRelations(ChartOfAccounts.ACCOUNTS_RELATION);
		chartDAOWrapper.setUseRelationsOnGetAll(true);
		chartDAOWrapper.setUseRelationsOnGet(true);
		
		transactionDAOWrapper = daoFactory.getDAO(Transaction.class).getWrapper(Integer.class);
		
		flowFamilyIDParamFactory = settingsDAO.getParamFactory("flowFamilyID", Integer.class);
	}
	
	@Override
	protected void moduleConfigured() throws Exception {
		
		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);
	}
	
	@Override
	public int getPriority() {
		
		return 0;
	}
	
	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {
		
		if (!flow.isPaymentSupportEnabled() || !AccessUtils.checkRecursiveModuleAccess(user, moduleDescriptor, systemInterface)) {
			
			return null;
		}
		
		Document doc = createDocument(req, uriParser, user);
		
		Element showViewElement = doc.createElement("ShowTransactionSettings");
		doc.getDocumentElement().appendChild(showViewElement);
		
		FlowFamilyTransactionSettings transactionSettings = getTransactionSettings(flow.getFlowFamily());
		
		boolean enabled = true;
		
		if(transactionSettings != null) {
		
			XMLUtils.append(doc, showViewElement, transactionSettings);
			enabled = true;
		}
		
		showViewElement.appendChild(flow.toXML(doc));
		
		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), enabled);
	}
	
	private FlowFamilyTransactionSettings getTransactionSettings(ImmutableFlowFamily flowFamily, Field... relations) throws SQLException {
		
		HighLevelQuery<FlowFamilyTransactionSettings> query = new HighLevelQuery<FlowFamilyTransactionSettings>();
		
		query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamily.getFlowFamilyID()));
		
		if (relations != null) {
			query.addRelations(relations);
		}
		
		return settingsDAO.get(query);
	}
	
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {
		
		Document doc = XMLUtils.createDomDocument();
		Element documentElement = doc.createElement("Document");
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(moduleDescriptor.toXML(doc));
		
		XMLUtils.appendNewElement(doc, documentElement, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, documentElement, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));
		
		doc.appendChild(documentElement);
		return doc;
	}
	
	@Override
	public String getExtensionViewTitle() {
		
		return extensionViewTitle;
	}
	
	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {
		
		return moduleDescriptor;
	}
	
	@Override
	public List<LinkTag> getLinkTags() {
		
		return links;
	}
	
	@Override
	public List<ScriptTag> getScriptTags() {
		
		return scripts;
	}
	
	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {
		
		if (flowAdminModule == null && this.flowAdminModule != null) {
			
			this.flowAdminModule.removeExtensionViewProvider(this);
		}
		
		this.flowAdminModule = flowAdminModule;
		
		if (this.flowAdminModule != null) {
			
			this.flowAdminModule.addExtensionViewProvider(this);
		}
	}
	
	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateFlowTransactionSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);
		
		if (flow == null) {
			
			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));
			
		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {
			
			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}
		
		FlowFamilyTransactionSettings transactionSettings = getTransactionSettings(flow.getFlowFamily(), Account.CHART_RELATION);
		
		List<ValidationError> validationErrors = null;
		
		if (req.getMethod().equalsIgnoreCase("POST")) {
			
			if (transactionSettings == null) {
				
				transactionSettings = new FlowFamilyTransactionSettings();
				transactionSettings.setFlowFamilyID(flow.getFlowFamily().getFlowFamilyID());
			}
			
			validationErrors = new ArrayList<>();
			
			Integer chartID = ValidationUtils.validateParameter("chart", req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
			Integer accountID = ValidationUtils.validateParameter("account", req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
			Integer vatAccountID = ValidationUtils.validateParameter("vatAccount", req, false, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
			
			ChartOfAccounts chart = getChart(validationErrors, chartID, "chart");
			Account account = getAccount(validationErrors, accountID, chart, "account");
			Account vatAccount = getVatAccount(validationErrors, vatAccountID, chart, "vatAccount");
			
			boolean paymentProductSettingsEnabled = BooleanUtils.toBoolean(req.getParameter("paymentProductSettingsEnabled"));
			List<PaymentProductTransactionSettings> paymentProductTransactionSettingsList = null;
			
			if (paymentProductSettingsEnabled) {
				
				List<Integer> paymentProductChartIDs = NumberUtils.toInt(req.getParameterValues("paymentProductChart"));
				
				if (!CollectionUtils.isEmpty(paymentProductChartIDs)) {
					
					paymentProductTransactionSettingsList = populatePaymentProductTransactionSettingsList(req, paymentProductChartIDs, validationErrors);
					
					if (CollectionUtils.isEmpty(paymentProductTransactionSettingsList)) {
						
						paymentProductSettingsEnabled = false;
					}
					
				} else {
					
					paymentProductSettingsEnabled = false;
				}
			}
			
//			POPULATOR.populate(transactionSettings, req);
			
			if (validationErrors.isEmpty()) {
				
				transactionSettings.setAccount(account);
				transactionSettings.setVatAccount(vatAccount);
				transactionSettings.setPaymentProductSettingsEnabled(paymentProductSettingsEnabled);
				
				if (paymentProductSettingsEnabled) {
					
					transactionSettings.setPaymentProductSettingsList(paymentProductTransactionSettingsList);
				}
				
				log.info("User " + user + " updating transaction settings for flow " + flow);
				
				settingsDAO.addOrUpdate(transactionSettings, null);
				
				flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#transactionsettings");
				return null;
			}
		}
		
		log.info("User " + user + " requesting update transaction settings form for flow " + flow);
		
		Document doc = createDocument(req, uriParser, user);
		
		Element transactionSettingsElement = doc.createElement("UpdateTransactionSettings");
		doc.getDocumentElement().appendChild(transactionSettingsElement);
		
		transactionSettingsElement.appendChild(flow.toXML(doc));
		
		XMLUtils.append(doc, transactionSettingsElement, "Charts", chartDAOWrapper.getAll());
		
		XMLUtils.append(doc, transactionSettingsElement, transactionSettings);

		appendPaymentProducts(doc, transactionSettingsElement, flow);
		
		if (paymentProvider != null) {
			
			XMLUtils.appendNewElement(doc, transactionSettingsElement, "Currency", paymentProvider.getCurrencyCode());
		}
		
		if (!CollectionUtils.isEmpty(validationErrors)) {
			
			XMLUtils.append(doc, transactionSettingsElement, "ValidationErrors", validationErrors);
			transactionSettingsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}
		
		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	private Account getVatAccount(List<ValidationError> validationErrors, Integer vatAccountID, ChartOfAccounts chart, String fieldName) throws SQLException {

		Account vatAccount = null;
		if (vatAccountID != null) {
			
			vatAccount = accountDAOWrapper.get(vatAccountID);
			
			if (vatAccount == null || !vatAccount.getChart().equals(chart) || vatAccount.isArchived()) {
				validationErrors.add(new ValidationError(fieldName, ValidationErrorType.InvalidFormat));
			}
		}
		return vatAccount;
	}

	private Account getAccount(List<ValidationError> validationErrors, Integer accountID, ChartOfAccounts chart, String fieldName) throws SQLException {

		Account account = null;
		if (accountID != null) {
			
			account = accountDAOWrapper.get(accountID);
			
			if (account == null || !account.getChart().equals(chart) || account.isArchived()) {
				
				validationErrors.add(new ValidationError(fieldName, ValidationErrorType.InvalidFormat));
			}
		}
		return account;
	}

	private ChartOfAccounts getChart(List<ValidationError> validationErrors, Integer chartID, String fieldName) throws SQLException {

		ChartOfAccounts chart = null;

		if (chartID != null) {
			
			chart = chartDAOWrapper.get(chartID);
			
			if (chart == null) {
				validationErrors.add(new ValidationError(fieldName, ValidationErrorType.InvalidFormat));
			}
		}
		return chart;
	}
	
	private List<PaymentProductTransactionSettings> populatePaymentProductTransactionSettingsList(HttpServletRequest req, List<Integer> paymentProductChartIDs, List<ValidationError> validationErrors) throws SQLException {
		
		List<PaymentProductTransactionSettings> settingsList = new ArrayList<>(paymentProductChartIDs.size());

		for (Integer paymentProductChartID : paymentProductChartIDs) {
			
			boolean enabled = BooleanUtils.toBoolean(req.getParameter("paymentProductSettingsEnabled-" + paymentProductChartID));

			if (enabled) {
				
				String descriptionParameterName = "description-" + paymentProductChartID;
				String chartParameterName = "chart-" + paymentProductChartID;
				String accountParameterName = "account-" + paymentProductChartID;
				String vatAccountParameterName = "vatAccount-" + paymentProductChartID;
				
				String description = ValidationUtils.validateParameter(descriptionParameterName, req, true, StringPopulator.getPopulator(), validationErrors);
				Integer chartID = ValidationUtils.validateParameter(chartParameterName, req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
				Integer accountID = ValidationUtils.validateParameter(accountParameterName, req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
				Integer vatAccountID = ValidationUtils.validateParameter(vatAccountParameterName, req, false, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
				
				ChartOfAccounts chart = getChart(validationErrors, chartID, chartParameterName);
				Account account = getAccount(validationErrors, accountID, chart, accountParameterName);
				Account vatAccount = getVatAccount(validationErrors, vatAccountID, chart, vatAccountParameterName);
				
				PaymentProductTransactionSettings settings = new PaymentProductTransactionSettings(description, account, vatAccount);
				
				settingsList.add(settings);
			}
		}
		
		return settingsList;
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse deleteFlowTransactionSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);
		
		if (flow == null) {
			
			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));
			
		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {
			
			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}
		
		FlowFamilyTransactionSettings transactionSettings = getTransactionSettings(flow.getFlowFamily());
		
		if (transactionSettings != null) {
			
			log.info("User " + user + " deleting transaction settings for flow " + flow);
			
			settingsDAO.delete(transactionSettings);
			
		} else {
			
			log.warn("User " + user + " trying to deleting transaction settings for flow " + flow + " which has no transaction settings");
		}
		
		flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#transactionsettings");
		
		return null;
	}
	
	@EventListener(channel = PaymentProvider.class, priority = 50)
	public void processDelayedPaymentEvent(DelayedPaymentEvent paymentEvent, EventSource source) {
		processPaymentEvent(paymentEvent, source);
	}
	
	@EventListener(channel = PaymentProvider.class, priority = 50)
	public void processPaymentEvent(PaymentEvent paymentEvent, EventSource source) {
		
		PaymentResponse paymentResponse = paymentEvent.getPaymentResponse();
		User user = paymentEvent.getUser();
		
		try {
			
			if ("flowinstance".equals(paymentEvent.getSourceType()) && paymentResponse.getPaymentType() == PaymentType.DIRECT) {
				
				Integer flowInstanceID = paymentEvent.getSourceID();
				
				FlowInstance flowInstance = flowAdminModule.getFlowInstance(flowInstanceID);
				
				FlowFamilyTransactionSettings settings = getTransactionSettings(flowInstance.getFlow().getFlowFamily());
				
				if (settings != null) {
					
					if (settings.isPaymentProductSettingsEnabled() && !CollectionUtils.isEmpty(settings.getPaymentProductSettingsList())) {
						
						for (InvoiceLine invoiceLine : paymentEvent.getInvoice().getInvoiceLines()) {
							
							String description = invoiceLine.getDescription();
							
							PaymentProductTransactionSettings paymentProductSettings = getPaymentProductSettings(settings, description);
							
							Account account = paymentProductSettings != null ? paymentProductSettings.getAccount() : settings.getAccount();
							Account vatAccount = paymentProductSettings != null ? paymentProductSettings.getVatAccount() : settings.getVatAccount();
							
							BigDecimal payedAmount = invoiceLine.getQuantity().multiply(invoiceLine.getUnitPrice());
							
							createTransaction(paymentEvent, user, flowInstance, payedAmount, account, vatAccount);
						}
						
					} else {
						
						Account account = settings.getAccount();
						Account vatAccount = settings.getVatAccount();
						BigDecimal paymentAmount = paymentResponse.getPaymentAmount();
						
						createTransaction(paymentEvent, user, flowInstance, paymentAmount, account, vatAccount);
					}
				}
			}
			
		} catch (Exception e) {
			
			log.error("Error adding payment transaction for " + paymentEvent, e);
		}
	}

	private PaymentProductTransactionSettings getPaymentProductSettings(FlowFamilyTransactionSettings settings, String description) {

		for (PaymentProductTransactionSettings paymentProductSettings : settings.getPaymentProductSettingsList()) {
			
			if (description.equals(paymentProductSettings.getDescription())) {
				
				return paymentProductSettings;
			}
		}
		
		return null;
	}

	private void createTransaction(PaymentEvent paymentEvent, User user, FlowInstance flowInstance, BigDecimal payedAmount, Account account, Account vatAccount) throws SQLException {

		TransactionHandler transactionHandler = transactionDAOWrapper.createTransaction();

		try {
			BigDecimal vatAmount = null;
			
			if (payedAmount.compareTo(BigDecimal.ZERO) == 0) {
				
				log.warn("Payed amount " + payedAmount.toPlainString() + " is 0 for " + paymentEvent + ", skipping");
				return;
			}
			
			if (payedAmount.scale() > 2) {
				
				log.warn("Payed amount " + payedAmount.toPlainString() + " " + payedAmount.scale() + " scale too large for " + paymentEvent + ", truncating");
				payedAmount = payedAmount.setScale(2, RoundingMode.DOWN);
			}
			
			if (vatAccount != null) {
				
				vatAmount = payedAmount.multiply(BigDecimal.valueOf(vatAccount.getVat()), MathContext.UNLIMITED);
				Integer vatCalc = vatAccount.getVat() + 100;
				vatAmount = vatAmount.divide(BigDecimal.valueOf(vatCalc), 2, RoundingMode.DOWN);
				payedAmount = payedAmount.subtract(vatAmount, MathContext.UNLIMITED);
				
				if (vatAmount.precision() - vatAmount.scale() > 7) {
					
					log.error("VAT amount " + vatAmount.toPlainString() + " too large for " + paymentEvent + ", saturating");
					vatAmount = BigDecimal.valueOf(9999999);
				}
			}
			
			if (payedAmount.precision() - payedAmount.scale() > 7) {
				
				log.error("Payed amount " + payedAmount.toPlainString() + " too large for " + paymentEvent + ", saturating");
				payedAmount = BigDecimal.valueOf(9999999);
			}
			
			String description = flowInstance.getFlow().getName();
			
			String userString = null;
			
			if (user != null) {
				
				userString = user.getFirstname() + " " + user.getLastname();
				
				String citizenID;
				
				if (user.getAttributeHandler() != null && (citizenID = user.getAttributeHandler().getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE)) != null) {
					
					userString += " (" + citizenID + ")";
				}
			}
			
			// use paymentResponse.getCurrencyCode() ?
			
			Transaction transaction = new Transaction();
			transaction.setSourceType(paymentEvent.getSourceType());
			transaction.setSourceID(paymentEvent.getSourceID());
			transaction.setAmount(payedAmount);
			transaction.setAdded(paymentEvent.getTimestamp());
			transaction.setDescription(description);
			transaction.setAccount(account);
			transaction.setUser(paymentEvent.getUser());
			transaction.setUserString(userString);
			
			transactionDAOWrapper.add(transaction, transactionHandler);
			
			if (vatAmount != null && vatAmount.compareTo(BigDecimal.ZERO) > 0) {
				
				Transaction vatTransaction = new Transaction();
				vatTransaction.setSourceType(paymentEvent.getSourceType());
				vatTransaction.setSourceID(paymentEvent.getSourceID());
				vatTransaction.setAmount(vatAmount);
				vatTransaction.setAdded(paymentEvent.getTimestamp());
				vatTransaction.setDescription(description);
				vatTransaction.setAccount(vatAccount);
				vatTransaction.setUser(paymentEvent.getUser());
				vatTransaction.setUserString(userString);
				
				transactionDAOWrapper.add(vatTransaction, transactionHandler);
			}
			
			transactionHandler.commit();
			
		} finally {
			TransactionHandler.autoClose(transactionHandler);
		}
	}
	
	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}
	
	private void appendPaymentProducts(Document doc, Element targetElement, Flow flow) {

		Element paymentProductsElement = XMLUtils.appendNewElement(doc, targetElement, "PaymentProducts");

		List<PaymentQuery> paymentQueries = getPaymentQueries(flow);

		for (PaymentQuery paymentQuery : paymentQueries) {

			for (PaymentProduct paymentProduct : paymentQuery.getPaymentProducts()) {

				Element paymentProductElement = XMLUtils.append(doc, paymentProductsElement, paymentProduct);
				
				XMLUtils.appendNewElement(doc, paymentProductElement, "QueryName", paymentQuery.getQueryDescriptor().getName());
			}
		}
	}

	private List<PaymentQuery> getPaymentQueries(Flow flow) {

		if (flow == null || flow.getSteps() == null) {

			return Collections.emptyList();
		}

		List<PaymentQuery> paymentQueries = new ArrayList<>();

		try {

			for (Step step : flow.getSteps()) {

				if (step.getQueryDescriptors() != null) {

					for (QueryDescriptor queryDescriptor : step.getQueryDescriptors()) {

						Query query = flowAdminModule.getQueryHandler().getQuery(queryDescriptor, false);

						if (query instanceof PaymentQuery) {

							PaymentQuery paymentQuery = (PaymentQuery) query;

							paymentQueries.add(paymentQuery);
						}
					}
				}
			}

		} catch (QueryProviderException e) {

			log.error("Error getting payment queries for flow " + flow, e);
		}

		return paymentQueries;
	}

}
