<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="scripts">
		/js/transactionsettings.js?v=3
	</xsl:variable>

	<xsl:template match="Document">

		<xsl:apply-templates select="NoTransactionSettingsError" />
		<xsl:apply-templates select="ShowTransactionSettings" />
		<xsl:apply-templates select="UpdateTransactionSettings" />

	</xsl:template>

	<xsl:template match="ShowTransactionSettings">

		<a name="transactionsettings" />

		<xsl:choose>
			<xsl:when test="FlowFamilyTransactionSettings">

				<xsl:value-of select="$i18n.Settings.account" />
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="FlowFamilyTransactionSettings/Account[accountType = 'INCOME']/name" />

				<xsl:if test="FlowFamilyTransactionSettings/Account[accountType = 'VAT']">
					<br />
					<xsl:value-of select="$i18n.Settings.vatAccount" />
					<xsl:text>:&#x20;</xsl:text>
					<xsl:value-of select="FlowFamilyTransactionSettings/Account[accountType = 'VAT']/name" />
				</xsl:if>
				
				<xsl:if test="FlowFamilyTransactionSettings/paymentProductSettingsEnabled = 'true'">
					<br />
					<xsl:value-of select="$i18n.Settings.PaymentProductSettingsIsEnabled" />
				</xsl:if>

			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoFlowTransactionSettings" />
			</xsl:otherwise>
		</xsl:choose>

		<a href="{/Document/ModuleURI}/updateflowtransactionsettings/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateTransactionSettings" />
			<img class="marginleft" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>

		<xsl:if test="FlowFamilyTransactionSettings">
			<a href="{/Document/ModuleURI}/deleteflowtransactionsettings/{Flow/flowID}" class="floatright clearboth" onclick="return confirm('{$i18n.DeleteTransactionSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteTransactionSettings" />
				<img class="marginleft" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>
		</xsl:if>

	</xsl:template>

	<xsl:template match="UpdateTransactionSettings">

		<div id="TransactionFlowAdminExtensionModule" class="contentitem errands-wrapper border-box">

			<h1>
				<xsl:value-of select="$i18n.UpdateTransactionSettingsTitle" />
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name" />
			</h1>

			<xsl:apply-templates select="ValidationErrors/validationError" />

			<form method="post" action="{/Document/ModuleURI}/updateflowtransactionsettings/{Flow/flowID}">

				<div class="floatleft full bigmarginbottom">

					<label for="chart" class="floatleft full required">
						<xsl:value-of select="$i18n.Settings.chart" />
					</label>

					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'chart'" />
						<xsl:with-param name="name" select="'chart'" />
						<xsl:with-param name="class" select="'forty'" />
						<xsl:with-param name="element" select="Charts/Chart[Accounts/Account[accountType = 'INCOME' and not(archived = 'true')]]" />
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="valueElementName" select="'chartID'" />
						<xsl:with-param name="selectedValue" select="FlowFamilyTransactionSettings/Account[accountType = 'INCOME']/Chart/chartID" />
						<xsl:with-param name="addEmptyOption" select="$i18n.ChooseChart" />
					</xsl:call-template>
				</div>

				<div class="floatleft full bigmarginbottom">

					<label for="account" class="floatleft full required">
						<xsl:value-of select="$i18n.Settings.account" />
					</label>

					<xsl:call-template name="createAccountDropdown">
						<xsl:with-param name="name" select="'account'" />
						<xsl:with-param name="type" select="'INCOME'" />
						<xsl:with-param name="accounts" select="Charts/Chart/Accounts" />
						<xsl:with-param name="selectedAccount" select="FlowFamilyTransactionSettings/Account[accountType = 'INCOME']/accountID" />
						<xsl:with-param name="requestparameters" select="requestparameters" />
						<xsl:with-param name="class" select="'forty'" />
					</xsl:call-template>
					
				</div>

				<div class="floatleft full bigmarginbottom">

					<label for="vatAccount" class="floatleft full">
						<xsl:value-of select="$i18n.Settings.vatAccount" />
					</label>

					<xsl:call-template name="createAccountDropdown">
						<xsl:with-param name="name" select="'vatAccount'" />
						<xsl:with-param name="type" select="'VAT'" />
						<xsl:with-param name="accounts" select="Charts/Chart/Accounts" />
						<xsl:with-param name="selectedAccount" select="FlowFamilyTransactionSettings/Account[accountType = 'VAT']/accountID" />
						<xsl:with-param name="requestparameters" select="requestparameters" />
						<xsl:with-param name="class" select="'forty'" />
					</xsl:call-template>

				</div>
				
				<div class="floatleft full bigmarginbottom">
				
					<div class="marginbottom">
					
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="'paymentProductSettingsEnabled'" />
							<xsl:with-param name="name" select="'paymentProductSettingsEnabled'" />
							<xsl:with-param name="element" select="FlowFamilyTransactionSettings" />
						</xsl:call-template>
					
						<label for="paymentProductSettingsEnabled">
							<xsl:value-of select="$i18n.Settings.paymentProductSettingsEnabled" />
						</label>
					
					</div>
					
					<div id="paymentProductSettingsContainer">
					
						<p>
							<xsl:value-of select="$i18n.Settings.paymentProductSettingsEnabled.Description" />
						</p>
						
						<table class="clearboth full coloredtable oep-table bigmarginbottom">
							<thead>
								<th></th>
								<th><xsl:value-of select="$i18n.Settings.name" /></th>
								<th><xsl:value-of select="$i18n.Settings.chart" /></th>
								<th><xsl:value-of select="$i18n.Settings.account" /></th>
								<th><xsl:value-of select="$i18n.Settings.vatAccount" /></th>
							</thead>
							<tbody>
								<xsl:apply-templates select="PaymentProducts/PaymentProduct"/>
							</tbody>
						</table>

					</div>
					
				</div>
				
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>

			</form>

		</div>

	</xsl:template>
	
	<xsl:template name="createAccountDropdown">
		<xsl:param name="name" />
		<xsl:param name="type" />
		<xsl:param name="accounts" />
		<xsl:param name="selectedAccount" />
		<xsl:param name="requestparameters" />
		<xsl:param name="class" select="null" />
		<xsl:param name="createTemplateDropdown" select="true()" />
		
		<xsl:variable name="paramValue" select="$requestparameters/parameter[name=$name]/value" />
	
		<select id="{$name}" name="{$name}" class="{$class}">
	
			<xsl:if test="not($createTemplateDropdown)">
				<xsl:attribute name="disabled">true</xsl:attribute>
			</xsl:if>
	
			<option value="">
				<xsl:value-of select="$i18n.ChooseAccount" />
			</option>
	
			<xsl:for-each select="$accounts/Account[accountType = $type and not(archived = 'true')]">
	
				<option value="{accountID}" data-chart="{../../chartID}">
	
					<xsl:choose>
						<xsl:when test="$requestparameters">
							<xsl:if test="accountID = $paramValue">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<xsl:if test="accountID = $selectedAccount">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>
	
					<xsl:value-of select="name" />
				</option>
	
			</xsl:for-each>
			
		</select>
		
		<xsl:if test="$createTemplateDropdown">
		
			<xsl:call-template name="createAccountDropdown">
				<xsl:with-param name="name" select="concat('template-', $name)" />
				<xsl:with-param name="type" select="$type" />
				<xsl:with-param name="accounts" select="$accounts" />
				<xsl:with-param name="selectedAccount" select="$selectedAccount" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="class" select="concat('hidden ', $class)" />
				<xsl:with-param name="createTemplateDropdown" select="false()" />
			</xsl:call-template>
		
		</xsl:if>
		
	</xsl:template>
	
	
	<xsl:template match="PaymentProduct">
	
		<xsl:variable name="description" select="description" />
		<xsl:variable name="settings" select="../../FlowFamilyTransactionSettings/PaymentProductSettingsList/PaymentProductTransactionSettings[description = $description]" />
		<xsl:variable name="charts" select="../../Charts/Chart" />
		<xsl:variable name="requestparameters" select="../../requestparameters" />
	
		<tr>
			<td>
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="concat('paymentProductSettingsEnabled-', position())" />
					<xsl:with-param name="checked" select="boolean($settings)" />
					<xsl:with-param name="requestparameters" select="$requestparameters" />
				</xsl:call-template>
															
				<input type="hidden" name="paymentProductChart" value="{position()}" />				
			</td>
			<td>
				
				<xsl:choose>
					<xsl:when test="description">
					
						<xsl:value-of select="description" />
		
						<xsl:text> (</xsl:text>
						<xsl:choose>
							<xsl:when test="unitPrice > 0">
								<xsl:value-of select="unitPrice" />
								<xsl:if test="../../Currency">
									<xsl:text> </xsl:text>
									<xsl:value-of select="../../Currency" />
								</xsl:if>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$i18n.Settings.UnknownPrice" />
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text>)</xsl:text>
						
					</xsl:when>
					<xsl:otherwise>
					
						<xsl:value-of select="$i18n.Settings.Unknown" />
						<xsl:text> (</xsl:text>
						<xsl:value-of select="QueryName" />
						<xsl:text>)</xsl:text>
						
					</xsl:otherwise>
				</xsl:choose>
				
				
				<input type="hidden" name="{concat('description-', position())}" value="{description}" />				
			</td>
			<td>
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="name" select="concat('chart-', position())" />
					<xsl:with-param name="class" select="'payment-product-chart'" />
					<xsl:with-param name="element" select="$charts[Accounts/Account[accountType = 'INCOME' and not(archived = 'true')]]" />
					<xsl:with-param name="selectedValue" select="$settings/Account[accountType = 'INCOME']/Chart/chartID" />
					<xsl:with-param name="labelElementName" select="'name'" />
					<xsl:with-param name="valueElementName" select="'chartID'" />
					<xsl:with-param name="requestparameters" select="$requestparameters" />
					<xsl:with-param name="addEmptyOption" select="$i18n.ChooseChart" />
				</xsl:call-template>
			</td>
			<td>
				<xsl:call-template name="createAccountDropdown">
					<xsl:with-param name="name" select="concat('account-', position())" />
					<xsl:with-param name="type" select="'INCOME'" />
					<xsl:with-param name="accounts" select="$charts/Accounts" />
					<xsl:with-param name="selectedAccount" select="$settings/Account[accountType = 'INCOME']/accountID" />
					<xsl:with-param name="requestparameters" select="$requestparameters" />
				</xsl:call-template>
			</td>
			<td>
				<xsl:call-template name="createAccountDropdown">
					<xsl:with-param name="name" select="concat('vatAccount-', position())" />
					<xsl:with-param name="type" select="'VAT'" />
					<xsl:with-param name="accounts" select="$charts/Accounts" />
					<xsl:with-param name="selectedAccount" select="$settings/Account[accountType = 'VAT']/accountID" />
					<xsl:with-param name="requestparameters" select="$requestparameters" />
				</xsl:call-template>
			</td>
		</tr>
		
	</xsl:template>

	<xsl:template match="validationError[starts-with(fieldName, 'description-')]">
		<p class="error">
			<xsl:value-of select="$i18n.validation.unknownProducts" />
		</p>
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>

				<xsl:text>&#x20;</xsl:text>

				<xsl:choose>
					<xsl:when test="fieldName = 'chart'">
						<xsl:value-of select="$i18n.Settings.chart" />
					</xsl:when>
					<xsl:when test="fieldName = 'account'">
						<xsl:value-of select="$i18n.Settings.account" />
					</xsl:when>
					<xsl:when test="fieldName = 'vatAccount'">
						<xsl:value-of select="$i18n.Settings.vatAccount" />
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'chart-')">
						<xsl:variable name="id" select="substring-after(fieldName, 'chart-')" />
						<xsl:variable name="descriptionParameterName" select="concat('description-', $id)" />
						
						<xsl:value-of select="../../requestparameters/parameter[name = $descriptionParameterName]/value" />
						<xsl:text> - </xsl:text>
						<xsl:value-of select="$i18n.Settings.chart" />
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'account-')">
						<xsl:variable name="id" select="substring-after(fieldName, 'account-')" />
						<xsl:variable name="descriptionParameterName" select="concat('description-', $id)" />
						
						<xsl:value-of select="../../requestparameters/parameter[name = $descriptionParameterName]/value" />
						<xsl:text> - </xsl:text>
						<xsl:value-of select="$i18n.Settings.account" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName" />
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>

		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>

	</xsl:template>

</xsl:stylesheet>