<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />
	
	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-ui.js?v=1
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/datatables.min.js
		/js/jquery.ui.datepicker-sv.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/datatables.css
		/css/transactions.css
	</xsl:variable>

	<xsl:template match="Document">

		<div id="AccountModule" class="contentitem errands-wrapper">
		
			<xsl:apply-templates select="ListCharts" />
			<xsl:apply-templates select="ListAccounts" />
			<xsl:apply-templates select="ListTransactions" />
		
		</div>

	</xsl:template>
	
	<xsl:template match="ListCharts">
	
		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.Charts" />
			</h1>
		</div>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<p class="floatleft full">
			<xsl:value-of select="$i18n.ListCharts.description"/>
		</p>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Chart.name" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Charts/Chart[not(archived = 'true')]">
						<xsl:apply-templates select="Charts/Chart[not(archived = 'true')]" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="1"><xsl:value-of select="$i18n.NoCharts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<xsl:if test="Charts/Chart[archived = 'true']">
		
			<h2 class="bigpaddingtop"><xsl:value-of select="$i18n.Charts.Archived" /></h2>
		
			<table class="clearboth full coloredtable oep-table bigmarginbottom">
				<thead class="sortable">
					<tr>
						<th class="default-sort">
							<span><xsl:value-of select="$i18n.Chart.name" /></span>
						</th>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="Charts/Chart[archived = 'true']" mode="list" />
				</tbody>
			</table>
		
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="Chart" mode="list">
	
		<tr>
			<td data-title="{$i18n.Chart.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/accounts/{chartID}">
					<xsl:value-of select="name" />
				</a>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="ListAccounts">

		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.ShowChart" />
				<xsl:text>: </xsl:text>
				<xsl:value-of select="Chart/name"/>
			</h1>
		</div>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<p class="floatleft full">
			<xsl:value-of select="$i18n.ShowChart.description"></xsl:value-of>
		</p>
		
		<h2 class="clearboth bigpaddingtop">
			<xsl:value-of select="$i18n.Accounts.Balance"/>
		</h2>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Account.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.number" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/Accounts/Account[accountType = 'BALANCE']">
						<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'BALANCE']" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="3"><xsl:value-of select="$i18n.NoAccounts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<h2 class="clearboth bigpaddingtop">
			<xsl:value-of select="$i18n.Accounts.Income"/>
		</h2>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Account.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.number" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/Accounts/Account[accountType = 'INCOME']">
						<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'INCOME']" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="3"><xsl:value-of select="$i18n.NoAccounts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<h2 class="clearboth bigpaddingtop">
			<xsl:value-of select="$i18n.Accounts.VAT"/>
		</h2>
		
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Account.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.number" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.vat" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/Accounts/Account[accountType = 'VAT']">
						<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'VAT']" mode="list-vat" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="4"><xsl:value-of select="$i18n.NoAccounts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<xsl:call-template name="transactionTable">
			<xsl:with-param name="ajaxURL">
				<xsl:value-of select="/Document/requestinfo/currentURI"/>
				<xsl:text>/</xsl:text>
				<xsl:value-of select="/Document/module/alias"/>
				<xsl:text>/charttransactions/</xsl:text>
				<xsl:value-of select="Chart/chartID"/>
			</xsl:with-param>
			<xsl:with-param name="showAccounts">true</xsl:with-param>
		</xsl:call-template>
		
		<div class="clearboth floatright marginright bigmargintop">
		
			<xsl:if test="TransactionFileExporter">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/exporttransactionfile/{Chart/chartID}" class="export btn btn-green btn-inline">
					<xsl:value-of select="$i18n.ExportTransactionFile" />
				</a>
			</xsl:if>
		
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/exportchartexcel/{Chart/chartID}" class="export btn btn-green btn-inline">
				<xsl:value-of select="$i18n.ExportToExcel" />
			</a>
			
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" class="btn btn-light btn-inline">
				<xsl:value-of select="$i18n.Back" />
			</a>
		</div>
		
		<div class="clearboth floatright marginright bigmargintop">
		</div>

	</xsl:template>
	
	<xsl:template match="Account" mode="list">
	
		<tr>
			<td data-title="{$i18n.Account.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/transactions/{accountID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Account.number}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/transactions/{accountID}">
					<xsl:value-of select="number"/>
				</a>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="Account" mode="list-vat">
	
		<tr>
			<td data-title="{$i18n.Account.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/transactions/{accountID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Account.number}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/transactions/{accountID}">
					<xsl:value-of select="number"/>
				</a>
			</td>
			<td data-title="{$i18n.Account.vat}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/transactions/{accountID}">
					<xsl:value-of select="vat" />
				</a>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="ListTransactions">

		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.ShowAccount" />
				<xsl:text>: </xsl:text>
				<xsl:value-of select="Account/name"/>
			</h1>
		</div>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<h2 class="bigpaddingtop">
			<xsl:value-of select="$i18n.AccountNumberPart"/>
			<xsl:text>: </xsl:text>
			<xsl:value-of select="Account/number"/>
		</h2>
	
		<xsl:call-template name="transactionTable">
			<xsl:with-param name="ajaxURL">
				<xsl:value-of select="/Document/requestinfo/currentURI"/>
				<xsl:text>/</xsl:text>
				<xsl:value-of select="/Document/module/alias"/>
				<xsl:text>/accounttransactions/</xsl:text>
				<xsl:value-of select="Account/accountID"/>
			</xsl:with-param>
		</xsl:call-template>
		
		<div class="clearboth floatright marginright bigmargintop">
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/exportaccountexcel/{Account/accountID}" class="export btn btn-green btn-inline">
				<xsl:value-of select="$i18n.ExportToExcel" />
			</a>
			
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/accounts/{Account/Chart/chartID}" class="btn btn-light btn-inline">
				<xsl:value-of select="$i18n.Back" />
			</a>
		</div>

	</xsl:template>
	
	<xsl:template name="transactionTable">
		<xsl:param name="ajaxURL"/>
		<xsl:param name="showAccounts" select="'false'"/>
	
		<div id="table-description" class="clearboth floatleft bigpaddingtop">
			<h2>
				<xsl:value-of select="$i18n.Transactions"/>
			</h2>
		</div>
		
		<div id="table-searchbuttons" class="floatright bigmarginright">
			
			<div class="floatleft marginright">
				
				<label for="fromDate" class="marginright">
					<xsl:value-of select="$i18n.fromDate" />
				</label>
				
				<xsl:call-template name="createTextField">
						<xsl:with-param name="name" select="'fromDate'" />
						<xsl:with-param name="id" select="'fromDate'" />
						<xsl:with-param name="size" select="10" />
						<xsl:with-param name="type" select="'date'"/>
						<xsl:with-param name="class" select="'width-auto'"/>
						<xsl:with-param name="value" select="DefaultStartDate"/>
				</xsl:call-template>
			</div>
			
			<div class="floatleft bigmarginleft">
				
				<label for="toDate" class="marginright">
					<xsl:value-of select="$i18n.toDate" />
				</label>
				
				<xsl:call-template name="createTextField">
						<xsl:with-param name="name" select="'toDate'" />
						<xsl:with-param name="id" select="'toDate'" />
						<xsl:with-param name="size" select="10" />
						<xsl:with-param name="type" select="'date'"/>
						<xsl:with-param name="class" select="'width-auto'"/>
				</xsl:call-template>
			</div>
		</div>
		
		<table id="transactionstable" class="stripe display">
			<thead>
				<tr>
					<th><xsl:value-of select="$i18n.Column.transactionID" /></th>
					<th><xsl:value-of select="$i18n.Column.account" /></th>
					<th><xsl:value-of select="$i18n.Column.added" /></th>
					<th><xsl:value-of select="$i18n.Column.sourceType" /></th>
					<th><xsl:value-of select="$i18n.Column.sourceID" /></th>
					<th><xsl:value-of select="$i18n.Column.amount" /></th>
					<th><xsl:value-of select="$i18n.Column.user" /></th>
					<th><xsl:value-of select="$i18n.Column.description" /></th>
				</tr>
			</thead>
			<tbody/>
			<tfoot>
				<tr>
					<td/>
					<td/>
					<td/>
					<td/>
					<td/>
					<td/>
					<td/>
					<td/>
				</tr>
			</tfoot>
		</table>
		
		<script type="text/javascript">
			$(function() {
			
				var table = $('#transactionstable').DataTable({
					ajax: {
						url: '<xsl:value-of select="$ajaxURL"/>?from=<xsl:value-of select="DefaultStartDate"/>',
						dataSrc: 'rows',
						beforeSend: function(jqXHR, settings) {
							jqXHR.setRequestHeader('NoLoginRedirect', 'true');
						},
						error: function(jqXHR, textStatus, errorThrown) {
							
							if (jqXHR.status == 403) { // Access denied
								window.location.reload();
								
							} else {
								$('#transactionstable').find("td.dataTables_empty").text("<xsl:value-of select="$i18n.DataTable.fetchError"/>");
								
								if (console != undefined) {
									console.error(errorThrown);
								}
							}
						}
					},
					deferRender: true,
					columns: [
						{ name: "id", sClass: "text-align-right"},
						{ name: "accounts", type: "string"
							<xsl:if test="not($showAccounts = 'true')">, visible: false</xsl:if>
						},
						{ name: "date", type: "date" },
						{ name: "sourceType", type: "string", 
							render: function(data, type, row) {
							
								if (type == "display") {
							
									if (data == "flowinstance") {
										return "<xsl:value-of select="$i18n.Column.sourceType.flowInstance"/>";
									}
								}
								
								return data;
							}
						},
						{ name: "sourceID",},
						{ name: "amount", sClass: "text-align-right", type: "num-fmt2" },
						{ name: "user", type: "string" },
						{ name: "description", type: "string" },
					],
					order: [[ 2, 'desc' ]],
					dom: 'frtipl',
					responsive: false, // If enabled here Ajax loading message is not shown
					bAutoWidth: false,
					processing: false,
					paging: true,
					pagingType: 'simple_numbers',
					pageLength: <xsl:value-of select="PageSize"/>,
					lengthMenu: [ 15, 25, 35, 50, 100, 500, 1000 ],
					language: {
						"decimal":        "<xsl:value-of select="$i18n.DataTable.decimal"/>",
						"emptyTable":     "<xsl:value-of select="$i18n.DataTable.emptyTable"/>",
						"info":           "<xsl:value-of select="$i18n.DataTable.info"/>",
						"infoEmpty":      "<xsl:value-of select="$i18n.DataTable.infoEmpty"/>",
						"infoFiltered":   "<xsl:value-of select="$i18n.DataTable.infoFiltered"/>",
						"infoPostFix":    "<xsl:value-of select="$i18n.DataTable.infoPostFix"/>",
						"thousands":      " ",
						"lengthMenu":     "<xsl:value-of select="$i18n.DataTable.lengthMenu"/>",
						"loadingRecords": "<xsl:value-of select="$i18n.DataTable.loadingRecords"/>",
						"processing":     "<xsl:value-of select="$i18n.DataTable.processing"/>",
						"search":         "<xsl:value-of select="$i18n.DataTable.search"/>",
						"zeroRecords":    "<xsl:value-of select="$i18n.DataTable.zeroRecords"/>",
						"paginate": {
							"first":      "<xsl:value-of select="$i18n.DataTable.paginate.first"/>",
							"last":       "<xsl:value-of select="$i18n.DataTable.paginate.last"/>",
							"next":       "<xsl:value-of select="$i18n.DataTable.paginate.next"/>",
							"previous":   "<xsl:value-of select="$i18n.DataTable.paginate.previous"/>"
						},
						"aria": {
							"sortAscending":  "<xsl:value-of select="$i18n.DataTable.aria.sortAscending"/>",
							"sortDescending": "<xsl:value-of select="$i18n.DataTable.aria.sortDescending"/>"
						}
					},
					stateSave: true,
					stateSaveParams: function (settings, data) {
						data.start = undefined;
//						data.length = undefined;
						data.order = undefined;
						data.search = undefined;
						data.columns = undefined;
						
//						data.custom = {};
					},
					stateLoadParams: function (settings, data) {
					
						if (data != undefined) {
							data.order = undefined;
							data.search = undefined;
							data.columns = undefined;
							
							if (data.custom != undefined) {
							
							}
						}
					},
					preDrawCallback: function(settings) {
					
						// Remove callback
						settings.aoPreDrawCallback.splice(0, 1);
						
						settings.oClasses.sPageButton = "btn btn-inline btn-light";
					},
					initComplete: function(settings, json){
						
						var api = this.api();
						
						// Must load responsive extension after init or else the ajax loading message is not shown
						new $.fn.dataTable.Responsive(api, {
							details: {
								display: $.fn.dataTable.Responsive.display.childRow,
								//type: "column",
								//target: "td.control"
							}
						});
					},
					footerCallback: function(row, data, start, end, display) {
						var api = this.api(), data;
					
						// Remove the formatting to get integer data for summation
						var intVal = function(i) {
							var ret = 0;
							
							if (typeof i === 'string') {
								var tmp = i.replace(',', '.').replace(/[^0-9.]+/g, '');
								ret = Number(tmp);
								
							} else if (typeof i === 'number') {
								ret = i;
							}
							
							return ret;
						};
					
						var amountTotal = api.column("amount:name", {search: 'applied'})
							.data().reduce( function (previousValue, currentValue) {
								return intVal(previousValue) + intVal(currentValue);
							}, 0);
 
						$(api.column("amount:name").footer()).html(amountTotal.toFixed(2).replace(/\B(?=(\d{3})+(?!\d))/g, " ").replace('.', ','));
					},
				});
				
				$("#transactionstable_wrapper").prepend($("#table-description"));
				$("#transactionstable_filter").after($("#table-searchbuttons"));
				
				if (!Modernizr.inputtypes.date) {
					$( "#fromDate" ).datepicker({
						showOn: "button",
						buttonImage: '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics/calendar_grid.png',
						buttonImageOnly: true,
						buttonText: '<xsl:value-of select="$i18n.fromDate"/>',
						changeMonth: true,
						changeYear: true,
						showWeek: true,
					});
					
					$( "#toDate" ).datepicker({
						showOn: "button",
						buttonImage: '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics/calendar_grid.png',
						buttonImageOnly: true,
						buttonText: '<xsl:value-of select="$i18n.toDate"/>',
						changeMonth: true,
						changeYear: true,
						showWeek: true,
					});
				}
				
				/*
				$("#transactionstable").on( 'click auxclick', 'tbody td', function(event) {
				
					var td = $(this);
					var tr = td.parent();
					
					if (tr.children("td").first().hasClass("dataTables_empty")) {
						return;
					}
					
					if (tr.closest("table").hasClass("collapsed") &amp;&amp; td.is(":first-child")) {
						return;
					}
					
					event.stopPropagation();
						
					var id = table.cell(tr, "id:name").data();
					var url = "<xsl:value-of select="/Document/requestinfo/currentURI"/>/<xsl:value-of select="/Document/module/alias"/>/showtransaction/" + id;
					
					if (event.which === 2) {
					
						event.preventDefault();
						
						var newWindow = window.open(url, "_blank");
						newWindow.trigger("blur");
						window.trigger("focus");
						
					} else if (event.which === 1) {
					
						window.location.href = url;
					}
				});
				*/
				
				var fromDate = $("#fromDate");
				var toDate = $("#toDate");
				
				var filterOnDate = function(render) {
					
					//TODO if table already contains requested date don't reload
					
					table.clear();
					table.draw();
					$('#transactionstable').find("td.dataTables_empty").text("<xsl:value-of select="$i18n.DataTable.loadingRecords"/>");
					
					table.ajax.url("<xsl:value-of select="$ajaxURL"/>?from=" + fromDate.val() + "&amp;to=" + toDate.val()).load();
				};
				
				$("#fromDate, #toDate").on('change', filterOnDate);

				$("a.export").on("click", function(event){
					event.preventDefault();
					
					var a = $(this);
					window.location.href = a.attr("href") + "?from=" + fromDate.val() + "&amp;to=" + toDate.val();
				});
			});
		</script>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ChartNotFound']">
		<p class="error">
			<xsl:value-of select="$i18n.Validation.ChartNotFound" />
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'AccountNotFound']">
		<p class="error">
			<xsl:value-of select="$i18n.Validation.AccountNotFound" />
		</p>
	</xsl:template>

	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.Validation.RequiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.Validation.InvalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.Validation.TooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.Validation.TooLong" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.Validation.UnknownError" />
					</xsl:otherwise>
				</xsl:choose>

				<xsl:text>&#x20;</xsl:text>

				<xsl:choose>
					<xsl:when test="fieldName = 'name'">
						<xsl:value-of select="$i18n.Chart.name" />
					</xsl:when>
					<xsl:when test="fieldName = 'vat'">
						<xsl:value-of select="$i18n.Account.vat" />
					</xsl:when>
					<xsl:when test="fieldName = 'from'">
						<xsl:value-of select="$i18n.fromDate" />
					</xsl:when>
					<xsl:when test="fieldName = 'to'">
						<xsl:value-of select="$i18n.toDate" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName" />
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>

		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.Validation.UnknownFault" />
			</p>
		</xsl:if>

	</xsl:template>

</xsl:stylesheet>