package com.nordicpeak.flowengine.queries.personquery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.SigningParty;
import com.nordicpeak.flowengine.interfaces.CitizenIdentifierQueryInstance;
import com.nordicpeak.flowengine.interfaces.ColumnExportableQueryInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.MultiSignQueryinstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;
import com.nordicpeak.personprovider.Person;

@Table(name = "person_query_instances")
@XMLElement
public class PersonQueryInstance extends BaseQueryInstance implements StringValueQueryInstance, ColumnExportableQueryInstance, CitizenIdentifierQueryInstance, FixedAlternativesQueryInstance, MultiSignQueryinstance {

	private static final long serialVersionUID = -2687318822472796809L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(PersonQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private PersonQuery query;

	@DAOManaged
	@XMLElement
	private String firstname;

	@DAOManaged
	@XMLElement
	private String lastname;

	@DAOManaged
	@XMLElement
	private String address;

	@DAOManaged
	@XMLElement
	private String careOf;

	@DAOManaged
	@XMLElement
	private String zipCode;

	@DAOManaged
	@XMLElement
	private String postalAddress;

	@DAOManaged
	@XMLElement
	private String citizenID;

	@DAOManaged
	@XMLElement
	private String email;

	@DAOManaged
	@XMLElement
	private String phone;

	@DAOManaged
	private boolean personNotFound;

	private final Map<String, Person> cache = new HashMap<>();

	public Map<String, Person> getCache() {

		return cache;
	}

	public String getAddress() {

		return address;
	}

	public String getFirstname() {

		return firstname;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public String getZipCode() {

		return zipCode;
	}

	public void setZipCode(String zipCode) {

		this.zipCode = zipCode;
	}

	public String getPostalAddress() {

		return postalAddress;
	}

	public void setPostalAddress(String postalAddress) {

		this.postalAddress = postalAddress;
	}

	public String getCareOf() {

		return careOf;
	}

	public void setCareOf(String careOf) {

		this.careOf = careOf;
	}

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public PersonQuery getQuery() {

		return query;
	}

	public void setQuery(PersonQuery query) {

		this.query = query;
	}

	public boolean isEmpty() {

		return ObjectUtils.isNull(firstname, lastname, address, careOf, zipCode, postalAddress, citizenID);
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		firstname = null;
		lastname = null;
		address = null;
		zipCode = null;
		postalAddress = null;
		careOf = null;
		citizenID = null;
		email = null;
		phone = null;

		if (query.isSetAsAttribute()) {

			resetAttribute(attributeHandler);
		}

		personNotFound = false;

		super.reset(attributeHandler);
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {

		if (!isEmpty()) {

			attributeHandler.setAttribute(query.getAttributeName() + ".firstname", firstname);
			attributeHandler.setAttribute(query.getAttributeName() + ".lastname", lastname);
			attributeHandler.setAttribute(query.getAttributeName() + ".address", address);
			attributeHandler.setAttribute(query.getAttributeName() + ".postalCode", zipCode);
			attributeHandler.setAttribute(query.getAttributeName() + ".postalAddress", postalAddress);
			attributeHandler.setAttribute(query.getAttributeName() + ".careOf", careOf);
			attributeHandler.setAttribute(query.getAttributeName() + ".citizenID", citizenID);

			if (query.isUseForMultipartSigning()) {
				attributeHandler.setAttribute(query.getAttributeName() + ".email", email);
				attributeHandler.setAttribute(query.getAttributeName() + ".phone", phone);
			}
		}
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler) {

		attributeHandler.removeAttribute(query.getAttributeName() + ".firstname");
		attributeHandler.removeAttribute(query.getAttributeName() + ".lastname");
		attributeHandler.removeAttribute(query.getAttributeName() + ".address");
		attributeHandler.removeAttribute(query.getAttributeName() + ".postalCode");
		attributeHandler.removeAttribute(query.getAttributeName() + ".postalAddress");
		attributeHandler.removeAttribute(query.getAttributeName() + ".careOf");
		attributeHandler.removeAttribute(query.getAttributeName() + ".citizenID");

		if (query.isUseForMultipartSigning()) {
			attributeHandler.removeAttribute(query.getAttributeName() + ".email");
			attributeHandler.removeAttribute(query.getAttributeName() + ".phone");
		}
	}

	@Override
	public String toString() {

		return "PersonQueryInstance (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public String getStringValue() {

		StringBuilder builder = new StringBuilder();

		builder.append(firstname + " " + lastname);

		if (address != null) {

			builder.append("\n" + address);
		}

		if (zipCode != null && postalAddress != null) {

			builder.append("\n" + zipCode + " " + postalAddress);
		}

		return builder.toString();
	}

	public String getCitizenID() {

		return citizenID;
	}

	public void setCitizenID(String citizenID) {

		this.citizenID = citizenID;
	}

	@Override
	public List<String> getColumnLabels(QueryHandler queryHandler) {

		PersonQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryInstanceDescriptor().getQueryDescriptor().getQueryTypeID(), PersonQueryProviderModule.class);

		List<String> labels = new ArrayList<>();

		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportFirstName());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportLastName());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportCareOf());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportAddress());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportZipCode());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportPostalAddress());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportCitizenID());

		if (query.isUseForMultipartSigning()) {

			labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportEmail());
			labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportPhone());
		}

		return labels;
	}

	@Override
	public List<String> getColumnValues() {

		List<String> values = new ArrayList<>();

		values.add(getFirstname());
		values.add(getLastname());
		values.add(getCareOf());
		values.add(getAddress());
		values.add(getZipCode());
		values.add(getPostalAddress());
		values.add(getCitizenID());

		if (query.isUseForMultipartSigning()) {

			values.add(email);
			values.add(phone);
		}

		return values;
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewCDATAElement(doc, element, "Firstname", firstname);
		XMLUtils.appendNewCDATAElement(doc, element, "Lastname", lastname);
		XMLUtils.appendNewCDATAElement(doc, element, "CareOf", careOf);
		XMLUtils.appendNewCDATAElement(doc, element, "Address", address);
		XMLUtils.appendNewCDATAElement(doc, element, "ZipCode", zipCode);
		XMLUtils.appendNewCDATAElement(doc, element, "PostalAddress", postalAddress);
		XMLUtils.appendNewCDATAElement(doc, element, "SocialSecurityNumber", citizenID);

		if (query.isUseForMultipartSigning()) {

			XMLUtils.appendNewCDATAElement(doc, element, "Email", email);
			XMLUtils.appendNewCDATAElement(doc, element, "Phone", phone);
		}

		return element;
	}

	public void setPerson(Person person) {

		firstname = person.getFirstname();
		lastname = person.getLastname();
		address = person.getAddress();
		zipCode = person.getZipCode();
		postalAddress = person.getPostalAddress();
		careOf = person.getCareOf();
		citizenID = person.getCitizenIdentifier();
		email = null;
		phone = null;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(HttpServletRequest req, User user, User poster, URIParser uriParser, QueryHandler queryHandler, InstanceRequestMetadata requestMetadata) throws Exception {

		return BaseQueryUtils.getGenericQueryInstanceProvider(this.getClass(), queryHandler, queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID()).getQueryRequestProcessor(this, req, user, poster, uriParser, requestMetadata);
	}

	@Override
	public String getCitizenIdentifier() {

		return citizenID;
	}

	@Override
	public boolean isTestCitizenIdentifier() {

		return false;
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		if (personNotFound) {
			return Collections.singletonList(query.getPersonNotFoundAlternative());
		}

		if (!isEmpty()) {
			return Collections.singletonList(query.getPersonSelectedAlternative());
		}

		return null;
	}

	@Override
	public String getFreeTextAlternativeValue() {

		return null;
	}

	public boolean isPersonNotFound() {

		return personNotFound;
	}

	public void setPersonNotFound(boolean personNotFound) {

		this.personNotFound = personNotFound;
	}

	@Override
	public List<? extends SigningParty> getSigningParties() {

		if (!getQuery().requiresMultipartSigning()) {
			return null;
		}

		if (citizenID == null) {
			return null;
		}

		return Collections.singletonList(new SigningParty(firstname, lastname, email, phone, citizenID, query.isSetMultipartsAsOwners()));
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getPhone() {

		return phone;
	}

	public void setPhone(String phone) {

		this.phone = phone;
	}

}
