package com.nordicpeak.flowengine.queries.plotquery.beans;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basequery.BaseQuery;

@Table(name = "plot_queries")
@XMLElement
public class PlotQuery extends BaseQuery {

	private static final long serialVersionUID = 4783053101162261987L;

	public static final Field INSTANCES_RELATION = ReflectionUtils.getField(PlotQuery.class, "instances");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean requireBookableDate;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<PlotQueryInstance> instances;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return "PlotQuery (queryID: " + queryID + ")";
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public boolean isRequireBookableDate() {

		return requireBookableDate;
	}

	public void setRequireBookableDate(boolean requireBookableDate) {

		this.requireBookableDate = requireBookableDate;
	}

	public List<PlotQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<PlotQueryInstance> instances) {

		this.instances = instances;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	@Override
	public String getXSDTypeName() {

		return "PlotQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		appendFieldDefinition("ObjectID", "xs:positiveInteger", true, doc, sequenceElement);
		appendFieldDefinition("PlotID", "xs:positiveInteger", true, doc, sequenceElement);
		appendFieldDefinition("GUID", "xs:string", true, doc, sequenceElement);
		appendFieldDefinition("Timestamp", "xs:string", true, doc, sequenceElement);
		appendFieldDefinition("PropertyUnitDesignation", "xs:string", true, doc, sequenceElement);
		appendFieldDefinition("Address", "xs:string", true, doc, sequenceElement);
		appendFieldDefinition("Area", "xs:positiveInteger", true, doc, sequenceElement);
		appendFieldDefinition("ConnectionFee", "xs:positiveInteger", true, doc, sequenceElement);
		appendFieldDefinition("Price", "xs:positiveInteger", true, doc, sequenceElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	private void appendFieldDefinition(String name, String type, boolean required, Document doc, Element sequenceElement) {

		Element fieldElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		fieldElement.setAttribute("name", name);
		fieldElement.setAttribute("type", type);
		fieldElement.setAttribute("minOccurs", required ? "1" : "0");
		fieldElement.setAttribute("maxOccurs", "1");

		sequenceElement.appendChild(fieldElement);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		requireBookableDate = xmlParser.getPrimitiveBoolean("requireBookableDate");
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

}
