package com.nordicpeak.flowengine.queries.plotquery.eventmodule.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Timestamp;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

@Table(name = "plot_event_module_events")
@XMLElement
public class PlotEvent extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = -2881372784113729471L;

	public static final Field FIELD_ADDED = ReflectionUtils.getField(PlotEvent.class, "added");

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement
	private Integer plotEventID;

	@DAOManaged
	private Integer userID;

	@DAOManaged
	@XMLElement
	private Integer objectID;

	@DAOManaged
	@XMLElement
	private String propertyUnitDesignation;

	@DAOManaged
	@XMLElement
	private PlotEventType eventType;

	@DAOManaged
	@XMLElement
	private String details;

	@DAOManaged
	@XMLElement
	private Long added;

	@XMLElement
	private User user;

	@XMLElement
	private String formattedAdded;

	@Override
	public String toString() {

		return "(plotEventID=" + plotEventID + ", objectID=" + objectID + ")";
	}

	public PlotEvent() {}

	public PlotEvent(Integer userID, Integer objectID, String propertyUnitDesignation, PlotEventType eventType, String details) {

		this.userID = userID;
		this.objectID = objectID;
		this.propertyUnitDesignation = propertyUnitDesignation;
		this.eventType = eventType;
		this.details = details;
		this.added = System.currentTimeMillis();
	}

	public JsonNode toJSON(UserHandler userHandler) {

		JsonObject json = new JsonObject();

		json.putField("plotEventID", plotEventID);
		json.putField("user", getUserText(userHandler));
		json.putField("objectID", objectID);
		json.putField("propertyUnitDesignation", propertyUnitDesignation);
		json.putField("eventType", eventType);
		json.putField("details", details);
		json.putField("added", getFormattedAdded());

		return json;
	}

	private String getUserText(UserHandler userHandler) {

		User currentUser = getUser(userHandler);

		if (currentUser == null) {

			return userID.toString();
		}

		return currentUser.getFirstname() + " " + currentUser.getLastname() + " (" + currentUser.getUserID() + ")";
	}

	public Integer getPlotEventID() {

		return plotEventID;
	}

	public void setPlotEventID(Integer plotEventID) {

		this.plotEventID = plotEventID;
	}

	public Integer getUserID() {

		return userID;
	}

	public void setUserID(Integer userID) {

		this.userID = userID;
	}

	public Integer getObjectID() {

		return objectID;
	}

	public void setObjectID(Integer objectID) {

		this.objectID = objectID;
	}

	public String getPropertyUnitDesignation() {

		return propertyUnitDesignation;
	}

	public void setPropertyUnitDesignation(String propertyUnitDesignation) {

		this.propertyUnitDesignation = propertyUnitDesignation;
	}

	public PlotEventType getEventType() {

		return eventType;
	}

	public void setEventType(PlotEventType eventType) {

		this.eventType = eventType;
	}

	public String getDetails() {

		return details;
	}

	public void setDetails(String details) {

		this.details = details;
	}

	public Timestamp getAdded() {

		return new Timestamp(added);
	}

	public void setAdded(Timestamp added) {

		this.added = added.getTime();
	}

	public User getUser(UserHandler userHandler) {

		if (user == null) {

			user = userHandler.getUser(userID, false, false);
		}

		return user;
	}

	private String getFormattedAdded() {

		if (formattedAdded == null) {

			formattedAdded = DateUtils.DATE_TIME_SECONDS_MILLISECONDS_FORMATTER.format(getAdded());
		}

		return formattedAdded;
	}

}