var i18nPlotEventModule = {
	"eventType": {
		"LOCKED": "LOCKED",
		"LOCK_REFRESHED": "LOCK_REFRESHED",
		"LOCK_FAILED": "LOCK_FAILED",
		"SUBMITTED": "SUBMITTED",
		"UNKNOWN_ERROR": "UNKNOWN_ERROR"
	},
	"dataTables": {
		"EMPTY_TABLE": "EMPTY_TABLE",
		"NEXT": "NEXT",
		"PREVIOUS": "PREVIOUS",
		"DELETE": "DELETE",
		"CONFIRM_DELETE": "CONFIRM_DELETE",
		"SEARCH": "SEARCH",
		"LENGTH_MENU": "LENGTH_MENU",
		"INFO": "INFO",
		"INFO_EMPTY": "INFO_EMPTY",
		"INFO_FILTERED": "INFO_FILTERED",
		"FETCH_ERROR": "FETCH_ERROR",
		"LOADING_RECORDS": "LOADING_RECORDS"
	},
	"filter": {
		"FROM_DATE": "FROM_DATE",
		"TO_DATE": "TO_DATE"
	}
};

$(function() {

	var $tableElement = $("#plotEvents");

	var dataTable = $tableElement.DataTable({
		"ajax": {
			url: $tableElement.data("plot-events-url"),
			data: function (d) {
				d.fromDate = $("input.from-date").length ? $("input.from-date").val() : getToday();
				d.toDate = $("input.to-date").val();
			},
			beforeSend: function(jqXHR, settings) {

				jqXHR.setRequestHeader('NoLoginRedirect', 'true');
			},
			error: function(jqXHR, textStatus, errorThrown) {

				if (jqXHR.status == 403) { // Access denied

					window.location.reload();

				} else {

					$tableElement.find("td.dataTables_empty").text(i18nPlotEventModule.dataTables.FETCH_ERROR);

					if (console != undefined) {

						console.error(errorThrown);
					}
				}
			}
		},
		"deferRender": true,
		"lengthMenu": [15, 50, 100, 200, 500],
		"order": [
			[4, 'desc']
		],
		"dom": 'rtipl',
		"autoWidth": false,
		"columns": [{
			"data": "objectID"
		}, {
			"data": "propertyUnitDesignation"
		}, {
			"data": "eventType",
			render(data) {

				return i18nPlotEventModule.eventType[data];
			}
		}, {
			"data": "details"
		}, {
			"data": "added"
		}, {
			"data": "user"
		}],
		"language": {
			"emptyTable": i18nPlotEventModule.dataTables.EMPTY_TABLE,
			"info": i18nPlotEventModule.dataTables.INFO,
			"infoEmpty": i18nPlotEventModule.dataTables.INFO_EMPTY,
			"infoFiltered": i18nPlotEventModule.dataTables.INFO_FILTERED,
			"infoPostFix": "",
			"paginate": {
				"next": i18nPlotEventModule.dataTables.NEXT,
				"previous": i18nPlotEventModule.dataTables.PREVIOUS
			},
			"search": i18nPlotEventModule.dataTables.SEARCH,
			"zeroRecords": i18nPlotEventModule.dataTables.ZERO_RECORDS,
			"lengthMenu": i18nPlotEventModule.dataTables.LENGTH_MENU,
			"loadingRecords": i18nPlotEventModule.dataTables.LOADING_RECORDS,

		},
		"preDrawCallback": function(settings) {

			// Remove callback
			settings.aoPreDrawCallback.splice(0, 1);

			settings.oClasses.sPageButton = "btn btn-inline btn-light";
		},
	});

	initFilters(dataTable, $tableElement);
});

function initFilters(dataTable, $tableElement) {

	//Date range filters
	var $dateFilterWrapper = $("<div class='dataTables_filter'>").prependTo($tableElement.closest(".dataTables_wrapper"));
	var $toDateFilterWrapper = $("<div class='date-filter-wrapper'>").appendTo($dateFilterWrapper).append("<strong>" + i18nPlotEventModule.filter.TO_DATE + "</strong>");
	var $fromDateFilterWrapper = $("<div class='date-filter-wrapper'>").appendTo($dateFilterWrapper).append("<strong>" + i18nPlotEventModule.filter.FROM_DATE + "</strong>");
	var $toDateFilter = $("<input class='to-date' type='date'>").appendTo($toDateFilterWrapper);
	var $fromDateFilter = $("<input class='from-date' type='date' value='" + getToday() + "'>").appendTo($fromDateFilterWrapper);
	
	$fromDateFilter.add($toDateFilter).on("change", function() {

		dataTable.ajax.reload();
	});

	//Column filters
	var $tableHead = $tableElement.find("thead");
	var $filterRow = $("<tr class='filter-row'>").appendTo($tableHead);

	$tableHead.find("th").each(function(i, header) {

		var placeHolder = i18nPlotEventModule.dataTables.SEARCH + " " + header.innerText.toLowerCase();

		var $filter = $("<input type='text' placeholder='" + placeHolder + "' data-index='" + i + "' />").appendTo($("<th>").appendTo($filterRow));

		$filter.on("keyup", function() {

			dataTable.column($(this).data("index"))
				.search(this.value, false, false)
				.draw();
		});
	});
}

function getToday() {
	
	return new Date().toISOString().split("T")[0];
}
