package com.nordicpeak.flowengine.queries.predefineduserquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleAttribute;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.handlers.GroupHandler;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.UserUtils;
import se.unlogic.hierarchy.core.utils.usergrouplist.UserGroupListConnector;
import se.unlogic.hierarchy.foregroundmodules.usersessionadmin.UserNameComparator;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.Stringyfier;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MultiSigningQueryProvider;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesAttributeDescriptor;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesConstants;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesSelectedAttribute;
import com.nordicpeak.flowengine.queries.internaluserqueries.interfaces.CommonInternalUserQueryProvider;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.ManagerAttributeValue;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.NoCitizenIDValidationError;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.PredefinedUserQuery;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.PredefinedUserQueryInstance;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.SelectedManagerAttribute;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.SelectedUserAttribute;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.UserAttributeValue;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class PredefinedUserQueryProviderModule extends BaseQueryProviderModule<PredefinedUserQueryInstance> implements BaseQueryCRUDCallback, MultiSigningQueryProvider, CommonInternalUserQueryProvider {

	@XSLVariable(prefix = "java.")
	private String exportFirstName;

	@XSLVariable(prefix = "java.")
	private String exportLastName;

	@XSLVariable(prefix = "java.")
	private String exportUsername;

	@XSLVariable(prefix = "java.")
	private String exportEmail;

	@XSLVariable(prefix = "java.")
	private String exportCitizenID;

	@XSLVariable(prefix = "java.")
	private String exportManager;

	@ModuleSetting
	@SplitOnLineBreak
	@TextAreaSettingDescriptor(name = "Supported attributes", description = "The format is name:[display_name[:formatterClass]] (without brackets). Only the name is required")
	private List<String> supportedAttributes;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "Username formatter", description = "formatter to use for usernames")
	private String usernameFormatter;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable closest manager", description = "Enable info and multisign support for posters closest manager")
	private boolean enableClosestManager;

	private AnnotatedDAO<PredefinedUserQuery> queryDAO;
	private AnnotatedDAO<PredefinedUserQueryInstance> queryInstanceDAO;

	private PredefinedUserQueryCRUD queryCRUD;

	private QueryParameterFactory<PredefinedUserQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<PredefinedUserQueryInstance, Integer> queryInstanceIDParamFactory;

	protected UserGroupListConnector userGroupListConnector;

	private List<InternalUserQueriesAttributeDescriptor> attributeDescriptors;

	protected Stringyfier<String> usernameStringFormatter;
	protected Map<String, Stringyfier<String>> attributeStringFormatter = new HashMap<String, Stringyfier<String>>();

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, PredefinedUserQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());

			if (upgradeResult.getInitialVersion() < 5 && upgradeResult.getCurrentVersion() >= 5) {

				List<String> backwardsCompatibilityAttributes = moduleDescriptor.getMutableSettingHandler().getStrings("supportedAttributes");

				if (backwardsCompatibilityAttributes == null) {
					backwardsCompatibilityAttributes = new ArrayList<>();
				}

				backwardsCompatibilityAttributes.add(0, "department:Avdelning");
				backwardsCompatibilityAttributes.add(0, "title:Titel");
				backwardsCompatibilityAttributes.add("mobilePhone:Mobiltelefon");

				moduleDescriptor.getMutableSettingHandler().setSetting("supportedAttributes", backwardsCompatibilityAttributes);
				moduleDescriptor.saveSettings(systemInterface);
			}
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(PredefinedUserQuery.class);
		queryInstanceDAO = daoFactory.getDAO(PredefinedUserQueryInstance.class);

		queryCRUD = new PredefinedUserQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<PredefinedUserQuery>(PredefinedUserQuery.class), "PredefinedUserQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);

		userGroupListConnector = new UserGroupListConnector(systemInterface);
	}

	@Override
	protected synchronized void moduleConfigured() throws Exception {

		super.moduleConfigured();

		if (ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR)) {

			attributeDescriptors = parseAttributes();

		} else {

			attributeDescriptors = null;
		}
	}

	private List<InternalUserQueriesAttributeDescriptor> parseAttributes() {

		usernameStringFormatter = null;

		if (usernameFormatter != null) {
			try {
				@SuppressWarnings("unchecked")
				Stringyfier<String> stringFormatter2 = (Stringyfier<String>) ReflectionUtils.getInstance(usernameFormatter);
				usernameStringFormatter = stringFormatter2;
			} catch (Exception e) {
				log.error("Unable to find stringformatter for username.");
			}
		}

		attributeStringFormatter.clear();
		List<InternalUserQueriesAttributeDescriptor> attributeDescriptors = new ArrayList<InternalUserQueriesAttributeDescriptor>(supportedAttributes == null ? 0 : supportedAttributes.size());

		if (supportedAttributes != null) {
			for (String attribute : supportedAttributes) {

				String[] attributeParts = attribute.replace("\r", "").split(":");

				if (attributeParts.length == 0 || attributeParts.length > 3) {

					log.warn("Unable to parse format of attribute: " + attribute + ", ignoring attribute.");
					continue;
				}

				String name = attributeParts[0];;
				String displayName;
				Stringyfier<String> stringFormatter = null;

				if (attributeParts.length > 1) {

					displayName = attributeParts[1];

				} else {

					displayName = null;
				}

				if (attributeParts.length >= 3) {

					try {
						@SuppressWarnings("unchecked")
						Stringyfier<String> stringFormatter2 = (Stringyfier<String>) ReflectionUtils.getInstance(attributeParts[2]);
						stringFormatter = stringFormatter2;

						attributeStringFormatter.put(name, stringFormatter);
					} catch (Exception e) {
						log.error("Unable to find stringformatter for attribute: " + attribute + ", adding without formatter.");
					}
				}

				attributeDescriptors.add(new InternalUserQueriesAttributeDescriptor(name, displayName, stringFormatter));
			}
		}

		if (attributeDescriptors.isEmpty()) {

			return null;

		} else {

			return attributeDescriptors;
		}
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PredefinedUserQuery query = new PredefinedUserQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		PredefinedUserQuery query = new PredefinedUserQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		if (query.getSelectedUserAttributes() != null) {

			if (getAttributeDescriptors() == null) {

				query.setSelectedUserAttributes(null);

			} else {

				List<SelectedUserAttribute> refreshedSelectedAttributes = new ArrayList<>(query.getSelectedUserAttributes().size());

				for (SelectedUserAttribute selectedAttribute : query.getSelectedUserAttributes()) {

					InternalUserQueriesAttributeDescriptor attributeDescriptor = getAttributeDescriptor(selectedAttribute);

					if (attributeDescriptor != null) {

						refreshedSelectedAttributes.add(new SelectedUserAttribute(attributeDescriptor.getName(), attributeDescriptor.getDisplayName(), selectedAttribute.getSortIndex()));
					}
				}

				query.setSelectedUserAttributes(refreshedSelectedAttributes);
			}
		}

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	public List<InternalUserQueriesAttributeDescriptor> getAttributeDescriptors() {
		return attributeDescriptors;
	}

	private InternalUserQueriesAttributeDescriptor getAttributeDescriptor(SelectedUserAttribute selectedAttribute) {

		for (InternalUserQueriesAttributeDescriptor attributeDescriptor : attributeDescriptors) {
			if (attributeDescriptor.getName().equalsIgnoreCase(selectedAttribute.getName())) {
				return attributeDescriptor;
			}
		}

		return null;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		PredefinedUserQuery query = this.getQuery(descriptor.getQueryID(), extraData);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PredefinedUserQuery query = this.getQuery(descriptor.getQueryID(), false, transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		PredefinedUserQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new PredefinedUserQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID(), true));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (queryInstance.getQuery().isLimitSelectionToMyEmployees() && queryInstance.getQuery().getSelectableUsers() != null) {

			if (poster == null || poster.getUsername() == null) {

				queryInstance.getQuery().setSelectableUsers(null);
				
			} else {

				String username = poster.getUsername();
				Iterator<User> it = queryInstance.getQuery().getSelectableUsers().iterator();

				while (it.hasNext()) {

					if (!username.equals(it.next().getAttributeHandler().getString(InternalUserQueriesConstants.USER_MANAGER_USERNAME_ATTRIBUTE))) {
						it.remove();
					}
				}
			}
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	@Override
	public ImmutableQueryInstance getImmutableQueryInstance(MutableQueryInstanceDescriptor descriptor, HttpServletRequest req, InstanceMetadata instanceMetadata) throws Throwable {

		if (descriptor.getQueryInstanceID() == null) {
			return null;
		}

		PredefinedUserQueryInstance queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {
			return null;
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID(), false));

		if (queryInstance.getQuery() == null) {
			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	public List<User> getSelectableUsers(PredefinedUserQuery query) {

		Set<User> users = new HashSet<User>();

		if (query.getUserIDs() != null) {
			CollectionUtils.add(users, systemInterface.getUserHandler().getUsers(query.getUserIDs(), false, true));
		}

		if (query.getGroupIDs() != null) {
			CollectionUtils.add(users, systemInterface.getUserHandler().getUsersByGroups(query.getGroupIDs(), true));
		}

		if(query.isExcludeInactiveUsers()) {
			
			Iterator<User> iterator = users.iterator();
			
			while (iterator.hasNext()) {

				if (!iterator.next().isEnabled()) {
					iterator.remove();
				}
			}
		}
		
		List<User> sortedUsers = new ArrayList<User>(users);
		Collections.sort(sortedUsers, UserNameComparator.getInstance());

		return sortedUsers;
	}

	public Map<String, User> getSelectableManagers(PredefinedUserQuery query) {
		
		if (query.getSelectableUsers() == null) {
			return null;
		}

		Set<String> managerUsernames = new HashSet<>();

		for (User user : query.getSelectableUsers()) {

			String managerUsername = user.getAttributeHandler().getString(InternalUserQueriesConstants.USER_MANAGER_USERNAME_ATTRIBUTE);

			if (managerUsername != null) {

				managerUsernames.add(managerUsername);
			}
		}
		
		if (managerUsernames.isEmpty()) {
			return null;
		}

		List<User> managers = systemInterface.getUserHandler().getUsersByUsername(managerUsernames, false, true);
		
		return UserUtils.getUsernameMap(managers);
	}

	private PredefinedUserQuery getQuery(Integer queryID, boolean withAlternatives) throws SQLException {

		HighLevelQuery<PredefinedUserQuery> getQuery = new HighLevelQuery<PredefinedUserQuery>();

		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));

		PredefinedUserQuery query = queryDAO.get(getQuery);

		if (query != null && withAlternatives) {

			query.setSelectableUsers(getSelectableUsers(query));

			if (this.enableClosestManager && (query.isShowManagerInfo() || query.isRequireManagerSigning())) {

				query.setSelectableManagers(getSelectableManagers(query));
			}
		}

		return query;
	}

	private PredefinedUserQuery getQuery(Integer queryID, boolean withAlternatives, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<PredefinedUserQuery> getQuery = new HighLevelQuery<PredefinedUserQuery>();

		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));

		PredefinedUserQuery query = queryDAO.get(getQuery, transactionHandler);

		if (query != null && withAlternatives) {

			query.setSelectableUsers(getSelectableUsers(query));

			if (this.enableClosestManager && (query.isShowManagerInfo() || query.isRequireManagerSigning())) {

				query.setSelectableManagers(getSelectableManagers(query));
			}
		}

		return query;
	}

	private PredefinedUserQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<PredefinedUserQueryInstance> query = new HighLevelQuery<PredefinedUserQueryInstance>(PredefinedUserQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(PredefinedUserQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public QueryResponse getFormHTML(PredefinedUserQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		Document doc = createDocument(req, poster);
		
		if (queryInstance.getQuery().isMaskCitizenIDLastFourDigits()) {

			XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
			
			genDoc.addAssignableRootElementableListener(SimpleAttribute.class, CitizenIdentifierUtils.getMaskCitizenIdentifierAttributeElementableListener());
			
			doc = genDoc;
		}
		
		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler, doc);
	}
	
	@Override
	protected void appendQueryInstance(PredefinedUserQueryInstance queryInstance, Document doc, Element targetElement, AttributeHandler attributeHandler) {

		super.appendQueryInstance(queryInstance, doc, targetElement, attributeHandler);

		if (queryInstance.getQuery().isLockOnOwnershipTransfer() && attributeHandler.getPrimitiveBoolean("OwnershipTransfered")) {

			XMLUtils.appendNewElement(doc, targetElement, "Locked", "true");
		}
	}

	@Override
	public void populate(PredefinedUserQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		PredefinedUserQuery query = queryInstance.getQuery();
		Integer queryID = query.getQueryID();

		if (query.isLockOnOwnershipTransfer() && attributeHandler.getPrimitiveBoolean("OwnershipTransfered")) {
			return;
		}

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		String userFieldName = "q" + queryID + "_userID";

		Integer userID = ValidationUtils.validateParameter(userFieldName, req, !allowPartialPopulation && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED, IntegerPopulator.getPopulator(), validationErrors);
		User selectedUser = null;

		if (userID != null) {

			for (User selectableUser : query.getSelectableUsers()) {

				if (selectableUser.getUserID().equals(userID)) {

					selectedUser = selectableUser;
					break;
				}
			}

			if (selectedUser == null) {

				validationErrors.add(new ValidationError(userFieldName, ValidationErrorType.InvalidFormat));
				
			} else if (query.isLimitSelectionToMyEmployees() && (poster == null || poster.getUsername() == null || !poster.getUsername().equals(selectedUser.getAttributeHandler().getString(InternalUserQueriesConstants.USER_MANAGER_USERNAME_ATTRIBUTE)))) {

				validationErrors.add(new ValidationError(userFieldName, ValidationErrorType.InvalidFormat));

			} else if(query.isExcludeInactiveUsers() && !selectedUser.isEnabled()) {
				
				validationErrors.add(new ValidationError("InactiveAccountSelected"));
				
			} else {

				queryInstance.reset(null);
				populateQueryInstance(queryInstance, selectedUser);

				if (allowPartialPopulation) {

					queryInstance.getQueryInstanceDescriptor().setPopulated(true);

					if (queryInstance.getQuery().isSetAsAttribute()) {

						queryInstance.resetAttributes(attributeHandler);
						queryInstance.setAttributes(attributeHandler);
					}
					
					return;
				}

				if (query.requiresMultipartSigning() && queryInstance.getCitizenIdentifier() == null) {

					validationErrors.add(new NoCitizenIDValidationError(selectedUser));
				}

				if(this.enableClosestManager) {
					
					if ((query.isShowManagerInfo() || query.isRequireManagerSigning()) && !queryInstance.isManagerPopulated()) {
						
						validationErrors.add(new ValidationError("UnableToGetManagerDetails"));
						
					} else if (query.isRequireManagerSigning() && (queryInstance.getManagerCitizenID() == null || (queryInstance.getManagerEmail() == null && queryInstance.getManagerMobilePhone() == null))) {
						
						validationErrors.add(new ValidationError("ManagerDoesNotHaveRequiredFieldsForSigning"));
					}
				}
			}
		}

		if (selectedUser == null) {
			queryInstance.reset(attributeHandler);
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		if (selectedUser == null) {

			queryInstance.getQueryInstanceDescriptor().setPopulated(false);

		} else {

			queryInstance.getQueryInstanceDescriptor().setPopulated(true);

			if (queryInstance.getQuery().isSetAsAttribute()) {

				queryInstance.resetAttributes(attributeHandler);
				queryInstance.setAttributes(attributeHandler);
			}
		}
	}

	private void populateQueryInstance(PredefinedUserQueryInstance queryInstance, User user) {

		PredefinedUserQuery query = queryInstance.getQuery();

		queryInstance.setUserID(user.getUserID());
		queryInstance.setCitizenIdentifier(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user));

		if (query.isShowName()) {
			queryInstance.setFirstname(user.getFirstname());
			queryInstance.setLastname(user.getLastname());
		}

		if (query.isShowUsername()) {
			queryInstance.setUsername(formatUsername(user.getUsername()));
		}

		if (query.isShowEmail() || query.isUseForMultipartSigning()) {
			queryInstance.setEmail(user.getEmail());
		}

		AttributeHandler attributeHandler = user.getAttributeHandler();

		if (attributeHandler != null) {

			if (query.isUseForMultipartSigning()) {

				queryInstance.setPhone(attributeHandler.getString("mobilePhone"));
			}

			queryInstance.setUserAttributeValues(new ArrayList<UserAttributeValue>());

			if (query.getSelectedUserAttributes() != null) {

				for (SelectedUserAttribute selectedAttribute : query.getSelectedUserAttributes()) {

					String value = attributeHandler.getString(selectedAttribute.getName());

					if (value != null) {

						queryInstance.getUserAttributeValues().add(new UserAttributeValue(selectedAttribute.getName(), formatAttribute(selectedAttribute, value)));
					}
				}
			}
		}

		if (this.enableClosestManager && (query.isShowManagerInfo() || query.isRequireManagerSigning())) {

			String managerUsername = user.getAttributeHandler().getString(InternalUserQueriesConstants.USER_MANAGER_USERNAME_ATTRIBUTE);

			if (managerUsername != null) {

				User manager = systemInterface.getUserHandler().getUserByUsername(managerUsername, false, true);

				if (manager != null) {

					queryInstance.setManagerUsername(managerUsername);
					queryInstance.setManagerCitizenID(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(manager));
					queryInstance.setManagerEmail(manager.getEmail());
					queryInstance.setManagerFirstname(manager.getFirstname());
					queryInstance.setManagerLastname(manager.getLastname());
					queryInstance.setManagerMobilePhone(manager.getAttributeHandler().getString("mobilePhone"));

					if (query.isShowManagerInfo() && query.getSelectedManagerAttributes() != null) {

						queryInstance.setManagerAttributeValues(new ArrayList<ManagerAttributeValue>());

						for (SelectedManagerAttribute selectedAttribute : queryInstance.getQuery().getSelectedManagerAttributes()) {

							String value = manager.getAttributeHandler().getString(selectedAttribute.getName());

							if (value != null) {
								queryInstance.getManagerAttributeValues().add(new ManagerAttributeValue(selectedAttribute.getName(), formatAttribute(selectedAttribute, value)));
							}
						}
					}
				}
			}
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PredefinedUserQuery query = getQuery(descriptor.getQueryID(), false);

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PredefinedUserQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		PredefinedUserQuery query = getQuery(sourceQueryDescriptor.getQueryID(), false, transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, PredefinedUserQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}

		XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "StaticContent", "classpath://" + getClass().getPackage().getName().replace('.', '/') + "/staticcontent");
	}

	@Override
	protected Class<PredefinedUserQueryInstance> getQueryInstanceClass() {

		return PredefinedUserQueryInstance.class;
	}

	@Override
	public List<Integer> getQueryInstanceIDs(String citizenIdentifier) throws SQLException {

		ArrayListQuery<Integer> query = new ArrayListQuery<Integer>(queryInstanceDAO.getDataSource(), "SELECT queryInstanceID FROM " + queryInstanceDAO.getTableName() + " WHERE citizenIdentifier = ?", IntegerPopulator.getPopulator());
		query.setString(1, citizenIdentifier);

		return query.executeQuery();
	}

	public UserHandler getUserHandler() {
		return systemInterface.getUserHandler();
	}

	public GroupHandler getGroupHandler() {
		return systemInterface.getGroupHandler();
	}

	@WebPublic(alias = "users")
	public ForegroundModuleResponse getUsers(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if(this.flowAdminModule == null || !AccessUtils.checkAccess(user, flowAdminModule.getModuleDescriptor())) {
			
			throw new AccessDeniedException("User does not have access to list users via this module");
		}
		
		return userGroupListConnector.getUsers(req, res, user, uriParser);
	}

	@WebPublic(alias = "groups")
	public ForegroundModuleResponse getGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if(this.flowAdminModule == null || !AccessUtils.checkAccess(user, flowAdminModule.getModuleDescriptor())) {
			
			throw new AccessDeniedException("User does not have access to list groups via this module");
		}
		
		return userGroupListConnector.getGroups(req, res, user, uriParser);
	}

	public String formatUsername(String username) {

		if (usernameStringFormatter != null) {
			return usernameStringFormatter.format(username);
		}

		return username;
	}

	public String formatAttribute(InternalUserQueriesSelectedAttribute selectedAttribute, String value) {

		Stringyfier<String> formatter = attributeStringFormatter.get(selectedAttribute.getName());

		if (formatter != null) {
			return formatter.format(value);
		}

		return value;
	}

	@Override
	public String getExportFirstName() {
		return exportFirstName;
	}

	@Override
	public String getExportLastName() {
		return exportLastName;
	}

	@Override
	public String getExportUsername() {
		return exportUsername;
	}

	@Override
	public String getExportEmail() {
		return exportEmail;
	}

	public String getExportCitizenID() {
		return exportCitizenID;
	}

	@Override
	public String getExportManager() {

		return exportManager;
	}

	public boolean isEnableClosestManager() {
		return enableClosestManager;
	}

}
