package com.nordicpeak.flowengine.queries.pricetablequery;

import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URL;
import java.sql.SQLException;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xsl.XSLVariableReader;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableQuery;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableQueryColumnDummy;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableQueryInstance;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableRow;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableRowValue;
import com.nordicpeak.flowengine.queries.pricetablequery.enums.DecimalSetting;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class PriceTableQueryProviderModule extends BaseQueryProviderModule<PriceTableQueryInstance> {

	public static DecimalFormatSymbols DEFAULT_DECIMAL_FORMAT_SYMBOLS = new DecimalFormatSymbols(Locale.getDefault(Locale.Category.FORMAT));

	@XSLVariable(prefix = "java.")
	private String exportSum;
	
	@XSLVariable(prefix = "java.")
	private String exportCount;

	@XSLVariable(prefix = "java.")
	private String exportAmount;

	@XSLVariable(prefix = "java.")
	private String exportComment;

	private String decimalsDisallowedError;

	private String tooManyDecimalsErrorPart1;

	private String tooManyDecimalsErrorPart2;

	private String tooManyDecimalsErrorPart3;

	protected RelationQuery SAVE_QUERY_INSTANCE_RELATION_QUERY;

	protected AnnotatedDAO<PriceTableQuery> queryDAO;
	protected AnnotatedDAO<PriceTableQueryInstance> queryInstanceDAO;

	protected QueryParameterFactory<PriceTableQuery, Integer> queryIDParamFactory;
	protected QueryParameterFactory<PriceTableQueryInstance, Integer> queryInstanceIDParamFactory;

	protected PriceTableQueryCRUD queryCRUD;
	private PriceTableRowCRUD rowCRUD;

	private PriceTableQueryColumnDummy sumColumn;
	private PriceTableQueryColumnDummy countColumn;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		SAVE_QUERY_INSTANCE_RELATION_QUERY = getSaveQueryInstanceRelationQuery();

		sumColumn = new PriceTableQueryColumnDummy(1, exportSum);
		countColumn = new PriceTableQueryColumnDummy(2, exportCount);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, PriceTableQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(PriceTableQuery.class);
		queryInstanceDAO = daoFactory.getDAO(PriceTableQueryInstance.class);

		queryCRUD = new PriceTableQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<>(PriceTableQuery.class), "PriceTableQuery", "query", null, this);
		rowCRUD = new PriceTableRowCRUD(daoFactory.getDAO(PriceTableRow.class).getWrapper(Integer.class), new AnnotatedRequestPopulator<>(PriceTableRow.class), "PriceTableRow", "pricetablerow", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public PriceTableQuery createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PriceTableQuery query = new PriceTableQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setDummyColumns(sumColumn, countColumn);

		return query;
	}

	@Override
	public PriceTableQuery importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		PriceTableQuery query = new PriceTableQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public PriceTableQuery getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		PriceTableQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setDummyColumns(sumColumn, countColumn);

		return query;
	}

	@Override
	public PriceTableQuery getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PriceTableQuery query = getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		query.setDummyColumns(sumColumn, countColumn);

		return query;
	}

	@Override
	public PriceTableQueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		PriceTableQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new PriceTableQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;

		} else if (!CollectionUtils.isEmpty(queryInstance.getQuery().getRows())) {

			for (PriceTableRow row : queryInstance.getQuery().getRows()) {

				row.setQuery(queryInstance.getQuery());
			}

			if (!CollectionUtils.isEmpty(queryInstance.getValues())) {

				for (PriceTableRowValue rowValue : queryInstance.getValues()) {

					rowValue.getRow().setQuery(queryInstance.getQuery());
				}
			}
		}

		queryInstance.getQuery().setDummyColumns(sumColumn, countColumn);

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		calculateQueryInstanceSummary(queryInstance, queryInstance.getQuery().getRows());

		return queryInstance;
	}

	protected PriceTableQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<PriceTableQuery> query = new HighLevelQuery<>(getTableQueryRelations());

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	protected PriceTableQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<PriceTableQuery> query = new HighLevelQuery<>(getTableQueryRelations());

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	protected PriceTableQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<PriceTableQueryInstance> query = new HighLevelQuery<>(getQueryInstanceRelations());

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(PriceTableQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
		}
	}

	public static String formatBigDecimal(BigDecimal bigDecimal, DecimalSetting decimalSetting) {

		return formatBigDecimal(bigDecimal, 2, decimalSetting);
	}

	public static String formatBigDecimal(BigDecimal bigDecimal, int wantedDecimals, DecimalSetting decimalSetting) {

		if (decimalSetting == DecimalSetting.FORCE) {

			if (bigDecimal.scale() < wantedDecimals) {

				bigDecimal = bigDecimal.setScale(wantedDecimals);
			}

		} else if (decimalSetting == DecimalSetting.DISALLOW) {

			bigDecimal = bigDecimal.setScale(0, RoundingMode.FLOOR);

		} else if (bigDecimal.compareTo(BigDecimal.ZERO) == 0) {

			bigDecimal = bigDecimal.setScale(0);

		} else {

			bigDecimal = bigDecimal.stripTrailingZeros();

			if (bigDecimal.scale() < 0) {

				bigDecimal = bigDecimal.setScale(0);

			} else if (bigDecimal.scale() > 0 && bigDecimal.scale() < wantedDecimals) {

				bigDecimal = bigDecimal.setScale(wantedDecimals);
			}
		}

		String output = bigDecimal.toPlainString();

		if (DEFAULT_DECIMAL_FORMAT_SYMBOLS.getDecimalSeparator() != '.') {

			output = output.replace('.', DEFAULT_DECIMAL_FORMAT_SYMBOLS.getDecimalSeparator());
		}

		return output;
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(alias = "updatequery")
	public ForegroundModuleResponse updateQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		PriceTableQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		PriceTableQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	public void checkUpdateQueryAccess(User user, PriceTableQuery query) throws AccessDeniedException, SQLException {

		flowAdminModule.checkFlowStructureManipulationAccess(user, (Flow) query.getQueryDescriptor().getStep().getFlow());

	}

	public void redirectToQueryConfig(PriceTableQuery query, HttpServletRequest req, HttpServletResponse res) throws IOException {

		res.sendRedirect(req.getContextPath() + this.getFullAlias() + "/config/" + query.getQueryID());

	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, PriceTableQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	public void populate(PriceTableQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		List<PriceTableRow> rows = queryInstance.getQuery().getRows();

		if (CollectionUtils.isEmpty(rows)) {

			queryInstance.reset(attributeHandler);
			return;
		}

		List<ValidationError> validationErrors = new ArrayList<>(rows.size());

		List<PriceTableRowValue> cellValues = new ArrayList<>(rows.size());

		for (PriceTableRow row : rows) {

			populateRow(req, cellValues, row, queryInstance, allowPartialPopulation, validationErrors);
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		if (cellValues.isEmpty()) {

			queryInstance.reset(attributeHandler);

		} else {

			queryInstance.setValues(cellValues);
			queryInstance.getQueryInstanceDescriptor().setPopulated(true);

			if (queryInstance.getQuery().isSetAsAttribute()) {

				queryInstance.setAttribute(attributeHandler);
			}

			calculateQueryInstanceSummary(queryInstance, rows);
		}
	}

	protected void populateRow(HttpServletRequest req, List<PriceTableRowValue> cellValues, PriceTableRow row, PriceTableQueryInstance queryInstance, boolean allowPartialPopulation, List<ValidationError> validationErrors) {

		final String parameterName = "q" + queryInstance.getQuery().getQueryID() + "_" + row.getRowIdentifier();

		String amount = req.getParameter(parameterName);

		if (StringUtils.isEmpty(amount)) {

			if (!allowPartialPopulation && row.isRequired() && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED) {

				validationErrors.add(new ValidationError(parameterName, ValidationErrorType.RequiredField));
			}

			return;
		}

		amount = amount.trim().replace(',', '.');

		PriceTableRowValue cellValue;

		try {
			BigDecimal bigDecimalValue = new BigDecimal(amount);

			validateBigDecimalSize(queryInstance.getQuery().getDecimalSetting(), validationErrors, parameterName, bigDecimalValue);

			if (bigDecimalValue.compareTo(BigDecimal.ZERO) == -1 && !queryInstance.getQuery().isAllowNegative()) {

				validationErrors.add(new ValidationError("AmountMayNotBeNegative", "", parameterName));
			}

			cellValue = new PriceTableRowValue(row, bigDecimalValue);

		} catch (NumberFormatException ignore) {

			validationErrors.add(new ValidationError(parameterName, ValidationErrorType.InvalidFormat));

			cellValue = new PriceTableRowValue(row, null);
		}

		cellValue.setComment(ValidationUtils.validateParameter(parameterName + "_comment", req, false, 0, 255, validationErrors));

		if (CollectionUtils.isEmpty(validationErrors)) {
			cellValues.add(cellValue);
		}
	}

	public void validateBigDecimalSize(DecimalSetting decimalSetting, List<ValidationError> validationErrors, final String parameterName, BigDecimal bigDecimalValue) {

		int maxScale = decimalSetting == DecimalSetting.DISALLOW ? 0 : 2;
		
		String tooManyDecimalsErrorMessage = maxScale == 0 ? decimalsDisallowedError : tooManyDecimalsErrorPart1 + bigDecimalValue.scale() + tooManyDecimalsErrorPart2 + maxScale + tooManyDecimalsErrorPart3;
		
		ValidationUtils.validateBigDecimalSize(parameterName, bigDecimalValue, 10, maxScale, validationErrors, tooManyDecimalsErrorMessage);
	}

	public void calculateQueryInstanceSummary(PriceTableQueryInstance queryInstance, List<PriceTableRow> rows) {

		if (CollectionUtils.isEmpty(rows) || CollectionUtils.isEmpty(queryInstance.getValues())) {
			return;
		}

		BigDecimal sum = new BigDecimal(0);
		BigDecimal count = new BigDecimal(0);

		for (PriceTableRow row : rows) {

			PriceTableRowValue rowValue = queryInstance.getRowValue(row.getRowIdentifier());

			if (rowValue != null) {

				if (rowValue.getValue() != null) {

					BigDecimal rowSum = row.getPrice().multiply(rowValue.getValue());

					if (sum != null) {
						sum = sum.add(rowSum);
					}

					if (rowSum.scale() > 2) {
						rowSum = rowSum.setScale(2, RoundingMode.UP);
					}

					row.setSum(formatBigDecimal(rowSum, queryInstance.getQuery().getDecimalSetting()));
					
					count = count.add(rowValue.getValue());

				} else {

					row.setSum("?");
					sum = null;
					count = null;
				}

			} else {

				row.setSum("");
			}
		}

		if (sum == null) {

			queryInstance.setSum(null, "?");

		} else {

			if (sum.scale() > 2) {
				sum = sum.setScale(2, RoundingMode.UP);
			}

			queryInstance.setSum(sum, formatBigDecimal(sum, queryInstance.getQuery().getDecimalSetting()));
		}
		
		if (count == null) {
			
			queryInstance.setCount(null, "?");
			
		} else {
			
			if (count.scale() > 2) {
				count = count.setScale(2, RoundingMode.UP);
			}
			
			queryInstance.setCount(count, formatBigDecimal(count, queryInstance.getQuery().getDecimalSetting()));
		}
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse getSummary(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Integer queryID = uriParser.getInt(2);

		if (queryID == null) {
			return null;
		}

		PriceTableQuery query = getQuery(queryID);

		if (query == null) {
			return null;
		}

		BigDecimal sum = BigDecimal.ZERO;
		BigDecimal count = BigDecimal.ZERO;
		JsonArray jsonArray = new JsonArray();

		String[] rows = req.getParameterValues("row");

		if (rows != null) {

			for (String rowID : rows) {

				BigDecimal price = NumberUtils.toBigDecimal(req.getParameter(rowID + "-price"));
				String value = req.getParameter(rowID + "-amount");

				if (StringUtils.isEmpty(value)) {

					jsonArray.addNode("");

				} else {

					BigDecimal amount = NumberUtils.toBigDecimalLocaleIndependent(value);

					if (price != null && amount != null) {

						BigDecimal rowSum = price.multiply(amount);

						if (sum != null) {
							sum = sum.add(rowSum);
						}

						if (rowSum.scale() > 2) {
							rowSum = rowSum.setScale(2, RoundingMode.UP);
						}

						jsonArray.addNode(formatBigDecimal(rowSum, query.getDecimalSetting()));
						
						if (count != null) {
							count = count.add(amount);
						}

					} else {

						sum = null;
						count = null;
						jsonArray.addNode("?");
					}
				}
			}
		}

		JsonObject jsonNode = new JsonObject(1);
		jsonNode.putField("rows", jsonArray);

		if (sum == null) {

			jsonNode.putField("sum", "?");

		} else {

			if (sum.scale() > 2) {
				sum = sum.setScale(2, RoundingMode.UP);
			}

			jsonNode.putField("sum", formatBigDecimal(sum, query.getDecimalSetting()));
		}
		
		if (count == null) {
			
			jsonNode.putField("count", "?");
			
		} else {
			
			if (count.scale() > 2) {
				count = count.setScale(2, RoundingMode.UP);
			}
			
			jsonNode.putField("count", formatBigDecimal(count, query.getDecimalSetting()));
		}

		HTTPUtils.sendReponse(jsonNode.toJson(), JsonUtils.getContentType(), res);
		return null;
	}

	@WebPublic(alias = "addrow")
	public ForegroundModuleResponse addRow(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		PriceTableQuery query = this.queryCRUD.getRequestedBean(req, res, user, uriParser, null);

		if (query != null) {

			this.checkUpdateQueryAccess(user, query);

			req.setAttribute("PriceTableQuery", query);

			return rowCRUD.add(req, res, user, uriParser);

		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "updaterow")
	public ForegroundModuleResponse updateRow(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return rowCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "deleterow")
	public ForegroundModuleResponse deleteRow(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return rowCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "sortrows")
	public ForegroundModuleResponse sortRows(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return queryCRUD.sortRows(req, res, user, uriParser);
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		PriceTableQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		if (query.getRows() != null) {

			for (PriceTableRow row : query.getRows()) {

				row.setRowID(null);
			}
		}

		this.queryDAO.add(query, transactionHandler, new RelationQuery(getTableQueryRelations()));
	}

	protected Field[] getQueryInstanceRelations() {

		return new Field[] { PriceTableQueryInstance.VALUES_RELATION, PriceTableRowValue.ROW_RELATION, PriceTableQueryInstance.QUERY_RELATION, PriceTableQuery.ROWS_RELATION };
	}

	protected RelationQuery getSaveQueryInstanceRelationQuery() {

		return new RelationQuery(PriceTableQueryInstance.VALUES_RELATION);
	}

	protected Field[] getTableQueryRelations() {

		return new Field[] { PriceTableQuery.ROWS_RELATION };
	}

	@Override
	protected Class<PriceTableQueryInstance> getQueryInstanceClass() {

		return PriceTableQueryInstance.class;
	}

	public String getExportComment() {

		return exportComment;
	}

	public String getExportSum() {

		return exportSum;
	}

	public String getExportAmount() {

		return exportAmount;
	}

	@Override
	public Document createDocument(HttpServletRequest req, User poster) {

		Document doc = super.createDocument(req, poster);

		XMLUtils.appendNewElement(doc, (Element) doc.getFirstChild(), "ModuleURL", RequestUtils.getFullContextPathURL(req) + getFullAlias());

		return doc;
	}

	@Override
	protected XSLVariableReader parseQueryXSLStyleSheet(URL styleSheetURL) {

		XSLVariableReader variableReader = super.parseQueryXSLStyleSheet(styleSheetURL);

		if (variableReader != null) {
			
			try {
				
				decimalsDisallowedError = variableReader.getValue("i18n.DecimalsDisallowed");
				tooManyDecimalsErrorPart1 = variableReader.getValue("i18n.TooManyDecimals.part1");
				tooManyDecimalsErrorPart2 = variableReader.getValue("i18n.TooManyDecimals.part2");
				tooManyDecimalsErrorPart3 = variableReader.getValue("i18n.TooManyDecimals.part3");

			} catch (Exception e) {

				log.error("Unable to get values from query style sheet " + queryStyleSheet, e);
			}
		}

		return variableReader;
	}

}
