package com.nordicpeak.flowengine.queries.pricetablequery.beans;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.serialization.SerializationUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.beans.BasePaymentProduct;
import com.nordicpeak.flowengine.interfaces.PaymentProduct;
import com.nordicpeak.flowengine.interfaces.PaymentQuery;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.basetablequery.interfaces.TableQuery;
import com.nordicpeak.flowengine.queries.pricetablequery.enums.DecimalSetting;

@Table(name = "price_table_queries")
@XMLElement
public class PriceTableQuery extends BaseQuery implements TableQuery, PaymentQuery {

	private static final long serialVersionUID = 5116885863935428968L;

	public static final Field ROWS_RELATION = ReflectionUtils.getField(PriceTableQuery.class, "rows");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<PriceTableRow> rows;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String rowsLabel;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showComments = true;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private DecimalSetting decimalSetting = DecimalSetting.ALLOW;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowNegative;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideEmptyRows;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String amountSumAttributeName;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true)
	@XMLElement
	private List<PriceTableQueryInstance> instances;

	private List<PriceTableQueryColumnDummy> dummyColumns;

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public List<PriceTableQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<PriceTableQueryInstance> instances) {

		this.instances = instances;
	}

	public List<PriceTableRow> getRows() {

		return rows;
	}

	public void setRows(List<PriceTableRow> rows) {

		this.rows = rows;
	}

	public String getRowsLabel() {

		return rowsLabel;
	}

	public void setRowsLabel(String rowsLabel) {

		this.rowsLabel = rowsLabel;
	}

	public boolean isAllowNegative() {

		return allowNegative;
	}

	public void setAllowNegative(boolean allowNegative) {

		this.allowNegative = allowNegative;
	}

	public boolean isHideEmptyRows() {

		return hideEmptyRows;
	}

	public void setHideEmptyRows(boolean hideEmptyRows) {

		this.hideEmptyRows = hideEmptyRows;
	}

	protected int getMinRowOccurance() {

		int minOccurs = 0;

		if (rows != null) {

			for (PriceTableRow row : rows) {

				if (row.isRequired()) {

					minOccurs++;
				}
			}
		}

		return minOccurs;
	}

	protected int getMaxRowOccurance() {

		return CollectionUtils.getSize(rows);
	}

	public DecimalSetting getDecimalSetting() {

		return decimalSetting;
	}

	public void setDecimalSetting(DecimalSetting decimalSetting) {

		this.decimalSetting = decimalSetting;
	}
	
	public boolean isShowComments() {

		return showComments;
	}

	public void setShowComments(boolean showComments) {

		this.showComments = showComments;
	}

	public String getAmountSumAttributeName() {

		return amountSumAttributeName;
	}

	public void setAmountSumAttributeName(String amountSumAttributeName) {

		this.amountSumAttributeName = amountSumAttributeName;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return this.getClass().getSimpleName() + " (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return this.getClass().getSimpleName() + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		addElementType(doc, sequenceElement, "Name", "xs:string", queryDescriptor.getName());
		addElementType(doc, sequenceElement, "Sum", "xs:decimal");

		Element rowElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		rowElement.setAttribute("name", "Row");
		rowElement.setAttribute("type", getXSDTypeName() + "Row");
		rowElement.setAttribute("minOccurs", "" + getMinRowOccurance());
		rowElement.setAttribute("maxOccurs", "" + getMaxRowOccurance());

		sequenceElement.appendChild(rowElement);

		doc.getDocumentElement().appendChild(complexTypeElement);

		appendXSDTypes(doc);
	}

	public void appendXSDTypes(Document doc) {

		// Row type
		Element rowComplexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		rowComplexTypeElement.setAttribute("name", getXSDTypeName() + "Row");

		Element rowSequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		rowComplexTypeElement.appendChild(rowSequenceElement);

		addElementType(doc, rowSequenceElement, "ID", "xs:positiveInteger");
		addElementType(doc, rowSequenceElement, "Name", "xs:string");
		addElementType(doc, rowSequenceElement, "Price", "xs:decimal");
		addElementType(doc, rowSequenceElement, "Value", "xs:decimal", false);
		addElementType(doc, rowSequenceElement, "Sum", "xs:decimal");
		addElementType(doc, rowSequenceElement, "Comment", "xs:string", false);

		doc.getDocumentElement().appendChild(rowComplexTypeElement);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type) {

		addElementType(doc, sequenceElement, name, type, null, true);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type, boolean required) {

		addElementType(doc, sequenceElement, name, type, null, required);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type, String fixed) {

		addElementType(doc, sequenceElement, name, type, fixed, true);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type, String fixed, boolean required) {

		Element element = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		element.setAttribute("name", name);
		element.setAttribute("type", type);
		element.setAttribute("minOccurs", required ? "1" : "0");
		element.setAttribute("maxOccurs", "1");

		if (!StringUtils.isEmpty(fixed)) {
			element.setAttribute("fixed", fixed);
		}

		sequenceElement.appendChild(element);
	}

	public static String generateElementName(String label, ArrayList<String> fieldElementNames) {

		String elementName = XMLUtils.toValidElementName(label);

		if (fieldElementNames.contains(elementName)) {

			int counter = 1;

			while (fieldElementNames.contains(elementName + counter)) {

				counter++;
			}

			elementName += counter;
		}

		fieldElementNames.add(elementName);

		return elementName;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		amountSumAttributeName = XMLValidationUtils.validateParameter("amountSumAttributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (amountSumAttributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		rowsLabel = XMLValidationUtils.validateParameter("rowsLabel", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		Boolean showDecimals = xmlParser.getBoolean("showDecimals");
		
		if (showDecimals != null) {
			
			decimalSetting = showDecimals ? DecimalSetting.FORCE : DecimalSetting.ALLOW;
			
		} else {
			
			decimalSetting = XMLValidationUtils.validateParameter("decimalSetting", xmlParser, true, new EnumPopulator<>(DecimalSetting.class), errors);
		}

		showComments = xmlParser.getPrimitiveBoolean("showComments");
		allowNegative = xmlParser.getPrimitiveBoolean("allowNegative");
		hideEmptyRows = xmlParser.getPrimitiveBoolean("hideEmptyRows");

		populateRows(xmlParser, errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	protected void populateRows(XMLParser xmlParser, List<ValidationError> errors) throws ValidationException {

		List<XMLParser> xmlParsers = xmlParser.getNodes("Rows/Row");

		if (xmlParsers != null) {

			rows = new ArrayList<>();

			for (XMLParser parser : xmlParsers) {

				PriceTableRow row = new PriceTableRow();

				row.populate(parser);

				row.setQuery(this);

				rows.add(row);
			}
		}
	}

	@Override
	public List<PriceTableQueryColumnDummy> getColumns() {

		return dummyColumns;
	}

	public void setDummyColumns(PriceTableQueryColumnDummy sumColumn, PriceTableQueryColumnDummy countColumn) {

		dummyColumns = new ArrayList<PriceTableQueryColumnDummy>(1);
		dummyColumns.add(SerializationUtils.cloneSerializable(sumColumn));
		dummyColumns.add(SerializationUtils.cloneSerializable(countColumn));
	}

	@Override
	public List<? extends PaymentProduct> getPaymentProducts() {

		if (rows != null) {

			List<PaymentProduct> paymentProducts = new ArrayList<>();

			for (PriceTableRow row : rows) {

				if (row.getPrice() != null && row.getPrice().compareTo(BigDecimal.ZERO) > 0 && row.getName() != null) {

					paymentProducts.add(new BasePaymentProduct(row.getPrice(), row.getName()));
				}
			}

			return paymentProducts;
		}

		return Collections.emptyList();
	}

}
