package com.nordicpeak.flowengine.queries.pricetablequery.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.BigDecimalPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.pricetablequery.PriceTableQueryProviderModule;

@Table(name = "price_table_query_rows")
@XMLElement(name = "Row")
public class PriceTableRow extends GeneratedElementable implements Serializable {
	
	
	private static final long serialVersionUID = -2724650199031970214L;
	
	public static final Field QUERY_RELATION = ReflectionUtils.getField(PriceTableRow.class, "query");
	
	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer rowID;
	
	@DAOManaged(columnName = "queryID")
	@ManyToOne
	private PriceTableQuery query;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String label;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean required;
	
	@DAOManaged
	@OrderBy
	@XMLElement
	private Integer sortIndex;
	
	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private BigDecimal price;
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<PriceTableRowValue> values;
	
	@XMLElement
	private String sum;
	
	public Integer getRowID() {
		
		return rowID;
	}
	
	public void setRowID(Integer rowID) {
		
		this.rowID = rowID;
	}
	
	public BigDecimal getPrice() {
		return price;
	}
	
	public void setPrice(BigDecimal price) {
		this.price = price;
	}
	
	@Override
	public String toString() {
		
		return this.getClass().getSimpleName() + " (rowID: " + rowID + ")";
	}
	
	public PriceTableQuery getQuery() {
		
		return query;
	}
	
	public void setQuery(PriceTableQuery query) {
		
		this.query = query;
	}
	
	public String getLabel() {
		
		return label;
	}
	
	public void setLabel(String name) {
		
		this.label = name;
	}
	
	public boolean isRequired() {
		
		return required;
	}
	
	public void setRequired(boolean required) {
		
		this.required = required;
	}
	
	public List<PriceTableRowValue> getValues() {
		
		return values;
	}
	
	public void setValues(List<PriceTableRowValue> values) {
		
		this.values = values;
	}
	
	public Integer getSortIndex() {
		
		return sortIndex;
	}
	
	public void setSortIndex(Integer sortIndex) {
		
		this.sortIndex = sortIndex;
	}
	
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<>();
		
		label = XMLValidationUtils.validateParameter("label", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		required = xmlParser.getPrimitiveBoolean("required");
		
		setSortIndex(XMLValidationUtils.validateParameter("sortIndex", xmlParser, true, NonNegativeStringIntegerPopulator.getPopulator(), errors));
		
		price = XMLValidationUtils.validateParameter("price", xmlParser, true, BigDecimalPopulator.getPopulator(), errors);
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
		
	}
	
	public String getRowIdentifier() {
		
		return getRowID().toString();
	}
	
	public String getName() {
		
		return label;
	}
	
	public String getSum() {
		
		return sum;
	}
	
	public void setSum(String sum) {
		
		this.sum = sum;
	}
	
	@Override
	public Element toXML(Document doc) {
		
		Element element = super.toXML(doc);
		
		if (price != null && query != null) {
			XMLUtils.appendNewElement(doc, element, "formattedPrice", PriceTableQueryProviderModule.formatBigDecimal(price, query.getDecimalSetting()));
		}
		
		return element;
	}
	
}
