package com.nordicpeak.flowengine.queries.priorityquery;

import java.lang.reflect.Field;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToMany;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativeQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

@Table(name = "priority_query_instances")
@XMLElement
public class PriorityQueryInstance extends BaseQueryInstance implements FixedAlternativesQueryInstance, StringValueQueryInstance {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field ALTERNATIVES_RELATION = ReflectionUtils.getField(PriorityQueryInstance.class, "alternatives");
	public static final Field QUERY_RELATION = ReflectionUtils.getField(PriorityQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private PriorityQuery query;

	@DAOManaged
	@ManyToMany(linkTable = "priority_query_instance_alternatives", indexColumn="sortIndex")
	@XMLElement(fixCase = true)
	private List<PriorityAlternative> alternatives;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public PriorityQuery getQuery() {

		return query;
	}

	public void setQuery(PriorityQuery query) {

		this.query = query;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.alternatives = null;
		this.resetAttributes(attributeHandler);
		super.reset(attributeHandler);
	}

	public void resetAttributes(MutableAttributeHandler attributeHandler){
		
		if(query.getAttributeName() != null){
			
			attributeHandler.removeAttribute(query.getAttributeName());
			
			if(query.getMaxChoices() != null){

				int index = 1;
				
				while(index <= query.getMaxChoices()){
					
					attributeHandler.removeAttribute(query.getAttributeName() + "-" + index);
					
					index++;
				}
			}
		}
	}
	
	public void setAttributes(MutableAttributeHandler attributeHandler){
		
		resetAttributes(attributeHandler);
		
		if(query.getAttributeName() != null && alternatives != null){
			
			attributeHandler.setAttribute(query.getAttributeName(), getStringValue());
			
			int index = 1;
			
			for(PriorityAlternative alternative : alternatives){
				
				attributeHandler.setAttribute(query.getAttributeName() + "-" + index, alternative.getName());
				
				index++;
			}
		}
	}
	
	public void copyQueryValues() {}

	@Override
	public String toString() {

		return PriorityQueryInstance.class.getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	public void setAlternatives(List<PriorityAlternative> alternatives) {

		this.alternatives = alternatives;
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		return alternatives;
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		FixedAlternativeQueryUtils.appendExportXMLAlternatives(doc, element, this);

		return element;
	}

	@Override
	public String getStringValue() {

		if (alternatives != null) {

			StringBuilder stringBuilder = new StringBuilder();

			for (PriorityAlternative alternative : alternatives) {

				if(stringBuilder.length() > 0){
					
					stringBuilder.append(", ");
				}
				
				stringBuilder.append(alternative.getName());
			}

			return stringBuilder.toString();
		}

		return null;
	}

	@Override
	public String getFreeTextAlternativeValue() {

		return null;
	}
}
