<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/CKEditor.xsl" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/jquery/plugins/jquery.qloader.js
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
		/featherlight/js/featherlight.min.js
		/js/confirmpost.js
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/jquery.ui.datepicker-sv.js
		/js/flowengine.tablesorter.js
		/js/flowengine.tablefilter.js
		/js/jquery.qloader-init.js
		/js/datatables.min.js
		/js/queueslotadmin.js?v=2
		/js/js.cookie-2.0.4.min.js
		/js/messages.js?v=3
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/flowengine.css
		/css/datatables.css
		/css/overrides.css
		/css/buttonspinner.css
		/css/messages.css
	</xsl:variable>

	<xsl:template match="Document">
		
		<div id="QueueSlotAdminModule" class="contentitem">
			<xsl:if test="ListQueues or ListQueueSlots or ListArchivedQueueSlots or AddQueueSlot">
				<xsl:attribute name="class">contentitem errands-wrapper</xsl:attribute>
			</xsl:if>
			
			<xsl:apply-templates select="ListQueues" />
			<xsl:apply-templates select="ListQueueSlots" />
			<xsl:apply-templates select="ListArchivedQueueSlots" />
			
			<xsl:apply-templates select="ShowQueueSlot" />
			<xsl:apply-templates select="AddQueueSlot" />
			<xsl:apply-templates select="UpdateQueueSlot" />
			<xsl:apply-templates select="UpdateQueueSlotDescription" />
			
			<xsl:apply-templates select="AddPayment" />
			<xsl:apply-templates select="RequestPayment" />
			
			<xsl:apply-templates select="UpdateContact" />
			<xsl:apply-templates select="ImportExcel" />
			<xsl:apply-templates select="SendMessageForm" />
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="ListQueues">
	
		<div class="floatright marginbottom">
			<label for="name" class="bigmarginleft marginright margintop"><xsl:value-of select="$i18n.Filter" />:</label>
			<input type="text" size="20" class="filter-input" data-tableid="queues" />
		</div>
		
		<h1>
			<xsl:value-of select="/Document/module/name" />
		</h1>
	
		<table id="queues" class="oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Queue.Name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Queue.ActiveSlots" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Queue.ArchivedSlots" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Queues/Queue">
						<xsl:apply-templates select="Queues/Queue" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="4"><xsl:value-of select="$i18n.NoQueues" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
			
	</xsl:template>
	
	<xsl:template match="Queue" mode="list">
	
		<tr>
			<td data-title="{$i18n.Queue.Name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{queueID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Queue.ActiveSlots}"><xsl:value-of select="activeSlots" /></td>
			<td data-title="{$i18n.Queue.ArchivedSlots}"><xsl:value-of select="archivedSlots" /></td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="ListQueueSlots">
	
		<h1>
			<xsl:value-of select="$i18n.ListQueueSlots.title" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="Queue/name" />
		</h1>
		
		<div id="table-description" class="floatleft" style="max-width: calc(100% - 220px);">
			<p><xsl:value-of select="Queue/description"/></p>
		</div>
		
		<table id="queueslots" class="stripe display marginbottom">
			<thead class="sortable">
				<tr>
					<th class="select-checkbox">
						<div class="select-page"></div>
					</th>
					<th class="hidden"/>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Position" /></span>
					</th>
					<th class="hidden"/>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Poster" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Attribute.Description" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Added" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.ArchivedDate" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.LatestPayment" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Digital" /></span>
					</th>
					<th class="hidden"/>
				</tr>
			</thead>
			<tbody/>
		</table>
		
		<xsl:variable name="ajaxURL">
			<xsl:value-of select="/Document/requestinfo/currentURI"/>
			<xsl:text>/</xsl:text>
			<xsl:value-of select="/Document/module/alias"/>
			<xsl:text>/slotdata/</xsl:text>
			<xsl:value-of select="Queue/queueID"/>
		</xsl:variable>
		
		<script type="text/javascript">
			$(function() {
			
				var $element = $('#queueslots');
				var table = $element.DataTable({
					ajax: { url: '<xsl:value-of select="$ajaxURL"/>',
									dataSrc: 'rows',
					},
					deferRender: true,
					columns: [
						{ "name": "select", className: 'select-checkbox', orderable: false, searchable: false, data: null, defaultContent: "" },
						{ "name": "queueSlotID", visible: false, },
						{ "name": "position" },
						{ "name": "paymentPending", "visible": false },
						{ "name": "poster" },
						{ "name": "description"},
						{ "name": "added" },
						{ "name": "archiveDate", "visible": false },
						{ "name": "latestPayment",
						render: function(data, type, row) {
							
								if (type == "display") {
							
									if (row[3] == 1) {
									
										return data + '<img class="floatright marginleft" title="{$i18n.QueueSlot.PaymentRequested}" alt="{$i18n.QueueSlot.PaymentRequested}" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/coinstack.png" />';
									}
								}
								
								return data;
							}
						},
						{ "name": "digital",
							render: function(data, type, row) {
							
								if (type == "display") {
							
									if (data == 1) {
									
										return '<xsl:value-of select="$i18n.Yes"/>';
										
									} else {
									
										return '<xsl:value-of select="$i18n.No"/>';
									}
								}
								
								return data;
							}
						 },
						{ "name": "searchExtra", "visible": false }
				  ],
				  order: [[ 2, 'asc' ]],
				  dom: 'frtipl',
					bAutoWidth: false,
					paging: true,
					pagingType: 'simple_numbers',
					pageLength: <xsl:value-of select="PageSize"/>,
					lengthMenu: [ 15, 50, 100, 200, 500 ],
					language: {
						"decimal":        "<xsl:value-of select="$i18n.DataTable.decimal"/>",
						"emptyTable":     "<xsl:value-of select="$i18n.DataTable.emptyTable"/>",
						"info":           "<xsl:value-of select="$i18n.DataTable.info"/>",
						"infoEmpty":      "<xsl:value-of select="$i18n.DataTable.infoEmpty"/>",
						"infoFiltered":   "<xsl:value-of select="$i18n.DataTable.infoFiltered"/>",
						"infoPostFix":    "<xsl:value-of select="$i18n.DataTable.infoPostFix"/>",
						"thousands":      "<xsl:value-of select="$i18n.DataTable.thousands"/>",
						"lengthMenu":     "<xsl:value-of select="$i18n.DataTable.lengthMenu"/>",
						"loadingRecords": "<xsl:value-of select="$i18n.DataTable.loadingRecords"/>",
						"processing":     "<xsl:value-of select="$i18n.DataTable.processing"/>",
						"search":         "<xsl:value-of select="$i18n.DataTable.search"/>",
						"zeroRecords":    "<xsl:value-of select="$i18n.DataTable.zeroRecords"/>",
						paginate: {
							"first":      "<xsl:value-of select="$i18n.DataTable.paginate.first"/>",
							"last":       "<xsl:value-of select="$i18n.DataTable.paginate.last"/>",
							"next":       "<xsl:value-of select="$i18n.DataTable.paginate.next"/>",
							"previous":   "<xsl:value-of select="$i18n.DataTable.paginate.previous"/>"
						},
						aria: {
							"sortAscending":  "<xsl:value-of select="$i18n.DataTable.aria.sortAscending"/>",
							"sortDescending": "<xsl:value-of select="$i18n.DataTable.aria.sortDescending"/>"
						},
						select: {
							rows: {
								_: "<xsl:value-of select="$i18n.DataTable.selection.many"/>",
								0: "<xsl:value-of select="$i18n.DataTable.selection.none"/>",
								1: "<xsl:value-of select="$i18n.DataTable.selection.one"/>"
							}
						},						
					},
					stateSave: true,
  				stateSaveParams: function (settings, data) {
					
						//data.start = undefined;
						//data.length = undefined;
						data.order = undefined;
						data.search = undefined;
						data.columns = undefined;
					},
					stateLoadParams: function (settings, data) {
					
						if (data != undefined) {
					
							//data.start = undefined;
							//data.length = undefined;
							data.order = undefined;
							data.search = undefined;
							data.columns = undefined;
						}
					},
					select: {
						style:    'multi',
						selector: 'td.select-checkbox'
					},
				});
				
				$("#queueslots_wrapper").prepend($("#table-description"));
				
				table.on( 'click', 'tbody td', function() {
				
					var td = $(this);
					var tr = td.parent();
					
					if(tr.children("td").first().hasClass("dataTables_empty") || td.hasClass("select-checkbox")){
						return;
					}
					
					if(tr.closest("table").hasClass("collapsed") &amp;&amp; td.is(":first-child")){
						return;
					}
					
					var id = table.cell(tr, "queueSlotID:name").data();
					window.location = "<xsl:value-of select="/Document/requestinfo/contextpath"/><xsl:value-of select="FullAlias"/>/show/" + id;
				});
				
				table.on('click', 'thead th.select-checkbox', function() {
				
					var visibleRows = table.rows({ page: 'current', filter: 'applied' });
					var selectedRows = table.rows({ page: 'current', filter: 'applied', selected: true});
					
					var allVisibleSelected = selectedRows.any() &amp;&amp; visibleRows.indexes().length == selectedRows.indexes().length;
					
					if (allVisibleSelected) {
					
						visibleRows.deselect();
					
					} else {
					
						visibleRows.select();
					}
				});
				
				var updateHeaderSelect = function() {
					
					var visibleRows = table.rows({ page: 'current', filter: 'applied' });
					var selectedRows = table.rows({ page: 'current', filter: 'applied', selected: true});
					
					var allVisibleSelected = selectedRows.any() &amp;&amp; visibleRows.indexes().length == selectedRows.indexes().length; 
					
					$element.find("thead th.select-checkbox").toggleClass("selected", allVisibleSelected);
				}
				
				var updateArchiveQueueSlotsButton = function() {
					
					$('#archiveQueueSlots').toggleClass('hidden', !table.rows({ selected: true }).any());
				}
				
				table.on('select.dt', function() {
					updateHeaderSelect();
					updateArchiveQueueSlotsButton();
				});
				
				table.on('deselect.dt', function() {
					updateHeaderSelect();
					updateArchiveQueueSlotsButton();
				});
				
				table.on('page.dt', function() {
					updateHeaderSelect();
				});
				
				table.on('length.dt', function() {
					updateHeaderSelect();
				});
				
				var exportButton = $("#exportbutton");
				exportButton.on("click", function() {
				
					var exportURL = exportButton.data("url");
					var rows = table.rows({ selected: true });
					
					if (!rows.any()) {
		
						window.location = exportURL;
					
					} else {
					
						var cells = table.cells(rows.toArray()[0], "queueSlotID:name");
						var selectedIDs = cells.data().toArray();
						
						submitSelection(selectedIDs, exportURL);
					}
				});
				
			});
			
			function submitSelection(selectedIDs, url) {
				
				var form = $("#dummyForm");
				form.attr("action", url);
				form.find("input[name != 'directFromQueueSlotList']").remove();
				
				for (var i = 0, len = selectedIDs.length; len > i; i++) {
					
					var field = $('<input/>');
					
					field.attr("type", "hidden");
					field.attr("name", "queueSlotID");
					field.attr("value", selectedIDs[i]);
					
					form.append(field);
				}
				
				form.trigger("submit");
			}
		</script>
		
		<article class="buttons floatright bigmargintop">
			
			<a class="btn btn-green btn-right next" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/add/{Queue/queueID}">
				<xsl:value-of select="$i18n.AddQueueSlot" />
			</a>
			
			<a class="btn btn-light btn-right next bigmarginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{Queue/queueID}?archived=t">
				<xsl:value-of select="$i18n.ShowArchivedQueueSlots" />
			</a>
			
			<a id="exportbutton" class="btn btn-blue btn-right next bigmarginright" href="#" data-url="{/Document/requestinfo/currentURI}/{/Document/module/alias}/export/{Queue/queueID}" title="{$i18n.ExportToExcel.title}">
				<xsl:value-of select="$i18n.ExportToExcel" />
				<span class="marginleft" data-icon-after="d" style="position: relative; top: -2px;"/>
			</a>
			
			<xsl:if test="AdminAccess">
				<a class="btn btn-blue btn-right next bigmarginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/import/{Queue/queueID}">
					<xsl:value-of select="$i18n.ImportFromExcel" />
				</a>
			</xsl:if>
			
			<form id="sendRequestedPayment" class="display-inline" action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/requestpayment/{Queue/queueID}" method="POST">
			
				<a class="btn btn-blue btn-right next bigmarginright" onclick="populateAndSubmitForm(event, this);">
					<xsl:value-of select="$i18n.RequestPayment" />
				</a>
			</form>
			
			<!-- <a class="btn btn-blue btn-right next bigmarginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/requestpayment/{Queue/queueID}">
				<xsl:value-of select="$i18n.RequestPayment" />
			</a> -->
			
			<form id="sendMessages" class="display-inline" action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/sendmessages/{Queue/queueID}" method="POST">
			
				<a class="btn btn-blue btn-right next bigmarginright" onclick="populateAndSubmitForm(event, this);">
					<xsl:value-of select="$i18n.SendMessages" />
				</a>
			</form>
			
			<form id="archiveQueueSlots" class="hidden display-inline" action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/archive" method="POST">
			
				<input type="hidden" name="queueID" value="{Queue/queueID}" />
				<input type="hidden" name="comment" />
				<input type="hidden" name="selectedWaitingPeriod" />
			
				<a class="btn btn-blue btn-right next bigmarginright" onclick="commentModal(event, this, prepareQueueSlotsModal)">
					<xsl:value-of select="$i18n.ArchiveQueueSlots" />
				</a>
				
				<div style="display: none;">
							
					<div class="comment-modal contentitem">
						<div class="modal-content">
						
							<div class="modal-header bigmarginbottom">
								<h1>
									<xsl:value-of select="$i18n.ArchiveQueueSlots.Title.Part1"/>
									<xsl:text> </xsl:text>
									<span id="queueSlotCount"></span>
									<xsl:text> </xsl:text>
									<xsl:value-of select="$i18n.ArchiveQueueSlots.Title.Part2"/>
								</h1>
							</div>
							
							<div class="modal-body">
							
								<div class="floatleft full bigmarginbottom">
								
									<p class="tiny floatleft full">
										<xsl:value-of select="$i18n.ArchiveQueueSlot.Description" />
								  	</p>
								  
									<label for="message" class="floatleft clearboth">
										<xsl:value-of select="$i18n.Comment" />
									</label>
									
									<div class="floatleft full">
										<xsl:call-template name="createTextArea">
											<xsl:with-param name="name" select="'message'"/>
											<xsl:with-param name="maxlength" select="'1024'"/>
										</xsl:call-template>
								  </div>
								  <xsl:if test="Queue/useWaitingPeriod='true'">
								  
								  		<label for="useWaitingPeriod" class="floatleft clearboth">
											<xsl:value-of select="$i18n.WaitingPeriod" />
										</label>
										
										<p class="tiny floatleft full">
											<xsl:value-of select="$i18n.WaitingPeriod.Description" />
									  	</p>
										
										<div class="floatleft full">
										<xsl:call-template name="createTextField">
											<xsl:with-param name="name" select="'waitingPeriod'" />
											<xsl:with-param name="id" select="'waitingPeriod'" />
											<xsl:with-param name="size" select="10" />
											<xsl:with-param name="type" select="'date'"/>
										</xsl:call-template>
										</div>
									</xsl:if>
								</div>
								
								
								<input class="btn btn-red close bigmargintop floatright" type="button" value="{$i18n.ArchiveQueueSlots}"/>
								
							</div>
							
						</div>
					</div>
				
				</div>
				
			</form>
			
			<div style="display: none;">
				
				<form id="dummyForm" method="POST">
					<xsl:call-template name="createHiddenField">
						<xsl:with-param name="name" select="'directFromQueueSlotList'"/>
						<xsl:with-param name="value" select="true"/>
					</xsl:call-template>
				</form>
				
			</div>
			
		</article>
		
	</xsl:template>
	
	<xsl:template match="ListArchivedQueueSlots">
	
		<h1>
			<xsl:value-of select="$i18n.ListArchivedQueueSlots.title" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="Queue/name" />
		</h1>
		
		<div id="table-description" class="floatleft" style="max-width: calc(100% - 220px);">
			<p><xsl:value-of select="Queue/description"/></p>
		</div>
		
		<table id="queueslots-archived" class="stripe display marginbottom">
			<thead class="sortable">
				<tr>
					<th class="hidden"/>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Poster" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Attribute.Description" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Added" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.ArchivedDate" /></span>
					</th>
					<xsl:if test="Queue/useWaitingPeriod = 'true'">
						<th>
							<span><xsl:value-of select="$i18n.QueueSlot.InWaitingPeriod" /></span>
						</th>
					</xsl:if>						
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.LatestPayment" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.QueueSlot.Digital" /></span>
					</th>
					<th class="hidden"/>
				</tr>
			</thead>
			<tbody/>
		</table>
		
		<xsl:variable name="ajaxURL">
			<xsl:value-of select="/Document/requestinfo/currentURI"/>
			<xsl:text>/</xsl:text>
			<xsl:value-of select="/Document/module/alias"/>
			<xsl:text>/slotdata/</xsl:text>
			<xsl:value-of select="Queue/queueID"/>
			<xsl:text>?archived=true</xsl:text>
		</xsl:variable>
		
		<script type="text/javascript">
			$(function() {
			
			var currentDate = new Date().setHours(0,0,0,0);
			
			
			
				var table = $('#queueslots-archived').DataTable({
					ajax: { url: '<xsl:value-of select="$ajaxURL"/>',
									dataSrc: 'rows',
					},
					deferRender: true,
					columns: [
						{ "name": "queueSlotID", visible: false},
						{ "name": "poster" },
						{ "name": "description"},
						{ "name": "added" },
						{ "name": "archiveDate" },
						<xsl:if test="Queue/useWaitingPeriod = 'true'">
							
						
							{ "name": "waitingPeriod" ,
								render: function(data, type, row) {
								
									if (data) {
										var dateParts = data.split("-");
										var selectedDate = new Date(+dateParts[0], dateParts[1] - 1, +dateParts[2]);
								
										if (currentDate > selectedDate) {
										
											return '<xsl:value-of select="$i18n.No"/> ('+data+')';
											
										} else {
										
											return '<xsl:value-of select="$i18n.Yes"/> ('+data+')';
										}
									}
									
									return '<xsl:value-of select="$i18n.No"/>';
								}
							},
						</xsl:if>
						{ "name": "latestPayment" },
						{ "name": "digital",
							render: function(data, type, row) {
							
								if (type == "display") {
							
									if (data == 1) {
									
										return '<xsl:value-of select="$i18n.Yes"/>';
										
									} else {
									
										return '<xsl:value-of select="$i18n.No"/>';
									}
								}
								
								return data;
							}
						},
						{ "name": "searchExtra", "visible": false }
				  ],
				  order: [[ 2, 'asc' ]],
				  dom: 'frtipl',
					bAutoWidth: false,
					paging: true,
					pagingType: 'simple_numbers',
					pageLength: <xsl:value-of select="PageSize"/>,
					lengthMenu: [ 15, 50, 100, 200, 500 ],
					language: {
						"decimal":        "<xsl:value-of select="$i18n.DataTable.decimal"/>",
						"emptyTable":     "<xsl:value-of select="$i18n.DataTable.emptyTable"/>",
						"info":           "<xsl:value-of select="$i18n.DataTable.info"/>",
						"infoEmpty":      "<xsl:value-of select="$i18n.DataTable.infoEmpty"/>",
						"infoFiltered":   "<xsl:value-of select="$i18n.DataTable.infoFiltered"/>",
						"infoPostFix":    "<xsl:value-of select="$i18n.DataTable.infoPostFix"/>",
						"thousands":      "<xsl:value-of select="$i18n.DataTable.thousands"/>",
						"lengthMenu":     "<xsl:value-of select="$i18n.DataTable.lengthMenu"/>",
						"loadingRecords": "<xsl:value-of select="$i18n.DataTable.loadingRecords"/>",
						"processing":     "<xsl:value-of select="$i18n.DataTable.processing"/>",
						"search":         "<xsl:value-of select="$i18n.DataTable.search"/>",
						"zeroRecords":    "<xsl:value-of select="$i18n.DataTable.zeroRecords"/>",
						paginate: {
							"first":      "<xsl:value-of select="$i18n.DataTable.paginate.first"/>",
							"last":       "<xsl:value-of select="$i18n.DataTable.paginate.last"/>",
							"next":       "<xsl:value-of select="$i18n.DataTable.paginate.next"/>",
							"previous":   "<xsl:value-of select="$i18n.DataTable.paginate.previous"/>"
						},
						aria: {
							"sortAscending":  "<xsl:value-of select="$i18n.DataTable.aria.sortAscending"/>",
							"sortDescending": "<xsl:value-of select="$i18n.DataTable.aria.sortDescending"/>"
						},
					},
					stateSave: true,
					stateSaveParams: function (settings, data) {
					
						//data.start = undefined;
						//data.length = undefined;
						data.order = undefined;
						data.search = undefined;
						data.columns = undefined;
					},
					stateLoadParams: function (settings, data) {
					
						if (data != undefined) {
					
							//data.start = undefined;
							//data.length = undefined;
							data.order = undefined;
							data.search = undefined;
							data.columns = undefined;
						}
					},
				});
				
				$("#queueslots-archived_wrapper").prepend($("#table-description"));
				
				table.on( 'click', 'tbody td', function() {
				
					var td = $(this);
					var tr = td.parent();
					
					if(tr.children("td").first().hasClass("dataTables_empty")){
						return;
					}
					
					if(tr.closest("table").hasClass("collapsed") &amp;&amp; td.is(":first-child")){
						return;
					}
					
					var id = table.cell(tr, "queueSlotID:name").data();
					window.location = "<xsl:value-of select="/Document/requestinfo/contextpath"/><xsl:value-of select="FullAlias"/>/show/" + id;
				});
			});
		</script>
		
		<article class="buttons floatright bigmargintop">
		
			<a class="btn btn-green btn-right next" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{Queue/queueID}">
				<xsl:value-of select="$i18n.ShowNonArchivedQueueSlots" />
			</a>
			
			<a class="btn btn-blue btn-right next bigmarginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/export/{Queue/queueID}?archived=t">
				<xsl:value-of select="$i18n.ExportToExcel" />
				<span class="marginleft" data-icon-after="d" style="position: relative; top: -2px;"/>
			</a>
			
		</article>
		
	</xsl:template>
	
	<xsl:template name="showQueueSlotControlPanel">
	
		<xsl:param name="queueSlot" select="QueueSlot" />
		<xsl:param name="adminAccess" select="AdminAccess" />
		<xsl:param name="template" select="local-name()" />
	
		<div class="errand-menu buttons-in-desktop" data-menu="errand">
	  	<a class="btn btn-dark" data-toggle-menu="errand" href="#">
	  		<span data-icon-after="_">
	  		</span>
	  	</a>
	  	<ul>
	 		
		  	<li>
		  	
		  		<xsl:choose>
		  			<xsl:when test="$queueSlot/archived = 'true'">
		  			
				  		<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{$queueSlot/Queue/queueID}?archived=t" class="btn btn-light">
								<span data-icon-before="o"><xsl:value-of select="$i18n.ShowQueue"/></span>
							</a>
		  			
		  			</xsl:when>
		  			<xsl:otherwise>
				  		
		  				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{$queueSlot/Queue/queueID}" class="btn btn-light">
								<span data-icon-before="o"><xsl:value-of select="$i18n.ShowQueue"/></span>
							</a>
				  		
		  			</xsl:otherwise>
		  		</xsl:choose>
	  			
	  		</li>
	  		
	  		<li>
	  			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/show/{$queueSlot/queueSlotID}" class="btn btn-light">
	  				<xsl:if test="$template = 'ShowQueueSlot'"><xsl:attribute name="class">btn btn-light active</xsl:attribute></xsl:if>
						<span data-icon-before="s"><xsl:value-of select="$i18n.ShowQueueSlot"/></span>
					</a>
	  		</li>
	  		
	  		<xsl:if test="$queueSlot/flowInstanceID">
					
					<li>
		  			<a target="_blank" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/preview/{$queueSlot/queueSlotID}" class="btn btn-light">
							<span data-icon-before="s"><xsl:value-of select="$i18n.ShowFlowInstance"/></span>
						</a>
		  		</li>
			  	
		  		<li>
		  			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/download/{$queueSlot/queueSlotID}" class="btn btn-light">
							<span data-icon-before="d"><xsl:value-of select="$i18n.DownloadFlowInstancePDF"/></span>
						</a>
		  		</li>
					
	 			</xsl:if>
	 			
	 			<xsl:if test="not($queueSlot/archived = 'true')">
	  			
		  		<li>
		  			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/payment/{$queueSlot/queueSlotID}" class="btn btn-light">
							<xsl:if test="$template = 'AddPayment'"><xsl:attribute name="class">btn btn-light active</xsl:attribute></xsl:if>
							<span data-icon-before="+"><xsl:value-of select="$i18n.AddPayment"/></span>
						</a>
		  		</li>
		  		
		  		<xsl:choose>
		  			<xsl:when test="$queueSlot/pendingPaymentRequestDate">
		  				<li>
		  					<form action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/revokepaymentrequest/{$queueSlot/queueSlotID}" method="post">
						
									<input type="hidden" name="comment" />
									
									<div style="display: none;">
								
										<div class="comment-modal contentitem">
											<div class="modal-content">
											
												<div class="modal-header bigmarginbottom">
													<h1>
														<xsl:value-of select="$i18n.RevokePaymentRequest.Title"/>
														<xsl:text>: </xsl:text>
														<xsl:value-of select="$queueSlot/position"/>
													</h1>
												</div>
												
												<div class="modal-body">
												
													<div class="floatleft full bigmarginbottom">
													
														<p class="tiny floatleft full">
															<xsl:value-of select="$i18n.RevokePaymentRequest.Description" />
													  </p>
													  
														<label for="save-email" class="floatleft clearboth">
															<xsl:value-of select="$i18n.Comment" />
														</label>
														
														<div class="floatleft full">
															<xsl:call-template name="createTextArea">
																<xsl:with-param name="name" select="'message'"/>
																<xsl:with-param name="maxlength" select="'1024'"/>
															</xsl:call-template>
													  </div>
													</div>
													
													<input class="btn btn-red close bigmargintop floatright" type="button" value="{$i18n.RevokePaymentRequest}"/>
													
												</div>
												
											</div>
										</div>
									
									</div>
									
					  				<a href="#" onclick="commentModal(event, this)" title="{$i18n.RevokePaymentRequest}: {$queueSlot/position}" class="btn btn-light">
										<span data-icon-before="t"><xsl:value-of select="$i18n.RevokePaymentRequest"/></span>
									</a>
									
								</form>
				  		</li>
		  			</xsl:when>
		  			<xsl:otherwise>
			  			<li>
			  				<form action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/requestpaymentforqueueslot/{$queueSlot/queueSlotID}" method="post">
						
									<input type="hidden" name="comment" />
									
									<div style="display: none;">
								
										<div class="comment-modal contentitem">
											<div class="modal-content">
											
												<div class="modal-header bigmarginbottom">
													<h1>
														<xsl:value-of select="$i18n.RequestPayment.Title"/>
														<xsl:text>: </xsl:text>
														<xsl:value-of select="$queueSlot/position"/>
													</h1>
												</div>
												
												<div class="modal-body">
												
													<div class="floatleft full bigmarginbottom">
													
														<p class="tiny floatleft full">
															<xsl:value-of select="$i18n.RequestPayment.Description" />
													  </p>
													  
														<label for="save-email" class="floatleft clearboth">
															<xsl:value-of select="$i18n.Comment" />
														</label>
														
														<div class="floatleft full">
															<xsl:call-template name="createTextArea">
																<xsl:with-param name="name" select="'message'"/>
																<xsl:with-param name="maxlength" select="'1024'"/>
															</xsl:call-template>
													  </div>
													</div>
													
													<input class="btn btn-green close bigmargintop floatright" type="button" value="{$i18n.RequestPayment}"/>
													
												</div>
												
											</div>
										</div>
									
									</div>
									
					  			<a href="#" onclick="commentModal(event, this)" title="{$i18n.RequestPayment.Title}: {$queueSlot/position}" class="btn btn-light">
										<span data-icon-before="+"><xsl:value-of select="$i18n.RequestPayment"/></span>
									</a>
									
								</form>
				  		</li>
		  			</xsl:otherwise>
		  		</xsl:choose>
			  	
		  		<li>
		  			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/update/{$queueSlot/queueSlotID}" class="btn btn-light">
							<xsl:if test="$template = 'UpdateQueueSlot'"><xsl:attribute name="class">btn btn-light active</xsl:attribute></xsl:if>
							<span data-icon-before="w"><xsl:value-of select="$i18n.UpdateQueueSlot"/></span>
						</a>
		  		</li>
		  		
	  		</xsl:if>
	  		
	  		<xsl:choose>
	  			<xsl:when test="$queueSlot/archived = 'true'">
	  			
			  		<li>
			  			<form action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/unarchive/{$queueSlot/queueSlotID}?show=t" method="post">
						
								<input type="hidden" name="comment" />
								
								<div style="display: none;">
							
									<div class="comment-modal contentitem">
										<div class="modal-content">
										
											<div class="modal-header bigmarginbottom">
												<h1>
													<xsl:value-of select="$i18n.UnArchiveQueueSlot.Title"/>
													<xsl:text>: </xsl:text>
													<xsl:value-of select="$queueSlot/position"/>
												</h1>
											</div>
											
											<div class="modal-body">
											
												<div class="floatleft full bigmarginbottom">
												
													<p class="tiny floatleft full">
														<xsl:value-of select="$i18n.UnArchiveQueueSlot.Description" />
												  </p>
												  
													<label for="save-email" class="floatleft clearboth">
														<xsl:value-of select="$i18n.Comment" />
													</label>
													
													<div class="floatleft full">
														<xsl:call-template name="createTextArea">
															<xsl:with-param name="name" select="'message'"/>
															<xsl:with-param name="maxlength" select="'1024'"/>
														</xsl:call-template>
												  </div>
												</div>
												
												<input class="btn btn-red close bigmargintop floatright" type="button" value="{$i18n.UnArchiveQueueSlot}"/>
												
											</div>
											
										</div>
									</div>
								
								</div>
								
								<a href="#" onclick="commentModal(event, this)" title="{$i18n.UnArchiveQueueSlot}: {$queueSlot/position}" class="btn btn-light">
									<span data-icon-before="+"><xsl:value-of select="$i18n.UnArchiveQueueSlot"/></span>
								</a>
								
							</form>
						</li>
						
						<xsl:if test="$adminAccess">
							<li>
								<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/delete/{$queueSlot/queueSlotID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.DeleteQueueSlot}: {$queueSlot/position}" class="btn btn-light">
									<span data-icon-before="t"><xsl:value-of select="$i18n.DeleteQueueSlot"/></span>
								</a>
							</li>
						</xsl:if>
					
					</xsl:when>
					<xsl:otherwise>
						
						<li>
							<form action="{/Document/requestinfo/currentURI}/{/Document/module/alias}/archive/{$queueSlot/queueSlotID}?show=t" method="post">
						
								<input type="hidden" name="comment" />
								<input type="hidden" name="selectedWaitingPeriod" />
								
								<div style="display: none;">
							
									<div class="comment-modal contentitem">
										<div class="modal-content">
										
											<div class="modal-header bigmarginbottom">
												<h1>
													<xsl:value-of select="$i18n.ArchiveQueueSlot.Title"/>
													<xsl:text>: </xsl:text>
													<xsl:value-of select="$queueSlot/position"/>
												</h1>
											</div>
											
											<div class="modal-body">
											
												<div class="floatleft full bigmarginbottom">
												
													<p class="tiny floatleft full">
														<xsl:value-of select="$i18n.ArchiveQueueSlot.Description" />
												  </p>
												  
													<label for="message" class="floatleft clearboth">
														<xsl:value-of select="$i18n.Comment" />
													</label>
													
													<div class="floatleft full">
														<xsl:call-template name="createTextArea">
															<xsl:with-param name="name" select="'message'"/>
															<xsl:with-param name="maxlength" select="'1024'"/>
														</xsl:call-template>
												  </div>
												</div>
												<xsl:if test="$queueSlot/Queue/useWaitingPeriod='true'">
												  	
												  	<label for="useWaitingPeriod" class="floatleft clearboth">
														<xsl:value-of select="$i18n.WaitingPeriod" />
													</label>
													
													<p class="tiny floatleft full">
														<xsl:value-of select="$i18n.WaitingPeriod.Description" />
												  	</p>
													
													<div class="floatleft full">
													<xsl:call-template name="createTextField">
														<xsl:with-param name="name" select="'waitingPeriod'" />
														<xsl:with-param name="id" select="'waitingPeriod'" />
														<xsl:with-param name="size" select="10" />
														<xsl:with-param name="type" select="'date'"/>
													</xsl:call-template>
													</div>
												</xsl:if>
												
												<input class="btn btn-red close bigmargintop floatright" type="button" value="{$i18n.ArchiveQueueSlot}"/>
												
											</div>
											
										</div>
									</div>
								
								</div>
								
								<a href="#" onclick="commentModal(event, this)" title="{$i18n.ArchiveQueueSlot}: {$queueSlot/position}" class="btn btn-light">
									<span data-icon-before="l"><xsl:value-of select="$i18n.ArchiveQueueSlot"/></span>
								</a>
								
							</form>
						</li>
						
					</xsl:otherwise>
				</xsl:choose>
				
			</ul>
		</div>
	
	</xsl:template>
	
	<xsl:template match="ShowQueueSlot">
	
		<div class="panel-wrapper official">
			<xsl:call-template name="showQueueSlotControlPanel"/>
		</div>
	
		<section class="child">
			<div class="section-inside step">
				<div class="heading-wrapper">
					<div class="inner">
						<div class="heading">
						
							<h1 class="xl">
							
								<xsl:choose>
									<xsl:when test="QueueSlot/archived = 'true'">
									
										<xsl:value-of select="$i18n.ShowQueueSlot.Archived.title"/>
									
									</xsl:when>
									<xsl:otherwise>
										
										<xsl:value-of select="$i18n.ShowQueueSlot.title"/>
										<xsl:text>:&#160;</xsl:text>
										<xsl:value-of select="QueueSlot/position" />
										
									</xsl:otherwise>
								</xsl:choose>
								
								<div class="only-mobile">
									<xsl:value-of select="$i18n.Queue.Name" />
									<xsl:text>:&#160;</xsl:text>
									<xsl:value-of select="QueueSlot/Queue/name" />
								</div>
								
								<div class="only-mobile">
									<xsl:value-of select="$i18n.QueueSlot.Added" />
									<xsl:text>:&#160;</xsl:text>
									<xsl:value-of select="QueueSlot/added" />
								</div>
								
								<xsl:if test="QueueSlot/archivedDate">
									<div class="only-mobile">
										<xsl:value-of select="$i18n.QueueSlot.ArchivedDate" />
										<xsl:text>:&#160;</xsl:text>
										<xsl:value-of select="QueueSlot/archivedDate" />
									</div>
								</xsl:if>
								
								<div class="only-mobile">
									<xsl:value-of select="$i18n.QueueSlot.LatestPayment" />
									<xsl:text>:&#160;</xsl:text>
									
									<xsl:choose>
										<xsl:when test="QueueSlot/lastPaymentDate">
											<xsl:value-of select="QueueSlot/lastPaymentDate" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$i18n.NotPayed" />
										</xsl:otherwise>
									</xsl:choose>
								</div>
								
								<xsl:if test="QueueSlot/pendingPaymentRequestDate">
									<div class="only-mobile">
										<xsl:value-of select="$i18n.QueueSlot.PaymentRequested" />
										<xsl:text>:&#160;</xsl:text>
										<xsl:value-of select="QueueSlot/pendingPaymentRequestDate" />
									</div>
								</xsl:if>
								
								<xsl:if test="QueueSlot/Attributes/Attribute[Name = 'description']">
									<div class="only-mobile">
										<xsl:value-of select="$i18n.QueueSlot.Attribute.Description" />
										<xsl:text>:&#160;</xsl:text>
										<xsl:value-of select="QueueSlot/Attributes/Attribute[Name = 'description']/Value" />
									</div>
								</xsl:if>
								
							</h1>
							
							<div class="hide-mobile">
							
								<span class="errandno">
									<div>
										<xsl:value-of select="$i18n.Queue.Name" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="QueueSlot/Queue/name" />
									</div>
									
									<div>
										<xsl:value-of select="$i18n.QueueSlot.Added" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="QueueSlot/added" />
									</div>

									<xsl:if test="QueueSlot/archivedDate">
										<div>
											<xsl:value-of select="$i18n.QueueSlot.ArchivedDate" /><xsl:text>:&#160;</xsl:text><xsl:value-of select="QueueSlot/archivedDate" />
											<xsl:if test="QueueSlot/waitingPeriod">
												(<xsl:value-of select="$i18n.WaitingPeriodInformation"/> <xsl:value-of select="QueueSlot/waitingPeriod"/>)
											</xsl:if>
										</div>
									</xsl:if>

									<div>
										<xsl:value-of select="$i18n.QueueSlot.LatestPayment" /><xsl:text>:&#160;</xsl:text>
										
										<xsl:choose>
											<xsl:when test="QueueSlot/lastPaymentDate">
												<xsl:value-of select="QueueSlot/lastPaymentDate" />
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="$i18n.NotPayed" />
											</xsl:otherwise>
										</xsl:choose>
									</div>
									
									<xsl:if test="QueueSlot/pendingPaymentRequestDate">
										<div>
											<xsl:value-of select="$i18n.QueueSlot.PaymentRequested" />
											<xsl:text>:&#160;</xsl:text>
											<xsl:value-of select="QueueSlot/pendingPaymentRequestDate" />
										</div>
									</xsl:if>
									
									<div>
										<xsl:value-of select="$i18n.QueueSlot.Attribute.Description" />
										<xsl:text>:&#160;</xsl:text>
										<xsl:value-of select="QueueSlot/Attributes/Attribute[Name = 'description']/Value" />
										
										<a class="bigmarginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatedescription/{QueueSlot/queueSlotID}" title="{$i18n.UpdateQueueSlotDescription}">
											<span data-icon-before="w"><xsl:value-of select="$i18n.Update"/></span>
										</a>
									</div>
									
								</span>
							
							</div>
							
						</div>
						
						<xsl:if test="validationError">
							<div class="bigmarginbottom">
								<xsl:apply-templates select="validationError"/>
							</div>
						</xsl:if>
						
					</div>
 				</div>
 			</div>
 			
 			<div class="floatleft full border-box" style="padding: 0 3.9283784%">
 			
 				<div class="fifty floatleft paddingright border-box" style="min-width: 300px;">
					<fieldset class="floatleft border-box full border-radius-medium border bigpadding" style="min-width: 300px;">
						<legend class="">
							<xsl:value-of select="$i18n.PrimaryContact"/>
						</legend>
						
						<xsl:call-template name="Contact">
							<xsl:with-param name="contact" select="QueueSlot/PrimaryContact"/>
						</xsl:call-template>
					</fieldset>
				</div>
				
				<div class="fifty floatleft paddingleft border-box" style="min-width: 300px;">
					<fieldset class="floatleft border-box full border-radius-medium border bigpadding" style="min-width: 300px;">
			
						<xsl:choose>
							<xsl:when test="QueueSlot/Poster">
							
								<legend>
									<xsl:value-of select="$i18n.UserContact"/>
								</legend>
								
								<xsl:call-template name="Contact-User">
									<xsl:with-param name="user" select="QueueSlot/Poster/user"/>
								</xsl:call-template>
							
							</xsl:when>
							<xsl:when test="QueueSlot/ManualContact">
								
								<legend>
									<xsl:value-of select="$i18n.ManualContact"/>
								</legend>
								
								<a class="marginright floatright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatecontact/{QueueSlot/queueSlotID}">
									<span data-icon-before="w"><xsl:value-of select="$i18n.Update"/></span>
								</a>
								
								<xsl:call-template name="Contact">
									<xsl:with-param name="contact" select="QueueSlot/ManualContact"/>
								</xsl:call-template>
								
							</xsl:when>
							<xsl:otherwise>
							
								<legend>
									<xsl:value-of select="$i18n.ManualContact"/>
								</legend>
								
								<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatecontact/{QueueSlot/queueSlotID}">
									<span data-icon-before="w"><xsl:value-of select="$i18n.Add"/></span>
								</a>
							
							</xsl:otherwise>
						</xsl:choose>
				
					</fieldset>
				</div>
				
 			</div>
  			
			<div id="tabs">
 				<ul class="tabs">
 					<li class="active">
 						<a data-icon-before="o" href="#history"><xsl:value-of select="$i18n.QueueSlotEvents" /></a>
 					</li>
 					
 					<xsl:choose>
 						<xsl:when test="QueueSlot/Digital">
 							<li data-tabid="#messages">
		  						<a data-icon-before="m" href="#messages">
		  							<xsl:value-of select="$i18n.ExternalMessages" />
		  							<!-- TODO count how many unread messages since last login -->
		  							<xsl:if test="false()">
		  								<span class="count">0</span>
		  							</xsl:if>
		  						</a>
	  						</li>
 						</xsl:when>
 						<xsl:when test="QueueSlot/externalMessages/ExternalMessage">
 							<li data-tabid="#messages">
		  						<a data-icon-before="m" href="#messages">
		  							<xsl:value-of select="$i18n.ExternalMessages" />
		  						</a>
	  						</li>
 						</xsl:when>
 					</xsl:choose>
 					
 					<li data-tabid="#notes" class="notes">
 						<a data-icon-before="i" href="#notes"><xsl:value-of select="$i18n.InternalMessages" /></a>
 					</li>
 				</ul>
 				
 				<xsl:if test="QueueSlot/Digital or QueueSlot/externalMessages/ExternalMessage">
  				<div id="messages">
  					
  					<xsl:if test="QueueSlot/Digital">
	  					<div id="new-message" class="tabs-content">
	  						
	  						<div class="heading-wrapper">
	  							<h2><xsl:value-of select="$i18n.NewMessage" /></h2>
	  							<a href="#" class="btn btn-light btn-right close_message"><xsl:value-of select="$i18n.Close" /><i data-icon-after="x"></i></a>
	  						</div>
	  						
	  						<form action="{/Document/requestinfo/uri}?externalmessage#messages" method="post" enctype="multipart/form-data">
	  							
	  							<div id="message-quote" style="display: none;">
	  								<a href="#" class="btn btn-light btn-right stop_quote">
	  									<xsl:value-of select="$i18n.Message.StopQuote" />
	  									<i data-icon-after="x"/>
	  								</a>
	  								
	  								<label><xsl:value-of select="$i18n.Message.Quote" /></label>
	  								<input type="hidden" name="quotedMessageID" disabled="true"/>
	  								<div class="quote clearboth bigmarginbottom"/>
	  							</div>
	  							
		  						<label class="required" for="message"><xsl:value-of select="$i18n.Message" /></label>
		  						<xsl:apply-templates select="validationError[fieldName = 'externalmessage']" />
		  						<textarea id="message" name="externalmessage" class="full"></textarea>
		  						<div class="heading-wrapper">
		  							<label><xsl:value-of select="$i18n.AttachFiles" /></label>
		  						</div>
		  						
		  						<script>
									imagePath = '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics';
									deleteFile = '<xsl:value-of select="$i18n.DeleteFile" />';
								</script>
								
								<xsl:apply-templates select="validationError[messageKey = 'FileSizeLimitExceeded' or messageKey = 'UnableToParseRequest']" />
								
								<div class="full">
									
									<div class="upload clearboth">
										<span class="btn btn-upload btn-blue">
											<xsl:value-of select="$i18n.ChooseFiles" />
											<input id="external-message" type="file" name="attachments" multiple="multiple" size="55" class="qloader externalmessages bigmarginbottom" />
										</span>
										<span><xsl:value-of select="$i18n.MaximumFileSize" />: <xsl:value-of select="FormatedMaxMessageFileSize" /></span>
									</div>
									
									<ul id="external-message-qloader-filelist" class="files" />
									
								</div>
								
		  						<input type="submit" value="{$i18n.SubmitMessage}" name="externalmessage" class="btn btn-green btn-inline" />
		  						<a href="#" class="btn btn-light btn-inline close_message"><xsl:value-of select="$i18n.Cancel" /></a>
		  						
	  						</form>
	  						
	  					</div>
  					</xsl:if>
  					
  					<div class="tabs-content">
	  					
	  					<div class="heading-wrapper">
	  						<h2><xsl:value-of select="$i18n.ExternalMessages" /></h2>
	  						
	  						<xsl:choose>
	  							<xsl:when test="QueueSlot/Digital">
	  								<a href="#" class="btn btn-blue btn-right open_message"><i data-icon-before="+"></i><xsl:value-of select="$i18n.NewMessage" /></a>
	  							</xsl:when>
	  							<xsl:otherwise>
	  								<a href="#" class="btn btn-light btn-right"><i data-icon-before="+"></i><xsl:value-of select="$i18n.NewMessageDisabled" /></a>
	  							</xsl:otherwise>
	  						</xsl:choose>
	  					</div>
	  					
	  					<xsl:choose>
	  						<xsl:when test="QueueSlot/externalMessages/ExternalMessage">
								
								<xsl:call-template name="sortMessagesButton">
									<xsl:with-param name="defaultOrder" select="DefaultMessageOrder" />
								</xsl:call-template>
																
	  							<ul class="messages">
	  								<xsl:apply-templates select="QueueSlot/externalMessages/ExternalMessage" mode="admin" />
	  							</ul>
	  						</xsl:when>
	  						<xsl:otherwise>
	  							<xsl:value-of select="$i18n.NoExternalMessages" />
	  						</xsl:otherwise>	  					
	  					</xsl:choose>
	  					
	  				</div>
	  				
  				</div>
 				</xsl:if>
 				
 				<div id="history" class="tabs-content nopadding" >
 					
 					<div class="errands-wrapper">
  					
	  					<div class="heading-wrapper">
		  					<h2><xsl:value-of select="$i18n.QueueSlotEvents" /></h2>
	  					</div>
	  					
	  					<table class="oep-table">
	  						<thead class="errand">
	  							<tr>
	  								<th class="service active"><span><xsl:value-of select="$i18n.Action" /></span></th>
		  							<th class="details"><span><xsl:value-of select="$i18n.Details" /></span></th>
	  								<th class="date"><span><xsl:value-of select="$i18n.Date" /></span></th>
	  								<th class="person"><span><xsl:value-of select="$i18n.Person" /></span></th>
	  							</tr>
	  						</thead>
	  						<tbody>
	  							<xsl:choose>
	  								<xsl:when test="QueueSlot/Events/QueueSlotEvent">
	  									<xsl:apply-templates select="QueueSlot/Events/QueueSlotEvent" />
	  								</xsl:when>
	  								<xsl:otherwise>
	  									<tr><td class="icon" /><td colspan="5"><xsl:value-of select="$i18n.NoEvents" /></td></tr>
	  								</xsl:otherwise>
	  							</xsl:choose>
	  						</tbody>
	  					</table>
  					
  					</div>
  				
  			</div>
  			
	 			<div id="notes">
		  			
  				<div id="new-note" class="tabs-content notes">
  				
 						<div class="heading-wrapper">
 							<h2><xsl:value-of select="$i18n.NewInternalMessage" /></h2>
 							<a href="#" class="btn btn-light btn-right close_message"><xsl:value-of select="$i18n.Close" /><i data-icon-after="x"></i></a>
 						</div>
 						
 						<form action="{/Document/requestinfo/uri}?internalmessage#notes" method="post" enctype="multipart/form-data">
 						
  						<label class="required" for="message"><xsl:value-of select="$i18n.InternalMessage" /></label>
  						<xsl:apply-templates select="validationError[fieldName = 'internalmessage']" />
  						<textarea id="message" name="internalmessage" class="full"></textarea>
  						<div class="heading-wrapper">
  							<label><xsl:value-of select="$i18n.AttachFiles" /></label>
  						</div>
  						
  						<script>
							imagePath = '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics';
							deleteFile = '<xsl:value-of select="$i18n.DeleteFile" />';
						</script>
						
						<xsl:apply-templates select="validationError[messageKey = 'FileSizeLimitExceeded' or messageKey = 'UnableToParseRequest']" />
						
						<div class="full">
							
							<div class="upload clearboth">
								<span class="btn btn-upload btn-blue">
									<xsl:value-of select="$i18n.ChooseFiles" />
									<input id="internal-message" type="file" name="attachments" multiple="multiple" size="55" class="qloader internalmessages bigmarginbottom" />
								</span>
								<span><xsl:value-of select="$i18n.MaximumFileSize" />: <xsl:value-of select="FormatedMaxMessageFileSize" /></span>
							</div>
							
							<ul id="internal-message-qloader-filelist" class="files" />
							
						</div>
						
  						<input type="submit" value="{$i18n.SubmitInternalMessage}" name="internalmessage" class="btn btn-green btn-inline" />
  						<a href="#" class="btn btn-light btn-inline close_message"><xsl:value-of select="$i18n.Cancel" /></a>
  						
 						</form>
 						
 					</div>
 					
 					<div class="tabs-content notes">
  					
  					<div class="heading-wrapper">
  						<h2><xsl:value-of select="$i18n.InternalMessagesTitle" /></h2>
  						<a href="#" class="btn btn-blue btn-right open_message"><i data-icon-before="+"></i><xsl:value-of select="$i18n.NewInternalMessage" /></a>
  					</div>
  					
  					<xsl:choose>
  						<xsl:when test="QueueSlot/internalMessages/InternalMessage">
  						
							<xsl:call-template name="sortMessagesButton">
								<xsl:with-param name="type" select="'internal'"/>
							</xsl:call-template>				
											
  							<ul class="messages">
  								<xsl:apply-templates select="QueueSlot/internalMessages/InternalMessage" mode="admin" />
  							</ul>
  						</xsl:when>
  						<xsl:otherwise>
  							<xsl:value-of select="$i18n.NoInternalMessages" />
  						</xsl:otherwise>	  					
  					</xsl:choose>
  					
  				</div>
  				
  			</div>
  			
 			</div>
 			
 			<script type="text/javascript">
					
				$(function(){
						
					$("#tabs").tabs();
				});
			</script>
 			
		</section>
		
	</xsl:template>
	
	<xsl:template name="Contact">
		<xsl:param name="contact" select="null"/>
		
		<div class="floatleft">
			<label class="floatleft"><xsl:value-of select="$i18n.Contact.Name" /></label>
			
			<div class="floatleft clearboth">
				<xsl:value-of select="$contact/firstname" />
					<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$contact/lastname" />
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.CitizenIdentifier" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/citizenIdentifier" />
				</div>
			</div>
			
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.Email" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/email" />
				</div>
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.Phone" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/phone" />
				</div>
			</div>
			
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.MobilePhone" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/mobilePhone" />
				</div>
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.Address" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/address" />
				</div>
			</div>
			
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.PostalCode" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/zipCode" />
				</div>
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.PostalAddress" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$contact/postalAddress" />
				</div>
			</div>
		
		</div>
	
	</xsl:template>
	
	<xsl:template name="Contact-User">
		<xsl:param name="user" select="null"/>
		
		<div class="floatleft">
			<label class="floatleft"><xsl:value-of select="$i18n.Contact.Name" /></label>
			
			<div class="floatleft clearboth">
				<xsl:value-of select="$user/firstname" />
					<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$user/lastname" />
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.CitizenIdentifier" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/Attributes/Attribute[Name = 'citizenIdentifier']/Value" />
				</div>
			</div>
			
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.Email" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/email" />
				</div>
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.Phone" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/Attributes/Attribute[Name = 'phone']/Value" />
				</div>
			</div>
			
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.MobilePhone" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/Attributes/Attribute[Name = 'mobilePhone']/Value" />
				</div>
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.Address" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/Attributes/Attribute[Name = 'address']/Value" />
				</div>
			</div>
			
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.PostalCode" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/Attributes/Attribute[Name = 'zipCode']/Value" />
				</div>
			</div>
		</div>
		
		<div class="floatleft full">
			<div class="floatleft fifty">
				<label class="floatleft"><xsl:value-of select="$i18n.Contact.PostalAddress" /></label>
				
				<div class="floatleft clearboth">
					<xsl:value-of select="$user/Attributes/Attribute[Name = 'postalAddress']/Value" />
				</div>
			</div>
		
		</div>
	
	</xsl:template>
	
	<xsl:template match="ExternalMessage" mode="admin">
		
		<xsl:variable name="messageTypeText">
			<xsl:call-template name="getExternalMessageTypeText"/>
		</xsl:variable>
	
		<xsl:call-template name="createAdminMessage">
			<xsl:with-param name="message" select="." />
			<xsl:with-param name="attachments" select="attachments/ExternalMessageAttachment" />
			<xsl:with-param name="prefix" select="'messages'" />
			<xsl:with-param name="messageTypeText" select="$messageTypeText" />
			<xsl:with-param name="repliesEnabled" select="true()" />
		</xsl:call-template>
		
	</xsl:template>	
		
	<xsl:template match="AddQueueSlot">
	
		<section class="settings">
		
			<div class="heading-wrapper">
				<h1>
					<xsl:value-of select="$i18n.AddQueueSlot" />
				</h1>
			</div>
		
			<form method="POST" action="{/Document/requestinfo/uri}" name="queueSlotForm">
			
				<article>
			
					<xsl:if test="validationException/validationError">
						<div class="bigmarginbottom">
							<xsl:apply-templates select="validationException/validationError"/>
						</div>
					</xsl:if>
					
					<div class="floatleft full bigmarginbottom">
						<label for="citizenIdentifier" class="required full">
							<xsl:value-of select="$i18n.Contact.CitizenIdentifier" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'citizenIdentifier'" />
							<xsl:with-param name="id" select="'citizenIdentifier'" />
							<xsl:with-param name="width" select="'300px'" />
							<xsl:with-param name="placeholder" select="$i18n.Contact.CitizenIdentifier.placeholder" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="addedDate" class="full">
							<xsl:value-of select="$i18n.QueueSlot.Added" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'addedDate'" />
							<xsl:with-param name="id" select="'addedDate'" />
							<xsl:with-param name="type" select="'date'" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full">
		
						<xsl:call-template name="createCheckbox">
							<xsl:with-param name="id" select="'payed'" />
							<xsl:with-param name="name" select="'payed'" />
							<xsl:with-param name="value" select="'true'" />
						</xsl:call-template>
						
						<label for="payed" class="checkbox">
							<xsl:value-of select="$i18n.AddPayment" />
						</label>
					</div>
					
					<div class="floatleft full bigmarginbottom" id="paydatecontainer">
						<label for="paymentDate" class="full">
							<xsl:value-of select="$i18n.Payment.Date" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'paymentDate'" />
							<xsl:with-param name="id" select="'paymentDate'" />
							<xsl:with-param name="type" select="'date'" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom"  id="payedamountcontainer">
						<label for="paymentAmount" class="required full">
							<xsl:value-of select="$i18n.Payment.Amount" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="'paymentAmount'" />
							<xsl:with-param name="name" select="'paymentAmount'" />
							<xsl:with-param name="value" select="Queue/price" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
					</div>					
					
					<div class="floatleft full bigmarginbottom">
						<label for="comment" class="full">
							<xsl:value-of select="$i18n.Comment" />
						</label>
						
						<span><xsl:value-of select="$i18n.AddQueueSlot.Description"/></span>
						
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="name" select="'comment'" />
							<xsl:with-param name="id" select="'comment'" />
							<xsl:with-param name="rows" select="4" />
						</xsl:call-template>
					</div>
				
				</article>
				
				<script type="text/javascript">
				
					$(function(){
					
						var payedInput = $("#payed");
						
						var showHidePaymentFields = function(){
							
							if(payedInput.is(":checked")){
							
								$("#paydatecontainer, #payedamountcontainer").show();
							
							}else{
							
								$("#paydatecontainer, #payedamountcontainer").hide();
							}
						};
					
						showHidePaymentFields();
						
						$( "#paymentDate" ).datepicker({
							showOn: "button",
							buttonImage: '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics/calendar_grid.png',
							buttonImageOnly: true,
							buttonText: '<xsl:value-of select="$i18n.Payment.Date"/>',
							changeMonth: true,
		            		changeYear: true,
		            		showWeek: true
						});						
						
						$( "#addedDate" ).datepicker({
							showOn: "button",
							buttonImage: '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics/calendar_grid.png',
							buttonImageOnly: true,
							buttonText: '<xsl:value-of select="$i18n.QueueSlot.Added"/>',
							changeMonth: true,
		            		changeYear: true,
		            		showWeek: true
						});						
						
						payedInput.on("change", function(){
							showHidePaymentFields();	
						});
					});
				</script>
				
				<article class="buttons">
					<input type="submit" value="{$i18n.Add}" class="btn btn-green btn-inline next" />
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{Queue/queueID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
				</article>
			
			</form>
		
		</section>
		
	</xsl:template>
	
	<xsl:template match="UpdateQueueSlot">
	
		<div class="panel-wrapper official">
			<xsl:call-template name="showQueueSlotControlPanel"/>
		</div>
		
		<section class="settings">
			<div class="heading-wrapper">
				<h1>
					<xsl:value-of select="$i18n.UpdateQueueSlot" />
				</h1>
			</div>
			
			<form method="POST" action="{/Document/requestinfo/uri}" name="queueSlotForm">
			
				<article>
				
					<xsl:if test="validationException/validationError">
						<div class="bigmarginbottom">
							<xsl:apply-templates select="validationException/validationError"/>
						</div>
					</xsl:if>
					
					<div class="floatleft full bigmarginbottom">
						<label for="position" class="required full">
							<xsl:value-of select="$i18n.QueueSlot.Position" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'position'" />
							<xsl:with-param name="id" select="'position'" />
							<xsl:with-param name="value" select="QueueSlot/position" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
						
						<div class="floatleft full bigmarginbottom">
							<label for="comment" class="full">
								<xsl:value-of select="$i18n.Comment" />
							</label>
							
							<span><xsl:value-of select="$i18n.UpdateQueueSlot.Description"/></span>
							
							<xsl:call-template name="createTextArea">
								<xsl:with-param name="name" select="'comment'" />
								<xsl:with-param name="id" select="'comment'" />
								<xsl:with-param name="rows" select="4" />
							</xsl:call-template>
						</div>
					</div>
				
				</article>
			
				<article class="buttons floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/show/{QueueSlot/queueSlotID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
					<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline next" />
				</article>
			
			</form>
			
		</section>
		
	</xsl:template>
	
	<xsl:template match="UpdateQueueSlotDescription">
	
		<div class="panel-wrapper official">
			<xsl:call-template name="showQueueSlotControlPanel"/>
		</div>
		
		<section class="settings">
			<div class="heading-wrapper">
				<h1>
					<xsl:value-of select="$i18n.UpdateQueueSlotDescription" />
				</h1>
			</div>
			
			<form method="POST" action="{/Document/requestinfo/uri}" name="queueSlotDescriptionForm">
			
				<article>
				
					<xsl:if test="ValidationErrors/validationError">
						<div class="bigmarginbottom">
							<xsl:apply-templates select="ValidationErrors/validationError"/>
						</div>
					</xsl:if>
					
					<div class="floatleft full bigmarginbottom">
						<label for="position" class="full">
							<xsl:value-of select="$i18n.QueueSlot.Attribute.Description" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'description'" />
							<xsl:with-param name="id" select="'description'" />
							<xsl:with-param name="value" select="QueueSlot/Attributes/Attribute[Name = 'description']/Value" />
						</xsl:call-template>
					</div>
				
				</article>
			
				<article class="buttons floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/show/{QueueSlot/queueSlotID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
					<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline next" />
				</article>
			
			</form>
			
		</section>
		
	</xsl:template>
	
	<xsl:template match="AddPayment">
	
		<div class="panel-wrapper official">
			<xsl:call-template name="showQueueSlotControlPanel"/>
		</div>
		
		<section class="settings">
			<div class="heading-wrapper">
				<h1>
					<xsl:value-of select="$i18n.AddPayment" />
				</h1>
			</div>
			
			<form method="POST" action="{/Document/requestinfo/uri}" name="addPaymentForm">
			
				<article>
				
					<xsl:if test="ValidationErrors/validationError">
						<div class="bigmarginbottom">
							<xsl:apply-templates select="ValidationErrors/validationError"/>
						</div>
					</xsl:if>
					
					<div class="floatleft full bigmarginbottom">
						<label for="paymentDate" class="full">
							<xsl:value-of select="$i18n.Payment.Date" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'paymentDate'" />
							<xsl:with-param name="id" select="'paymentDate'" />
							<xsl:with-param name="type" select="'date'" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
					</div>
					
					<script>
						$(function() {
							$( "#paymentDate" ).datepicker({
								showOn: "button",
								buttonImage: '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics/calendar_grid.png',
								buttonImageOnly: true,
								buttonText: '<xsl:value-of select="$i18n.Payment.Date"/>',
								changeMonth: true,
			            		changeYear: true,
			            		showWeek: true
							});
						});
					</script>
					
					<div class="floatleft full bigmarginbottom">
						<label for="paymentAmount" class="required full">
							<xsl:value-of select="$i18n.Payment.Amount" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'paymentAmount'" />
							<xsl:with-param name="id" select="'paymentAmount'" />
							<xsl:with-param name="value" select="QueueSlot/Queue/price" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
					</div>
				
				</article>
			
				<article class="buttons floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/show/{QueueSlot/queueSlotID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
					<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline next" />
				</article>
			
			</form>
			
		</section>
		
	</xsl:template>
	
	<xsl:template match="RequestPayment">
	
		<h1>
			<xsl:value-of select="$i18n.RequestPaymentTitle"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="Queue/name"/>
		</h1>
		
		<br/>
		
		<xsl:if test="NewPaymentRequests">
			
			<div class="border-radius-small lightbackground padding bigmarginbottom">
				<strong>
					<xsl:value-of select="$i18n.RequestPayment.Sent.1"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="NewPaymentRequests"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.RequestPayment.Sent.2"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="SentEmails"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.RequestPayment.Sent.3"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="SentSMSs"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.RequestPayment.Sent.4"/>
				</strong>
			</div>
			
		</xsl:if>
		
		<xsl:apply-templates select="ValidationErrors/validationError" />
		
		<form method="post" action="{/Document/requestinfo/uri}">
			<xsl:if test="SelectedQueueSlots">
		
				<div>
					<p><xsl:value-of select="count(SelectedQueueSlots/queueSlotID)"/>
						<xsl:choose>
							<xsl:when test="count(SelectedQueueSlots/queueSlotID)=1">
								<xsl:value-of select="$i18n.SelectedQueueSlot" /> 
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$i18n.SelectedQueueSlots" /> 
							</xsl:otherwise>
						</xsl:choose>
					</p>							
				</div>
				<xsl:for-each select="SelectedQueueSlots/queueSlotID">
					<input type="hidden" name="queueSlotID" >
						<xsl:attribute name="value"><xsl:value-of select="."/></xsl:attribute>
					</input>
				</xsl:for-each>
			</xsl:if>
			
			<div class="floatleft full bigmargintop bigmarginbottom">
			
				<h2><xsl:value-of select="$i18n.RequestPayment.recipientType.title"/></h2>
		
				<p class="nomargin"><xsl:value-of select="$i18n.RequestPayment.recipientType.description"/></p>
							
				<div class="floatleft full">
							
					<label for="olderThan">
							<xsl:value-of select="$i18n.RequestPayment.olderThanDate" />
					</label>
					
					<div class="floatleft full">
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'olderThanDate'" />
							<xsl:with-param name="id" select="'olderThanDate'" />
							<xsl:with-param name="type" select="'date'" />
							<xsl:with-param name="size" select="10" />
						</xsl:call-template>
					</div>
				</div>
				
				<script type="text/javascript">
			
					$(function(){
						
						if (!Modernizr.inputtypes.date) {
					
							$( "#olderThanDate" ).datepicker({
								showOn: "button",
								buttonImage: '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics/calendar_grid.png',
								buttonImageOnly: true,
								buttonText: '<xsl:value-of select="$i18n.RequestPayment.olderThanDate"/>',
								changeMonth: true,
						        changeYear: true,
						        showWeek: true
							});
						}						
					});
					
				</script>
				
				<div class="floatleft full bigmarginbottom">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'override-user-sms-payment-settings'" />
						<xsl:with-param name="name" select="'override-user-sms-payment-settings'" />
					</xsl:call-template>
					
					<label for="override-user-sms-settings">
						<xsl:value-of select="$i18n.SendMessage.OverrideUserSMSSettings" />
					</label>
						<p class="tiny floatleft full">
							<xsl:value-of select="$i18n.SendMessage.OverrideUserSMSSettings.Information" />
					  	</p>
				</div>
			
			</div>
			
			<div class="floatright bigmargintop">
			
				<a class="btn btn-light btn-inline marginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{Queue/queueID}">
					<xsl:value-of select="$i18n.BackToListQueueSlots" />
				</a>
			
				<input class="btn btn-green btn-inline" type="submit" value="{$i18n.RequestPayment.SendPaymentRequests}"/>
			</div>
		
		</form>
		
	</xsl:template>
	
	<xsl:template match="UpdateContact">
	
		<div class="panel-wrapper official">
			<xsl:call-template name="showQueueSlotControlPanel"/>
		</div>
		
		<section class="settings">
			<div class="heading-wrapper">
				<h1>
					<xsl:value-of select="$i18n.UpdateContact" />
				</h1>
			</div>
			
			<form method="POST" action="{/Document/requestinfo/uri}" name="contactForm">
			
				<article>
				
					<xsl:if test="validationError">
						<div class="bigmarginbottom">
							<xsl:apply-templates select="validationError"/>
						</div>
					</xsl:if>
					
					<div class="floatleft full bigmarginbottom">
						<label for="firstname" class="required full">
							<xsl:value-of select="$i18n.Contact.FirstName" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'firstname'" />
							<xsl:with-param name="id" select="'firstname'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="lastname" class="required full">
							<xsl:value-of select="$i18n.Contact.LastName" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'lastname'" />
							<xsl:with-param name="id" select="'lastname'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="email" class="full">
							<xsl:value-of select="$i18n.Contact.Email" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'email'" />
							<xsl:with-param name="id" select="'email'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="phone" class="full">
							<xsl:value-of select="$i18n.Contact.Phone" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'phone'" />
							<xsl:with-param name="id" select="'phone'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="mobilePhone" class="full">
							<xsl:value-of select="$i18n.Contact.MobilePhone" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'mobilePhone'" />
							<xsl:with-param name="id" select="'mobilePhone'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="address" class="full">
							<xsl:value-of select="$i18n.Contact.Address" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'address'" />
							<xsl:with-param name="id" select="'address'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="zipCode" class="full">
							<xsl:value-of select="$i18n.Contact.PostalCode" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'zipCode'" />
							<xsl:with-param name="id" select="'zipCode'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
					
					<div class="floatleft full bigmarginbottom">
						<label for="postalAddress" class="full">
							<xsl:value-of select="$i18n.Contact.PostalAddress" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="'postalAddress'" />
							<xsl:with-param name="id" select="'postalAddress'" />
							<xsl:with-param name="element" select="QueueSlot/ManualContact" />
						</xsl:call-template>
					</div>
				
				</article>
			
				<article class="buttons floatright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/show/{QueueSlot/queueSlotID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
					<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline next" />
				</article>
			
			</form>
			
		</section>
		
	</xsl:template>
	
	<xsl:template match="ImportExcel">
	
		<h1>
			<xsl:value-of select="$i18n.ImportExcel.title" />
		</h1>
				
		<p>
			<xsl:value-of select="$i18n.ImportExcel.description" />
		</p>			
	
		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}" enctype="multipart/form-data">

			<div class="floatleft full bigmarginbottom">
				
				<label for="file" class="floatleft full">
					<xsl:value-of select="$i18n.ImportExcel.selectFile" />
				</label>
				
				<div class="floatleft full">
					<input type="file" name="file" id="file"/>
				</div>
			</div>
			
			<div class="floatright">
				<input type="submit" value="{$i18n.ImportExcel.submit}" />
			</div>
		
		</form>
		
	</xsl:template>
	
	<xsl:template match="SendMessageForm">
	
		<h1>
			<xsl:value-of select="$i18n.SendMessageTitle"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="Queue/name"/>
		</h1>
		
		<br/>
		
		<xsl:if test="SentEmails">
			
			<div class="border-radius-small lightbackground padding bigmarginbottom">
				<strong>
					<xsl:value-of select="$i18n.SendMessage.EmailsSent"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="SentEmails"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.SendMessage.Recipients"/>
<!-- 						<xsl:text>&#160;av&#160;</xsl:text> -->
<!-- 						<xsl:value-of select="QueueSlotCount"/> -->
				</strong>
			</div>
			
		</xsl:if>

		<xsl:if test="SentSMSs">
			
			<div class="border-radius-small lightbackground padding bigmarginbottom">
				<strong>
					<xsl:value-of select="$i18n.SendMessage.SMSsSent"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="SentSMSs"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.SendMessage.Recipients"/>
				</strong>
			</div>
			
		</xsl:if>

		<xsl:if test="SentExternalMessages">
			
			<div class="border-radius-small lightbackground padding bigmarginbottom">
				<strong>
					<xsl:value-of select="$i18n.SendMessage.ExternalMessagesSent"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="SentExternalMessages"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.SendMessage.Recipients"/>
					<xsl:text>&#160;(</xsl:text>
					<xsl:value-of select="SentExternalMessagesEmailNotifications"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.SendMessage.ExternalMessagesSentEmailNotifications"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="SentExternalMessagesSMSNotifications"/>
					<xsl:text>&#160;</xsl:text>
					<xsl:value-of select="$i18n.SendMessage.ExternalMessagesSentSMSNotifications"/>
					<xsl:text>)</xsl:text>
				</strong>
			</div>
			
		</xsl:if>
		
		<xsl:apply-templates select="ValidationErrors/validationError" />

		<form id="sendMessageForm" method="post" action="{/Document/requestinfo/uri}" enctype="multipart/form-data">
			
			<div class="floatleft full">
			
			<xsl:choose>
			
				<xsl:when test="not(SelectedQueueSlots)">
					<h2 class="floatleft full bigmargintop">
						<xsl:value-of select="$i18n.SendMessage.recipientType.title" />
					</h2>
	
					<p class="floatleft full nomargin">
						<xsl:value-of select="$i18n.SendMessage.recipientType.description" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="$i18n.SendMessage.AnalogNoEmail.1" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="QueueSlotCounts/AnalogNoEmailOrPhone"/>
						<xsl:text> </xsl:text>
						<xsl:value-of select="$i18n.SendMessage.AnalogNoEmail.2" />
					</p>
	
					<div class="floatleft full">
									
						<div class="floatleft full">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'toDigital'"/>
								<xsl:with-param name="name" select="'toDigital'"/>
							</xsl:call-template>
							
							<label for="toDigital">
								<xsl:value-of select="$i18n.SendMessage.toDigital" />
								<xsl:text>&#160;(</xsl:text>
								<xsl:value-of select="QueueSlotCounts/Digital" />
								<xsl:text>)</xsl:text>
							</label>
						</div>
					</div>	
				
					<div class="floatleft full">
									
						<div class="floatleft full">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'toAnalogWithEmail'"/>
								<xsl:with-param name="name" select="'toAnalogWithEmail'"/>
							</xsl:call-template>
							
							<label for="toAnalogWithEmail">
								<xsl:value-of select="$i18n.SendMessage.toAnalogWithEmail" />
								<xsl:text>&#160;(</xsl:text>
								<xsl:value-of select="QueueSlotCounts/AnalogWithEmail" />
								<xsl:text>)</xsl:text>
							</label>
						</div>
					</div>
					
					<div class="floatleft full">
									
						<div class="floatleft full">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'toDigitalPaymentPending'" />
								<xsl:with-param name="name" select="'toDigitalPaymentPending'" />
							</xsl:call-template>
	
							<label for="toDigitalPaymentPending">
								<xsl:value-of select="$i18n.SendMessage.toDigitalPaymentPending.1" />
								<xsl:text>&#160;(</xsl:text>
								<xsl:value-of select="QueueSlotCounts/DigitalPaymentPending" />
								<xsl:text>)&#160;</xsl:text>
								<xsl:value-of select="$i18n.SendMessage.toDigitalPaymentPending.2" />
							</label>
	
							<div class="floatleft full bigmarginbottom" id="digitalpaymentpendingcontainer">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="'minimumDaysDigitalPaymentRequestHasBeenActive'" />
									<xsl:with-param name="size" select="10" />
								</xsl:call-template>
							</div>
						</div>
					</div>
	
					<div class="floatleft full">
	
						<div class="floatleft full">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'toAnalogPaymentPendingWithEmail'" />
								<xsl:with-param name="name" select="'toAnalogPaymentPendingWithEmail'" />
							</xsl:call-template>
	
							<label for="toAnalogPaymentPendingWithEmail">
								<xsl:value-of select="$i18n.SendMessage.toAnalogPaymentPendingWithEmail.1" />
								<xsl:text>&#160;(</xsl:text>
								<xsl:value-of select="QueueSlotCounts/AnalogPaymentPendingWithEmail" />
								<xsl:text>)&#160;</xsl:text>
								<xsl:value-of select="$i18n.SendMessage.toAnalogPaymentPendingWithEmail.2" />
							</label>
	
							<div class="floatleft full bigmarginbottom" id="analogpaymentpendingcontainer">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="'minimumDaysAnalogPaymentRequestHasBeenActive'" />
									<xsl:with-param name="size" select="10" />
								</xsl:call-template>
							</div>
						</div>
					</div>
				
					</xsl:when>
					<xsl:otherwise>
						<div>
							<p><xsl:value-of select="count(SelectedQueueSlots/queueSlotID)"/>
								<xsl:choose>
									<xsl:when test="count(SelectedQueueSlots/queueSlotID)=1">
										<xsl:value-of select="$i18n.SelectedQueueSlot" /> 
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="$i18n.SelectedQueueSlots" /> 
									</xsl:otherwise>
								</xsl:choose>
							</p>							
						</div>
						<xsl:for-each select="SelectedQueueSlots/queueSlotID">
							<input type="hidden" name="queueSlotID" >
								<xsl:attribute name="value"><xsl:value-of select="."/></xsl:attribute>
							</input>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>
				

				<script type="text/javascript">
			
					$(function(){

						var toDigitalPending = $("#toDigitalPaymentPending");
	
						var showHideDigitalPending = function(){
							var checked = toDigitalPending.is(":checked");
							$("#digitalpaymentpendingcontainer").toggle(checked).find("input").attr("disabled", !checked);
						};
	
						showHideDigitalPending();
	
						toDigitalPending.on("change", function(){
							showHideDigitalPending();
						});
	
	
						var toAnalogPending = $("#toAnalogPaymentPendingWithEmail");
	
						var showHideAnalogPending = function(){
							var checked = toAnalogPending.is(":checked");
							$("#analogpaymentpendingcontainer").toggle(checked).find("input").attr("disabled", !checked);
						};
	
						showHideAnalogPending();
	
						toAnalogPending.on("change", function(){
							showHideAnalogPending();
						});
	
	
						var sendEmail = $("#send-email");
						var sendSMS = $("#send-sms");
						var overrideUserSettings = $("#override-user-sms-form");
						 overrideUserSettings.toggle(sendSMS.prop("checked"));
						
						var sendExternalMessage = $("#send-external-message");
						
						var userTags = $("#user-tags");
						userTags.toggle(sendSMS.prop("checked") || sendEmail.prop("checked") || sendExternalMessage.prop("checked"));
						
						
						var sendEmailBox = $("#email-text-box");
						sendEmailBox.toggle(sendEmail.prop("checked"));
						
						sendEmail.on("click", function(){
							sendEmailBox.toggle(sendEmail.prop("checked"));
							userTags.toggle(sendSMS.prop("checked") || sendEmail.prop("checked") || sendExternalMessage.prop("checked"));
						});
						
						
						var sendSMSBox = $("#sms-text-box");
						sendSMSBox.toggle(sendSMS.prop("checked"));
						
						sendSMS.on("click", function(){
							sendSMSBox.toggle(sendSMS.prop("checked"));
							userTags.toggle(sendSMS.prop("checked") || sendEmail.prop("checked") || sendExternalMessage.prop("checked"));
							overrideUserSettings.toggle(sendSMS.prop("checked"));
						});
						
						
						var sendExternalMessageBox = $("#external-message-text-box");
						sendExternalMessageBox.toggle(sendExternalMessage.prop("checked"));
						
						sendExternalMessage.on("click", function(){
							sendExternalMessageBox.toggle(sendExternalMessage.prop("checked"));
							userTags.toggle(sendSMS.prop("checked") || sendEmail.prop("checked") || sendExternalMessage.prop("checked"));
						});
					});
				</script>
				
			</div>

			<h2 class="floatleft full bigmargintop">
				<xsl:value-of select="$i18n.SendMessage.Transport" />
			</h2>

			<p class="floatleft full nomargin">
				<xsl:value-of select="$i18n.SendMessage.Transport.description" />
			</p>

			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'send-email'" />
					<xsl:with-param name="name" select="'send-email'" />
				</xsl:call-template>
				
				<label for="send-email">
					<xsl:value-of select="$i18n.SendMessage.SendEmail" />
				</label>
			</div>
			
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'send-sms'" />
					<xsl:with-param name="name" select="'send-sms'" />
				</xsl:call-template>
				
				<label for="send-sms">
					<xsl:value-of select="$i18n.SendMessage.SendSMS" />
				</label>
			</div>
			
			<div id="override-user-sms-form" class="floatleft full bigmarginleft bigmarginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'override-user-sms-settings'" />
					<xsl:with-param name="name" select="'override-user-sms-settings'" />
				</xsl:call-template>
				
				<label for="override-user-sms-settings">
					<xsl:value-of select="$i18n.SendMessage.OverrideUserSMSSettings" />
				</label>
					<p class="tiny floatleft full">
						<xsl:value-of select="$i18n.SendMessage.OverrideUserSMSSettings.Information" />
				  	</p>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'send-external-message'" />
					<xsl:with-param name="name" select="'send-external-message'" />
				</xsl:call-template>
				
				<label for="send-external-message">
					<xsl:value-of select="$i18n.SendMessage.SendExternalMessage" />
				</label>
			</div>
			
							
			
			<div class="floatleft full" id="email-text-box">
				<fieldset class="border-radius-small lightbackground">
					<legend><xsl:value-of select="$i18n.SendMessage.Email"/></legend>
						
					<div class="floatleft full bigmarginbottom">
						<label class="floatleft twenty" for="emailSubject"><xsl:value-of select="$i18n.SendMessage.Subject"/>:</label>
						<div class="floatleft eighty">
							<xsl:call-template name="createTextField">
								<xsl:with-param name="name" select="'emailSubject'"/>
								<xsl:with-param name="id" select="'emailSubject'"/>
							</xsl:call-template>
						</div>
					</div>
					
					<label class="floatleft full" for="emailMessage"><xsl:value-of select="$i18n.SendMessage.Message"/>:</label>
						
					<div class="floatleft full bigmarginbottom">
					
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="name" select="'emailMessage'"/>
							<xsl:with-param name="id" select="'emailMessage'"/>
							<xsl:with-param name="rows" select="'8'"/>
							<xsl:with-param name="class" select="'flow-ckeditor'"/>
						</xsl:call-template>
						
						<xsl:call-template name="initializeFCKEditor">
							<xsl:with-param name="basePath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/ckeditor/</xsl:with-param>
							<xsl:with-param name="customConfig">config.js</xsl:with-param>
							<xsl:with-param name="editorContainerClass">flow-ckeditor</xsl:with-param>
							<xsl:with-param name="editorHeight">150</xsl:with-param>
							
							<xsl:with-param name="contentsCss">
								<xsl:if test="cssPath">
									<xsl:value-of select="cssPath"/>
								</xsl:if>
							</xsl:with-param>
							
							<xsl:with-param name="filebrowserBrowseUri">
								
								<xsl:choose>
									<xsl:when test="ckConnectorModuleAlias">
										<xsl:text>filemanager/index.html?Connector=</xsl:text>
										<xsl:value-of select="/Document/requestinfo/contextpath"/><xsl:value-of select="ckConnectorModuleAlias" />
										<xsl:text>/connector</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="''"/>
									</xsl:otherwise>				
								</xsl:choose>
				
							</xsl:with-param>
							
							<xsl:with-param name="filebrowserImageBrowseUri">
							
								<xsl:choose>
									<xsl:when test="ckConnectorModuleAlias">
										<xsl:text>filemanager/index.html?Connector=</xsl:text>
										<xsl:value-of select="/Document/requestinfo/contextpath"/><xsl:value-of select="ckConnectorModuleAlias" />
										<xsl:text>/connector</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="''"/>
									</xsl:otherwise>
								</xsl:choose>
							
							</xsl:with-param>
							
						</xsl:call-template>

  						<div class="heading-wrapper">
  							<label><xsl:value-of select="$i18n.AttachFiles" /></label>
  						</div>
  						
  						<script>
							imagePath = '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics';
							deleteFile = '<xsl:value-of select="$i18n.DeleteFile" />';
						</script>				
						
						<xsl:apply-templates select="validationError[messageKey = 'FileSizeLimitExceeded' or messageKey = 'UnableToParseRequest']" />

						<div class="full">
							
							<div class="upload clearboth">
								<span class="btn btn-upload btn-blue">
									<xsl:value-of select="$i18n.ChooseFiles" />
									<input id="external-message" type="file" name="email-attachments" multiple="multiple" size="55" class="qloader externalmessages bigmarginbottom" />
								</span>
								<span><xsl:value-of select="$i18n.MaximumFileSize" />: <xsl:value-of select="FormatedMaxEmailFileSize" /></span>
							</div>
							
							<ul id="external-message-qloader-filelist" class="files" />
							
						</div>
						
					</div>
						
				</fieldset>
			</div>
			
			<div class="floatleft full" id="sms-text-box">
				<fieldset class="border-radius-small lightbackground">
					<legend><xsl:value-of select="$i18n.SendMessage.SMS"/></legend>
				
					<div class="floatleft full">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="'smsMessage'" />
							<xsl:with-param name="name" select="'smsMessage'" />
							<xsl:with-param name="rows" select="5"/>
							<xsl:with-param name="class" select="'border-box'"/>
						</xsl:call-template>
					</div>
				</fieldset>
			</div>
			
			<div class="floatleft full" id="external-message-text-box">
				<fieldset class="border-radius-small lightbackground">
					<legend><xsl:value-of select="$i18n.SendMessage.ExternalMessage"/></legend>
				
					<div class="full">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="'externalMessage'" />
							<xsl:with-param name="name" select="'externalMessage'" />
							<xsl:with-param name="rows" select="5"/>
							<xsl:with-param name="class" select="'border-box'"/>
						</xsl:call-template>
					</div>
					
					<div class="heading-wrapper">
						<label><xsl:value-of select="$i18n.AttachFiles" /></label>
					</div>
						
					<script>
						imagePath = '<xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics';
						deleteFile = '<xsl:value-of select="$i18n.DeleteFile" />';
					</script>				
				
					<xsl:apply-templates select="validationError[messageKey = 'FileSizeLimitExceeded' or messageKey = 'UnableToParseRequest']" />
		
					<div class="full">
						
						<div class="upload clearboth">
							<span class="btn btn-upload btn-blue">
								<xsl:value-of select="$i18n.ChooseFiles" />
								<input id="external-message" type="file" name="external-message-attachments" multiple="multiple" size="55" class="qloader externalmessages bigmarginbottom" />
							</span>
							<span><xsl:value-of select="$i18n.MaximumFileSize" />: <xsl:value-of select="FormatedMaxMessageFileSize" /></span>
						</div>
						
						<ul id="external-message-qloader-filelist" class="files" />
						
					</div>
					
				</fieldset>
			</div>				
			
			<xsl:call-template name="addUserTagsTable"/>
			
			<div class="floatright bigmargintop">
			
				<a class="btn btn-light btn-inline marginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list/{Queue/queueID}">
					<xsl:value-of select="$i18n.BackToListQueueSlots" />
				</a>
			
				<div class="spinner-wrapper">
					<input class="btn btn-green btn-inline" type="submit" value="{$i18n.SendMessage.SubmitMessage}"/>
				</div>
				
			</div>
		
		</form>
				
		<script type="text/javascript">
			$(function(){
			
				SendMessageFormi18n = {
					SubmittingMessage: "<xsl:value-of select="$i18n.SendMessage.SubmittingMessage" />",
				}
			});
		</script>
		
	</xsl:template>
	
	<xsl:template name="addUserTagsTable">
	
		<div class="floatleft full bigmarginbottom" id="user-tags">

			<p><xsl:value-of select="$i18n.UserTagsTable.description"/></p>
		
			<table class="full border">
				<tr>
					<th>
						<xsl:value-of select="$i18n.Tag"/>
					</th>
					<th>
						<xsl:value-of select="$i18n.Description"/>
					</th>
				</tr>
				<tr>
					<td>
						<xsl:text>$queue.name</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.QueueNameTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$queueSlot.position</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.QueueSlotPositionTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$queueSlot.url</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.QueueSlotURLTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$contact.firstname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.PosterFirstnameTag"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$contact.lastname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.PosterLastnameTag"/>
					</td>
				</tr>
			</table>
		
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseFile']">
	
		<p class="error">
			<xsl:value-of select="$i18n.UnableToParseFile.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.UnableToParseFile.part2"/>
		</p>
			
	</xsl:template>

	<xsl:template match="validationError[messageKey='FileSizeLimitExceeded']">
	
		<p class="error">
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part2"/>
			<xsl:value-of select="size"/>
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part3"/>
			<xsl:value-of select="maxFileSize"/>
			<xsl:value-of select="$i18n.FileSizeLimitExceeded.part4"/>
		</p>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseRequest']">
	
		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.UnableToParseRequest" />
		</p>
				
	</xsl:template>

	<xsl:template match="validationError[messageKey='InvalidFileExtension']">
	
		<p class="error">
			<xsl:value-of select="$i18n.InvalidFileExtension.Part1" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="filename" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.InvalidFileExtension.Part2" />

			<xsl:if test="AllowedExtensions">
			
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.InvalidFileExtension.Part3" />
				<xsl:text>&#160;</xsl:text>
				
				<xsl:apply-templates select="AllowedExtensions/Extension" mode="AllowedExtensions"/>
			</xsl:if>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='NoRecipientTypesSelected']">
		<p class="error">
			<xsl:value-of select="$i18n.NoRecipientTypesSelected"/>
		</p>
	</xsl:template>

	<xsl:template match="validationError[messageKey='NoMessagePathSelected']">
		<p class="error">
			<xsl:value-of select="$i18n.NoMessagePathSelected"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='EmailSenderNotSet']">
		<p class="error">
			<xsl:value-of select="$i18n.EmailSenderNotSet"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='InvalidRecipientEmail']">
		<p class="error">
			<xsl:value-of select="$i18n.InvalidRecipientEmail.1"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="QueueSlot/position"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.InvalidRecipientEmail.2"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="email"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='InvalidRecipientPhoneNumber']">
		<p class="error">
			<xsl:value-of select="$i18n.InvalidRecipientPhoneNumber.1"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="QueueSlot/position"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.InvalidRecipientPhoneNumber.2"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="phoneNumber"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ErrorSendingToRecipientViaEmail']">
		<p class="error">
			<xsl:value-of select="$i18n.ErrorSendingToRecipientViaEmail.1"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="QueueSlot/position"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.ErrorSendingToRecipientViaEmail.2"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="email"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='ErrorSendingToRecipientViaSMS']">
		<p class="error">
			<xsl:value-of select="$i18n.ErrorSendingToRecipientViaSMS.1"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="QueueSlot/position"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.ErrorSendingToRecipientViaSMS.2"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="phoneNumber"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='DontContactRecipientBySMS']">
		<p class="error">
			<xsl:value-of select="$i18n.DontContactRecipientBySMS.1"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="QueueSlot/position"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.DontContactRecipientBySMS.2"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="phoneNumber"/>
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.DontContactRecipientBySMS.3"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<xsl:if test="fieldName and validationErrorType">
			<p class="error">
				<span data-icon-before="!">
					<xsl:choose>
						<xsl:when test="validationErrorType='RequiredField'">
							<xsl:value-of select="$i18n.validationError.RequiredField" />
						</xsl:when>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.validationError.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooShort'">
							<xsl:value-of select="$i18n.validationError.TooShort" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.validationError.TooLong" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.validationError.Other" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>:&#x20;</xsl:text>
					<xsl:choose>
						<xsl:when test="fieldName = 'payed'">
							<xsl:value-of select="$i18n.AddPayment" />
						</xsl:when>
						<xsl:when test="fieldName = 'citizenIdentifier'">
							<xsl:value-of select="$i18n.Contact.CitizenIdentifier" />
						</xsl:when>
						<xsl:when test="fieldName = 'paymentDate'">
							<xsl:value-of select="$i18n.Payment.Date" />
						</xsl:when>
						<xsl:when test="fieldName = 'paymentAmount'">
							<xsl:value-of select="$i18n.Payment.Amount" />
						</xsl:when>
						<xsl:when test="fieldName = 'firstname'">
							<xsl:value-of select="$i18n.Contact.FirstName" />
						</xsl:when>
						<xsl:when test="fieldName = 'lastname'">
							<xsl:value-of select="$i18n.Contact.LastName" />
						</xsl:when>
						<xsl:when test="fieldName = 'email'">
							<xsl:value-of select="$i18n.Contact.Email" />
						</xsl:when>
						<xsl:when test="fieldName = 'phone'">
							<xsl:value-of select="$i18n.Contact.Phone" />
						</xsl:when>
						<xsl:when test="fieldName = 'mobilePhone'">
							<xsl:value-of select="$i18n.Contact.MobilePhone" />
						</xsl:when>
						<xsl:when test="fieldName = 'address'">
							<xsl:value-of select="$i18n.Contact.Address" />
						</xsl:when>
						<xsl:when test="fieldName = 'zipCode'">
							<xsl:value-of select="$i18n.Contact.PostalCode" />
						</xsl:when>
						<xsl:when test="fieldName = 'postalAddress'">
							<xsl:value-of select="$i18n.Contact.PostalAddress" />
						</xsl:when>
						<xsl:when test="fieldName = 'postalAddress'">
							<xsl:value-of select="$i18n.Contact.PostalAddress" />
						</xsl:when>
						<xsl:when test="fieldName = 'externalmessage'">
							<xsl:value-of select="$i18n.SendMessage.Message" />
						</xsl:when>
						<xsl:when test="fieldName = 'internalmessage'">
							<xsl:value-of select="$i18n.SendMessage.Message" />
						</xsl:when>
						<xsl:when test="fieldName = 'emailSubject'">
							<xsl:value-of select="$i18n.SendMessage.Email" />
							<xsl:text>&#x20;</xsl:text>
							<xsl:value-of select="$i18n.SendMessage.Subject" />
						</xsl:when>
						<xsl:when test="fieldName = 'emailMessage'">
							<xsl:value-of select="$i18n.SendMessage.Email" />
							<xsl:text>&#x20;</xsl:text>
							<xsl:value-of select="$i18n.SendMessage.Message" />
						</xsl:when>
						<xsl:when test="fieldName = 'olderThanDate'">
							<xsl:value-of select="$i18n.RequestPayment.olderThanDate" />
						</xsl:when>
						<xsl:when test="fieldName = 'minimumDaysDigitalPaymentRequestHasBeenActive'">
							<xsl:value-of select="$i18n.SendMessage.toDigitalPaymentPending.1" />
							<xsl:text> </xsl:text>
							<xsl:value-of select="$i18n.SendMessage.toDigitalPaymentPending.2" />
						</xsl:when>
						<xsl:when test="fieldName = 'minimumDaysAnalogPaymentRequestHasBeenActive'">
							<xsl:value-of select="$i18n.SendMessage.toAnalogPaymentPendingWithEmail.1" />
							<xsl:text> </xsl:text>
							<xsl:value-of select="$i18n.SendMessage.toAnalogPaymentPendingWithEmail.2" />
						</xsl:when>
						<xsl:when test="fieldName = 'smsMessage'">
							<xsl:value-of select="$i18n.SendMessage.smsMessage" />
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessage'">
							<xsl:value-of select="$i18n.SendMessage.ExternalMessage" />
						</xsl:when>
						<xsl:when test="fieldName = 'description'">
							<xsl:value-of select="$i18n.QueueSlot.Attribute.Description" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>!</xsl:text>
				</span>
			</p>
		</xsl:if>
		<xsl:if test="messageKey">
			<p class="error">
				<span data-icon-before="!">
					<xsl:choose>
						<xsl:when test="messageKey = 'ErrorImportingCitizen'">
							<xsl:value-of select="$i18n.validationError.ErrorImportingCitizen" />!
						</xsl:when>
						<xsl:when test="messageKey = 'ErrorImportingExcel'">
							<xsl:value-of select="$i18n.validationError.ErrorImportingExcel" />
							<xsl:text>: </xsl:text>
							<xsl:value-of select="displayName" />
						</xsl:when>
						<xsl:when test="messageKey = 'PaymentDateInFuture'">
							<xsl:value-of select="$i18n.validationError.PaymentDateInFuture" />!
						</xsl:when>
						<xsl:when test="messageKey = 'PersonAlreadyInQueue'">
							<xsl:value-of select="$i18n.validationError.PersonAlreadyInQueue" />!
						</xsl:when>
						<xsl:when test="messageKey = 'PersonIsInWaitingPeriod'">
							<xsl:value-of select="$i18n.validationError.PersonIsInWaitingPeriod" /> <xsl:value-of select="displayName"/>!
						</xsl:when>
						<xsl:when test="messageKey = 'AddedDateInFuture'">
							<xsl:value-of select="$i18n.validationError.AddedDateInFuture" />!
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.validationError.unknownMessageKey" />!
						</xsl:otherwise>
					</xsl:choose>
				</span>
			</p>
		</xsl:if>
		<xsl:apply-templates select="message" />
		
	</xsl:template>
	
</xsl:stylesheet>