package com.nordicpeak.flowengine.queue.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.List;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringTag;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.queue.query.QueueQuery;

@Table(name = "queues")
@XMLElement
public class Queue extends GeneratedElementable implements AccessInterface, Serializable {

	private static final long serialVersionUID = 4175567992506321877L;

	public static final Field MANAGER_GROUPS_RELATION = ReflectionUtils.getField(Queue.class, "managerGroupIDs");
	public static final Field MANAGER_USERS_RELATION = ReflectionUtils.getField(Queue.class, "managerUserIDs");
	public static final Field QUEUE_SLOTS_RELATION = ReflectionUtils.getField(Queue.class, "queueSlots");
	public static final Field INTERNAL_GROUPS_RELATION = ReflectionUtils.getField(Queue.class, "internalGroupIDs");

	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer queueID;

	@DAOManaged
	@WebPopulate(required = true)
	@OrderBy
	@StringTag
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 65534)
	@XMLElement
	private String description;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String emailSenderName;

	@DAOManaged
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@XMLElement
	private String emailSenderAddress;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean enabled;

	@DAOManaged
	@WebPopulate(required = true, populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer price;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showQueuePosition;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean leavingRequiresManagerConfirmation;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendEnteredUserEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendEnteredUserSMS;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExternalMessageReceivedUserSMS;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExternalMessageReceivedUserEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExternalMessageReceivedManagerEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExternalMessageReceivedGlobalEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "queue_extmessage_global", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendExternalMessageReceivedGlobalEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> externalMessageReceivedGlobalEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitRequestUserEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitRequestUserSMS;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitUserSMS;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitRequestManagerEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitRequestGlobalEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "queue_exit_request_global", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendExitRequestGlobalEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> exitRequestGlobalEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitManagerEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendExitGlobalEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "queue_exit_global", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendExitGlobalEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> exitGlobalEmailAddresses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendPaymentCompleteUserSMS;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendPaymentCompleteUserEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendPaymentCompleteManagerEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendPaymentCompleteGlobalEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideAddedDateFromUser;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "queue_payment_global", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendPaymentCompleteGlobalEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "address")
	private List<String> paymentCompleteGlobalEmailAddresses;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<QueueSlot> queueSlots;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<QueueQuery> queueQueries;

	@DAOManaged
	@OneToMany
	@SimplifiedRelation(table = "queue_manager_users", remoteValueColumnName = "userID")
	@WebPopulate(paramName = "user")
	@XMLElement(childName = "userID")
	private List<Integer> managerUserIDs;

	@DAOManaged
	@OneToMany
	@SimplifiedRelation(table = "queue_manager_groups", remoteValueColumnName = "groupID")
	@WebPopulate(paramName = "group")
	@XMLElement(childName = "groupID")
	private List<Integer> managerGroupIDs;

	@DAOManaged
	@XMLElement
	private String eventPaymentInvoiceMessage;

	@XMLElement
	private Integer activeSlots;

	@XMLElement
	private Integer archivedSlots;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer profileID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useQueueForInternalUsers;

	@DAOManaged
	@OneToMany
	@SimplifiedRelation(table = "queue_internal_groups", remoteValueColumnName = "groupID")
	@WebPopulate(paramName = "internalGroup")
	@RequiredIfSet(paramNames = "useQueueForInternalUsers")
	@XMLElement(childName = "internalGroupID")
	private List<Integer> internalGroupIDs;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowDuplicateQueueSlots;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useWaitingPeriod;

	@Override
	public boolean allowsAdminAccess() {

		return false;
	}

	@Override
	public boolean allowsUserAccess() {

		return false;
	}

	@Override
	public boolean allowsAnonymousAccess() {

		return false;
	}

	@Override
	public Collection<Integer> getAllowedGroupIDs() {

		return managerGroupIDs;
	}

	@Override
	public Collection<Integer> getAllowedUserIDs() {

		return managerUserIDs;
	}

	public Integer getQueueID() {

		return queueID;
	}

	public void setQueueID(Integer queueID) {

		this.queueID = queueID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getDescription() {

		return description;
	}

	public void setDescription(String description) {

		this.description = description;
	}

	public boolean isEnabled() {

		return enabled;
	}

	public void setEnabled(boolean enabled) {

		this.enabled = enabled;
	}

	public Integer getPrice() {

		return price;
	}

	public void setPrice(Integer price) {

		this.price = price;
	}

	public boolean isShowQueuePosition() {

		return showQueuePosition;
	}

	public void setShowQueuePosition(boolean showQueuePosition) {

		this.showQueuePosition = showQueuePosition;
	}

	public List<QueueSlot> getQueueSlots() {

		return queueSlots;
	}

	public void setQueueSlots(List<QueueSlot> queueSlots) {

		this.queueSlots = queueSlots;
	}

	public List<Integer> getManagerGroupIDs() {

		return managerGroupIDs;
	}

	public void setManagerGroupIDs(List<Integer> managerGroupIDs) {

		this.managerGroupIDs = managerGroupIDs;
	}

	public List<Integer> getManagerUserIDs() {

		return managerUserIDs;
	}

	public void setManagerUserIDs(List<Integer> managerUserIDs) {

		this.managerUserIDs = managerUserIDs;
	}

	public Integer getActiveSlots() {

		return activeSlots;
	}

	public void setActiveSlots(Integer activeSlots) {

		this.activeSlots = activeSlots;
	}

	public Integer getArchivedSlots() {

		return archivedSlots;
	}

	public void setArchivedSlots(Integer archivedSlots) {

		this.archivedSlots = archivedSlots;
	}

	public boolean isLeavingRequiresManagerConfirmation() {

		return leavingRequiresManagerConfirmation;
	}

	public void setLeavingRequiresManagerConfirmation(boolean leavingRequiresManagerConfirmation) {

		this.leavingRequiresManagerConfirmation = leavingRequiresManagerConfirmation;
	}

	public String getEmailSenderName() {

		return emailSenderName;
	}

	public void setEmailSenderName(String emailSenderName) {

		this.emailSenderName = emailSenderName;
	}

	public String getEmailSenderAddress() {

		return emailSenderAddress;
	}

	public void setEmailSenderAddress(String emailSenderAddress) {

		this.emailSenderAddress = emailSenderAddress;
	}

	@Override
	public int hashCode() {

		final int prime = 31;
		int result = 1;
		result = prime * result + (queueID == null ? 0 : queueID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {

		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Queue other = (Queue) obj;
		if (queueID == null) {
			if (other.queueID != null) {
				return false;
			}
		} else if (!queueID.equals(other.queueID)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {

		return StringUtils.toLogFormat(name, 30) + " (queueID: " + queueID + ")";
	}

	public boolean isSendExternalMessageReceivedUserSMS() {

		return sendExternalMessageReceivedUserSMS;
	}

	public void setSendExternalMessageReceivedUserSMS(boolean sendExternalMessageReceivedUserSMS) {

		this.sendExternalMessageReceivedUserSMS = sendExternalMessageReceivedUserSMS;
	}

	public boolean isSendExternalMessageReceivedUserEmail() {

		return sendExternalMessageReceivedUserEmail;
	}

	public void setSendExternalMessageReceivedUserEmail(boolean sendExternalMessageReceivedUserEmail) {

		this.sendExternalMessageReceivedUserEmail = sendExternalMessageReceivedUserEmail;
	}

	public boolean isSendExternalMessageReceivedManagerEmail() {

		return sendExternalMessageReceivedManagerEmail;
	}

	public void setSendExternalMessageReceivedManagerEmail(boolean sendExternalMessageReceivedManagerEmail) {

		this.sendExternalMessageReceivedManagerEmail = sendExternalMessageReceivedManagerEmail;
	}

	public boolean isSendExternalMessageReceivedGlobalEmail() {

		return sendExternalMessageReceivedGlobalEmail;
	}

	public void setSendExternalMessageReceivedGlobalEmail(boolean sendExternalMessageReceivedGlobalEmail) {

		this.sendExternalMessageReceivedGlobalEmail = sendExternalMessageReceivedGlobalEmail;
	}

	public List<String> getExternalMessageReceivedGlobalEmailAddresses() {

		return externalMessageReceivedGlobalEmailAddresses;
	}

	public void setExternalMessageReceivedGlobalEmailAddresses(List<String> externalMessageReceivedGlobalEmailAddresses) {

		this.externalMessageReceivedGlobalEmailAddresses = externalMessageReceivedGlobalEmailAddresses;
	}

	public boolean isSendEnteredUserEmail() {

		return sendEnteredUserEmail;
	}

	public void setSendEnteredUserEmail(boolean sendEnteredUserEmail) {

		this.sendEnteredUserEmail = sendEnteredUserEmail;
	}

	public boolean isSendEnteredUserSMS() {

		return sendEnteredUserSMS;
	}

	public void setSendEnteredUserSMS(boolean sendEnteredUserSMS) {

		this.sendEnteredUserSMS = sendEnteredUserSMS;
	}

	public boolean isSendExitRequestUserEmail() {

		return sendExitRequestUserEmail;
	}

	public void setSendExitRequestUserEmail(boolean sendExitRequestUserEmail) {

		this.sendExitRequestUserEmail = sendExitRequestUserEmail;
	}

	public boolean isSendExitRequestUserSMS() {

		return sendExitRequestUserSMS;
	}

	public void setSendExitRequestUserSMS(boolean sendExitRequestUserSMS) {

		this.sendExitRequestUserSMS = sendExitRequestUserSMS;
	}

	public boolean isSendExitUserSMS() {

		return sendExitUserSMS;
	}

	public void setSendExitUserSMS(boolean sendExitConfirmationUserSMS) {

		this.sendExitUserSMS = sendExitConfirmationUserSMS;
	}

	public boolean isSendExitRequestManagerEmail() {

		return sendExitRequestManagerEmail;
	}

	public void setSendExitRequestManagerEmail(boolean sendExitRequestManagerEmail) {

		this.sendExitRequestManagerEmail = sendExitRequestManagerEmail;
	}

	public boolean isSendExitRequestGlobalEmail() {

		return sendExitRequestGlobalEmail;
	}

	public void setSendExitRequestGlobalEmail(boolean sendExitRequestGlobalEmail) {

		this.sendExitRequestGlobalEmail = sendExitRequestGlobalEmail;
	}

	public List<String> getExitRequestGlobalEmailAddresses() {

		return exitRequestGlobalEmailAddresses;
	}

	public void setExitRequestGlobalEmailAddresses(List<String> exitRequestGlobalEmailAddresses) {

		this.exitRequestGlobalEmailAddresses = exitRequestGlobalEmailAddresses;
	}

	public boolean isSendExitManagerEmail() {

		return sendExitManagerEmail;
	}

	public void setSendExitManagerEmail(boolean sendExitManagerEmail) {

		this.sendExitManagerEmail = sendExitManagerEmail;
	}

	public boolean isSendExitGlobalEmail() {

		return sendExitGlobalEmail;
	}

	public void setSendExitGlobalEmail(boolean sendExitGlobalEmail) {

		this.sendExitGlobalEmail = sendExitGlobalEmail;
	}

	public List<String> getExitGlobalEmailAddresses() {

		return exitGlobalEmailAddresses;
	}

	public void setExitGlobalEmailAddresses(List<String> exitGlobalEmailAddresses) {

		this.exitGlobalEmailAddresses = exitGlobalEmailAddresses;
	}

	public boolean isSendPaymentCompleteUserSMS() {

		return sendPaymentCompleteUserSMS;
	}

	public void setSendPaymentCompleteUserSMS(boolean sendPaymentCompleteUserSMS) {

		this.sendPaymentCompleteUserSMS = sendPaymentCompleteUserSMS;
	}

	public boolean isSendPaymentCompleteUserEmail() {

		return sendPaymentCompleteUserEmail;
	}

	public void setSendPaymentCompleteUserEmail(boolean sendPaymentCompleteUserEmail) {

		this.sendPaymentCompleteUserEmail = sendPaymentCompleteUserEmail;
	}

	public boolean isSendPaymentCompleteManagerEmail() {

		return sendPaymentCompleteManagerEmail;
	}

	public void setSendPaymentCompleteManagerEmail(boolean sendPaymentCompleteManagerEmail) {

		this.sendPaymentCompleteManagerEmail = sendPaymentCompleteManagerEmail;
	}

	public boolean isSendPaymentCompleteGlobalEmail() {

		return sendPaymentCompleteGlobalEmail;
	}

	public void setSendPaymentCompleteGlobalEmail(boolean sendPaymentCompleteGlobalEmail) {

		this.sendPaymentCompleteGlobalEmail = sendPaymentCompleteGlobalEmail;
	}

	public boolean isHideAddedDateFromUser() {

		return hideAddedDateFromUser;
	}

	public void setHideAddedDateFromUser(boolean hideAddedDateFromUser) {

		this.hideAddedDateFromUser = hideAddedDateFromUser;
	}

	public List<String> getPaymentCompleteGlobalEmailAddresses() {

		return paymentCompleteGlobalEmailAddresses;
	}

	public void setPaymentCompleteGlobalEmailAddresses(List<String> paymentCompleteGlobalEmailAddresses) {

		this.paymentCompleteGlobalEmailAddresses = paymentCompleteGlobalEmailAddresses;
	}

	public String getPaymentConfigurationID() {

		return "queue-" + queueID;
	}

	public String getEventPaymentInvoiceMessage() {

		return eventPaymentInvoiceMessage;
	}

	public void setEventPaymentInvoiceMessage(String eventPaymentInvoiceMessage) {

		this.eventPaymentInvoiceMessage = eventPaymentInvoiceMessage;
	}

	public Integer getProfileID() {

		return profileID;
	}

	public void setProfileID(Integer profileID) {

		this.profileID = profileID;
	}

	public boolean isUseQueueForInternalUsers() {

		return useQueueForInternalUsers;
	}

	public void setUseQueueForInternalUsers(boolean useQueueForInternalUsers) {

		this.useQueueForInternalUsers = useQueueForInternalUsers;
	}

	public List<Integer> getInternalGroupIDs() {

		return internalGroupIDs;
	}

	public void setInternalGroupIDs(List<Integer> internalGroupIDs) {

		this.internalGroupIDs = internalGroupIDs;
	}

	public boolean isAllowDuplicateQueueSlots() {

		return allowDuplicateQueueSlots;
	}

	public void setAllowDuplicateQueueSlots(boolean allowDuplicateQueueSlots) {

		this.allowDuplicateQueueSlots = allowDuplicateQueueSlots;
	}
	
	public boolean isUseWaitingPeriod() {

		return useWaitingPeriod;
	}

	public void setUseWaitingPeriod(boolean useWaitingPeriod) {

		this.useWaitingPeriod = useWaitingPeriod;
	}
	
	

}
