package com.nordicpeak.flowengine.queries.ratingscalequery;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativeQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

@Table(name = "rating_scale_query_instances")
@XMLElement
public class RatingScaleQueryInstance extends BaseQueryInstance implements FixedAlternativesQueryInstance, StringValueQueryInstance {

	private static final long serialVersionUID = 8648512088409127744L;

	public static final Field ALTERNATIVE_RELATION = ReflectionUtils.getField(RatingScaleQueryInstance.class, "alternative");
	public static final Field QUERY_RELATION = ReflectionUtils.getField(RatingScaleQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private RatingScaleQuery query;

	@DAOManaged(columnName = "alternativeID")
	@ManyToOne
	@XMLElement
	private RatingScaleAlternative alternative;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public RatingScaleQuery getQuery() {

		return query;
	}

	public void setQuery(RatingScaleQuery query) {

		this.query = query;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.alternative = null;

		if (query.isSetAsAttribute()) {

			resetAttribute(attributeHandler);
		}

		super.reset(attributeHandler);
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {

		if (alternative != null) {

			if (!StringUtils.isEmpty(alternative.getAttributeValue())) {

				attributeHandler.setAttribute(query.getAttributeName(), alternative.getAttributeValue());

			} else {

				attributeHandler.setAttribute(query.getAttributeName(), alternative.getName());
			}
		}
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler) {

		attributeHandler.removeAttribute(query.getAttributeName());
	}

	public void copyQueryValues() {}

	@Override
	public String toString() {

		return "RatingScaleQueryInstance (queryInstanceID=" + queryInstanceID + ")";
	}

	public RatingScaleAlternative getAlternative() {

		return alternative;
	}

	public void setAlternative(RatingScaleAlternative alternative) {

		this.alternative = alternative;
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		if (alternative == null) {

			return null;
		}

		return Collections.singletonList(alternative);
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		FixedAlternativeQueryUtils.appendExportXMLAlternatives(doc, element, this);

		return element;
	}

	@Override
	public String getStringValue() {

		if (alternative != null) {

			return alternative.getName();

		}

		return null;
	}

	@Override
	public String getFreeTextAlternativeValue() {

		// Not used
		return null;
	}
}
