package com.nordicpeak.flowengine.recommendedflows;

import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.login.LoginEvent;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileHandler;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.datatypes.SimpleEntry;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.DBUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.CharsetValidator;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPResponse;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SessionUtils;
import se.unlogic.webutils.http.SimpleRequest;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.UserFlowInstanceModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.ListFlowInstancesViewFragmentExtensionProvider;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;

public class RecommendedFlowsModule extends AnnotatedForegroundModule implements ViewFragmentModule<ForegroundModuleDescriptor>, ListFlowInstancesViewFragmentExtensionProvider {

	private static final String SESSION_ATTRIBUTE = "recommendedFlows";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Title", description = "The title displayed above the recommended flows", required = true)
	private String title = "Not set";
	
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Description", description = "The descripton dsiplayed above the recommended flows", required = true)
	private String description = "Not set";
	
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Help text", description = "The help text for the recommended flows", required = true)
	private String helpText = "Not set";
	
	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "API URL", description = "URL to the external API. $citizenIdentifier will be replaced with the users citizen identifier.")
	private String apiURL;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "API username", description = "")
	private String apiUsername;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "API password", description = "")
	private String apiPassword;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Encoding", description = "Encoding to use when parsing the API response", required = true, formatValidator = CharsetValidator.class)
	private String encoding = "UTF-8";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection", required = true)
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response", required = true)
	private Integer readTimeout = 10;

	@InstanceManagerDependency(required = true)
	private FlowBrowserModule flowBrowserModule;

	private UserFlowInstanceModule userFlowInstanceModule;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		viewFragmentTransformer = new ModuleViewFragmentTransformer<ForegroundModuleDescriptor>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());
	}

	@InstanceManagerDependency(required = true)
	public void setUserFlowInstanceModule(UserFlowInstanceModule userFlowInstanceModule) {

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.addListFlowInstancesViewFragmentExtensionProvider(this);

		} else {

			this.userFlowInstanceModule.removeListFlowInstancesViewFragmentExtensionProvider(this);
		}

		this.userFlowInstanceModule = userFlowInstanceModule;
	}

	@Override
	public void unload() throws Exception {

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.removeListFlowInstancesViewFragmentExtensionProvider(this);
		}

		super.unload();
	}

	@se.unlogic.hierarchy.core.annotations.EventListener(channel = User.class)
	public void processLoginEvent(LoginEvent event, EventSource source) throws SQLException {

		final User user = event.getUser();
		final HttpSession session = event.getSession();

		String citizenID = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

		if (apiURL != null && citizenID != null) {

			log.info("Getting recommended flows for user " + user + "...");

			SimpleRequest simpleRequest = new SimpleRequest(apiURL.replace("$citizenIdentifier", citizenID));

			simpleRequest.setFollowRedirects(false);
			simpleRequest.setConnectionTimeout(connectionTimeout * 1000);
			simpleRequest.setReadTimeout(readTimeout * 1000);

			if (apiUsername != null && apiPassword != null) {

				simpleRequest.setUsername(apiUsername);
				simpleRequest.setPassword(apiPassword);
			}

			List<Entry<String, String>> headerEntries = new ArrayList<Entry<String, String>>(2);

			headerEntries.add(new SimpleEntry<String, String>("NoLoginRedirect", "NoLoginRedirect"));
			simpleRequest.setHeaders(headerEntries);

			try {
				HTTPResponse response = HTTPUtils.sendHTTPGetRequest(simpleRequest, Charset.forName(encoding));

				try {
					XMLParser parser = new XMLParser(XMLUtils.parseXML(response.getValue(), false, false));

					List<XMLParser> items = parser.getNodes("/Recommendations/Recommendation");

					if (items.size() > 0) {

						List<RecommendedFlow> recommendedFlows = new ArrayList<RecommendedFlow>(items.size());

						Connection connection = dataSource.getConnection();

						try {
							for (XMLParser item : items) {

								Integer flowFamilyID = item.getInteger("FlowFamilyID");
								Integer flowInstanceCount = item.getInteger("Count");
								String description = item.getString("Description");

								Date fromDate = null;
								Date toDate = null;

								String fromDateString = item.getString("Period/Start");
								String toDateString = item.getString("Period/End");

								if (!StringUtils.isEmpty(fromDateString)) {
									fromDate = DateUtils.DATE_FORMATTER.parse(fromDateString);
								}

								if (!StringUtils.isEmpty(toDateString)) {
									toDate = DateUtils.DATE_FORMATTER.parse(toDateString);
								}

								if (flowFamilyID != null && flowInstanceCount != null) {

									if (flowBrowserModule.getLatestPublishedFlowVersion(flowFamilyID) != null) {

										RecommendedFlow recommendedFlow = new RecommendedFlow(flowFamilyID, flowInstanceCount, fromDate, toDate, description);

										Integer submittedFlowInstances = getSubmittedFlowInstanceCount(recommendedFlow, user, connection);

										if (submittedFlowInstances < flowInstanceCount) {

											recommendedFlows.add(recommendedFlow);
										}
									}
								}
							}

						} finally {
							DBUtils.closeConnection(connection);
						}

						if (recommendedFlows.size() > 0) {

							log.info("Got " + recommendedFlows.size() + " recommended flows for user " + user);
							SessionUtils.setAttribute(SESSION_ATTRIBUTE, recommendedFlows, session);
						}
					}

				} catch (Exception e) {

					log.warn("Error parsing recommended flow API response", e);
				}

			} catch (Throwable e) {

				log.warn("Error connecting to recommend flows API", e);
			}
		}
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 100)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {

		if (event.getFlowInstanceManager().getFlowInstance().getOwners() != null) {

			Integer flowFamilyID = event.getFlowInstanceManager().getFlowInstance().getFlow().getFlowFamily().getFlowFamilyID();

			for (User owner : event.getFlowInstanceManager().getFlowInstance().getOwners()) {

				@SuppressWarnings("unchecked")
				List<RecommendedFlow> recommendedFlows = (List<RecommendedFlow>) SessionUtils.getAttribute(SESSION_ATTRIBUTE, owner.getSession());

				if (recommendedFlows != null) {

					Iterator<RecommendedFlow> it = recommendedFlows.iterator();

					while (it.hasNext()) {

						RecommendedFlow recommendedFlow = it.next();

						if (recommendedFlow.getFlowFamilyID().equals(flowFamilyID)) {

							Integer submittedFlowInstances = getSubmittedFlowInstanceCount(recommendedFlow, owner, null);

							if (submittedFlowInstances >= recommendedFlow.getFlowInstanceCount()) {

								log.info("Removing recommended flow " + recommendedFlow + " from session of " + owner);
								it.remove();
							}
						}
					}

					if (recommendedFlows.isEmpty()) {
						SessionUtils.removeAttribute(SESSION_ATTRIBUTE, owner.getSession());
					}
				}
			}
		}
	}

	private int getSubmittedFlowInstanceCount(RecommendedFlow recommendedFlow, User user, Connection connection) throws SQLException {

		//@formatter:off
		StringBuilder sql = new StringBuilder(
				"SELECT COUNT(instance.flowInstanceID) FROM flowengine_flow_instances instance " +
				"INNER JOIN flowengine_flows flows ON instance.flowID = flows.flowID " +
				"INNER JOIN flowengine_flow_instance_owners owners ON instance.flowInstanceID = owners.flowInstanceID " +
				"WHERE instance.firstSubmitted IS NOT NULL AND flows.flowFamilyID = ? AND owners.userID = ?"
		);
		//@formatter:on

		if (recommendedFlow.getFromDate() != null) {
			sql.append(" AND instance.firstSubmitted >= ?");
		}

		if (recommendedFlow.getToDate() != null) {
			sql.append(" AND instance.firstSubmitted <= ?");
		}

		ObjectQuery<Integer> query;

		if (connection != null) {
			query = new ObjectQuery<Integer>(connection, false, sql.toString(), IntegerPopulator.getPopulator());

		} else {
			query = new ObjectQuery<Integer>(dataSource, sql.toString(), IntegerPopulator.getPopulator());
		}

		int index = 1;

		query.setInt(index++, recommendedFlow.getFlowFamilyID());
		query.setInt(index++, user.getUserID());

		if (recommendedFlow.getFromDate() != null) {
			query.setDate(index++, recommendedFlow.getFromDate());
		}

		if (recommendedFlow.getToDate() != null) {
			query.setDate(index++, recommendedFlow.getToDate());
		}

		return query.executeQuery();
	}

	@Override
	public ViewFragment getListFlowInstancesViewFragment(List<FlowInstance> flowInstances, SiteProfileHandler profileHandler, HttpServletRequest req, URIParser uriParser, User user) throws Exception {

		@SuppressWarnings("unchecked")
		List<RecommendedFlow> recommendedFlows = (List<RecommendedFlow>) SessionUtils.getAttribute(SESSION_ATTRIBUTE, req);

		if (recommendedFlows != null) {

			Document doc = createDocument(req, uriParser);

			Element listFlowInstancesExtensionElement = doc.createElement("ListFlowInstancesExtension");
			doc.getDocumentElement().appendChild(listFlowInstancesExtensionElement);

			XMLUtils.appendNewElement(doc, listFlowInstancesExtensionElement, "title", title);
			XMLUtils.appendNewElement(doc, listFlowInstancesExtensionElement, "description", description);
			XMLUtils.appendNewElement(doc, listFlowInstancesExtensionElement, "helpText", helpText);
			
			Element flowsElement = XMLUtils.appendNewElement(doc, listFlowInstancesExtensionElement, "Flows");

			for (RecommendedFlow recommendedFlow : recommendedFlows) {

				Flow flow = flowBrowserModule.getLatestPublishedFlowVersion(recommendedFlow.getFlowFamilyID());

				if (flow != null) {

					Element recommendedFlowElement = recommendedFlow.toXML(doc);
					flowsElement.appendChild(recommendedFlowElement);

					XMLUtils.appendNewElement(doc, recommendedFlowElement, "flowName", flow.getName());
					XMLUtils.appendNewElement(doc, recommendedFlowElement, "URL", uriParser.getContextPath() + flowBrowserModule.getFullAlias() + "/overview/" + recommendedFlow.getFlowFamilyID());
				}
			}

			return viewFragmentTransformer.createViewFragment(doc);
		}

		return null;
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser) {

		Document doc = XMLUtils.createDomDocument();
		Element documentElement = doc.createElement("Document");
		doc.appendChild(documentElement);
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		return doc;
	}

	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return null;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return null;
	}
}
