package com.nordicpeak.flowengine.queries.remoteinfoquery;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;

public class RemoteInfoQueryCRUD extends BaseQueryCRUD<RemoteInfoQuery, RemoteInfoQueryProviderModule> {

	public RemoteInfoQueryCRUD(AnnotatedDAOWrapper<RemoteInfoQuery, Integer> queryDAO, BeanRequestPopulator<RemoteInfoQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, RemoteInfoQueryProviderModule callback) {
		
		super(RemoteInfoQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
	}

	@Override
	protected RemoteInfoQuery populateFromUpdateRequest(RemoteInfoQuery oldQuery, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		RemoteInfoQuery query = super.populateFromUpdateRequest(oldQuery, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);
		
		Integer endpointID = ValidationUtils.validateParameter("endpointID", req, true, IntegerPopulator.getPopulator(), validationErrors);
		
		if(endpointID != null) {
			
			RemoteInfoQueryEndpoint endpoint = callback.getEndpoint(endpointID);
			
			if(endpoint == null) {

				validationErrors.add(new ValidationError("endpointID", ValidationErrorType.RequiredField));
				
			}else {
				
				query.setEndpoint(endpoint);
			}
			
			
		} else {
		
			validationErrors.add(new ValidationError("endpointID", ValidationErrorType.RequiredField));
		}
		
		
		if(!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}

	@Override
	protected void appendUpdateFormData(RemoteInfoQuery bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);
		
		XMLUtils.append(doc, updateTypeElement, callback.getEndpoints());
	}

}
