package com.nordicpeak.flowengine.queries.remotesinglesignquery.api;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.menu.MenuItemDescriptor;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.userproviders.SimpleUser;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.datatypes.SimpleEntry;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.string.URLEncodingTagReplacer;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPResponse;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SimpleRequest;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.interfaces.APISource;
import com.nordicpeak.flowengine.interfaces.APISourceHandler;
import com.nordicpeak.flowengine.interfaces.APISourceProvider;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.queries.remotesinglesignquery.RemoteSigningParty;
import com.nordicpeak.flowengine.queries.remotesinglesignquery.RemoteSinglesignQuery;
import com.nordicpeak.flowengine.queries.remotesinglesignquery.RemoteSinglesignQueryProviderModule;
import com.nordicpeak.flowengine.utils.UserAttributeTagUtils;

public class RemoteSinglesignQueryEndpointAdminModule extends AnnotatedForegroundModule implements CRUDCallback<User>, APISourceProvider {

	public static final AnnotatedBeanTagSourceFactory<User> USER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$user.");

	@XSLVariable(prefix = "java.")
	private String apiSourceShortTypeName;
	
	@XSLVariable(prefix = "java.")
	private String apiSourceFullTypeName;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;

	@ModuleSetting
	@SplitOnLineBreak
	@TextAreaSettingDescriptor(name = "Allowed encodings", description = "Encodings allowed to use")
	private List<String> allowedEncodings = new ArrayList<String>(Arrays.asList("ISO-8859-1", "UTF-8"));

	@InstanceManagerDependency
	private QueryHandler queryHandler;

	private AnnotatedDAO<RemoteSinglesignQueryEndpoint> endpointDAO;
	private AnnotatedDAO<RemoteSinglesignQuery> queryDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;

	private QueryParameterFactory<RemoteSinglesignQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<RemoteSinglesignQueryEndpoint, Integer> endpointIDParamFactory;

	private RemoteSinglesignQueryEndpointCRUD endpointCRUD;
	
	private APISourceHandler apiSourceHandler;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(RemoteSinglesignQueryEndpointAdminModule.class, this)) {

			throw new RuntimeException("Unable to register module " + moduleDescriptor + " in global instance handler using key " + RemoteSinglesignQueryEndpointAdminModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, RemoteSinglesignQueryProviderModule.class.getName(), new XMLDBScriptProvider(RemoteSinglesignQueryProviderModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(RemoteSinglesignQuery.class);
		endpointDAO = daoFactory.getDAO(RemoteSinglesignQueryEndpoint.class);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		endpointIDParamFactory = endpointDAO.getParamFactory("endpointID", Integer.class);

		AdvancedAnnotatedDAOWrapper<RemoteSinglesignQueryEndpoint, Integer> endpointDAOWrapper = endpointDAO.getAdvancedWrapper(Integer.class);
		
		endpointDAOWrapper.getGetAllQuery().addRelations(RemoteSinglesignQueryEndpoint.QUERIES_RELATION);
		endpointDAOWrapper.getGetAllQuery().disableAutoRelations(true);
		
		endpointDAOWrapper.getGetQuery().addRelations(RemoteSinglesignQueryEndpoint.QUERIES_RELATION);
		endpointDAOWrapper.getGetQuery().disableAutoRelations(true);
		endpointDAOWrapper.getGetQuery().setRowLimiter(MySQLRowLimiter.SINGLE_ROW);

		endpointCRUD = new RemoteSinglesignQueryEndpointCRUD(endpointDAOWrapper, this);

		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());

		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(RemoteSinglesignQueryEndpointAdminModule.class, this);

		if (this.apiSourceHandler != null) {
			
			this.apiSourceHandler.removeAPISourceProvider(this);
		}
		
		super.unload();
	}
	
	@InstanceManagerDependency
	public void setAPISourceHandler(APISourceHandler apiSourceHandler) throws SQLException {
		
		if (apiSourceHandler != null) {
			
			apiSourceHandler.addAPISourceProvider(this);
			
		} else if (this.apiSourceHandler != null) {
			
			this.apiSourceHandler.removeAPISourceProvider(this);
		}
		
		this.apiSourceHandler = apiSourceHandler;
		
		//Check if this module is fully cached, then trigger an update of the menu items
		if(sectionInterface.getForegroundModuleCache().isCached(moduleDescriptor)) {
			
			this.sectionInterface.getMenuCache().moduleUpdated(moduleDescriptor, this);
		}
	}	

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.list(req, res, user, uriParser, null);
	}

	@WebPublic
	public ForegroundModuleResponse show(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.show(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse add(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.add(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse update(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.update(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse delete(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.delete(req, res, user, uriParser);
	}

	public List<RemoteSinglesignQuery> getQueries(RemoteSinglesignQueryEndpoint endpoint) {

		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<Integer>(queryDAO.getDataSource(), "SELECT queryID FROM " + queryDAO.getTableName() + " WHERE endpointID = ?", IntegerPopulator.getPopulator());
			idQuery.setInt(1, endpoint.getEndpointID());

			List<Integer> queryIDs = idQuery.executeQuery();

			if (!CollectionUtils.isEmpty(queryIDs)) {

				List<RemoteSinglesignQuery> queries = new ArrayList<RemoteSinglesignQuery>(queryIDs.size());

				for (Integer queryID : queryIDs) {

					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(queryID);

					if (queryDescriptor != null) {

						queries.add((RemoteSinglesignQuery) getQuery(queryDescriptor));
					}
				}

				return queries;
			}

		} catch (SQLException e) {

			log.error("Unable to get queries using endpoint " + endpoint, e);
		}

		return null;
	}

	public Query getQuery(MutableQueryDescriptor descriptor) throws SQLException {

		RemoteSinglesignQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	public RemoteSinglesignQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<RemoteSinglesignQuery> getQuery = new HighLevelQuery<RemoteSinglesignQuery>();

		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));

		RemoteSinglesignQuery query = queryDAO.get(getQuery);

		return query;
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();

		Element documentElement = doc.createElement("Document");
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(documentElement);

		return doc;
	}

	public RemoteSigningParty getRemoteSigningPartyFromEndpoint(RemoteSinglesignQueryEndpoint endpoint, User poster, AttributeHandler attributeHandler) throws ValidationException {

		if (endpoint == null) {
			return null;
		}

		String address = endpoint.getAddress();

		User usedPoster = poster;

		if (usedPoster == null) {

			usedPoster = new SimpleUser();
		}

		TagReplacer tagReplacer = new URLEncodingTagReplacer();
		tagReplacer.addTagSource(USER_TAG_SOURCE_FACTORY.getTagSource(usedPoster));

		address = tagReplacer.replace(address);

		address = UserAttributeTagUtils.replaceTags(address, usedPoster, true);

		address = AttributeTagUtils.replaceTags(address, attributeHandler, false, true);

		log.info("User " + poster + " getting field values for api endpoint " + endpoint + " from: " + address);

		SimpleRequest simpleRequest = new SimpleRequest(address);

		simpleRequest.setFollowRedirects(false);
		simpleRequest.setConnectionTimeout(connectionTimeout * 1000);
		simpleRequest.setReadTimeout(readTimeout * 1000);

		if (endpoint.getUsername() != null && endpoint.getPassword() != null) {

			simpleRequest.setUsername(endpoint.getUsername());
			simpleRequest.setPassword(endpoint.getPassword());
		}

		List<Entry<String, String>> headerEntries = new ArrayList<Entry<String, String>>(2);

		headerEntries.add(new SimpleEntry<String, String>("NoLoginRedirect", "NoLoginRedirect"));
		simpleRequest.setHeaders(headerEntries);

		try {
			HTTPResponse response = HTTPUtils.sendHTTPGetRequest(simpleRequest, endpoint.getCharset());

			try {
				XMLParser parser = new XMLParser(XMLUtils.parseXML(response.getValue(), endpoint.getCharset().toString(), false, false));

				XMLParser signingPartiesNode = parser.getNode("/SigningParties");

				if (signingPartiesNode == null) {
					throw new ValidationException(new ValidationError("EndpointError", "/SigningParties missing"));
				}

				List<XMLParser> partyNodes = parser.getNodes("/SigningParties/SigningParty", true);

				if (partyNodes.isEmpty()) {
					return null;
				}

				XMLParser partyNode = partyNodes.get(0);

				RemoteSigningParty signingParty = new RemoteSigningParty();

				signingParty.setCitizenID(parseString(partyNode, "CitizenIdentifier"));
				signingParty.setFirstname(parseString(partyNode, "Firstname"));
				signingParty.setLastname(parseString(partyNode, "Lastname"));;
				signingParty.setEmail(parseString(partyNode, "Email"));
				signingParty.setMobilePhone(parseString(partyNode, "MobilePhone"));
				
				StringBuilder parseErrors = new StringBuilder();

				if (signingParty.getCitizenID() == null) {
					parseErrors.append("SigningParty/CitizenIdentifier missing or empty. ");
				}

				if (signingParty.getFirstname() == null) {
					parseErrors.append("SigningParty/Firstname missing or empty. ");
				}

				if (signingParty.getLastname() == null) {
					parseErrors.append("SigningParty/Lastname missing or empty. ");
				}

				if (parseErrors.length() > 0) {
					throw new ValidationException(new ValidationError("EndpointError", parseErrors.toString()));
				}

				return signingParty;

			} catch (ValidationException e) {

				log.warn("Validation errors parsing response XML for " + endpoint, e);
				throw e;
				
			} catch (Exception e) {

				log.warn("Error parsing response XML for " + endpoint, e);
				throw new ValidationException(new ValidationError("EndpointError", "Error parsing response XML, malformed"));
			}

		} catch (IOException e) {

			log.warn("Error connecting to " + endpoint, e);
			throw new ValidationException(new ValidationError("EndpointError", "Error connecting to " + address));
		}
	}

	public String parseString(XMLParser xmlParser, String nodeName) {

		String value = xmlParser.getString(nodeName);

		if (StringUtils.isEmpty(value)) {
			return null;
		}

		return value;
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public List<String> getAllowedEncodings() {

		return allowedEncodings;
	}

	public List<RemoteSinglesignQueryEndpoint> getEndpoints() throws SQLException {

		HighLevelQuery<RemoteSinglesignQueryEndpoint> query = new HighLevelQuery<RemoteSinglesignQueryEndpoint>();

		return endpointDAO.getAll(query);
	}

	public RemoteSinglesignQueryEndpoint getEndpoint(Integer endpointID) throws SQLException {

		HighLevelQuery<RemoteSinglesignQueryEndpoint> query = new HighLevelQuery<RemoteSinglesignQueryEndpoint>();
		query.addParameter(endpointIDParamFactory.getParameter(endpointID));

		return endpointDAO.get(query);
	}

	@Override
	public List<? extends APISource> getAPISources() throws Exception {

		return endpointDAO.getAll();
	}

	@Override
	public String getShortTypeName() {

		return apiSourceShortTypeName;
	}
	
	@Override
	public String getFullTypeName() {

		return apiSourceFullTypeName;
	}

	@Override
	public String getAddURL() {

		return this.getFullAlias() + "/add";
	}
	
	@Override
	public String getBaseShowURL() {

		return this.getFullAlias() + "/show/";
	}
	
	@Override
	public String getBaseUpdateURL() {

		return this.getFullAlias() + "/update/";
	}

	@Override
	public String getBaseDeleteURL() {

		return this.getFullAlias() + "/delete/";
	}

	@Override
	public boolean isInUse(APISource apiSource) throws SQLException {
		
		RemoteSinglesignQueryEndpoint endpoint = (RemoteSinglesignQueryEndpoint)apiSource;
		
		return endpoint.getQueries() != null;
	}

	public APISourceHandler getAPISourceHandler() {

		return apiSourceHandler;
	}
	
	@Override
	public List<? extends MenuItemDescriptor> getVisibleMenuItems() {

		if(this.apiSourceHandler != null) {
			
			return null;
		
		}else {
		
			return super.getVisibleMenuItems();
		}
	}
}