package com.nordicpeak.flowengine.queries.ssbt;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.enums.EnumUtils;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;
import com.nordicpeak.flowengine.queries.ssbt.enums.CompanyType;
import com.nordicpeak.flowengine.queries.ssbt.enums.TargetEnvironment;

@Table(name = "ssbt2_company_detail_queries")
@XMLElement
public class CompanyDetailsQuery extends BaseQuery implements FixedAlternativesQuery {

	public static final String ROLE_ACCESS_DENIED = "ROLE-ACCESS-DENIED";
	
	public static final UDAlternative UD0004_ALTERNATIVE = new UDAlternative(10004, "E-postadress");
	public static final UDAlternative UD0006_ALTERNATIVE = new UDAlternative(10006, "Telefon");
	
	public static final UDAlternative UD0029_ALTERNATIVE_YES = new UDAlternative(10029, "F-skatt eller FA-skatt - Ja");
	public static final UDAlternative UD0029_ALTERNATIVE_NO = new UDAlternative(10129, "F-skatt eller FA-skatt - Nej");
	public static final UDAlternative UD0029_ALTERNATIVE_ACCESS_DENIED = new UDAlternative(10229, "F-skatt eller FA-skatt - Kunde inte hmtas fr aktuell roll");
	
	public static final UDAlternative UD0031_ALTERNATIVE_YES = new UDAlternative(10031, "Momsregistrering - Ja");
	public static final UDAlternative UD0031_ALTERNATIVE_NO = new UDAlternative(10131, "Momsregistrering - Nej");
	public static final UDAlternative UD0031_ALTERNATIVE_ACCESS_DENIED = new UDAlternative(10231, "Momsregistrering - Kunde inte hmtas fr aktuell roll");
	
	public static final UDAlternative UD0033_ALTERNATIVE_YES = new UDAlternative(10033, "Arbetsgivarregistrering - Ja");
	public static final UDAlternative UD0033_ALTERNATIVE_NO = new UDAlternative(10133, "Arbetsgivarregistrering - Nej");
	public static final UDAlternative UD0033_ALTERNATIVE_ACCESS_DENIED = new UDAlternative(10233, "Arbetsgivarregistrering - Kunde inte hmtas fr aktuell roll");

	private static final EnumPopulator<TargetEnvironment> TARGET_ENVIRONMENT_POPULATOR = new EnumPopulator<TargetEnvironment>(TargetEnvironment.class);

	private static final long serialVersionUID = 1427199144529841941L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private TargetEnvironment targetEnvironment;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String serviceName;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "ssbt2_company_types", remoteValueColumnName = "companyType")
	@WebPopulate(required = true)
	@XMLElement
	private List<CompanyType> companyTypes;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0001;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0003;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0004;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0006;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0023;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0025;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0029;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0031;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0033;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0045;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0046;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0047;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useUD0048;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showCompanyTypeDescription;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showCompaniesAsRadiosAndKeepShowing;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<CompanyDetailsQueryInstance> instances;

	@Override
	public String getXSDTypeName() {

		return "CompanyDetailsQuery" + queryID;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		this.serviceName = XMLValidationUtils.validateParameter("serviceName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		this.targetEnvironment = XMLValidationUtils.validateParameter("targetEnvironment", xmlParser, false, TARGET_ENVIRONMENT_POPULATOR, errors);

		this.companyTypes = EnumUtils.toEnum(CompanyType.class, xmlParser.getStrings("companyTypes/value"));

		useUD0001 = xmlParser.getPrimitiveBoolean("useUD0001");
		useUD0003 = xmlParser.getPrimitiveBoolean("useUD0003");
		useUD0004 = xmlParser.getPrimitiveBoolean("useUD0004");
		useUD0006 = xmlParser.getPrimitiveBoolean("useUD0006");
		useUD0023 = xmlParser.getPrimitiveBoolean("useUD0023");
		useUD0025 = xmlParser.getPrimitiveBoolean("useUD0025");
		
		useUD0029 = xmlParser.getPrimitiveBoolean("useUD0029");
		useUD0031 = xmlParser.getPrimitiveBoolean("useUD0031");
		useUD0033 = xmlParser.getPrimitiveBoolean("useUD0033");
		
		useUD0045 = xmlParser.getPrimitiveBoolean("useUD0045");
		useUD0046 = xmlParser.getPrimitiveBoolean("useUD0046");
		useUD0047 = xmlParser.getPrimitiveBoolean("useUD0047");
		useUD0048 = xmlParser.getPrimitiveBoolean("useUD0048");

		showCompanyTypeDescription = xmlParser.getPrimitiveBoolean("showCompanyTypeDescription");
		showCompaniesAsRadiosAndKeepShowing = xmlParser.getPrimitiveBoolean("showCompaniesAsRadiosAndKeepShowing");

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	//	public boolean usesUD0001() {
	//
	//		return useUD0001;
	//	}
	//
	//	public void setUseUD0001(boolean useUD0001) {
	//
	//		this.useUD0001 = useUD0001;
	//	}

	public boolean usesUD0003() {

		return useUD0003;
	}

	public void setUseUD0003(boolean useUD0003) {

		this.useUD0003 = useUD0003;
	}

	public boolean usesUD0004() {

		return useUD0004;
	}

	public void setUseUD0004(boolean useUD0004) {

		this.useUD0004 = useUD0004;
	}

	public boolean usesUD0006() {

		return useUD0006;
	}

	public void setUseUD0006(boolean useUD0006) {

		this.useUD0006 = useUD0006;
	}

	public boolean usesUD0023() {

		return useUD0023;
	}

	public void setUseUD0023(boolean useUD0023) {

		this.useUD0023 = useUD0023;
	}

	//	public boolean usesUD0025() {
	//
	//		return useUD0025;
	//	}
	//
	//	public void setUseUD0025(boolean useUD0025) {
	//
	//		this.useUD0025 = useUD0025;
	//	}

	public boolean usesUD0029() {

		return useUD0029;
	}

	public void setUseUD0029(boolean useUD0029) {

		this.useUD0029 = useUD0029;
	}

	public boolean usesUD0031() {

		return useUD0031;
	}

	public void setUseUD0031(boolean useUD0031) {

		this.useUD0031 = useUD0031;
	}

	public boolean usesUD0033() {

		return useUD0033;
	}

	public void setUseUD0033(boolean useUD0033) {

		this.useUD0033 = useUD0033;
	}

	public boolean usesUD0045() {

		return useUD0045;
	}

	public void setUseUD0045(boolean useUD0045) {

		this.useUD0045 = useUD0045;
	}

	public boolean usesUD0046() {

		return useUD0046;
	}

	public void setUseUD0046(boolean useUD0046) {

		this.useUD0046 = useUD0046;
	}

	public boolean usesUD0047() {

		return useUD0047;
	}

	public void setUseUD0047(boolean useUD0047) {

		this.useUD0047 = useUD0047;
	}

	public boolean usesUD0048() {

		return useUD0048;
	}

	public void setUseUD0048(boolean useUD0048) {

		this.useUD0048 = useUD0048;
	}

	public List<CompanyType> getCompanyTypes() {

		return companyTypes;
	}

	public void setCompanyTypes(List<CompanyType> companyTypes) {

		this.companyTypes = companyTypes;
	}

	public List<CompanyDetailsQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<CompanyDetailsQueryInstance> instances) {

		this.instances = instances;
	}

	public String getServiceName() {

		return serviceName;
	}

	public void setServiceName(String serviceName) {

		this.serviceName = serviceName;
	}

	public TargetEnvironment getTargetEnvironment() {

		return targetEnvironment;
	}

	public void setTargetEnvironment(TargetEnvironment targetEnvironment) {

		this.targetEnvironment = targetEnvironment;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		appendFieldDefenition("CompanyName", false, doc, sequenceElement);

		appendFieldDefenition("OrganizationNumber", false, doc, sequenceElement);

		appendFieldDefenition("NameProtectionNumber", false, doc, sequenceElement);

		appendFieldDefenition("LegalEntityAddress", false, doc, sequenceElement);
		appendFieldDefenition("LegalEntityCOAddress", false, doc, sequenceElement);
		appendFieldDefenition("LegalEntityZip", false, doc, sequenceElement);
		appendFieldDefenition("LegalEntityPostalAddress", false, doc, sequenceElement);

		appendFieldDefenition("Email", false, doc, sequenceElement);
		appendFieldDefenition("Phone", false, doc, sequenceElement);

		appendFieldDefenition("CivilRegistrationAddress", false, doc, sequenceElement);
		appendFieldDefenition("CivilRegistrationCOAddress", false, doc, sequenceElement);
		appendFieldDefenition("CivilRegistrationZip", false, doc, sequenceElement);
		appendFieldDefenition("CivilRegistrationPostalAddress", false, doc, sequenceElement);

		appendFieldDefenition("CompanyType", false, doc, sequenceElement);
		appendFieldDefenition("CompanyTypeDescription", false, doc, sequenceElement);

		appendFieldDefenition("Signing", false, doc, sequenceElement);

		appendFieldDefenition("BusinessDescription", false, doc, sequenceElement);

		appendFieldDefenition("Officers", false, doc, sequenceElement);

		appendFieldDefenition("Shares", false, doc, sequenceElement);
		
		appendFieldDefenition("FTaxRegistered", false, doc, sequenceElement);
		
		appendFieldDefenition("VATRegistered", false, doc, sequenceElement);
		
		appendFieldDefenition("EmployerRegistration", false, doc, sequenceElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	private void appendFieldDefenition(String name, boolean required, Document doc, Element sequenceElement) {

		Element fieldElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		fieldElement.setAttribute("name", name);
		fieldElement.setAttribute("type", "xs:string");
		fieldElement.setAttribute("minOccurs", required ? "1" : "0");
		fieldElement.setAttribute("maxOccurs", "1");

		sequenceElement.appendChild(fieldElement);
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return "CompanyDetailsQuery (queryID: " + queryID + ")";
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		if (companyTypes != null) {

			ArrayList<ImmutableAlternative> alternatives = new ArrayList<ImmutableAlternative>(companyTypes.size() + 2);

			for (CompanyType companyType : companyTypes) {

				alternatives.add(companyType.getAlternative());

				if (companyType == CompanyType.E) {

					alternatives.add(new UnregisteredCompanyTypeAlternative());
				}
			}

			if (useUD0004) {

				alternatives.add(UD0004_ALTERNATIVE);
			}

			if (useUD0006) {

				alternatives.add(UD0006_ALTERNATIVE);
			}

			if (useUD0029) {

				alternatives.add(UD0029_ALTERNATIVE_YES);
				alternatives.add(UD0029_ALTERNATIVE_NO);
				alternatives.add(UD0029_ALTERNATIVE_ACCESS_DENIED);
			}

			if (useUD0031) {

				alternatives.add(UD0031_ALTERNATIVE_YES);
				alternatives.add(UD0031_ALTERNATIVE_NO);
				alternatives.add(UD0031_ALTERNATIVE_ACCESS_DENIED);
			}
			
			if (useUD0033) {

				alternatives.add(UD0033_ALTERNATIVE_YES);
				alternatives.add(UD0033_ALTERNATIVE_NO);
				alternatives.add(UD0033_ALTERNATIVE_ACCESS_DENIED);
			}
			
			return alternatives;
		}

		return null;
	}

	@Override
	public String getFreeTextAlternative() {

		return null;
	}

	@Override
	public Map<Integer, Integer> getAlternativeConversionMap() {

		return null;
	}

	public boolean isShowCompanyTypeDescription() {

		return showCompanyTypeDescription;
	}

	public void setShowCompanyTypeDescription(boolean showCompanyTypeDescription) {

		this.showCompanyTypeDescription = showCompanyTypeDescription;
	}

	public boolean isShowCompaniesAsRadiosAndKeepShowing() {

		return showCompaniesAsRadiosAndKeepShowing;
	}

	public void setShowCompaniesAsRadiosAndKeepShowing(boolean showCompaniesAsRadiosAndKeepShowing) {

		this.showCompaniesAsRadiosAndKeepShowing = showCompaniesAsRadiosAndKeepShowing;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

}
