package com.nordicpeak.flowengine.evaluators.sendtextfieldsmsevaluator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.SwedishPhoneNumberValidator;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.EvaluatorDescriptor;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.events.StatusChangedByManagerEvent;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryInstanceNotFoundInQueryProviderException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderErrorException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderNotFoundException;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQueryInstance;

public class SendTextFieldSMSEvaluationProviderModule extends BaseEvaluationProviderModule<SendTextFieldSMSEvaluator> {

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency
	protected FlowInstanceAdminModule flowInstanceAdminModule;
	
	@InstanceManagerDependency
	private StandardFlowNotificationHandler notificationHandler;

	protected AnnotatedDAO<SendTextFieldSMSEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SendTextFieldSMSEvaluator, Integer> evaluatorIDParamFactory;

	protected SendTextFieldSMSEvaluatorCRUD evaluatorCRUD;

	
	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);
	}	
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SendTextFieldSMSEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SendTextFieldSMSEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SendTextFieldSMSEvaluatorCRUD(SendTextFieldSMSEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SendTextFieldSMSEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		return null;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 50)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {

		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();

		sendSMS(flowInstanceManager, event.getEvent().getPoster(), event.getSiteProfile());
	}

	@EventListener(channel = FlowInstance.class, priority = 50)
	public void processEvent(StatusChangedByManagerEvent event, EventSource eventSource) throws SQLException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException {
		
		Flow flow = flowAdminModule.getCachedFlow(event.getFlowInstance().getFlow().getFlowID());
		
		if(containsStatusFilteredEvaluators(flow)){
			
			FlowInstance flowInstance = flowAdminModule.getFlowInstance(event.getFlowInstance().getFlowInstanceID());
			
			ImmutableFlowInstanceManager flowInstanceManager = getImmutableFlowInstanceManager(flowInstance, event.getSiteProfile());
			
			sendSMS(flowInstanceManager, event.getUser(), event.getSiteProfile());
			
		}
	}
	
	protected void sendSMS(FlowInstanceManager flowInstanceManager, User eventUser, SiteProfile siteProfile) throws SQLException{
		
		List<TextFieldQueryInstance> queryInstances = flowInstanceManager.getQueries(TextFieldQueryInstance.class);

		if (queryInstances != null) {

			for (TextFieldQueryInstance queryInstance : queryInstances) {

				if (queryInstance == null || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {

					continue;
				}

				for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {

					if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {

						if(notificationHandler == null){
							
							log.error("Notification handler not set, unable to send SMS regarding flow instance " + flowInstanceManager.getFlowInstance());
							return;
						}						
						
						SendTextFieldSMSEvaluator evaluator = getEvaluator(evaluatorDescriptor);

						if(evaluator.enableStatusFiltering() && (evaluator.getStatusIDs() == null || !evaluator.getStatusIDs().contains(flowInstanceManager.getFlowInstance().getStatus().getStatusID()))){
							
							continue;
						}
						
						String mobile = queryInstance.getFieldValue(evaluator.getTextFieldID());
						
						if (mobile != null && SwedishPhoneNumberValidator.getValidator().validateFormat((mobile))) {

							try {

								ImmutableFlowInstance flowInstance = flowInstanceManager.getFlowInstance();

								log.info("Sending SMS about submitted flow instance " + flowInstance + " to recipients " + mobile + " trigged by evaluator " + evaluatorDescriptor);
								
								Contact contact = notificationHandler.getPosterContact(flowInstance);
								
								notificationHandler.sendGlobalSMS(flowInstance, contact, mobile, evaluator.getSMSMessage());

							} catch (Exception e) {

								log.error("Unable to send SMS to " + mobile + " about submitted flow instance " + flowInstanceManager.getFlowInstance() + " triggered by user " + eventUser, e);
							}
						}
					}
				}
			}
		}		
	}
	
	private boolean containsStatusFilteredEvaluators(Flow flow) throws SQLException {

		for(Step step : flow.getSteps()){
			
			if(step.getQueryDescriptors() != null){
				
				for(QueryDescriptor queryDescriptor : step.getQueryDescriptors()){
					
					if(queryDescriptor.getEvaluatorDescriptors() != null){

						for(EvaluatorDescriptor evaluatorDescriptor : queryDescriptor.getEvaluatorDescriptors()){
							
							if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {
							
								SendTextFieldSMSEvaluator evaluator = getEvaluator(evaluatorDescriptor);
								
								if(evaluator.enableStatusFiltering()){
									
									return true;
								}
							}
						}
					}
				}
			}
		}
		
		return false;
	}
	
	public ImmutableFlowInstanceManager getImmutableFlowInstanceManager(FlowInstance flowInstance, SiteProfile siteProfile) throws DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException{
		
		return new ImmutableFlowInstanceManager(flowInstance, queryHandler, null, new DefaultInstanceMetadata(siteProfile), null);
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SendTextFieldSMSEvaluator evaluator = new SendTextFieldSMSEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	private SendTextFieldSMSEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {

		return evaluatorDAO.get(new HighLevelQuery<SendTextFieldSMSEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SendTextFieldSMSEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SendTextFieldSMSEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<SendTextFieldSMSEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return TextFieldQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SendTextFieldSMSEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SendTextFieldSMSEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		List<TextField> sourceTextFields = ((TextFieldQuery) sourceQuery).getFields();
		List<TextField> copyTextFields = ((TextFieldQuery) copyQuery).getFields();

		if(sourceTextFields != null && copyTextFields != null) {
		
			int textFieldIndex = 0;

			while (textFieldIndex < sourceTextFields.size()) {

				if (sourceTextFields.get(textFieldIndex).getTextFieldID().equals(evaluator.getTextFieldID())) {

					evaluator.setTextFieldID(copyTextFields.get(textFieldIndex).getTextFieldID());
					
					break;
				}

				textFieldIndex++;
			}
		
		}		
		
		convertStatusIDs(evaluator, statusConversionMap);
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);

	}

	@Override
	public SendTextFieldSMSEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SendTextFieldSMSEvaluator evaluator = new SendTextFieldSMSEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		if (evaluator.getTextFieldID() != null && query instanceof TextFieldQuery && ((TextFieldQuery) query).getTextFieldConversionMap() != null) {

			Map<Integer, Integer> conversionMap = ((TextFieldQuery) query).getTextFieldConversionMap();

			Integer newTextFieldID = conversionMap.get(evaluator.getTextFieldID());

			evaluator.setTextFieldID(newTextFieldID);
		}

		convertStatusIDs(evaluator, statusConversionMap);
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	private void convertStatusIDs(SendTextFieldSMSEvaluator evaluator, Map<Integer, ImmutableStatus> statusConversionMap) {

		if(evaluator.getStatusIDs() != null && statusConversionMap != null){
			
			List<Integer> convertedStatusIDs = new ArrayList<Integer>(evaluator.getStatusIDs().size());
			
			for(Integer statusID : evaluator.getStatusIDs()){
				
				ImmutableStatus status = statusConversionMap.get(statusID);
				
				if(status != null){
					
					convertedStatusIDs.add(status.getStatusID());
				}
			}
			
			evaluator.setStatusIDs(convertedStatusIDs);
		}
	}	
	
	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

	public String getAvailableTags() {
		
		return StringUtils.toCommaSeparatedString(StandardFlowNotificationHandler.getGlobalTags());
	}
}
