<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateSetContactDetailsEvaluator" />
			
		</div>

	</xsl:template>

	<xsl:template match="UpdateSetContactDetailsEvaluator">

		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluator.Title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="SetContactDetailsEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<div class="floatleft full bigmarginbottom">

				<label for="name" class="floatleft full">
					<xsl:value-of select="$i18n.EvaluatorDescriptor.Name" />
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'name'" />
						<xsl:with-param name="name" select="'name'" />
						<xsl:with-param name="element" select="SetContactDetailsEvaluator/EvaluatorDescriptor" />
					</xsl:call-template>
				</div>
				
			</div>

			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'enabled'" />
						<xsl:with-param name="name" select="'enabled'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="SetContactDetailsEvaluator/EvaluatorDescriptor" />
					</xsl:call-template>
	
					<label for="enabled">
						<xsl:value-of select="$i18n.EvaluatorDescriptor.Enabled" />
					</label>
				</div>
			</div>

			<label class="floatleft">
				<xsl:value-of select="$i18n.SetContactDetailsEvaluator.Title" />
			</label>

			<div class="clearboth floatleft marginbottom">

				<xsl:choose>
					<xsl:when test="TextFields/TextField">
					
						<xsl:value-of select="$i18n.SetContactDetailsEvaluator.Description" />
	
						<table>
							<tbody>

								<xsl:call-template name="contactDetailRow">
									<xsl:with-param name="name" select="'emailTextFieldID'" />
									<xsl:with-param name="selectedValue" select="SetContactDetailsEvaluator/emailTextFieldID" />
									<xsl:with-param name="label" select="$i18n.SetContactDetailsEvaluator.Email" />
								</xsl:call-template>								

								<xsl:call-template name="contactDetailRow">
									<xsl:with-param name="name" select="'phoneTextFieldID'" />
									<xsl:with-param name="selectedValue" select="SetContactDetailsEvaluator/phoneTextFieldID" />
									<xsl:with-param name="label" select="$i18n.SetContactDetailsEvaluator.Phone" />
								</xsl:call-template>								

								<xsl:call-template name="contactDetailRow">
									<xsl:with-param name="name" select="'mobilePhoneTextFieldID'" />
									<xsl:with-param name="selectedValue" select="SetContactDetailsEvaluator/mobilePhoneTextFieldID" />
									<xsl:with-param name="label" select="$i18n.SetContactDetailsEvaluator.MobilePhone" />
								</xsl:call-template>								

							</tbody>
						</table>
						
						<div class="bigmargintop">
							
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="id" select="'overwriteExistingContactDetails'" />
								<xsl:with-param name="name" select="'overwriteExistingContactDetails'" />
								<xsl:with-param name="element" select="SetContactDetailsEvaluator" />
							</xsl:call-template>
						
							<label for="overwriteExistingContactDetails">
								<xsl:value-of select="$i18n.SetContactDetailsEvaluator.OverwriteExistingContactDetails" />
							</label>
							
						</div>
					
					</xsl:when>
					<xsl:otherwise>
						<p>
							<xsl:value-of select="$i18n.QueryHasNoTextFields" />
						</p>
					</xsl:otherwise>
				</xsl:choose>
			</div>

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.UpdateEvaluator.Submit}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template name="contactDetailRow">
		<xsl:param name="name" />
		<xsl:param name="selectedValue" />
		<xsl:param name="label" />
		
		<tr>
			<td>
				<xsl:value-of select="$label" />
			</td>
			<td>
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="name" select="$name"/>
					<xsl:with-param name="element" select="TextFields/TextField"/>
					<xsl:with-param name="labelElementName" select="'label'"/>
					<xsl:with-param name="valueElementName" select="'textFieldID'"/>
					<xsl:with-param name="addEmptyOption" select="$i18n.SetContactDetailsEvaluator.Select"/>
					<xsl:with-param name="selectedValue" select="$selectedValue"/>
				</xsl:call-template>			
			</td>
		</tr>
	
	</xsl:template>

	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>

				<xsl:text>&#x20;</xsl:text>

				<xsl:choose>
					<xsl:when test="fieldName = 'name'">
						<xsl:value-of select="$i18n.EvaluatorDescriptor.Name" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName" />
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>

		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>

	</xsl:template>

</xsl:stylesheet>