package com.nordicpeak.flowengine.integration.sokigo.beans;

import java.util.List;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationSettings;

@Table(name = "sokigo_integration_settings")
@XMLElement
public class IntegrationSettings extends BaseIntegrationSettings {

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "password")
	@XMLElement
	private String username;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "username")
	@XMLElement
	private String password;

	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoGet = true)
	@XMLElement
	private IntegrationEndpoint endpoint;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String serviceName;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String apiKey;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendErrorEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "sokigo_integration_email_recipients_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "Address")
	private List<String> errorEmailAddresses;

	public String getUsername() {

		return username;
	}

	public void setUsername(String username) {

		this.username = username;
	}

	public String getPassword() {

		return password;
	}

	public void setPassword(String password) {

		this.password = password;
	}

	@Override
	public boolean isPopulated() {

		if (isEnabled() || !ObjectUtils.isNull(endpoint, username, password, apiKey, serviceName)) {
			return true;
		}

		return false;
	}

	public IntegrationEndpoint getEndpoint() {
		return endpoint;
	}

	public void setEndpoint(IntegrationEndpoint integrationEndpoint) {
		this.endpoint = integrationEndpoint;
	}

	public String getApiKey() {
		return apiKey;
	}

	public void setApiKey(String apiKey) {
		this.apiKey = apiKey;
	}

	public String getServiceName() {
		return serviceName;
	}

	public void setServiceName(String serviceName) {
		this.serviceName = serviceName;
	}

	public boolean isSendErrorEmail() {

		return sendErrorEmail;
	}

	public void setSendErrorEmail(boolean sendErrorEmail) {

		this.sendErrorEmail = sendErrorEmail;
	}

	public List<String> getErrorEmailAddresses() {

		return errorEmailAddresses;
	}

	public void setErrorEmailAddresses(List<String> errorEmailAddresses) {

		this.errorEmailAddresses = errorEmailAddresses;
	}

}
