<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateStatusOverrideEvaluator" />
		</div>

	</xsl:template>

	<xsl:template match="UpdateStatusOverrideEvaluator">

		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluatorDescriptor.title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="StatusOverrideEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="StatusOverrideEvaluator" />
			</xsl:call-template>
			
			<label class="floatleft">
				<xsl:value-of select="$i18n.useForTitle" />
			</label>
			
			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'useForSubmit'" />
						<xsl:with-param name="name" select="'useForSubmit'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="StatusOverrideEvaluator" />
					</xsl:call-template>
	
					<label for="useForSubmit">
						<xsl:value-of select="$i18n.useForSubmit" />
					</label>
				</div>
			</div>
			
			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'useForManagerEdit'" />
						<xsl:with-param name="name" select="'useForManagerEdit'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="StatusOverrideEvaluator" />
					</xsl:call-template>
	
					<label for="useForManagerEdit">
						<xsl:value-of select="$i18n.useForManagerEdit" />
					</label>
				</div>
			</div>

			<label class="floatleft">
				<xsl:value-of select="$i18n.alternatives.title" />
			</label>

			<div class="clearboth floatleft marginbottom">
				<xsl:value-of select="$i18n.alternatives.description" />

				<xsl:choose>
					<xsl:when test="Alternatives/Alternative">
					
						<table>
							<thead>
								<tr>
									<th><xsl:value-of select="$i18n.alternatives.name" /></th>
									<th><xsl:value-of select="$i18n.alternatives.status" /></th>
								</tr>
							</thead>
							<tbody>
								<xsl:apply-templates select="Alternatives/Alternative" />
							</tbody>
						</table>
					
					</xsl:when>
					<xsl:otherwise>
						<p>
							<xsl:value-of select="$i18n.QueryHasNoAlternatives" />
						</p>
					</xsl:otherwise>
				</xsl:choose>
			</div>

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.UpdateStatusOverrideEvaluator.submit}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template match="Alternative">

		<xsl:variable name="alternativeID" select="alternativeID"/>

		<tr>
			<td>
				<xsl:value-of select="name" />
			</td>
			<td>
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="name">
						<xsl:value-of select="'alternative'"/>
						<xsl:value-of select="alternativeID"/>
					</xsl:with-param>
					<xsl:with-param name="element" select="../../Statuses/Status[not(contentType = 'NEW' or contentType = 'WAITING_FOR_MULTISIGN' or contentType = 'WAITING_FOR_PAYMENT')]"/>
					<xsl:with-param name="labelElementName" select="'name'"/>
					<xsl:with-param name="valueElementName" select="'statusID'"/>
					<xsl:with-param name="addEmptyOption" select="$i18n.alternatives.empty"/>
					<xsl:with-param name="selectedValue" select="../../StatusOverrideEvaluator/AlternativeStatusOverrides/AlternativeStatusOverride[alternativeID = $alternativeID]/statusID"/>
					<xsl:with-param name="requestparameters" select="../../requestparameters"/>
				</xsl:call-template>
			</td>
		</tr>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'UseForRequired']">
		<p class="error">
			<xsl:value-of select="$i18n.Validation.UseForRequired" />
		</p>
	</xsl:template>

</xsl:stylesheet>