package com.nordicpeak.flowengine.evaluators.statusoverrideevaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;

public class StatusOverrideEvaluatorCRUD extends BaseEvaluatorCRUD<StatusOverrideEvaluator, StatusOverrideEvaluationProviderModule> {

	private static final BeanRequestPopulator<StatusOverrideEvaluator> POPULATOR = new AnnotatedRequestPopulator<StatusOverrideEvaluator>(StatusOverrideEvaluator.class);

	public StatusOverrideEvaluatorCRUD(Class<StatusOverrideEvaluator> beanClass, AnnotatedDAOWrapper<StatusOverrideEvaluator, Integer> evaluatorDAO, StatusOverrideEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "StatusOverrideEvaluator", "status override evaluator", null, callback);
	}

	@Override
	protected void appendUpdateFormData(StatusOverrideEvaluator evaluator, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(evaluator, doc, updateTypeElement, user, req, uriParser);

		FixedAlternativesQuery query = (FixedAlternativesQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) evaluator.getEvaluatorDescriptor().getQueryDescriptor(), true);

		List<? extends ImmutableAlternative> alternatives = query.getAlternatives();

		if (alternatives != null) {

			Element alternativesElement = doc.createElement("Alternatives");
			updateTypeElement.appendChild(alternativesElement);

			for (ImmutableAlternative alternative : alternatives) {

				Element alternativeElement = doc.createElement("Alternative");
				alternativesElement.appendChild(alternativeElement);

				XMLUtils.appendNewElement(doc, alternativeElement, "alternativeID", alternative.getAlternativeID());
				XMLUtils.appendNewCDATAElement(doc, alternativeElement, "name", alternative.getName());
			}
		}

		List<? extends ImmutableStatus> statuses = callback.getFlowStatuses(evaluator.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow().getFlowID());

		XMLUtils.append(doc, updateTypeElement, "Statuses", statuses);
	}

	@Override
	protected StatusOverrideEvaluator populateFromUpdateRequest(StatusOverrideEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		StatusOverrideEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		if (!evaluator.isUseForSubmit() && !evaluator.isUseForManagerEdit()) {
			
			validationErrors.add(new ValidationError("UseForRequired"));
		}
		
		FixedAlternativesQuery query = (FixedAlternativesQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), true);
		
		List<? extends ImmutableAlternative> alternatives = query.getAlternatives();
		
		List<AlternativeStatusOverride> alternativeWeights = null;
		
		if (alternatives != null) {
			
			alternativeWeights = new ArrayList<AlternativeStatusOverride>();
			
			for (ImmutableAlternative alternative : alternatives) {
				
				String paramName = "alternative" + alternative.getAlternativeID();
				
				Integer statusID = ValidationUtils.validateParameter(paramName, req, false, IntegerPopulator.getPopulator(), validationErrors);
				
				if (statusID != null) {

					boolean statusFound = false;
					
					List<? extends ImmutableStatus> statuses = callback.getFlowStatuses(evaluator.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow().getFlowID());

					if (statuses != null) {
						for (ImmutableStatus status : statuses) {

							if (status.getStatusID().equals(statusID)) {

								statusFound = true;
								break;
							}
						}
					}
					
					if (statusFound) {
						
						AlternativeStatusOverride alternativeWeight = new AlternativeStatusOverride();
						
						alternativeWeight.setAlternativeID(alternative.getAlternativeID());
						alternativeWeight.setStatusID(statusID);
						
						alternativeWeights.add(alternativeWeight);
						
					} else {
					
						validationErrors.add(new ValidationError(paramName, ValidationErrorType.InvalidFormat));
					}
				}
			}
		}
		
		evaluator.setAlternativeStatusOverrides(alternativeWeights);
		
		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}
		
		return evaluator;
	}

}
