package com.nordicpeak.flowengine.evaluators.statusquerystate;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

public class StatusQueryStateEvaluationProviderModule extends BaseQueryStateEvaluationProviderModule<StatusQueryStateEvaluator> {

	protected AnnotatedDAO<StatusQueryStateEvaluator> evaluatorDAO;

	protected QueryParameterFactory<StatusQueryStateEvaluator, Integer> evaluatorIDParamFactory;

	protected StatusQueryStateEvaluatorCRUD evaluatorCRUD;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, StatusQueryStateEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(StatusQueryStateEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new StatusQueryStateEvaluatorCRUD(StatusQueryStateEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, StatusQueryStateEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (evaluator.getStatusIDs() == null || callback.getFlowInstance().getStatus() == null) {
			return null;
		}

		if (evaluator.getStatusIDs().contains(callback.getFlowInstance().getStatus().getStatusID())) {

			return applyEvaluatorQueryStates(queryInstance, evaluator, callback, attributeHandler);

		} else {

			return restoreDefaultQueryStates(queryInstance, evaluator, callback, attributeHandler);
		}
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		StatusQueryStateEvaluator evaluator = new StatusQueryStateEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return true;
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		StatusQueryStateEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<StatusQueryStateEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		if (evaluator.getStatusIDs() != null) {

				List<Integer> newIDList = new ArrayList<Integer>(evaluator.getStatusIDs().size());

				for (Integer statusID : evaluator.getStatusIDs()) {

					ImmutableStatus newStatus = statusConversionMap.get(statusID);

					if (newStatus != null) {

						newIDList.add(newStatus.getStatusID());
					}
				}

				evaluator.setStatusIDs(newIDList);
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	public StatusQueryStateEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		StatusQueryStateEvaluator evaluator = importEvaluatorDescriptor(descriptor, transactionHandler, query, statusConversionMap);

		if (evaluator.getStatusIDs() != null) {

			List<Integer> newIDList = new ArrayList<>(evaluator.getStatusIDs().size());

			for (Integer statusID : evaluator.getStatusIDs()) {

				ImmutableStatus newStatus = statusConversionMap.get(statusID);

				if (newStatus != null) {

					newIDList.add(newStatus.getStatusID());
				}
			}

			evaluator.setStatusIDs(newIDList);
		}

		evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	@Override
	protected Class<StatusQueryStateEvaluator> getEvaluatorClass() {

		return StatusQueryStateEvaluator.class;
	}

	@Override
	protected AnnotatedDAO<StatusQueryStateEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	@Override
	protected QueryParameterFactory<StatusQueryStateEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return this.evaluatorIDParamFactory;
	}

	@Override
	protected List<Integer> getTargetQueryIDs(StatusQueryStateEvaluator evaluator, EvaluationCallback callback) {

		return Collections.singletonList(callback.getQueryInstance().getQueryInstanceDescriptor().getQueryDescriptor().getQueryID());
	}

	public List<? extends ImmutableStatus> getFlowStatuses(Integer flowID) {

		ImmutableFlow flow = flowAdminModule.getFlow(flowID);

		if (flow != null) {

			return flow.getStatuses();
		}

		return null;
	}
}
