package com.nordicpeak.flowengine.evaluators.tableattributesummaryevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "table_attribute_summary_evaluators")
@XMLElement
public class TableAttributeSummaryEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -911567587723344081L;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer typeColumnID;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer valueColumnID;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<AlternativeAttribute> alternativeAttributes;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		typeColumnID = XMLValidationUtils.validateParameter("typeColumnID", xmlParser, false, 1, 255, NonNegativeStringIntegerPopulator.getPopulator(), errors);
		valueColumnID = XMLValidationUtils.validateParameter("valueColumnID", xmlParser, false, 1, 255, NonNegativeStringIntegerPopulator.getPopulator(), errors);

		List<XMLParser> alternativeNodes = xmlParser.getNodes("AlternativeAttributes/AlternativeAttribute");

		if (!CollectionUtils.isEmpty(alternativeNodes)) {

			alternativeAttributes = new ArrayList<AlternativeAttribute>(alternativeNodes.size());

			for (XMLParser parser : alternativeNodes) {

				AlternativeAttribute alternativeAttribute = new AlternativeAttribute();

				alternativeAttribute.populate(parser);

				alternativeAttribute.setEvaluator(this);

				alternativeAttributes.add(alternativeAttribute);
			}
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public Integer getTypeColumnID() {

		return typeColumnID;
	}

	public void setTypeColumnID(Integer typeColumnID) {

		this.typeColumnID = typeColumnID;
	}

	public Integer getValueColumnID() {

		return valueColumnID;
	}

	public void setValueColumnID(Integer valueColumnID) {

		this.valueColumnID = valueColumnID;
	}

	public List<AlternativeAttribute> getAlternativeAttributes() {

		return alternativeAttributes;
	}

	public void setAlternativeAttributes(List<AlternativeAttribute> alternativeAttributes) {

		this.alternativeAttributes = alternativeAttributes;
	}
	
	@Override
	public String toString() {
		
		return evaluatorDescriptor.toString();
	}

}
