package com.nordicpeak.flowengine.targetgroupflowbrowser;

import java.io.IOException;
import java.net.URLDecoder;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLink;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLinkProvider;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;

public class TargetGroupAdminModule extends AnnotatedForegroundModule implements ViewFragmentModule<ForegroundModuleDescriptor>, FlowAdminExtensionViewProvider, ExtensionLinkProvider, CRUDCallback<User> {

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Priority", description = "The priority of this extension provider compared to other providers. A low value means a higher priority. Valid values are 0 - " + Integer.MAX_VALUE + ".", required = true, formatValidator = NonNegativeStringIntegerValidator.class)
	protected int priority = 0;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Extension view title", description = "The title of the view framgment for administrating target groups in show flow view", required = true)
	protected String extensionViewTitle = "Target groups";

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmententXML;

	private CopyOnWriteArrayList<TargetGroup> targetGroupsCacheList;

	private AnnotatedDAO<TargetGroup> targetGroupDAO;

	private TargetGroupCRUD targetGroupCRUD;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	private FlowAdminModule flowAdminModule;

	private StaticContentModule staticContentModule;

	private ExtensionLink extensionLink;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(TargetGroupAdminModule.class, this)) {

			throw new RuntimeException("Unable to register module in global instance handler using key " + TargetGroupAdminModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}

		this.viewFragmentTransformer = new ModuleViewFragmentTransformer<ForegroundModuleDescriptor>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());

		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return targetGroupCRUD.list(req, res, user, uriParser, null);
	}

	@Override
	public void update(ForegroundModuleDescriptor descriptor, DataSource dataSource) throws Exception {

		super.update(descriptor, dataSource);

		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, "com.nordicpeak.flowengine.TargetGroupAdminModule", new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		targetGroupDAO = daoFactory.getDAO(TargetGroup.class);
		targetGroupCRUD = new TargetGroupCRUD(targetGroupDAO.getWrapper(Integer.class), this);

		cacheTargetGroups();
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse addTargetGroup(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return targetGroupCRUD.add(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateTargetGroup(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return targetGroupCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse deleteTargetGroup(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return targetGroupCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse searchFlow(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		String search = req.getParameter("q");

		if (StringUtils.isEmpty(search)) {

			sendEmptyJSONResponse(res);
			return null;
		}

		search = URLDecoder.decode(search, "UTF-8");
		search = search.toLowerCase();

		Collection<Flow> flows = flowAdminModule.getCachedFlows();

		JsonArray jsonArray = new JsonArray();

		if (flows != null) {

			for (Flow flow : flows) {

				if (flow.isLatestVersion() && flow.getName().toLowerCase().matches("(.*)" + search + "(.*)")) {

					JsonObject instance = new JsonObject(2);
					instance.putField("FlowID", flow.getFlowID().toString());
					instance.putField("FlowFamilyID", flow.getFlowFamily().getFlowFamilyID().toString());
					instance.putField("Name", flow.getName());
					jsonArray.addNode(instance);
				}
			}

		}

		log.info("User " + user + " searching for flows using query " + search + ", found " + jsonArray.size() + " hits");

		sendJSONResponse(jsonArray, res);

		return null;

	}

	protected static void sendEmptyJSONResponse(HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(1);
		jsonObject.putField("hitCount", "0");
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	protected void sendJSONResponse(JsonArray jsonArray, HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(2);
		jsonObject.putField("hitCount", Integer.toString(jsonArray.size()));
		jsonObject.putField("hits", jsonArray);
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateFlowTargetGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());

		}

		if (req.getMethod().equalsIgnoreCase("POST")) {

			Integer flowFamilyID = flow.getFlowFamily().getFlowFamilyID();

			List<Integer> targetGroupIDs = NumberUtils.toInt(req.getParameterValues("targetGroup"));

			List<TargetGroup> targetGroups = targetGroupDAO.getAll();

			if (targetGroups != null) {

				for (TargetGroup targetGroup : targetGroups) {

					boolean selected = (targetGroupIDs != null && targetGroupIDs.contains(targetGroup.getTargetGroupID()));

					List<Integer> flowFamilyIDs = targetGroup.getFlowFamilyIDs();

					if (selected) {

						if (flowFamilyIDs == null || !flowFamilyIDs.contains(flowFamilyID)) {

							flowFamilyIDs = CollectionUtils.addAndInstantiateIfNeeded(flowFamilyIDs, flowFamilyID);
						}

					} else if (flowFamilyIDs != null) {

						flowFamilyIDs.remove(flowFamilyID);

					}

					targetGroup.setFlowFamilyIDs(flowFamilyIDs);

				}

				targetGroupDAO.update(targetGroups, null);

				cacheTargetGroups();

			}

			log.info("User " + user + " updated target groups for flow " + flow);

			flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#targetgroups");

			return null;

		}

		log.info("User " + user + " requesting update target groups form for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element targetGroupsElement = doc.createElement("UpdateFlowTargetGroups");
		doc.getDocumentElement().appendChild(targetGroupsElement);

		targetGroupsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, targetGroupsElement, getCachedTargetGroups());

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());

	}

	public void cacheTargetGroups() throws SQLException {

		List<TargetGroup> targetGroups = targetGroupDAO.getAll();

		if (targetGroups == null) {

			targetGroupsCacheList = null;

			return;
		}

		Collections.sort(targetGroups);

		this.targetGroupsCacheList = new CopyOnWriteArrayList<TargetGroup>(targetGroups);

	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");
		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		document.appendChild(this.sectionInterface.getSectionDescriptor().toXML(doc));
		document.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(document);

		return doc;
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	public List<TargetGroup> getCachedTargetGroups() {

		return targetGroupsCacheList;
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(TargetGroupAdminModule.class, this);

		if(flowAdminModule != null) {
		
			flowAdminModule.removeExtensionViewProvider(this);

			flowAdminModule.removeFlowListExtensionLinkProvider(this);
		
		}

		super.unload();
	}

	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if(flowAdminModule == null && this.flowAdminModule != null){
			
			this.flowAdminModule.removeExtensionViewProvider(this);
		}
		
		this.flowAdminModule = flowAdminModule;

		if (this.flowAdminModule != null) {

			this.flowAdminModule.addExtensionViewProvider(this);

			this.flowAdminModule.addFlowListExtensionLinkProvider(this);
		}

	}

	@Override
	public int getPriority() {

		return priority;
	}

	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {

		if (!AccessUtils.checkAccess(user, this.moduleDescriptor) && getCachedTargetGroups() != null) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element showViewElement = doc.createElement("ShowView");
		doc.getDocumentElement().appendChild(showViewElement);

		XMLUtils.append(doc, showViewElement, getCachedTargetGroups());

		showViewElement.appendChild(flow.toXML(doc));

		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc), true);

	}

	@Override
	public String getExtensionViewTitle() {

		return extensionViewTitle;
	}

	@Override
	public AccessInterface getAccessInterface() {

		return flowAdminModule;
	}

	@InstanceManagerDependency(required = true)
	public void setStaticContentModule(StaticContentModule staticContentModule) {

		generateExtensionLink(staticContentModule);

		this.staticContentModule = staticContentModule;
	}

	@Override
	public ExtensionLink getExtensionLink(User user) {

		if (hasRequiredDependencies) {

			return extensionLink;
		}

		return null;
	}

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		generateExtensionLink(staticContentModule);
	}

	private void generateExtensionLink(StaticContentModule staticContentModule) {

		if (staticContentModule != null) {

			extensionLink = new ExtensionLink(moduleDescriptor.getName(), systemInterface.getContextPath() + this.getFullAlias(), staticContentModule.getModuleContentURL(moduleDescriptor) + "/pics/pen.png", "bottom-right");

		} else {

			extensionLink = null;
		}
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public FlowAdminModule getFlowAdminModule() {

		return flowAdminModule;
	}

	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}

}
