<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/popularflowfamilies.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/targetgroupflowbrowser.css
	</xsl:variable>
	
	<xsl:template match="Document">
	
		<div class="popular-flows-wrapper">
 			<div class="inner">
	 			<h2><xsl:value-of select="module/name"/></h2>
	 			<ul class="flow-list">
	 				<xsl:apply-templates select="Flows/Flow"/>
	 			</ul>
 			</div>
 		</div>
	
	</xsl:template>
	
	<xsl:template match="Flow">
	
		<li id="popularflow_{flowID}" data-flowid="{flowID}" data-flowname="{name}">
			
			<xsl:attribute name="class">
				<xsl:if test="enabled = 'false'">disabled </xsl:if>
			</xsl:attribute>

			<xsl:variable name="targetBlank" select="../openExternalFlowsInNewWindow = 'true' and skipOverview = 'true' and externalLink" />
			<xsl:variable name="flowLink">
				<xsl:value-of select="../../browserModuleAlias" />/overview/<xsl:value-of select="FlowFamily/flowFamilyID" />
			</xsl:variable>

			<div class="flow">
				<figure>
					<img alt="" src="{../../browserModuleAlias}/icon/{flowID}" />
				</figure>
				<div>
					<a title="{$i18n.ShowFlow} {name}" aria-label="{$i18n.ShowFlow} {name}" href="{$flowLink}">
						<xsl:if test="$targetBlank">
							<xsl:attribute name="target">_blank</xsl:attribute>
						</xsl:if>
						<xsl:if test="enabled = 'false'">
							<xsl:attribute name="href">#</xsl:attribute>
						</xsl:if>
						<h2>
							<xsl:value-of select="name" />
						</h2>
						<div class="icons only-desktop-tablet">
							<xsl:choose>
								<xsl:when test="externalLink">
									<i class="material-icons icon-launch" title="{$i18n.IsExternal}" aria-label="{$i18n.IsExternal}"></i>
								</xsl:when>
								<xsl:when test="Steps/Step">
									<i class="material-icons icon-computer" title="{$i18n.HasFlow}" aria-label="{$i18n.HasFlow}"></i>
								</xsl:when>
							</xsl:choose>
							<xsl:if test="hasPDF = 'true'">
								<i class="material-icons icon-file-download" title="{$i18n.HasPDF}" aria-label="{$i18n.HasPDF}"></i>
							</xsl:if>
							<xsl:if test="not(../../loggedIn) and requireAuthentication = 'true'">
								<i class="material-icons icon-person" title="{$i18n.AuthenticationRequired}" aria-label="{$i18n.AuthenticationRequired}"></i>
							</xsl:if>
						</div>
					</a>
					<div class="description"><xsl:value-of select="shortDescription" disable-output-escaping="yes" /></div>
					<div class="description-info">
						<a href="#" tabindex="0" />
						<div class="description-tooltip"><div><xsl:value-of select="shortDescription" disable-output-escaping="yes" /><div class="marker"></div></div></div>
					</div>
					
					<xsl:if test="enabled = 'false'">
						<div class="disabled-info"><xsl:value-of select="$i18n.FlowDisabled" /></div>
					</xsl:if>
					
				</div>
				
				<a title="{$i18n.ShowFlow} {name}" href="{$flowLink}" class="btn btn-blue btn-inline only-desktop-tablet" aria-hidden="true" tabindex="-1">
					<xsl:if test="$targetBlank">
						<xsl:attribute name="target">_blank</xsl:attribute>
					</xsl:if>
					<xsl:if test="enabled = 'false'">
						<xsl:attribute name="href">#</xsl:attribute>
					</xsl:if>
					<xsl:value-of select="$i18n.ShowFlow" />
					<i class="material-icons icon-arrow-right" aria-hidden="true"></i>
				</a>
				<a title="{$i18n.ShowFlow}" href="{$flowLink}" class="btn btn-blue btn-inline only-mobile" aria-hidden="true" tabindex="-1">
					<xsl:if test="$targetBlank">
						<xsl:attribute name="target">_blank</xsl:attribute>
					</xsl:if>
					<xsl:if test="enabled = 'false'">
						<xsl:attribute name="href">#</xsl:attribute>
					</xsl:if>
					<span class="arrow" />
				</a>
				
			</div>
		</li>
	
	</xsl:template>					
	
</xsl:stylesheet>