package com.nordicpeak.flowengine.evaluators.setweightbyexistingtreservacaseevaluator;

import java.io.IOException;
import java.math.BigDecimal;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.integration.base.exceptions.IntegrationRequestException;
import com.nordicpeak.flowengine.integration.treserva.TreservaIntegrationProviderModule;
import com.nordicpeak.flowengine.integration.treserva.beans.CaseData;
import com.nordicpeak.flowengine.integration.treserva.beans.IntegrationSettings;
import com.nordicpeak.flowengine.integration.treserva.clients.TreservaMinaIFOArendeClient;
import com.nordicpeak.flowengine.interfaces.CitizenIdentifierQuery;
import com.nordicpeak.flowengine.interfaces.CitizenIdentifierQueryInstance;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;

public class SetWeightByExistingTreservaCaseEvaluationProviderModule extends BaseEvaluationProviderModule<SetWeightByExistingTreservaCaseEvaluator> {

	protected SetWeightByExistingTreservaCaseEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetWeightByExistingTreservaCaseEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetWeightByExistingTreservaCaseEvaluator, Integer> evaluatorIDParamFactory;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	private TreservaIntegrationProviderModule treservaIntegrationProviderModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetWeightByExistingTreservaCaseEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetWeightByExistingTreservaCaseEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetWeightByExistingTreservaCaseEvaluatorCRUD(SetWeightByExistingTreservaCaseEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetWeightByExistingTreservaCaseEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		String queryInstanceCitizenIdentifier = null;

		if (queryInstance instanceof CitizenIdentifierQueryInstance) {

			queryInstanceCitizenIdentifier = ((CitizenIdentifierQueryInstance) queryInstance).getCitizenIdentifier();
		}

		IntegrationSettings settings = getSettings(callback.getFlowInstance());

		if (settings == null || !settings.isEnabled()) {

			return null;
		}

		TreservaMinaIFOArendeClient client = getClient(settings);

		BigDecimal weight = BigDecimal.ZERO;

		try {

			boolean hasCaseData = hasCaseData(queryInstance, queryInstanceCitizenIdentifier, poster, client, evaluator, attributeHandler);

			weight = BigDecimal.valueOf(hasCaseData ? evaluator.getCaseFoundWeight() : evaluator.getCaseNotFoundWeight());

		} catch (IntegrationRequestException e) {

			log.warn("Setting api error weight due to error checking for case data for query instance " + queryInstance + " and poster " + poster, e);

			weight = BigDecimal.valueOf(evaluator.getApiErrorWeight());
		}

		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), weight, attributeHandler), true);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}

		return null;
	}

	private IntegrationSettings getSettings(ImmutableFlowInstance flowInstance) {
		
		try {

			return treservaIntegrationProviderModule.getIntegrationSettings(flowInstance);

		} catch (SQLException e) {

			log.error("Error getting integration settings for flow instance " + flowInstance, e);
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetWeightByExistingTreservaCaseEvaluator evaluator = new SetWeightByExistingTreservaCaseEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return CitizenIdentifierQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetWeightByExistingTreservaCaseEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	protected Class<SetWeightByExistingTreservaCaseEvaluator> getEvaluatorClass() {

		return SetWeightByExistingTreservaCaseEvaluator.class;
	}

	protected AnnotatedDAO<SetWeightByExistingTreservaCaseEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetWeightByExistingTreservaCaseEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws SQLException {

		SetWeightByExistingTreservaCaseEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

	@Override
	public SetWeightByExistingTreservaCaseEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetWeightByExistingTreservaCaseEvaluator evaluator = getEvaluatorClass().newInstance();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	private boolean hasCaseData(QueryInstance queryInstance, String queryInstanceCitizenIdentifier, User poster, TreservaMinaIFOArendeClient client, SetWeightByExistingTreservaCaseEvaluator evaluator, MutableAttributeHandler attributeHandler) throws IntegrationRequestException {

		String posterCitizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(poster);

		if (!StringUtils.isEmpty(posterCitizenIdentifier)) {

			CaseData posterCaseData = getCaseData(queryInstance, client, posterCitizenIdentifier, queryInstanceCitizenIdentifier);

			if (posterCaseData != null) {

				log.info("Found case data for poster " + poster + " while evaluating query instance " + queryInstance);

				if (evaluator.isSetAsAttribute()) {

					evaluator.setAttribute(attributeHandler, posterCaseData);
				}

				return true;
			}

			if (!StringUtils.isEmpty(queryInstanceCitizenIdentifier)) {

				CaseData queryInstanceCaseData = getCaseData(queryInstance, client, queryInstanceCitizenIdentifier, posterCitizenIdentifier);

				if (queryInstanceCaseData != null) {

					log.info("Found case data for citizen identifier in query instance " + queryInstance + " where poster " + poster + " is co-applicant");

					if (evaluator.isSetAsAttribute()) {

						evaluator.setAttribute(attributeHandler, queryInstanceCaseData);
					}

					return true;
				}
			}
		}

		log.info("Found no case data for poster " + poster + " or query instance " + queryInstance);

		return false;
	}

	private CaseData getCaseData(QueryInstance queryInstance, TreservaMinaIFOArendeClient client, String citizenIdentifier, String coApplicantCitizenIdentifier) throws IntegrationRequestException {

		CaseData caseData = null;

		try {

			caseData = client.getLatestValidCaseData(citizenIdentifier, coApplicantCitizenIdentifier);

		} catch (InvalidKeyException | NoSuchAlgorithmException | IOException e) {

			throw new IntegrationRequestException("Error getting arende data by personnr for query instance " + queryInstance, e);
		}

		if (caseData == null) {

			log.warn("Found no valid case data in response for query instance " + queryInstance);
			return null;
		}
		
		return caseData;
	}

	private TreservaMinaIFOArendeClient getClient(IntegrationSettings settings) {

		return treservaIntegrationProviderModule.getTreservaMinaIFOArendeClient(settings);
		//		return new TreservaDecisionDummyClient();
	}

}
