package com.nordicpeak.flowengine.integration.treserva.beans;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.List;

import flexjson.JSON;

public class CaseData {

	private static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
	private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd");

	@JSON(name = "arendeNr")
	private String number;

	private String email;

	@JSON(name = "arendeFrom")
	private String fromDate;

	private LocalDateTime parsedFromDate;

	private String formattedFromDate;

	@JSON(name = "name")
	private String manager;

	@JSON(name = "arendeType")
	private String type;

	@JSON(name = "objListArendePersons")
	private List<CaseDataPerson> persons;

	@Override
	public String toString() {

		return "(number=" + number + ")";
	}

	public String getNumber() {

		return number;
	}

	public void setNumber(String number) {

		this.number = number;
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getFromDate() {

		return fromDate;
	}

	public void setFromDate(String fromDate) {

		this.fromDate = fromDate;

		parsedFromDate = null;
		formattedFromDate = null;
	}

	public LocalDateTime getParsedFromDate() {

		if (fromDate != null && parsedFromDate == null) {

			try {

				parsedFromDate = LocalDateTime.parse(fromDate, DATE_TIME_FORMATTER);

			} catch (DateTimeParseException e) {

				return null;
			}
		}

		return parsedFromDate;
	}

	public String getFormattedFromDate() {

		if (getParsedFromDate() != null && formattedFromDate == null) {

			formattedFromDate = getParsedFromDate().format(DATE_FORMATTER);
		}

		return formattedFromDate;
	}

	public String getManager() {

		return manager;
	}

	public void setManager(String manager) {

		this.manager = manager;
	}

	public String getType() {

		return type;
	}

	public void setType(String type) {

		this.type = type;
	}

	public List<CaseDataPerson> getPersons() {

		return persons;
	}

	public void setPersons(List<CaseDataPerson> persons) {

		this.persons = persons;
	}

	public boolean isCoApplicant(String citizenIdentifier) {

		if (persons != null) {

			for (CaseDataPerson caseDataPerson : persons) {

				if (caseDataPerson.getCitizenIdentifier() != null && caseDataPerson.getCitizenIdentifier().replaceAll("-| ", "").equals(citizenIdentifier)) {

					return true;
				}
			}
		}

		return false;
	}

}
