package com.nordicpeak.flowengine.evaluators.setbookingqueryslotparam;

import java.sql.SQLException;
import java.util.Collections;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.queries.BookingQuery;
import com.nordicpeak.flowengine.booking.queries.BookingQueryInstance;
import com.nordicpeak.flowengine.enums.ModificationAction;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.urlparameterfilter.URLParameterFilterModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

public class SetBookingQuerySlotParamEvaluationProviderModule extends BaseEvaluationProviderModule<SetBookingQuerySlotParamEvaluator> {
	
	protected SetBookingQuerySlotParamEvaluatorCRUD evaluatorCRUD;
	
	protected AnnotatedDAO<SetBookingQuerySlotParamEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<SetBookingQuerySlotParamEvaluator, Integer> evaluatorIDParamFactory;
	
	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetBookingQuerySlotParamEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetBookingQuerySlotParamEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new SetBookingQuerySlotParamEvaluatorCRUD(SetBookingQuerySlotParamEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetBookingQuerySlotParamEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (evaluator.isInitialized() || evaluator.getParamName() == null) {
			
			return null;
		}
		
		if (!(queryInstance instanceof BookingQueryInstance)) {
			
			log.warn("Query instance " + queryInstance + " is not of the type " + BookingQueryInstance.class.getName());
			
			return null;
		}
		
		BookingQueryInstance bookingQueryInstance = ((BookingQueryInstance) queryInstance);
		
		BookingSlot oldValue = bookingQueryInstance.getBookingSlot();
		
		String paramValue = getParamValue(evaluator, callback);
		
		if (paramValue != null) {
			
			evaluator.setInitialized(true);
			
			Integer bookingSlotID = NumberUtils.toInt(paramValue);
			
			if (bookingSlotID != null && bookingQueryInstance.getQuery().getCalendar().getBookingSlots() != null) {
				
				for (BookingSlot bookingSlot : bookingQueryInstance.getQuery().getCalendar().getBookingSlots()) {
					
					if (bookingSlot.getBookingSlotID().equals(bookingSlotID)) {
						
						if (oldValue == null || !oldValue.getBookingSlotID().equals(bookingSlotID)) {
							
							bookingQueryInstance.setBookingSlot(bookingSlot);
							
							if (queryInstance.getQueryInstanceDescriptor().getQueryState() != QueryState.HIDDEN) {
								
								return new EvaluationResponse(Collections.singletonList(new QueryModification(queryInstance, ModificationAction.RELOAD, attributeHandler)));
							}
						}
						
						break;
					}
				}
			}
		}
		
		return null;
	}
	
	private String getParamValue(SetBookingQuerySlotParamEvaluator evaluator, EvaluationCallback callback) {
		
		HttpSession session = callback.getRequest().getSession(false);
		
		if (session != null) {
			
			Object object = session.getAttribute(URLParameterFilterModule.PARAM_MAP_SESSION_ATTRIBUTE);
			
			if (object != null && object instanceof Map) {
				
				@SuppressWarnings("unchecked")
				Map<String, String> paramMap = (Map<String, String>) object;
				
				String value = paramMap.get(evaluator.getParamName());
				
				if (value != null) {
					paramMap.remove(evaluator.getParamName());
				}
				
				return value;
			}
		}
		
		return null;
	}
	
	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		SetBookingQuerySlotParamEvaluator evaluator = new SetBookingQuerySlotParamEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return BookingQuery.class.isAssignableFrom(queryClass);
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		SetBookingQuerySlotParamEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetBookingQuerySlotParamEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	protected Class<SetBookingQuerySlotParamEvaluator> getEvaluatorClass() {
		
		return SetBookingQuerySlotParamEvaluator.class;
	}
	
	protected AnnotatedDAO<SetBookingQuerySlotParamEvaluator> getEvaluatorDAO() {
		
		return evaluatorDAO;
	}
	
	protected QueryParameterFactory<SetBookingQuerySlotParamEvaluator, Integer> getEvaluatorIDParameterFactory() {
		
		return evaluatorIDParamFactory;
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		SetBookingQuerySlotParamEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetBookingQuerySlotParamEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<SetBookingQuerySlotParamEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	public QueryHandler getQueryHandler() {
		
		return this.queryHandler;
	}
	
	@Override
	public SetBookingQuerySlotParamEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		SetBookingQuerySlotParamEvaluator evaluator = getEvaluatorClass().newInstance();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
}
