package com.nordicpeak.flowengine.evaluators.setdropdownalternativeparameter;

import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.enums.ModificationAction;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.urlparameterfilter.URLParameterFilterModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.alternativelistdropdownquery.AlternativeListDropDownAlternative;
import com.nordicpeak.flowengine.queries.alternativelistdropdownquery.AlternativeListDropDownQuery;
import com.nordicpeak.flowengine.queries.alternativelistdropdownquery.AlternativeListDropDownQueryInstance;
import com.nordicpeak.flowengine.queries.dropdownquery.DropDownAlternative;
import com.nordicpeak.flowengine.queries.dropdownquery.DropDownQuery;
import com.nordicpeak.flowengine.queries.dropdownquery.DropDownQueryInstance;

public class SetDropDownAlternativeParameterEvaluationProviderModule extends BaseEvaluationProviderModule<SetDropDownAlternativeParameterEvaluator> {

	protected SetDropDownAlternativeParameterEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetDropDownAlternativeParameterEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetDropDownAlternativeParameterEvaluator, Integer> evaluatorIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetDropDownAlternativeParameterEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetDropDownAlternativeParameterEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetDropDownAlternativeParameterEvaluatorCRUD(SetDropDownAlternativeParameterEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetDropDownAlternativeParameterEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (evaluator.isInitialized() || evaluator.getParamName() == null || callback.getRequestMetadata().isManager()) {

			return null;
		}

		evaluator.setInitialized(true);
		boolean modified = false;

		if (queryInstance instanceof DropDownQueryInstance) {

			DropDownQueryInstance dropDownQueryInstance = (DropDownQueryInstance) queryInstance;

			modified = updateAlternativeFromParameter(dropDownQueryInstance, evaluator, callback);

		} else if (queryInstance instanceof AlternativeListDropDownQueryInstance) {

			AlternativeListDropDownQueryInstance alternativeListDropDownQueryInstance = (AlternativeListDropDownQueryInstance) queryInstance;

			modified = updateAlternativeFromParameter(alternativeListDropDownQueryInstance, evaluator, callback);

		} else {

			log.warn("Query instance " + queryInstance + " is not of a supported type");
			return null;
		}

		if (modified && queryInstance.getQueryInstanceDescriptor().getQueryState() != QueryState.HIDDEN) {

			return new EvaluationResponse(Collections.singletonList(new QueryModification(queryInstance, ModificationAction.RELOAD, attributeHandler)));
		}

		return null;
	}

	private boolean updateAlternativeFromParameter(DropDownQueryInstance queryInstance, SetDropDownAlternativeParameterEvaluator evaluator, EvaluationCallback callback) {

		DropDownAlternative oldAlternative = queryInstance.getAlternative();

		DropDownAlternative paramAlternative = (DropDownAlternative) getParamAlternative(queryInstance.getQuery().getAlternatives(), evaluator, callback);

		if (paramAlternative != null && !paramAlternative.equals(oldAlternative)) {

			queryInstance.setAlternative(paramAlternative);
			return true;
		}

		return false;
	}

	private boolean updateAlternativeFromParameter(AlternativeListDropDownQueryInstance queryInstance, SetDropDownAlternativeParameterEvaluator evaluator, EvaluationCallback callback) {

		AlternativeListDropDownAlternative oldAlternative = queryInstance.getAlternative();

		ImmutableAlternative paramAlternative = getParamAlternative(queryInstance.getQuery().getAlternatives(), evaluator, callback);

		if (paramAlternative != null && !paramAlternative.equals(oldAlternative)) {

			queryInstance.setAlternative(paramAlternative);
			return true;
		}

		return false;
	}

	private ImmutableAlternative getParamAlternative(List<? extends ImmutableAlternative> alternatives, SetDropDownAlternativeParameterEvaluator evaluator, EvaluationCallback callback) {

		String paramValue = getParamValue(evaluator, callback);

		if (alternatives == null || paramValue == null) {

			return null;
		}

		for (ImmutableAlternative alternative : alternatives) {

			String oldValue = evaluator.isSetAlternativeBasedOnID() ? StringUtils.toString(alternative.getAlternativeID()) : alternative.getName();

			if (paramValue.equalsIgnoreCase(oldValue)) {

				return alternative;
			}
		}

		return null;
	}

	private String getParamValue(SetDropDownAlternativeParameterEvaluator evaluator, EvaluationCallback callback) {

		HttpSession session = callback.getRequest().getSession(false);

		if (session != null) {

			Object object = session.getAttribute(URLParameterFilterModule.PARAM_MAP_SESSION_ATTRIBUTE);

			if (object instanceof Map) {

				@SuppressWarnings("unchecked")
				Map<String, String> paramMap = (Map<String, String>) object;

				return paramMap.get(evaluator.getParamName());
			}
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetDropDownAlternativeParameterEvaluator evaluator = new SetDropDownAlternativeParameterEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return AlternativeListDropDownQuery.class.isAssignableFrom(queryClass) || DropDownQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetDropDownAlternativeParameterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetDropDownAlternativeParameterEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	protected Class<SetDropDownAlternativeParameterEvaluator> getEvaluatorClass() {

		return SetDropDownAlternativeParameterEvaluator.class;
	}

	protected AnnotatedDAO<SetDropDownAlternativeParameterEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetDropDownAlternativeParameterEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SetDropDownAlternativeParameterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetDropDownAlternativeParameterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<SetDropDownAlternativeParameterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	@Override
	public SetDropDownAlternativeParameterEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetDropDownAlternativeParameterEvaluator evaluator = getEvaluatorClass().newInstance();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}
}
