package com.nordicpeak.flowengine.queries.unsecuremultisignquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MultiSigningQueryProvider;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class UnsecureMultiSignQueryProviderModule extends BaseQueryProviderModule<UnsecureMultiSignQueryInstance> implements MultiSigningQueryProvider {
	
	private static final EmailPopulator EMAIL_POPULATOR = new EmailPopulator();
	
	@XSLVariable(prefix = "java.")
	private String exportFirstName;
	
	@XSLVariable(prefix = "java.")
	private String exportLastName;
	
	@XSLVariable(prefix = "java.")
	private String exportEmail;
	
	@XSLVariable(prefix = "java.")
	private String exportMobilePhone;
	
	private AnnotatedDAO<UnsecureMultiSignQuery> queryDAO;
	private AnnotatedDAO<UnsecureMultiSignQueryInstance> queryInstanceDAO;
	
	private UnsecureMultiSignQueryCRUD queryCRUD;
	
	private QueryParameterFactory<UnsecureMultiSignQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<UnsecureMultiSignQueryInstance, Integer> queryInstanceIDParamFactory;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, UnsecureMultiSignQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);
		
		queryDAO = daoFactory.getDAO(UnsecureMultiSignQuery.class);
		queryInstanceDAO = daoFactory.getDAO(UnsecureMultiSignQueryInstance.class);
		
		queryCRUD = new UnsecureMultiSignQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<UnsecureMultiSignQuery>(UnsecureMultiSignQuery.class), "UnsecureMultiSignQuery", "query", null, this);
		
		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}
	
	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		UnsecureMultiSignQuery query = new UnsecureMultiSignQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, null);
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws Throwable {
		
		UnsecureMultiSignQuery query = this.getQuery(descriptor.getQueryID());
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		UnsecureMultiSignQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws Throwable {
		
		UnsecureMultiSignQueryInstance queryInstance = null;
		
		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance = new UnsecureMultiSignQueryInstance();
			
		} else {
			
			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());
			
			if (queryInstance == null) {
				
				return null;
			}
		}
		
		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));
		
		if (queryInstance.getQuery() == null) {
			
			return null;
		}
		
		if (req != null) {
			
			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);
			
			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());
		
		queryInstance.set(descriptor);
		
		//If this is a new query instance copy the default values
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance.copyQueryValues();
		}
		
		return queryInstance;
	}
	
	private UnsecureMultiSignQuery getQuery(Integer queryID) throws SQLException {
		
		HighLevelQuery<UnsecureMultiSignQuery> query = new HighLevelQuery<UnsecureMultiSignQuery>();
		
		query.addParameter(queryIDParamFactory.getParameter(queryID));
		
		return queryDAO.get(query);
	}
	
	private UnsecureMultiSignQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {
		
		HighLevelQuery<UnsecureMultiSignQuery> query = new HighLevelQuery<UnsecureMultiSignQuery>();
		
		query.addParameter(queryIDParamFactory.getParameter(queryID));
		
		return queryDAO.get(query, transactionHandler);
	}
	
	private UnsecureMultiSignQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {
		
		HighLevelQuery<UnsecureMultiSignQueryInstance> query = new HighLevelQuery<UnsecureMultiSignQueryInstance>(UnsecureMultiSignQueryInstance.QUERY_RELATION);
		
		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));
		
		return queryInstanceDAO.get(query);
	}
	
	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		UnsecureMultiSignQuery query = getQuery(descriptor.getQueryID());
		
		if (query == null) {
			
			return false;
		}
		
		this.queryDAO.delete(query, transactionHandler);
		
		return true;
	}
	
	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		UnsecureMultiSignQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());
		
		if (queryInstance == null) {
			
			return false;
		}
		
		this.queryInstanceDAO.delete(queryInstance, transactionHandler);
		
		return true;
	}
	
	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {
		
		UnsecureMultiSignQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);
		
		query.setQueryID(copyQueryDescriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, null);
	}
	
	@Override
	public void save(UnsecureMultiSignQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {
		
		//Check if the query instance has an ID set and if the ID of the descriptor has changed
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {
			
			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());
			
			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);
			
		} else {
			
			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}
	
	@Override
	protected void appendQueryInstance(UnsecureMultiSignQueryInstance queryInstance, Document doc, Element targetElement, AttributeHandler attributeHandler) {
		
		super.appendQueryInstance(queryInstance, doc, targetElement, attributeHandler);
		
		if (queryInstance.getQuery().isLockOnOwnershipTransfer() && attributeHandler.getPrimitiveBoolean("OwnershipTransfered")) {
			
			XMLUtils.appendNewElement(doc, targetElement, "Locked", "true");
		}
	}
	
	@Override
	public void populate(UnsecureMultiSignQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {
		
		UnsecureMultiSignQuery query = queryInstance.getQuery();
		Integer queryID = query.getQueryID();
		
		if (query.isLockOnOwnershipTransfer() && attributeHandler.getPrimitiveBoolean("OwnershipTransfered")) {
			return;
		}
		
		//If the fields are disabled use the previous values if there are any
		if (query.isDisabled()) {
			
			if (!queryInstance.isEmpty() || queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED) {
				
				List<ValidationError> errors = new ArrayList<ValidationError>();
				
				if (queryInstance.getFirstname() == null) {
					errors.add(new ValidationError("q" + queryID + "_firstname", ValidationErrorType.RequiredField));
				}
				
				if (queryInstance.getLastname() == null) {
					errors.add(new ValidationError("q" + queryID + "_lastname", ValidationErrorType.RequiredField));
				}
				
				if (queryInstance.getEmail() != null && !EMAIL_POPULATOR.validateFormat(queryInstance.getEmail())) {
					errors.add(new ValidationError("q" + queryID + "_email", ValidationErrorType.InvalidFormat));
				}
				
				if (queryInstance.getMobilePhone() != null && !StringSwedishPhoneNumberPopulator.getPopulator().validateFormat(queryInstance.getMobilePhone())) {
					errors.add(new ValidationError("q" + queryID + "_mobilePhone", ValidationErrorType.InvalidFormat));
				}
				
				if (!allowPartialPopulation && errors.isEmpty() && queryInstance.getEmail() == null && queryInstance.getMobilePhone() == null) {
					
					errors.add(new ValidationError("NoContactChannelSpecified"));
				}
				
				if (!errors.isEmpty()) {
					
					throw new ValidationException(errors);
				}
				
				queryInstance.getQueryInstanceDescriptor().setPopulated(true);
				
				if (query.isSetAsAttribute()) {
					
					queryInstance.setAttribute(attributeHandler);
				}
				
			} else {
				
				queryInstance.getQueryInstanceDescriptor().setPopulated(false);
				queryInstance.reset(attributeHandler);
			}
			
		} else {
			
			boolean firstnameEmpty = StringUtils.isEmpty(req.getParameter("q" + queryID + "_firstname"));
			boolean lastnameEmpty = StringUtils.isEmpty(req.getParameter("q" + queryID + "_lastname"));
			boolean emailEmpty = StringUtils.isEmpty(req.getParameter("q" + queryID + "_email"));
			boolean phoneEmpty = StringUtils.isEmpty(req.getParameter("q" + queryID + "_mobilePhone"));
			
			if (!allowPartialPopulation && queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && firstnameEmpty && lastnameEmpty && emailEmpty && phoneEmpty) {
				throw new ValidationException(new ValidationError("Required"));
			}
			
			if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED || !firstnameEmpty || !lastnameEmpty || !emailEmpty || !phoneEmpty) {
				
				List<ValidationError> errors = new ArrayList<ValidationError>();
				
				String email = ValidationUtils.validateParameter("q" + queryID + "_email", req, false, 1, 255, EMAIL_POPULATOR, errors);
				String mobilePhone = ValidationUtils.validateParameter("q" + queryID + "_mobilePhone", req, false, StringSwedishPhoneNumberPopulator.getPopulator(), errors);
				
				if (!allowPartialPopulation && errors.isEmpty() && email == null && mobilePhone == null) {
					
					errors.add(new ValidationError("NoContactChannelSpecified"));
					
				} else if (queryInstance.getQuery().isRequireContactInfoVerification() && !requestMetadata.isManager()) {

					String emailID2 = "q" + queryID + "_email2";
					String phoneID2 = "q" + queryID + "_mobilePhone2";

					String email2 = ValidationUtils.validateParameter(emailID2, req, email != null, EmailPopulator.getPopulator(), errors);
					String phone2 = ValidationUtils.validateParameter(phoneID2, req, mobilePhone != null, StringSwedishPhoneNumberPopulator.getPopulator(), errors);

					if (((email == null && email2 != null) || (email != null && !email.equals(email2))) && !ValidationUtils.containsValidationErrorForField(emailID2, errors)) {

						errors.add(new ValidationError("EmailVerificationMismatch", "", emailID2));
					}

					if (((mobilePhone == null && phone2 != null) || (mobilePhone != null && !mobilePhone.equals(phone2))) && !ValidationUtils.containsValidationErrorForField(phoneID2, errors)) {

						errors.add(new ValidationError("PhoneVerificationMismatch", "", phoneID2));
					}
				}
				
				String firstname = ValidationUtils.validateParameter("q" + queryID + "_firstname", req, !allowPartialPopulation, 1, 255, errors);
				String lastname = ValidationUtils.validateParameter("q" + queryID + "_lastname", req, !allowPartialPopulation, 1, 255, errors);
				
				if (!errors.isEmpty()) {
					
					throw new ValidationException(errors);
				}
				
				queryInstance.setFirstname(firstname);
				queryInstance.setLastname(lastname);
				queryInstance.setEmail(email);
				queryInstance.setMobilePhone(mobilePhone);
				
				queryInstance.getQueryInstanceDescriptor().setPopulated(true);
				
				if (query.isSetAsAttribute()) {
					
					queryInstance.setAttribute(attributeHandler);
				}
				
			} else {
				
				queryInstance.getQueryInstanceDescriptor().setPopulated(false);
				queryInstance.reset(attributeHandler);
			}
		}
	}
	
	@Override
	public String getTitlePrefix() {
		
		return this.moduleDescriptor.getName();
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.queryCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	protected Class<UnsecureMultiSignQueryInstance> getQueryInstanceClass() {
		
		return UnsecureMultiSignQueryInstance.class;
	}
	
	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {
		
		UnsecureMultiSignQuery query = new UnsecureMultiSignQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);
		
		this.queryDAO.add(query, transactionHandler, null);
		
		return query;
	}
	
	public String getExportFirstName() {
		
		return exportFirstName;
	}
	
	public String getExportLastName() {
		
		return exportLastName;
	}
	
	public String getExportEmail() {
		
		return exportEmail;
	}
	
	public String getExportMobilePhone() {
		
		return exportMobilePhone;
	}
	
	@Override
	public List<Integer> getQueryInstanceIDs(String citizenIdentifier) throws SQLException {
		
		return null;
	}
}
