package com.nordicpeak.flowengine.evaluators.useraddressquerystateevaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;

public class UserAddressQueryStateEvaluatorCRUD extends BaseEvaluatorCRUD<UserAddressQueryStateEvaluator, UserAddressQueryStateEvaluationProviderModule> {

	private static final BeanRequestPopulator<UserAddressQueryStateEvaluator> POPULATOR = new AnnotatedRequestPopulator<UserAddressQueryStateEvaluator>(UserAddressQueryStateEvaluator.class);

	public UserAddressQueryStateEvaluatorCRUD(Class<UserAddressQueryStateEvaluator> beanClass, AnnotatedDAOWrapper<UserAddressQueryStateEvaluator, Integer> evaluatorDAO, UserAddressQueryStateEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "UserAddressQueryStateEvaluator", "user address query state evaluator", null, callback);
	}
	
	@Override
	protected void appendUpdateFormData(UserAddressQueryStateEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
	
		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);
		
		XMLUtils.appendNewElement(doc, updateTypeElement, "SelectByStreetNumber", callback.isSelectByStreetNumber());
	}

	@Override
	protected UserAddressQueryStateEvaluator populateFromUpdateRequest(UserAddressQueryStateEvaluator evaluator, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		evaluator = super.populateFromUpdateRequest(evaluator, req, user, uriParser);

		String[] addressIDs = req.getParameterValues("addressID");

		List<Address> addresses = null;

		if (addressIDs != null && addressIDs.length > 0) {

			addresses = new ArrayList<Address>(addressIDs.length);

			for (int i = 0; i < addressIDs.length; i++) {

				String addressID = addressIDs[i];

				String street = req.getParameter("street-" + addressID);
				String postalAddress = req.getParameter("postalAddress-" + addressID);

				if (!StringUtils.isEmpty(street) && !StringUtils.isEmpty(postalAddress)) {

					Address address = new Address();

					address.setStreet(street);
					address.setPostalAddress(postalAddress);

					if (!addresses.contains(address)) {

						addresses.add(address);
					}
				}
			}
		}

		evaluator.setAddresses(addresses);

		return evaluator;
	}

}
