<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/plugins/jquery.qloader.js
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/jquery.blockui.js
		/js/flowengine.helpdialog.js
		/js/flowengine.js?v=1
		/js/flowengine.tablesorter.js
		/js/userflowinstancemodule.js?v=1
		/js/js.cookie-2.0.4.min.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/flowengine.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<div id="FlowBrowser" class="contentitem userflowinstances">
			
			<xsl:apply-templates select="InvoiceList" />
			
		</div>
		
	</xsl:template>
		
	<xsl:template match="InvoiceList">
			
		<section class="my-errands">
			
			<div class="errands-wrapper">
				<div class="heading-wrapper">
					<h2><xsl:value-of select="title" /></h2>
					<h3 class="clearboth">
						<xsl:value-of select="description" disable-output-escaping="yes"/>
					</h3>
					<xsl:call-template name="createHelpDialog">
						<xsl:with-param name="id" select="'invoices'" />
						<xsl:with-param name="text" select="helpText" />
					</xsl:call-template>
					
					<xsl:apply-templates select="Invoices/Invoice" mode="popup"/>
				</div>
				<table class="oep-table errand-table">
					<thead class="sortable">
						<tr>
							<th class="icon no-sort"></th>
							<th class="errando"><span data-icon-after="_"><xsl:value-of select="$i18n.InvoiceID" /></span></th>
							<th class="description"><span><xsl:value-of select="$i18n.Description"/></span></th>							
							<th class="amount"><span data-icon-after="_"><xsl:value-of select="$i18n.Amount" /></span></th>
							<th class="status"><span data-icon-after="_"><xsl:value-of select="$i18n.Status" /></span></th>
							<th class="date"><span data-icon-after="_"><xsl:value-of select="$i18n.InvoiceDate" /></span></th>
							<th class="date"><span data-icon-after="_"><xsl:value-of select="$i18n.DueDate" /></span></th>
							<th class="link no-sort"></th>
						</tr>
					</thead>
					<tbody>
						<xsl:choose>
							<xsl:when test="Invoices/Invoice">
								<xsl:apply-templates select="Invoices/Invoice" />
							</xsl:when>
							<xsl:otherwise>
								<tr><td class="icon" /><td colspan="8"><xsl:value-of select="$i18n.NoInvoices" /></td></tr>
							</xsl:otherwise>
						</xsl:choose>
					</tbody>
				</table>
			</div>

		</section>
		
	</xsl:template>
		
	<xsl:template match="Invoice">
		
		<tr>
			<td class="icon"/>

			<td data-title="{$i18n.InvoiceID}" class="errando"><xsl:value-of select="invoiceID" /></td>
			<td data-title="{$i18n.Description}" class="description"><xsl:value-of select="description" /></td>
			<td data-title="{$i18n.Amount}" class="amount"><xsl:value-of select="amount" /></td>
			<td data-title="{$i18n.Status}" class="status">
			
				<xsl:choose>
					<xsl:when test="status">
						<xsl:value-of select="status" />
					</xsl:when>
					<xsl:when test="payed = 'true'">
						<xsl:value-of select="$i18n.Payed" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.Unpayed" />
					</xsl:otherwise>
				</xsl:choose>
			</td>

			<td data-title="{$i18n.InvoiceDate}" class="date"><xsl:value-of select="invoiceDate" /></td>
			<td data-title="{$i18n.DueDate}" class="date"><xsl:value-of select="dueDate" /></td>
			
			<td class="link">
				<xsl:choose>
					<xsl:when test="longDescription">
						<a class="btn btn-green vertical-align-middle" href="#" data-help-box="helpdialog_{concat('invoice',position())}"><xsl:value-of select="$i18n.ShowInvoice" /></a>
					</xsl:when>
					<xsl:otherwise>
						<a class="btn btn-green vertical-align-middle" href="{/Document/BaseURL}/invoice/{invoiceID}"><xsl:value-of select="$i18n.ShowInvoice" /></a>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template match="Invoice" mode="popup">
	
		<xsl:if test="longDescription">
			<xsl:call-template name="createInformationDialog">
				<xsl:with-param name="id" select="concat('invoice',position())" />
				<xsl:with-param name="text" select="longDescription" />
			</xsl:call-template>		
		</xsl:if>
	
	</xsl:template>	
	
	<xsl:template name="createHelpDialog">
		
		<xsl:param name="id" />
		<xsl:param name="text" />
		<xsl:param name="class" select="''" />
		
		<div class="help {$class}">
			<a class="open-help" href="#" data-icon-after="?" data-help-box="helpdialog_{$id}"><span><xsl:value-of select="$i18n.Help" /></span></a>
			<div class="help-box" data-help-box="helpdialog_{$id}">
				<div>
		  			<div> 
		  				<a class="close" href="#" data-icon-after="x"></a>
		  				<xsl:value-of select="$text" disable-output-escaping="yes"/>
		  			</div> 
				</div>
			</div>
		</div>
		
		<div class="help-backdrop" data-help-box="helpdialog_{$id}" />
		
	</xsl:template>
	
	<xsl:template name="createInformationDialog">
		
		<xsl:param name="id" />
		<xsl:param name="text" />
		<xsl:param name="class" select="''" />
		
		
		<div class="{$class}">
			<div class="help-box" data-help-box="helpdialog_{$id}">
				<div>
		  			<div> 
		  				<a class="close" href="#" data-icon-after="x"></a>
		  				<xsl:value-of select="$text" disable-output-escaping="yes"/>
		  			</div> 
				</div>
			</div>
		</div>
		
		<div class="help-backdrop" data-help-box="helpdialog_{$id}" />
		
	</xsl:template>			
	
</xsl:stylesheet>