<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/UserGroupList.js
		/js/userlistqueryadmin.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/UserGroupList.css?v=1
		/css/userlistquery.css
	</xsl:variable>

	<xsl:template match="Document">

		<div id="UserListQueryProvider" class="contentitem">

			<xsl:apply-templates select="validationError" />
			<xsl:apply-templates select="UpdateUserListQuery" />

		</div>

	</xsl:template>

	<xsl:template match="UpdateUserListQuery">

		<h1>
			<xsl:value-of select="$i18n.UpdateQuery" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="UserListQuery/QueryDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form id="updateUserListQueryForm" name="queryAdminForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="UserListQuery" />
			</xsl:call-template>
			
			<label class="floatleft"><xsl:value-of select="$i18n.UserSourceType.Title"/></label>
				
			<div class="clearboth floatleft marginbottom"><xsl:value-of select="$i18n.UserSourceType.Description"/></div>
			
				<div class="floatleft full marginbottom">
							
				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'useGroups'"/>
						<xsl:with-param name="name" select="'userSourceType'"/>
						<xsl:with-param name="value" select="'GROUPS'"/>
						<xsl:with-param name="element" select="UserListQuery" />
					</xsl:call-template>
					
					<label for="useGroups">
						<xsl:value-of select="$i18n.UserSourceType.Groups" />
					</label>
				</div>
			</div>		
			
			<div class="floatleft full marginbottom">
							
				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'useListUserProvider'"/>
						<xsl:with-param name="name" select="'userSourceType'"/>
						<xsl:with-param name="value" select="'LIST_USER_PROVIDER'"/>
						<xsl:with-param name="element" select="UserListQuery" />
					</xsl:call-template>
					
					<label for="useListUserProvider">
						<xsl:value-of select="$i18n.UserSourceType.ListUserProvider" />
					</label>
				</div>
			</div>			
			
			<div id="groupContainer" class="floatleft full bigmarginbottom">

				<label for="group-search">
					<xsl:value-of select="$i18n.Groups" />
				</label>
				
				<xsl:call-template name="GroupList">
					<xsl:with-param name="connectorURL">
						<xsl:value-of select="/Document/requestinfo/currentURI"/>
						<xsl:text>/</xsl:text>
						<xsl:value-of select="/Document/module/alias"/>
						<xsl:text>/groups</xsl:text>
					</xsl:with-param>
					<xsl:with-param name="name" select="'group'" />
					<xsl:with-param name="groups" select="Groups" />
				</xsl:call-template>
			
			</div>
			
			<div class="floatleft full bigmarginbottom" id="providerContainer">

				<label for="providerIdentifier">
					<xsl:value-of select="$i18n.ProviderIdentifier" />
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'providerIdentifier'" />
						<xsl:with-param name="name" select="'providerIdentifier'" />
						<xsl:with-param name="width" select="'300px'" />
						<xsl:with-param name="labelElementName" select="'Name'" />
						<xsl:with-param name="valueElementName" select="'ID'" />
						<xsl:with-param name="element" select="Providers/Provider" />
						<xsl:with-param name="selectedValue" select="UserListQuery/providerIdentifier" />
					</xsl:call-template>
				</div>

			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="minSelectedUsers" class="floatleft"><xsl:value-of select="$i18n.MinSelected" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'minSelectedUsers'"/>
						<xsl:with-param name="name" select="'minSelectedUsers'"/>
						<xsl:with-param name="title" select="$i18n.MinSelected"/>
						<xsl:with-param name="size" select="'30'"/>
						<xsl:with-param name="element" select="UserListQuery" />
					</xsl:call-template>
			    </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="maxSelectedUsers" class="floatleft"><xsl:value-of select="$i18n.MaxSelected" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'maxSelectedUsers'"/>
						<xsl:with-param name="name" select="'maxSelectedUsers'"/>
						<xsl:with-param name="title" select="$i18n.MaxSelected"/>
						<xsl:with-param name="size" select="'30'"/>
						<xsl:with-param name="element" select="UserListQuery" />
					</xsl:call-template>
			    </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showCitizenIdentifier'" />
					<xsl:with-param name="name" select="'showCitizenIdentifier'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="UserListQuery" />
				</xsl:call-template>
				
				<label for="showCitizenIdentifier">
					<xsl:value-of select="$i18n.ShowCitizenIdentifier" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showUsername'" />
					<xsl:with-param name="name" select="'showUsername'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="UserListQuery" />
				</xsl:call-template>
				
				<label for="showUsername">
					<xsl:value-of select="$i18n.ShowUsername" />
				</label>
			</div>			
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showContactInformation'" />
					<xsl:with-param name="name" select="'showContactInformation'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="UserListQuery" />
				</xsl:call-template>
				
				<label for="showContactInformation">
					<xsl:value-of select="$i18n.ShowContactInformation" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'excludeInactiveUsers'" />
						<xsl:with-param name="name" select="'excludeInactiveUsers'" />
						<xsl:with-param name="element" select="UserListQuery" /> 
						<xsl:with-param name="class" select="'vertical-align-middle'" />
					</xsl:call-template>
						
					<label for="listEnabledUsersOnly">
						<xsl:value-of select="$i18n.Query.excludeInactiveUsers" />
					</label>
			    </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'useForMultipartSigning'" />
						<xsl:with-param name="name" select="'useForMultipartSigning'" />
						<xsl:with-param name="element" select="UserListQuery" /> 
					</xsl:call-template>
						
					<label for="useForMultipartSigning">
						<xsl:value-of select="$i18n.UseForMultipartSigning" />
					</label>
			    </div>
			</div>
			
			<div id="multipart" class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'setMultipartsAsOwners'" />
						<xsl:with-param name="name" select="'setMultipartsAsOwners'" />
						<xsl:with-param name="element" select="UserListQuery" /> 
					</xsl:call-template>
						
					<label for="setMultipartsAsOwners">
						<xsl:value-of select="$i18n.SetMultipartsAsOwners" />
					</label>
			    </div>
			</div>

			<div class="floatleft full bigmarginbottom bigmargintop">
				<h2><xsl:value-of select="$i18n.AdvancedSettings" /></h2>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'setAsAttribute'" />
						<xsl:with-param name="name" select="'setAsAttribute'" />
						<xsl:with-param name="element" select="UserListQuery" /> 
						<xsl:with-param name="class" select="'vertical-align-middle'" />
					</xsl:call-template>
						
					<label for="setAsAttribute">
						<xsl:value-of select="$i18n.setAsAttribute" />
					</label>
			    </div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="attributeName" class="floatleft clearboth"><xsl:value-of select="$i18n.attributeName" /></label>
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'attributeName'"/>
						<xsl:with-param name="name" select="'attributeName'"/>
						<xsl:with-param name="title" select="$i18n.attributeName"/>
						<xsl:with-param name="element" select="UserListQuery" />
						<xsl:with-param name="maxlength" select="'255'"/>
					</xsl:call-template>
			    </div>
			</div>

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'UpdateFailedUserListQueryNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.UserListQueryNotFound" />
		</p>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'MinSelectedBiggerThanMaxSelected']">
		
		<p class="error">
			<xsl:value-of select="$i18n.MinSelectedBiggerThanMaxSelected" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'userSourceType'">
				<xsl:value-of select="$i18n.UserSourceType.Title" />
			</xsl:when>
			<xsl:when test="$fieldName = 'group'">
				<xsl:value-of select="$i18n.Groups" />
			</xsl:when>			
			<xsl:when test="$fieldName = 'providerIdentifier'">
				<xsl:value-of select="$i18n.ProviderIdentifier" />
			</xsl:when>
			<xsl:when test="$fieldName = 'minSelectedUsers'">
				<xsl:value-of select="$i18n.minSelectedUsers" />
			</xsl:when>
			<xsl:when test="$fieldName = 'maxSelectedUsers'">
				<xsl:value-of select="$i18n.maxSelectedUsers" />
			</xsl:when>
			<xsl:when test="$fieldName = 'attributeName'">
				<xsl:value-of select="$i18n.attributeName" />
			</xsl:when>			
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>