package com.nordicpeak.flowengine.queries.userlistquery;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.userlistquery.beans.UserListQueryInstance;
import com.nordicpeak.listuserprovider.ListUser;
import com.nordicpeak.listuserprovider.ListUserProvider;

public class UserListQueryRequestProcessor implements QueryRequestProcessor {

	protected final Logger log = Logger.getLogger(this.getClass());

	private final String encoding;
	private final ListUserProvider provider;
	private final UserHandler userHandler;
	private final UserListQueryInstance queryInstance;

	public UserListQueryRequestProcessor(ListUserProvider provider, UserListQueryInstance queryInstance, String encoding) {

		super();

		this.provider = provider;
		this.userHandler = null;
		this.queryInstance = queryInstance;
		this.encoding = encoding;
	}
	
	public UserListQueryRequestProcessor(UserHandler userHandler, UserListQueryInstance queryInstance, String encoding) {

		super();

		this.userHandler = userHandler;
		this.provider = null;
		this.queryInstance = queryInstance;
		this.encoding = encoding;
	}

	protected static void sendEmptyJSONResponse(HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(1);
		jsonObject.putField("hitCount", "0");
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	protected void sendJSONResponse(JsonArray jsonArray, HttpServletResponse res) throws IOException {

		JsonObject jsonObject = new JsonObject(2);
		jsonObject.putField("hitCount", Integer.toString(jsonArray.size()));
		jsonObject.putField("hits", jsonArray);
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	@Override
	public void processRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		String searchString = req.getParameter("q");

		if (searchString != null && !encoding.equalsIgnoreCase("UTF-8")) {
			searchString = URLDecoder.decode(searchString, "UTF-8");
		}

		if (StringUtils.isEmpty(searchString)) {

			sendEmptyJSONResponse(res);
			return;
		}
		
		if(provider != null) {
			
			List<ListUser> users = provider.searchUsers(queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getStep().getFlow().getFlowID(), searchString, queryInstance.getSearchParameters(), user);
		
			if (users == null) {

				sendEmptyJSONResponse(res);
				return;
			}

			if (queryInstance.getQuery().isExcludeInactiveUsers()) {
				
				users = users.stream().filter(Objects::nonNull).filter(ListUser::isEnabled).collect(Collectors.toList());
			}
			
			JsonArray jsonArray = new JsonArray();

			for (ListUser currentUser : users) {

				JsonObject instance = new JsonObject(4);
				instance.putField("ID", currentUser.getProviderUserID());
				instance.putField("Name", currentUser.getFirstname() + " " + currentUser.getLastname());
				instance.putField("Email", currentUser.getEmail());

				if (currentUser.getUsername() != null) {

					instance.putField("Username", currentUser.getUsername());
				}

				instance.putField("Enabled", currentUser.isEnabled() == null || currentUser.isEnabled());

				jsonArray.addNode(instance);
			}

			sendJSONResponse(jsonArray, res);
			
		}else {
			
			List<User> users = userHandler.searchUsers(searchString, false, true, 50, queryInstance.getQuery().getGroupIDs(), queryInstance.getQuery().isExcludeInactiveUsers());
			
			if (users == null) {

				sendEmptyJSONResponse(res);
				return;
			}

			JsonArray jsonArray = new JsonArray();

			for (User currentUser : users) {

				JsonObject instance = new JsonObject(4);
				instance.putField("ID", currentUser.getUserID());
				instance.putField("Name", currentUser.getFirstname() + " " + currentUser.getLastname());
				instance.putField("Email", currentUser.getEmail());

				if (currentUser.getUsername() != null) {

					instance.putField("Username", currentUser.getUsername());
				}

				instance.putField("Enabled", currentUser.isEnabled());

				jsonArray.addNode(instance);
			}

			sendJSONResponse(jsonArray, res);
		}
	}

	@Override
	public void close() throws Exception {

	}
}
