package com.nordicpeak.flowengine.evaluators.userquerymailevaluator;

import java.io.File;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.EvaluatorDescriptor;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.events.StatusChangedByManagerEvent;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryInstanceNotFoundInQueryProviderException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderErrorException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderNotFoundException;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.PDFProvider;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.UserValueQuery;
import com.nordicpeak.flowengine.interfaces.UserValueQueryInstance;
import com.nordicpeak.flowengine.interfaces.UserValuesQuery;
import com.nordicpeak.flowengine.interfaces.UserValuesQueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;

public class UserQueryMailEvaluationProviderModule extends BaseEvaluationProviderModule<UserQueryMailEvaluator> {
	
	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@InstanceManagerDependency
	protected PDFProvider pdfProvider;
	
	@InstanceManagerDependency
	protected FlowInstanceAdminModule flowInstanceAdminModule;
	
	@InstanceManagerDependency
	private StandardFlowNotificationHandler notificationHandler;
	
	protected AnnotatedDAO<UserQueryMailEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<UserQueryMailEvaluator, Integer> evaluatorIDParamFactory;
	
	protected UserQueryMailEvaluatorCRUD evaluatorCRUD;
	
	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {
		
		super.init(moduleDescriptor, sectionInterface, dataSource);
	}
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, UserQueryMailEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(UserQueryMailEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new UserQueryMailEvaluatorCRUD(UserQueryMailEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return UserValueQuery.class.isAssignableFrom(queryClass) || UserValuesQuery.class.isAssignableFrom(queryClass);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, UserQueryMailEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		return null;
	}
	
	@EventListener(channel = FlowInstanceManager.class, priority = 50)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {
		
		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();
		
		sendMail(flowInstanceManager, event.getEvent().getPoster(), event.getEvent().getEventID(), event.getSiteProfile());
	}
	
	@EventListener(channel = FlowInstance.class, priority = 50)
	public void processEvent(StatusChangedByManagerEvent event, EventSource eventSource) throws SQLException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException {
		
		Flow flow = flowAdminModule.getCachedFlow(event.getFlowInstance().getFlow().getFlowID());
		
		if (containsStatusFilteredEvaluators(flow)) {
			
			FlowInstance flowInstance = flowAdminModule.getFlowInstance(event.getFlowInstance().getFlowInstanceID());
			
			ImmutableFlowInstanceManager flowInstanceManager = getImmutableFlowInstanceManager(flowInstance, event.getSiteProfile());
			
			FlowInstanceEvent pdfEvent = FlowInstanceUtils.getLatestPDFEvent(flowInstance);
			
			Integer pdfEventID;
			
			if (pdfEvent != null) {
				
				pdfEventID = pdfEvent.getEventID();
				
			} else {
				
				pdfEventID = null;
			}
			
			sendMail(flowInstanceManager, event.getUser(), pdfEventID, event.getSiteProfile());
		}
	}
	
	protected void sendMail(FlowInstanceManager flowInstanceManager, User eventUser, Integer pdfEventID, SiteProfile siteProfile) throws SQLException {
		
		List<UserValueQueryInstance> userValueQueries = flowInstanceManager.getQueries(UserValueQueryInstance.class);
		List<UserValuesQueryInstance> userValuesQueries = flowInstanceManager.getQueries(UserValuesQueryInstance.class);
		
		int totalSize = CollectionUtils.getSize(userValueQueries) + CollectionUtils.getSize(userValuesQueries);
		
		if (totalSize == 0) {
			return;
		}
		
		HashSet<QueryInstance> queryInstances = new HashSet<QueryInstance>(totalSize);
		
		if (userValueQueries != null) {
			queryInstances.addAll(userValueQueries);
		}
		
		if (userValuesQueries != null) {
			queryInstances.addAll(userValuesQueries);
		}
		
		for (QueryInstance queryInstance : queryInstances) {
			
			if (queryInstance == null || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {
				continue;
			}
			
			for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {
				
				if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {
					
					if (notificationHandler == null) {
						
						log.error("Notification handler not set, unable to send emails regarding flow instance " + flowInstanceManager.getFlowInstance());
						return;
					}
					
					UserQueryMailEvaluator evaluator = getEvaluator(evaluatorDescriptor);
					
					if (evaluator.enableStatusFiltering() && (evaluator.getStatusIDs() == null || !evaluator.getStatusIDs().contains(flowInstanceManager.getFlowInstance().getStatus().getStatusID()))) {
						
						continue;
					}
					
					Collection<User> recipients = null;
					
					if (queryInstance instanceof UserValuesQueryInstance) {
						
						recipients = ((UserValuesQueryInstance) queryInstance).getUserValues(queryHandler);
						
					} else {
						
						User user = ((UserValueQueryInstance) queryInstance).getUserValue(queryHandler);
						
						if (user != null) {
							recipients = Collections.singletonList(user);
						}
					}
					
					if (recipients == null) {
						continue;
					}
					
					ImmutableFlowInstance flowInstance = flowInstanceManager.getFlowInstance();
					
					for (User recipient : recipients) {
						
						String emailAddress = recipient.getEmail();
						
						if (emailAddress != null && EmailUtils.isValidEmailAddress(emailAddress)) {
							
							SimpleEmail email = null;
							
							try {
								log.info("Sending email about submitted flow instance " + flowInstance + " to recipient " + recipient + " at " + emailAddress + " trigged by evaluator " + evaluatorDescriptor);
								
								Contact contact = notificationHandler.getPosterContact(flowInstance);
								
								File file = null;
								
								if (evaluator.attachPDF() && pdfEventID != null) {
									
									if (pdfProvider != null) {
										
										file = pdfProvider.getPDF(flowInstanceManager.getFlowInstanceID(), pdfEventID);
										
									} else {
										
										log.warn("No pdf provider found, pdf cant be attached to generated email triggered by evaluator " + evaluatorDescriptor + " for flow instance " + flowInstanceManager.getFlowInstance() + " triggered by user " + eventUser);
									}
								}
								
								notificationHandler.sendGlobalEmail(siteProfile, flowInstance, contact, emailAddress, evaluator.getEmailSubject(), evaluator.getEmailMessage(), file, evaluator.sendPDFAttachmentsSeparately());
								
							} catch (Exception e) {
								
								log.error("Unable to send email " + email + " about submitted flow instance " + flowInstanceManager.getFlowInstance() + " triggered by user " + eventUser, e);
							}
						}
					}
				}
			}
		}
	}
	
	private boolean containsStatusFilteredEvaluators(Flow flow) throws SQLException {
		
		for (Step step : flow.getSteps()) {
			
			if (step.getQueryDescriptors() != null) {
				
				for (QueryDescriptor queryDescriptor : step.getQueryDescriptors()) {
					
					if (queryDescriptor.getEvaluatorDescriptors() != null) {
						
						for (EvaluatorDescriptor evaluatorDescriptor : queryDescriptor.getEvaluatorDescriptors()) {
							
							if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {
								
								UserQueryMailEvaluator evaluator = getEvaluator(evaluatorDescriptor);
								
								if (evaluator.enableStatusFiltering()) {
									
									return true;
								}
							}
						}
					}
				}
			}
		}
		
		return false;
	}
	
	public ImmutableFlowInstanceManager getImmutableFlowInstanceManager(FlowInstance flowInstance, SiteProfile siteProfile) throws DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException {
		
		return new ImmutableFlowInstanceManager(flowInstance, queryHandler, null, new DefaultInstanceMetadata(siteProfile), null);
	}
	
	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		UserQueryMailEvaluator evaluator = new UserQueryMailEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	private UserQueryMailEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {
		
		return evaluatorDAO.get(new HighLevelQuery<UserQueryMailEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		UserQueryMailEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<UserQueryMailEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<UserQueryMailEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		UserQueryMailEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<UserQueryMailEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		convertStatusIDs(evaluator, statusConversionMap);
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	@Override
	public UserQueryMailEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		UserQueryMailEvaluator evaluator = new UserQueryMailEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		convertStatusIDs(evaluator, statusConversionMap);
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
	
	private void convertStatusIDs(UserQueryMailEvaluator evaluator, Map<Integer, ImmutableStatus> statusConversionMap) {
		
		if (evaluator.getStatusIDs() != null && statusConversionMap != null) {
			
			List<Integer> convertedStatusIDs = new ArrayList<Integer>(evaluator.getStatusIDs().size());
			
			for (Integer statusID : evaluator.getStatusIDs()) {
				
				ImmutableStatus status = statusConversionMap.get(statusID);
				
				if (status != null) {
					
					convertedStatusIDs.add(status.getStatusID());
				}
			}
			
			evaluator.setStatusIDs(convertedStatusIDs);
		}
	}
	
	public QueryHandler getQueryHandler() {
		
		return this.queryHandler;
	}
	
	public String getAvailableTags() {
		
		return StringUtils.toCommaSeparatedString(StandardFlowNotificationHandler.getGlobalTags());
	}
}
