package com.nordicpeak.flowengine.queries.usertreequery;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.RequestMetadata;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.usertreequery.beans.TreeNode;
import com.nordicpeak.flowengine.queries.usertreequery.beans.TreeUser;
import com.nordicpeak.flowengine.queries.usertreequery.beans.UserTreeQuery;
import com.nordicpeak.flowengine.queries.usertreequery.beans.UserTreeQueryInstance;
import com.nordicpeak.flowengine.queries.usertreequery.interfaces.UserTreeQueryDataProvider;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class UserTreeQueryProviderModule extends BaseQueryProviderModule<UserTreeQueryInstance> implements BaseQueryCRUDCallback {

	@XSLVariable(prefix = "java.")
	private String missingTreeProvider = "Missing";

	private AnnotatedDAO<UserTreeQuery> queryDAO;
	private AnnotatedDAO<UserTreeQueryInstance> queryInstanceDAO;

	private UserTreeQueryCRUD queryCRUD;

	private QueryParameterFactory<UserTreeQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<UserTreeQueryInstance, Integer> queryInstanceIDParamFactory;

	private ConcurrentHashMap<String, UserTreeQueryDataProvider> treeProviders = new ConcurrentHashMap<String, UserTreeQueryDataProvider>();

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(UserTreeQueryProviderModule.class, this)) {

			throw new RuntimeException("Unable to register module " + moduleDescriptor + " in global instance handler using key " + UserTreeQueryProviderModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(UserTreeQueryProviderModule.class, this);

		super.unload();
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, UserTreeQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(UserTreeQuery.class);
		queryInstanceDAO = daoFactory.getDAO(UserTreeQueryInstance.class);

		queryCRUD = new UserTreeQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<UserTreeQuery>(UserTreeQuery.class), "UserTreeQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		UserTreeQuery query = new UserTreeQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		UserTreeQuery query = new UserTreeQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		UserTreeQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		UserTreeQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		UserTreeQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new UserTreeQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		//If this is a new query instance copy the default values
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance.copyQueryValues();
			queryInstance.setFullTree(getFullTree(queryInstance, poster, null));
		}

		return queryInstance;
	}

	@Override
	public QueryResponse getFormHTML(UserTreeQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if (queryInstance.getFullTree() == null && !requestMetadata.isManager()) {

			queryInstance.setFullTree(getFullTree(queryInstance, poster, requestMetadata));
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	private UserTreeQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<UserTreeQuery> query = new HighLevelQuery<UserTreeQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private UserTreeQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<UserTreeQuery> query = new HighLevelQuery<UserTreeQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private UserTreeQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<UserTreeQueryInstance> query = new HighLevelQuery<UserTreeQueryInstance>(UserTreeQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(UserTreeQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public void populate(UserTreeQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		Integer queryID = queryInstance.getQuery().getQueryID();

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		String treeNodeIdentifier = ValidationUtils.validateParameter("q" + queryID + "_selectedkey", req, false, StringPopulator.getPopulator(), validationErrors);
		String userIdentifier = ValidationUtils.validateParameter("q" + queryID + "_selecteduser", req, false, StringPopulator.getPopulator(), validationErrors);

		if (!allowPartialPopulation) {

			if (treeNodeIdentifier == null) {
				validationErrors.add(new ValidationError("RequiredTreeNode"));
			}

			if (userIdentifier == null) {
				validationErrors.add(new ValidationError("RequiredUser"));
			}
		}

		TreeNode selectedTreeNode = null;
		TreeUser selectedUser = null;

		if (treeNodeIdentifier != null) {

			if (queryInstance.getFullTree() != null) {

				selectedTreeNode = queryInstance.getFullTree().find(treeNodeIdentifier);

			} else if (queryInstance.getSelectedNodeKey() != null && treeNodeIdentifier.equals(queryInstance.getSelectedNodeKey())) {

				selectedTreeNode = queryInstance.getSelectedTreeNodeWithParents();
			}

			if (selectedTreeNode == null || (queryInstance.getQuery().isOnlyAllowSelectingLeafs() && !selectedTreeNode.isEmpty())) {

				validationErrors.add(new ValidationError("InvalidTreeNode"));

			} else if (userIdentifier != null) {

				UserTreeQueryDataProvider provider = getTreeProvider(queryInstance.getQuery().getProviderIdentifier());

				if (provider != null) {

					List<TreeUser> users = provider.getUsers(selectedTreeNode.getKey());

					if (!CollectionUtils.isEmpty(users)) {
						for (TreeUser treeUser : users) {

							if (treeUser.getProviderUserID().equals(userIdentifier)) {
								selectedUser = treeUser;
								break;
							}
						}
					}
				}

				if (queryInstance.getQuery().isRequireEmail() && selectedUser != null && StringUtils.isEmpty(selectedUser.getEmail())) {
					selectedUser = null;
				}

				if (selectedUser == null) {
					validationErrors.add(new ValidationError("InvalidUser"));
				}
			}
		}

		if (queryInstance.getQueryInstanceDescriptor().getQueryState() != QueryState.VISIBLE_REQUIRED && ObjectUtils.isNull(selectedUser)) {

			queryInstance.reset(attributeHandler);
			return;
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		queryInstance.setSelectedNodeKey(selectedTreeNode.getKey());
		queryInstance.setSelectedTreeNodes(selectedTreeNode.getDirectParents());
		queryInstance.setFirstname(selectedUser.getFirstname());
		queryInstance.setLastname(selectedUser.getLastname());
		queryInstance.setCitizenIdentifier(selectedUser.getCitizenIdentifier());
		queryInstance.setProviderUserID(selectedUser.getProviderUserID());
		queryInstance.setEmail(selectedUser.getEmail());
		queryInstance.setPhone(selectedUser.getPhone());

		queryInstance.getQueryInstanceDescriptor().setPopulated(true);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		UserTreeQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		UserTreeQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		UserTreeQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, UserTreeQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}

		XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "StaticContent", "classpath://" + getClass().getPackage().getName().replace('.', '/') + "/staticcontent");
	}

	@Override
	protected Class<UserTreeQueryInstance> getQueryInstanceClass() {

		return UserTreeQueryInstance.class;
	}

	private TreeNode getFullTree(UserTreeQueryInstance queryInstance, User poster, RequestMetadata requestMetadata) {

		UserTreeQuery query = queryInstance.getQuery();

		if (!StringUtils.isEmpty(query.getProviderIdentifier())) {

			UserTreeQueryDataProvider provider = getTreeProvider(query.getProviderIdentifier());

			if (provider != null) {

				return provider.getTree();
			}
		}

		return null;
	}

	public void addTreeProvider(UserTreeQueryDataProvider treeProvider) {

		if (treeProviders.containsKey(treeProvider.getTreeID())) {

			log.error("UserTreeProvider \"" + treeProvider.getTreeName() + "\" (ID= " + treeProvider.getTreeID() + ") is already registered");
			return;
		}

		treeProviders.put(treeProvider.getTreeID(), treeProvider);

		log.info("UserTreeQueryDataProvider " + treeProvider.getTreeName() + " registered");
	}

	public boolean removeTreeProvider(UserTreeQueryDataProvider treeProvider) {

		boolean removed = treeProviders.remove(treeProvider.getTreeID(), treeProvider);

		if (removed) {
			log.info("UserTreeQueryDataProvider " + treeProvider.getTreeName() + " unregistered");
		}

		return removed;
	}

	public UserTreeQueryDataProvider getTreeProvider(String id) {

		UserTreeQueryDataProvider provider = treeProviders.get(id);

		if (provider == null) {

			log.warn("UserTreeQueryDataProvider " + id + " not found");
		}

		return provider;
	}

	public Collection<UserTreeQueryDataProvider> getTreeProviders() {

		return treeProviders.values();
	}

	public String getMissingTreeProvider() {

		return missingTreeProvider;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(UserTreeQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws IOException {

		UserTreeQuery query = queryInstance.getQuery();

		if (!StringUtils.isEmpty(query.getProviderIdentifier())) {

			UserTreeQueryDataProvider provider = getTreeProvider(query.getProviderIdentifier());

			if (provider != null) {

				return new UserTreeQueryRequestProcessor(provider);
			}
		}

		return null;
	}
}
