package com.nordicpeak.flowengine.queries.usertreequery.beans;

import java.lang.reflect.Field;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.interfaces.UserValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryUtils;
import com.nordicpeak.flowengine.queries.usertreequery.UserTreeQueryProviderModule;
import com.nordicpeak.flowengine.queries.usertreequery.interfaces.UserTreeQueryDataProvider;

@Table(name = "usertree_query_instances")
@XMLElement
public class UserTreeQueryInstance extends BaseQueryInstance implements StringValueQueryInstance, UserValueQueryInstance {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(UserTreeQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private UserTreeQuery query;

	@DAOManaged
	@XMLElement
	private String firstname;

	@DAOManaged
	@XMLElement
	private String lastname;

	@DAOManaged
	@XMLElement
	private String providerUserID;

	@DAOManaged
	@XMLElement
	private String citizenIdentifier;

	@DAOManaged
	@XMLElement
	private String email;

	@DAOManaged
	@XMLElement
	private String phone;

	@DAOManaged
	@XMLElement
	private String selectedNodeKey;

	@DAOManaged
	@OneToMany(autoGet = true, autoAdd = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<TreeNode> selectedTreeNodes;

	private TreeNode fullTree;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public UserTreeQuery getQuery() {

		return query;
	}

	public void setQuery(UserTreeQuery query) {

		this.query = query;
	}

	public String getSelectedNodeKey() {

		return selectedNodeKey;
	}

	public void setSelectedNodeKey(String selectedNodeID) {

		this.selectedNodeKey = selectedNodeID;
	}

	public TreeNode getFullTree() {

		return fullTree;
	}

	public void setFullTree(TreeNode fullTree) {

		this.fullTree = fullTree;
	}

	public List<TreeNode> getSelectedTreeNodes() {

		return selectedTreeNodes;
	}

	public void setSelectedTreeNodes(List<TreeNode> selectedTreeNodes) {

		this.selectedTreeNodes = selectedTreeNodes;
	}

	public TreeNode getSelectedTreeNode() {

		if (selectedNodeKey != null) {

			return getTreeNodeFromSelectedNodes(selectedNodeKey);
		}

		return null;
	}

	public TreeNode getTreeNodeFromSelectedNodes(String key) {

		if (!CollectionUtils.isEmpty(selectedTreeNodes)) {
			for (TreeNode node : selectedTreeNodes) {

				if (node.getKey().equals(key)) {
					return node;
				}
			}
		}

		return null;
	}

	public TreeNode getSelectedTreeNodeWithParents() {

		TreeNode node = getSelectedTreeNode();

		setParentsForSelectedTreeNodes(node);

		return node;
	}

	private void setParentsForSelectedTreeNodes(TreeNode node) {

		if (node != null && node.getParentNodeKey() != null) {

			TreeNode parent = getTreeNodeFromSelectedNodes(node.getParentNodeKey());

			if (parent != null) {

				parent.put(node);
				setParentsForSelectedTreeNodes(parent);
			}
		}
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		selectedNodeKey = null;
		selectedTreeNodes = null;
		firstname = null;
		lastname = null;
		citizenIdentifier = null;
		providerUserID = null;
		email = null;
		phone = null;

		super.reset(attributeHandler);
	}

	public void copyQueryValues() {

	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(HttpServletRequest req, User user, User poster, URIParser uriParser, QueryHandler queryHandler, InstanceRequestMetadata requestMetadata) throws Exception {

		return BaseQueryUtils.getGenericQueryInstanceProvider(this.getClass(), queryHandler, queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID()).getQueryRequestProcessor(this, req, user, poster, uriParser, requestMetadata);
	}

	@Override
	public Element toXML(Document doc) {

		Element element = XMLGenerator.toXML(this, doc);

		if (fullTree != null) {

			Element fullTreeElement = XMLUtils.appendNewElement(doc, element, "FullTree");
			XMLUtils.append(doc, fullTreeElement, fullTree);
		}

		return element;
	}

	@Override
	public String toString() {

		return getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewElement(doc, element, "Firstname", firstname);
		XMLUtils.appendNewElement(doc, element, "Lastname", lastname);
		XMLUtils.appendNewElement(doc, element, "ProviderUserID", providerUserID);
		XMLUtils.appendNewElement(doc, element, "CitizenIdentifier", citizenIdentifier);
		XMLUtils.appendNewElement(doc, element, "Email", email);
		XMLUtils.appendNewElement(doc, element, "Phone", phone);
		XMLUtils.appendNewElement(doc, element, "SelectedNodeKey", selectedNodeKey);
		XMLUtils.append(doc, element, "TreeNodes", selectedTreeNodes);

		return element;
	}

	@Override
	public String getStringValue() {

		StringBuilder builder = new StringBuilder();

		TreeNode selectedNode = getSelectedTreeNode();

		if (selectedNode != null) {

			builder.append(selectedNode.getName());
			builder.append(": ");
		}

		if (firstname != null) {

			builder.append(firstname);
			builder.append(" ");
		}

		if (lastname != null) {

			builder.append(lastname);
		}

		return builder.toString();
	}

	public String getFirstname() {

		return firstname;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public String getProviderUserID() {

		return providerUserID;
	}

	public void setProviderUserID(String providerUserID) {

		this.providerUserID = providerUserID;
	}

	public String getCitizenIdentifier() {

		return citizenIdentifier;
	}

	public void setCitizenIdentifier(String citizenIdentifier) {

		this.citizenIdentifier = citizenIdentifier;
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getPhone() {

		return phone;
	}

	public void setPhone(String phone) {

		this.phone = phone;
	}

	@Override
	public User getUserValue(QueryHandler queryHandler) {

		UserTreeQueryProviderModule queryProviderModule = queryHandler.getQueryProvider(queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID(), UserTreeQueryProviderModule.class);

		UserTreeQueryDataProvider queryDataProvider = queryProviderModule.getTreeProvider(getQuery().getProviderIdentifier());

		if (queryDataProvider != null) {

			return queryDataProvider.getLocalUser(providerUserID);
		}

		return null;
	}

}
