package com.nordicpeak.flowengine.evaluators.dynamicweightquerystateevaluator;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluationCRUD;

public class DynamicWeightQueryStateEvaluatorCRUD extends BaseQueryStateEvaluationCRUD<DynamicWeightQueryStateEvaluator, DynamicWeightQueryStateEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<DynamicWeightQueryStateEvaluator> POPULATOR = new AnnotatedRequestPopulator<>(DynamicWeightQueryStateEvaluator.class);

	public DynamicWeightQueryStateEvaluatorCRUD(Class<DynamicWeightQueryStateEvaluator> beanClass, AnnotatedDAOWrapper<DynamicWeightQueryStateEvaluator, Integer> evaluatorDAO, DynamicWeightQueryStateEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "DynamicWeightQueryStateEvaluator", "dynamic weight query state evaluator", null, callback);
	}

	@Override
	protected void validateUpdatePopulation(DynamicWeightQueryStateEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		bean.validate();

		bean.getEvaluatorDescriptor().setTargetQueryIDs(null);

	}

	@Override
	protected DynamicWeightQueryStateEvaluator populateFromUpdateRequest(DynamicWeightQueryStateEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		DynamicWeightQueryStateEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<DynamicWeightQueryStateEvaluatorValues> list = getValues(req, req.getParameterValues("rowID"));
		evaluator.setDynamicWeightQueryStateEvaluatorValueList(list);

		return evaluator;
	}

	@Override
	protected HttpServletRequest parseRequest(HttpServletRequest req, User user) throws ValidationException {

		Map<String, String[]> parametermap = req.getParameterMap();
		if (parametermap != null && !parametermap.isEmpty()) {

			DynamicWeightQueryStateEvaluator evaluator = new DynamicWeightQueryStateEvaluator();
			evaluator.setWeightType(req.getParameter("weightType"));

			String[] idkeys = req.getParameterValues("rowID");
			if (idkeys != null && idkeys.length > 0) {
				evaluator.setDynamicWeightQueryStateEvaluatorValueList(getValues(req, idkeys));
			}

			evaluator.validate();

		}

		return req;
	}

	private List<DynamicWeightQueryStateEvaluatorValues> getValues(HttpServletRequest req, String[] idKeys) {

		List<DynamicWeightQueryStateEvaluatorValues> list = new ArrayList<>();

		for (String key : idKeys) {
			String fromValue = req.getParameter("fromValue" + key);
			String toValue = req.getParameter("toValue" + key);

			DynamicWeightQueryStateEvaluatorValues evaluatorValues = new DynamicWeightQueryStateEvaluatorValues();
			if (NumberUtils.isInt(key)) {
				evaluatorValues.setValueID(Integer.parseInt(key));
			}

			try {
				evaluatorValues.setFromValue(Integer.parseInt(fromValue));
			} catch (NumberFormatException e) {
				// do nothing
			}

			try {
				evaluatorValues.setToValue(Integer.parseInt(toValue));
			} catch (NumberFormatException e) {
				// do nothing
			}

			list.add(evaluatorValues);
		}

		return list;
	}

	@Override
	protected void appendUpdateFormData(DynamicWeightQueryStateEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		//overridden to ignore appending flow-data
	}

}
