package com.nordicpeak.flowengine.evaluators.setattributefromweightevaluator;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.EvaluationEventListener;
import com.nordicpeak.flowengine.interfaces.EvaluationHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

@Table(name = "set_attribute_from_weight_evaluators")
@XMLElement
public class SetAttributeFromWeightEvaluator extends BaseEvaluator implements EvaluationEventListener {
	
	private static final long serialVersionUID = -1383195490043894435L;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;
	
	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer fromValue;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer toValue;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String attributeValue;
	
	private HashMap<Integer, BigDecimal> weightSourceMap;
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<ValidationError>(1);
		
		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		fromValue = xmlParser.getInteger("fromValue");
		toValue = xmlParser.getInteger("toValue");
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		attributeValue = XMLValidationUtils.validateParameter("attributeValue", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}
	
	@Override
	public boolean supportsEvent(Object event) {
		
		if (event instanceof SetWeightEvaluationEvent) {
			
			return true;
		}
		
		return false;
	}
	
	@Override
	public List<QueryModification> processEvent(Object event, QueryInstance queryInstance, User user, User poster, EvaluationCallback callback, EvaluationHandler evaluationHandler) {
		
		SetWeightEvaluationEvent weightEvent = (SetWeightEvaluationEvent) event;
		
		if (this.weightType != null && weightEvent.getWeightType().equalsIgnoreCase(weightType)) {
			
			if (weightSourceMap == null) {
				
				weightSourceMap = new HashMap<Integer, BigDecimal>();
			}
			
			this.weightSourceMap.put(weightEvent.getSourceID(), weightEvent.getWeight());
			
			EvaluationResponse response = super.evaluate(queryInstance, user, poster, callback, evaluationHandler, false, weightEvent.getAttributeHandler());
			
			if (response != null) {
				
				return response.getModifications();
			}
		}
		
		return null;
	}
	
	public BigDecimal getWeight() {
		
		if (weightSourceMap != null) {
			
			BigDecimal totalWeight = BigDecimal.ZERO;
			
			for (BigDecimal weight : weightSourceMap.values()) {
				
				totalWeight = totalWeight.add(weight);
			}
			
			return totalWeight;
		}
		
		return null;
	}
	
	public Integer getFromValue() {
		
		return fromValue;
	}
	
	public void setFromValue(Integer fromValue) {
		
		this.fromValue = fromValue;
	}
	
	public Integer getToValue() {
		
		return toValue;
	}
	
	public void setToValue(Integer toValue) {
		
		this.toValue = toValue;
	}
	
	public String getAttributeName() {
		return attributeName;
	}
	
	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}
	
	public String getAttributeValue() {
		return attributeValue;
	}
	
	public void setAttributeValue(String attributeValue) {
		this.attributeValue = attributeValue;
	}
	
}
