package com.nordicpeak.flowengine.evaluators.setweightbydateevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.enums.Month;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_date_evaluators")
@XMLElement
public class SetWeightByDateEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -3617607585720127002L;

	private static final EnumPopulator<Month> MONTH_POPULATOR = new EnumPopulator<Month>(Month.class);

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "set_weight_by_date_evaluator_months", remoteValueColumnName = "month")
	@WebPopulate(required = true)
	@XMLElement(childName = "month")
	private List<Month> months;

	@DAOManaged
	@WebPopulate(populator = DayOfMonthPopulator.class)
	@XMLElement
	private Integer minDayOfMonth;

	@DAOManaged
	@WebPopulate(populator = DayOfMonthPopulator.class)
	@XMLElement
	private Integer maxDayOfMonth;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true, populator = IntegerPopulator.class)
	@XMLElement
	private Integer weight;
	
	private boolean initialized;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		months = XMLValidationUtils.validateParameters("months/month", xmlParser, true, MONTH_POPULATOR, errors);
		minDayOfMonth = XMLValidationUtils.validateParameter("minDayOfMonth", xmlParser, false, NonNegativeStringIntegerPopulator.getPopulator(), errors);
		maxDayOfMonth = XMLValidationUtils.validateParameter("maxDayOfMonth", xmlParser, false, NonNegativeStringIntegerPopulator.getPopulator(), errors);

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, IntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public List<Month> getMonths() {

		return months;
	}

	public void setMonths(List<Month> months) {

		this.months = months;
	}

	public Integer getMinDayOfMonth() {

		return minDayOfMonth;
	}

	public void setMinDayOfMonth(Integer minDayOfMonth) {

		this.minDayOfMonth = minDayOfMonth;
	}

	public Integer getMaxDayOfMonth() {

		return maxDayOfMonth;
	}

	public void setMaxDayOfMonth(Integer maxDayOfMonth) {

		this.maxDayOfMonth = maxDayOfMonth;
	}

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	public boolean isInitialized() {

		return initialized;
	}

	public void setInitialized(boolean initialized) {

		this.initialized = initialized;
	}
}
