package com.nordicpeak.flowengine.evaluators.setweightbyflowinstancestatus;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;

public class SetWeightByFlowInstanceStatusEvaluatorCRUD extends BaseEvaluatorCRUD<SetWeightByFlowInstanceStatusEvaluator, SetWeightByFlowInstanceStatusEvaluationProviderModule> {

	private static final String WEIGHT_NAME = "weight";
	private static final String STATUS_ID = "statusID";
	private static final String WEIGHT_TYPE = "weightType";
	private static final AnnotatedRequestPopulator<SetWeightByFlowInstanceStatusEvaluator> POPULATOR = new AnnotatedRequestPopulator<>(SetWeightByFlowInstanceStatusEvaluator.class);

	public SetWeightByFlowInstanceStatusEvaluatorCRUD(Class<SetWeightByFlowInstanceStatusEvaluator> beanClass, AnnotatedDAOWrapper<SetWeightByFlowInstanceStatusEvaluator, Integer> evaluatorDAO, SetWeightByFlowInstanceStatusEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SetWeightByFlowInstanceStatusEvaluator", "set weight by flow instance status evaluator", null, callback);

	}
	
	@Override
	protected HttpServletRequest parseRequest(HttpServletRequest req, User user) throws ValidationException {

		Map<String, String[]> parametermap = req.getParameterMap();
		if (parametermap != null && !parametermap.isEmpty()) {

			SetWeightByFlowInstanceStatusEvaluator evaluator = new SetWeightByFlowInstanceStatusEvaluator();
			evaluator.setWeightType(req.getParameter(WEIGHT_TYPE));
			
			evaluator.setWeightByFlowInstanceStatusValueList(getValues(req, getStatusID(req)));
			
			evaluator.validate();

		}

		return req;
	}
	
	@Override
	protected SetWeightByFlowInstanceStatusEvaluator populateFromUpdateRequest(SetWeightByFlowInstanceStatusEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		SetWeightByFlowInstanceStatusEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<SetWeightByFlowInstanceStatusEvaluatorValues> list = getValues(req, getStatusID(req));
		evaluator.setWeightByFlowInstanceStatusValueList(list);

		return evaluator;
	}
	
	private List<String> getStatusID(HttpServletRequest req) {
		
		Map<String, String[]> map = req.getParameterMap();
		
		List<String> output = new ArrayList<>();
		
		if(map != null) {
			//get all status mappings
			output.addAll(map.keySet().stream().filter(e -> e.startsWith(STATUS_ID)).collect(Collectors.toList()));
		}
		
		return output;
		
	}
	
	
	@Override
	protected void appendUpdateFormData(SetWeightByFlowInstanceStatusEvaluator evaluator, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(evaluator, doc, updateTypeElement, user, req, uriParser);
		
		List<? extends ImmutableStatus> statuses = callback.getFlowStatuses(evaluator.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow().getFlowID());

		XMLUtils.append(doc, updateTypeElement, "Statuses", statuses);
	}

	private List<SetWeightByFlowInstanceStatusEvaluatorValues> getValues(HttpServletRequest req, List<String> idKeys) {

		List<SetWeightByFlowInstanceStatusEvaluatorValues> list = new ArrayList<>();

		if(!CollectionUtils.isEmpty(idKeys)) {
			for (String key : idKeys) {
				key = key.substring(STATUS_ID.length());
				String weight = req.getParameter(WEIGHT_NAME + key);
	
				SetWeightByFlowInstanceStatusEvaluatorValues evaluatorValues = new SetWeightByFlowInstanceStatusEvaluatorValues();
				
				if(key != null) {
					try {
						evaluatorValues.setStatusID(Integer.parseInt(key));
					} catch (NumberFormatException e) {
						// do nothing
					}
				}
	
				if(weight != null) {
					try {
						evaluatorValues.setWeight(Integer.parseInt(weight));
					} catch (NumberFormatException e) {
						// do nothing
					}
				}
	
				list.add(evaluatorValues);
			}
		}

		return list;
	}

}
