package com.nordicpeak.flowengine.evaluators.setweightbymunicipalitycodeevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_municipality_code_evaluators")
@XMLElement
public class SetWeightByMunicipalityCodeEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -336493853396860255L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer weight;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "set_weight_by_municipality_code_evaluator_codes", remoteValueColumnName = "municipalityCode")
	@WebPopulate(required = true, populator=NonNegativeStringIntegerPopulator.class)
	@SplitOnLineBreak
	@NoDuplicates
	@XMLElement(name = "AllowedMunicipalityCodes", childName = "MunicipalityCode")
	private List<Integer> allowedMunicipalityCodes;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean inclusive;
	
	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		
		inclusive = xmlParser.getPrimitiveBoolean("inclusive");

		allowedMunicipalityCodes = XMLValidationUtils.validateParameters("AllowedMunicipalityCodes/MunicipalityCode", xmlParser, false, IntegerPopulator.getPopulator(), errors);		
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	
	public List<Integer> getAllowedMunicipalityCodes() {
	
		return allowedMunicipalityCodes;
	}

	
	public void setAllowedMunicipalityCodes(List<Integer> allowedMunicipalityCodes) {
	
		this.allowedMunicipalityCodes = allowedMunicipalityCodes;
	}

	
	public boolean isInclusive() {
	
		return inclusive;
	}

	
	public void setInclusive(boolean inclusive) {
	
		this.inclusive = inclusive;
	}
}
