<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateSetWeightByTextFieldValueEvaluator" />
		</div>

	</xsl:template>

	<xsl:template match="UpdateSetWeightByTextFieldValueEvaluator">

		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluatorDescriptor.title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="SetWeightByTextFieldValueEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="SetWeightByTextFieldValueEvaluator" />
			</xsl:call-template>

			<div class="floatleft full bigmarginbottom">

				<label for="weightType" class="floatleft full">
					<xsl:value-of select="$i18n.WeightType" />
					<span class="required">*</span>
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'weightType'" />
						<xsl:with-param name="name" select="'weightType'" />
						<xsl:with-param name="element" select="SetWeightByTextFieldValueEvaluator" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<label for="weightType" class="floatleft full">
					<xsl:value-of select="$i18n.Weight" />
					<span class="required">*</span>
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'weight'" />
						<xsl:with-param name="name" select="'weight'" />
						<xsl:with-param name="element" select="SetWeightByTextFieldValueEvaluator" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<label for="weightType" class="floatleft full">
					<xsl:value-of select="$i18n.AllowedValues" />
					<span class="required">*</span>
				</label>
	
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'allowedValues'" />
						<xsl:with-param name="name" select="'allowedValues'" />
						<xsl:with-param name="element" select="SetWeightByTextFieldValueEvaluator/AllowedValues/Value" />
						<xsl:with-param name="separateListValues" select="'true'"/>
					</xsl:call-template>
				</div>
			</div>
				
			<div class="floatleft full">
		
				<label>
					<xsl:value-of select="$i18n.Inclusive.title" />
				</label>
				
				<br/>
				
				<xsl:value-of select="$i18n.Inclusive.description" />
			</div>	
				
			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'inclusive'" />
						<xsl:with-param name="name" select="'inclusive'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="SetWeightByTextFieldValueEvaluator" />
					</xsl:call-template>
	
					<label for="inclusive">
						<xsl:value-of select="$i18n.Inclusive" />
					</label>
				</div>
			</div>
	
			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'not-inclusive'" />
						<xsl:with-param name="name" select="'inclusive'" />
						<xsl:with-param name="value" select="'false'" />
						<xsl:with-param name="element" select="SetWeightByTextFieldValueEvaluator" />
					</xsl:call-template>
	
					<label for="not-inclusive">
						<xsl:value-of select="$i18n.NotInclusive" />
					</label>
				</div>
			</div>
		
			<div class="floatleft full bigmarginbottom">

				<xsl:variable name="fieldsCount" select="count(TextFields/TextField)" />
	
				<label class="floatleft full">
					<xsl:value-of select="$i18n.fields.title" />
				</label>
		
				<xsl:choose>
					<xsl:when test="$fieldsCount > 1">
						
						<div class="clearboth floatleft marginbottom">
							<xsl:value-of select="$i18n.fields.description" />
						</div>
					
						<xsl:apply-templates select="TextFields/TextField">
							<xsl:with-param name="evaluator" select="SetWeightByTextFieldValueEvaluator" />
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$fieldsCount = 1">
						
						<div class="clearboth floatleft marginbottom">
							<xsl:value-of select="$i18n.fields.single.description" />
						</div>
						
						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="id" select="'textFieldID'" />
							<xsl:with-param name="name" select="'textFieldID'" />
							<xsl:with-param name="value" select="TextFields/TextField/fieldID" />
						</xsl:call-template>
						
						<p class="floatleft full bold nomargin"><xsl:value-of select="TextFields/TextField/name" /></p>
						
					</xsl:when>
					<xsl:otherwise>
						<p>
							<xsl:value-of select="$i18n.QueryHasNoTextFields" />
						</p>
					</xsl:otherwise>
				</xsl:choose>
			
			</div>

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.submit}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template match="TextField">

		<xsl:param name="evaluator" />

		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createRadio">
					<xsl:with-param name="id">
						<xsl:value-of select="'textfield'"/>
						<xsl:value-of select="fieldID"/>
					</xsl:with-param>				
					<xsl:with-param name="name" select="'textFieldID'" />
					<xsl:with-param name="value" select="fieldID" />
					<xsl:with-param name="element" select="$evaluator" />
				</xsl:call-template>

				<label for="textfield{fieldID}">
					<xsl:value-of select="name" />
				</label>
			</div>
			
		</div>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TextFieldNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.TextFieldNotFound" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'allowedValues'">
				<xsl:value-of select="$i18n.AllowedValues" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>