package com.nordicpeak.flowengine.evaluators.setweightbytextfieldvalueevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CaseInsensitiveStringComparator;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_textfield_value_evaluators")
@XMLElement
public class SetWeightByTextFieldValueEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = 9036808825693500094L;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer textFieldID;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer weight;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean inclusive;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "set_weight_by_textfield_value_evaluator_values", remoteValueColumnName = "value")
	@WebPopulate(required = true)
	@SplitOnLineBreak
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@XMLElement(name = "AllowedValues", childName = "Value")
	private List<String> allowedValues;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		textFieldID = XMLValidationUtils.validateParameter("textFieldID", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, IntegerPopulator.getPopulator(), errors);

		inclusive = xmlParser.getPrimitiveBoolean("inclusive");

		allowedValues = XMLValidationUtils.validateParameters("AllowedValues/Value", xmlParser, false, StringPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getTextFieldID() {

		return textFieldID;
	}

	public void setTextFieldID(Integer textFieldID) {

		this.textFieldID = textFieldID;
	}

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	public boolean isInclusive() {

		return inclusive;
	}

	public void setInclusive(boolean inclusive) {

		this.inclusive = inclusive;
	}

	public List<String> getAllowedValues() {

		return allowedValues;
	}

	public void setAllowedValues(List<String> allowedValues) {

		this.allowedValues = allowedValues;
	}

}
