package com.nordicpeak.flowengine.evaluators.setweightbyuserageevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_user_age_evaluators")
@XMLElement
public class SetWeightByUserAgeEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = 9147248763136420215L;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer minAge;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer maxAge;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean calculateBasedOnYearOnly;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true, populator = IntegerPopulator.class)
	@XMLElement
	private Integer weight;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAttributeIfAgeInRange;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "setAttributeIfAgeInRange")
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "setAttributeIfAgeInRange")
	@XMLElement
	private String attributeValue;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		minAge = XMLValidationUtils.validateParameter("minAge", xmlParser, false, NonNegativeStringIntegerPopulator.getPopulator(), errors);
		maxAge = XMLValidationUtils.validateParameter("maxAge", xmlParser, false, NonNegativeStringIntegerPopulator.getPopulator(), errors);
		calculateBasedOnYearOnly = xmlParser.getPrimitiveBoolean("calculateBasedOnYearOnly");

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		
		setAttributeIfAgeInRange = xmlParser.getPrimitiveBoolean("setAttributeIfAgeInRange");
		attributeName = xmlParser.getString("attributeName");
		attributeValue = xmlParser.getString("attributeValue");

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getMinAge() {

		return minAge;
	}

	public void setMinAge(Integer minAge) {

		this.minAge = minAge;
	}

	public Integer getMaxAge() {

		return maxAge;
	}

	public void setMaxAge(Integer maxAge) {

		this.maxAge = maxAge;
	}

	public boolean isCalculateBasedOnYearOnly() {

		return calculateBasedOnYearOnly;
	}

	public void setCalculateBasedOnYearOnly(boolean calculateBasedOnYearOnly) {

		this.calculateBasedOnYearOnly = calculateBasedOnYearOnly;
	}

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	
	public boolean isSetAttributeIfAgeInRange() {
	
		return setAttributeIfAgeInRange;
	}

	
	public void setSetAttributeIfAgeInRange(boolean setAttributeIfAgeInRange) {
	
		this.setAttributeIfAgeInRange = setAttributeIfAgeInRange;
	}

	
	public String getAttributeName() {
	
		return attributeName;
	}

	
	public void setAttributeName(String attributeName) {
	
		this.attributeName = attributeName;
	}

	
	public String getAttributeValue() {
	
		return attributeValue;
	}

	
	public void setAttributeValue(String attributeValue) {
	
		this.attributeValue = attributeValue;
	}

}
