package com.nordicpeak.flowengine.evaluators.setweightevaluator;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;

public class SetWeightEvaluatorCRUD extends BaseEvaluatorCRUD<SetWeightEvaluator, SetWeightEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<SetWeightEvaluator> POPULATOR = new AnnotatedRequestPopulator<SetWeightEvaluator>(SetWeightEvaluator.class);

	public SetWeightEvaluatorCRUD(Class<SetWeightEvaluator> beanClass, AnnotatedDAOWrapper<SetWeightEvaluator, Integer> evaluatorDAO, SetWeightEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SetWeightEvaluator", "weight evaluator", null, callback);
	}

	@Override
	protected void appendUpdateFormData(SetWeightEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);
		
		FixedAlternativesQuery query = (FixedAlternativesQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), true);

		List<? extends ImmutableAlternative> alternatives = query.getAlternatives();

		if (alternatives != null) {

			Element alternativesElement = doc.createElement("Alternatives");
			updateTypeElement.appendChild(alternativesElement);

			for (ImmutableAlternative alternative : alternatives) {

				Element alternativeElement = doc.createElement("Alternative");
				alternativesElement.appendChild(alternativeElement);

				XMLUtils.appendNewElement(doc, alternativeElement, "alternativeID", alternative.getAlternativeID());
				XMLUtils.appendNewCDATAElement(doc, alternativeElement, "name", alternative.getName());
			}
		}

	}

	@Override
	protected SetWeightEvaluator populateFromUpdateRequest(SetWeightEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		SetWeightEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		FixedAlternativesQuery query = (FixedAlternativesQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), true);

		List<? extends ImmutableAlternative> alternatives = query.getAlternatives();
		
		List<AlternativeWeight> alternativeWeights = null;
		
		if(alternatives != null){
			
			alternativeWeights = new ArrayList<AlternativeWeight>();
			
			for(ImmutableAlternative alternative : alternatives){
				
				BigDecimal weight = NumberUtils.toBigDecimalLocaleIndependent(req.getParameter("alternative" + alternative.getAlternativeID()));
				
				//TODO use ValidationUtils to generate validation errors
				
				if(weight != null){
					
					AlternativeWeight alternativeWeight = new AlternativeWeight();
					
					alternativeWeight.setAlternativeID(alternative.getAlternativeID());
					alternativeWeight.setWeight(weight);
					
					alternativeWeights.add(alternativeWeight);
				}
			}
		}
		
		evaluator.setAlternativeWeights(alternativeWeights);
		
		return evaluator;
	}

}
